# prjLib.tcl - project manager library
#
# Copyright (c) 1998-2006 Wind River Systems, Inc. 
#
# modification history
# --------------------
# 03m,20Apr06,tfo  Fixed WIND_HOME path issue on Windows hosts
# 03l,21Mar06,pee  SPR 119127. Fixed path returned by Wind::tgtDir
# 03k,3Augl05,zhr  Fixed SPR#105600, WIND_SOURCE_BASE is not well substituted
#                  when it appears in a project file
# 03j,08aug05,pee  don't generate dependencies anymore.
# 03i,23dec04,c_c  removed references to foundation.
# 03h,08sep04,fle  SPR#88074 : added prjMakefile
# 03g,03sep04,fle  Save host value before sourcing prjXmlLib.tcl
# 03f,02sep04,fle  Fixed use of Wind::path since foundation tree moved
# 03e,30aug04,fle  Added notion of a library related standard output
# 03d,23jul04,fle  SPR#99055 : fixed copy when there are some extra files
# 03c,22jun04,fle  SPR#98396 : source FileLib.tcl instead of relying on
#                  PrjUtils package
# 03b,15jun04,fle  SPR#98254 : made prjFullPathRelativeTo also substitute the
#                  WIND_HOME, WIND_GNU_PATH and WIND_DIAB_PATH environment
#                  variables
# 03a,18may04,fle  Do not use file copy on read-only files on Linux
# 02z,13may04,fle  fixed prjOpen to set the toolchain and current build before
#                  calling for prjFullPath
# 02y,06may04,fle  Removed all bullet lists refgen markups
#                  + made prjCopy also copy files within the project directory
#                    which belong to the files dependencies
# 02x,23apr04,fle  change files permissions when copying a read only project
#                  + made prjFullPathRelativeTo also expand TGT_DIR
# 02w,09apr04,fle  SPR#94279 : Added prjConfigletteExpand procedure
# 02v,07apr04,fle  Added CC_BIN_DIR expansion to prjFullPathRelativeTo
# 02u,08mar04,fle  Added prjVfsFile, and made it be copied in prjCopy
# 02t,18feb04,fle  Speed up prjSave, and make it provide PrjLib package
# 02s,30dec03,fle  Made it source the prjFsFileLib.tcl file
# 02r,08dec03,p_b  restore wpj format
# 02q,28nov03,p_b  moved isXmlFile to Utils.tcl
# 02p,25nov03,fle  Fixed backward compatibility issue with 'prjOpen' since we
#                  use xml format for project files
# 02o,25nov03,p_b  fixed full path substituion problem
# 02n,21nov03,p_b  Ported wpj format to xml
# 02m,24oct03,fle  SPR#90863 : fixed WIND_SOURCE_BASE substitutions
# 02l,01sep03,fle  SPR#86913 : do not substitute WIND_SOURCE_BASE in
#                  prjFullPathRelativeTo if there is no directory after the
#                  trailing slash
# 02k,01sep03,fle  SPR#87162 : substitute some paths to use slashes instead of
#                  backslashes
# 02j,01jul03,fle  fixed prjRelPathRelativeTo to be able to colapse again
#                  WIND_BASE
# 02i,30jun03,fle  Fixed prjFullPathRelativeTo since it does not use wtxPath
# 02h,19jun03,fle  Updated to new location
# 02g,12jun03,fle  Enhanced documentation
# 02f,24feb03,fle  fixed WIND_SOURCE_BASE access
# 02e,04feb03,rbl  speed up project open for large projects (e.g. godzilla)
# 02d,15may02,rbl  modify last change to fix SPR 77344 -- 
#                  Projects containing user source don't copy correctly.
#                  fix bug where project version was not being written
#                  correctly into workspace. 
# 02d,17may02,cjs  Moving prjDirDefault to Utils.tcl
# 02c,30apr02,cjs  Fix spr 73093 -- copy of downloable project screws up builds
# 02b,10dec01,rbl  fix bug where prjSaveAs was re-generating the old
#                  Makefile...
# 02a,19oct01,cjs  Fix spr 25520 -- use recursive directory creation in
#                  wizards
# 01z,11oct01,rbl  make it possible (still not recommended!) to edit project
#                  files on Windows
# 01z,15oct01,fle  made it use the Wind package
# 01y,27sep01,rbl  update project path functions (prjRelPath, prjFullPath) to
#                  T3 versions to support e.g. WIND_PROJ_BASE,
#                  WIND_SOURCE_BASE, and calling with no project (hProj
#                  argument blank)
#                  write project file to temporary file first when saving, 
#                  to avoid corruption or loss on error or interruption
# 01y,04oct01,cjs  Fixing defaultPrjWarnLogFuncGet
# 01x,08feb01,t_m  Fix for ARM GCC2.97 depends path
# 01w,22mar99,rbl  escaping special regular expression characters in path
#                  substitutions
#                  fixing prjSaveAs so it returns the new filename even 
#                  when no filename is passed in
# 01v,19mar99,rbl  extending the special handling in prjRelPathRelativeTo so
#                  that the new project gets the new project directory in its
#                  -I include paths and in the .cdf search path
# 01u,16mar99,cjs  Make sure that makeGen is supported for project before
#                  calling it (won't work for workspaces, for example)
# 01t,12mar99,cjs  Adding call to makeGen() in project save routines
# 01s,09mar99,rbl  added new description as parameter to prjCopy and
#                  prjSaveAs. Updated prjCreateable to check if any core files
#                  are present and fixed prjOverwritable to correctly handle 
#                  core files.
# 01r,01mar99,rbl  Fix prjRelPathRelativeTo: handle core files first,
#                  then make other substitutions.
# 01q,02feb99,rbl  Changes to handle missing or read-only projects
# 01p,21jan99,cjs  Alter prjCopy() to copy project file in memory, not one on
#                  disk
# 01o,24nov98,jmp  added WIND_HOST_TYPE substitution in prj[Rel|Full]Path.
# 01n,10nov98,cjs  add routine to fetch default warning and logging
#                  output handler   
# 01m,26oct98,ms   on UNIX, only emit debug output if TORNADO_DEBUG is set
# 01l,09sep98,cjs  added prjHasComponents() 
# 01k,04sep98,cjs  added optional params to prjCoreCreate(); do case
#                  insensitive subst in prjRelPath()
# 01j,12aug98,ms   fixed prjCopy, removed prjSaveAs.
# 01h,31jul98,ms   added prjDirDefault
# 01g,29jul98,ms   new syntax for prj[Rel|Full]Path
# 01g,20jul98,ms   prjCreateable fails if directory already exists.
# 01f,07jul98,ms   added hook in prjClose.
# 01e,18jun98,cjs  changed signature of prjStatusLog 
# 01d,18jun98,jmp  added return value to prjTagRemove{}.
# 01c,01jun98,ms   added prjStatusLog and prjWarnLog
# 01c,08apr98,ms   fixed prjSaveAs
# 01b,02apr98,ms   prjInfo hProj changed - now contains timestamp of last mod.
#		   prjClose now defaults to not saving the file.
# 01a,13mar98,ms   written.
#
# DESCRIPTION
# This is the generic project managment library.
#
# It manages a projects files, build settings, and other project information.
# Each project has a <type>, such as 'vxWorks', 'WDB', 'bootROM', etc.
#
# This library also interacts with specific project-type libraries that
# handle project information specific to that project type (for example,
# the vxWorks project manager needs to know about what OS components
# are included).
#
# Adding support for a new project-type is easy; just add a TCL file
# called prjType_*.tcl. The project library will source that file on
# startup. The new project type library needs to provide certain
# functions in a TCL namespace:
#
# \ts
# create | launch wizard to create a new project of this type
# name | the name of this project type (e.g., vxWorks)
# toolChainListGet | returns the list of compatible toolchains
# buildRuleListGet | returns list of available build rules (make targets)
# buildRuleDefault | returns the default build rule (make targets)
# buildModify <hProj> <build> | modify a build spec for this project type
# build <image> | build the current image
# coreFilesGet | returns list of files (apart from the project file itself) 
#	      | that define the "core" of the project. In a <save as>, these
#	      | are the files that get copied. 
# \te
#
# The new project-type manager should then call 'prjTypeAdd' to register
# itself with the project manager.
#
# This library provides low-level routines to allow the file manager,
# build manager and project-type managers to store and retrieve information
# in a project.
#
# Data is stored and retrieved under tags using the routines:
#
# \is
# \i prjTagDataSet <hProj> <tag> <data>
# \i prjTagDataGet <hProj> <tag>
# \i prjTagListGet <hProj>
# \i prjTagRemove <hProj> <tag>
# \ie
#
# For example, the file manager might store it's data under the tag
# PROJECT_FILES.
#
# The file manager is implemented in prjFileLib.tcl.
# The build manager is implemented in prjBuildLib.tcl.
#
# INTERNAL: if two workspaces open and modify the same project, they will
# conflict with each other. Should we try to avoid this with a lock file?
#

package require PrjUtils
package require Wind

# SPR#98396 : don't know why when in java, the FileLib procedure are not well
# retrieved by the 'package require' statement

source [file join [file dirname [info script]] FileLib.tcl]

set prjTypes	""
set prjHandleIx	0

variable stdoutChannel			stdout
variable stderrChannel			stderr
variable defaultPrjStatusLogRtn
variable defaultPrjWarnLogRtn

# ------------------------------ initialisation ------------------------------ #

if {([Wind::isWindows]) || ([info exists env(TORNADO_DEBUG)])} {
    set defaultPrjStatusLogRtn prjPuts
    set defaultPrjWarnLogRtn prjPuts
    set prjStatusLogRtn $defaultPrjStatusLogRtn
    set prjWarnLogRtn $defaultPrjWarnLogRtn
} else {
    proc nop arg {}
    set defaultPrjStatusLogRtn nop
    set defaultPrjWarnLogRtn  nop
    set prjStatusLogRtn $defaultPrjStatusLogRtn
    set prjWarnLogRtn $defaultPrjWarnLogRtn
}

# prjTagData(hProj,tag)
if {[info exists prjTagData]} {
    unset prjTagData
}

# prjInfo(hProj,type)		type = fileName, changed, ...
if {[info exists prjInfo]} {
    unset prjInfo
}

################################################################################
#
# prjFullPath - convert paths relative to project directory to full paths
#
# SYNOPSIS 
# \cs
#   prjFullPath <hProj> <files>
# \ce
#
# DESCRIPTION
# This function prepares a path string read from a file, which is allowed to 
# contain the strings $(PRJ_DIR), $(WIND_SOURCE_BASE), $(WIND_PROJ_BASE), 
# $(WIND_BASE) and $(WIND_HOST_TYPE), by making the following substitutions:
#
# \is
# \i the project directory is substituted for $(PRJ_DIR)
# \i the WIND_SOURCE_BASE environment variable is substituted for
#      $(WIND_SOURCE_BASE)
# \i the WIND_PROJ_BASE directory ($(WIND_BASE)/target/proj unless the 
#      WIND_PROJ_BASE environment variable is defined) is substituted
#      for $(WIND_PROJ_BASE)
# \i the WIND_BASE directory is substituted for $(WIND_BASE)
# \i the WIND_HOST_TYPE string is substituted for $(WIND_HOST_TYPE) 
# \ie
#
# If the <hProj> parameter is blank, only $(WIND_SOURCE_BASE), 
# $(WIND_PROJ_BASE), $(WIND_BASE) and $(WIND_HOST_TYPE) are substituted. 
#
# PARAMETERS 
#   hProj:  handle of project
#   files:  string to convert
#
# RETURNS: path with the above substitutions made.  
#
# ERRORS: N/A
#

proc prjFullPath {hProj files} {
    set prjDir ""
    if {$hProj != ""} {
	set prjDir [file dirname [prjInfoGet $hProj fileName]]
    }
    return [prjFullPathRelativeTo $hProj $files $prjDir]
}

################################################################################
#
# prjFullPathRelativeTo - convert paths relative to project dir  to full paths
#
# SYNOPSIS
# \cs
#   prjFullPathRelativeTo <hProj> <files> [newPrjDir]
# \ce
#
# DESCRIPTION
# This function prepares a path string read from a file, which is allowed to 
# contain the strings $(PRJ_DIR), $(WIND_SOURCE_BASE), $(WIND_PROJ_BASE), 
# $(WIND_BASE) and $(WIND_HOST_TYPE), by making the following substitutions:
#
# \is
# \i the project directory is substituted for $(PRJ_DIR)
# \i the WIND_SOURCE_BASE environment variable is substituted for
#      $(WIND_SOURCE_BASE)
# \i the WIND_PROJ_BASE directory ($(WIND_BASE)/target/proj unless the
#      WIND_PROJ_BASE environment variable is defined) is substituted for
#      $(WIND_PROJ_BASE)
# \i the WIND_BASE directory is substituted for $(WIND_BASE)
# \i the WIND_HOST_TYPE string is substituted for $(WIND_HOST_TYPE) 
# \ie
#
# PARAMETERS 
#   hProj : handle of project. If newPrjDir is specified this may be blank. If
#	   <hProj> and <newPrjDir> are both blank, only $(WIND_SOURCE_BASE), 
#	   $(WIND_PROJ_BASE), $(WIND_BASE) and $(WIND_HOST_TYPE) are
#	   substituted. 
#   files : string to convert
#   newPrjDir : directory, if blank the project directory for hProj is used.
#
# RETURNS: files string with substitutions made
#
# ERRORS: N/A
#

proc prjFullPathRelativeTo {hProj files {newPrjDir ""}} {

    if {$newPrjDir == "" && $hProj != ""} {
	set prjDir [file dirname [prjInfoGet $hProj fileName]]
    } else {
	set prjDir $newPrjDir
    }

    if {$hProj != "" || $newPrjDir != ""} {
	regsub -all {\$\(PRJ_DIR\)} $files $prjDir files
    }

    set srcBase [PrjUtils::srcDir]

    if {$srcBase != ""} {

	regsub -all -- {\$\(WIND_SOURCE_BASE\)/([^ \t]+)} $files ${srcBase}/\\1  files
    }

    # get the toolchain to be able to expand CC_BIN_DIR and CC_HOST

    set build [prjBuildCurrentGet $hProj]

    if {$build != ""} {
	set tc		[prjBuildTcGet $hProj $build]
	set ccBinDir	[file join [Wind::tcPath [${tc}::family]] bin]
	set ccHost	[Wind::tcHostType [${tc}::family]]

	regsub -all {\$\(CC_BIN_DIR\)/} $files [format "%s/" $ccBinDir] files
	regsub -all {\$\(CC_HOST\)} $files $ccHost files
    }

    regsub -all {\$\(WIND_PROJ_BASE\)/} $files [PrjUtils::defaultDir]/ files
    regsub -all {\$\(WIND_BASE\)/} $files [Wind::path] files
    regsub -all {\$\(TGT_DIR\)/} $files [file join [Wind::tgtDir]]/ files
    regsub -all {\$\(WIND_HOST_TYPE\)} $files [Wind::hostType] files
    regsub -all {\$\(WIND_HOME\)/} $files [file join [Wind::home]]/ files
    regsub -all {\$\(WIND_DIAB_PATH\)/} $files \
		[file dirname [file join [Wind::tcPath diab]]]/ files
    regsub -all {\$\(WIND_GNU_PATH\)/} $files \
		[file dirname [file join [Wind::tcPath gnu]]]/ files


    # eat a single /../ caused by $(PRJ_DIR)/../ 
    # timw: added -all

    regsub -all /\[^/\\.\]+/\\.\\./ $files "/" files

    return $files
}

################################################################################
#
# prjInfoGet - get info about the project
#
# SYNOPSIS
# \cs
#   prjInfoGet <hProj> <infoType>
# \ce
#
# DESCRIPTION
# This routine is for internal use only
#
# PARAMETERS:
#   hProj : handle of the project to get info for
#   infoType : type of information to get
#
# NOMANUAL
#

proc prjInfoGet {hProj infoType} {
    global prjInfo

    return $prjInfo($hProj,$infoType)
}

###############################################################################
#
# prjInfoSet - set info about the project
#
# SYNOPSIS
# \cs
#   prjInfoSet <hProj> <infoType> <info>
# \ce
#
# DESCRIPTION
# This routine is for internal use only
#
# PARAMETERS:
#   hProj : handle of the project to set info for
#   infoType : type of information to set
#   info : value of the info to set
#
# RETURNS: N/A
#
# NOMANUAL
#

proc prjInfoSet {hProj infoType info} {
    global prjInfo

    set prjInfo($hProj,$infoType) $info
}

################################################################################
#
# prjNameGet - get the name of the project
#
# SYNOPSIS
# \cs
#   prjNameGet <hProj>
# \ce
#
# DESCRIPTION
# Get the name of the project which handle is <hProj>
#
# PARAMETERS:
#   hProj : handle of the project to get name of
#
# RETURNS: the name of the project
#

proc prjNameGet {hProj} {
    return [file tail [file rootname [prjInfoGet $hProj fileName]]]
}

################################################################################
#
# prjTypeAdd - add a project type
#
# SYNOPSIS
# \cs
#   prjTypeAdd <prjType>
# \ce
#
# DESCRIPTION
# Add a project type
#
# PARAMETERS:
#   prjType : new project type to add
#
# RETURNS: N/A
#
# ERRORS:
# \is
# \i invalid project namespace <prjType>
# \ie
#
# SEE ALSO: prjTypeRemove
#

proc prjTypeAdd {prjType} {
    global prjTypes

    if {[catch "namespace eval $prjType {info procs create}" val]} {
	error "invalid project namespace $prjType"
    }
    if {$val != "create"} {
	error "invalid project namespace $prjType"
    }

    lappend prjTypes $prjType
}

################################################################################
#
# prjTypeRemove - remove a project type
#
# SYNOPSIS
# \cs
#   prjTypeRemove <prjType>
# \ce
#
# DESCRIPTION
# Remove a project type
#
# PARAMETERS:
#   prjType : project type to remove from this project.
#
# RETURNS: N/A
#
# ERRORS:
# \is
# \i <prjType> doesn't exist
# \ie
#
# SEE ALSO: prjTypeAdd
#

proc prjTypeRemove {prjType} {

    global prjTypes

    set ix [lsearch $prjTypes $prjType]
    if {$ix == -1} {
	error "$prjType doesn't exist"
    }

    set prjTypes [lreplace $prjTypes $ix $ix]
}

################################################################################
#
# prjTypeListGet - get a list of available project types
#
# SYNOPSIS
# \cs
#   prjTypeListGet
# \ce
#
# DESCRIPTION
# Get the list of available project types
#
# PARAMETERS: N/A
#
# RETURNS: a list of project types
#
# ERRORS: N/A
#
# SEE ALSO: prjTypeAdd, prjTypeRemove
#

proc prjTypeListGet {} {
    global prjTypes

    return $prjTypes
}

################################################################################
#
# prjTypeIsValid - check if a project type is valid
#
# SYNOPSIS
# \cs
#   prjTypeIsValid <prjType>
# \ce
#
# DESCRIPTION
# Check if a project type is valid.
#
# A project type is valid only if it appears in the list of avaliable project
# types (as returned by 'prjTypeListGet')
#
# PARAMETERS: N/A
#
# RETURNS: '1' if the project type is valid, else '0'
#
# ERRORS: N/A
#
# SEE ALSO: prjTypeListGet
#

proc prjTypeIsValid {prjType} {
    return [expr {[lsearch [prjTypeListGet] $prjType]} != -1]
}

################################################################################
#
# prjHandleValidCheck - check if a project handle is valid
#
# SYNOPSIS
# \cs
#   prjHandleValidCheck <hProj>
# \ce
#
# DESCRIPTION
# Checks if the given <hProj> project handle is valid.
#
# A project handle is valid if it has been returned by a call to 'prjOpen',
# an that no 'prjClose' was performed on that handle.
#
# PARAMETERS:
#   hProj : project handle to check
#
# RETURNS: '1', or exits on error.
#
# ERRORS:
# \is
# \i invalid project handle <hProj>
# \ie
#
# NOMANUAL
#

proc prjHandleValidCheck hProj {
    global prjInfo

    if {![info exists prjInfo($hProj,fileName)]} {
	error "invalid project handle $hProj"
    }

    return 1
}

################################################################################
#
# prjTagDataSet - set project data
#
# SYNOPSIS
# \cs
#   prjTagDataSet <hProj> <tag> <data>
# \ce
#
# DESCRIPTION
# This routine saves project data under a given tag.
#
# PARAMETERS:
#   hProj : project handle to set <tag>'s data to <data>
#   tag : project tag to set to value to <data>
#   data : value to give to <tag>'s value
#
# RETURNS: N/A
#
# ERRORS:
# \is
# \i invalid project handle <hProj>
# \ie
#
# SEE ALSO: prjTagDataGet
#

proc prjTagDataSet {hProj tag data} {

    prjHandleValidCheck $hProj

    global prjTagData

    # XXX : fle : fix for SPR#87162. All path names referring to
    # WIND_BASE should not contain any backslash. To achieve this, there
    # is no possibility with a good old regsub. I had to rather replace
    # the matching paths with their "file join" value (which does the
    # slash-backslash substitution as we expect it).
    #
    # Quite sure we should also do this processing with PROJ_DIR and some
    # other macros, but for now the SPR only puts forth problems with
    # WIND_BASE.

    set tmpData $data
    set REGEXP "\\(WIND_BASE\\)\\\\(\[^ \t]+)"

    set ixes {}

    while {[regexp -indices -- $REGEXP $tmpData ixes]} {

	set start [string range $tmpData 0 [expr {[lindex $ixes 0]} - 1]]
	set tail [string range $tmpData [expr {[lindex $ixes 1]} + 1] \
			 [expr {[string length $tmpData]} + 1]]

	set replaced [file join [string range $tmpData [lindex $ixes 0]\
					[lindex $ixes 1]]]

	set tmpData "$start$replaced$tail"
    }

    set prjTagData($hProj,$tag) $data

    prjInfoSet $hProj changed [clock seconds]
}

################################################################################
#
# prjTagDataGet - get project data
#
# SYNOPSIS
# \cs
#   prjTagDataGet <hProj> <tag>
# \ce
#
# DESCRIPTION
# This routine returns project data stored under a given tag
#
# PARAMETERS:
#   hProj : project handle to get <tag>'s value for.
#   tag : tag to get value for.
#
# RETURNS: The <tag>'s value for <hProj> if it exists, or an empty string if it
# does not exist.
#
# ERRORS:
# \is
# \i invalid project handle <hProj>
# \ie
#
# SEE ALSO: prjTagDataSet
#

proc prjTagDataGet {hProj tag} {

    prjHandleValidCheck $hProj

    global prjTagData

    if {![info exists prjTagData($hProj,$tag)]} {
	return ""
    }

    return $prjTagData($hProj,$tag)
}

################################################################################
#
# prjDirtyGet - gets <dirty> state of project
#
# SYNOPSIS
# \cs
#   prjDirtyGet <hProj>
# \ce
#
# DESCRIPTION
# If the project has been changed and not yet save, it state is set to <dirty>.
# This procedure allows to know it the project has to be saved or not.
#
# PARAMETERS:
#   hProj:   project handle
#
# RETURNS: '1' if project is dirty, '0' if project is clean
#
# ERRORS: N/A
#
# SEE ALSO: prjSave
#

proc prjDirtyGet {hProj} {
    if {[::prjInfoGet $hProj changed] != ""} {
	return 1
    } else {
	return 0
    }
}

################################################################################
#
# prjProjectFileWritable - checks if the project file is writable
#
# SYNOPSIS
# \cs
#   prjProjectFileWritable <hProj>
# \ce
#
# DESCRIPTION
# This procedure just checks if the project file is writable.
#
# PARAMETERS:
#   hProj : project handle
#
# RETURNS: '1' if project is writable, '0' if not writable
#

proc prjProjectFileWritable {hProj} {
    return [file writable [prjInfoGet $hProj fileName]]	
}

################################################################################
#
# prjTagListGet - return the list of tags under which project data is stored
#
# SYNOPSIS
# \cs
#   prjTagListGet <hProj> [prefix]
# \ce
#
# DESCRIPTION
# This procedure gets the sorted list of tags the <hProj> project handles.
#
# If <prefix> parameter is provided, only tags prefixed with <prefix> are
# returned.
#
# PARAMETERS:
#   hProj : project handle to get tags from.
#   prefix : specifies to only get the <prefix> prefixed tags.
#
# RETURNS: a list of tags for project <hProj>
#

proc prjTagListGet {hProj {prefix ""}} {
    prjHandleValidCheck $hProj

    global prjTagData
    set tags ""
    foreach ix [array names prjTagData $hProj,${prefix}*] {
	regsub $hProj, $ix "" tag
	lappend tags $tag
    }

    return [lsort $tags]
}

################################################################################
#
# prjTagRemove - remove a data tag (and its associated data) from the project
#
# SYNOPSIS
# \cs
#   prjTagRemove <hProj> <tag>
# \ce
#
# DESCRIPTION
# This procedure removes a tag from given <hProj> project. It also removed its
# associated value.
#
# PARAMETERS
#   hProj : project handle to remove <tag> from
#   tag : tag to remove from <hProj> tags
#
# RETURNS: '1' if the tag was sucessfully removed, '0' if it does not exist in
#	  <hProj> project.
#
# ERRORS: N/A
#

proc prjTagRemove {hProj tag} {
    prjHandleValidCheck $hProj

    global prjTagData
    if {![info exists prjTagData($hProj,$tag)]} {
    	# Tag does not exist, return ERROR
    	return 0
    }

    unset prjTagData($hProj,$tag)
    prjInfoSet $hProj changed [clock seconds]

    return 1
}

################################################################################
#
# prjSave - write out a project
#
# SYNOPSIS
# \cs
#   prjSave <hProj>
# \ce
#
# DESCRIPTION
# This routine only saves info if the project has changed.
#
# PARAMETERS
#   hProj : handle of the project to save.
#
# RETURNS: N/A
#
# ERRORS:
# \is
# \i invalid project handle <hProj>
# \ie
#

proc prjSave {hProj} {

    catch {prjInfoGet $hProj changed} changed

    if {"$changed" == ""} {
	return
    }

    set prjFile [prjInfoGet $hProj fileName]
    set prjDir	[file dirname $prjFile]

    # write to a temporary file to avoid corrupting the project file

    set tempFile $prjFile
    append tempFile ".windtemp"

    set prjFileError ""
    catch {
	set fd [open $tempFile w+]

	fconfigure $fd -translation lf
	puts $fd "Document file - DO NOT EDIT"
	puts $fd ""
	foreach tag [prjTagListGet $hProj] {
	    puts $fd "<BEGIN> [PrjUtils::relPath $tag $prjDir]"
	    puts $fd [PrjUtils::relPath [prjDataSplit [prjTagDataGet $hProj \
								     $tag]] \
					$prjDir]
	    puts $fd "<END>\n"
	}
	close $fd
    } prjFileError

    if {$prjFileError == ""} {
	if {![file exists $prjFile] || [file writable $prjFile]} {

	    # XXX : fle : file copy does not really work well on Linux, better
	    # use FileLib::copy

	    FileLib::copy $tempFile $prjFile 1
	} else {
	    error "Permission denied: can't overwrite read-only file \
		   $prjFile"
	}
	prjInfoSet $hProj changed ""
	file delete $tempFile
    } else {
	catch {
	    close $fd
	    file delete $tempFile
	}
	error "Save of project [prjNameGet $hProj] failed: $prjFileError"
    }

    # Not the cleanest way to do things.  We generate the makefile here
    # in order to guarantee that it always exists.  This allows
    # subprojects to be built w/o calling buildCmdsGet().  We can't count
    # on makeGen() existing, as this could be a workspace project handle

    set hiddenNamespace [::prjTypeGet $hProj]_hidden
    if {[namespace eval ${hiddenNamespace} {info procs makeGen}] != ""} {
	${hiddenNamespace}::makeGen $hProj [ \
	    file join [file dir [prjInfoGet $hProj fileName]] \
		      [prjMakefile $hProj] \
	]
    }
}

################################################################################
#
# prjSaveAs - write out a project file to a new fileName.
#
# SYNOPSIS
# \cs
#   prjSaveAs <hProj> [fileName [fFailIfExists [newDescription]]]
# \ce
#
# DESCRIPTION
# This routine only saves the project file.  The rest has to be dealt w/
# explicitly.  Marks old project file clean. 
#
# PARAMETERS
#   hProj : handle of the project to save
#   fileName : specifies the new file to save this project in. If <fileName> is
#	       not specified, <hProj> is saved in its original project file.
#   fFailIfExists : If set to '1', the save should fail if <fileName> already
#		    exists. Default is '1'
#   newDescription : If this variable is provided, the saved project file
#		     will have <newDescription> for descriptio. Else, it keeps
#		     the old <hProj> project's description.
#
# RETURNS: The new filename, even when no filename is passed in
#
# ERRORS: invalid project handle, can't create new file
#

proc prjSaveAs {hProj {fileName ""} {fFailIfExists 1} {newDescription ""}} {

    if {$fileName == ""} {
	prjSave $hProj
	return [prjInfoGet $hProj fileName]
    }

    # Make sure we save all the paths relative to the NEW project directory...

    set oldProj		[prjInfoGet $hProj fileName]
    set oldProjName	[file rootname [file tail $oldProj]]
    set newProjName	[file rootname [file tail $fileName]]
    set oldProjDir	[file dir $oldProj]
    set newProjDir	[file dirname $fileName]
    set prjType		[prjTypeGet $hProj]

    # Error if the file exists

    if {($fFailIfExists) && ([file exists $fileName])} {
	error "Project Save As: file $fileName exists"
    }

    # create the coreFiles regular expression

    set coreFiles	[${prjType}::coreFilesGet]

    if {[llength $coreFiles] != ""} {

	set reg	[format "%s/(" [EscapeRegexpCharacters "${oldProjDir}"]]

	for {set ix 0} {$ix < [llength $coreFiles]} {incr ix} {
	    set coreFile [lindex $coreFiles $ix]
	    append reg [EscapeRegexpCharacters $coreFile]
	    if {$ix != [expr {[llength $coreFiles]} - 1]} {
		append reg "|"
	    }
	}
	append reg ")"
	set subst "${newProjDir}/\\1"

	if {[Wind::isWindows]} {
	    regsub -all -- {/} [string tolower $reg] {[\\\/]} reg
	}
    }

    set fd [open $fileName w+]
    fconfigure $fd -translation lf
    set error ""

    catch {
	puts $fd "Document file - DO NOT EDIT"
	puts $fd ""

	# write everything except description

	foreach tag [prjTagListGet $hProj] {

	    if {$tag != "userComments"} {
		set tagData [prjTagDataGet $hProj $tag]

		if {$newProjName != ""} {

		    # Replace e.g. oldProject.out, but NOT oldProject/foo.c
		    # in tag data

		    if {[regexp $oldProjName\\.\[^/\]*$ $tagData] == 1} {
			regsub -all $oldProjName $tagData $newProjName tagData
		    }

		    # Replace e.g. oldProject.out, but NOT oldProject/foo.c
		    # in tag name as well

		    if {[regexp $oldProjName\\.\[^/\]*$ $tag] == 1} {
			regsub -all $oldProjName $tag $newProjName tag
		    }
		}

		# replace corefiles paths in tags and tag value too

		if {([llength $coreFiles] != 0) && ([regexp -- $reg $tag])} {
		    if {[Wind::isWindows]} {
			regsub -all -nocase -- $reg $tag $subst tag
			regsub -all -nocase -- $reg $tagData $subst tagData
		    } else {
			regsub -all -- $reg $tag $subst tag
			regsub -all -- $reg $tagData $subst tagData
		    }
		}

		puts $fd "<BEGIN> [PrjUtils::relPath $tag $newProjDir\
						     $oldProjDir]"

		# pass the tag type to PrjUtils::relPath so that WCC__CDF_PATH \
		# gets special handling...

		puts $fd [PrjUtils::relPath [prjDataSplit $tagData] $newProjDir\
					     $oldProjDir $tag]
		puts $fd "<END>\n" 
	    }
	}

	# write description

	set tag "userComments"
	set tagData $newDescription

	puts $fd "<BEGIN> [PrjUtils::relPath $tag $newProjDir $oldProjDir]"
	puts $fd [PrjUtils::relPath [prjDataSplit $tagData] $newProjDir \
				    $oldProjDir]
	puts $fd "<END>\n"
    } error

    close $fd

    # If we have successfully saved as, mark the old project clean
    if {$error == ""} {
	prjInfoSet $hProj changed ""
    } else {
	error "Project Save As failed: $error"
    }

    # Not the cleanest way to do things.  We generate the makefile here
    # in order to guarantee that it always exists.  This allows
    # subprojects to be built w/o calling buildCmdsGet().  We can't count
    # on makeGen() existing, as this could be a workspace project handle

    set hiddenNamespace [::prjTypeGet $hProj]_hidden

    if {[namespace eval ${hiddenNamespace} {info procs makeGen}] != ""} {
	set hProjNew [prjOpen $fileName]
	${hiddenNamespace}::makeGen $hProjNew \
				    [file join $newProjDir \
					       [prjMakefile $hProjNew]]
	prjClose $hProjNew
    }

    return $fileName
}

################################################################################
#
# prjOpen - open a project.
#
# SYNOPSIS
# \cs
#   prjOpen <prjFile> [how {r|w|a}]
# \ce
#
# DESCRIPTION
# This procedure opens a Project, and returns its associated handle. This handle
# is use in most of the 'prjLib' procedures to define which project the
# called procedure is referring to.
#
# PARAMETERS
#   prjFile : path to the project file (.wpj) to open
#   how : open mode (read/write/append).
#
# RETURNS: a handle to the project
#
# ERRORS:
# \is
# \i <prjFile>: no such project (<status>)
# The project file could not be opened. The <status> should tell why
# \ie
#

proc prjOpen {prjFile {how "r"}} {

    global prjInfo
    global prjHandleIx
    global prjTagData

    set hProj [incr prjHandleIx]
    if {[catch {open $prjFile $how} fd]} {
	error "$prjFile: no such project ($fd)"
    }
    fconfigure $fd -translation auto

    set prjInfo($hProj,fileName) $prjFile
    set prjInfo($hProj,changed) ""

    set tag	""
    set data	""

    # check if it is the old .wpj format, or new xml format

    if {[isXmlFile $prjFile]} {

	set retData [prjXmlOpen $prjFile]

	if {[info exists retData]} {

	    foreach {key data} "$retData" {

		set	prjTagData($hProj,[prjFullPath $hProj $key]) \
			[prjFullPath $hProj $data]
	    }
	}

    } else {

	# first thing to do, should be to set the current build to be sure
	# prjFullPath will proceed correctly

	set fileContent		[read $fd]
	set begunBuild		0
	set currentBuild	""

	foreach line [split $fileContent \n] {
	    if {[regexp {<BEGIN> BUILD__CURRENT} $line]} {
		set begunBuild	1
	    } elseif {$begunBuild} {
		set currentBuild	[string trim $line]
		set prjTagData($hProj,BUILD__CURRENT) $currentBuild
		set prjTagData($hProj,BUILD__LIST) $currentBuild
		break
	    }
	}

	set begunTc	0
	set tcTag	[format "BUILD_%s_TC" $currentBuild]

	foreach line [split $fileContent \n] {
	    if {[regexp "<BEGIN> $tcTag" $line]} {
		set begunTc	1
	    } elseif {$begunTc} {
		set currentTc	[string trim $line]
		set prjTagData($hProj,$tcTag) $currentTc
		break
	    }
	}

	# open an old .wpj file

	foreach line [split [prjFullPath $hProj $fileContent] \n] {

	    if {[regexp {<BEGIN> .*} $line] == 1} {
		regsub {<BEGIN> } $line "" tag
		continue
	    }

	    if {$line == "<END>"} {

		set tmpData [prjDataJoin [string trimright $data \n]]

		# XXX : fle : fix for SPR#87162. All path names referring to
		# WIND_BASE should not contain any backslash. To achieve this,
		# there is no possibility with a good old regsub. I had to
		# rather replace the matching paths with their "file join"
		# value (which does the slash-backslash substitution as we
		# expect it).
		#
		# Quite sure we should also do this processing with PROJ_DIR and
		# some other macros, but for now the SPR only puts forth
		# problems with WIND_BASE.

		set REGEXP "\\(WIND_BASE\\)\\\\(\[^ \t]+)"

		set ixes {}

		while {[regexp -indices -- $REGEXP $tmpData ixes]} {

		    set start [string range $tmpData 0 [expr {[lindex $ixes 0]}\
							     - 1]]
		    set tail [string range $tmpData \
				     [expr {[lindex $ixes 1]} + 1] \
				     [expr {[string length $tmpData]} + 1]]

		    set replaced [file join [string range $tmpData \
					    [lindex $ixes 0] [lindex $ixes 1]]]

		    set tmpData [format "%s%s%s" $start $replaced $tail]
		}

		set prjTagData($hProj,$tag) $tmpData
		unset tmpData

		set tag ""
		set data ""
		continue
	    }

	    if {$tag == ""} {
		continue
	    }

	    set data "${data}${line}\n"
	}
    }

    catch {close $fd}

    prjInfoSet $hProj PROJECT_FILE_STATUS OK
    return $hProj
}

################################################################################
#
# prjOpenDead - open a "dead" project, one who's file cannot be found
#
# SYNOPSIS
# \cs
#   prjOpenDead <prjFile>
# \ce
#
# DESCRIPTION
# This procedure opens a project which project file cannot be found. No
# information about the project is retrieved, , but its PROJECT_FILE_STATUS
# tag is set to CANNOT_FIND_PROJECT_FILE.
#
# PARAMETERS
#   prjFile : project file to open
#
# RETURNS: a handle to the project.
#

proc prjOpenDead {prjFile} {

    global prjInfo
    global prjHandleIx
    global prjTagData

    set hProj [incr prjHandleIx]

    prjInfoSet $hProj fileName $prjFile
    prjInfoSet $hProj PROJECT_FILE_STATUS CANNOT_FIND_PROJECT_FILE
    prjInfoSet $hProj changed ""

    return $hProj
}

################################################################################
#
# prjHandleGet - return a handle to a project file
#
# SYNOPSIS
# \cs
#   prjHandleGet <prjFile>
# \ce
#
# DESCRIPTION
# This routine is like 'prjOpen', but if the project is already open
# it returns an existing handle instead.
#
# PARAMETERS
#   prjFile : path to the project file to open.
#
# RETURNS: The project handle of the opened project.
#

proc prjHandleGet {prjFile} {

    global prjInfo
    global prjHandleIx
    global prjTagData

    foreach ix [array names prjInfo *,fileName] {
	if {$prjInfo($ix) == $prjFile} {
	    return [lindex [split $ix ,] 0]
	}
    }

    return [prjOpen $prjFile]
}


################################################################################
#
# prjClose - close a project
#
# SYNOPSIS
# \cs
#   prjClose <hProj> [save]
# \ce
#
#
# DESCRIPTION
# Close the project associated with project handle <hProj>.
#
# If the <save> parameter is set to "SAVE", the project is saved before it is
# actually closed.
#
# PARAMETERS:
#   hProj : handle of the project to close
#   save : also save the project if this is set to "SAVE"
#
# RETURNS: N/A
#
# ERRORS:
# \is
# \i invalid handle
# \ie
#

proc prjClose {hProj {save ""}} {

    global prjTagData
    global prjInfo

    prjHandleValidCheck $hProj

    if {$save == "SAVE"} {
	prjSave $hProj
    }

    # call the project managers close hook, if any

    set prjType [prjTypeGet $hProj]

    if {[namespace eval ${prjType} "info procs close"] != ""} {
	${prjType}::close $hProj
    }

    # free project handle memory

    foreach ix [array names prjTagData $hProj,*] {
	unset prjTagData($ix)
    }

    foreach ix [array names prjInfo $hProj,*] {
	unset prjInfo($ix)
    }
}

################################################################################
#
# prjCreateable - check if a project is creatable
#
# SYNOPSIS
# \cs
#   prjCreateable <prjDir> [prjType]
# \ce
#
# PARAMETERS 
#   prjDir: project directory
#   prjType: (optional) project type
#
# RETURNS: 1 if project is creatable, 0 otherwise. Prints error message 
# is project not creatable. 
#
# ERRORS
# \is
# \i invalid project type <prjType>
# \i Can't create directory <prjDir>: <rootDir> is not writable
# \i <rootDir> is a file
# \i Cannot have more than one Project in a directory. Directory
#      <prjDirParent> already contains a project
# \ie
#
# SEE ALSO: prjTypeIsValid
#

proc prjCreateable {prjDir {prjType ""}} {

    if {$prjType != "" && ![prjTypeIsValid $prjType]} {
	prjPuts "invalid project type $prjType"
	return 0
    }

    if {![file exists $prjDir]} {

	# we can create as many levels of directory as required.
	# first find the lowest level directory that exists...

	set rootDir [file dir $prjDir]

	while {![file exists $rootDir] && $rootDir != [file dir $rootDir]} {
	    set rootDir [file dir $rootDir]
	}

	# now check that this is a root directory and is writable...

	if {![file writable $rootDir]} {
	    error "Can't create directory $prjDir:\n$rootDir is not writable"
	} elseif {![file isdirectory $rootDir]} {
	    error "'$rootDir' is a file"
	}

    } else {

	# Directory exists, check if we already have a project or any core files
	# in this directory

	set prjList [glob -nocomplain [file join $prjDir *.wpj]]

	if {[llength $prjList] > 0} {
	    error "Cannot have more than one Project in a directory.\
		   \nDirectory [file nativename $prjDir] already contains a
		   project"
	}
    }

    return 1
}

################################################################################
#
# prjOverwritable - check if a project is overwritable
#
# SYNOPSIS
# \cs
#   prjOverwritable <prjFile> <prjType>
# \ce
#
# DESCRIPTION
# Tells if the given <prjFile> can be overwritten.
#
# PARAMETERS
#   prjFile : project file to find overwritable status for
#   prjType : type of the project to check overwritable status
#
# ERRORS:
# \is
# \i invalid project type <prjType>
# \i Project file <prjFile> cannot be overwritten
# \i Project Core File <coreFile> exists and cannot be overwritten
# \ie
#
# SEE ALSO: prjTypeIsValid
#

proc prjOverwritable {prjFile prjType} {

    set prjDir [file dir $prjFile]

    if {![prjTypeIsValid $prjType]} {
	error "invalid project type $prjType"
    }

    if {![file writable $prjFile]} {
	error "Project file [file nativename $prjFile] cannot be overwritten"
    }

    if {[info commands ${prjType}::coreFilesGet] != ""} {

	foreach coreFile [${prjType}::coreFilesGet] {
	    if {([file exists [file join $prjDir $coreFile]]) && \
		(![file writable [file join $prjDir $coreFile]])} {

		error "Project Core File\n\n[file nativename [file join $prjDir\
		       $coreFile]]\n\nexists and cannot be overwritten"
	    }
	}
    }

    return 1
}

################################################################################
#
# prjCoreCreate - create a project core
#
# SYNOPSIS
# \cs
#   prjCoreCreate <prjFile> <prjType> [version [bValidate {0|1}]]
# \ce
#
# DESCRIPTION
# This is a support routine used by various project-type managers
# to initialize the project code.
#
# PARAMETERS
#   prjFile : project file to create a core project for
#   prjType : type of project to create
#   version : version of project. Default to 'Wind::version'
#   bValidate : checs if the project is creatable. Default is '1'.
#
# ERRORS
# \is
# \i can't create project <prjFile> or type <prjType>
# \ie
#
# SEE ALSO: Wind::version, prjCreateable
#

proc prjCoreCreate {prjFile prjType {version ""} {bValidate 1}} {

    set prjDir [file dir $prjFile]

    if {$version == ""} {
	set version [Wind::version]
    }

    if {$bValidate} {
    	if {![prjCreateable $prjFile $prjType]} {
	    error "can't create project $prjFile or type $prjType"
	}

	if {![file exists $prjDir]} {
	    file mkdir $prjDir
	}
    }

    set fd [open $prjFile "w+"]
    fconfigure $fd -translation lf
    close $fd

    set hProj [prjOpen $prjFile]
    prjTagDataSet $hProj CORE_INFO_TYPE $prjType
    prjTagDataSet $hProj CORE_INFO_VERSION $version
    return $hProj
}

################################################################################
#
# prjDelete - delete a project
#
# SYNOPSIS
# \cs
#   prjDelete <prjFile>
# \ce
#
# DESCRIPTION
# Also delete the project directory if it becomes empty.
#
# PARAMETERS
#   prjFile : project file to delete
#
# RETURNS: N/A
#
# ERRORS: N/A
#

proc prjDelete {prjFile} {
    file delete $prjFile

    catch "file delete [file dir $prjFile]"
}

################################################################################
#
# prjCopy - copy a project file and its "core" files.
#
# SYNOPSIS
# \cs
#   prjCopy <hProj> <newProj> [fCheckCreatable {0|1} [newDescription]]
# \ce
#
# DESCRIPTION
# Copy a project file and its associated core files.
#
# If <fCheckCreatable> is set to '1', 'prjCreateable' is called to check that
# the project can really be copied. If it is not creatable, the procedure will
# exit on error.
#
# If the <newDescription> parameter is provided, the copied project will have
# <newDescription> for description, else it will use copied project description.
#
# PARAMETERS
#   hProj : handle of the project to copy
#   newProj : file path to the project to create
#   fCheckCreatable : set to '1' to check if the project is creatable first.
#   newDescription : description to give to the copied project.
#
# RETURNS: N/A
#
# ERRORS:
# \is
# \i can't create <newProj>
# \ie
#
# SEE ALSO : prjCreateable
#

proc prjCopy {hProj newProj {fCheckCreatable 1} {newDescription ""}} {

    set oldProj		[prjInfoGet $hProj fileName]
    set oldProjDir	[file dir $oldProj]
    set newProjDir	[file dir $newProj]

    set prjType [prjTypeGet $hProj]
    set prjDir	[file dirname $oldProj]

    # make sure the new project is createable

    if {($fCheckCreatable == 1) && (![prjCreateable $newProj $prjType])} {
	error "can't create $newProj"
    }

    # copy "core" files from old project directory to new

    if {![file exists $newProjDir]} {
	file mkdir $newProjDir
    }

    # get the list of files which belong to the files dependencies and
    # are located in the project directory ... they should be copied too

    set depFiles	{}

    foreach fileName [prjFileListGet $hProj] {

	# this should generate the dependencies if needed

	if {[catch {prjFileBuildInfoGen $hProj $fileName FALSE} status]} {
	    puts [prjStderr] "Error generating Build info : $status"
	}

	# now get the dependencies list

	set deps	[prjFileInfoGet $hProj $fileName dependencies]

	foreach depFile $deps {

	    # SPR#99055 : remove dot paths from file path

	    regsub -all -- {\\} $depFile {/} depFile
	    regsub -all -- {/\./} $depFile {/} depFile
	    regsub -all -- {/[^/]+/\.\./} $depFile {/} depFile

	    set relFile	[PrjUtils::relPath $depFile $prjDir]
	    if {[regexp {\$\(PRJ_DIR\)} $relFile]} {
		regsub -all -- {\$\(PRJ_DIR\)/} $relFile "" relFile
		if {[lsearch -exact $depFiles $relFile] == -1} {
		    lappend depFiles $relFile
		}
	    }
	}
    }

    foreach coreFile [concat [${prjType}::coreFilesGet] $depFiles] {

	set coreFilePath	[file join $oldProjDir $coreFile]
	set newCoreFilePath	[file join $newProjDir $coreFile]

	# XXX : fle : file copy does not really work well on Linux, better
	# use FileLib::copy

	FileLib::copy $coreFilePath $newCoreFilePath 1

	# change files mode if needed

	if {[Wind::isWindows]} {
	    if {[file attributes $newCoreFilePath -readonly] == 1} {
		file attributes $newCoreFilePath -readonly 0
	    }
	} else {
	    if {! [file writable $newCoreFilePath]} {
		file attributes $newCoreFilePath -permissions 00644
	    }
	}
    }

    # Now save the project in memory into a new project file. Do not complain
    # if it exists.

    prjSaveAs $hProj $newProj 0 $newDescription

    # copy the vfs file if needed, and change its value in the new project

    if {[file exists [set oldVfsFile [prjVfsFile $hProj]]]} {

	# find a new name for it

	set vfsFile	[format "%s.vfs" [file rootname [file tail $newProj]]]
	set newVfsFile	[file join $newProjDir $vfsFile]

	# XXX : fle : file copy does not really work well on Linux, better
	# use FileLib::copy

	FileLib::copy $oldVfsFile $newVfsFile 1

	# open the copied project and set the VFS_FILE tag

	set copiedHProj	[prjOpen $newProj]
	prjVfsFile $copiedHProj $newVfsFile
	prjClose $copiedHProj
    }
}

################################################################################
#
# prjVfsFile - get/set the vfs file path (if any)
#
# SYNOPSIS
# \cs
#   prjVfsFile <hProj> [<path>]
# \ce
#
# DESCRIPTION
# This procedure gets the value of the VFS_FILE tag from the project which
# handle is <hProj>
#
# If the VFS_FILE tag does not exist, an empty string is returned.
#
# If a value is specified by <path> the new value of the VFS_FILE tag is
# set to <path>, and returned to the caller.
#
# PARAMETERS:
#   hProj : handle of the project to get Vfs file for
#   path : new path of the Vfs file
#
# RETURNS: The path to the Vfs file, or an empty string if it is not set
#
# ERRORS: N/A
#
# SEE ALSO: prjFsFileLib, prjCopy
#

proc prjVfsFile {hProj {path ""}} {

    if {$path == ""} {
	if {[catch {prjTagDataGet $hProj VFS_FILE} vfsFile]} {
	    return ""
	}
    } else {
	if {[catch {prjTagDataSet $hProj VFS_FILE $path}]} {
	    return ""
	}
	set vfsFile $path
    }

    return $vfsFile
}

################################################################################
#
# prjTypeGet - get the project type
#
# SYNOPSIS
# \cs
#   prjTypeGet <hProj>
# \ce
#
# DESCRIPTION
# Get the type of the project associated with project handle <hProj>. In fact,
# this procedure just checks that the given <hProj> project handle is valid,
# and get the project tag value for tag CORE_INFO_TYPE.
#
# PARAMETERS
#   hProj : project handle to get type for.
#
# RETURNS: the project type for project associated to <hProj>.
#
# ERRORS:
# \is
# \i invalid project handle <hProj>
# \ie
#
# SEE ALSO: prjHandleValidCheck
#

proc prjTypeGet {hProj} {
    prjHandleValidCheck $hProj

    return [prjTagDataGet $hProj CORE_INFO_TYPE]
}

#################################################################################
#
# prjProjectFileStatusGet - get the project status
#
# SYNOPSIS
# \cs
#   prjProjectFileStatusGet <hProj>
# \ce
#
# DESCRIPTION
# Checks that the project handle <hProj> is valid (calling for
# 'prjHandleValidCheck'), and return tag value for tag PROJECT_FILE_STATUS.
#
# PARAMETERS
#   hProj : project handle to get file status for.
#
# RETURNS: OK or CANNOT_FIND_PROJECT_FILE
#
# ERRORS:
# \is
# \i invalid project handle <hProj>
# \ie
#
# NOMANUAL
#

proc prjProjectFileStatusGet {hProj} {

    prjHandleValidCheck $hProj
    return [prjInfoGet $hProj PROJECT_FILE_STATUS]
}

################################################################################
#
# prjHasComponents - can this project have components associated with it?
#
# SYNOPSIS
# \cs
#   prjHasComponents <hProj>
# \ce
#
# DESCRIPTION
# Depending on the project type, this procedure should return '1' (project
# has components), or '0' (project does not have components).
#
# In the case of a <vxWorks> project ('prjTypeGet' returns <prj_vxWorks>), it
# returns '1', and returns '0' else.
#
# PARAMETERS
#   hProj : handle of the project to check components for.
#
# RETURNS: '1' if the project type is <vxWorks>, '0' else.
#
# SEE ALSO : prjTypeGet
#

proc prjHasComponents {hProj} {

    set retval 0

    switch [::prjTypeGet $hProj] {
	::prj_vxWorks {
	    set retval 1
	}
	default {
	    set retval 0
	}
    }
    return $retval
}

################################################################################
#
# prjVersionGet - get the project version
#
# SYNOPSIS
# \cs
#   prjVersionGet <hProj>
# \ce
#
# DESCRIPTION
# Get the project version for project <hProj>. A handle check is first
# performed, then the value of tag CORE_INFO_VERSION is returned.
#
# PARAMETERS:
#   hProj : handle of the project to get version for.
#
# RETURNS: The version of project <hProj>
#
# ERRORS:
# \is
# \i invalid project handle <hProj>
# \ie
#
# SEE ALSO: prjHandleValidCheck
#

proc prjVersionGet {hProj} {
    prjHandleValidCheck $hProj

    return [prjTagDataGet $hProj CORE_INFO_VERSION]
}

################################################################################
#
# prjStatusLog - log status info
#
# SYNOPSIS
# \cs
#   prjStatusLog <args>
# \ce
#
# DESCRIPTION
# Log status info. The global <prjStatusLogRtn> is called with <args> if it
# exists, else the procedure exists on error.
#
# PARAMETERS:
#   args : arguments to give to the status log routine
#
# RETURNS: N/A
#
# ERRORS:
# \is
# \i Error logging routine not initialized
# \ie
#
# SEE ALSO: prjStatusLogFuncSet
#

proc prjStatusLog {args} {
    global prjStatusLogRtn
    if {$prjStatusLogRtn != ""} {
    	eval {$prjStatusLogRtn $args} 
    } else {
	error "Error logging routine not initialized"
    }
}

################################################################################
#
# prjWarnLog - log a warning.
#
# SYNOPSIS
# \cs
#   prjWarnLog <msg>
# \ce
#
# DESCRIPTION
# Log a message as a warning using the warning log procedure.
#
# PARAMETERS
#   msg : the message to log as a warning
#
# RETURNS: N/A
#
# ERRORS: N/A
#
# SEE ALSO: prjWarnLogFuncSet
#

proc prjWarnLog msg {
    global prjWarnLogRtn
    $prjWarnLogRtn $msg
}

################################################################################
#
# prjStatusLogFuncSet - set the status logging function
#
# SYNOPSIS
# \cs
#   prjStatusLogFuncSet <func>
# \ce
#
# DESCRIPTION
# Set the status log procedure.
#
# NOTE: This procedure does not check if <func> actuallu does exist.
#
# PARAMETERS:
#   func : the procedure to use for further status logs.
#
# RETURNS: N/A
#
# ERRORS: N/A
#
# SEE ALSO: prjStatusLog
#

proc prjStatusLogFuncSet {func} {
    global prjStatusLogRtn
    set prjStatusLogRtn $func
}

################################################################################
#
# defaultPrjStatusLogFuncGet - get the default status logging routine 
#
# SYNOPSIS
# \cs
#   defaultPrjStatusLogFuncGet
# \ce
#
# DESCRIPTION
# Get the default status logging procedure.
#
# PARAMETERS: N/A
#
# RETURNS: the defaul status logging procedure.
#
# ERRORS: N/A
#
# SEE ALSO: prjStatusLogFuncSet
#

proc defaultPrjStatusLogFuncGet {} {
    variable defaultPrjStatusLogRtn
    return $defaultPrjStatusLogRtn
}

################################################################################
#
# prjWarnLogFuncSet - set the warning logging function
#
# SYNOPSIS
# \cs
#   prjWarnLogFuncSet <func>
# \ce
#
# DESCRIPTION
# Set the warning log function to be used for any subsequent warning logs.
#
# PARAMETERS:
#   func : the warning log procedure to be used for any subsequent warning logs.
#
# RETURNS: N/A
#
# ERRORS: N/A
#
# SEE ALSO: prjWarnLog
# 

proc prjWarnLogFuncSet func {
    global prjWarnLogRtn
    set prjWarnLogRtn $func
}


################################################################################
#
# defaultPrjWarnLogFuncGet - get the default warning logging routine 
#
# SYNOPSIS
# \cs
#   defaultPrjWarnLogFuncGet
# \ce
#
# DESCRIPTION
# Get the default warning logging procedure.
#
# PARAMETERS: N/A
#
# RETURNS: The default warnings log procedure.
#
# ERRORS: N/A
#
# SEE ALSO: prjWarnLogFuncSet
#

proc defaultPrjWarnLogFuncGet {} {
    variable defaultPrjWarnLogRtn
    return $defaultPrjWarnLogRtn
}

################################################################################
#
# prjDataSplit - split long lines
# 
# SYNOPSIS
# \cs
#   prjDataSplit <data>
# \ce
#
# DESCRIPTION
# If the string length of <data> is longer than 80 characters, every occurrence
# of at least two spaces is changed into a line continuation character (\\),
# plus a Carriage Return, plus one tab.
#
# PARAMETERS:
#   data : the string to be splitted
#
# RETURNS: The splitted <data> if it was longer than 80 characters, or <data>
# itself if ti was not longer than 80 charactes.
#
# ERRORS: N/A
#
# SEE ALSO: prjDataJoin
#

proc prjDataSplit {data} {

    if {[string length $data] > 80} {
	regsub -all {[ 	]+} $data " \\\n	" data
    }
    return $data
}

################################################################################
#
# prjDataJoin - join lines that were split
#
# SYNOPSIS
# \cs
#   prjDataJoin <data>
# \ce
#
# DESCRIPTION
# This procedure just joins the <data> which has been split by a call to
# 'prjDataSplit'.
#
# Please have a look at the 'prjDataSplit' help to know the format of splitted
# lines.
#
# PARAMETERS:
#   data : the splitted data to be re-joined.
#
# RETURNS: The re-joined data (if there was any split pattern found).
#
# ERRORS: N/A
# 
# SEE ALSO: prjDataSplit
#

proc prjDataJoin {data} {
    regsub -all { \\
	} $data " " data 
    return $data
}

################################################################################
#
# prjConfigletteExpand - expand all the given configlettes
#
# SYNOPSIS
# \cs
#   prjConfigletteExpand <hProj> <configlettes>
# \ce
#
# DESCRIPTION
# This procedure analyses each of the configlettes, looking for strings of
# type $(VARNAME). If a configlette contains such a patter, the environment
# variables are parsed, looking for the VARNAME anvironment variable.
#
# If the VARNAME environment variable is found, the $(VARNAME) string is
# replaced by the VARNAME value.
#
# Else, the build macros are parsed, again, looking for the $(VARNAME) string is
# If it is found, the $(VARNAME) string is replaced by the VARNAME value.
#
# Else, the configlettes is returned as is
#
# RETURNS: The list of expanded configlettes
#
# ERRORS: N/A
#
# EXAMPLES:
# \cs
# % set env(MYENVVAR) {C:\temp}
# % prjConfigletteExpand 1 {$(MYENVVAR)\myFile.c anotherFile.c}
# C:\temp\myFile.c anotherFile.c
# %
# \ce
#
# SEE ALSO: prjBuildCurrentGet, prjBuildMacroListGet, prjBuildMacroGet
#

proc prjConfigletteExpand {hProj configlettes} {

    global env

    set res	{}

    if {$hProj == ""} {
	set hSpec	"default"
	set macros	{}
    } else {
	set hSpec	[prjBuildCurrentGet $hProj]
	set macros	[prjBuildMacroListGet $hProj $hSpec]
    }

    foreach configlette $configlettes {

	if {[string first "\$" $configlette] != -1} {

	    # seems like there is a macro or environment variable to
	    # expand

	    if {[regexp {\$\(([^\)]+)\)} $configlette all varName]} {

		# first look in the environment variables array, if
		# it is not there, look in the macros values

		if {[array names ::env $varName] == $varName} {

		    regsub -all -- {\$\([^\)]+\)} $configlette \
			   $::env($varName) configlette

		} elseif {[set macroIx [lsearch -exact $macros $varName]] \
			  != -1} {

		    set macroName	[lindex $macros $macroIx]
		    set macroValue	[prjBuildMacroGet $hProj $hSpec \
						      $macroName]
		    regsub -all -- {\$\([^\)]+\)} $configlette $macroValue \
			   configlette
		}
	    }
	}

	lappend res $configlette
    }

    return $res
}

################################################################################
#
# prjStdout - get or set the standard output channel for this library
#
# SYNOPSIS
# \cs
#   prjStdout [<channel>]
# \ce
#
# DESCRIPTION
# Get the standard output channel used by the kernel configuration Tcl libraries
#
# If <channel> is specified, then the standard output channel for the kernel
# configuration libraries is set to <channel>, else, it is set to the regular
# 'stdout'
#
# PARAMETERS:
#   channel : the channel to set as the standard output for this library
#
# RETURNS: The current channel used as a standard output for this library
#
# NOMANUAL
#

proc prjStdout {{channel ""}} {

    variable stdoutChannel

    if {($channel != "")} {
	set stdoutChannel $channel
    }

    return $stdoutChannel
}

################################################################################
#
# prjStderr - get or set the standard error channel for this library
#
# SYNOPSIS
# \cs
#   prjStderr [<channel>]
# \ce
#
# DESCRIPTION
# Get the standard error channel used by the kernel configuration Tcl libraries
#
# If <channel> is specified, then the standard error channel for the kernel
# configuration libraries is set to <channel>, else, it is set to the regular
# 'stderr'
#
# PARAMETERS:
#   channel : the channel to set as the standard error for this library
#
# RETURNS: The current channel used as a standard error for this library
#
# NOMANUAL
#

proc prjStderr {{channel ""}} {

    variable stderrChannel

    if {($channel != "")} {
	set stderrChannel $channel
    }

    return $stderrChannel
}

################################################################################
#
# prjPuts - puts a message on the library standard output
#
# SYNOPSIS
# \cs
#   prjPuts <msg>
# \ce
#
# DESCRIPTION
# Put a message on the library standard output. The library standard output
# is the channel returned by a call to 'prjStdout'.
#
# The message <msg> is printed on the library standard output channel.
#
# PARAMETERS:
#   msg : the message to print to the library standard output channel
#
# RETURNS: N/A
#
# SEE ALSO: prjStdout
#
# NOMANUAL
#

proc prjPuts {msg} {

    puts [prjStdout] $msg
    flush [prjStdout]
}

################################################################################
#
# prjMakefile - get or set the kernel configuration projevt Makefile name
#
# SYNOPSIS
# \cs
#   prjMakefile <hProj> [<makefile>]
# \ce
#
# DESCRIPTION
# This procedure returns the name of the Makefile used by the kernel
# configuration project pointed to by <hProj>.
#
# If <makefile> is provided, it is set as the Makefile name for the given
# project. Only the tail of the given <makefile> is kept. It means that
# if <makefile> contains a path, only the last element of the path is kept
#
# PARAMETERS:
#   hProj : handle of the project to get Makefile name for
#   makefile : name of the Makefile to give to the given project
#
# RETURNS: The name of the Makefile used by the given project
#
# ERRORS: N/A
#
# SEE ALSO: N/A
#

proc prjMakefile {hProj {makefile ""}} {

    if {$makefile != ""} {
	prjTagDataSet $hProj MAKEFILE [file tail $makefile]
    }

    set res	[prjTagDataGet $hProj MAKEFILE]

    # check that the Makefile was set. Default is Makefile

    if {$res == ""} {
	return Makefile
    } else {
	return $res
    }
}

# source required files

set rcDir	[file join [Wind::base] host resource osconfig tcl]

# There is no need to use prjXmlLib now, but keep the code to not fall in the
# host saving trick anymore
#
# # XXX : fle : sourcing the prjXmlLib.tcl library overrides host variable if
# # declared. Keep its value, and restore it if needed
# 
# set savedHost		0
# if {[info var host] == "host"} {
#     set savedHostValue	$host
#     set savedHost	1
# }
# 
# source [file join $rcDir prjXmlLib.tcl]
# 
# if {$savedHost == 1} {
#     set host	$savedHostValue
# }

source [file join $rcDir prjFileLib.tcl]
source [file join $rcDir prjFsFileLib.tcl]
source [file join $rcDir prjBuildLib.tcl]

foreach fileName [glob -nocomplain [file join $rcDir prj_*.tcl]] {

    source $fileName
}
