#!/bin/sh
# The next line restarts using tclsh \
exec tclsh "$0" "$@"

# vxprj.tcl - Wind River Os Configuration command line interface

# Copyright 2003-2006 Wind River Systems, Inc.

# The right to copy, distribute, modify, or otherwise make use 
# of this software may be licensed only pursuant to the terms 
# of an applicable Wind River license agreement. 

# modification history
# --------------------
# 02t,19mar06,zhr  Fixed SPR#114711: Bad handling of unknown commands 
# 02s,14mar06,pee  added switch for scaling BSPs. SPR 119056
# 02r,21Feb06,pee  optimization.
# 02q,10feb06,zhr  Added enhanced component check
# 02s,09jan06,pee  avoid adding dependencies explicitly.
# 02r,05dec05,pee  New component handling strategy
# 02q,25Oct05,zhr  Added new display messages for error reporting
# 02o,27sep05,zhr  SPR#112925 : SYNOPSIS for vxprj::create is incomplete
# 02n,07sep05,pee  Remove PROFILE_DEFAULT. Using Bsp DEFAULTS instead
# 02m,29aug05,pee  Using PROFILE_DEFAULT by default
# 02l,18Aug05,zhr  Fix the bug of blocking orginal vxprj::build command 
# 02k,08Aug05,pee  Updated source build functinality.
# 02j,17jul05,zhr  Added interface to enable building from source
# 02i,28apr05,edc  initialize workspace environment variable used in defs.link
#                  during project creation
# 02h,23dec04,c_c  removed references to foundation.
# 02g,28oct04,fle  SPR#103195 : Fixed profile facilty with current project file
# 02f,26oct04,fle  SPR#103039 : fixed file remove on Windows hosts
#                  + added inet6, fr and inet6_fr options to vxprj help
# 02e,22oct04,fle  Doc fixes
# 02d,11oct04,fle  SPR#102450 : vxprj docs fixes
# 02c,01oct04,fle  Added profile facility
#                  + added autoscale command
# 02b,28sep04,fle  Added the tccheck command
# 02a,08sep04,fle  Added -removeall option to create facility
#                  + added makefile facility
# 01z,01sep04,fle  Made it use Wind::foundation when needed
# 01y,08aug04,dlk  Added option build support in vxprj::create.
# 01y,14jul04,fle  Made component dependencies use cmpDependenciesGet
# 01x,25jun04,fle  SPR#98463 : fixed some typoes in the help messages
#                  + SPR#92427 : added the 'vxprj parameter setstring' command
#                    and a NOTE in the 'vxprj parameter' help message
# 01w,24may04,fle  Added buildrules facility
#                  + SPR#97115 : exit on error when bundle does not exist
#                  + SPR#97719 : exit on error on project creation failure
#		   + removed fsfile documentation
# 01v,19may04,fle  Fixed vxprj build to be able to build build specs
# 01u,14may04,fle  Fixed empty components list parsing
# 01t,06may04,fle  Removed all bullet lists refgen markups
#                  + made it require PrjUtils package
# 01s,29apr04,fle  Added the vxprj file facility
# 01r,30mar04,fle  Fixed prj::parameter::diff call
# 01q,10mar04,fle  Fixed wpj file detection in vxprj create
# 01p,27feb04,fle  Added possibility to specify more
#                  components/parameters/bundle when creating project
#                  + added buildmacro facility
# 01o,24feb04,fle  Adapted to new vxprj name
# 01n,13feb04,fle  SPR#93993 : made vxprj create verify bsp
# 01m,11feb04,fle  Added fsfile create command
# 01l,02feb04,fle  Added and documented the fsfile facility
#                  + fixed a problem of 'vxprj create' which was sometimes
#                    creating the project file in the current directory
#                  + made it possible to specify a BSP directory to vxprj create
#                  + made it possible to specify a project directory rather than
#                    a project file in vxprj create
#                  + added vxprj copy
# 01k,10dec03,fle  Validate components when adding or removing them from
#                  project
#                  + SPR#92523 : create the project parent directory if possible
#                  + added fsfile facility
#                  + use askForHelp procedure rather than a regexp on argv
#                  + fixed 'vxprj build get' ...
# 01j,02dec03,fle  Removed deprecated calls, and added 'vxprj build' commands
#                  + fixed typo on vxprj::component dependencies errors
# 01i,28nov03,fle  Added bundles support
# 01h,14nov03,fle  Added components dependencies queries
# 01g,14nov03,fle  Fixed TGT_DIR uses
# 01f,14oct03,fle  added vxprj delete facility
#                  + fixed deprecated calls
# 01e,13oct03,fle  SPR#90923 : made vxprj create able to deal with more
#                  toolchains
#                  + fixed 'vxprj diff' arguments parsing
# 01d,29sep03,fle  Added unavailable to components command, and fixed some
#                  other oddities
# 01c,04sep03,fle  added new functionalities
# 01b,29aug03,fle  Removed some log messages
#                  + added the -force option to vxprj::create
# 01a,22aug03,fle  written

# SYNOPSIS
# \cs
#   vxprj help
#         autoscale  [-terse|-shrink] [<prjfile>]
#         copy       [-force] [<prjfile>] <prjfile|prjdir>
#         create     [-force] [-source ][-b <bundle>] [-c <component>] [-inet6]
#                    [-fr] [-inet6_fr] [-noopt] [-p <parameter>=<parameterValue>]
#                    [-profile <profile>] <bsp|bspdir> <tool> [<prjfile|prjdir>]
#         delete     <prjfile>
#         diff       [<prjfile>] <prjfile|prjdir>
#         makefile   [<prjFile>] [<makefile>]
#         tccheck
#         build      {get|list|reset|set|source|lib} ...
#         buildmacro {add|get|help|list|remove|set|value} ...
#         buildrule  {add|dependencies|get|help|list|remove|set} ...
#         bundle     {add|get|help|list|remove} ...
#         component  {add|check|dependencies|diff|help|list|remove} ...
#         file       {add|list|remove} ...
#         module     {dependencies|help} ...
#         parameter  {diff|help|list|reset|set|setstring|value} ...
#         profile    {get|help|list} ...
# \ce
#
# DESCRIPTION
# This tool is a kernel configuration command line interface. It should allow
# to manipulate a kernel configuration project file through simple command line
# requests.
#
# As it is designed to be a command-line tool, each of the 'vxprj' request does
# an 'open' and 'close' action on the kernel configuration file. Those
# actions may take time if the file system is slow, or if the content of the
# project file changed. To perform several actions in one shot, it is
# recommended to use the 'cmpScriptLib' Tcl library, which this tool is based
# on.
#
# This tool handles several modules, such as 'build', 'bundle', 'component',
# 'file' or 'parameter' manipulation, in order to make kernel
# configuration the more flexible possible.
#
# PARAMETERS:
#   help : print the 'vxprj' tool help message
#   autoscale : auto scales the current kernel configuration. Please refer to
#               'vxprj::autoscale'
#   build : give access to the build facilities. Please see 'vxprj::build'
#   buildmacro : give access to the build facilities. Please see
#                'vxprj::buildmacro'
#   buildrule : give access to the build facilities. Please see
#               'vxprj::buildrule'
#   bundle : give access to the bundle manipulation module. Please see
#            'vxprj::bundle'
#   component : give access to the components manipulation module. Please see
#               'vxprj::component'.
#   copy : copy current project to given destination, please see 'vxprj::copy'
#   create : create a new project based on a <bsp> and a <tool>. Please see
#            'vxprj::create'.
#   file : give access to the kernel files configuration facility. Please see
#          'vxprj::file'
#   makefile : get or set the kernel configuration project Makefile. Please
#              see 'vxprj::makefile'
#   parameter : give access to the parameters manipulation module. Please see
#              'vxprj::parameter'.
#   profile : give access to the profile manipulation module. Please see the
#             'vxprj::profile' help
#   tcheck : check if the toolchain infogrmation files are up to date, and
#            regenerate them if needed
#
# RETURNS: N/A
#
# EXAMPLES:
# Ask for help
# \cs
#   vxprj help
# \ce
# Create a new project based on <simpc> bsp, <gnu> toolchain in the default
# WIND_BASE/target/proj/simpc_gnu/simpc_gnu.wpj project file
# \cs
#   vxprj create simpc gnu
# \ce
# Create a new project based on <simpc> bsp, <gnu> toolchain in the named
# my_proj in the C:\temp directory :
# \cs
#   vxprj create simpc gnu C:\temp\my_proj.wpj
# \ce
# Get the parameters and components differences between current directory's
# project and ../mv2604_gnu/mv2604_gnu.wpj project
# \cs
#   vxprj diff ../mv2604_gnu
# \ce
# Add the INCLUDE_SHELL component to project in the current directory
# \cs
#   vxprj component add INCLUDE_SHELL
# \ce
# List all the parameters which name matches WDB, and value matches 0x of the
# project in the current directory.
# \cs
#   vxprj parameter value WDB 0x
# \ce
#
# REQUIRES:
# \is
# \i 'PrjUtils' : Kernel configuration utilities
# \i 'Wind' : Wind River specific routines
# \ie
#
# ERRORS: N/A
#
# SEE ALSO: cmpScriptLib, dataDocLib
#

package require PrjUtils
package require Wind

namespace eval vxprj {

    namespace export build bundle component copy create _file fsfile help \
		     parameter

    namespace eval component {

	namespace export add check dependencies dependDetail diff help lst project remove 

	variable mExcluded	{}	;# excluded components cache
	variable mIncluded	{}	;# included components cache
	variable mHProj		""	;# current project handle (if any)
    }

    namespace eval module {

	variable mSupportedRequests	[list dependencies]

	namespace export dependencies
    }

    namespace eval parameter {

	namespace export add diff check help lst _set remove
    }

    namespace eval bundle {

	namespace export add get lst remove
    }

    namespace eval famous {

	namespace export sayings
    }

    namespace eval build {

	namespace export get help source lib lst reset _set
    }

    namespace eval _file {

	variable mSupportedRequests	[list add get "list" remove]

	namespace export add list remove
    }

    namespace eval fsfile {

	variable mSupportedRequests	[list add create get help "list" remove]

	namespace export add get help lst remove
    }

    namespace eval buildmacro {

	variable mSupportedRequests	[list add get help "list" remove set \
					      value]

	namespace export add get help lst remove _set
    }

    namespace eval buildrule {

	variable mSupportedRequests	[list add dependencies get help "list" \
					      remove set]

	namespace export add current dependencies get help lst remove _set
    }

    namespace eval profile {

	variable mSupportedRequests	[list get help "list"]

	namespace export get help lst
    }


    # -------------------- component namespace procedures -------------------- #

    ############################################################################
    #
    # vxprj::component::add - add a set of components to a given project
    #
    # SYNOPSIS
    # \cs
    #   vxprj::component::add <prjfile> <component> [<component> ...]
    # \ce
    #
    # DESCRIPTION
    # This procedure allows to add a set of components to the given <prjfile>
    # project.
    #
    # If a folder is specified (e.g. FOLDER_NETWORK), all default components
    # below that folder are added.
    #
    # PARAMETERS:
    #   prjfile : path to the project file to add components to
    #   component : name of a project component to be added to the project
    #
    # RETURNS: The 'cmpValidate' results if any, or an empty string if
    # 'cmpValidate' had nothing to say
    #
    # ERRORS:
    # \is
    # \i Unknown project file <prjfile>
    # \ie
    #

    proc component::add {prjFile component args} {

	set components	$component

	# check if the project file exists

	if {! [file exists $prjFile]} {
	    error "Unknown project file \"$prjFile\""
	}

	# create the list of components if needed

	foreach elt $args {
	    lappend components $elt
	}

	projectFileOpen $prjFile
        set hProj   [cmpProjHandleGet]

        set preBuildMode  [cmpBuildModeGet $hProj]

	# get the list of components already included

	set included	[lindex [lst "" included] 0]
	set excluded	[lindex [lst "" excluded] 0]

	# check if components are really to be added

	set compsToAdd	{}

	set existing [concat $included $excluded]

	foreach comp $components {
	    if {[lsearch -exact $existing $comp] == -1} {
		puts stdout "Component $comp does not exist ..."
	    } else {
		lappend compsToAdd $comp
	    }
	}

	# now add the needed components (if any)

	if {[llength $compsToAdd] != 0} {

	    cmpAdd $compsToAdd
	}

	# validate component configuration

	set validate [cmpValidate]

        # retrieve build Mode and check if the build mode has been changed 
        # from source to library. If so
        # show the unspported components for source build

        if { ($preBuildMode == "source") && ($preBuildMode != [cmpBuildModeGet $hProj]) } {
             set unsupportedCompList [cmpUnsupportedSBCompsGet $hProj]
             if { $unsupportedCompList != "" } {
                 puts stdout "The following components are not supported for source build: \n$unsupportedCompList"   
                 puts stdout "\nProject can not be built from source."
                 puts stdout "Build mode has been changed to libraries."
             }         
        }
        
	projectFileClose 1

	return $validate
    }

    ############################################################################
    #
    # vxprj::component::check - check a project or one of its components
    #
    # SYNOPSIS
    # \cs
    #   vxprj::component::check <prjfile> [<component> [<component> ...]]
    # \ce
    #
    # DESCRIPTION
    # This procedure allows to check a project file or one of the project's
    # possible components.
    #
    # Some messages are sent to the standard output to let the user know what
    # shall be done for its project file, or for the specified component(s).
    #
    # PARAMETERS:
    #   prjfile : path to the project file to be checked
    #   component : name of a component to be checked
    #
    # RETURNS: N/A
    #
    # ERRORS:
    # \is
    # \i Unknown project file <prjfile>
    # \ie
    #

    proc component::check {prjFile args} {

	set components {}

	# check if the project file exists

	if {! [file exists $prjFile]} {
	    error "Unknown project file \"$prjFile\""
	}

	# create the list of components if needed

	foreach elt $args {
	    lappend components $elt
	}

	projectFileOpen	$prjFile

	if {[llength $components] == 0} {
	    cmpTest
	} else {
	    cmpTest $components
	}

	projectFileClose 1
    }

    ############################################################################
    #
    # vxprj::component::dependencies - get a component dependencies
    #
    # SYNOPSIS
    # \cs
    #   vxprj::component::dependencies <prjFile> <component> [<component> ...]
    # \ce
    #
    # DESCRIPTION
    # This procedure returns a list of the dependencies the given components
    # have.
    #
    # First element of the returned list is a list of components the given
    # <component> depends on.
    #
    # Second element of the list is a list of modules <component> depends on,
    # and the way they are dependent. This second list contains <n> elements,
    # where <n> is the number of modules required by <component>.
    #
    # Each element contains <i> elements, where <i> is the number of modules
    # required by each of the modules. First element is the module itself, then
    # a list of dependencies.
    #
    # Each dependency contains the name of the module required, and the names
    # of the symbols which were required in that module.
    #
    # PARAMETERS:
    #   prjFile : file name of the project to look at
    #   component : name of the component to get dependencies for
    #
    # RETURNS: A list of dependencies
    #
    # SEE ALSO: cmpDependsGet, cmpModuleDependsGet
    #

    proc component::dependencies {prjFile args {opt ""}} {

	variable mHProj

	set components {}

	# check if the project file exists

	if {! [file exists $prjFile]} {
	    error "Unknown project file \"$prjFile\""
	}

	# create the list of components if needed

	foreach elt $args {
	    lappend components $elt
	}

	# check that there is at least one component to look at

	if {[llength $components] <= 0} {
	    error "vxprj::component::dependencies : Please provide with a\
		   component name"
	}

	projectFileOpen	$prjFile

	set cxrDoc	[cxrDocCreate $mHProj]
	set cxrSet	[$cxrDoc setCreate]

	$cxrSet = $components
        
        if { $opt == "detail"} {
            set cmpDepends		[cmpDependenciesGet $components "extended"]
        } else {
	    set cmpDepends		[cmpDependenciesGet $components]
        }

	$cxrSet delete
#	cxrDocDelete $mHProj

	projectFileClose 1

	return $cmpDepends
    }

    ############################################################################
    #
    # vxprj::component::remove - remove a set of components to a given project
    #
    # SYNOPSIS
    # \cs
    #   vxprj::component::remove <prjfile> <component> [<component> ...]
    # \ce
    #
    # DESCRIPTION
    # This procedure allows to remove a set of components from the given
    # <prjfile> project.
    #
    # PARAMETERS:
    #   prjfile : path to the project file to remove components from
    #   component : name of a project component to be removed from the project
    #
    # RETURNS: The 'cmpValidate' results if any, or an empty string if
    # 'cmpValidate' had nothing to say
    #
    # ERRORS:
    # \is
    # \i Unknown project file <prjfile>
    # \ie
    #

    proc component::remove {prjFile component args} {

	set components	$component

	# check if the project file exists

	if {! [file exists $prjFile]} {
	    error "Unknown project file \"$prjFile\""
	}


	# create the list of components if needed

	foreach elt $args {
	    lappend components $elt
	}

	projectFileOpen $prjFile
        set mHProj   [cmpProjHandleGet]
        set preBuildMode  [cmpBuildModeGet $mHProj]

	# get the list of included components to be sure it is a component to
	# be removed

	set included	[lindex [lst "" included] 0]

	foreach comp $components {
	    if {[set ix [lsearch -exact $included $comp]] == -1} {
		puts stdout "Component $comp is not included ..."
		set components	[lreplace $components $ix $ix]
	    }
	}

	if {[llength $components] != 0} {
	    cmpRemove $components
	}

	# validate component configuration

	set validate [cmpValidate]

        if { ($preBuildMode == "library") && ($preBuildMode != [cmpBuildModeGet $mHProj]) } {
          
             puts stdout "All remaining components can be built from source"
             puts stdout "Build mode has been changed to source\nDone"
            
        }

	projectFileClose 1

	return $validate
    }

    ############################################################################
    #
    # vxprj::component::lst - get the list of components
    #
    # SYNOPSIS
    # \cs
    #   vxprj::component::lst <prjFile> [<type>] [<pattern>]
    # \ce
    #
    # DESCRIPTION
    # This procedure returns the list of components for project defined in
    # file <prjFile>.
    #
    # If the <mHProj> variable is initialised (a project is currently opened),
    # the <prjFile> parameter may be set to "". This will make this procedure
    # use the currently opened project.
    #
    # If <type> is specified, it may have three possible values :
    # \is
    # \i all : all possible components for the current project are returned
    #            (included and excluded, unavailable components are not listed)
    # \i included : only components included by the current project are
    #                 returned. This is the default behaviour.
    # \i excluded : only components not currently in the project are returned
    # \i unavailable : list components which are not installed in the
    #                    target tree, or which belong to uninstalled components.
    # \ie
    #
    # If <type> has any other value, it is considered as the <pattern>
    # parameter.
    #
    # If <pattern> is specified, only components which names match <pattern>
    # are returned.
    #
    # PARAMETERS:
    #   prjFile : project file containing the project to get components for
    #   type : type of components which should be returned, may have any of the
    #          {all|excluded|included|unavailable} values.
    #   pattern : pattern the component names should match
    #
    # RETURNS: Two lists. First list id the list of components for the current
    # project matching <pattern>, second list is the list of validation problems
    # for this project.
    #
    # SEE ALSO: vxprj::component, vxprj::component::add,
    # vxprj::component::remove
    #

    proc component::lst {prjFile {type "included"} {pattern ""}} {

	variable mIncluded
	variable mExcluded
	variable mHProj

	set closePrjFile	1

	# determine the type of parameter

	if {($type != "all") && ($type != "excluded") && \
	    ($type != "included") && ($type != "unavailable") && \
            ($type != "unavailableDetail")} {

	    set pattern $type
	    set type	"included"
	}

	# if a project file is set to "" and a project handle exists, we may
	# use the already opened project

	if {$prjFile == ""} {
	    set closePrjFile	0
	}

	projectFileOpen $prjFile

	switch -- $type {

	    all {

		# use cache variables if possible

		if {([llength $mIncluded] != 0) &&
		    ([llength $mExcluded] != 0)} {

		    set excluded	$mExcluded
		    set included	$mIncluded
		} else {

		    set cxrDoc	[cxrDocCreate $mHProj]
		    set excluded [format "%s" [$cxrDoc instances Component]]
		    set included [cmpListGet]
#		    cxrDocDelete $mHProj
		}

		# depending on the biggest list, append to the resulting list
		# only components which are not in the biggest list

		if {[llength $excluded] > [llength $included]} {
		    set parsedList	$included
		    set components	$excluded
		} else {
		    set parsedList	$excluded
		    set components	$included
		}

		foreach elt $parsedList {
		    if {[lsearch -exact $components $elt] == -1} {
			lappend components $elt
		    }
		}
	    }

	    unavailable {

		# XXX : fle : seems like the cmpUnavailListGet returns a Tcl
		# list which is badly initialised. To be sure the result list
		# is valid, we have to make it go through a format call ...

		set components	[format "%s" [cmpUnavailListGet]]
	    }

	    unavailableDetail {
                 cmpUnavailListDetailGet
                 set components ""
	    }

	    excluded {

		if {[llength $mExcluded] != 0} {
		    set components	$mExcluded
		} else {
		    set cxrDoc [cxrDocCreate $mHProj]
		    set components [format "%s" [$cxrDoc instances Component]]

		    # the above command lists all possible components. We have
		    # now to remove the included components to really have the
		    # list of excluded components

		    set included [cmpListGet]

		    foreach comp $included {

			if {[set ix [lsearch -exact $components $comp]] != -1} {
			    set components [lreplace $components $ix $ix]
			}
		    }

		    set mExcluded $components
#		    cxrDocDelete $mHProj
		}
	    }

	    included {

		# use compoent lists cache if possible

		if {[llength $mIncluded] != 0} {
		    set components	$mIncluded
		} else {
		    set components	[cmpListGet]
		    set mIncluded	$components
		}
	    }
	}

        if { ($type == "unavailable") || ($type == "unavailableDetail") } {
        	set validation "" 
        } else {
        	set validation [cmpValidate]
        }

	projectFileClose $closePrjFile

	# now try to match the pattern

	if {$pattern != ""} {

	    set tmpComponents	$components
	    set components	{}

	    foreach component $tmpComponents {
		if {[regexp $pattern $component]} {
		    lappend components	$component
		}
	    }

	    unset tmpComponents
	}

	return [list $components $validation]
    }

    ############################################################################
    #
    # vxprj::component::diff - get the component difference between two projects
    #
    # SYNOPSIS
    # \cs
    #   vxprj::component::diff <prjFile> <withPrjFile>
    # \ce
    #
    # DESCRIPTION
    # This procedure gets the list of components included by project in
    # <prjFile>, and compares it with the list of components contained in
    # project defined by <withPrjFile>.
    #
    # It returns two lists of components. The first list contains the
    # components of <prjFile> which are not in <withPrjFile>, the second list
    # contains the components of <withPrjFile> which are not in the list
    # of components of <prjFile>.
    #
    # NOTE
    # The parameter <withPrjFile> may be a directory. In this case, the projet
    # file is assumed to have the same name than its parent directory. For
    # example, if <withPrjFile> is set to <c:\temp>, then the project file is
    # assumed to be <c:\temp\temp.wpj>.
    #
    # PARAMETERS:
    #   prjFile : project file to get included components from
    #   withPrjFile : project file to compare component list with components of
    #                 <prjFile>.
    #
    # RETURNS: Two lists of components (se above for details).
    #
    # SEE ALSO: vxprj::component::lst
    #
    #

    proc component::diff {prjFile withPrjFile} {

	# check if <withPrjFile> is a directory

	if {[file isdirectory $withPrjFile]} {
	    set withPrjFile [ \
		file join $withPrjFile [format "%s.wpj" \
					       [file tail $withPrjFile]] \
	    ]
	}

	set prjComponents	[lindex [vxprj::component::lst $prjFile] 0]
	set withPrjComponents	[lindex [vxprj::component::lst $withPrjFile] 0]

	set prjComponents [concat [lindex $prjComponents 0] [lindex $prjComponents 1]]
	set withPrjComponents [concat [lindex $withPrjComponents 0] [lindex $withPrjComponents 1]]

	set onlyPrj {}
	set onlyWithPrj {}

	# gather all components which are present only in prjFile components

	foreach component $prjComponents {
	    if {[lsearch -exact $withPrjComponents $component] == -1} {
		lappend onlyPrj $component
	    }
	}

	# gather all components which are present only in withPrjFile components

	foreach component $withPrjComponents {
	    if {[lsearch -exact $prjComponents $component] == -1} {
		lappend onlyWithPrj $component
	    }
	}

	return [list $onlyPrj $onlyWithPrj]
    }


    ############################################################################
    #
    # vxprj::component::project - check project's build mode, images size and
    #                   component availablity 
    #

    proc component::project {prjFile  {type ""} args} {

        global cxrUnavailInfo
        set projUnavailComps {}

        # check if the project file exists

	if {! [file exists $prjFile]} {
	    error "Unknown project file \"$prjFile\""
	}
  
        projectFileOpen $prjFile
        set hProj   [cmpProjHandleGet]
        switch -- $type {

	    check {
                     puts stdout "\nProject build mode is [cmpBuildModeGet $hProj]"
              
                     set included  [cmpListGet]
                     set cxrDoc [cxrDocCreate $hProj]
                     set mxrDoc [mxrDocCreate $hProj]
                     set cmpUnavailList [cmpUnavailListGet]
                     array set misCompsObj [lindex $cxrUnavailInfo 3]
                     puts stdout "\nCheck all included components...\n" 

                     foreach component $included {
                         if {[lsearch $cmpUnavailList $component] != -1 } {
                             if { [lsearch [array names misCompsObj] $component] !=-1 } {
                                 puts stdout "$component is missing $misCompsObj($component)"
                              } else {
                                lappend projUnavailComps $component
                             }
                          }
                     }
                     if {$projUnavailComps !=""} {
                        puts stdout "\nUse 'vxprj component check' to find reasons for the following components:\n"
                        foreach c $projUnavailComps {
                            puts stdout "$c"
                        }                 
                     }
                     puts stdout "Done\n" 
             } 
             default {
                vxprj::component::help
             } 
        }     
    }

 
    ############################################################################
    #
    # vxprj::component::help - print vxprj component help message
    #
    # SYNOPSIS
    # \cs
    #   vxprj::component::help [<thenExit>]
    # \ce
    #
    # DESCRIPTION
    # This procedure simply print the usage message for the 'vxprj' tool,
    # 'component' facilities, and exits the program with status '-1'.
    #
    # PARAMETERS: N/A
    #
    # RETURNS: N/A
    #
    # ERRORS: N/A
    #

    proc component::help {} {

	puts stdout "usage: vxprj component add          \[<prjfile>]\
		     <component> \[<component> ...]"
        
	puts stdout "                       check        \[<prjfile>]"

	puts stdout "                       check        \[<prjfile>]\
		      <component>"        
        puts stdout "                       project      \[<prjfile>]\
                                            check"

	puts stdout "                       dependencies \[<prjfile>]\
		     <component>"
        puts stdout "                       dependDetail \[<prjfile>]\
		     <component>"
	puts stdout "                       diff         \[<prjfile>]\
		     <prjfile|prjdir>"
	puts stdout "                       list         \[<prjfile>] \\"
	puts stdout "\t\t\t\t    \[{all|excluded|included|unavailable|unavailableDetail}] \\"
	puts stdout "                                    \[<pattern>]"
	puts stdout "                       remove       \[<prjfile>]\
		     <component> \[<component> ...]"
        

	puts stdout "\nwhere:"
	puts stdout "    component : name of a component"
	puts stdout "    pattern   : pattern the component name should match"
	puts stdout "    prjfile   : project file path"
	puts stdout "    prjdir    : project directory containing a .wpj\
		     file with the same name"
	puts stdout "                than the directory name (e.g if prjdir =\
		     myDir/myProj then"
	puts stdout "                prjfile = myDir/myProj/myProj.wpj)"
	exit -1
    }

    # ---------------- component namespace private procedures ---------------- #

    ############################################################################
    #
    # vxprj::component::projectFileOpen - Open a project file if needed
    #
    # SYNOPSIS
    # \cs
    #   vxprj::component::projectFileOpen <prjFile>
    # \ce
    #
    # DESCRIPTION
    # This procedure opens the given project file and initialised the components
    # cache variables.
    #
    # If <prjFile> is an empty string (""), nothing is done. Else, the project
    # file is opened calling for 'cmpProjOpen', and the project handle,
    # excluded and included components list caches are updated.
    #
    # PARAMETERS:
    #   prjFile : the project file to be opened, or "" if there is already an
    #             opened project file
    #
    # RETURNS: N/A
    #
    # SEE ALSO: cmpProjOpen, vxprj::component::projectFileClose
    #
    # NOMANUAL
    #

    proc component::projectFileOpen {prjFile} {

	variable mIncluded
	variable mExcluded
	variable mHProj

	if {$prjFile != ""} {

	    cmpProjOpen $prjFile

	    # as soon as we open a new project file, included and excluded
	    # components list caches are wrong

	    set mIncluded	{}
	    set mExcluded	{}
	    set mHProj		[cmpProjHandleGet]
	}
    }

    ############################################################################
    #
    # vxprj::component::projectFileClose - Close a project file if needed
    #
    # SYNOPSIS
    # \cs
    #   vxprj::component::projectFileOpen <closeIt>
    # \ce
    #
    # DESCRIPTION
    # This procedure osespens the current project resets the components
    # cache variables.
    #
    # If <closeIt> is set to '1', the 'cmpProjClose' procedure is called, and
    # the project handle, included and excluded components lists caches are
    # reset.
    #
    # PARAMETERS:
    #   closeIt : '1' if the project should actually be closed, '0' else.
    #
    # RETURNS: N/A
    #
    # SEE ALSO: cmpProjClose, vxprj::component::projectFileOpen
    #
    # NOMANUAL
    #

    proc component::projectFileClose {closeIt} {

	variable mIncluded
	variable mExcluded
	variable mHProj

	if {$closeIt == 1} {
	    cmpProjClose
	    set mHProj		""
	    set mIncluded	{}
	    set mExcluded	{}
	}
    }

    # -------------------- parameter namespace procedures -------------------- #

    ############################################################################
    #
    # vxprj::parameter::_set - set a parameter to a given project
    #
    # SYNOPSIS
    # \cs
    #   vxprj::parameter::_set <prjfile> <parameter> <value>
    # \ce
    #
    # DESCRIPTION
    # This procedure allows to set a parameter to the given <prjfile> project.
    #
    # The <parameter>'s value is set to <value>.
    #
    # If the <parameter> is not member of the parameter list returned by
    # 'vxprj::parameter::list', the procedure exits with a warning printed on
    # the standard error stream.
    #
    # PARAMETERS:
    #   prjfile : path to the project file to add components to
    #   parameter : name of a parameter to be set for the project
    #   value : value to give to the given <parameter>
    #
    # RETURNS: N/A
    #
    # ERRORS:
    # \is
    # \i Unknown project file <prjfile>
    # \ie
    #

    proc parameter::_set {prjFile parameter {value ""}} {

	# check if the project file exists

	if {! [file exists $prjFile]} {
	    error "Unknown project file \"$prjFile\""
	}

	cmpProjOpen $prjFile

	set hProj	[cmpProjHandleGet]
	set wcc		[wccCurrentGet $hProj]

	# if there is no Wind Component Configuration, we may be in an
	# application project, just return.

	if {$wcc == ""} {
	    cmpProjClose
	    return
	}

	# do not set parameters which are not in the list of parameters

	set parameters	[format "%s" [wccParamListGet $hProj $wcc]]

	if {[lsearch -exact $parameters $parameter] != -1} {
	    wccParamSet $hProj $wcc $parameter $value
	} else {
	    puts stderr "*** WARNING ***"
	    puts stderr "Parameter $parameter does not exist. Aborting ..."
	}

	cmpProjClose
    }

    ############################################################################
    #
    # vxprj::parameter::setstring - set a parameter value between double quotes
    #
    # SYNOPSIS
    # \cs
    #   vxprj::parameter::setstring <prjFile> <parameter> <value>
    # \ce
    #
    # DESCRIPTION
    # Set a parameter value in double quotes. Whatever the given <value> is, it
    # will be set between double quotes, and any subsequent call to 'get', will
    # return the value between double quotes.
    #
    # PARAMETERS:
    #   prjFile : project file to set parameter value for
    #   parameter : name of the parameter to set a string value for
    #   value : value to set for the parameter, and to surround with double
    #           quotes
    #
    # RETURNS: N/A
    #
    # ERRORS: see 'vxprj::parameter::_set'
    #
    # SEE ALSO: vxprj::parameter::_set
    #

    proc parameter::setstring {prjFile parameter {value ""}} {
	_set $prjFile $parameter "\"$value\""
    }

    ############################################################################
    #
    # vxprj::parameter::reset - reset a set of parameters from a given project
    #
    # SYNOPSIS
    # \cs
    #   vxprj::parameter::reset <prjfile> <parameter> [<parameter> ...]
    # \ce
    #
    # DESCRIPTION
    # This procedure allows to remove a set of parameters from the given
    # <prjfile> project. This makes the given parameters get their default
    # value when the project is built.
    #
    # PARAMETERS:
    #   prjfile : path to the project file to reset parameters for
    #   parameter : name of a project parameter to be reset in the project
    #
    # RETURNS: N/A
    #
    # ERRORS:
    # \is
    # \i Unknown project file <prjfile>
    # \ie
    #

    proc parameter::reset {prjFile parameter args} {

	set parameters	$parameter

	# check if the project file exists

	if {! [file exists $prjFile]} {
	    error "Unknown project file \"$prjFile\""
	}

	# create the list of parameters if needed

	foreach elt $args {
	    lappend parameters $elt
	}

	cmpProjOpen $prjFile

	set hProj	[cmpProjHandleGet]
	set wcc		[wccCurrentGet $hProj]

	# if there is no Wind Component Configuration, we may be in an
	# application project, just return.

	if {$wcc == ""} {
	    cmpProjClose
	    return
	}

	# now remove the parameters

	foreach param $parameters {
	    wccParamRemove $hProj $wcc $param
	}

	cmpProjClose
    }

    ############################################################################
    #
    # vxprj::parameter::lst - get the list of parameters for <prjFile>
    #
    # SYNOPSIS
    # \cs
    #   vxprj::parameter::lst <prjFile> [<pattern>]
    # \ce
    #
    # DESCRIPTION
    # This procedure returns the list of parameters for project defined in
    # file <prjFile>.
    #
    # If <pattern> is specified, only parameters which names match <pattern>
    # are returned.
    #
    # PARAMETERS:
    #   prjFile : project file containing the project to get parameters for
    #   pattern : pattern the parameter names should match
    #
    # RETURNS: The list of parameters for the current project matching <pattern>
    #

    proc parameter::lst {prjFile {pattern ""}} {

	cmpProjOpen $prjFile

	set hProj	[cmpProjHandleGet]
	set wcc		[wccCurrentGet $hProj]

	# if there is no Wind Component Configuration, we may be in an
	# application project, just return.

	if {$wcc == ""} {
	    cmpProjClose
	    return {}
	}

	# XXX : fle : seems like the wccParamList returns a Tcl list which is
	# badly initialised. To be sure the result list is valid, we have to
	# make it go through a format call ...

	set parameters	[format "%s" [wccParamListGet $hProj $wcc]]

	cmpProjClose

	# now try to match the pattern

	if {$pattern != ""} {

	    set tmpParameters	$parameters
	    set parameters	{}

	    foreach component $tmpParameters {
		if {[regexp $pattern $component]} {
		    lappend parameters	$component
		}
	    }

	    unset tmpParameters
	}

	return [lsort -ascii $parameters]
    }

    ############################################################################
    #
    # vxprj::parameter::value - get the list of parameters and their value
    #
    # SYNOPSIS
    # \cs
    #   vxprj::parameter::value <prjFile> [<namepattern> [<valuepattern>]]
    # \ce
    #
    # DESCRIPTION
    # This procedure returns the list of parameters for project defined in
    # file <prjFile> and their associated value.
    #
    # If <namepattern> is specified, only parameters which names match
    # <namepattern> are returned.
    #
    # If <valuepattern> is specified, only parameters which value matches
    # <valuepattern> is returned. To get all parameters which value matches
    # <valuepattern>, the <namepattern> should be set to '.*'.
    #
    # PARAMETERS:
    #   prjFile : project file containing the project to get parameters for
    #   namepattern : pattern the parameter names should match
    #   valuepattern : pattern the parameter values should match
    #
    # RETURNS: A list of elements. Each element is a two elements list. First
    # element is the parameter name, second element is the parameter value.
    #

    proc parameter::value {prjFile {namepattern ""} {valuepattern ""}} {

	cmpProjOpen $prjFile

	set hProj	[cmpProjHandleGet]
	set wcc		[wccCurrentGet $hProj]

	# if there is no Wind Component Configuration, we may be in an
	# application project, just return.

	if {$wcc == ""} {
	    cmpProjClose
	    return {}
	}

	# XXX : fle : seems like the wccParamList returns a Tcl list which is
	# badly initialised. To be sure the result list is valid, we have to
	# make it go through a format call ...

	set parameters	[format "%s" [wccParamListGet $hProj $wcc]]
	set parameters	[lsort -ascii $parameters]

	set returnedList	{}

	foreach paramName $parameters {

	    # get the parameters and their values

	    set paramValue		[wccParamGet $hProj $wcc $paramName]

	    if {$namepattern != ""} {

		# match on parameter name

		if {[regexp $namepattern $paramName]} {

		    if {$valuepattern != ""} {

			# match on parameter value

			if {[regexp $valuepattern $paramValue]} {
			    lappend returnedList [list $paramName $paramValue]
			}
		    } else {
			lappend returnedList [list $paramName $paramValue]
		    }
		}
	    } else {

		# nothing to match, just add it

		lappend returnedList	[list $paramName $paramValue]
	    }
	}

	cmpProjClose

	return [lsort -ascii $returnedList]
    }

    ############################################################################
    #
    # vxprj::component::diff - get the component difference between two projects
    #
    # SYNOPSIS
    # \cs
    #   vxprj::component::diff <prjFile> <withPrjFile>
    # \ce
    #
    # DESCRIPTION
    # This procedure gets the list of parameters and their value, defined by
    # <prjFile> project, and compares it with the list of parameters (and their
    # associated values) defined by project <withPrjFile>.
    #
    # It returns two lists of parameters. The first list contains the
    # parameters of <prjFile> which are not in <withPrjFile>, the second list
    # contains the parameters of <withPrjFile> which are different from
    # parameters of <prjFile>.
    #
    # NOTE
    # The parameter <withPrjFile> may be a directory. In this case, the projet
    # file is assumed to have the same name than its parent directory. For
    # example, if <withPrjFile> is set to <c:\temp>, then the project file is
    # assumed to be <c:\temp\temp.wpj>.
    #
    # PARAMETERS:
    #   prjFile : project file to get included parameters from
    #   withPrjFile : project file to compare parameters list with parameters of
    #                 <prjFile>.
    #
    # RETURNS: Two lists of parameters (see above for details).
    #
    # SEE ALSO: vxprj::parameter::value
    #

    proc parameter::diff {prjFile withPrjFile} {

	set prjParamList	{}
	set withPrjParamList	{}

	# check if <withPrjFile> is a directory

	if {[file isdirectory $withPrjFile]} {
	    set withPrjFile [ \
		file join $withPrjFile [format "%s.wpj" \
					       [file tail $withPrjFile]] \
	    ]
	}

	set prjParameters	[vxprj::parameter::value $prjFile]
	set withPrjParameters	[vxprj::parameter::value $withPrjFile]

	# create strings made of <paramname> = <paramvalue>, and do the
	# comparison on those strings

	foreach param $prjParameters {
	    lappend prjParamList	[format "%s = %s" [lindex $param 0] \
						[lindex $param 1]]
	}

	foreach param $withPrjParameters {
	    lappend withPrjParamList	[format "%s = %s" [lindex $param 0] \
						[lindex $param 1]]
	}

	set onlyPrj {}
	set onlyWithPrj {}

	# gather all parameters which are present only in prjFile parameters

	foreach parameter $prjParamList {
	    if {[lsearch -exact $withPrjParamList $parameter] == -1} {
		lappend onlyPrj $parameter
	    }
	}

	# gather all parameters which are present only in withPrjFile parameters

	foreach parameter $withPrjParamList {
	    if {[lsearch -exact $prjParamList $parameter] == -1} {
		lappend onlyWithPrj $parameter
	    }
	}

	return [list $onlyPrj $onlyWithPrj]
    }

    ############################################################################
    #
    # vxprj::parameter::help - print vxprj parameter help message
    #
    # SYNOPSIS
    # \cs
    #   vxprj::parameter::help
    # \ce
    #
    # DESCRIPTION
    # This procedure simply print the usage message for the 'vxprj' tool,
    # 'parameter' facilities, and exits the program with status '-1'.
    #
    # PARAMETERS: N/A
    #
    # RETURNS: N/A
    #
    # ERRORS: N/A
    #

    proc parameter::help {} {

	puts stdout "usage: vxprj parameter diff      \[<prjfile>]\
		     <prjfile|prjdir>"
	puts stdout "                       list      \[<prjfile>]\
		     \[<namepattern>]"
	puts stdout "                       reset     \[<prjfile>] <parameter>\
		     \[<parameter> ...]"
	puts stdout "                       set       \[<prjfile>] <parameter>\
		     \[<value>]"
	puts stdout "                       setstring \[<prjfile>] <parameter>\
		     \[<value>]"
	puts stdout "                       value     \[<prjfile>]\
		     \[<namepattern> \[<valuepattern>]]"

	puts stdout "\nwhere:"
	puts stdout "    parameter    : name of a parameter"
	puts stdout "    namepattern  : pattern the parameter name should match"
	puts stdout "    valuepattern : pattern the parameter value should\
		     match"
	puts stdout "    prjfile      : project file path"
	puts stdout "    prjdir       : project directory containing a .wpj\
		     file with the same name"
	puts stdout "                   than the directory name (e.g dir=a/b\
		     => file=a/b/b.wpj)"
	puts stdout "\nNOTE: in order to set a parameter value to appear\
		     between double quotes,"
	puts stdout "      better use 'setstring', or escape the double\
		     quotes :"
	puts stdout "          Unixes  : vxprj parameter set MY_PARAM '\"my\
		     value\"'"
	puts stdout "          Windows : vxprj parameter set MY_PARAM \"\\\"my\
		     value\\\"\""
	exit -1
    }

    # --------------------------- bundle namespace --------------------------- #

    ############################################################################
    #
    # vxprj::bundle::lst - get the list of bundles for given project
    #
    # SYNOPSIS
    # \cs
    #   vxprj::bundle::lst <prjFile> [<pattern>]
    # \ce
    #
    # DESCRIPTION
    # This procedure gets the list of bundles for the project defined by
    # <prjFile>
    #
    # If <pattern> is provided, only bundles which name match <pattern> will
    # be returned.
    #
    # PARAMETERS:
    #   prjFile : project file path
    #   pattern : bundle name pattern to match
    #
    # RETURNS: The list of bundles which names match <pattern>
    #
    # SEE ALSO: vxprj::bundle::add, vxprj::bundle::get, vxprj::bundle::remove
    #

    proc bundle::lst {prjFile {type "included"} {pattern ""}} {

	# check the list type command first

	if {($type != "all") && ($type != "included") &&
	    ($type != "excluded")} {

	    set pattern	$type
	    set type	"included"
	}

	cmpProjOpen $prjFile

	switch -- $type {

	    all {
		set included	[cmpBundleListGet $pattern]
		set excluded	[cmpBundleExcludedListGet $pattern]
		set bundles	[lsort [concat $included $excluded]]
	    }

	    excluded {
		set bundles	[lsort [cmpBundleExcludedListGet $pattern]]
	    }

	    default {
		set bundles	[lsort [cmpBundleListGet $pattern]]
	    }
	}

	cmpProjClose

	return $bundles
    }

    ############################################################################
    #
    # vxprj::bundle::get - get a bundle description
    #
    # SYNOPSIS
    # \cs
    #   vxprj::bundle::get <bundle>
    # \ce
    #
    # DESCRIPTION
    # This procedure just returns the <bundle> properties and their associated
    # values, as described in 'cmpBundleGet' Tcl procedure.
    #
    # PARAMETERS:
    #   prjFile : project file path
    #   bundle : name of the bundle to get properties for
    #
    # RETURNS: A list of bundle properties and their associated values
    #
    # SEE ALSO: vxprj::bundle::lst
    #

    proc bundle::get {prjFile bundle} {

	cmpProjOpen $prjFile
	set bundleDesc [cmpBundleGet $bundle]
	cmpProjClose

	return $bundleDesc
    }

    ############################################################################
    #
    # vxprj::bundle::add - add a bundle (or several) to the current project.
    #
    # SYNOPSIS
    # \cs
    #   vxprj::bundle::add <prjfile> <bundle> [<bundle> ...]
    # \ce
    #
    # DESCRIPTION
    # This procedure adds all the components handled by <bundle> (and the
    # other bundles if any) to the current project.
    #
    # It returns the list of added, ignored, and errorneous components this
    # command handled. Ignored components are the components which were already
    # included by the project, while errorneous are the unavailable components.
    #
    # PARAMETERS:
    #   prjfile : path to the project file
    #   bundle : name of the bundle which components should be added to the
    #            current project. There may be several <bundle>
    #
    # RETURNS: Three lists of added, ignored and erroneous components.
    #
    # ERRORS: N/A
    #
    # SEE ALSO: vxprj::bundle::remove, vxprj::bundle::lst, vxprj::bundle::get
    #

    proc bundle::add {prjFile bundle args} {

	# gather bundle names

	set bundleList		$bundle
	set addedComps		{}
	set ignoredComps	{}
	set errorComps		{}

	foreach arg $args {

	    lappend bundleList $arg
	}

	# open project

	cmpProjOpen $prjFile

	set bundles		[cmpBundleExcludedListGet]

	foreach elt $bundleList {

	    if {! [catch {cmpBundleAdd $elt} compLists]} {

		set addedComps [concat $addedComps [lindex $compLists 0]]
		set ignoredComps [concat $ignoredComps [lindex $compLists 1]]
		set errorComps [concat $errorComps [lindex $compLists 2]]
	    } else {

		cmpProjClose
		error $compLists
	    }
	}

	# close the project and return the lists

	cmpProjClose

	return [list $addedComps $ignoredComps $errorComps]
    }

    ############################################################################
    #
    # vxprj::bundle::remove - remove a bundle (or several) from current project
    #
    # SYNOPSIS
    # \cs
    #   vxprj::bundle::remove <prjfile> <bundle> [<bundle> ...]
    # \ce
    #
    # DESCRIPTION
    # This procedure removes all the components handled by <bundle> (and the
    # other bundles if any) from the current project.
    #
    # It returns the list of removed, ignored, and errorneous components this
    # command handled. Ignored components are the components which were not
    # included by the project, while errorneous are the unavailable components.
    #
    # PARAMETERS:
    #   prjfile : path to the project file
    #   bundle : name of the bundle which components should be removed from the
    #            current project. There may be several <bundle>
    #
    # RETURNS: Three lists of removed, ignored and erroneous components.
    #
    # ERRORS: N/A
    #
    # SEE ALSO: vxprj::bundle::add, vxprj::bundle::lst, vxprj::bundle::get
    #

    proc bundle::remove {prjFile bundle args} {

	# gather bundle names

	set bundleList		$bundle
	set addedComps		{}
	set ignoredComps	{}
	set errorComps		{}

	foreach arg $args {
	    lappend bundleList $arg
	}

	# open project

	cmpProjOpen $prjFile

	foreach elt $bundleList {

	    if {! [catch {cmpBundleRemove $elt} compLists]} {

		set addedComps [concat $addedComps [lindex $compLists 0]]
		set ignoredComps [concat $ignoredComps [lindex $compLists 1]]
		set errorComps [concat $errorComps [lindex $compLists 2]]
	    }
	}

	# close the project and return the lists

	cmpProjClose

	return [list $addedComps $ignoredComps $errorComps]
    }

    ############################################################################
    #
    # vxprj::bundle::help - print vxprj bundle help message
    #
    # SYNOPSIS
    # \cs
    #   vxprj::bundle::help
    # \ce
    #
    # DESCRIPTION
    # This procedure simply prints the usage message for the 'vxprj' tool,
    # 'bundle' facilities, and exits the program with status '-1'.
    #
    # PARAMETERS: N/A
    #
    # RETURNS: N/A
    #
    # ERRORS: N/A
    #

    proc bundle::help {} {
	puts stdout "usage: vxprj bundle help"
	puts stdout "                    add    \[<prjfile>] <bundle>\
		     \[<bundle> ...]"
	puts stdout "                    get    \[<prjfile>] <bundle>"
	puts stdout "                    list   \[<prjfile>]\
		     \[{all|excluded|includes}] \[<pattern>]"
	puts stdout "                    remove \[<prjfile>] <bundle>\
		     \[<bundle> ...]"
	puts stdout "\nwhere:"
	puts stdout "    bundle  : name of a components bundle"
	puts stdout "    pattern : bundle name pattern to match"
	puts stdout "    prjfile : path to a project file"
	exit -1
    }

    # --------------------------- build namespace ---------------------------- #

    ############################################################################
    #
    # vxprj::build::get - get the current build specification
    #
    # SYNOPSIS
    # \cs
    #   vxprj::build::get <prjfile>
    # \ce
    #
    # DESCRIPTION
    # This procedure returns the current build specification of project defined
    # by <prjfile> file path.
    #
    # PARAMETERS:
    #   prjfile : project file path
    #
    # RETURNS: The current build specification used by <prjfile> project.
    #
    # ERRORS: N/A
    #
    # SEE ALSO : vxprj::build::_set, vxprj::build::list, vxprj::build::reset
    #

    proc build::get {prjFile} {

	cmpProjOpen $prjFile

	set hProj	[cmpProjHandleGet]
	set bld		[prjBuildCurrentGet $hProj]

	cmpProjClose

	return $bld
    }

    ############################################################################
    #
    # vxprj::build::lst - list all available build specifications for <prjFile>
    #
    # SYNOPSIS
    # \cs
    #   vxprj::build::lst <prjFile> [<pattern>]
    # \ce
    #
    # DESCRIPTION
    # This procedure lists all the build specifications of project specified
    # by <prjFile>.
    #
    # If <pattern> is provided, then only the build specifications which name
    # match <pattern> will be returned.
    #
    # PARAMETERS:
    #   prjFile : project file path
    #   pattern : build specification name pattern to match. Default is '.*'
    #
    # RETURNS: A list of build specifications matching <pattern>
    #
    # ERRORS: N/A
    #
    # SEE ALSO: vxprj::build::get, vxprj::build::reset, vxprj::build::set
    #

    proc build::lst {prjFile {pattern ""}} {

 	cmpProjOpen $prjFile
	set tmpBuildList [prjBuildListGet [cmpProjHandleGet]]
	cmpProjClose

	set buildList {}

	# only keep build specs matching "pattern" if any

	if {$pattern != ""} {

	    foreach bld $tmpBuildList {
		if {[regexp $pattern $bld]} {
		    lappend buildList $bld
		}
	    }

	} else {
	    set buildList	$tmpBuildList
	}

	return $buildList
    }

    ############################################################################
    #
    # vxprj::build::reset - reset the project build specification to 'default'
    #
    # SYNOPSIS
    # \cs
    #   vxprj::build::reset <prjfile>
    # \ce
    #
    # DESCRIPTION
    # This procedure simply resets the current build specification of project
    # defined by <prjFile>, to 'default'.
    #
    # PARAMETERS:
    #   prjfile : project file path
    #
    # RETURNS: The project default build specification, aka 'default'.
    #
    # ERRORS: N/A
    #
    # SEE ALSO: vxprj::build::get, vxprj::build::lst, vxprj::build::set
    #

    proc build::reset {prjFile} {

	set defaultBuildSpec	"default"

 	cmpProjOpen $prjFile
	prjBuildCurrentSet [cmpProjHandleGet] $defaultBuildSpec
	cmpProjClose

	return $defaultBuildSpec
    }

    ############################################################################
    #
    # vxprj::build::set - set the project build specification
    #
    # SYNOPSIS
    # \cs
    #   vxprj::build::set <prjFile> <buildspec>
    # \ce
    #
    # DESCRIPTION
    # This procedure checks if the given <buildspec> is a valid build
    # specification for project defined by <prjfile>.
    #
    # If the build specification is avilable, it is set as the current build
    # specification for the current project.
    #
    # If it is not a valid build specification (it is not retrned by
    # 'vxprj::build::lst'), the procedure exits with error (see the ERRORS
    # section)
    #
    # PARAMETERS:
    #   prjFile : path to the project
    #   buildspec : name of a build specification to set as the current build
    #               specification for project specified by <prjFile>
    #
    # RETURNS: The name of the so-set build specification
    #
    # ERRORS:
    # \is
    # \i Invalid build specification <buildspec> Please chose one of
    #      <speclist>.
    # \ie
    #
    # SEE ALSO: vxprj::build::get, vxprj::build::lst, vxprj::build::reset
    #

    proc build::_set {prjFile buildspec} {

 	cmpProjOpen $prjFile

	# first check that is is a valid build spec

	set hProj	[cmpProjHandleGet]
	set buildList	[prjBuildListGet $hProj]

	if {[lsearch -exact $buildList $buildspec] == -1} {

	    cmpProjClose
	    error "Invalid build specification \"$buildspec\"\nPlease choose\
		   one of : $buildList"
	}

	prjBuildCurrentSet [cmpProjHandleGet] $buildspec
	cmpProjClose

	return $buildspec
    }

    ############################################################################
    #
    # vxprj::build::source - switch the project build from library to source 
    #
    # SYNOPSIS
    # \cs
    #   vxprj::build::source <prjFile>
    # \ce
    #
    # DESCRIPTION
    # This procedure simply change the current build state of project from 
    # lib mode to source build.
    #
    # PARAMETERS:
    # prjFile 
    #
    # ERRORS: N/A
    #
    # SEE ALSO: vxprj::build::lib
    #

    proc build::source {prjFile} {

        if {! [file exists $prjFile]} {
            error "Unknown project file \"$prjFile\""
        }
        vxprj::component::projectFileOpen  $prjFile
        set mHProj   [cmpProjHandleGet]

        if { [cmpSourceBuildEnableGet $mHProj] } {
            puts stdout "Build mode is source already\nNothing to be done"
        } else { 
            cmpBuildSourceEnable  $mHProj 1  
	    if {[cmpBuildModeGet $mHProj] == "source" } {
                puts stdout "Source build enabled."
	        puts stdout "Project will be built from source."
	    } else {
                puts stdout "\nThe following components are not supported for source build: \n[cmpUnsupportedSBCompsGet $mHProj]"
	        puts stdout "\nProject cannot be built from source. Remove the above component(s) for source build."
	    }
            puts stdout "Done"
        }
        vxprj::component::projectFileClose 1 
    }

    ############################################################################
    #
    # vxprj::build::lib - switch the project build between source and library 
    #
    # SYNOPSIS
    # \cs
    #   vxprj::build::lib <prjFile>
    # \ce
    #
    # DESCRIPTION
    # This procedure simply change the current build state of project from 
    # source build to lib mode.
    # 
    # PARAMETERS:
    # prjFile 
    #
    # ERRORS: N/A
    #
    # SEE ALSO: vxprj::create::source, vxprj::build::source
    #

    proc build::lib {prjFile} {

        if {! [file exists $prjFile]} {
            error "Unknown project file \"$prjFile\""
        }
        vxprj::component::projectFileOpen  $prjFile
        set mHProj   [cmpProjHandleGet]

        if { ![cmpSourceBuildEnableGet $mHProj] } {
            puts stdout "Build mode is lib already\nNothing to be done"
        } else {
            puts stdout "Source build disabled. Bulding project using libraries."
            cmpBuildSourceEnable  $mHProj    0 
            puts stdout "Done"
        }
        vxprj::component::projectFileClose 1
    }


    ############################################################################
    #
    # vxprj::build::help - print the help message of project build facility
    #
    # SYNOPSIS
    # \cs
    #   vxprj::build::help
    # \ce
    #
    # DESCRIPTION
    # This procedure simply prints the 'vxprj' build facility help message, and
    # exits with code '-1'.
    #
    # PARAMETERS: N/A
    #
    # RETURNS: N/A
    #
    # ERRORS: N/A
    #
    # SEE ALSO: vxprj::help
    #

    proc build::help {} {

	puts stdout "usage: vxprj build \[<prjfile>] \[<target>]"
	puts stdout "                   help"
	puts stdout "                   source   \[<prjfile>]"
	puts stdout "                   lib   \[<prjfile>]"
	puts stdout "                   get   \[<prjfile>]"
	puts stdout "                   list  \[<prjfile>] \[<pattern>]"
	puts stdout "                   reset \[<prjfile>]"
	puts stdout "                   set   \[<prjfile>] <buildspec>"
	puts stdout "\nwhere:"
	puts stdout "    prjfile   : path to the project file to use. Default\
		     is the project file"
	puts stdout "                in the current directory"
	puts stdout "    target    : name of a target to build (aka clean ...)"
	puts stdout "    pattern   : pattern the build specification name\
		     should match"
	puts stdout "    buildspec : name of a valid build specification for\
		     the project"
	puts stdout "\nExamples :"
	puts stdout "    vxprj build get"
	puts stdout "    vxprj build source"
	puts stdout "    vxprj build lib"
	puts stdout "    vxprj build"
	puts stdout "    vxprj build clean"
	puts stdout "    vxprj build list ../test/test.wpj myBuildSpec"

	exit -1
    }

    # ------------------------------------------------------------------------ #

    proc famous::sayings {} {

	set str    "Dave Fraser            : Is there an actual schedule?\n"
	append str "Dave Wilner            : Its just another port!\n"
	append str "John Fogelin           : So, who's gonna type what?\n"
	append str "Tracy Butler           : Isn't anyone going to use their\
		    own presentation?\n"
	append str "John Hartman           : Wait 'til they hear my french!\n"
	append str "Colin Smith            : Yes, that's an execellent idea.\
		    No, there isn't time.\n                         Well...\n"
	append str "Philippe Maisonneuve   : He must be knowing the Paimpolaise\
		    song ! He just\n                         lives close to\
		    Paimpol !\n"
	append str "Simon Waddington       : Rain on my berries!\n"
	append str "Roland Pesch           : November is remarkably close from\
		    my point of view.\n"
	append str "Jim Irwin              : Perhaps for diehards, we could\
		    arrange to have a web\n                         browser\
		    available for downloading from WindSurf?\n"
	append str "Kent Long              : Now we're cookin' with gas!\n"
	append str "Debbie Steiman-Cameron : If they fit, I'll be real\
		    depressed.\n"
	append str "Suresh Sastry          : Unix changes in-coming, take\
		    cover!\n"
	append str "Marc Shepard           : I quit!  Oh wait... never mind.\n"
	append str "D'Anne Thompson        : Well it depends what you mean by\
		    done.\n"
	append str "Mark Zhou              : It doesn't boot, 5.2 does, is\
		    that a problem?\n"
	append str "Jan Kakarla            : It would be easier in TCL.\n"
	append str "Yogu Ponnambalam       : Someday someone else will take\
		    care of this, right?\n"
	append str "Jimmy Blair            : The i960 work{ed,s,ed,s,...}.\n"
	append str "Scott Herzinger        : I would like to propose a\
		    toast...\n"
	append str "Marisa Hurowitz        : Uh oh.\n"
	append str "Nancy Pakter           : That's the matrix?!\n"
	append str "Johnny Wong            : Checklist for a GUI?\n"
	append str "Meredith Leslie        : Beta customer sit!  Roll over!\
		    Test!\n"
	append str "Peter Nguyen           : 98% and it hangs?  Forever?\n"
	append str "Kitty Kong             : You wrote that!?\n"
	append str "Thierry Preyssler      : You're f___ing right, again!\n"
	append str "Pierre-Alain Darlet    : I don't reminder what ice cream\
		    it was?\n"
	append str "Jean-Claude Oriot      : I dialog with the customer.\n"
	append str "Jean-Michel Pedrono    : I'm pretty sure Frederic wrote\
		    that.\n"
	append str "Frederic Vanborre      : I'm pretty sure Karine wrote\
		    that.\n"
	append str "Karine Penvern         : Moi?  Non!  Monsieur Colin!\n"
	append str "Eric Le Port           : I should take notes.\n"
	append str "Laura Cousins          : Would you like to java with me?\n"
	append str "Didier Brachet         : The frog spit doesn't reach the\
		    white colomb.\n"
	append str "Christophe Cleraux     : When I was at NCR...\n"
	append str "Frederic Leger         : Who said it's because of the\
		    registry?\n"
	append str "Pascal Corbillon       : There is a party at home\
		    tonight!\n"
	append str "Renan Le Padellec      : Water ? Where ? I hope it's a wave\
		    !\n"
	append str "Philippe Basciano      : It does not work with a Breton\
		    layout keyboard ...\n"
	append str "Xavier Pouyollon       : I brake !\n"
	append str "Morvan Legoff          : I was not drunk ! I just fell from\
		    the chapel and\n                         broke my wrist.\n"

	return $str
    }

    # ---------------------------- file namespace ---------------------------- #

    ############################################################################
    #
    # vxprj::_file::isSupportedRequest - test if request is supported
    #
    # SYNOPSIS
    # \cs
    #   vxprj::_file::isSupportedRequest <request>
    # \ce
    #

    proc _file::isSupportedRequest {request} {

	variable mSupportedRequests

	if {[lsearch -exact $mSupportedRequests $request] == -1} {
	    return 0
	} else {
	    return 1
	}
    }

    ############################################################################
    #
    # vxprj::_file::add - add a file to the kernel configuration files
    #
    # SYNOPSIS
    # \cs
    #   vxprj::_file::add <prjFile> <filePath>
    # \ce
    #

    proc _file::add {prjFile filePath} {

	cmpProjOpen $prjFile
	cmpFileAdd [file join $filePath]
	cmpProjClose
    }

    ############################################################################
    #
    # vxprj::_file::get - get a kernel configuration file and its properties
    #
    # SYNOPSIS
    # \cs
    #   vxprj::_file::get <prjFile> <filePath>
    # \ce
    #

    proc _file::get {prjFile filePath} {

	cmpProjOpen $prjFile

	# first check if file is part of the kernel configuration files

	set filePath	[file join $filePath]
	set fileList	[cmpFileListGet]

	if {[lsearch $fileList $filePath] == -1} {

	    # try to see if it is not just the tail of one of the kernel
	    # configuration files

	    set newFilePath	""

	    foreach fileName $fileList {

		if {$filePath == [file tail $fileName]} {

		    # this is it, replace filePath with its full name

		    set newFilePath	$fileName
		    break
		}
	    }

	    if {$newFilePath != ""} {
		set filePath	$newFilePath
	    } else {
		return {}
	    }
	}

	set hProj	[cmpProjHandleGet]
	set fileInfo	{}

	foreach infoType [prjFileInfoListGet $hProj $filePath] {
	    lappend fileInfo [list $infoType [prjFileInfoGet $hProj $filePath \
							     $infoType]]
	}

	cmpProjClose

	return [list $filePath $fileInfo]
    }

    ############################################################################
    #
    # vxprj::_file::help - print the help message of this facility and exits
    #
    # SYNOPSIS
    # \cs
    #   vxprj::_file::help
    # \ce
    #

    proc _file::help {} {

	puts stdout "usage: vxprj file help"
	puts stdout "                  add    \[<prjFile>] <filePath>"
	puts stdout "                  get    \[<prjFile>] <filePath>"
	puts stdout "                  list   \[<prjFile>] \[<pattern>]"
	puts stdout "                  remove \[<prjFile>] <filePath>"
	puts stdout "\nwhere:"
	puts stdout "    add        : add a file to the current kernel\
		     configuration"
	puts stdout "    get        : get given file information"
	puts stdout "    list       : list all the files of current kernel\
		     configuration matching"
	puts stdout "                 <pattern>"
	puts stdout "    remove     : remove given <filePath> from current\
		     kernel configuration"
	puts stdout "    <prjFile>  : kernel configuration file"
	puts stdout "    <filePath> : path to the file to be added to the\
		     current kernel"
	puts stdout "                 configuration"
	exit -1
    }

    ############################################################################
    #
    # vxprj::_file::lst - list all kernel configuration files matching <pattern>
    #
    # SYNOPSIS
    # \cs
    #   vxprj::_file::lst <prjFile> [<pattern>]
    # \ce
    #

    proc _file::lst {prjFile {pattern ".*"}} {

	cmpProjOpen $prjFile
	set fileList [cmpFileListGet]
	cmpProjClose

	# only return matching files

	if {$pattern != ".*"} {
	    set tmpList		$fileList
	    set fileList	{}

	    foreach filePath $tmpList {
		if {[regexp $pattern $filePath]} {
		    lappend fileList [file join $filePath]
		}
	    }
	}

	return $fileList
    }

    ############################################################################
    #
    # vxprj::_file::remove - remove given <filePath> from kernel configuration
    #
    # SYNOPSIS
    # \cs
    #   vxprj::_file::remove <filePath>
    # \ce
    #

    proc _file::remove {prjFile filePath} {

	cmpProjOpen $prjFile
	cmpFileRemove [file join $filePath]
	cmpProjClose
    }

    # -------------------------- profile namespace --------------------------- #

    ############################################################################
    #
    # vxprj::profile::isSupportedRequest - test if request is supported
    #
    # SYNOPSIS
    # \cs
    #   vxprj::profile::isSupportedRequest <request>
    # \ce
    #

    proc profile::isSupportedRequest {request} {

	variable mSupportedRequests

	if {[lsearch -exact $mSupportedRequests $request] == -1} {
	    return 0
	} else {
	    return 1
	}
    }

    ############################################################################
    #
    # vxprj::profile::help - print the help message of this facility and exits
    #
    # SYNOPSIS
    # \cs
    #   vxprj::profile::help
    # \ce
    #

    proc profile::help {} {

	puts stdout "usage: vxprj profile help"
	puts stdout "                     get  \[<prjFile>] <name>"
	puts stdout "                     list \[<prjFile>] \[<pattern>]"
	puts stdout "\nwhere:"
	puts stdout "    get       : get the given profile properties"
	puts stdout "    list      : list all the profiles of current kernel\
		     configuration matching"
	puts stdout "                 <pattern>"
	puts stdout "    <name>    : name of an existing profile"
	puts stdout "    <pattern> : profile name pattern to match (default\
		     is .*)"
	puts stdout "    <prjFile> : kernel configuration file"
	exit -1
    }

    ############################################################################
    #
    # vxprj::profile::get - get kernel config profile properties for <name>
    #
    # SYNOPSIS
    # \cs
    #   vxprj::profile::get <prjFile> <name>
    # \ce
    #

    proc profile::get {prjFile name} {

	if {$prjFile != ""} {
	    cmpProjOpen $prjFile
	}

	# get the profile

	set profile [cmpProfileGet $name]

	# close the project file if needed

	if {$prjFile != ""} {
	    cmpProjClose
	}

	return $profile
    }

    ############################################################################
    #
    # vxprj::profile::lst - list kernel config profiles matching <pattern>
    #
    # SYNOPSIS
    # \cs
    #   vxprj::profile::lst [<prjFile>] [<pattern>]
    # \ce
    #

    proc profile::lst {{prjFile ""} {pattern ".*"}} {

	if {$prjFile != ""} {
	    cmpProjOpen $prjFile
	}

	# get the profile list

	set profileList [cmpProfileListGet $pattern]

	# close the project file if needed

	if {$prjFile != ""} {
	    cmpProjClose
	}

	return $profileList
    }

    # --------------------------- fsfile namespace --------------------------- #

    proc fsfile::isSupportedRequest {request} {

	variable mSupportedRequests
	
	if {[lsearch -exact $mSupportedRequests $request] == -1} {
	    return 0
	} else {
	    return 1
	}
    }

    proc fsfile::add {prjFile targetFile hostFile {buildrule ""}} {

	cmpProjOpen $prjFile
	cmpFsFileAdd $targetFile $hostFile $buildrule
	cmpProjClose
    }

    proc fsfile::get {prjFile targetFile} {

	cmpProjOpen $prjFile
	if {[catch {cmpFsFileGet $targetFile} fileInfo]} {

	    cmpProjClose
	    error "prj fsfile get : $fileInfo"
	}
	cmpProjClose

	return $fileInfo
    }

    proc fsfile::create {prjFile} {

	cmpProjOpen $prjFile
	set warnings [cmpFsFileCreate]
	cmpProjClose

	return $warnings
    }

    proc fsfile::help {} {

	puts stdout "usage: vxprj fsfile help"
	puts stdout "                    add    \[<prjFile>] <targetPath> \
		     <hostPath \[<buildrule>]"
	puts stdout "                    get    \[<prjFile>] <targetPath>"
	puts stdout "                    list   \[<prjFile>] \[<pattern>]"
	puts stdout "                    remove \[<prjFile>] <targetPath> \
		     \[<targetPath ...]"
	puts stdout "\nExamples:"
	puts stdout "    vxprj fsfile add /usr/sbin/ntp.vxe\
			 /tmp/ntpd/ntpd.vxe \\"
	puts stdout "                   \"cd /tmp/ntpd && make -f Makefile\""
	exit -1
    }

    proc fsfile::lst {prjFile {pattern ".*"}} {

	cmpProjOpen $prjFile
	set fsFileList [cmpFsFileListGet $pattern]
	cmpProjClose

	return $fsFileList
    }

    proc fsfile::remove {prjFile targetPath args} {

	cmpProjOpen $prjFile

	set fsFileList	$targetPath
	cmpFsFileRemove $targetPath

	foreach arg $args {
	    cmpFsFileRemove $arg
	}

	cmpProjClose
    }

    # ------------------------- buildmacro namespace ------------------------- #

    ############################################################################
    #
    # vxprj::buildmacro::isSupportedRequest - test is <request> is supported
    #

    proc buildmacro::isSupportedRequest {request} {

	variable mSupportedRequests
	
	if {[lsearch -exact $mSupportedRequests $request] == -1} {
	    return 0
	} else {
	    return 1
	}
    }

    ############################################################################
    #
    # vxprj::buildmacro::add - add a build macro, and set its value
    #

    proc buildmacro::add {prjFile name {value ""}} {

	# open project file, and get its handle and build spec

	cmpProjOpen $prjFile

	set hProj	[cmpProjHandleGet]
	set hSpec	[prjBuildCurrentGet $hProj]

	# set the given macro to the given value

	prjBuildMacroSet $hProj $hSpec $name $value

	# close the kernel configuration project

	cmpProjClose
    }

    ############################################################################
    #
    # vxprj::buildmacro::get - get a build macro value
    #

    proc buildmacro::get {prjFile name} {

	cmpProjOpen $prjFile
	set macroValue [cmpBuildMacroGet $name]
	cmpProjClose

	return $macroValue
    }

    ############################################################################
    #
    # vxprj::buildmacro::help - print the build macro facility help and exit
    #

    proc buildmacro::help {args} {

	puts stdout "usage: vxprj buildmacro help"
	puts stdout "                        add     \[<prjfile>] <name>\
		     \[<value>]"
	puts stdout "                        get     \[<prjfile>] <name>"
	puts stdout "                        list    \[<prjfile>] \[<pattern>]"
	puts stdout "                        remove  \[<prjfile>] <name>\
		     \[<name> ...]"
	puts stdout "                        set     \[<prjfile>] <name>\
		     \[<value>]"
	puts stdout "                        value   \[<prjfile>] \[<pattern>]\
		     \[<valuePattern>]"
	puts stdout "\nwhere:"
	puts stdout "    add            : add given <name> build macro"
	puts stdout "    get            : get value for build macro named\
		     <name>"
	puts stdout "    help           : print this helkp message"
	puts stdout "    list           : list all build macros matching\
		     <pattern> (default = .*)"
	puts stdout "    remove         : remove macro named <name>"
	puts stdout "    set            : set build macro to <value>"
	puts stdout "    value          : get value for given macro"
	puts stdout "    <name>         : build macro name"
	puts stdout "    <pattern>      : build macro name pattern"
	puts stdout "    <prjfile>      : path to a kernel configuration file"
	puts stdout "    <valuePattern> : pattern the macro value should match"
	exit -1
    }

    ############################################################################
    #
    # vxprj::buildmacro::lst - list build macros
    #

    proc buildmacro::lst {prjFile {namePattern ".*"}} {

	# open project file, and get all macros for the current build spec

	cmpProjOpen $prjFile

	set hProj	[cmpProjHandleGet]
	set hSpec	[prjBuildCurrentGet $hProj]
	set tmpList	[prjBuildMacroListGet $hProj $hSpec]
	set macroList	{}

	# we ca close project now

	cmpProjClose

	# only return the macros which match namePattern

	if {($namePattern != ".*")} {
	    foreach macro $tmpList {
		if {[regexp -- $namePattern $macro]} {
		    lappend macroList $macro
		}
	    }
	} else {
	    set macroList $tmpList
	}

	return $macroList
    }

    ############################################################################
    #
    # vxprj::buildmacro::remove - remove a build macro
    #

    proc buildmacro::remove {prjFile name args} {

	# open project, and get its handle and build spec

	cmpProjOpen $prjFile

	set hProj		[cmpProjHandleGet]
	set hSpec		[prjBuildCurrentGet $hProj]

	# remove macro from current kernel configuration

	prjBuildMacroRemove $hProj $hSpec $name

	# remove other macros if any

	foreach macroName $args {
	    prjBuildMacroRemove $hProj $hSpec $macroName
	}

	# close the current kernel configuration project

	cmpProjClose
    }

    ############################################################################
    #
    # vxprj::buildmacro::set - set a build macro value
    #

    proc buildmacro::_set {prjFile name {value ""}} {

	[namespace current]::add $prjFile $name $value
    }

    ############################################################################
    #
    # vxprj::buildmacro::value - get a list of macro values
    #

    proc buildmacro::value {prjFile {namePattern ".*"} {valuePattern ".*"}} {

	set macroDescList	{}
	set macroList		{}

	# open project file

	cmpProjOpen $prjFile

	set hProj	[cmpProjHandleGet]
	set hSpec	[prjBuildCurrentGet $hProj]

	set tmpList	[prjBuildMacroListGet $hProj $hSpec]

	# only return the macros which match namePattern

	if {($namePattern != ".*")} {
	    foreach macro $tmpList {
		if {[regexp -- $namePattern $macro]} {
		    lappend macroList $macro
		}
	    }
	} else {
	    set macroList $tmpList
	}

	# now only keep macros with matching values

	foreach macro $macroList {

	    set value	[prjBuildMacroGet $hProj $hSpec $macro]

	    if {[regexp -- $valuePattern $value]} {
		lappend macroDescList [list $macro $value]
	    }
	}

	# the project may be closed now

	cmpProjClose

	return $macroDescList

    }

    # ------------------------- buildrule namespace ------------------------- #

    ############################################################################
    #
    # vxprj::buildrule::isSupportedRequest - test is <request> is supported
    #

    proc buildrule::isSupportedRequest {request} {

	variable mSupportedRequests
	
	if {[lsearch -exact $mSupportedRequests $request] == -1} {
	    return 0
	} else {
	    return 1
	}
    }

    ############################################################################
    #
    # vxprj::buildrule::get - get a build macro value
    #

    proc buildrule::get {prjFile name} {

	cmpProjOpen $prjFile
	set ruleValue	[cmpBuildRule $name]
	set ruleStr	[format "%s : %s\n%s" [lindex $ruleValue 0] \
					      [lindex $ruleValue 1] \
					      [lindex $ruleValue 2]]
	cmpProjClose

	return $ruleStr
    }

    ############################################################################
    #
    # vxprj::buildrule::help - print the build macro facility help and exit
    #

    proc buildrule::help {args} {

	puts stdout "usage: vxprj buildrule \[prjfile>] \[<rule>]"
	puts stdout "                       add          \[<prjfile>] <rule>\
		     \[<value>]"
	puts stdout "                       dependencies \[<prjfile>] <rule>\
		     \[<value>]"
	puts stdout "                       get          \[<prjfile>] <rule>"
	puts stdout "                       list         \[<prjfile>]\
		     \[<pattern>]"
	puts stdout "                       remove       \[<prjfile>] <rule>"
	puts stdout "                       set          \[<prjfile>] <rule>\
		     \[<value>]"
	puts stdout "\nwhere:"
	puts stdout "    add          : add given <name> build rule"
	puts stdout "    dependencies : get or set a build rule dependencies"
	puts stdout "    get          : get value for build rule named <rule>"
	puts stdout "    help         : print this help message"
	puts stdout "    list         : list all build rules matching <pattern>\
		     (default = .*)"
	puts stdout "    remove       : remove build rule named <rule>"
	puts stdout "    set          : set build rule <rule> to <value>"
	puts stdout "    <rule>       : build rule name"
	puts stdout "    <pattern>    : build rule name pattern"
	puts stdout "    <prjfile>    : path to a kernel configuration file"
	puts stdout "    <value>      : value the build rule should be set to"
	exit -1
    }

    ############################################################################
    #
    # vxprj::buildrule::lst - list build rules
    #

    proc buildrule::lst {prjFile {namePattern ".*"}} {

	cmpProjOpen $prjFile
	set rules	[cmpBuildRuleListGet $namePattern]
	cmpProjClose

	return $rules
    }

    ############################################################################
    #
    # vxprj::buildrule::remove - remove a build rule
    #

    proc buildrule::remove {prjFile name} {

	# open project, and get its handle and build spec

	cmpProjOpen $prjFile
	set value [cmpBuildRuleRemove $name]
	cmpProjClose

	return [list $name $value]
    }

    ############################################################################
    #
    # vxprj::buildrule::set - set a build rule
    #

    proc buildrule::_set {prjFile name {value ""}} {

	cmpProjOpen $prjFile
	cmpBuildRule $name "" $value
	cmpProjClose

	return [list $name $value]
    }

    ############################################################################
    #
    # vxprj::buildrule::add - add a build rule
    #

    proc buildrule::add {prjFile name {value ""}} {

	cmpProjOpen $prjFile
	cmpBuildRuleAdd $name "" $value
	cmpProjClose

	return [list $name $value]
    }

    ############################################################################
    #
    # buildrule::dependencies - get or set the build dependencies for given rule
    #

    proc buildrule::dependencies {prjfile name {value ""}} {

	cmpProjOpen $prjfile

	set rule	[cmpBuildRule $name]

	if {$value == ""} {

	    # just return the rule's dependencies

	    set deps	[lindex $rule 1]
	} else {

	    # set dependencies to an ampty string if needed

	    if {[llength $value] == 0} {
		set deps	{}
	    } else {
		set deps	$value
	    }

	    # modify the build rule dependencies

	    cmpBuildRule $name $deps
	}

	cmpProjClose

	# return the dependencies list

	return $deps
    }

    ############################################################################
    #
    # buildrule::current - get or set the current build rule
    #

    proc buildrule::current {prjfile {rule ""}} {

	cmpProjOpen	$prjfile
	set currentRule	[cmpBuildRuleDefault $rule]
	cmpProjClose

	return $currentRule
    }

    # --------------------------- module namespace --------------------------- #

    ############################################################################
    #
    # vxprj::module::isSupportedRequest - test is <request> is supported
    #

    proc module::isSupportedRequest {request} {

	variable mSupportedRequests
	
	if {[lsearch -exact $mSupportedRequests $request] == -1} {
	    return 0
	} else {
	    return 1
	}
    }

    ############################################################################
    #
    # vxprj::module::dependencies - get the modules dependencies
    #
    # SYNOPSIS
    # \cs
    #   vxprj::module::dependencies <prjfile> <module> [<module> ...]
    # \ce
    #

    proc module::dependencies {prjfile module args} {

	set moduleList	$module

	foreach arg $args {
	    lappend moduleList $arg
	}

	cmpProjOpen	$prjfile
	set deps	[cmpModuleDependsGet $moduleList]
	cmpProjClose

	return $deps
    }

    ############################################################################
    #
    # vxprj::module::help - print the module help message and exit with '-1'
    #

    proc module::help {} {

	puts stdout "usage: vxprj module"
	puts stdout "usage: vxprj module help"
	puts stdout "                    dependencies \[<prjFile>] <module>\
		     \[<module> ...]"
	puts stdout "\nwhere:"
	puts stdout "    help         : print the help message of this facility"
	puts stdout "    dependencies : print out the dependencies of <module>"
	puts stdout "    <module>     : name of a kernel module"
	puts stdout "    <prjFile>    : path of the project file (default is\
		     the .wpj file in the"
	puts stdout "                   current directory)"
	exit -1
    }

    # ------------------ vxprj namespace private procedures ------------------ #

    variable mSupportedRequests		{}	;# list of vxprj requests

    set mSupportedRequests	[list build bundle component create diff \
				      file fsfile parameter delete famous copy\
				      buildmacro buildrule module makefile \
				      tccheck profile autoscale]
}

# --------------------------- exported procedures ---------------------------- #

################################################################################
#
# vxprj::copy - copy a kernel configuration project to another place
#
# SYNOPSIS
# \cs
#   vxprj copy [-force] [<sourcefile>] <destfile|destdir>
# \ce
#
# DESCRIPTION
# The current project (.wpj file in the current directory), or project given
# by <sourcefile> project file is copied to <destfile> (or <destdir>)
#
# If <sourcefile> is not provided, the .wpj in the current directory is used,
# and considered as the current project
#
# If <destfile> is specified, the copied project will be created in that file,
# else, if a <destdir> is specified, the copied project is created in that
# directory, and the project file is a .wpj file which has the same name than
# the parent directory.
#
# For example, if <destdir> is set to 'C:\temp\myProj', the project file is
# then 'C:\temp\myProj\myProj.wpj'
#
# PARAMETERS:
#   '-force' : delete the destination directory if it already exists
#   <sourcefile> : project file of the project to be copied
#   <destfile> : destination project file (or directory)
#
# RETURNS: The path of the so-created project
#
# ERRORS:
# \is
# \i vxprj copy needs at least a destination file or directory ...
# \i File <fileName> cannot be removed. Exiting ...
# \ie
#
# EXAMPLES:
# Copy current project in ../copied directory :
# \cs
#  # vxprj copy ../copied
#  Copying Project ...
#  Project copied in : ../copied/copied.wpj
# \ce
#
# SEE ALSO: 'cmpProjCopy'
#

proc vxprj::copy {args} {

    # first check if there is the -force option

    set removeFirst	0
    set srcFile		""
    set destFile	""

    if {[set ix [lsearch -exact $args "-force"]] != -1} {

	set removeFirst	1
	set args	[lreplace $args $ix $ix]
    }

    # now, if there are two arguments, let's assume it is <sourcefile> and
    # <destfile|destdir>. Else, try to find a project file

    if {[llength $args] == 0} {

	error "vxprj copy needs at least a destination file or directory ..."

    } elseif {[llength $args] == 1} {

	# look for the source project file

	set srcFile	[lindex [vxprj::fileFind vxprj::help ""] 0]
	set destFile	[lindex $args 0]

    } else {

	set srcFile	[lindex $args 0]
	set destFile	[lindex $args 1]
    }

    # check if destFile is a .wpj file. If not, create a path for it

    if {[file extension $destFile] != ".wpj"} {

	set destPrjName	[file tail $destFile]
	set destFile	[file join $destFile [format "%s.wpj" $destPrjName]]
    }

    # remove the destination directory if required

    if {($removeFirst == 1) && ([file exists $destFile])} {

	set destDir		[file dirname $destFile]
	set destFileList	[FileLib::listGet $destDir 1]

	# be sure all files are writable

	foreach fileName $destFileList {

	    if {! [file writable [file join $destDir $fileName]]} {

		error "File $fileName cannot be removed. Exiting ..."
	    }
	}

	puts stdout "Removing directory $destDir ..."
	catch {file delete -force $destDir}
    }

    # now we can copy the project ... at last !

    puts stdout "Copying Project ..."

    cmpProjOpen $srcFile

    if {[catch {cmpProjCopy $destFile $srcFile} status]} {

	error $status
    }

    puts stdout "Project copied in : $destFile"

    cmpProjClose

    return $destFile
}

################################################################################
#
# vxprj::create - create a new Os Configuration project
#
# SYNOPSIS
# \cs
#   vxprj::create [-force] [-source] [-b <bundle>] [-c <component>] [-inet6] [-fr]
#                 [-inet6_fr] [-noopt] [-p <parameter>=<value>]
#                 [-profile <profile>] <bsp|bspdir> <tool> [<prjfile|prjdir>]
# \ce
#
# DESCRIPTION
# Create a new Os configuration project.
#
# The new project is based on the given <bsp>. If <bsp> is a directory which
# contains a Makefile, then the content of that directory is used to create
# the new project. If it is just a BSP name, the content of the
# ['Wind::tgtDir']/config/<bsp> directory is used.
#
# If no project file is specified, the default location for the newly created
# project is ['Wind::prjDir']/<bsp>_<tool>/<bsp>_<tool>.wpj
#
# If the given <prjfile> has not '.wpj' extension, it is considered a directory
# path, and the project file will be named <parentdir>.wpj. For example, if
# <prjfile> is set to 'C:/temp/myProj', the project file is saved in
# 'C:/temp/myProj/myProj.wpj'
#
# ADDING ELEMENTS
# As the 'vxprj' tool has to load the OsConfig libraries, and open the kernel
# configuration file each time it is called, some options can be used at
# project creation time :
#
# \is
# \i '-b' : add a component bundle (see 'vxprj::bundle')
# \i '-c' : add a component (see 'vxprj::component')
# \i '-p' : set a parameter value (see 'vxprj::parameter')
# \ie
#
# REMOVING ALL COMPONENTS
# It is possible to remove all the components the kernel configuration project
# contains at creation time, by using the '-removeall' option. This option
# is generally used with the '-b' option which will add the given bundle.
#
# PARAMETERS:
#   '-b' : add given <bundle> to project once it is created (may be repeated
#          several times)
#   '-c' : add given <component> to project once it is created (may be repeated
#          several times)
#   '-force' : if this option is specified, PRJ_DIR/<bsp>_<tool> is first
#              removed
#   '-inet6' : create a project appropriate for the IPv6-enabled build
#             (OPT=-inet6)
#   '-fr' : create a project appropriate for the free build (OPT=-fr)
#   '-inet6_fr' : create a project appropriate for the IPv6-enabled free build
#                 (OPT=-inet6_fr)
#   '-noopt' : create a non-option-build project. This is the default. The last
#              '-inet6', '-fr', '-inet6_fr', or '-noopt' argument governs.
#   '-p' : set <parameter>'s value to <value> once the project has been created.
#          This flag may be repeated several times.
#   '-profile' : use <profile> to create the kernel component configuration
#   '-source'  : enable building from source
#   <bsp> : name or directory of the bsp to create project from.
#   <bundle> : name of the bundle to be added after project creation
#   <component> : name of a component to be added after project creation
#   <parameter> : name of a parameter to set value after project creation
#   <tool> : name of the toolchain to create project with (any of the gnu or
#            diab variants)
#   <prjfile> : Kernel configuration file or parent directory.
#   <profile> : name of the profile to use to create the kernel component
#               configuration
#   <value> : value to give to <parameter>
#
# RETURNS: N/A
#
# ERRORS:
# \is
# \i Project creation failed : <error>
# \i Unknown BSP "<bsp>" : could not find "<bspdir>"
# \i Unknown BSP "<bsp>" : could not find "<makefile>"
# \i Unknown tool "<tool>"
# \ie
#
# EXAMPLES:
# Create a project in the default location, for a 'simpc' BSP, using the 'diab'
# toolchain.
# \cs
# # vxprj create simpc diab
# Creating Project ...
# Project created in : C:\wpide\target\proj\simpc_diab\simpc_diab.wpj
# #
# \ce
#
# Create a project with a BSP which is not in the regular installation tree,
# and with a given project file destination :
# \cs
# # vxprj.bat create c:\temp\pcPentium2 diab c:\temp\myProj\myProj.wpj
# Creating Project ...
# Project created in : c:\temp\myProj\myProj.wpj
# #
# \ce
#
# Create a project in the default location, for the 'ebony' BSP,
# using the 'gnu' toolchain, for the IPv6 enabled (OPT=-inet6) build:
# \cs
# # vxprj create -inet6 wrSbc8260 gnu
# Creating Project ...
# Project created in : C:\wpide\target\proj\wrSbc8260_gnu\wrSbc8260_gnu.wpj
# #
# \ce
#
# SEE ALSO: 'vxprj::profile', 'cmpProjCreate'
#

proc vxprj::create {args} {

    global env		;# user's environment variables

    set removeFirst	0
    set bundleList	{}
    set paramList	{}
    set arguments	$args
    set compList	{}
    set profile		""
    set nArgs		[llength $arguments]
    set args		{}
    set opt		""

    # initialize workspace environment variable used in defs.link during
    # project creation
    set env(WS_ROOT_DIR) UNKNOWN

    # parse arguments

    for {set ix 0} {$ix < $nArgs} {incr ix} {

	set arg	[lindex $arguments $ix]

	switch -exact -- $arg {

	    -b {

		# should be a bundle name

		incr ix

		if {$ix < $nArgs} {
		    lappend bundleList	[lindex $arguments $ix]
		}
	    }

	    -c {

		# should be a component name

		incr ix

		if {$ix < $nArgs} {
		    lappend compList	[lindex $arguments $ix]
		}
	    }

	    -force {

		# project directory should be cleaned first. We can remove the
		# -force option from arguments

		set removeFirst	1
	    }

	    -inet6 {
		set opt "-inet6"
	    }

	    -fr {
		set opt "-fr"
	    }

	    -inet6_fr {
		set opt "-inet6_fr"
	    }

	    -noopt {
		set opt ""
	    }

	    -profile {

		incr ix

		if {$ix < $nArgs} {
		    set profile	[lindex $arguments $ix]
		}
	    }

	    -p {

		# parameter assignment

		incr ix

		if {$ix < $nArgs} {
		    set arg	[lindex $arguments $ix]

		    if {[regexp -- {([^=]+)=(.*)} $arg dummy name value]} {
			lappend paramList	[list $name $value]
		    }
		}
	    }
            -source {
               lappend opt "-source"
            }
	    default {lappend args $arg}
	}
    }

    # assign parameters, and check there are enough of them

    if {([llength $args] < 2) || ([llength $args] > 3)} {
	vxprj::help
    } elseif {[llength $args] == 2} {
	set prjFile	""
    } else {
	set prjFile	[lindex $args 2]
    }

    set bsp	[lindex $args 0]

    # check if it is a BSP path, or a bsp name

    if {([file isdirectory $bsp]) &&
	([file exists [file join $bsp Makefile]])} {

	# it's a BSP directory, give its absolute path if possible

	if {[file pathtype $bsp] == "relative"} {

	    set bsp [file join [pwd] $bsp]
	}

	set bspName	[file tail $bsp]
    } else {

	# check bsp

	set bspName	$bsp
	set bspMakefile	[file join [Wind::tgtDir] config $bsp Makefile]

	if {! [file exists $bspMakefile]} {

	    # unknown BSP, or BSP without Makefile ... Exit with the appropriate
	    # message

	    if {! [file exists [file dirname $bspMakefile]]} {
		error "Unknown BSP \"$bsp\" : could not find \"[file dirname\
		       $bspMakefile]\""
	    } else {
		error "Unknown BSP \"$bsp\" : could not find \"$bspMakefile\""
	    }
	}
    }

    set tool	[lindex $args 1]

    # first check arguments

    if {! [regexp -- {^(sf)?(r3k)?(diab|gnu)(le|be)?$} $tool]} {

	error "Unknown tool \"$tool\""
    }

    # if prjFile is not provided, create one

    if {$prjFile == ""} {
	set prjName	[format "%s_%s" $bspName $tool]
	set prjFile	[file join [Wind::prjDir]  $prjName \
				   [format "%s.wpj" $prjName]]
    } else {

	# check if there is a .wpj extension

	if {[file extension $prjFile] != ".wpj"} {

	    # it must be a directory path

	    set prjName	[file tail $prjFile]
	    set prjFile	[file join $prjFile [format "%s.wpj" $prjName]]

	} else {
	    set prjName	[file root [file tail $prjFile]]
	}
    }

    # create the project parent directory if needed

    set parentDir	[file dirname [file dirname $prjFile]]

    if {! [file isdirectory $parentDir]} {

	# check if we can create it ...

	if {[catch {file mkdir $parentDir} status]} {

	    error $status
	}
    }

    # if the -force option was specified, use it here

    if {($removeFirst) && ([file exists [file dirname $prjFile]])} {
	puts stdout "Removing [file dirname $prjFile] directory ..."
	catch {file delete -force [file dirname $prjFile]}
    }

    # If the parent directory of the project file already contains some
    # project files, it is better to create a different parent directory,
    # appending it a _<x>

    set originalPrj	$prjFile
    set originalPrjName	$prjName
    set ix		1

    while {([file exists [file dirname $prjFile]]) &&
	   ([llength [glob -nocomplain [file join [file dirname $prjFile] \
						  "*.wpj"]]] != 0)} {

	incr ix
	set newPrjName	[format "%s_%i" $originalPrjName $ix]
	set prjFile	[file join $parentDir $newPrjName \
				   [format "%s.wpj" $newPrjName]]
    }

    # we should now be able to create the project itself

    puts stdout "Creating Project ..."

    lappend opt "-scale"

    if {[catch {cmpProjCreate $bsp $prjFile vxWorks "" $tool $opt} status]} {
	puts stderr "Project creation failed : $status"
	exit -1
    }

    set hProj	[cmpProjHandleGet]
    set wcc	[wccCurrentGet $hProj]

    if {$profile == ""} {
	set default [cmpDefaultProfileGet $hProj]
	if {$default != ""} {
		wccComponentAdd $hProj $wcc $default
#		wccComponentAdd $hProj $wcc [wccDependsGet $hProj $wcc]
		cmpRecalculate $hProj
	}
    }

    if {$profile == "none"} {
	set profile ""
    }


    if {$profile != ""} {
	cmpProfileAdd [cmpProjHandleGet] $profile
	cmpProfileInitSet $profile
    }


    # check if some bundles were given

    foreach bundle $bundleList {

	puts stdout "    Adding bundle $bundle ..."
	cmpBundleAdd	$bundle
    }

    # add some components too

    foreach component $compList {

	puts stdout "    Adding component $component ..."
	cmpAdd $component
    }

    # set parameter values if needed

    foreach param $paramList {

	set name	[lindex $param 0]
	set value	[lindex $param 1]

	puts stdout "    Setting $name to $value ..."

	cmpParamSet $name $value
    }
    if { ([cmpSourceBuildEnableGet $hProj]) && ([cmpBuildModeGet $hProj] != "source") } {
        puts stdout "\nThe following components are not supported for source build:\
                     \n[cmpUnsupportedSBCompsGet $hProj]" 
        puts stdout "\nProject can not be built from source."
        puts stdout "Build mode has been changed to libraries.\n"

    }

    cmpProjClose

    puts stdout "Project created in : [file nativename $prjFile]"

    # warn the use if project name was changed

    if {$ix != 1} {
	puts stdout "Warning : project has been created in :\n[file nativename\
		     $prjFile]\ninstead of :\n[file nativename $originalPrj]"
    }
}

################################################################################
#
# vxprj::diff - get the differences between two projects
#
# SYNOPSIS
# \cs
#   vxprj::diff [<prjfile>] <withPrjfile|prjdir>
# \ce
#
# DESCRIPTION
# Lists the components and parameters differences that may exist between two
# kernel configuration project.
#
# A list of components/parameters unique to the current kernel configuration
# project is printed on the standard output, then the list of
# components/parameters unique to the kernel configuration project specified
# by <withPrjfile> (or <withPrjdir>) is printed.
#
# PARAMETERS:
#   <prjfile> : reference project file. If it is not provided, the project file
#               (.wpj) of the current directory is used.
#   <withPrjFile> : project file to compare <prjfile> with
#   <prjdir> : project directory containing the project file to compare with
#              <prjfile>. This project file should have the same name than its
#              parent directory (e.g. dir=a/b => file=a/b/b.wpj)
#
# RETURNS: N/A
#
# ERRORS:
# \is
# \i Project file <withPrjFile> does not exist
# \ie
#
# EXAMPLES:
# Checking differences between current kernel configuration project and project
# in ../linux_diab_2 :
# \cs
# # vxprj diff ../linux_diab_2
#
# Parameters/Components unique to linux_diab.wpj :
# INCLUDE_STANDALONE_SYM_TBL
# WV_MAX_LOGS = 10
#
# Parameters/Components unique to ../linux_diab_2/linux_diab_2.wpj :
# INCLUDE_PASSFS_SYM_TBL
# WV_MAX_LOGS = 5
# \ce
#
# SEE ALSO: 'vxprj::component::diff', 'vxprj::parameter::diff'
#

proc vxprj::diff {prjFile {withPrjFile ""}} {

    # commute <prjFile> and <withPrjFile> if <withPrjFile> is not set

    if {$withPrjFile == ""} {
	set withPrjFile	$prjFile
	set prjFile	""
    }

    # if withPrjFile is a directory, try to us the project file it contains

    if {[file isdirectory $withPrjFile]} {
	set withPrjFile	[file join $withPrjFile \
				   [format "%s.wpj" [file tail $withPrjFile]]]

	# check that the file exists

	if {! [file exists $withPrjFile]} {

	    error "Project file $withPrjFile does not exist"
	}
    }

    if {! [file exists $prjFile]} {

	# try to find a .wpj file in the current directory

	set fileList	[glob -nocomplain *.wpj]

	if {[llength $fileList] >= 1} {
	    set prjFile	[lindex $fileList 0]
	} else {
	    puts stdout "Please provide with two project files, or go in a\
			 directory containing project"
	    puts stdout "files."
	    vxprj::help
	}
    }

    set compsDiff	[vxprj::component::diff $prjFile $withPrjFile]
    set paramsDiff	[vxprj::parameter::diff $prjFile $withPrjFile]

    set prjDiffs	[concat [lindex $compsDiff 0] [lindex $paramsDiff 0]]
    set withPrjDiffs	[concat [lindex $compsDiff 1] [lindex $paramsDiff 1]]

    return [list [list $prjFile $prjDiffs] [list $withPrjFile $withPrjDiffs]]
}

################################################################################
#
# vxprj::help - print this tool's help message and exits
#
# SYNOPSIS
# \cs
#   vxprj::help
# \ce
#
# DESCRIPTION
# This procedure simply print the usage message for this 'vxprj' tool. It then
# exist the program with status '-1'.
#
# PARAMETERS: N/A
#
# RETURNS: N/A
#
# ERRORS: N/A
#
# NOMANUAL
#

proc vxprj::help {} {

    puts stdout "usage: vxprj help"
    puts stdout "             autoscale  \[-terse|-shrink] \[<prjfile>]"
    puts stdout "             copy       \[-force] \[<prjfile>]\
		 <prjfile|prjdir>"
    puts stdout "             create     \[-force] \[-source] \[-profile <profile>]\
		 \[-inet6|-fr|-inet6_fr]"
    puts stdout "                        <bsp|bspdir> <tool>\
		 \[<prjfile|prjdir>]"
    puts stdout "             delete     <prjfile>"
    puts stdout "             diff       \[<prjfile>] <prjfile|prjdir>"
    puts stdout "             makefile   \[<prjfile>] \[<makefile>]"
    puts stdout "             tccheck"
    puts stdout "             bundle     {add|get|help|list|remove} ..."
    puts stdout "             build      \[{source|lib|get|help|list|reset|set}] ..."
    puts stdout "             buildmacro {add|get|help|list|remove|set|value}\
		 ..."
    puts stdout "             buildrule \
		 {add|dependencies|get|help|list|remove|set} ..."
    puts stdout "             component \
		 {add|check|dependencies|diff|help|list|remove} ..."
    puts stdout "             file       {add|list|remove} ..."
    puts stdout "             module     {dependencies|help} ..."
    puts stdout "             parameter \
		 {diff|help|list|reset|set|setstring|value} ..."
    puts stdout "             profile    {get|help|list} ..."
    puts stdout "\nwhere:"
    puts stdout "    autoscale   : scales down kernel configuration"
    puts stdout "    copy        : copy a kernel configuration project"
    puts stdout "    create      : create a new kernel configuration project"
    puts stdout "    delete      : delete an existing kernel configuration\
		 project"
    puts stdout "    diff        : print differences between two projects"
    puts stdout "    makefile    : get the current makefile name"
    puts stdout "    tccheck     : regenerates toolchain info files if needed"
    puts stdout "    -force      : deletes the project file's directory first"
    puts stdout "    -inet6      : creates a project for IPv6 enabled kernel"
    puts stdout "    -fr         : creates a project for free enabled kernel"
    puts stdout "    -inet6_fr   : creates a project for IPv6 and free enabled\
		 kernel"
    puts stdout "    -profile    : use <profile> to create the new kernel\
		 configuration"
    puts stdout "    -shrink     : performs the autoscale operation"
    puts stdout "    -terse      : only print terse information for autoscae"
    puts stdout "    <bsp>       : name of the Board Support Package to rely on"
    puts stdout "    <bspdir>    : path to a directory containing a BSP"
    puts stdout "    <makefile>  : name to give to current kernel configuration\
		 project makefile"
    puts stdout "    <tool>      : name of the toolchain to be used for project"
    puts stdout "    <prjfile>   : path to the project file (.wpj)"
    puts stdout "    <prjdir>    : path to a directory containing a project\
		 file which name is"
    puts stdout "                  the name of its parent directory (e.g.\
		 if prjdir=myDir/myProj"
    puts stdout "                  then prjfile=myDir/myProj/myProj.wpj)"
    puts stdout "    <profile>   : name of the profile to use at creation time"
    puts stdout "\nFor more help on subcommands (bundle, build, buildmacro,\
		 buildrule, component,"
    puts stdout "file, module, parameter, profile), please try vxprj\
		 <subcommand> help"
    puts stdout "For Example :"
    puts stdout "    vxprj component help"
    exit -1
}

################################################################################
#
# vxprj::component - vxprj component facilities dispatcher
#
# SYNOPSIS
# \cs
#   vxprj::component add          [<prjfile>] <component> [<component> ...]
#                    check        [<prjfile>] <component> [<component> ...]
#                    dependencies [<prjfile>] <component> [<component> ...]
#                    diff         [<prjfile>] <prjfile|prjdir>
#                    help
#                    list         [<prjfile>] 
#                                 [{all|excluded|included|unavailable}]
#                                 [<pattern>]
#                    remove       [<prjfile>] <component> [<component> ...]
# \ce
#
# DESCRIPTION
# The component manipulation module. This module allows to manipulate the
# project components.
#
# For all the features where the <prjfile> is optional (between square
# brackets), the project file of the current directory is used.
#
# ADDING COMPONENTS
# One can add one or several components using the 'add' command. The kernel
# configuration engine may also find some dependencies on the added component(s)
# and add the dependencies.
#
# In this case, a message is sent to the standard output channel
#
# CHECKING COMPONENTS
# When creating components in CDF files, it is possible to check if they
# are valid or not, using the 'check' command.
#
# GETTING COMPONENT DEPENDENCIES
# When a component is added, it sometimes also adds some dependencies, the
# 'dependencies' command helps finding why those components depend on each
# other
#
# GETTING COMPONENT DIFFERENCES
# It is possible to know which components are in one project compared to
# another using the 'diff' command. It may help tracking why a project works
# while anoter does not ...
#
# LISTING COMPONENTS
# The components of a kernel configuration project may be listed using the
# 'list' command. As a project may inclue or not some components, several
# filters are available for the 'list' command
# \is
# \i 'all'
# List all the possible components (aka, all components defined in all the
# various CDF files). Even the unavailable components are listed.
# \i 'excluded'
# List the components which are not included by the current kernel configuration
# project. Unavailable components are not listed here.
# \i 'included'
# This is the default behaviour. Omitting to specify a list filter assumes only
# the included components are listed. This lists all the components which are
# currently included by the kernel configuration
# \i 'unavailable'
# Get the list of unavailable components. The components listed here cannot be
# added because they refer to modules which are not available in the kernel
# libraries
# \ie
# If a <pattern> is provided, only the components which name match <pattern>
# are listed.
#
# REMOVING A COMPONENT
# The 'remove' command can remove one or several components from the kernel
# configuration. Removing a component may also remove its dependencies.
#
# PARAMETERS:
#   'add' : add component named <component> to the current project. Several
#           component names may be specified.
#   'check' : check the given <component> of the current file. If there is no
#             <component>, the whole project is checked.
#   'dependencies' : get the dependencies of a given component, or set of
#                    components
#   'diff' : list the component wchih are in a project, and not in the other.
#   'help' : print the project component module help message
#   'list' : list all the components of the current project matching <pattern>.
#            If <pattern> is not specified, all the components are returned.
#   'remove' : remove component named <component> from the current project.
#              Several component names may be given.
#   <pattern> : pattern the component name should match
#   <prjdir> : project in which the project file stands. The project file name
#              should be the same as its parent directory (e.g. if prjdir =
#              myDir/myProj then prjfile = myDir/myProj/myProj.wpj)
#   <prjfile> : project file to be used
#
# RETURNS: N/A
#
# ERRORS: N/A
#
# EXAMPLES:
# List components which name match TSFS. Try to find some in the different
# lists of components ...
# \cs
#  # vxprj component list TSFS
#  INCLUDE_WDB_TSFS
#  INCLUDE_WVUPLOAD_TSFSSOCK
#  # vxprj component list all TSFS 
#  INCLUDE_WDB_TSFS
#  INCLUDE_WVUPLOAD_TSFSSOCK
#  JAVA_FILE_CONFIG_TSFS
#  # vxprj component list excluded TSFS
#  JAVA_FILE_CONFIG_TSFS
#  # vxprj component list unavailable TSFS
#  #
# \ce
#
# SEE ALSO: 'vxprj::parameter'
#

proc vxprj::component {args} {

    set procName	[lindex $args 0]
    set args		[lreplace $args 0 0]

    if {($procName == "") ||
	(! [regexp "add|check|diff|dependencies|dependDetail|list|remove|project" $procName])} {

	vxprj::component::help

    } else {

	# check if the first argument is a valid file name
	#
	# If the first argument ends with ".wpj", we may consider it is a
	# project file name. If it does not exist, the user asked for a wrong
	# file, we have to tell him

	set prjFile	[lindex $args 0]

	if {([file extension $prjFile] == ".wpj") && (![file exist $prjFile])} {

	    puts stderr "Invalid Project file \"$prjFile\" ... Exiting\n"
	    vxprj::component::help
	}

	if {((! [file exists [lindex $args 0]]) && ($procName != "diff")) ||
	    (($procName == "diff") && ([llength $args] < 2))} {

	    # try to find a .wpj file in the current directory

	    set fileList	[glob -nocomplain *.wpj]

	    if {[llength $fileList] >= 1} {
		set fileName	[lindex $fileList 0]
		set args	[linsert $args 0 $fileName]
                set prjFile    $fileName
	    }
	}
    }

    # keep the validation status in mind

    set validation ""

    if {$procName == "list"} {

	# get the component list

	if {[catch {eval vxprj::component::lst ${args}} res]} {
	    puts stderr $res
	    exit -1
	}

	# print components to standard output

	set components	[lindex $res 0]
	set validation	[lindex $res 1]

	foreach component $components {
	    puts stdout $component
	}

    } elseif {$procName == "diff"} {

	# get the components differences

	if {[catch {eval vxprj::component::diff $args} comps]} {
	    puts stderr $comps
	    exit -1
	}

	set thisComps	[lindex $comps 0]
	set otherComps	[lindex $comps 1]

	puts stdout "\nComponents unique to [file tail [lindex $args 0]] :"
	foreach component $thisComps {
	    puts stdout $component
	}

	puts stdout "\nComponents unique to [file tail [lindex $args 1]] :"
	foreach component $otherComps {
	    puts stdout $component
	}

    } elseif {$procName == "dependencies"}  {

        # get the components depedencies

        if {[catch {eval vxprj::component::dependencies $args} deps]} {
            puts stderr $deps
            exit -1
        }

        set titles              [list "\nRequired Components :" \
                                      "\nDepending Components :"]
        set ix                  0

        foreach depType $deps {

            set printed {}
            puts stdout [lindex $titles $ix]
            incr ix

            foreach comp [lsort -index 0 -dictionary $depType] {

                set compName            [lindex $comp 0]
                set reasons             [lindex $comp 1]
                puts  -nonewline "$compName"

                # XXX : fle : some dependencies are listed twice, just try to
                # ouput them once

                foreach reason $reasons {
                    if {[llength $reason] != 0} {

                        if {[lsearch -exact $printed [lindex $reason 0]]==-1} {

                            puts stdout "    - [lindex $reason 0]"
                        }

                        lappend printed [lindex $reason 0]
                    }
                }
            }
        }


    } elseif {$procName == "dependDetail"} {

	# get the components  depedencies details

        global cxrUnavailInfo

	if {[catch {eval vxprj::component::dependencies $args "detail"} deps]} {
	    puts stderr $deps
	    exit -1
	}

        set cmpName [lindex $args 1]
        vxprj::component::projectFileOpen $prjFile
        set hProj	[cmpProjHandleGet]
        array set compRequires {}
        set opt ""
        puts "\nRetrieving depdendency message for $cmpName..."
        set cxrDoc [cxrDocCreate $hProj]
        set mxrDoc [mxrDocCreate $hProj]
      	set cmpSet	[$cxrDoc setCreate]
     	set cmpSet2	[$cxrDoc setCreate]
        set modSet	[$mxrDoc setCreate]

        $cmpSet = $cmpName
        $cmpSet2 = [$cmpSet get REQUIRES] 
        set components [format "%s" [$cmpSet2 instances Component]]    
        set included [cmpListGet]
        set unavailCmpList [cmpUnavailListGet]
        cxrUnavailListGet $cxrDoc $mxrDoc "extended"
        array set misCompsObj [lindex $cxrUnavailInfo 3]
	set titles		[list "\nRequired Components :" \
				      "\nSuper Components :"]
	set ix	0
        cxrInfoParse2 $deps compRequires $titles $ix $included $unavailCmpList $opt
        set tmp {}
  
        foreach cmp [array names compRequires] {

             foreach cm  $compRequires($cmp) {
                set ix [lsearch -exact $cm "symbol"]
                    if { $ix != -1 } {
                         set cm [lreplace $cm $ix $ix]
                         set cn [cxrCmpGetBySym $cxrDoc $cm]
                         lappend tmp [lreplace $cm 0 end [lindex $cn 0]]
                         set cm [lreplace $cm 0 end ]
                    }
                    if { [string first ".o" $cm] !=-1} {
                         set cn [cxrCmpGetBySym $cxrDoc $cm]  
                         lappend tmp [lreplace $cm 0 end [lindex $cn 0]]
                         set  cm [lreplace $cm 0 end ]
                    }
                    if { $cm != "" } {
                        lappend tmp $cm
                    }
             }
             if { $tmp != "" } {
                 set  compRequires($cmp) $tmp
                 set tmp {}
             }
         }
         set compRequires($cmpName) $components 
         puts stdout "\nTracing the dependency tree...\n"                  
         cxrTrace $cmpName $hProj misCompsObj compRequires "dep"
         $cmpSet delete
         $cmpSet2 delete
         $modSet delete

    } elseif  {$procName == "project"} {
      	if {[catch {eval vxprj::component::${procName} ${args}} status]} {
	    puts stderr $status
	    exit -1
	}
          
    } else {

	# evaluate the expression

	if {[catch {eval vxprj::component::${procName} ${args}} status]} {
	    puts stderr $status
	    exit -1
	}

	# if it was an add or remove command, the result should be the
	# validation status of the current project

	if {($procName == "add") || ($procName == "remove")} {
	    set validation $status
	}
    }

    # print the validation result if needed

    if {$validation != ""} {
	foreach problem $validation {
	    vxprj::componentProblemReport [lindex $problem 0] \
					  [lindex $problem 1]
	}
    }
}

proc vxprj::fsfile {args} {

    set procName	[lindex $args 0]
    set args		[lreplace $args 0 0]

    if {($procName == "") ||
	(! [vxprj::fsfile::isSupportedRequest $procName])} {

	vxprj::fsfile::help

    } else {

	set args [vxprj::fileFind vxprj::fsfile::help $args]
    }

    if {$procName == "list"} {

	# get the fsfile list

	if {[catch {eval vxprj::fsfile::lst ${args}} fsFileList]} {
	    puts stderr $fsFileList
	    exit -1
	}

	# print kernel file system files to standard output

	foreach fsFile $fsFileList {
	    puts stdout $fsFile
	}

    } elseif {$procName == "get"} {

	# evaluate the expression

	if {[catch {eval vxprj::fsfile::get ${args}} fileInfo]} {
	    puts stderr $fileInfo
	    exit -1
	}

	puts stdout $fileInfo

    } elseif {$procName == "create"} {

	# evaluate the expression

	if {[catch {eval vxprj::fsfile::create ${args}} warnings]} {
	    puts stderr $warnings
	    exit -1
	}

	# print the warnings if any

	if {[llength $warnings] > 0} {
	    foreach warning $warnings {
		puts stdout $warning
	    }
	}

    } else {

	# evaluate the expression

	if {[catch {eval vxprj::fsfile::${procName} ${args}} status]} {
	    puts stderr $status
	    exit -1
	}
    }
}

################################################################################
#
# vxprj::parameter - vxprj parameters facilities dispatcher
#
# SYNOPSIS
# \cs
#   vxprj::parameter diff      [<prjfile>] <prjfile|prjdir>
#                    help
#                    list      [<prjfile>] [<namepattern>]
#                    reset     [<prjfile>] <parameter> [<parameter> ...]
#                    set       [<prjfile>] <parameter> <value>
#                    setstring [<prjfile>] <parameter> <value>
#                    value     [<prjfile>] [<namepattern> [<valuepattern>]]
# \ce
#
# DESCRIPTION
# Process kernel configuration parameter actions.
#
# GETTING PARAMETERS DIFFERENCES
# Using the 'diff' command prints out the kernel configuration parameters
# which differ between the current kernel configuration project (either
# the .wpj file in the current directory, either the first .wpj file given to
# the command line), and the kernel configuration project specified by either
# <prjfile>, or <prjdir>
#
# GETTING PARAMETERS LIST
# The 'list' command lists all the parameters of the current kernel
# configuration project which name match <namepattern>. If no <namepattern> is
# specified, all the parameters are listed.
#
# RESETTING PARAMETER VALUE
# Using the 'reset' command allows to reset the given parameter(s) value to
# its default value.
#
# SETTING PARAMETER VALUE
# The 'set' command allow to set the parameter value to the give <value>.
# Special case for strings are handled by the 'setstring' command.
#
# SETTING PARAMETER VALUE TO STRING
# As some of the parameters values have to be surrounded by double quotes
# (for example a configuration string with several words), and as the various
# shells may interprete differetly the double quotes, the 'setstring' command
# allows to set a parameter value to a string surrounded by double quotes.
#
# Whatever the <value> is set to, using this command will surround it with
# double quotes.
#
# GETTING PARAMETERS VALUE
# The 'value' command displays the parameter name and value like
# \cs
# PARAMETER = VALUE
# \ce
# for each of the parameters matching <namepattern> and <valuepattern>.
#
# Providing <namepattern> allows to filter based on the parameter name, while
# providing <valuepattern> also filters on the parameter value. In order to get
# all the parameters values matching <valuepattern> without any name pattern
# to match, please set <namepattern> to '.*'
#
# PARAMETERS
#   'diff' : pocesses a parameters difference between parameters (and their
#            associated values) of two project files
#   'help' : print the help message of the project parameter facility
#   'list' : list all parameters of <prjfile> which name matches <namepattern>.
#            If <namepattern> is not passed, all names are returned
#   'reset' : reset the value of <parameter> to its default value. Several
#             <parameter> names can be given
#   'set' : set the <parameter> value to <value>
#   'setstring' : set the parameter value to <value>, and surrounf it with
#                 double quotes unless it is already surrounded by double quotes
#   'value' : get the list of parameters, and their associated values. If
#             <namepattern> is given, the parameter names should match it. If
#             <valuepattern> is given, the parameter values should match it. To
#             only filter on parameter value, set the <namepattern> to '.*'
#   <namepattern> : pattern the parameter name should match
#   <prjdir> : project in which the project file stands. The project file name
#              should be the same as its parent directory (e.g. dir=a/b =>
#              file=a/b/b.wpj)
#   <prjfile> : project file to be used
#   <valuepattern> : pattern the parameter value should match
#
# RETURNS: N/A
#
# ERRORS: All errors should be caugth, and printed to the standard error
# channel.
#
# SEE ALSO: 'vxprj::component'
#

proc vxprj::parameter {args} {

    set procName	[lindex $args 0]
    set args		[lreplace $args 0 0]

    if {($procName == "") ||
        ([lsearch -exact "diff list reset set setstring value" $procName] == -1) } {

        vxprj::parameter::help

    } else {

	# If the first argument ends with ".wpj", we may consider it is a
	# project file name. If it does not exist, the user asked for a wrong
	# file, we have to tell him

	set prjFile	[lindex $args 0]

	if {([file extension $prjFile] == ".wpj") && (![file exist $prjFile])} {

	    puts stderr "Invalid Project file \"$prjFile\" ... Exiting\n"
	    vxprj::parameter::help
	}

	# check if the first argument is a valid file name

	if {((! [file exists [lindex $args 0]]) && ($procName != "diff")) ||
	    (($procName == "diff") && ([llength $args] < 2))} {

	    # try to find a .wpj file in the current directory

	    set fileList	[glob -nocomplain *.wpj]

	    if {[llength $fileList] >= 1} {
		set fileName	[lindex $fileList 0]
		set args	[linsert $args 0 $fileName]
	    }
	}
    }

    if {$procName == "list"} {

	if {[catch {eval vxprj::parameter::lst $args} parameters]} {

	    puts stderr $parameters
	    exit -1
	} else {

	    foreach parameter $parameters {
		puts stdout $parameter
	    }
	}

    } elseif {$procName == "value"} {

	if {[catch {eval vxprj::parameter::value ${args}} parameters]} {
	    puts stderr $parameters
	    exit -1
	} else {

	    foreach parameter $parameters {
		puts stdout "[lindex $parameter 0] = [lindex $parameter 1]"
	    }
	}

    } elseif {$procName == "diff"} {

	if {[catch {eval vxprj::parameter::diff $args} params]} {
	    puts stderr $params
	    exit -1
	}

	set thisComps	[lindex $params 0]
	set otherComps	[lindex $params 1]

	puts stdout "\nParameters unique to [file tail [lindex $args 0]] :"
	foreach parameter $thisComps {
	    puts stdout $parameter
	}

	puts stdout "\nParameters unique to [file tail [lindex $args 1]] :"
	foreach parameter $otherComps {
	    puts stdout $parameter
	}

    } elseif {$procName == "set"} {

	eval vxprj::parameter::_set ${args}

    } else {
	eval vxprj::parameter::${procName} ${args}
    }
}

################################################################################
#
# vxprj::fileFind - find a valid project file from arguments
#
# SYNOPSIS
# \cs
#   vxprj::fileFind <help> <args>
# \ce
#
# DESCRIPTION
# Find a valid project file from arguments.
#
# The <args> arguments are parsed, looking for a valid project file. If the
# first argument is a '.wpj' file, and that it does not exist, this procedure
# will output <Invalid Project file ...> on standard error, and the
# help (as defined by <help> parameter) will be called.
#
# Else, if the first argument is not a '.wpj' file, a '.wpj' file will be
# looked for in the current directory. If several '.wpj' are found, then the
# first in the list (as returned by Tcl's 'glob' procedure) will be prepended
# to the arguments.
#
# If no '.wpj' file is found, then the procedure will return the <args>
# unchanged.
#
# PARAMETERS:
#   'help' : help procedure to be called if project file is not valid
#   <args> : arguments to look for project file in
#
# RETURNS: <args> prepended with a project file (if any was found)
#
# NOMANUAL
#

proc vxprj::fileFind {help args} {

    # XXX : fle : dunno why, but it seems like arguments are only in one list
    # element ...

    set arguments	[lindex $args 0]
    set prjFile		[lindex $arguments 0]

    if {([file extension $prjFile] == ".wpj") && (![file exist $prjFile])} {

	puts stderr "Invalid Project file \"$prjFile\" ... Exiting\n"
	$help
    }

    # check if the first argument is a valid file name

    if {(! [file exists $prjFile]) || (! [file isfile $prjFile]) ||
	([file extension $prjFile] != ".wpj")} {

	# try to find a .wpj file in the current directory

	set fileList	[glob -nocomplain *.wpj]

	if {[llength $fileList] >= 1} {

	    set fileName	[lindex $fileList 0]

	    if {[string length [lindex $args 0]] == 0} {
		set arguments	$fileName
	    } else {
		set arguments	[linsert $arguments 0 $fileName]
	    }
	}
    }

    return $arguments
}

################################################################################
#
# vxprj::bundle - vxprj bundle facilities dispatcher
#
# SYNOPSIS
# \cs
#   vxprj::bundle help
#                 add    [<prjfile>] <bundle> [<bundle> ...]
#                 get    [<prjfile>] <bundle>
#                 list   [<prjfile>] [{all|excluded|included}] [<pattern>]
#                 remove [<prjfile>] <bundle> [<bundle> ...]
# \ce
#
# DESCRIPTION
# A 'vxprj' facility to handle bundles.
#
# Bundles are just a list of kernel components. Adding a bundle adds all the
# components it contains, and their dependencies, but there is no way to really
# know if a bundle has been added or not.
#
# ADDING A BUNDLE
# When adding a bundle, all the components it contains are added to the
# kernel components. All dependencies are also fullfilled.
#
# GETTING A BUNDLE
# All the fields and the values of the given <bundle> are displayed on
# the standard output.
#
# LISTING BUNDLES
# Issuing a 'vxprj list' command displays all the bundles currently inclued
# by the kernel configuration project.
#
# To get the list of all available bundles, the 'all' keyword has to be used,
# while specifying 'excluded' get the list of all bundles which are
# available, but not included by the current kernel configuration project.
#
# REMOVING A BUNDLE
# When removing a bundle, all the components of the bundle are removed from the
# current kernel components configuratio.
#
# PARAMETERS:
#   'add' : Adds the given <bundle> (or list of bundles) components to the
#         current project
#   'get' : Get the properties of the given <bundle> and their associated values
#   'help' : print the help message for the 'prj bundle' facility
#   'list' : Get the list of bundles matching <pattern> for the current project
#          the 'all', 'excluded' or 'included' modifiers may be used
#   'remove' : remove <bundle> (or list of bundles) components from the current
#            project
#   <bundle> : name of a bundle available from to the current project
#   <prjfile> : path to a project file which will define the current project
#   <pattern> : bundle name pattern to match
#
# RETURNS: N/A
#
# ERRORS: All errors are caught, and error message sent to standard error
# channel
#
# EXAMPLES:
# Get the list of all available bundles, add one of them, check that it really
# has been added :
# \cs
#   # vxprj bundle list
#   # vxprj bundle list all
#   BUNDLE_EDR
#   BUNDLE_NET_SHELL
#   BUNDLE_POSIX
#   BUNDLE_RTP_DEPLOY
#   BUNDLE_RTP_DEVELOP
#   BUNDLE_STANDALONE_SHELL
#   # vxprj bundle add BUNDLE_STANDALONE_SHELL
#   adding INCLUDE_STANDALONE_SYM_TBL
#   
#   Removing INCLUDE_PASSFS_SYM_TBL because INCLUDE_STANDALONE_SYM_TBL has been
#   added to SELECT_SYM_TBL_INIT
#   # vxprj bundle list
#   BUNDLE_STANDALONE_SHELL
#   #
# \ce
#
# SEE ALSO: 'vxprj::component', 'vxprj::parameter'
#

proc vxprj::bundle {args} {

    set procName	[lindex $args 0]
    set args		[lreplace $args 0 0]

    if {($procName == "") ||
	(! [regexp "add|get|list|remove" $procName])} {

	vxprj::bundle::help

    } else {

	set args [vxprj::fileFind vxprj::bundle::help $args]
    }

    if {$procName == "list"} {

	if {[catch {eval vxprj::bundle::lst $args} bundles]} {

	    puts stderr $bundles
	    exit -1
	} else {

	    foreach bundle $bundles {
		puts stdout $bundle
	    }
	}

    } elseif {$procName == "get"} {

	if {[catch {eval vxprj::bundle::get $args} bundle]} {

	    puts stderr $bundle
	    exit -1
	} else {

	    foreach property $bundle {
		puts stdout [format "%s = %s" [lindex $property 0] \
				    [lindex $property 1]]
	    }
	}

    } else {
	if {[catch {eval vxprj::bundle::${procName} ${args}} status]} {
	    puts stderr $status
	    exit -1
	}
    }
}

################################################################################
#
# vxprj::delete - delete the given project and all its associated files
#
# SYNOPSIS
# \cs
#   vxprj::delete <prjfile>
# \ce
#
# DESCRIPTION
# This procedure deletes all the directory tree of the <prjfile>. The project
# file itself is erased as long as all the sub directories of the file location,
# all the files of the same location, and possibly the parent directory once
# all files have been erased
#
# Theres is no possibility of restoring it, the project and all the files
# in the same tree are lost
#
# NOTE
# Please make sure there is no application which accesses the directory where
# the project is. This directory is gonna be removed
#
# PARAMETERS:
#   <prjfile> : path of the project file to be deleted
#
# RETURNS: N/A
#
# ERRORS: N/A
#
# EXAMPLES:
# Remove the project defined by linux_diab/linux_diab.wpj
# \cs
#  # vxprj delete linux_diab/linux_diab.wpj
#  #
# \ce
#
# SEE ALSO: 'cmpProjDelete'
#

proc vxprj::delete {prjFile} {

    cmpProjOpen $prjFile
    cmpProjDelete
}

################################################################################
#
# vxprj::famous - gives access to the (hidden) famous sayings
#
# SYNOPSIS
# \cs
#   vxprj::famous <args>
# \ce
#
# DESCRIPTION
# Prints Wind River's famous sayings to the standard output
#
# PARAMETERS:
#   <args> : must be set to <sayings>, or nothing will happen
#
# RETURNS: N/A
#
# ERRORS: N/A
#
# NOMANUAL
#

proc vxprj::famous {args} {

    set procName	[lindex $args 0]

    if {$procName != "sayings"} {

	exit 0

    } else {

	puts stdout [vxprj::famous::sayings]
    }
}

################################################################################
#
# vxprj::build - vxprj build facility dispatcher
#
# SYNOPSIS
# \cs
#   vxprj::build [<prjfile>] <buildspec|rule>
#                help
#                get   [<prjfile>]
#                list  [<prjfile>] [<pattern>]
#                set   [<prjfile>] <buildspec>
#                reset [<prjfile>]
# \ce
#
# DESCRIPTION
# This procedure dispatches the build command to the right build procedure.
#
# If the command is to build (first argument is either <prjfile> or
# <buildspec|rule>), a build will be started on <buildspec|rule>
#
# If <buildspec|rule> does not belong to the list of available build
# specifications, it will be considered a rule.
#
# \is
# \i If <buildspec|rule> is a build rule
# The rule will be built for the current build specification
# \i If <buildspec|rule> is a build spec
# The default build rule of the given build spec is being built
# \ie
#
# GETTING BUILD SPECIFICATION
# The 'get' action gets the name of the currently used build specification. This
# value is printed on the standard output.
#
# LISTING BUILD SPECIFICATIONS
# The 'list' action lists all the build specifications of project specified
# by <prjFile>, and print them on the standard output, one line per build
# specification.
#
# If <pattern> is provided, only the build specifications which name match
# <pattern> will be returned.
#
# SETTING A BUILD SPECIFICATION
# The 'set' action sets the kernel configuration's current build specification
# to <buildspec>. The so-given <buildspec> has to be one of the possible
# build specifications returned by 'vxprj build list'.
#
# RESETTING A BUILD SPECIFICATION
# The 'reset' action just resets the build spec to its default value.
#
# PARAMETERS:
#   'get' : get the current build specification used by <prjfile> project
#   'help' : print the help message for the project build facility and exit
#   'list' : list all available build specifications
#   'set' : set the build specification
#   'reset' : reset the build specification to its default value
#   <prjfile> : the project file path. Default is the '.wpj' file in the current
#               directory
#   <target> : a valid build target name
#   <pattern> : pattern the build specification's name should match
#   <buildspec> : name of a valid build specification for the given project
#
# RETURNS: N/A
#
# EXAMPLES:
# Build clean the local project (aka project file is in the current directory) :
# \cs
#   vxprj::build clean
# \ce
# Build vxWorks in romResident :
# \cs
#   vxprj::build default_romResident
# \ce
#
# SEE ALSO: 'vxprj::bundle', 'vxprj::component', 'vxprj::parameter'
#

proc vxprj::build {args} {

    # first, check if the first argument is one of the authorised command. Else
    # consider it is a build command on a given target

    set procName	[lindex $args 0]

    if {$procName == "help"} {
	vxprj::build::help
    } else {

	# remove the procedure name from arguments

	if {($procName == "list")   || ($procName == "reset") ||
	    ($procName == "set")    || ($procName == "get")   ||
            ($procName == "source") || ($procName == "lib") } {

	    if {[llength $args] == 1} {
		set args	{}
	    } else {
		set args	[lreplace $args 0 0]
	    }
	}

	# check if the project file is given, if not, find one

	set args [vxprj::fileFind vxprj::bundle::help $args]

	# call the corresponding procedure

	if {$procName == "set"} {

	    if {[catch {eval vxprj::build::_set $args} status]} {

		puts stderr $status
		exit -1

	    } else {

		puts stdout "Project build specification set to \"$status\""
	    }

	} elseif {$procName == "reset"} {

	    if {[catch {eval vxprj::build::reset $args} status]} {

		puts stderr $status
		exit -1

	    } else {

		puts stdout "Project build specification set to \"$status\""
	    }

	} elseif {($procName == "list")} {

	    if {[catch {eval vxprj::build::lst $args} buildList]} {

		puts stderr $buildList
		exit -1
	    } else {

		foreach bld $buildList {

		    puts stdout $bld
		}
	    }

	} elseif {$procName == "get"} {

	    if {[catch {eval vxprj::build::get $args} bld]} {

		puts stderr $bld
		exit -1

	    } else {

		puts stdout $bld
	    }

	} elseif {$procName == "source"} {
           if {[catch {eval vxprj::build::source $args} srce]} {

                puts stderr $srce
                exit -1
            } 

        } elseif {$procName == "lib"} {
           if {[catch {eval vxprj::build::lib $args} libe]} {

                puts stderr $libe
                exit -1
            } 

        } else {

	    # this is a build request

	    set prjFile	[lindex $args 0]
	    set hSpec	[lindex $args 1]

	    if {[catch {
		    cmpProjOpen $prjFile
		    set hProj	[cmpProjHandleGet]

		    # determine if hSpec is a build rule or a build spec

		    if {[lsearch -exact [prjBuildListGet $hProj] $hSpec]==-1} {
			set rule	$hSpec
			set hSpec	[prjBuildCurrentGet $hProj]
		    } else {
			set rule	[prjBuildRuleCurrentGet $hProj $hSpec]
		    }

		    cmpBuild $rule $hSpec
		    cmpProjClose
		    } status] } {

		puts stderr $status
		exit -1
	    }
	}
    }
}

################################################################################
#
# vxprj::buildrule - vxprj build rules facility dispatcher
#
# SYNOPSIS
# \cs
#   vxprj::buildrule [<prjfile>] <rule>
#                    add    [<prjfile>] <rule> <value>
#                    help
#                    get    [<prjfile>] <rule>
#                    list   [<prjfile>] [<pattern>]
#                    set    [<prjfile>] <rule> <value>
#                    remove [<prjfile>] <rule>
# \ce
#
# DESCRIPTION
# This procedure dispatches the build rules command to the right build rules
# procedure.
#
# SETTING THE DEFAULT BUILD RULE FOR THE CURRENT BUILD SPECIFICATION
# If the first argument is either the project file (<prjfile>) followed by a
# rule name (<rule>), or just the rule name, then the default build rule for the
# current build specfication is set to <rule> (assuming <rule> is part of the
# rules for the current build specification)
#
# ADDING A BUILD RULE
# The 'add' action adds the given rule named <rule> to the list of possible
# build rules for the current build specification, giving it the <value>
# value.
#
# GETTING A BUILD RULE
# The 'get' action gets the value of the build rule named <rule> for the current
# build specification. Build rule name and values are printed to the standard
# output with the followinf pattern : <rule>: <value>
#
# LISTING BUILD RULE
# The 'list' action lists all the registered build rules for the current build
# specification of project specified by <prjFile>, and print them on the
# standard output, one line per build rule.
#
# If <pattern> is provided, only the build rules which name match <pattern> are
# returned.
#
# SETTING A BUILD RULE
# The 'set' action sets the kernel configuration's <rule> build rule value to
# <value>. If the given <rule> does not exist yet, it is added to the list of
# possible build rules for the current build specification, and will be listed
# in any subsequent build rules list (unless it is removed)
#
# REMOVING A BUILD RULE
# The 'remove' action just removes the build rule named <rule> for the current
# build specification build rules.
#
# PARAMETERS:
#   'add' : add a build rule to the current build specification
#   'get' : get the given build rule for the current build specification
#   'help' : print the help message for the project build facility and exit
#   'list' : list all available build specifications
#   'set' : set a build rule value
#   'remove' : remove a build rule from the current build specification
#   <prjfile> : the project file path. Default is the '.wpj' file in the current
#               directory
#   <rule> : name of a build rule
#   <pattern> : pattern the build rule's name should match
#   <value> : value to give to the given build rule
#
# RETURNS: N/A
#
# EXAMPLES:
# List all the build rules for a default linux BSP project using gnu toolchain :
# \cs
#  # vxprj buildrule list
#  vxWorks
#  linkSyms.o
#  prjConfig.o
#  sysALib.o
#  sysLib.o
#  vxWorks_rom
#  vxWorks_rom.bin
#  vxWorks_rom.hex
#  vxWorks_romCompress
#  vxWorks_romCompress.bin
#  vxWorks_romCompress.hex
#  vxWorks_romResident
#  vxWorks_romResident.bin
#  vxWorks_romResident.hex
#  #
# \ce
#
# SEE ALSO: 'vxprj::build'
#

proc vxprj::buildrule {args} {

    # first, check if the first argument is one of the authorised command. Else
    # consider it is a build command on a given target

    set procName	[lindex $args 0]

    if {$procName == "help"} {
	vxprj::buildrule::help
    } else {

	if {[lsearch -exact $vxprj::buildrule::mSupportedRequests $procName]
	    == -1} {

	    set args [vxprj::fileFind vxprj::bundle::help $args]

	    # it is a request to set or get the default build rule

	    if {[catch {eval vxprj::buildrule::current $args} rule]} {

		puts stderr $rule
		exit -1

	    } else {

		puts stdout $rule
	    }
	} else {

	    set args [vxprj::fileFind vxprj::bundle::help [lreplace $args 0 0]]

	    if {$procName == "set"} {

		if {[catch {eval vxprj::buildrule::_set $args} rule]} {

		    puts stderr $rule
		    exit -1

		} else {

		    puts stdout "Project build rule [lindex $rule 0] set to\
				 \"[lindex $rule 1]\""
		}

	    } elseif {$procName == "add"} {

		if {[catch {eval vxprj::buildrule::add $args} rule]} {

		    puts stderr $rule
		    exit -1

		} else {

		    puts stdout "Project build rule [lindex $rule 0] set to\
				 \"[lindex $rule 1]\""
		}

	    } elseif {$procName == "remove"} {

		if {[catch {eval vxprj::buildrule::remove $args} rule]} {

		    puts stderr $rule
		    exit -1

		} else {

		    puts stdout "Removed [lindex $rule 0] rule from build spec"
		}

	    } elseif {($procName == "list")} {

		if {[catch {eval vxprj::buildrule::lst $args} ruleList]} {

		    puts stderr $ruleList
		    exit -1
		} else {

		    foreach rule $ruleList {

			puts stdout $rule
		    }
		}

	    } elseif {$procName == "get"} {

		if {[catch {eval vxprj::buildrule::get $args} rule]} {

		    puts stderr $rule
		    exit -1

		} else {

		    puts stdout $rule
		}
	    } elseif {$procName == "dependencies"} {

		if {[catch {eval vxprj::buildrule::dependencies $args} deps]} {

		    puts stderr $deps
		    exit -1

		} else {

		    puts stdout $deps
		}
	    }
	}
    }
}

################################################################################
#
# vxprj::buildmacro - vxprj build macro facility dispatcher
#
# SYNOPSIS
# \cs
#   vxprj buildmacro help
#                    add    [<prjfile>] <macro> [<value>]
#                    get    [<prjfile>] <macro>
#                    list   [<prjfile>] [<namePattern>]
#                    remove [<prjfile>] <macro> [<macro> ...]
#                    set    [<prjfile>] <macro> [<value>]
#                    value  [<prjfile>] [<namePattern> [<valuePattern>]]
# \ce
#
# DESCRIPTION
# This 'vxprj' facility allows to manipulate kernel configuration build
# macro values.
#
# ADDING OR SETTING A BUILD MACRO VALUE
# The 'set' and 'add' actions are very similar, and just set/add the build
# macro value to the given <value>. If no <value> is given to the command line,
# the macro value is set to an empty string.
#
# This action does not return anything to the standard output
#
# GETTING A BUILD MACRO VALUE
# The 'get' action prints the value of the given <macro> to the standard
# output. If the macro does not exist, or if its value is an empty string, then
# nothing is printed to the standard output.
#
# To be sure the macro exists, it must be part of the list returned by
# a 'vxprj buildmacro list' call.
#
# LISTING BUILD MACRO VALUES
# The 'list' action prints all build macros matching <namePattern> on the
# standard output. Each macro name comes on a new line. If no <namePattern>
# is provided, all the macros are printed.
#
# REMOVING A BUILD MACRO
# The 'remove' action simply removes the given <macro> and its associated value
# from the current kernel configuration. After that, there is no way to get
# the value the macro had.
#
# Several macros can be removed in one shot if they all are given to the
# 'remove' parameters.
#
# GETTING BUILD MACRO VALUES
# Using the 'value' action prints on the standard output all the build macros
# which name match <namePattern>, and which value match <valuePattern>.
#
# Macros and values are printed on a single line, like MACRO=value
#
# If <valuePattern> is provided, and there is no special match to do on the
# macro name, <namePattern> should be set to '.*'
#
# If no <valuePattern> not <namePattern> is provided, '.*' pattern is used for
# both of them.
#
# PARAMETERS:
#   'add' : add <macro> to the build macros, which value set to <value>. If no
#           <value> is provided, it is set to an empty string
#   'get' : get the value of macro named <macro> if it exists
#   'help' : display the 'vxprj buildmacro' help message
#   'list' : list all the possible macros which name match <namePattern>.
#            Default <namePattern> is '.*'
#   'remove' : remove macro named <macro> from build macros list
#   'set' : set the value of macro named <macro> to <value>. If no <value> is
#           provided, the macro value is set to an empty string
#   'value' : return a list of macro and theyr associated values. Macro names
#             should match <namePatter> while macro values should match
#             <valuePattern>. Default values for patterns are '.*'
#   <macro> : name of a build macro
#   <namePattern> : pattern the macro name should match (default is '.*')
#   <value> : value to give to build macro
#   <valuePattern> : pattern the macro value should match (default is '.*')
#
# RETURNS: N/A
#
# ERRORS: N/A
#
# SEE ALSO: 'vxprj::build'
#

proc vxprj::buildmacro {args} {

    # first, check if the first argument is one of the authorised command. Else
    # consider it is a build command on a given target

    set procName	[lindex $args 0]
    set args		[lreplace $args 0 0]

    if {($procName == "") ||
	(! [vxprj::buildmacro::isSupportedRequest $procName])} {

	vxprj::buildmacro::help

    } else {

	set args [vxprj::fileFind vxprj::buildmacro::help $args]
    }

    # call the corresponding procedure

    if {$procName == "set"} {
	if {[catch {eval vxprj::buildmacro::_set $args} status]} {
	    puts stderr $status
	    exit -1
	}
    } elseif {($procName == "list")} {

	if {[catch {eval vxprj::buildmacro::lst $args} macroList]} {
	    puts stderr $macroList
	    exit -1
	} else {
	    foreach macro $macroList {
		puts stdout $macro
	    }
	}
    } elseif {$procName == "get"} {

	if {[catch {eval vxprj::buildmacro::get $args} macro]} {
	    puts stderr $macro
	    exit -1
	} else {
	    puts stdout $macro
	}
    } elseif {$procName == "value"} {

	if {[catch {eval vxprj::buildmacro::value $args} macroDescList]} {
	    puts stderr $macroDescList
	    exit -1
	} else {
	    foreach macroDesc $macroDescList {
		puts stdout [format "%s = %s" [lindex $macroDesc 0] \
					      [lindex $macroDesc 1]]
	    }
	}
    } else {
	if {[catch {eval vxprj::buildmacro::${procName} ${args}} status]} {
	    puts stdout "Caught error : $status\nWhile executing\
			 vxprj::buildmacro::${procName} ${args}"
	}
    }
}

################################################################################
#
# vxprj::file - vxprj file manipulation facility dispatcher
#
# SYNOPSIS
# \cs
#   vxprj file help
#              add    [<prjFile>] <filePath>
#              get    [<prjFile>] <filePath>
#              list   [<prjFile>] [<pattern>]
#              remove [<prjFile>] <filePath>
# \ce
#
# DESCRIPTION
# Manipulate kernel configuration source files.
#
# ADDING A FILE
# Using the 'add' command adds the given <filePath> to the list of source files
# the current kernel configuration project handles. This file is compiled, and
# made part of the generated kernel image.
#
# GETTING A FILE
# The 'get' command lists the properties of the given <filePath>, where
# <filePath> must be one of the file paths returned by a 'list' call.
#
# A list of file properties like 'depenencies' or 'tool' is printed on the
# standard output. The list of properties depends on the file.
#
# LISTING FILES
# Using 'list' command allows to get the list of source files currently handled
# by the kernel configuration project. If a <pattern> is provided, only the
# files matching <pattern> are listed.
#
# REMOVING A FILE
# The 'remove' command removes the given <filePath> from the files handled by
# the kernel configuration project.
#
# PARAMETERS:
#   'add' : add <filePath> to the list of files handled by the current project
#   'help' : print the help message and exit
#   'get' : get <filePath> properties
#   'list' : list the files handled by the current kernel configuration project
#   'remove' : remove <filePath> from the files handled by the current project
#   <filePath> : path to a source file
#   <pattern> : pattern the file path should match
#   <prjFile> : path to the kernel configuration project file. Default is the
#               .wpj file in the current directory
#
# RETURNS: N/A
#
# ERRORS: All errors are caught and printed on the standard error channel
#
# EXAMPLES:
# Add file myFile.c to the current kernel configuration project:
# \cs
#  # vxprj file add <myFile.c>
#  #
# \ce
# List the files :
# \cs
#  # vxprj file list
#  /wind/vxworks-6.0/target/config/linux/sysALib.s
#  /wind/vxworks-6.0/target/config/linux/sysLib.c
#  ./usrAppInit.c
#  ./prjConfig.c
#  ./linkSyms.c
#  ./myFile.c
#  #
# \ce
# Get the properties for that file :
# \cs
#  # vxprj file get myFile.c
#  ./myFile.c
#     objects : myFile.o
#     tool : C/C++ compiler
#  #
# \ce
# And now remove it :
# \cs
#  # vxprj file remove ./myFile.c
#  #
# \ce
#
# SEE ALSO: 'cmpFileAdd', 'cmpFileRemove', 'cmpFileListGet'
#

proc vxprj::_file {args} {

    # first, check if the first argument is one of the authorised command. Else
    # consider it is a build command on a given target

    set procName	[lindex $args 0]
    set args		[lreplace $args 0 0]

    if {($procName == "") ||
	(! [vxprj::_file::isSupportedRequest $procName])} {

	vxprj::_file::help

    } else {

	set args [vxprj::fileFind vxprj::_file::help $args]
    }

    # call the corresponding procedure

    if {($procName == "list")} {

	if {[catch {eval vxprj::_file::lst $args} fileList]} {
	    puts stderr $fileList
	    exit -1
	} else {
	    foreach filePath $fileList {
		puts stdout $filePath
	    }
	}
    } elseif {$procName == "get"} {

	if {[catch {eval vxprj::_file::get $args} fileDesc]} {
	    puts stderr $fileDesc
	    exit -1
	} else {
	    set filePath	[lindex $fileDesc 0]
	    set fileInfo	[lindex $fileDesc 1]
	    puts stdout $filePath
	    foreach info $fileInfo {
		puts stdout "    [lindex $info 0] : [lindex $info 1]"
	    }
	}
    } else {
	if {[catch {eval vxprj::_file::${procName} ${args}} status]} {
	    puts stdout "Caught error : $status\nWhile executing\
			 vxprj::_file::${procName} ${args}"
	}
    }
}

################################################################################
#
# vxprj::module - vxprj module manipulation facility dispatcher
#
# SYNOPSIS
# \cs
#   vxprj module help
#                dependencies [<prjFile>] <module> [<module> ...]
# \ce
#
# DESCRIPTION
#
# PARAMETERS:
#   'help' : print the help message of this facility and exit with status '-1'
#   'dependencies' : print out the dependencies of <module>
#   <module> : name of a kernel module
#   <prjFile> : path of the project file (default is the .wpj file in the
#               current directory)
#
# RETURNS: N/A
#
# ERRORS: N/A
#
# SEE ALSO:
#

proc vxprj::module {args} {

    # first, check if the first argument is one of the authorised command. Else
    # consider it is a build command on a given target

    set procName	[lindex $args 0]
    set args		[lreplace $args 0 0]

    if {($procName == "") ||
	(! [vxprj::module::isSupportedRequest $procName])} {

	vxprj::module::help

    } else {

	set args [vxprj::fileFind vxprj::module::help $args]
    }

    # call the corresponding procedure

    if {$procName == "dependencies"} {

	if {[catch {eval vxprj::module::dependencies $args} deps]} {
	    puts stderr $deps
	    exit -1
	} else {

	    set titles	[list "Required Modules :" "\nDepending Modules :"]
	    set ix			0

	    foreach depType $deps {

		set printed	{}
		puts stdout [lindex $titles $ix]
		incr ix

		foreach mod [lsort -index 0 -dictionary $depType] {

		    set modName		[lindex $mod 0]
		    set reasons		[lindex $mod 1]
		    puts stdout $modName

		    # XXX : fle : some dependencies are listed twice, just try
		    # to ouput them once

		    foreach reason $reasons {
			if {[llength $reason] != 0} {

			    if {[lsearch -exact $printed [lindex $reason 0]]\
				== -1} {

				puts stdout "    - [lindex $reason 0]"
			    }

			    lappend printed [lindex $reason 0]
			}
		    }
		}
	    }
	}
    } else {
	if {[catch {eval vxprj::_file::${procName} ${args}} status]} {
	    puts stdout "Caught error : $status\nWhile executing\
			 vxprj::_file::${procName} ${args}"
	    exit -1
	}
    }
}

################################################################################
#
# vxprj::makefile - vxprj makefile manipulation facility
#
# SYNOPSIS
# \cs
#   vxprj makefile [<prjFile>] [<makefile>]
# \ce
#
# DESCRIPTION
# Get or set the kernel configuration project Makefile name.
#
# PARAMETERS:
#   <prjFile> : path to the project file to get or set Makefile for. Default is
#               the wpj file in the current directory
#   <makefile> : name of the Makefile kernel configuration project Makefile
#                should be set to
#
# RETURNS: The name of the makefile the kernel configuration project uses
#
# ERRORS: N/A
#
# EXAMPLES:
# \cs
# # vxprj makefile
# Makefile
# # vxprj makefile myMakefile.mk
# myMakefile.mk
# #
# \ce
#
# SEE ALSO: 'cmpMakefile'
#

proc vxprj::makefile {args} {

    set args [vxprj::fileFind vxprj::help $args]

    set prjFile		[lindex $args 0]
    set fileName	[lindex $args 1]

    cmpProjOpen $prjFile

    if {$fileName == ""} {
	set currentMakefile	[cmpMakefile]
    } else {
	set currentMakefile	[cmpMakefile $fileName]
    }

    cmpProjClose

    return $currentMakefile
}

################################################################################
#
# vxprj::tccheck - check if toolchain info files are up to date
#
# SYNOPSIS
# \cs
#   vxprj tccheck
# \ce
#
# DESCRIPTION
# This facility checks if the toolchain information files are up to date
# with the kernel make rules.
#
# It checks if the 'tcInfo_*.tcl' files located in
# WIND_BASE/host/resource/osconfig/tcl are up to date.
#
# If one of the files is older than on of the following files:
#
# \is
# \i TGT_DIR/h/make/*
# \i TGT_DIR/h/tool/<family>/*
# Where <family> is the toolchain family (aka, gnu or diab)
# \i TGT_DIR/h/tool/common/*
# \i WIND_FOUNDATION_PATH/resource/osconfig/tcl/tcInfoGen.tcl
# \ie
#
# or if the file does not exist, it is regenerated.
#
# PARAMETERS: N/A
#
# RETURNS: N/A
#
# ERRORS: N/A
#
# SEE ALSO:
#

proc vxprj::tccheck {} {

    # get the possible toolchains form the target/h/tool dir, ommit the common
    # one, and keep only those with make fragments

    foreach file [glob -nocomplain [file join [Wind::tgtDir] h tool *]] {

	if {[file tail $file] == "common"} {
	    continue
	}

	if {[file isdirectory $file]} {
	    set makeStubs [glob -nocomplain [file join $file make.*]]
	    if {$makeStubs != ""} {
		lappend tool_families [file tail $file]
	    }
	}
    }

    set updatedFiles	{}
    set res		""

    foreach family $tool_families {

	set tcInfoFile		[tcInfoFileGet $family]
	set tcInfoGenScript	[file join [Wind::base] host resource osconfig\
					   tcl tcInfoGen.tcl]

	set dependencies [list [file join [Wind::tgtDir] h tool $family *] \
			       [file join [Wind::tgtDir] h tool common *] \
			       [file join [Wind::tgtDir] h make *] \
			       $tcInfoGenScript]

	if {(![file exists $tcInfoFile] || [file writable $tcInfoFile]) &&
	    ([outOfDate $tcInfoFile $dependencies])} {

	    lappend updatedFiles	$tcInfoFile
	    set startDir [pwd]

	    # if tcInfo file dir does not exist, create it

	    if {! [file exists [file dirname $tcInfoFile]]} {
		if {[catch {file mkdir [file dirname $tcInfoFile]} \
			   mkdirStatus]} {

		    puts [prjStderr] "Could not create [file dirname\
				      $tcInfoFile] directory : \"$mkdirStatus\""
		}
	    }

	    cd [file dirname $tcInfoFile]

	    puts stdout "Generating $family toolchain info file from makefiles\
			 in [file nativename [file join [Wind::tgtDir] h tool\
			 $family]] ..."

	    if {[catch {exec [info nameofexecutable] $tcInfoGenScript $family \
			     >@ stdout} status]} {

		set errorMessage "Error creating $family toolchain information\
				  files. You will not be able to open or create\
				  projects until this error is resolved:\
				  \n\n$status"

		error $errorMessage
	    }

	    # restore argv and argc

	    cd $startDir
	}
    }

    if {[llength $updatedFiles] == 0} {
	puts stdout "Toolchain information files are up to date"
    }
}

################################################################################
#
# vxprj::profile - vxprj profile manipulation facility dispatcher
#
# SYNOPSIS
# \cs
#   vxprj::profile help
#                  get  [<prjfile>] <profile>
#                  list [<prjfile>] [<pattern>]
# \ce
#
# DESCRIPTION
# Kernel components profile manipulation
#
# GETTING A LIST OF PROFILES
# The 'list' options allows to get the list of currently available profiles.
# If no <prjfile> is given, only the profiles of the default vxWorks kernel
# component directory (TGT_DIR/config/comps/vxWorks) are listed.
#
# If a <prjfile> is provided, the architecture, BSP and project directories
# are also parsed to look for profiles in the CDF files.
#
# Only profiles which name matches <pattern> are returned. If <pattern> is
# ommitted, all profile names are returned.
#
# GETTING A PROFILE DESCRIPTION
# The 'get' option returns the full description of <profile>, where <profile>
# must be one of the profile names returned by the 'list' option.
#
# PARAMETERS:
#   <prjfile> : the project file to open if needed
#   <pattern> : the profile name pattern to match (if any)
#   <profile> : name of the profile to get description for
#
# RETURNS: N/A
#
# ERRORS: N/A
#
# EXAMPLES:
# \cs
# # vxprj profile list NET
# PROFILE_ENHANCED_NET
# #
# \ce
#
# SEE ALSO: vxprj::create
#

proc vxprj::profile {args} {

    # first, check if the first argument is one of the authorised command. Else
    # consider it is a build command on a given target

    set procName	[lindex $args 0]
    set args		[lreplace $args 0 0]

    if {($procName == "") ||
	(! [vxprj::profile::isSupportedRequest $procName])} {

	vxprj::profile::help

    } else {

	set args	[vxprj::fileFind vxprj::help $args]

	# look at the arguments. First should be the project file (which
	# extension should be .wpj). If it is not the case, provide with
	# an empty project file

	if {[llength $args] >= 1} {

	    # check the file extension

	    if {[file extension [lindex $args 0]] != ".wpj"} {
		set args [linsert $args 0 ""]
	    }
	}
    }

    # call the corresponding procedure

    if {($procName == "list")} {

	if {[catch {eval vxprj::profile::lst $args} profileList]} {
	    puts stderr $profileList
	    exit -1
	} else {
	    foreach profile $profileList {
		puts stdout $profile
	    }
	}

    } elseif {$procName == "get"} {

	if {[catch {eval vxprj::profile::get $args} profile]} {
	    puts stderr $profile
	    exit -1
	} else {
	    foreach property $profile {
		puts stdout $property
	    }
	}

    } else {

	if {[catch {eval vxprj::profile::${procName} ${args}} status]} {
	    puts stdout "Caught error : $status\nWhile executing\
			 vxprj::profile::${procName} ${args}"
	    exit -1
	}
    }
}

################################################################################
#
# vxprj::autoscale - automatically scale kernel configuration project
#
# SYNOPSIS
# \cs
#   vxprj::autoscale [-terse|-shrink] [<prjfile>]
# \ce
#
# DESCRIPTION
# Configure kernel configuration components based on what is required by the
# project source files.
#
# There are three possible options :
# \is
# \i 'default' (no option specified)
# Is the default mode, it just tells what scaling can/should be done, and why
# \i '-terse'
# Prints only terse information
# \i '-shrink'
# Causes autoscaling to actually be done
# \ie
#
# PARAMETERS
#    <prjfile> : kernel configuration project file to do autoscale for
#
# RETURNS: N/A
#
# SEE ALSO: 'autoscale'
#

proc vxprj::autoscale {args} {

    set firstArg	[lindex $args 0]

    if {($firstArg == "-terse")} {
	set option	terse
	set args	[lreplace $args 0 0]
    } elseif {($firstArg == "-shrink")} {
	set option	shrink
	set args	[lreplace $args 0 0]
    } else {
	set option	verbose
    }

    set args [vxprj::fileFind vxprj::help $args]

    set prjFile		[lindex $args 0]

    cmpProjOpen	$prjFile
    ::autoscale $option
    cmpProjClose
}


# ---------------------------- private procedures ---------------------------- #

################################################################################
#
# vxprj::componentProblemReport - report a component configuration problem
#
# SYNOPSIS
# \cs
#   vxprj::componentProblemReport <name> <reason>
# \ce
#
# DESCRIPTION
# This procedure prints a project configuration problem to the standard output.
#
# PARAMETERS:
#   name : name of the problem which has a problem
#   reason : reason of the problem
#
# RETURNS: N/A
#
# ERRORS: N/A
#
# NOMANUAL
#

proc vxprj::componentProblemReport {name reason} {

    set str	[format "%s : %s" $name $reason]

    if {[string length $str] > 80} {

	set str	[format "%s :" $name]
	set line	""

	foreach word $reason {

	    set wordLength	[string length $word]
	    set lineLength	[string length $line]

	    if {$line == ""} {
		set line	[format "    %s" $word]
	    } else {
		if {[expr {$lineLength} + {$wordLength} + 1] > 80} {
		    set str		[format "%s\n%s" $str $line]
		    set line	[format "    %s" $word]
		} else {
		    append line " " $word
		}
	    }
	}

	set str		[format "%s\n%s" $str $line]
    }

    puts stdout $str
}

################################################################################
#
# vxprj::askForHelp - test if help is asked by <argv>
#
# SYNOPSIS
# \cs
#   vxprj::askForHelp <argv>
# \ce
#
# DESCRIPTION
# This procedure looks in <argv> for an argument exactly equal to either :
# \is
# \i '-h'
# \i '-help'
# \i '--h'
# \i '--help'
# \i 'help'
# \ie
#
# RETURNS: '1' if one of the above test is true, '0' else.
#
# NOMANUAL
#

proc vxprj::askForHelp {argv} {

    if {([lsearch -exact $argv "-h"] != -1) ||
	([lsearch -exact $argv "-help"] != -1) ||
	([lsearch -exact $argv "--h"] != -1) ||
	([lsearch -exact $argv "--help"] != -1) ||
	([lsearch -exact $argv "help"] != -1)} {

	return 1
    }

    return 0
}

proc vxprj::isSupportedRequest {request} {

    variable mSupportedRequests		;# list of vxprj supported requests

    if {[lsearch -exact $mSupportedRequests $request] == -1} {

	return 0
    }

    return 1
}

# ----------------------------------- main ----------------------------------- #

# if no argument is given, or the magic keywords (-h, -help, --h or --help)
# appear in the arguments list, call for help

if {($argc == 0) || ([vxprj::askForHelp [lindex $argv 0]])} {

    vxprj::help

} elseif {(! [vxprj::isSupportedRequest [lindex $argv 0]])} {

    puts stdout "Unsupported request \"[lindex $argv 0]\""
    vxprj::help

} else {
    
    if {[catch {package require OsConfig 1.0} status]} {

	set oldPath	[pwd]
	set osconfigDir	[file join [Wind::base] host resource osconfig tcl]
	set projectDir	[file join [Wind::base] host resource tcl app-config \
				   Project]

	# check if there is at least the osconfig directory

	if {[file exists $osconfigDir]} {

	    cd $osconfigDir
	    source cmpScriptLib.tcl

	} elseif {[file exists $projectDir]} {

	    cd $projectDir
	    source cmpScriptLib.tcl

	} else {
	    error "Could not find cmpScriptLib.tcl"
	}

	# go back to previous path

	cd $oldPath
    }

    # some of the project procedure require the prjFile argument. To ease user's
    # point view, if there is no project file given as an argument, tro to find
    # one in the current directory

    set firstArg	[lindex $argv 0]

    switch -- $firstArg {

	autoscale {

	    set args	[lreplace $argv 0 0]
	    if {[catch {eval vxprj::autoscale $args} autoscaleRes]} {
		puts stdout "Autoscale operation failed : $autoscaleRes"
		exit -1
	    }
	}

	bundle {

	    set args	[lreplace $argv 0 0]
	    eval vxprj::bundle $args
	}

	build {

	    set args	[lreplace $argv 0 0]
	    eval vxprj::build $args
	}

	buildmacro {

	    set args	[lreplace $argv 0 0]
	    eval vxprj::buildmacro $args
	}

	buildrule {

	    set args	[lreplace $argv 0 0]
	    eval vxprj::buildrule $args
	}

	component {

	    set args	[lreplace $argv 0 0]
	    eval vxprj::component $args
	}

	copy {

	    set args	[lreplace $argv 0 0]
	    if {[catch {eval vxprj::copy $args} status]} {

		puts stderr "Exception while copying project : \"$status\""
		exit -1
	    }
	}

	create {

	    set args	[lreplace $argv 0 0]
	    if {[catch {eval vxprj::create $args} status]} {

		puts stderr "Exception while creating project : \"$status\""
		exit -1
	    }
	}

	delete {

	    set args	[lreplace $argv 0 0]

	    if {[llength $args] == 1} {
		eval vxprj::delete $args
	    } else {
		if {[llength $args] > 1} {
		    puts stdout "Unknown parameters \"[lrange $args 0 end]\""
		} else {
		    puts stdout "Please provide with a project file path"
		}
		vxprj::help
	    }
	}

	diff {

	    set args	[lreplace $argv 0 0]
	    set diffs	[eval vxprj::diff $args]

	    puts stdout "\nParameters/Components unique to\
			 [lindex [lindex $diffs 0] 0] :"

	    foreach elt [lindex [lindex $diffs 0] 1] {
		puts stdout $elt
	    }

	    puts stdout "\nParameters/Components unique to\
			 [lindex [lindex $diffs 1] 0] :"

	    foreach elt [lindex [lindex $diffs 1] 1] {
		puts stdout $elt
	    }
	}

	famous {
	    set args	[lreplace $argv 0 0]
	    eval vxprj::famous $args
	}

	file {

	    set args	[lreplace $argv 0 0]
	    if {[catch {eval vxprj::_file $args} status]} {

		puts stdout "File operation failed : $status"
		exit -1
	    }
	}

	fsfile {

	    set args	[lreplace $argv 0 0]
	    if {[catch {eval vxprj::fsfile $args} status]} {

		puts stdout "File System operation failed : $status"
		exit -1
	    }
	}

	makefile {
	    set args	[lreplace $argv 0 0]
	    if {[catch {eval vxprj::makefile $args} makefileName]} {

		puts stdout "Project Makefile operation failed : $makefileName"
		exit -1
	    } else {
		puts stdout $makefileName
	    }
	}

	module {

	    set args	[lreplace $argv 0 0]
	    if {[catch {eval vxprj::module $args} status]} {

		puts stdout "Module operation failed : $status"
		exit -1
	    }
	}

	parameter {

	    set args	[lreplace $argv 0 0]
	    eval vxprj::parameter $args
	}

	profile {

	    set args	[lreplace $argv 0 0]
	    if {[catch {eval vxprj::profile $args} status]} {
		puts stdout "Profile operation failed : $status"
		exit -1
	    }
	}
       
	tccheck {

	    if {[catch {vxprj::tccheck} status]} {
		puts stdout "Toolchain info files check failed : $status"
		exit -1
	    }
	}

	default {
	    puts stderr "*** ERROR ***"
	    puts stderr "Unknown command '$firstArg' ... Exiting\n"
	    vxprj::help
	}
    }
}
