# Utils.tcl - handy, global utilities used by the config tool
#
# Copyright 1998-2004 Wind River Systems, Inc.
#
# modification history
# --------------------
# 02x,28sep05,pee  removed WIND_PROJ_BASE substitution
# 02w,21apr05,tpw  Let the OS resolve symlinks for us after all.
# 02v,10mar05,pee  SPR# 106605: removed /./ from path, disabled parent dir substitution
# 02u,23dec04,c_c  removed references to foundation.
# 02t,21oct04,fle  Doc fixes
# 02s,01sep04,fle  Use of Wind::foundation where needed
# 02r,15may04,fle  SPR#98254 : made relPath also substitute the WIND_HOME,
#                  WIND_GNU_PATH and WIND_DIAB_PATH environment variables
# 02q,06may04,fle  Removed all bullet lists refgen markups
# 02p,19apr04,fle  doc : updated PrjUtil::relPath help
#                  + made outOfDat follow symbolic links
#                  + made PrjUtils::relPath substitute for TGT_DIR and PRJ_DIR
#                    when an <oldPrjDir> is provided
# 02o,07apr04,fle  Added CC_BIN_DIR substitution to PrjUtils::relPath
# 02n,31mar04,fle  Modified tcInfoFileGet since platform is not needed in the
#                  name anymore
# 02m,16feb04,fle  Created PrjUtil package
# 02l,29dec03,fle  Rather use Wind::tgtDir when possible
#                  + removed UiTclSh oriented code
# 02k,02dec03,p_b  moved isXmlFile to Utils
# 02j,26nov03,fle  Added tcInfoFileGet procedure
# 02i,13nov03,fle  made outOfDate return false if the file cannot be found
# 02h,19jun03,fle  Updated to new location
# 02g,05jun03,fle  Added shareDllLoad from Tornado AE init.tcl
# 02f,18jun02,rbl  adding printStack function
# 02e,28may02,rbl  randomize file name in tempFileGet to support parallel
#                  builds
# 02d,22may02,cjs  Move filenameQuoteIfNecessary() to Utils.tcl
# 02c,22may02,cjs  Use Wind::tmpDirGet() in tempFileGet()
# 02b,15may02,cjs  Fix spr 73941: problems running in restricted mode on
#                  Windows XP by making tempFileGet() smarter
# 02a,14may02,cjs  Fix spr 75899: allow user to type in a read-only dependency
#                  path; defer terrible directory search dialog
# 01z,02apr02,rbl  add more characters to the list of unacceptable file name
#                  characters. Also add new textControlTextSetKeepingSelection
#                  function to set text in an edit control while retaining
#                  the cursor position or selection
# 01y,11dec01,rbl  fix buildRuleSplit to cope with macros in target or
#                  dependency names
# 01x,13nov01,rbl  merge project file/dir manipulation routines from T3
# 01w,24oct01,cjs  Add extra param, support for files, to
#                  removeDuplicatesFromList; merge T3's filePathEatDots()
# 01v,23oct01,cjs  Adding tempFileGet(), a routine to get a reusable temporary
#                  file
# 01u,17sep01,rbl  merge from T3: (1) add outOfDate functionality, needed for
#                  auto-generation of tcInfo_<tool_family> files
#                  (2) add removeTrailingSlash function, needed by prjRelPath
#                  (3) fix regsubPath so that it does case insensitive
#                  match on Windows, and copes with back slashes in
#                  match pattern (4) add removeDuplicatesFromList function
# 01t,19mar99,rbl  adding a barePathSubstitute function that does a "shallow" 
#                  substitution on paths, and regsubPath which does a case
#                  insensitive 
#                  regsub for windows and a case sensitive regsub for UNIX.
# 01s,11mar99,cjs  Add support for build rules with *either* blank rules or
#                  dependencies in buildRuleSplit/Join()
# 01r,05mar99,ren  changing unix GetWtxLoadOptions to use gui
# 01q,04mar99,ren  Changed behavior of GetWtxLoadOptions on windows
# 01p,03mar99,ren  Adding rename dialog.
# 01o,02mar99,ren  Adding file tails proc
# 01n,01mar99,ren  Exposing download options to GUI
# 01m,24feb99,ren  Added GetWtxObjModuleLoadOptions
# 01l,18feb99,ren  Added EscapeRegexpCharacters
# 01k,15jan99,cjs  Add routine for validating files (no bad chars)
# 01j,06nov98,cjs  Protect filePathFix, filePathUnFix from bad argument 
# 01i,29oct98,cjs  Add an optional sequence of prjRelPath/prjFullPath calls
#                  to filePathFix() 
# 01h,22oct98,cjs  added filePathUnFix(); removed DOSify(); moved mTimeGet()
#                  here from file.psheet.tcl
# 01g,15sep98,cjs  added preferredPageSort() to sort tab sheets against
#                  a desired order
# 01g,02aug98,cjs  added buildRuleSplit/Join() 
# 01f,14aug98,cjs  added pathTruncate() 
# 01e,23jun98,cjs  changed  nameExtract, uniqueNameCreate to work with
#                  more args
# 01d,01may98,cjs  change filePathFix() to unix-ify files; add DOSify() 
# 01c,16apr89,cjs  remove wtxPath() implementation (wtxtcl.dll is now loaded).
# 01b,30mar98,cjs  documentation update.
# 01a,27mar98,cjs  written.
#
# DESCRIPTION
# This module contains handy utilites for parsing lists, identifying flags in
# arg lists, implementing inheritance between namespaces, and so forth.
#
# It also implements the namespace Object, which is the root of all the config
# tool friendly objects.
#

package provide PrjUtils 1.0

# Needed to use utils in the Wind package, like tmpDirGet()

package require Wind 1.1

namespace eval PrjUtils {

    namespace export archDirFind barePathSubstitute defaultDir relPath srcDir

    variable mSrcDir		""	;# WIND_SOURCE_BASE if it exists
    variable mSrcDirNotSet	1	;# test if source dir is set
    variable mDefaultDir	""	;# default project directory

    variable mReg1		""	;# project directory regular expression
    variable mReg2		""	;# parent prj dir regular expression
    variable mReg3		""	;# source base regular expression
    variable mReg4		""	;# default dir regular expression
    variable mReg5		""	;# wind base regular expression
    variable mReg6		""	;# wind host type regular expression
    variable mReg7		""	;# old prj path regular expression
    variable mReg8		""	;# CC_BIN_DIR path regular expression
    variable mReg9		""	;# CC_HOST regular expression
    variable mReg10		""	;# old prj path to substitute PRJ_DIR
    variable mReg11		""	;# target directory regular expression
    variable mReg12		""	;# wind home regular expression
    variable mReg13		""	;# wind diab path regular expression
    variable mReg14		""	;# wind gnu path regular expression

    variable mSubst1		""	;# new prj path substitution expression

    variable mPrjDir		""	;# project directory cache
    variable mOldPath		""	;# old project dir path
    variable mNewPath		""	;# new project dir path
    variable mOldPrjDir		""	;# previous prj directory
}

# Check to see if file name contains invalid chars

proc ::badFileCharsGet {} {
    return "\[|*<>;\${}\() \r\n\t]|\\\[|\\\]" 
}

################################################################################
#
# tcInfoFileGet - get the tcInfo file path for given <family>
#
# SYNOPSIS
# \cs
#   tcInfoFileGet <family>
# \ce
#
# DESCRIPTION
# Get the name of the file which holds the toolchain information for given
# <family> toolchain.
#
# This file resides in the WIND_BASE/host/resources/osconfig/tcl directory, and
# is named tcInfo_<family>.tcl
#
# PARAMETERS:
#   family : toolchain family to get inf file path for
#
# RETURNS: Full path to the toolchain info file
#
# EXAMPLES:
# Get the tcInfo file path for diab family
# \cs
# % package require Wind
# 1.1
# % cd [Wind::path host resource osconfig tcl]
# % source Utils.tcl
# % tcInfoFileGet diab
# c:/view/fredl.b6/wind/river/host/resource/osconfig/tcl/tcInfo_diab.tcl
# %
# \ce
#

proc tcInfoFileGet {family} {

    set fileName	[format "tcInfo_%s.tcl" $family]

    return [file join [Wind::base] host resource osconfig tcl $fileName]
}

################################################################################
#
# regsubPath - case insensitive regsub on windows, and case sensitive on Unixes
#
# SYNOPSIS
# \cs
#   regsubPath   args
# \ce
#
# DESCRIPTION
# For windows, we use -nocase AND lower the case of the match expression.
#
# We also replace all /'s in the match expression with [\\/] (match forward or
# backslash)
#
# PARAMETERS
#   args: same arguments as for the tcl regsub function.
#
# RETURNS: output of regsub function.
#
# ERRORS: N/A
#

proc ::regsubPath {args} {

    # first, lower the case of the match expression
    # so find the match expression, skipping past options
    set i 0
    set isArgOption 1
    while { $i < [llength $args] && $isArgOption} {
	set strArg [lindex $args $i]
	if { $strArg == "--" } {
	    incr i
	    set isArgOption 0
	} elseif { [string match "-*" $strArg] } {
	    incr i
	} else {
	    set isArgOption 0
	}
    }

    # now lower the case of the match expression and match backslashes as well
    # as forward slashes

    global tcl_platform

    if { $tcl_platform(platform) == "windows" } {
	set loweredMatchPattern [string tolower [lindex $args $i]]
	regsub -all {/} $loweredMatchPattern {[\\\/]} loweredMatchPattern
	set args [lreplace $args $i $i $loweredMatchPattern]
	return [uplevel regsub -nocase $args]
    } else {
	return [uplevel regsub $args]
    }
}

################################################################################
#
# printStack - prints the current call stack
#
# SYNOPSIS
# \cs
#   printStack
# \ce
#
# DESCRIPTION
# Borrowed from Ousterhout "Tcl and the Tk Toolkit", pg 131
#
# PARAMETERS: N/A
#
# RETURNS: N/A
#
# ERRORS: N/A
#

proc ::printStack {} {
    set level [info level]
    for {set i [expr {$level} - 1]} {$i >= 1} {incr i -1} {
	puts "Level $i: [info level $i]"
    }
}

################################################################################
#
# barePathSubstitute - does a <shallow> substitution of <strNewPath> for 
# <strOldPath> in <strOldString>.
#
# SYNOPSIS
# \cs
#   PrjUtils::barePathSubstitute <strOldString> <strOldPath> <strNewPath>
# \ce
#
# DESCRIPTION
# Only paths that go no deeper than ${strOldPath} (i.e. they end in 
# whitespace, a new line or a continuation character) are substituted.
# Trailing forward slashes are taken into account.
#
# PARAMETERS:
# strOldPath: the old path, e.g. c:/temp
# strNewPath: the new path, e.g. c:/newtemp
# strOldString: a string containing one or more directories, with forward
#            slash separators, and possibly <backslash> <newline> between
#
# EXAMPLES:
# If <strOldString> is set to :
# \cs
# c:/temp/fred \
# c:/temp/\
# e:/temp/bill
# \ce
#
# Then <strOldString> also contains the output, e.g. 
# \cs
# c:/temp/fred \
# c:/newtemp/\
# e:/temp/bill
# \ce
#
# Notice that c:/temp\ was substituted but c:/temp/fred \ was not. 
#            
# RETURNS: none
#
# ERRORS: N/A
#

proc PrjUtils::barePathSubstitute {strOldString strOldPath strNewPath} {

    variable mOldPath
    variable mNewPath
    variable mReg7
    variable mSubst1

    if {($strOldPath != $mOldPath) || ($strNewPath != $mNewPath)} {
	set mReg7	[ \
	    EscapeRegexpCharacters "${strOldPath}\(/\? |/\?\t|/\?\$|/\?\\\\\)" \
	]
	set mSubst1	[EscapeRegexpCharacters "${strNewPath}\\1"]

	if {[Wind::isWindows]} {
	    regsub -all -- {/} [string tolower $mReg7] {[\\\/]} mReg7
	}
    }

    upvar $strOldString str

    if {[Wind::isWindows]} {
	regsub -all -nocase -- $mReg7 $str $mSubst1 str
    } else {
	regsub -all -- $mReg7 $str $mSubst1 str
    }
}

proc ::isFileNameOk {file} {
    return [expr {![regexp [::badFileCharsGet] $file tmp]}]
}

# Substitue the system path delimiter for "/" in the path 

proc ::filePathFix {pathName {hProj ""}} {
    upvar $pathName path
    if {$path != ""} {
	set path [file join $path]
	if {$hProj != ""} {
	    set prjDir	[file dirname [prjInfoGet $hProj fileName]]
	    set path [PrjUtils::relPath $path $prjDir]
	    set path [::prjFullPath $hProj $path]
	}
    }
}

proc ::filePathUnFix {pathName} {
    upvar $pathName path
    if {$path != ""} {
	set path [file nativename $pat]
    }
}

# Truncate the beginning of a file path and replace missing
# portion w/ ".../" so that it fits in 'len' chars

proc ::pathTruncate {pathName len} {
    set bTruncated 0
    set retval $pathName
    while {[string length $retval] > [expr {$len} - 4]} {
	set split [file split $retval]
	set split [lrange $split 1 end]
	set retval [eval file join $split]
	set bTruncated 1
    }

    if {$bTruncated} {
	set retval [format ".../%s" $retval]
    }
    return $retval
}

proc mTimeGet {fileName} {
    if {[file exists $fileName]} {
	set retval [clock format [file mtime $fileName] \
			  -format "Last modified: %I:%M:%S %p %m/%d/%Y"]
    } else {
	set retval "(file not found)"
    }
}

proc ::car {lst} {
    if {[llength $lst] > 0} {
	return [lindex $lst 0]
    } else {
	return ""
    }
}

proc ::cdr {lst} {
    if {[llength $lst] > 0} {
	return [lrange $lst 1 end]
    } else {
	return ""
    }
}

proc ::flagFind {argz flag} {
    set idx [lsearch $argz $flag]
    if {$idx == -1} {
	return 0
    } else {
	return 1
    }
}

proc ::flagValueGet {argz flag} {
    set retval ""
    set idx [lsearch $argz $flag]
    if {$idx != -1} {
	if {[llength $argz] > $idx} {
	    set retval [lindex $argz [expr {$idx} + 1]]
	}
    }
    return $retval
}

proc ::ifaceValidate {namespace procList} {
    set status 1
    foreach proc $procList { 
	set cmd "info proc $proc"
	if {[namespace eval $namespace $cmd] == ""} {
	    set status 0
	    break
	}
    }
    return $status
}

proc ::inherit {from} {
    if {[catch {namespace parent $from}]} {
	error "attempt to inherit from non-existent namespace '$from'"
    } else {
	set to [uplevel {namespace current}]
	catch {namespace eval $to [set [format "%s::public" $from]]}
	catch {namespace eval $to [set [format "%s::protected" $from]]}
    }
}

proc ::preferredPageSort {preferredList realList} {
    set panes [lsort -increasing $realList]
    foreach preferred $preferredList {
	set idx [lsearch $panes $preferred]
	if {$idx != -1} {
	    lappend orderedPanes $preferred
	    set panes [lreplace $panes $idx $idx]
	}
    }
    if {[info exists orderedPanes]} {
	set orderedPanes [concat $orderedPanes $panes]
	set panes $orderedPanes
    }
    return $panes
}


##########################################################################
#
# buildRuleSplit - given a build rule:
#	{rule : deps} {rule1} {rule2} ...
# return {rule deps {rule1 rule2 ...}}
#
proc buildRuleSplit {ruleCommands} {
    set retval ""
    if {$ruleCommands != ""} {
	set targetLine [lindex $ruleCommands 0]
	set idx [lsearch $targetLine \:]
	if {$idx != -1} {
	    set targetName [join [lrange $targetLine 0 [expr {$idx} - 1]]]
	    set dependencies [join [lrange $targetLine [expr {$idx} + 1] end]]
	    regsub "^ *" $dependencies "" dependencies

	    set ruleCommands [lrange $ruleCommands 1 end]
	    set data ""
	    foreach line $ruleCommands {
		if {$data != ""} {
		    set data [format "%s\r\n%s" $data $line]
		} else {
		    set data $line
		}
	    }
	set retval [list $targetName $dependencies $data]
	}
    }
    return $retval
}

##########################################################################
#
# buildRuleJoin - join a list consisting of {ruleName deps rule1 rule2 ...}
# 	into a rule of the form {rule : deps} {rule1} {rule2} ...
#

proc buildRuleJoin {ruleName deps commands} {
    set ruleToAdd ""
    if {$ruleName != ""} {
	regsub -all "\r\n" $commands "\n" commands
	foreach line [split $commands \n] {
	    lappend ruleToAdd $line
	}

	# Get rule name
	scan $ruleName "%s" ruleName
	set dependencies [join $deps]
	set targetLine "$ruleName : $dependencies"
	set ruleToAdd [linsert $ruleToAdd 0 $targetLine]
    }
    return $ruleToAdd
}

namespace eval ::Utils {
    variable _uniqueIndex 0
}

################################################################################
#
# Object - Namespace to create low overhead objects
#
# DESCRIPTION
# The ::Object namespace implements the following routines for creating a low
# overhead <object> whose instance data is defined by a list, and whose class
# methods and class data are defined by a namespace.
#
# \cs
# ::Object::objectCreate <namespace> <objectDataList>
# ::Object::objEval <object> <proc> [args]
# \ce
#
# All objects (and therefore, the underlying namespace) are
# expected to implement the following functions:
#
# \cs
# objName {objData}
# objType {objData} 
# \ce
#
# Other methods implemented by the namespace can expect that
# the first argument is always the instance data, and any
# additional arguments are specific to the method.
#
# The following are all equally valid signatures for methods:
#
# \cs
# proc objType {data} {...}
# proc test1Arg {data arg1} {...}
# proc testnArgs {args} {...}
# \ce
#

namespace eval ::Object {

    #
    # evaluate <object> <proc> [args]
    #
    proc evaluate {handle method {argz ""}} {
	set namespace [lindex $handle 0]
	set objData [lindex $handle 1]

	if {$argz != ""} {
	    return [eval ${namespace}::$method [list $objData] $argz]
	} else {
	    return [eval ${namespace}::$method [list $objData]]
	}
    }

    proc create {namespace objData} {
	namespace eval $namespace {
	    if {([info procs Name] == "") || \
		([info procs Type] == "")} {

		set error "Error: namespace '[namespace current]' must"
		set error "$error support Name() and Type()"
		error $error
	    }
	}

	return [list $namespace $objData]
    }

    proc nameExtract {id} {
	if {[regexp {^(.+)@@(.+)@@(.+)$} $id dummy prefix prefix2 suffix]} {
	    return $suffix
	} elseif {[regexp {^(.+)@@(.+)$} $id dummy prefix suffix]} {
	    return $suffix
	} else {
	    error "nameExtract '$id' failed"
	}
    }

    proc uniqueNameCreate {args} {
	if {[llength $args] == 3} {
	    set prefix [lindex $args 0]
	    set prefix2 [lindex $args 1]
	    set suffix [lindex $args 2]
	    return [format "%s@@%s@@%s" $prefix $prefix2 $suffix]
	} elseif {[llength $args] == 2} {
	    set prefix [lindex $args 0]
	    set suffix [lindex $args 1]
	    return [format "%s@@%s" $prefix $suffix]
	} else {
	    error "uniqueNameCreate: bad args '$args'"
	}
    }

    # Object creation test program; invoke as ::Object::test

    proc test {} {
	namespace eval ::Object::test {
	    proc objName {data} { return [lindex $data 0] }
	    proc objType {data} { return [lindex $data 1] }
	    proc test1Arg {data arg1} {
		    puts "arg1 is $arg1"
	    }
	    proc testnArg {args} {
		puts "data is [lindex $args 0]; args is [lrange $args 1 end]"
	    }
	}
	set myObj [ \
	    ::Object::objectCreate ::Object::test \
				   {INCLUDE_WDB componentObject {foo bar} grub \
				    slickdit} \
	]
	puts [::Object::objEval $myObj objName]
	puts [::Object::objEval $myObj objType]
	puts [::Object::objEval $myObj test1Arg arg1]
	puts [::Object::objEval $myObj testnArg]
	puts [::Object::objEval $myObj testnArg arg1]
	puts [::Object::objEval $myObj testnArg arg1 arg2]
    }
}

namespace eval ::Null {
    proc Type {args} {
	return nullObject
    }

    proc Name {args} {
	return None
    }

    proc DisplayName {args} {
	return None
    }
}

proc NativeFileNames { listFileNames } {
    mapcar [list file nativename] $listFileNames
}

# the old lisp method, apply the proc to every element in the list
# return a list of the results

proc mapcar { proc list } {
    set listResult [list ]
    foreach element $list {
	lappend listResult [eval $proc [list $element]]
    }
    return $listResult
}

proc FileTails { listFileNames } {
    mapcar [list file tail] $listFileNames
}

################################################################################
#
# maxModTime - maximum modification time 
#
# SYNOPSIS 
# \cs
#   maxModTime  files 
# \ce
#
# DESCRIPTION
# Finds the maximum modification time of a set of files, using the Tcl function
# 'file mtime'
#
# PARAMETERS 
#   files:  list of files. Wildcards can be used, e.g. "hello.c *.cpp"
#
# RETURNS: the maximum modification time of files, using the Tcl function
# 'file mtime'
#
# ERRORS: files not found
#

proc ::maxModTime {files} {
    set files [eval glob $files]
    set nFiles [llength $files]
    set modTimes [lsort -decreasing -integer [mapcar [list file mtime] $files]]
    return [lindex $modTimes 0]
}

################################################################################
#
# minModTime - minimum modification time 
# 
# SYNOPSIS 
# \cs
#   minModTime  files 
# \ce
#
# DESCRIPTION
# Finds the minimum modification time of a set of files, using the Tcl function
# 'file mtime'
#
# PARAMETERS 
#   files:  list of files. Wildcards can be used, e.g. "hello.c *.cpp"
#
# RETURNS: the minimum modification time of files, using the Tcl function
# 'file mtime'
#
# ERRORS: files not found
#

proc ::minModTime {files} {
    set files [eval glob $files]
    set nFiles [llength $files]
    set modTimes [lsort -increasing -integer [mapcar [list file mtime] $files]]
    return [lindex $modTimes 0]
}

################################################################################
#
# outOfDate - tests if target files are out of date with dependency files
#
# SYNOPSIS 
# \cs
#   outOfDate <targets> <dependencies>
# \ce
#
# DESCRIPTION
# Checks wether the files given by <targets> are out of date with regard to the
# <dependencies> files.
#
# If any of the <dependencies> does not exist, it is removed from the list
# of <dependencies> to be check against. If there are no dependencies to check
# <targets> against, '0' is returned.
#
# PARAMETERS 
#   targets : set of target files. Wildcards can be used, e.g. "hello.o *.out"
#   dependencies : set of dependency files. Wildcards can be used, e.g.
#                  "hello.c *.cpp"   
#
# RETURNS: 1 if any target files are out of date with respect to any dependency
# files, or if any of the target files do not exist; 0 otherwise. 
#
# ERRORS: N / A
#

proc ::outOfDate {targets dependencies} {

    set targets [eval glob -nocomplain $targets]
    set dependencies [eval glob -nocomplain $dependencies]

    # check if there are dependencies and targets to look at

    if {[llength $targets] == 0} {
	return 1
    }

    if {[llength $dependencies] == 0} {
	return 0
    }

    # check that each of the targets and dependencies do exist

    set tmpTargets	$targets
    set targets		{}

    foreach target $tmpTargets {
	if {![file exists $target]} {
	    return 1
	} else {
# This code should be unnecessary, and it fails on some long relative symlinks
# when the path to the link also contains symlinks which make it appear to be
# lower or higher in the directory tree than it actually is. These names are
# eventually passed to "file mtime" which should resolve symlinks correctly.
#
#	    while {([file type $target] == "link") && ([file exists $target])} {
#		set target [file join [file dirname $target] \
#				      [file readlink $target]]
#	    }
	    lappend targets $target
	}
    }

    set existingDeps	{}

    foreach dep $dependencies {
	if {[file exists $dep]} {
# This code should be unnecessary, and it fails on some long relative symlinks
# when the path to the link also contains symlinks which make it appear to be
# lower or higher in the directory tree than it actually is. These names are
# eventually passed to "file mtime" which should resolve symlinks correctly.
#
#
#	    while {([file type $dep] == "link") && ([file exists $dep])} {
#		set dep [file join [file dirname $dep] [file readlink $dep]]
#	    }
	    lappend existingDeps $dep
	}
    }

    if {[llength $existingDeps] == 0} {
	return 0
    }

    # both target and dependencies files exist, let's check for their
    # modification time

    if {[maxModTime $existingDeps] > [minModTime $targets]} {
	return 1
    }

    return 0
}

#########################################################################
#
# removeTrailingSlash - remove a trailing slash from a directory
#
# PARAMETERS 
#   dirName: name of the variable holding the directory
#
# RETURNS: N / A
#
# ERRORS: N / A
#
proc removeTrailingSlash {dirName} {
    upvar $dirName dir
    set lastCharIndex [expr {[string length $dir]} - 1]
    set lastChar [string index $dir $lastCharIndex]
    if {$lastChar == "/" || $lastChar == "\\"} {
	set dir [string range $dir 0 [expr {$lastCharIndex} - 1]]
    }
}

#########################################################################
#
# removeDuplicatesFromList - remove duplicate items from a list
#
# PARAMETERS 
#   nameOfList:   name of list to remove item from
#   bElementsAreFiles: 1, if elements should be treated as file paths
#
# RETURNS: 1 if items were removed, 0 if no duplicates found
#
# ERRORS: N / A

proc removeDuplicatesFromList {nameOfList {bElementsAreFiles 0}} {

    upvar $nameOfList pathList

    # Make Tcl hash list to avoid duplicates
    # Along the way,
    # a) flatten all relative paths
    # b) make sure hashing is case insensitive for Windows
    foreach path $pathList {
	if {$bElementsAreFiles == 1} {
	    set path [filePathEatDots $path]
	}

	if {$bElementsAreFiles == 1} {
	    if {[Wind::hostType] == "x86-win32"} {
		set paths([string tolower $path]) $path
	    } else {
		set paths($path) $path
	    }
	} else {
	    set paths($path) $path
	}
    }

    if {$bElementsAreFiles == 1} {
	# If Windows, recover originals from hashed list
	if {[Wind::hostType] == "x86-win32"} {
	    foreach name [array names paths] {
		lappend returnList $paths($name)
	    }
	    set pathList $returnList
	} else {
	    set pathList [array names paths]
	}
    } else {
	set pathList [array names paths]
    }
}

################################################################################
#
# filePathToAbsolute - convert a relative path to an absolute path
#
# SYNOPSIS
# \cs
#   filePathToAbsolute  file
# \ce
#
# DESCRIPTION
# Converts a relative path to an absolute path, 
# prepending the current working directory and unix-ifying the path if
# necessary.
# Any ../../'s are eaten up, the path is unixified (all slashes converted
# to forward slashes), and variables (e.g. $(WIND_BASE)) are substituted for
# using prjFullPath.
#
# PARAMETERS
#   file:   relative file path.
#   currentDirectory:  (optional) directory which is prepended to any relative
#                      path. If not specified, [pwd] is used.
#
# RETURNS: N/A
#
# ERRORS: N/A
#

proc ::filePathToAbsolute {file {currentDirectory ""}} {

    # get pwd if necessary

    if {$currentDirectory == ""} {
	set currentDirectory [pwd]
    } else {
	set currentDirectory [toPrjDirAbsolute $currentDirectory]
    }

    # trim whitespace from path
    set file [string trim $file]

    # unix-ify path
    ::filePathFix file

    # do substitutions, e.g. $(WIND_BASE), $(PRJ_DIR)...
    set file [prjFullPathRelativeTo "" $file $currentDirectory]

    # prepend pwd to a relative file path name (any path not beginning with
    # "/" or "<drive letter>:"

    if {![regexp {^/} $file] && ![regexp {^[a-zA-Z]:/} $file]} {
	set file $currentDirectory/$file
    }

    return [filePathEatDots $file]
}

################################################################################
#
# filePathEatDots - removes ..'s and .'s from file path
#
# SYNOPSIS 
#   filePathEatDots  file 
#
# PARAMETERS 
#   file: file path, e.g. ./joe/myDir/../fred
#
# RETURNS: path with dots eaten, e.g. joe/fred
#
# ERRORS: N / A
#

proc filePathEatDots {file} {

    # split path into elements (stuff between slashes) and eat up ..'s and .'s

    set pathElements [split $file /]
    set i 1

    while {$i < [llength $pathElements]} {

	# if we find a "..", eat it and the element to the left of it

	if {[lindex $pathElements $i] == ".."} {
	    set pathElements [lreplace $pathElements [expr {$i} - 1] $i]
	    incr i -1
	} elseif {[lindex $pathElements $i] == "."} {
	    # if we find a ".", eat it
	    set pathElements [lreplace $pathElements $i $i]
	} else {
	    incr i
	}
    }

    return [join $pathElements /]
}

################################################################################
#
# toPrjDirAbsolute - convert to an absolute project dir path
#
# SYNOPSIS
# \cs
#   toPrjDirAbsolute <prjDir>
# \ce
#
# DESCRIPTION
# Converts a project name, project file name or project directory to a project
# directory
#
# Takes as input:
# \ml
# \m -
# a project name
# \m -
# a project file name
# \m -
# a relative or absolute path to a directory
# \me
# and outputs an absolute path to a directory. If a name is specified, the
# directory is under [PrjUtils::defaultDir].
#
# SYNOPSIS
#   toPrjDirAbsolute prjDir
#
# PARAMETERS
#   prjDir: a project name, project file name or project directory
#
# RETURNS: absolute path to the directory corresponding to prjDir
#
# ERRORS: N/A
#

proc ::toPrjDirAbsolute {prjDir} {

    # Don't do this to workspace files, which can live in arbitrary directories

    if {[string match [file extension $prjDir] ".wsp"]} {
	return $prjDir
    }

    # substitute [pwd] for "."

    if {$prjDir == "."} {
	return [pwd]
    }

    # if we have a name without a path, prepend [PrjUtils::defaultDir] to the
    # name

    if {[file tail $prjDir] == $prjDir} {

	# strip off any extension
	#set prjDir [file rootname $prjDir]
	return [file join [PrjUtils::defaultDir] $prjDir]
    }

    # if we have a file with .wpj extension, return the directory

    if {[string match [file extension $prjDir] ".wpj"]} {
	set prjDir [file rootname $prjDir]
	set prjDir [file dirname $prjDir]
	return [filePathToAbsolute $prjDir]
    }

    # default: convert relative paths to absolute paths

    return [filePathToAbsolute $prjDir]
}

################################################################################
#
# EscapeRegexpCharaters - make regexp chars safe from immediate evaluation
#
# SYNOPSIS
# \cs
#   EscapeRegexpCharacters <str>
# \ce
#
# DESCRIPTION
# Make regular expression chars safe from immediate evaluation
#
# PARAMETERS:
#   str: any string
#
# RETURNS: takes the input string and escapes any characters with special
# meaning in regular expressions
#

proc EscapeRegexpCharacters { str } {
    regsub -all {[\?|^$.*+()]|\[|\]} $str {\\\0} str
    set str
}

################################################################################
#
# tempFileGet - Get a name for a temp file
#
# SYNOPSIS
# \cs
# \ce
#
# DESCRIPTION
# To get the filename we use the TCL rand() function, which returns a random
# number from a seed based on the internal clock of the machine at the time
# rand() is first called.
#
# The seed can be reset by calling srand(). 
# 
# For the directory we use [Wind::tmpDirGet]. We check that the file does not
# currently exist. 
#
# The client is responsible for creating the file and cleaning it up when done
# with it. 
#
# RETURNS: name of a temp file
#
# EXAMPLES:
# On Windows this might be:
#
# \cs
# C:/Documents and Settings/kiwi/Local Settings/Application Data/Wind River 
# Systems/Tornado/2.2/435269992536Tclprocess_output.tmp
# \ce
#
# NB: this filename may (on Windows probably will) contain spaces. If you
# intend to use this with an {eval exec}, be sure to wrap the result in
# 'filenameQuoteIfNecessary'.
#
# ERRORS:
# \is
# \i Could not find an unused temp file name after 100 tries
# \ie
#

proc tempFileGet {} {
    set maxcount 100
    set count 0
    set file ""
    while {$file == "" && $count < $maxcount} {
	incr count
	set file [expr rand()]
	# remove decimal place
	regsub \\. $file {} file
	append file Tclprocess_output.tmp
	set file [file join [Wind::tmpDirGet] $file]
	if {[file exists $file]} {
	    set file ""
	}
    }
    if {$count == [expr {$maxcount}]} {
	error "Could not find a free temp file in [Wind::tmpDirGet]"
    }
    return $file
}

################################################################################
#
# bspsEnumerate - get the list of BSPs found in <wind_base>/target/config 
#
# SYNOPSIS
# \cs
#   bspsEnumerate
# \ce
# DESCRIPTION
# BSPs are directories found in this path containing the file config.h
#
# RETURNS: list of fully qualified BSP directory paths
#
# ERRORS N/A
#

proc bspsEnumerate {} {
    ::beginWaitCursor
    ::prjStatusLog "Reading directory.  Please wait..." 0 20 

    set dirs [glob -nocomplain [file join [Wind::tgtDir] config *]]
    ::prjStatusLog "Looking for BSPs.  Please wait..." 20 20

    set count [llength $dirs]
    set idx 0
    foreach dir $dirs {
	set name [file tail $dir]
	if {[file isdirectory $dir]} {
	    if {[file exists [file join $dir config.h]]} {
		::prjStatusLog "Found '$name'..." $idx $count
		lappend bspList $dir
	    } else {
		::prjStatusLog "Checking '$name'.  Please wait..." $idx $count
	    }
	} else {
	    ::prjStatusLog "Checking '$name'.  Please wait..." $idx $count
	}
	incr idx
    }
    if {![info exists bspList]} {
	set bspList ""
    }

    ::prjStatusLog ""
    ::endWaitCursor

    return $bspList
}

################################################################################
#
# projectsEnumerate - get the list of projects found in
#     <wind_base>/target/proj matching the type specified by prjNS
#
# SYNOPSIS
#    projectsEnumerate
#
# RETURNS: list of projects of type prjNS, as fully qualified .wpj file paths
#
# ERRORS: N/A
#

proc projectsEnumerate {prjNS} {

    ::beginWaitCursor
    ::prjStatusLog "Reading directory.  Please wait..." 0 20 
    set dirs [glob -nocomplain [file join [::PrjUtils::defaultDir] *]]
    set count [llength $dirs]
    ::prjStatusLog "Looking for projects.  Please wait..." 0 $count 

    set idx 0
    foreach dir $dirs {
	if {[file isdirectory $dir]} {
	    set prjs [glob -nocomplain [file join $dir \*.wpj]]
	    if {$prjs != ""} { 
		foreach prj $prjs {
		    set name [file tail $prj]
		    ::prjStatusLog "Found '$name'..." $idx $count
		    lappend prjList $prj
		}
	    } else {
		::prjStatusLog "Checking '$dir'.  Please wait..." $idx $count
	    }
	}
	incr idx
    }
    if {![info exists prjList]} {
	set prjList ""
    }

    ::prjStatusLog ""
    ::endWaitCursor

    return $prjList
}

################################################################################
#
# PrjUtils::defaultDir - default directory for creating new projects.
#
# This fetches the environment variable WIND_PROJ_BASE if defined, or returns
# $WIND_BASE/target/proj otherwise. It returns a UNIX-style path, i.e. with 
# forward slashes. 
#
# SYNOPSIS 
#   PrjUtils::defaultDir   
#
# RETURNS: N / A
#
# ERRORS: N / A
#

proc PrjUtils::defaultDir {} {

    variable mDefaultDir

    if {$mDefaultDir == ""} {
	if {[info exist ::env(WIND_PROJ_BASE)]} {
	    set mDefaultDir [file join $::env(WIND_PROJ_BASE)]
	} else {
	    set cwd [pwd]
	    cd [file join [Wind::prjDir]]
	    set mDefaultDir [file join [pwd]]
	    cd $cwd
	}
    }

    return $mDefaultDir
}

################################################################################
#
# filenameQuoteIfNecessary - quotes a filename if it has spaces
#
# SYNOPSIS
# \cs
#   filenameQuoteIfNecessary  filename
# \ce
#
# DESCRIPTION
# TCL file commands (e.g. file, open) can handle filenames with spaces.
# But when executing a shell command using exec, such filenames need to
# be quoted. So a typical usage is
#
# \cs
#   eval exec $command 2> [filenameQuoteIfNecessary $errorFile]
# \ce
#
# PARAMETERS
#   filename: file path, with forward slashes, e.g. c:/temp, c:/te mp
#
# RETURNS: path quoted if necessary, e.g. c:/temp, "c:/te mp"
#
# ERRORS: N / A
#
# EXAMPLES
# \cs
#   eval exec $command 2> [filenameQuoteIfNecessary $errorFile]
# \ce

proc filenameQuoteIfNecessary {filename} {
    if {[string first " " $filename] != -1} {
	set filename \"$filename\"
    }
    return $filename
}

#############################################################################
#
# shareDllLoad - load shared library if the library is not already loaded.
#
# SYNOPSIS
# \cs
#   shareDllLoad  <args>
# \ce
#
# DESCRIPTION
# Load shared library if the library is not already loaded
#
# PARAMETERS:
#   args : shared libs to load
#
# RETURNS: N/A
#
# ERRORS: N/A
#
# NOMANUAL
#

proc shareDllLoad {args} {

    # first create a table of the already loded Dlls to avoid loading the Dll
    # again

    foreach recPair [info loaded] {
	set loaded([file rootname [file tail [lindex $recPair 0]]]) "loaded"
    }

    # try to find the Dll extension, depending on the host type, and debug mode

    set ext	""
    set libDir lib

    if {[Wind::hostType] == "x86-win32"} {
	set libDir bin
	if {![catch {uitclinfo debugmode} debugMode]} {
	    if {$debugMode} {
		set ext "-d"
	    }
	}
    }

    append ext [info sharedlibextension]

    foreach lib $args {
	if {![info exists loaded($lib)]} {

	    load [file join [Wind::base] host [Wind::hostType] $libDir \
			    $lib$ext]
	}
    }
}

################################################################################
#
# isXmlFile - test if <prjFile> is in Wind River Xml format
#
# SYNOPSIS
# \cs
#   isXmlFile <prjFile>
# \ce
#
# DESCRIPTION
# This procedure looks for the <wrxml> tag within given <prjFile>. If it finds
# it, then '1' is returned, else, '0' is returned.
#
# PARAMETERS:
#   prjFile : file to be parsed for Xml format test
#
# RETURNS: '1' if <prjFile> has been found to be in Xml format, '0' else.
#
# SEE ALSO: prjSave
#
# NOMANUAL
#

proc ::isXmlFile {prjFile} {

    set formatIsXml     0

    if {! [catch {open $prjFile r} fileId]} {

	# read the file content, looking for the <wrxml> tag

	set fileContent [read $fileId]

	if {[string first "<wrxml>" $fileContent] != -1} {
	    set formatIsXml     1
	}

	# close the file

	catch {::close $fileId}
    }

    return $formatIsXml
}

################################################################################
#
# PrjUtils::relPath - converts paths relative to Project directory
#
# SYNOPSIS
# \cs
#   PrjUtils::relPath <files> [<prjDir> [<oldPrjDir> [<tag>]]]
# \ce
#
# DESCRIPTION
# Subsitutes all occurrences of project directory, and wind base with its
# associated Make macro ($(WIND_BASE) or $(WIND_PROJ_BASE))
#
# PARAMETERS:
#   files : list of files to subsitute WIND_BASE and WIND_PROJ_BASE for
#   prjDir : when some of the substitued <files> require to also substitue
#            PRJ_DIR, substitutions will be made on match of <prjDir>
#   oldPrjDir : <substitutes> matches of <oldPrjDir> with PRJ_DIR. A special
#               case is to also substitute value of <tag>, only if <tag> is set
#               to WCC__CDF_PATH
#   tag : if <tag> is set to WCC__CDF_PATH, subsitutions of <oldPrjDir> and
#         <prjDir> will occur
#
# RETURNS: N/A
#

proc PrjUtils::relPath {files {prjDir ""} {oldPrjDir ""} {tag ""}} {

    variable mReg1
    variable mReg2
    variable mReg3
    variable mReg4
    variable mReg5
    variable mReg6
    variable mReg8
    variable mReg9
    variable mReg10
    variable mReg11
    variable mReg12
    variable mReg13
    variable mReg14
    variable mPrjDir
    variable mOldPrjDir

    set srcBase		[PrjUtils::srcDir]

    # remove /./ from all paths.
    # this avoids problems when prjDir is .
    # a better solution would be to properly anchor all regular expressions,
    # but it is not possible yet because flags (-I<path>)
    # and tags (FILE_<path>_) are passed.
    # Until this is changed, patterns cannot be anchored.
    # in the situation where a directory name ends with a . and
    # prjDir is ., this function may produce bogus output.
    regsub -all {/(\./)*} $files {/} files
    regsub -all {/(\./)*} $prjDir {/} prjDir


    # handle special cases first

    if {($oldPrjDir != "")} {

	# -I include paths: -I{$oldPrjDir} is mapped to -I{$newPrjDir}, but
	# paths deeper than this are left pointing to $oldPrjDir. So we match
	# -I{$oldPrjDir} followed by a possible "/", then whitespace or "\"
	# continuation or end of string

	if {$oldPrjDir != $mOldPrjDir} {
	    PrjUtils::barePathSubstitute files "-I${oldPrjDir}" "-I${prjDir}"
	    set mOldPrjDir	$oldPrjDir
	    set mReg10 "[EscapeRegexpCharacters ${mOldPrjDir}](/| |\t|\$|\n)"
	    if {[Wind::isWindows]} {
		regsub -all -- {/} [string tolower $mReg10] {[\\\/]} mReg10
	    }
	}

	# WCC__CDF_PATH paths: {$oldPrjDir} is mapped to {$newPrjDir}, but paths
	# deeper than this are left pointing to $oldPrjDir. So we match
	# {$oldPrjDir} followed by a possible "/", then whitespace or "\"
	# continuation or end of string

	if {$tag == "WCC__CDF_PATH"} {
	    PrjUtils::barePathSubstitute files $oldPrjDir $prjDir
	}

	# if a path of an old tag value matches <oldPrjDir>, it should be
	# replaced with $(PRJ_DIR)

	if {[Wind::isWindows]} {
	    regsub -nocase -all -- $mReg10 $files {$(PRJ_DIR)\1} files
	} else {
	    regsub -all -- $mReg10 $files {$(PRJ_DIR)\1} files
	}
    }

    # add a space at the end of the files variable so that we do not need
    # to do two regular expressions ...

    append files " "

    if {($prjDir != "") && ($prjDir != $mPrjDir)} {

	set prjDirParent [file dirname $prjDir]

	set mReg1 "[EscapeRegexpCharacters ${prjDir}](/| |\t)"
	set mReg2 "[EscapeRegexpCharacters $prjDirParent]/"

	if {[Wind::isWindows]} {

	    regsub -all -- {/} [string tolower $mReg1] {[\\\/]} mReg1
	    regsub -all -- {/} [string tolower $mReg2] {[\\\/]} mReg2
	}

	set mPrjDir $prjDir
    }

    if {[Wind::isWindows]} {

	if {$prjDir != ""} {
	    regsub -nocase -all -- $mReg1 $files {$(PRJ_DIR)\1} files
	    regsub -nocase -all -- $mReg2 $files {$(PRJ_DIR)/../} files
	}

	if {$srcBase != ""} {
	    regsub -nocase -all -- $mReg3 $files {$(WIND_SOURCE_BASE)\1}  files
	}

	regsub -nocase -all -- $mReg8 $files {$(CC_BIN_DIR)\4} files

	# substitute the WIND_GNU_PATH and WIND_DIAB_PATH after the CC_BIN_DIR

	regsub -nocase -all -- $mReg13 $files {$(WIND_DIAB_PATH)\1} files
	regsub -nocase -all -- $mReg14 $files {$(WIND_GNU_PATH)\1} files

#	regsub -nocase -all -- $mReg4 $files {$(WIND_PROJ_BASE)\1} files
	regsub -nocase -all -- $mReg11 $files {$(TGT_DIR)\1} files
	regsub -nocase -all -- $mReg5 $files {$(WIND_BASE)\1} files

	# WIND_HOME substitution should come after WIND_BASE since WIND_BASE
	# should be a subdirectory of WIND_HOME

	regsub -nocase -all -- $mReg12 $files {$(WIND_HOME)\1} files

	regsub -nocase -all -- $mReg6 $files {$(WIND_HOST_TYPE)} files
	regsub -nocase -all -- $mReg9 $files {$(CC_HOST)} files
    } else {

	if {$prjDir != ""} {
	    regsub -all -- $mReg1 $files {$(PRJ_DIR)\1} files
	    regsub -all -- $mReg2 $files {$(PRJ_DIR)/../} files
	}

	if {$srcBase != ""} {
	    regsub -all -- $mReg3 $files {$(WIND_SOURCE_BASE)\1}  files
	}

	regsub -all -- $mReg8 $files {$(CC_BIN_DIR)\4} files

	# substitute the WIND_GNU_PATH and WIND_DIAB_PATH after the CC_BIN_DIR

	regsub -all -- $mReg13 $files {$(WIND_DIAB_PATH)\1} files
	regsub -all -- $mReg14 $files {$(WIND_GNU_PATH)\1} files

#	regsub -all -- $mReg4 $files {$(WIND_PROJ_BASE)\1} files
	regsub -all -- $mReg11 $files {$(TGT_DIR)\1} files
	regsub -all -- $mReg5 $files {$(WIND_BASE)\1} files

	# WIND_HOME substitution should come after WIND_BASE since WIND_BASE
	# should be a subdirectory of WIND_HOME

	regsub -all -- $mReg12 $files {$(WIND_HOME)\1} files

	regsub -all -- $mReg6 $files {$(WIND_HOST_TYPE)} files
	regsub -all -- $mReg9 $files {$(CC_HOST)} files
    }

    # we appended a " " at the beginning of this procedure. Remove it now

    set files [string trimright $files]
    regsub -all -- "\[ \t]*\\\\\[ \t]*\$" $files "\\0\n\t" files

    return $files
}

proc PrjUtils::srcDir {} {

    variable mSrcDir
    variable mSrcDirNotSet

    global env

    if {$mSrcDirNotSet} {

	if {[array names env WIND_SOURCE_BASE] == "WIND_SOURCE_BASE"} {
	    set mSrcDir [file join $env(WIND_SOURCE_BASE)]
	}

	set mSrcDirNotSet	0
    }

    return $mSrcDir
}

#########################################################################
#
# PrjUtils::archDirFind - find the architecture directory for a given CPU
#
# SYNOPSIS 
# \cs
#   PrjUtils::archDirFind  <cpu>
# \ce
#
# PARAMETERS 
#   cpu:    cpu, e.g. PPC603
#
# RETURNS: architecture directory, e.g. ppc
#
# ERRORS: Could not find architecture directory for cpu
#

proc PrjUtils::archDirFind {cpu} {

    switch -glob $cpu {
	MC68*		{return mc68k}
	MCF**		{return coldfire}
	CPU32		{return mc68k}
	I80*86		{return i86}
	PENTIUM*	{return i86}
	I960*		{return i960}
	PPC*		{return ppc}
	ARM*		{return arm}
	STRONGARM*	{return arm}
	XSCALE*		{return arm}
	MIPS*		{return mips}
	SPARC*		{return sparc}
	SIMLINUX	{return simlinux}
	SIMSPARCSOLARIS	{return simsolaris}
	SIMNT		{return simnt}
	SH*		{return sh}
	MCORE*		{return mcore}
	ULTRASPARC*	{return sparcv9}
	FR*	        {return frv}
	default		{error "Could not find arch. dir for cpu $cpu"}
    }
}     

proc PrjUtils::init {} {

    variable mReg3
    variable mReg4
    variable mReg5
    variable mReg6
    variable mReg8
    variable mReg9
    variable mReg11
    variable mReg12
    variable mReg13
    variable mReg14

    set mReg3 "\[ \t^][EscapeRegexpCharacters [PrjUtils::srcDir]](/| |\t)"
    set mReg4 "[EscapeRegexpCharacters [PrjUtils::defaultDir]](/| |\t)"
    set mReg5 "[EscapeRegexpCharacters [file join [Wind::base]]](/| |\t)"
    set mReg6 [Wind::hostType]

    # compute the CC_BIN_DIR regular expression

    set diabPReg "[EscapeRegexpCharacters [file join [Wind::tcPath diab] bin]]"
    set gnuPReg "[EscapeRegexpCharacters [file join [Wind::tcPath gnu] bin]]"
    set mReg8 [format "((%s)|(%s))(/| |\t)" $diabPReg $gnuPReg]

    # compute the CC_HOST regular expression

    set diabHReg	"[EscapeRegexpCharacters [Wind::tcHostType diab]]"
    set gnuHReg		"[EscapeRegexpCharacters [Wind::tcHostType gnu]]"
    set mReg9		[format "((%s)|(%s))" $diabHReg $gnuHReg]

    set mReg11 "[EscapeRegexpCharacters [file join [Wind::tgtDir]]](/| |\t)"

    # compute WIND_HOME, WIND_GNU_PATH and WIND_DIAB_PATH regular expressions

    set diabPReg "[EscapeRegexpCharacters [file dirname [file join\
		  [Wind::tcPath diab]]]]"
    set gnuPReg "[EscapeRegexpCharacters [file dirname [file join\
		 [Wind::tcPath gnu]]]]"

    set mReg12 "[EscapeRegexpCharacters [file join [Wind::home]]](/| |\t)"
    set mReg13 [format "%s(/| |\t)" $diabPReg]
    set mReg14 [format "%s(/| |\t)" $gnuPReg]

    if {[Wind::isWindows]} {

	regsub -all -- {/} [string tolower $mReg3] {[\\\/]} mReg3
	regsub -all -- {/} [string tolower $mReg4] {[\\\/]} mReg4
	regsub -all -- {/} [string tolower $mReg5] {[\\\/]} mReg5
	set mReg6 [string tolower $mReg6]
	set mReg8 [string tolower $mReg8]
	set mReg9 [string tolower $mReg9]
	regsub -all -- {/} [string tolower $mReg11] {[\\\/]} mReg11
	regsub -all -- {/} [string tolower $mReg12] {[\\\/]} mReg12
	regsub -all -- {/} [string tolower $mReg13] {[\\\/]} mReg13
	regsub -all -- {/} [string tolower $mReg14] {[\\\/]} mReg14
    }

    return
}

PrjUtils::init
