# cmpScriptLib.tcl - TCL scripting commands for project manipulation

# Copyright 1984-2006 Wind River Systems, Inc.

# The right to copy, distribute, modify, or otherwise make use 
# of this software may be licensed only pursuant to the terms 
# of an applicable Wind River license agreement. 

# modification history
# --------------------
# 03m,10Apr06,pee  SPR 120294. vxprj build forces rebuild of all sources.
# 03l,31Mar06,pee  adding BSP default first before PROFILE, SPR 119742
# 03k,14Mar06,pee  cleaning up soft exclude list. SPR 119058.
# 03j,21feb06,pee  optimization of cmpAddDepends
# 03i,10feb06,zhr  Added enhanced component check
# 03h,09jan06,pee  modified Profiles to support bundles and nested profiles
#		   improved creation performance by avoiding close/open cycle.
#		   partial implementation of API support for multiple implementations
# 03g,05dec05,pee  new component management strategy
# 03f,25oct05,zhr  Added display messages for error reporting
# 03e,25oct05,pee  separated explicit and implicit components
# 03d,06sep05,pee  added default profile support for scalable BSPs
# 03c,12aug05,pee  add dependencies when switching to lib mode
# 03b,11aug05,pee  fixed typo
# 03a,08aug05,pee  added support to build from source
# 02z,20Jun05,pee  Tool does not generate file dependencies anymore
# 02y,11may05,pee  SPR 108779. Explicitely sourcing dataDocLib.tcl instead of
#                  relying on package mechanism.
# 02x,23dec04,c_c  removed references to foundation.
# 02w,22oct04,fle  Doc fixes
# 02v,20oct04,fle  SPR#102786 : added cmpSizeGet
# 02u,07oct04,fle  Catch wccDelete in cmpBackup
#                  + SPR#102448 : added cmpBundleExcludedListGet
#                  + SPR#102452 : created cmpProfileInitGet, cmpProfileInitGet
#                  + SPR#102449 : created cmpOsNameSet and cmpOsNameGet
# 02t,01oct04,fle  Added cmpProfileListGet procedure
#                  + removed any vxApp reference
# 02s,08sep04,fle  SPR#88074 : added cmpMakefile to set or get the Makefile
#                  name
# 02r,01sep04,fle  Made it use Wind::foundation when needed
# 02q,30aug04,fle  created cmpStdout to also be able to get the stdout channel
# 02p,20aug04,fle  SPR#101170 : fixed copied project file when it is a
#                  relative file path
# 02o,17aug04,fle  Made it use new Wind::bspInfo and Wind::bspList
# 02n,08aug04,dlk  Added option build support to cmpProjCreate.
# 02n,14jul04,fle  Added cmpDependenciesGet
# 02m,25may04,fle  Added some cmpBuildRuleXxx procedures
#		   + removed the cmpFsFileXxx doc
# 02l,19may04,fle  Fixed cmpBuild to also be able to pass the build spec
# 02k,14may04,fle  Adapted to vxApps when the components list is empty
# 02j,14may04,fle  Added cmpBspListGet and cmpToolListGet
# 02i,06may04,fle  Removed all bullet lists refgen markups
# 02h,29apr04,fle  Removed some debug messages
# 02g,21apr04,fle  Fixed a missing close bracket in treeGet()
# 02f,09apr04,fle  SPR#94279 : made configlettes be expanded through
#                  prjConfigletteExpand procedure
# 02e,30mar04,fle  Force BSP dir to use slash path notation (not backslash)
# 02d,17feb04,fle  Faster use of packages
# 02c,11feb04,fle  Added cmpFsFileCreate
# 02b,05feb04,fle  Do not use prjFsFileInfoXxx anymore
# 02a,09dec03,fle  Added parameters to cmpTreeGet returned list
#                  + made autoscale return two lists of parameters
#                  + made 'set cmpProjHandle' be the last statement of
#                    cmpProjOpen for old application which were trying to get
#                    the project handle from a 'cmpProjOpen' call (while it
#                    should not return anything).
#                  + added prjFsFileLib wrappers
# 01z,02dec03,fle  Made cmpProjCreate use full path for project files
# 01y,28nov03,fle  Added bundle facilities
#                  + made cmpTreeGet also return bundles
#                  + made it use Wind::tgtDir rather than Wind::base/target
# 01x,17nov03,fle  Added cmpDependsGet and cmpModuleDependsGet
# 01w,12nov03,fle  Added the cmpTreeGet procedure
# 01v,14oct03,fle  Fixed cmpProjDelete to not stay in the directory it tries
#                  to delete
#                  + added cmpExcludedListGet
# 01u,07aug03,fle  made cmpBuild always throw an error
# 01t,07jul03,fle  made cmpInfo return the lines it prints to standard output
#                  + fixed cmpBuildMacroSet which has never been working
# 01s,25jun03,fle  Enhanced documentation
#                  + fixed cmpProjCopy which has never been working
# 01r,19jun03,fle  Updated to new location
# 01q,24jan03,nrj  merged changes made by Jifei Song (1/15/03)
#                  modify cmpBuildMacroGet to call prjBuildMacroGet with right
#                  number of arguments
# 01p,12jun02,cjs  Fix spr 78618 -- doc bug
# 01o,15may02,rbl  fixing SPR 77280, "Project facility messes up
#                  dependencies on a selection  e.g. can't change WDB
#                  connection"
# 01n,23apr02,rbl  fix cmpProjCreate to cope with bsp directories outside the
#                  tornado tree
# 01m,10nov01,dat  Adding cmpSave command
# 01l,05nov01,dat  Fixing cmpBuildSpecCreate
# 01k,01oct01,dat  Gnu-diab infrastructure
# 01j,13nov98,ms   updated docs for refgen.
# 01i,26oct98,ms   fixed cmpCurrentRuleSet.
# 01h,11oct98,ms   added autoscale
# 01g,25sep98,ms   library routines can now be added to a config via cmpProc*
# 01f,15sep98,ms   beefed up cmpTest.
# 01e,03sep98,ms   reworked cmpBuild. Fixed cmpFileRemove yet again.
# 01d,12aug98,ms   fixed cmpFile[Add|Remove]
# 01c,06aug98,ms   added cmpLoad
# 01b,30jul98,ms   added cmpFile[Add|Remove|ListGet], cmpBuildFlagsAdd,
#		   cmpBuildRuleSet, cmpValidate, cmpProjCopy
# 01a,27jul98,ms   written

# DESCRIPTION
# Tornado comes with a project facility that allows one to graphically
# organize source files into projects, change the build rules, and
# configure the operating system. See the Tornado Users Guide for
# details on what facilities the project facility offers.
# This library provides a way to perform these tasks in a script.
# To use this library, one must first source this library from
# tclsh. As an example, suppose you wanted to do a clean build of
# your project every night. One could write a script as follows:
#
# example.tcl:
# .CS
#    package require Wind
#    source [file join [Wind::base] host resource osconfig cmpScriptLib.tcl]
#    cmpProjOpen myProjectFile.wpj
#    cmpBuild clean
#    cmpBuild
# .CE
#
# One can execute the script from a UNIX or Windows command shell by
# typing "tclsh example.tcl."
# Alternatively, one can type the scripting commands directly into the
# tclsh shell.
#
# TREE
# cmpScriptLib.tcl
#   +-> prjLib.tcl
#         +-> prjXmlLib.tcl
#         +-> Utils.tcl
#         +-> prjFileLib.tcl
#         +-> prjFsFileLib.tcl
#         +-> prjBuildLib.tcl
#               +-> tc_cplus.tcl
#                     +-> Utils.tcl
#                     +-> gnuInfoGen.tcl
#               +-> macrosInfo.tcl
#         +-> prj_vxApp.tcl
#               +-> wccLib.tcl
#                     +-> dataDocLib.tcl
#         +-> prj_vxWorks.tcl
#               +-> wccLib.tcl
#                     +-> dataDocLib.tcl
# 
# SEE ALSO
#    dataDocLib

package provide OsConfig 1.0

package require Wind

set cmpProjHandle ""
set cmpInteractive ""
set cmpProjFile ""

################################################################################
#
# cmpProjOpen - open a project
#
# SYNOPSIS
# \cs
#   cmpProjOpen <prjFile>
# \ce
#
# DESCRIPTION
# This routine opens a project file and sets it as the active project for all
# subsequent scripting commands. This first closes any project that was
# previously opened.
#
# PARAMETERS
#    prjFile : project file to open
#
# RETURNS: N/A
#
# ERRORS:
# \is
# \i no such project
# \ie
#
# SEE ALSO: prjOpen, cmpProjHandleGet, cmpProjClose, cmpProjCopy

proc cmpProjOpen {prjFile} {

    global cmpProjHandle
    global cmpProjFile

    if {$cmpProjHandle != ""} {
	cmpProjClose
    }

    set cmpProjFile	[file join $prjFile]
    set cmpProjHandle	[prjOpen $prjFile]
}

################################################################################
#
# cmpProjClose - close the currently open project
#
# SYNOPSIS
# \cs
#   cmpProjClose
# \ce
#
# DESCRIPTION
# Close and save the currently open project.
#
# RETURNS: N/A
#
# SEE ALSO: cmpSave
# 

proc cmpProjClose {} {

    global cmpProjHandle
    global cmpProjFile
    prjClose $cmpProjHandle SAVE 

    set cmpProjHandle ""
    set cmpProjFile ""
}

################################################################################
#
# cmpBuildSourceEnable - enable building from source
#
# SYNOPSIS
# \cs
#   cmpBuildSourceEnable
# \ce
#
# DESCRIPTION
# Enable building from source or lib
#
# RETURNS: N/A
#
#

proc cmpBuildSourceEnable { cmpProjHandle srcBuildEnableState } {
     prjTagDataSet $cmpProjHandle BUILD_SOURCE_ENABLE $srcBuildEnableState

    cmpRecalculate $cmpProjHandle

    # check dependencies if going to lib mode
#    if { $srcBuildEnableState == 0 } {
#	set wcc [wccCurrentGet $cmpProjHandle]
#	set depends [wccDependsGet $cmpProjHandle $wcc]
#	if {[llength $depends] > 0} {
#	    cmpAdd $depends
#	}
#    }
}

################################################################################
#
# cmpSysLayerGet - Get system layer value
#
# SYNOPSIS
# \cs
#   cmpSysLayerGet
# \ce
#
# DESCRIPTION
# Enable building from source or lib
#
# RETURNS: system layer
#
# \NOMANUAL
#

proc cmpSysLayerGet { hProj components } {
    set defaultLayerValue  4
    set sysLayer 0
    set cmpLib [cxrDocCreate $hProj]
    set cmpSet [$cmpLib setCreate]

	# filter non-components as list may include containers
	$cmpSet = $components
	set components [$cmpSet instances Component]

    foreach component $components {
        $cmpSet = $component
        $cmpSet = [$cmpSet get LAYER]
        set compL [$cmpSet contents]
        if { $compL == "" } {
	    set compL $defaultLayerValue
        }
        if { $compL > $sysLayer } {
            set sysLayer $compL
        }
    }
    return $sysLayer
}

proc cmpBuildModeEval { hProj components } {

	set enable [prjTagDataGet $hProj BUILD_SOURCE_ENABLE]

	# if not enabled, return

	if { ($enable == 0) || ($enable == "") } {
		return "library"
	}

	# enabled. See if we can do it.
	# this time, check layer value.

	set layer [cmpSysLayerGet $hProj $components ]

	if { $layer <= 3 } {
		return "source"
	} else {
		return "library"
	}
}

################################################################################
#
# cmpUnsupportedSBCompsGet - retrieve unsupported components for source build
#
# SYNOPSIS
# \cs
#   cmpUnsupportedSBCompsGet
# \ce
#
# DESCRIPTION
# Get unsupported components for source build 
#
# RETURNS: component list unavailable for source
#

proc cmpUnsupportedSBCompsGet { hProj } {

       set components [wccComponentListGet $hProj [wccCurrentGet $hProj]]
       set unspportedSBLayerValue  4
       set cmpLib [cxrDocCreate $hProj]
       set cmpSet [$cmpLib setCreate]
       set unspportedSBCompList {}

	# filter non-components as list may include containers
	$cmpSet = $components
	set components [$cmpSet instances Component]

       foreach component $components {
           $cmpSet = $component
           $cmpSet = [$cmpSet get LAYER]
           set compL [$cmpSet contents]

           if { ($compL == "") || ($compL >= $unspportedSBLayerValue) } {
               lappend  unspportedSBCompList  $component 
           }
       }

       $cmpSet delete
       return $unspportedSBCompList
}

 

proc cmpSourceBuildEnableGet { hProj } {

    return [prjTagDataGet $hProj BUILD_SOURCE_ENABLE]

}

 
################################################################################
#
# cmpBuildModeGet - retrieve current building mode 
#
# SYNOPSIS
# \cs
#   cmpBuildModeGet
# \ce
#
# DESCRIPTION
# Get building state 
#
# RETURNS: N/A
#
#

proc cmpBuildModeGet { hProj } {

	set components [wccComponentListGet $hProj [wccCurrentGet $hProj]]

	return [ cmpBuildModeEval $hProj $components ]

}

################################################################################
#
# cmpProjCreate - create a project and open it
#
# SYNOPSIS
# \cs
#   cmpProjCreate <bspDir> [prjFile [prjType [cpu [tool [opt]]]]]
# \ce
#
# DESCRIPTION
# This procedure creates a project and opens it. If the project already exists,
# it just opens it.
#
# The <prjType> argument to this procedure can only be 'vxWorks' (a system
# image project)
#
# The project file to be created must live in a directory which does not
# already exists, and by convention should end in a .wpj extension.
#
# For 'vxWorks' projects, <bspDir> should be the name of a BSP directory in
# target/config (e.g., pc386), or the full path to a BSP directory.
#
# If no project <prjFile> file name is specified, a default project name of
# $(TGT_DIR)/proj/<bsp>_<tool>/<bsp>_<tool>.wpj is used, where <bsp> is the name
# of the BSP directory.
#
# PARAMETERS
#    bspFile : BSP file to rely on to create project
#    prjFile : name of the project file to be created
#    prjType : type of project (vxWorks only)
#    cpu     : optional CPU override (vxWorks only)
#    tool    : optional TOOL override (vxWorks only)
#    opt     : optional OPT value (-inet6, -fr, or -inet6_fr allowed presently)
#
# RETURNS: The path of the so-created project
#

proc cmpProjCreate {bspFile {prjFile ""} {prjType vxWorks} {cpu ""} {tool ""} \
		    {opt ""} } {

    global cmpProjHandle
    global cmpProjFile

    if {$cmpProjHandle != ""} {
	cmpProjClose
    }

    if {$prjType == "vxApp"} {
	set tc $bspFile
	::prj_vxApp::create $prjFile $tc
	set cmpProjHandle [prjOpen $prjFile]
	set cmpProjFile [file join $prjFile]
	return $prjFile
    }

    if {$bspFile == [file tail $bspFile]} {
        set bspDir	[file join [Wind::tgtDir] config $bspFile]
        set bspName	$bspFile
    } else {
        set bspDir	[file join $bspFile]
        set bspName	[file tail $bspFile]
    }

    if {$prjFile == ""} {
	set prjFile [file join [Wind::prjDir] ${bspName}_${tool} \
			       ${bspName}_${tool}.wpj]
    } else {

	# seems like the project file path must be absolute, or some of the -I
	# CC flags will be wrong :(

	if {[file pathtype $prjFile] != "absolute"} {

	    set prjFile [file join [pwd] $prjFile]
	}
    }

    if {![file exists $prjFile]} {
	set cmpProjHandle [::prj_vxWorks::create $prjFile $bspDir "" "" $cpu $tool $opt]
	set cmpProjFile [file join $prjFile]
	::prj_vxWorks_hidden::makeGen $cmpProjHandle \
				      [file join [file dir $prjFile] \
					         [prjMakefile $cmpProjHandle]]
    } else {
	set cmpProjHandle [prjOpen $prjFile]
    }
    set cmpProjFile [file join $prjFile]

    # set the os name this project was created for

    cmpOsName [Wind::osName]

    return $prjFile
}

################################################################################
#
# cmpProjCopy - create a copy of an existing project and open it
#
# SYNOPSIS
# \cs
#   cmpProjCopy <newPrjFile> <oldPrjFile>
# \ce
#
# DESCRIPTION
# Make a copy of an existing project, and open the copy. This is useful
# if you want to make configuration changes but not disturb the
# original project.
# The project file being created has the same restrictions as described
# in cmpProjCreate.
#
# PARAMETERS
#    newPrjFile : project file to create
#    oldPrjFile : existing project file
#
# SEE ALSO
#    cmpProjCreate
#
# RETURNS: N/A
#

proc cmpProjCopy {newPrjFile oldPrjFile} {

    global cmpProjHandle
    global cmpProjFile

    # if the given <oldPrjFile> is not the current project file, close the
    # current project, open <oldPrjFile> and use it

    if {$cmpProjFile != $oldPrjFile} {
	cmpProjClose
    }

    # now open the <oldPrjFile> project if <cmpProjHandle> is set to ""

    if {$cmpProjHandle == ""} {
	set cmpProjHandle [prjOpen $oldPrjFile]
    }

    # SPR#101170 : modify relative path files

    if {[file pathtype $newPrjFile] == "relative"} {

	set newPrjFile	[file join [pwd] $newPrjFile]

	regsub -all -- {\\} $newPrjFile {/} newPrjFile
	regsub -all -- {^\./} $newPrjFile "[pwd]/" newPrjFile
	regsub -all -- {/[^/]+/\.\./} $newPrjFile {/} newPrjFile
    }

    prjCopy $cmpProjHandle $newPrjFile

    if {$cmpProjHandle != ""} {
	cmpProjClose
    }

    set cmpProjHandle	[prjOpen $newPrjFile]
    set cmpProjFile	[file join $newPrjFile]
}

################################################################################
#
# cmpProjDelete - delete the currently open project
#
# SYNOPSIS
# \cs
#   cmpProjDelete
# \ce
#
# DESCRIPTION
# Delete the currently open project.
#
# RETURNS: N/A
#
# ERRORS: N/A
#

proc cmpProjDelete {} {

    global cmpProjHandle
    set prjDir [file dir [prjInfoGet $cmpProjHandle fileName]]

    cmpProjClose

    # if the current application is in a subtree od the project file, we'd
    # better first leave or the delete action will fail

    while {[regexp [file join $prjDir] [file join [pwd]]]} {
	cd ..
    }

    file delete -force $prjDir
}

################################################################################
#
# cmpProjHandleGet - get the handle to the currently open project
#
# SYNOPSIS
# \cs
#   cmpProjHandleGet
# \ce
#
# DESCRIPTION
# This routine returns the <project handle> to the currently open project.
#
# Since this library supports only one open project at a time, one does not
# need to use this routine in order to use this library.
#
# However this library is built on top of lower-level libraries that must be
# passed a project handle. These libraries are currently not documented.
#
# PARAMETERS: N/A
#
# RETURNS: The current project handle, or an empty String if there is no
# project opened yet
#
# ERRORS: N/A
#
# SEE ALSO: cmpProjOpen
# 

proc cmpProjHandleGet {} {
    global cmpProjHandle
    return $cmpProjHandle
}

################################################################################
#
# cmpBuild - build the project
#
# SYNOPSIS
# \cs
#   cmpBuild [rule]
# \ce
#
# DESCRIPTION
# This routine builds the project using the currently active buildSpec.
#
# The build output goes to <Standard Output> and <Standard Error>.
#
# If no rule is specified, it builds the default rule for the currently active
# buildSpec.
#
# The 'clean' rule removes all binaries.
# 
# PARAMETERS
#    rule : (optional) build rule to invoke
#
# RETURNS: N/A
#
# ERRORS: N/A
# 

proc cmpBuild {{rule ""} {buildSpec ""}} {

    global cmpProjHandle

    prjSave $cmpProjHandle
    prjBuild $cmpProjHandle $rule $buildSpec 1
}

################################################################################
#
# cmpBuildRuleSet - set the build rule for the project
#
# SYNOPSIS
# \cs
#   cmpBuildRuleSet <rule>
# \ce
#
# DESCRIPTION
# This routine sets the default build rule for the active buildSpec within the
# currently open project.
#
# The 'vxWorks' projects support 'clean', 'vxWorks', 'vxWorks_rom',
# 'vxWorks_romResident', and 'vxWorks_romCompress'.
#
# PARAMETERS
#    rule : build rule to set
#
# RETURNS: N/A
#
# ERRORS: N/A
# 

proc cmpBuildRuleSet {rule} {

    global cmpProjHandle

    set build [prjBuildCurrentGet $cmpProjHandle]
    prjBuildRuleCurrentSet $cmpProjHandle $build $rule
}

################################################################################
#
# cmpBuildSpecSet - set the active buildSpec
#
# SYNOPSIS
# \cs
#   cmpBuildSpecSet <buildSpec>
# \ce
#
# DESCRIPTION
# A project can contain one or more buildSpecs. This routine is used to set the
# active buildSpec.
# 
# PARAMETERS
#    buildSpec : name of the buildSpec to set as active
#
# RETURNS: N/A
#
# ERRORS: N/A
#
# SEE ALSO: cmpBuildSpecCreate
#

proc cmpBuildSpecSet {buildSpec} {
    global cmpProjHandle
    prjBuildCurrentSet $cmpProjHandle $buildSpec
}

################################################################################
#
# cmpBuildSpecCreate - create a new buildSpec
#
# SYNOPSIS
# \cs
#   cmpBuildSpecCreate <build> <arg>
# \ce
#
# DESCRIPTION
# Create a new buildSpec for the project, and set it as the active buildSpec.
#
# One can create a buildSpec by copying an existing buildSpec, or create a new
# one for a different toolchain.
#
# If <arg> is of the form ::tc_<CPUTOOL> (e.g., ::tc_PPC604gnu), then a build is
# created for that toolchain. Otherwise <arg> is interpreted to be the name of
# an existing build.
#
# PARAMETERS
#    build : name of the buildSpec to create
#    arg : name of a toolchain or existing build
# 
# RETURNS: N/A
#
# ERRORS: N/A
#
# SEE ALSO: cmpBuildSpecSet
#

proc cmpBuildSpecCreate {build arg} {
    global cmpProjHandle
    if {[info procs ::${arg}::name] == ""} {
	prjBuildCopy $cmpProjHandle $arg $build
    } else {
	prjBuildCreate $cmpProjHandle $build $arg
    }
    prjBuildCurrentSet $cmpProjHandle $build
}

################################################################################
#
# cmpBuildMacroSet - set a build macro
#
# SYNOPSIS
# \cs
#   cmpBuildMacroSet <macro> <value>
# \ce
#
# DESCRIPTION
# Set the value of a build macro for the active buildSpec.
#
# Build macros include RAM_LOW_ADRS, 'CFLAGS', etc.
#
# PARAMETERS
#    macro : name of the build macro to set
#    value : value to assign the macro
# 
# RETURNS: N/A
#
# ERRORS: N/A
#
# SEE ALSO: cmpBuildMacroGet
#

proc cmpBuildMacroSet {macro value} {
    global cmpProjHandle
    prjBuildMacroSet $cmpProjHandle [prjBuildCurrentGet $cmpProjHandle] $macro \
		     $value
}

################################################################################
#
# cmpBuildMacroGet - get a build macro
#
# SYNOPSIS
# \cs
#   cmpBuildMacroGet <macro>
# \ce
#
# DESCRIPTION
# Get the value of a build macro for the active buildSpec.
#
# Build macros include RAM_LOW_ADRS, 'CFLAGS', etc.
#
# PARAMETERS
#    macro : name of the build macro to get
# 
# RETURNS: The value of the macro
#
# ERRORS: N/A
#
# SEE ALSO: cmpBuildMacroSet
#

proc cmpBuildMacroGet macro {
    global cmpProjHandle
    
    prjBuildMacroGet $cmpProjHandle [prjBuildCurrentGet $cmpProjHandle] $macro
}

################################################################################
#
# cmpBuildFlagsAdd - add flags to the build
#
# SYNOPSIS
# \cs
#   cmpBuildFlagsAdd <flags>
# \ce
#
# DESCRIPTION
# This routine is just a quick way to append to the build macros 'CFLAGS' and
# CFLAGS_AS.
#
# It could also be acheived using 'cmpBuildMacroSet' and 'cmpBuildMacroGet'
# procedures
#
# PARAMETERS
#    flags : flags to add to 'CFLAGS' and CFLAGS_AS
# 
# RETURNS: N/A
#
# ERRORS: N/A
#
# SEE ALSO: cmpBuildMacroSet, cmpBuildMacroGet
#

proc cmpBuildFlagsAdd {flags} {
    global cmpProjHandle
    set hProj $cmpProjHandle
    set build [prjBuildCurrentGet $hProj]

    set tc [prjBuildTcGet $hProj $build]
    set cc [${tc}::toolFindByFile foo.c]
    set as [${tc}::toolFindByFile foo.s]

    set cflags   [format "%s %s" $flags [prjBuildFlagsGet $hProj $build $cc]]
    set asflags  [format "%s %s" $flags [prjBuildFlagsGet $hProj $build $as]]

    prjBuildFlagsSet $hProj $build $cc  $cflags
    prjBuildFlagsSet $hProj $build $as  $asflags
}

################################################################################
#
# cmpProjTcGet - get project's active toolchain
#
# SYNOPSIS
# \cs
#   cmpProjTcGet
# \ce
#
# DESCRIPTION
# This procedure returns the current project active toolchain. The toolchain
# is of the form :
#
# \cs
#   ::tc_<CPU><TOOL>
# \ce
#
# Where <CPU> is the target 'CPU' and <TOOL> the toolchain name.
#
# PARAMETERS: N/A
#
# RETURNS: The toolchain name.
#
# ERRORS: N/A
#
# EXAMPLES:
# In a 'tclsh' shell:
# \cs
# % package require Wind
# 1.0
# % cd [file join [Wind::base] host resource osconfig tcl]
# % source cmpScriptLib.tcl
# % cmpProjOpen [file join [Wind::prjDir] simpc_gnu simpc_gnu.wpj]
# c:/Tornado/target/proj/simpc_gnu/simpc_gnu.wpj
# % cmpProjTcGet
# ::tc_SIMNTgnu
# %
# \ce
#
# NOMANUAL
#

proc cmpProjTcGet {} {
    global cmpProjHandle
    set build [prjBuildCurrentGet $cmpProjHandle]
    return [prjBuildTcGet $cmpProjHandle $build]
}

################################################################################
#
# cmpAdd - add components to vxWorks
#
# SYNOPSIS
# \cs
#   cmpAdd <components>
# \ce
#
# DESCRIPTION
# This routine adds one or more optional vxWorks components to the project.
#
# For example, one can use
#
# \cs
# cmpAdd INCLUDE_NFS
# \ce
#
# to add the NFS client to the vxWorks image.
#
# If a folder is specified (e.g., FOLDER_NETWORK), then all default components
# below that folder are added.
# 
# The tool will also add any other components needed by the ones being added, as
# determined by dependency analysis.
#
# If the global tcl variable <cmpInteractive> is set to 'true"', then the user
# will be prompted to confirm before the components and dependencies are added.
#
# PARAMETERS
#    components : list of components to add
#
# RETURNS: The list of components really added
#
# ERRORS: N/A
#
# SEE ALSO: cmpRemove, cmpListGet
# 

proc cmpAdd {components} {
    global cmpProjHandle
    global cmpInteractive

    set wcc		[wccCurrentGet $cmpProjHandle]

    # if there is no Wind Component Configuration, we may be in an application
    # project, just return.

    if {$wcc == ""} {
	return {}
    }

    set cxrDoc	[cxrDocCreate $cmpProjHandle]
    set cmpSet	[$cxrDoc setCreate]

    $cmpSet = $components
    if {[$cmpSet - [$cmpSet instances]] != {}} {
	puts [cmpStdout] "components [$cmpSet - [$cmpSet instances]] do not exist"
	set components [$cmpSet instances]
    }

    if {$components == {}} {return {}}

    puts [cmpStdout] "adding [lsort $components]"

    set differences [cmpAddDepends $cmpProjHandle $components detailed ]

    if {$differences == {}} {return {}}

    set new_ex		[lindex $differences 0]
    set new_im		[lindex $differences 1]
    set new_excl	[lindex $differences 2]
    set default		[lindex $differences 3]
    set convenience	[lindex $differences 4]
    set implicit	[lindex $differences 5]
    set removed		[lindex $differences 6]
    set incomplete_c	[lindex $differences 7]
    set overloaded_c	[lindex $differences 8]

    if {$default != ""} {
	puts [cmpStdout] "\nNOTE: the following default components have been included\n\
		[lsort $default]"
    }
    if {$convenience != ""} {
	puts [cmpStdout] "\nNOTE: the following components have been included for convenience"
	puts [cmpStdout] "[lsort $convenience]"
    }
    if {$implicit != ""} {
	puts [cmpStdout] "\nNOTE: the following component have been added as required to complete configuration"
	puts [cmpStdout] "[lsort $implicit]"
    }
    if {$removed != ""} {
	puts [cmpStdout] "\nNOTE: the following component are no longer required and have been removed"
	puts [cmpStdout] "[lsort $removed]"
    }
    if {$incomplete_c != ""} {
	puts [cmpStdout] "\nWARNING: the following containers are incomplete\n\
		[lsort $incomplete_c]\n\
		Project will not build until container requirements are met or until container is removed"
    }
    if {$overloaded_c != ""} {
	puts [cmpStdout] "\nWARNING: the following containers have too many selected components\n\
		[lsort $overloaded_c]\n\
		Project will not build until extra components are removed."
    }

    set dependencies [concat $default $convenience $implicit]
    set comps [concat $components $dependencies]

    if {$cmpInteractive != ""} {
	puts stdout -nonewline "OK to proceed (y for yes, anything else to\
				cancel)? "
	gets stdin reply
	if {"$reply" != "y"} {
	    puts stdout "canceled"
	    return $comps
        }
    }

    # sort out list, only save Components
    $cmpSet = $new_ex
    set new_ex [$cmpSet instances Component]
    $cmpSet = $new_im
    set new_im [$cmpSet instances Component]

    $cmpSet delete

    wccComponentSet $cmpProjHandle $wcc $new_ex $new_im $new_excl

    return $comps
}

####################################################################################
#
# cmpAddDepends - evaluate dependencies when adding components to vxWorks
#
# SYNOPSIS
# \cs
#   cmpAddDepends <components>
# \ce
#
# DESCRIPTION
# This routine adds one or more optional vxWorks components to the project.
#
# For example, one can use
#
# \cs
# cmpAdd INCLUDE_NFS
# \ce
#
# to add the NFS client to the vxWorks image.
#
# If a folder is specified (e.g., FOLDER_NETWORK), then all default components
# below that folder are added.
# 
# The tool will also add any other components needed by the ones being added, as
# determined by dependency analysis.
#
# If the global tcl variable <cmpInteractive> is set to 'true"', then the user
# will be prompted to confirm before the components and dependencies are added.
#
# PARAMETERS
#    components : list of components to add
#
# RETURNS: The list of components really added
#
# ERRORS: N/A
#
# SEE ALSO: cmpRemove, cmpListGet
# 

proc cmpAddDepends { hProj components {outputFlag implicit}} {

    global cmpInteractive
    set retval {}

    set wcc		[wccCurrentGet $hProj]

    # if there is no Wind Component Configuration, we may be in an application
    # project, just return.

    if {$wcc == ""} {
	return {}
    }

    set cxrDoc		[cxrDocCreate $hProj]
    set cmpSet		[$cxrDoc setCreate]

    set l_s [wccComponentListGet $hProj $wcc both]
    set l_ex [lindex $l_s 0]
    set l_im [lindex $l_s 1]
    set l_excl [wccComponentListGet $hProj $wcc softexclude]

    $cmpSet = $l_excl
    set new_excl [$cmpSet - $components]

    set bundles [cmpBundleListGet]

    $cmpSet = $components
    $cmpSet = [$cmpSet - $l_ex]

    if {[$cmpSet contents] == ""} {
	prjWarnLog "no components to add"
	return {} 
    }

    # check if we add components to containers
    set rm_ex {}
    set new_ex $l_ex
    set exp_comps [cxrRequires $hProj $cmpSet 0]
    $cmpSet = [$cmpSet get _CHILDREN]
    set apis [$cmpSet instances Api]
    set selections [$cmpSet instances Selection]
    set one_containers {}
    foreach selection $selections {
	$cmpSet = $selection
	set max 100
	scan [format "%s" [$cmpSet get COUNT]] "%d-%d" min max
	if {$max == 1} {
	    lappend one_containers $selection
	}
    }
    $cmpSet = $new_ex
    set expanded_ex [cxrRequires $hProj $cmpSet 0]
    foreach container $one_containers {
	$cmpSet = $container
	$cmpSet = [$cmpSet get CHILDREN]
	set children [$cmpSet - $exp_comps]
	$cmpSet = $expanded_ex
	$cmpSet = [$cmpSet & $children]
	cxrRequiredBy $hProj $cmpSet 0
	set rm_ex [concat $rm_ex [$cmpSet & $new_ex]]
    }
    $cmpSet = $exp_comps
    $cmpSet = [$cmpSet get PROVIDES]
    set apis [$cmpSet instances Api]
    foreach container $apis {
	$cmpSet = $container
	$cmpSet = [$cmpSet get _PROVIDES]
	set children [$cmpSet - $exp_comps]
	$cmpSet = $expanded_ex
	$cmpSet = [$cmpSet & $children]
	cxrRequiredBy $hProj $cmpSet 0
	set rm_ex [concat $rm_ex [$cmpSet & $new_ex]]
    }

    $cmpSet = $new_ex
    $cmpSet = [$cmpSet - $rm_ex]
    set new_ex [$cmpSet + $components]

    set before [join $l_s]

    set l_after [cxrSubtreeDetailed $hProj [concat $new_ex $bundles] $new_excl]

    set new_l_excl [lindex $l_after 7]

    # check for containers with too many selections
    if {[lindex $l_after 6] != {}} {
	set toomany [lindex $l_after 6]
	foreach container $toomany {
	}
    }

    set after [join [lrange $l_after 0 4]]

    switch $outputFlag {
	implicit {
	    $cmpSet = $after
	    $cmpSet = [$cmpSet - $before ]
	    set retval [$cmpSet instances Component]
	}
	detailed {
	    # 0, 1, 2 the new lists of explicit, implicit and soft exclude
	    set retval [list $new_ex $after $new_l_excl]

	    # 3 components included as default
	    $cmpSet = [lindex $l_after 4]
	    $cmpSet = [$cmpSet instances Component]
	    lappend retval [$cmpSet - $before]

	    # 4 component s included for conveninence. Only INCLUDE_WHEN for now
	    $cmpSet = [lindex $l_after 3]
	    $cmpSet = [$cmpSet instances Component]
	    lappend retval [$cmpSet - $before]

	    # new implicit components
	    $cmpSet = [join [lrange $l_after 0 2]]
	    $cmpSet = [$cmpSet instances Component]
	    lappend retval [$cmpSet - $before]

	    # components removed
	    $cmpSet = $before
	    $cmpSet = [$cmpSet - $new_ex]
	    lappend retval [$cmpSet - $after]
	    set retval [concat $retval [lrange $l_after 5 6]]
	}
	default {
	    error "nternal error: invalid parameter outputFlag to cmpAddDepends"
	}
    }

    $cmpSet delete

    return $retval
}

proc cmpRecalculate { hProj } {
    set wcc	[wccCurrentGet $hProj]

    if {$wcc == ""} {
	return {}
    }

    set cxrDoc [cxrDocCreate $hProj]
    set cxrSet [$cxrDoc setCreate]

    set lists	[wccComponentListGet $hProj $wcc both]
    set l_ex [lindex $lists 0]
    set l_im [lindex $lists 1]
    set l_excl [wccComponentListGet $hProj $wcc softexclude]

    set bundles	[prjTagDataGet $hProj BUNDLE__LIST]
    set lists [cxrSubtreeDetailed $hProj [concat $l_ex $bundles] $l_excl]

    set new_l_excl [lindex $lists 7]

    $cxrSet = [join [lrange $lists 0 4]]
    set new_l_im [$cxrSet instances Component]

    prjTagDataSet $hProj WCC_${wcc}_IMPLICIT_COMPONENTS $new_l_im
    prjTagDataSet $hProj WCC_${wcc}_SOFT_EXCLUDE_COMPONENTS $new_l_excl

    $cxrSet = $new_l_im
    set l_added [$cxrSet - $l_im]

    $cxrSet = l_im
    set l_removed [$cxrSet - $new_l_im]

    $cxrSet delete

    return [list $l_added $l_removed]
}

proc cmpAddBuildModeEval { hProj components } {
    set dependencies [cmpAddDepends $hProj $components ]
    set existingComponents [cmpListGet]
    return [cmpBuildModeEval $hProj [concat $existingComponents $components $dependencies]]
}

################################################################################
#
# cmpRemoveDepends - Evaluate dependencies when removing components from vxWorks
#
# SYNOPSIS
# \cs
#   cmpRemoveDepends <hProj components>
# \ce
#
# DESCRIPTION
# This routine removes one or more optional vxWorks components from the project.
#
# For example, one can use
#
# \cs
# cmpRemove INCLUDE_NFS
# \ce
#
# to remove the NFS client from the vxWorks image.
#
# If a folder is specified (e.g., FOLDER_NETWORK), then all components below
# that folder are removed.
#
# The tool will also remove any components that require by the ones being
# removed, as determined by dependency analysis.
#
# If the global tcl variable <cmpInteractive> is set to 'true', then the user
# will be prompted to confirm before the components and dependencies are
# removed.
#
# PARAMETERS
#    components : list of components to remove
#    forceFlag  : tells the function wether dependecies would be forced out or not
#		values:
#		force : compute as if dependencies would be force out
#		doNotForce : compute as if dependencies would not be forced out.
#    outputFlag : tells the function what to calculate and return
#		values:
#		explicit : components that would be removed from the explicit list
#		implicit : components that would be removed from the implicit list
#		both     : both lists, formatted as a list of list
#		all      : all components that would be removed, formatted as a single list
#
# RETURNS: The list of components removed (including dependencies)
#
# ERRORS: N/A
#
# SEE ALSO: cmpAdd, cmpListGet
#

proc cmpRemoveDepends {hProj components {forceFlag force} {outputFlag all}} {

    global cmpInteractive

    set wcc	[wccCurrentGet $hProj]

    # if there is no Wind Component Configuration, we may be in an application
    # project, just return.

    if {$wcc == ""} {
	return {}
    }

    set cxrDoc	[cxrDocCreate $hProj]
    set mxrDoc	[mxrDocCreate $hProj]
    set cmpSet	[$cxrDoc setCreate]
    set comps	{}
    set clists [wccComponentListGet $hProj $wcc both]
    set l_excl [wccComponentListGet $hProj $wcc softexclude]
    set clist_ex [lindex $clists 0]
    set clist_im [lindex $clists 1]
    set rmlist_ex {}
    set rmlist_im {}
    set mvlist {}
    set retval {}

    if {$forceFlag == "force"} {
	$cmpSet = [concat $clist_ex $clist_im]
    } else {
	$cmpSet = $clist_ex
    }

    $cmpSet = [$cmpSet & $components]

    if {$forceFlag == "force"} {
	set depend [cxrSupertree $cmpSet $mxrDoc [concat $clist_ex $clist_im]]
	$cmpSet = [concat $clist_ex $clist_im]
	set depend [$cmpSet & $depend]
    } else {
	set depend [$cmpSet contents]
    }

    set bundles [cmpBundleListGet]

    set l_before [cxrSubtreeDetailed $hProj [concat $clist_ex $bundles] $l_excl]
    set before [join [lrange $l_before 0 4]]

    if {$forceFlag == "force"} {
	$cmpSet = $l_excl
	set l_excl [$cmpSet + $components]
    }

    $cmpSet = $clist_ex
    set new_ex [$cmpSet - $depend]
    set l_after [cxrSubtreeDetailed $hProj [concat $new_ex $bundles] $l_excl]

    set after [join [lrange $l_after 0 4]]

    $cmpSet = $after
    set mvlist [$cmpSet & $depend]

    $cmpSet = $clist_im
    set rmlist_im [$cmpSet - $after]

    $cmpSet = $clist_ex
    set rmlist_ex [$cmpSet & $depend]

    set new_l_excl [lindex $l_after 7]

    switch $outputFlag {
	lists {
	    set retval [list $rmlist_ex $rmlist_im $mvlist]
	}
	all {
	    $cmpSet = $rmlist_ex
	    $cmpSet = [$cmpSet - $mvlist]
	    $cmpSet = [$cmpSet + $rmlist_im]
	    set retval [$cmpSet instances Component]
	}
	detailed {
	    set retval [list $new_ex $after $new_l_excl]
	    $cmpSet = $depend
	    lappend retval [$cmpSet - $components]
	    $cmpSet = $before
	    lappend retval [$cmpSet - [concat $after $depend]]
	    $cmpSet = [lindex $l_after 4]
	# conponent included as default
	    lappend retval [$cmpSet - $before]
	    $cmpSet = [lindex $l_after 3]
 	# include_when
	    lappend retval [$cmpSet - $before]
	    $cmpSet = [join [lrange $l_after 0 1]]
	# imnplicit
	    lappend retval [$cmpSet - $before]
	    $cmpSet = $depend
	    lappend retval [$cmpSet & $after]
	    set retval [concat $retval [lrange $l_after 5 6]]
	}
	default {
	    error "internal error: invalid parameter outputFlag to cmpRemoveDepends"
	}
    }

    $cmpSet delete
    return $retval
}

################################################################################
#
# cmpRemove - remove components from vxWorks
#
# SYNOPSIS
# \cs
#   cmpRemove <components>
# \ce
#
# DESCRIPTION
# This routine removes one or more optional vxWorks components from the project.
#
# For example, one can use
#
# \cs
# cmpRemove INCLUDE_NFS
# \ce
#
# to remove the NFS client from the vxWorks image.
#
# If a folder is specified (e.g., FOLDER_NETWORK), then all components below
# that folder are removed.
#
# The tool will also remove any components that require by the ones being
# removed, as determined by dependency analysis.
#
# If the global tcl variable <cmpInteractive> is set to 'true', then the user
# will be prompted to confirm before the components and dependencies are
# removed.
#
# PARAMETERS
#    components : list of components to remove
#    forceFlag  : tells the function wether dependecies would be forced out or not
#		values:
#		force : compute as if dependencies would be force out
#		doNotForce : compute as if dependencies would not be forced out.
#
# RETURNS: The list of components removed (including dependencies)
#
# ERRORS: N/A
#
# SEE ALSO: cmpAdd, cmpListGet
#

proc cmpRemove {components {forceFlag force}} {

    global cmpProjHandle
    global cmpInteractive

    set wcc	[wccCurrentGet $cmpProjHandle]

    # if there is no Wind Component Configuration, we may be in an application
    # project, just return.

    if {$wcc == ""} {
	return {}
    }

    set cxrDoc	[cxrDocCreate $cmpProjHandle]
    set cmpSet	[$cxrDoc setCreate]
    set comps	{}

    $cmpSet = $components

    $cmpSet = [$cmpSet & [wccComponentListGet $cmpProjHandle $wcc]]

    if {[$cmpSet contents] == ""} {
	prjWarnLog "no components to remove"
	$cmpSet delete
	return $comps
    }

    set components [$cmpSet contents]
    set wccList [format "%s" [wccComponentListGet $cmpProjHandle $wcc]]

    set dependancies [cmpRemoveDepends $cmpProjHandle $components $forceFlag detailed]

    # filter containers for now
    $cmpSet =  [lindex $dependancies 0]
    set l_ex [$cmpSet instances Component]
    $cmpSet = [lindex $dependancies 1]
    set l_im [$cmpSet instances Component]
    set l_excl [lindex $dependancies 2]

    # all components removed
    set cmps [join [lrange $dependancies 3 4]]

    # dependencies removed
    $cmpSet = $l_ex
    set dep [lsort [$cmpSet - $components]]

    # components dropped
    $cmpSet = $l_im
    set drop [lsort [$cmpSet - $components]]

    $cmpSet delete

    puts [cmpStdout] "removing [lsort $components]"

    set dep [lindex $dependancies 3]
    if {$dep != {}} {
	puts [cmpStdout] "\nWARNING: also removing dependencies\n$dep"
    }

    set drop [lindex $dependancies 4]
    if {$drop != {}} {
	puts [cmpStdout] "\nWARNING: the following components are no longer required and have been removed\n$drop"
    }

    set l_mv [lindex $dependancies 8]
    if {$l_mv != {}} {
	puts [cmpStdout] "\nWARNING: the following components are still required and have been kept\n$l_mv"
    }

    if {$cmpInteractive != ""} {
	puts stdout -nonewline "\nOK to proceed (y for yes, anything else to\
				cancel)? "
	gets stdin reply
	if {$reply != "y"} {
	    puts stdout "canceled"
	    return $comps
        }
    }

    wccComponentSet $cmpProjHandle $wcc $l_ex $l_im $l_excl

    return $cmps
}

proc cmpRemoveBuildModeEval { hProj components } {
    set cxrDoc [cxrDocCreate $hProj]
    set cmpSet [$cxrDoc setCreate]

    set dependencies [cmpRemoveDepends $hProj $components ]
    set existingComponents [cmpListGet]
    set remove [concat $components $dependencies]

    $cmpSet = $existingComponents
    $cmpSet = [$cmpSet - $remove]

    set whatsleft [format "%s" [$cmpSet contents]]

    $cmpSet delete

    return [cmpBuildModeEval $hProj $whatsleft]
}

################################################################################
#
# cmpListGet - get a list of components in vxWorks
#
# SYNOPSIS
# \cs
#   cmpListGet
# \ce
#
# DESCRIPTION
# This routine returns the list of components in the current vxWorks project.
#
# PARAMETERS:
#    outputFlag : tells the function what to calculate and return
#		values:
#		explicit : components that would be removed from the explicit list
#		implicit : components that would be removed from the implicit list
#		both     : both lists, formatted as a list of list
#		all      : all components, formatted as a single list
#
# RETURNS: a list of components
#
# ERRORS: N/A
# 

proc cmpListGet {{outputFlag all}} {
    global cmpProjHandle
    set wcc [wccCurrentGet $cmpProjHandle]

    if {$wcc == ""} {
	return {}
    } else {
	return [wccComponentListGet $cmpProjHandle $wcc $outputFlag]
    }
}

################################################################################
#
# cmpExcludedListGet - get the list of excluded components for current project
#
# SYNOPSIS
# \cs
#   cmpExcludedListGet
# \ce
#
# This procedure gets the list of all the components available for the current
# project, removes all the components returned by 'cmpListGet', and returns the
# so-created list.
#
# PARAMETERS: N/A
#
# RETURNS: A list of all the components not available, but not included by this
#          project.
#
# ERRORS: N/A
#
# SEE ALSO: cmpListGet, cmpUnavailListGet
#

proc cmpExcludedListGet {} {

    global cmpProjHandle

    # get the list of all components, and the list of included components

    set cxrDoc	[cxrDocCreate $cmpProjHandle]
    set excluded [format "%s" [$cxrDoc instances Component]]
    set included [cmpListGet]
#    cxrDocDelete $cmpProjHandle

    # now remove all components of included from excluded list

    foreach comp $included {

	if {[set ix [lsearch -exact $excluded $comp]] != -1} {
	    set excluded [lreplace $excluded $ix $ix]
	}
    }

    return $excluded
}

################################################################################
#
# cmpProcAdd - add library routine(s) to the configuration
#
# SYNOPSIS
# \cs
#   cmpProcAdd <routineList>
# \ce
#
# DESCRIPTION
# This routine adds a library routine to the configuration.
#
# So if a user knows he want the routine 'i', but don't know what INCLUDE_*
# component it belongs to (in this case none), they can still bring it into the
# vxWorks configuration and also bring in all components that it needs via
# dependency analysis.
#
# The implimentation is to create a .cdf file in the project directory that
# corresponds to the routine.
#
# PARAMETERS:
#   routineList : list of routines to add to the current project.
#
# RETURNS: N/A
#
# ERRORS: N/A
#
# SEE ALSO: cmpProcRemove, cmpProcListGet
# NOMANUAL
#

proc cmpProcAdd {routineList} {

    global cmpProjHandle
    global cmpInteractive

    set wcc [wccCurrentGet $cmpProjHandle]

    if {$wcc == ""} {
	return
    }

    set hProj $cmpProjHandle
    set dependancies [wccProcDependsGet $hProj $wcc $routineList]

    if {$dependancies != ""} {
	puts [cmpStdout] "NOTE: also adding dependencies\n[lsort\
			  $dependancies]\n"
    }

    if {$cmpInteractive != ""} {

	puts stdout -nonewline "OK to proceed (y for yes, anything else to\
				cancel)? "
	gets stdin reply

	if {$reply != "y"} {
	    puts stdout "canceled"
	    return
        }
    }
    wccProcAdd $hProj $wcc $routineList
    wccComponentAdd $hProj $wcc $dependancies
}

################################################################################
#
# cmpProcRemove - remove library routine(s) from a vxWorks configuration
#
# SYNOPSIS
# \cs
#   cmpProcRemove <routineList>
# \ce
#
# DESCRIPTION
# Removes the routines of <routineList> from the current project.
#
# PARAMETERS:
#   routineList : list of routines to remove from this project.
#
# RETURNS: N/A
#
# ERRORS: N/A
#
# SEE ALSO: cmpProcListGet, cmpProcAdd
#
# NOMANUAL
#

proc cmpProcRemove {routineList} {
    global cmpProjHandle
    global cmpInteractive
    set wcc [wccCurrentGet $cmpProjHandle]

    # if there is no Wind Component Configuration, we may be in an application
    # project, just return.

    if {$wcc == ""} {
	return
    }

    set hProj $cmpProjHandle
    wccProcRemove $hProj $wcc $routineList
}

################################################################################
#
# cmpProcListGet - get a list of library routine(s) in the configuration
#
# SYNOPSIS
# \cs
#   cmpProcListGet
# \ce
#
# DESCRIPTION
# This procedure gets the list of all the library routine this project's
# configuration contains.
#
# PARAMETERS: N/A
#
# RETURNS: A list of routines contained by the current project.
#
# ERRORS: N/A
#
# SEE ALSO: cmpProcAdd
#
# NOMANUAL
#

proc cmpProcListGet {} {

    global cmpProjHandle
    global cmpInteractive

    set wcc [wccCurrentGet $cmpProjHandle]

    # if there is no Wind Component Configuration, we may be in an application
    # project, just return.

    if {$wcc == ""} {
	return {}
    }

    set hProj $cmpProjHandle
    wccProcListGet $hProj $wcc
}

################################################################################
#
# cmpBackup - backup the current component set
#
# SYNOPSIS
# \cs
#   cmpBackup <backupName>
# \ce
#
# DESCRIPTION
# Backup a component configuration. This procedure is to be used together with
# the 'cmpRestore' procedure.
#
# PARAMETERS:
#   backUpName : name to give to the backed up component configuration
#
# RETURNS: N/A
#
# ERRORS: N/A
#
# SEE ALSO: cmpRestore
#
# NOMANUAL
# 

proc cmpBackup {backUpName} {

    global cmpProjHandle
    set wcc [wccCurrentGet $cmpProjHandle]

    # if there is no Wind Component Configuration, we may be in an application
    # project, just return.

    if {$wcc == ""} {
	return
    }

    # catch the wccdelete since <backupName> may not already exist

    catch {wccDelete $cmpProjHandle $backUpName}

    wccCreate $cmpProjHandle $backUpName $wcc
    wccCurrentSet $cmpProjHandle $wcc
}

################################################################################
#
# cmpRestore - restore a backed up component set
#
# SYNOPSIS
# \cs
#   cmpRestore <backUpName>
# \ce
#
# DESCRIPTION
# Restore a backed-up component configuration.
#
# PARAMETERS:
#   backUpName : name of the backed up configuration
#
# RETURNS: N/A
#
# ERRORS: N/A
#
# SEE ALSO: cmpBackup
#
# NOMANUAL
# 

proc cmpRestore backUpName {
    global cmpProjHandle
    set wcc [wccCurrentGet $cmpProjHandle]

    # if there is no Wind Component Configuration, we may be in an application
    # project, just return.

    if {$wcc == ""} {
	return
    }

    wccDelete $cmpProjHandle $wcc
    wccCreate $cmpProjHandle $wcc $backUpName
}

################################################################################
#
# cmpParamSet - set a parameter in the current project
#
# SYNOPSIS
# \cs
#   cmpParamSet <param> <value>
# \ce
#
# DESCRIPTION
# This routine sets the value of a component parameter.
#
# For example, the component INCLUDE_IOS has parameter MAX_FDS (max # file
# descriptors that can be open at once). One can use
#
# \cs
# cmpParamSet MAX_FDS 60
# \ce
#
# to increase the value to 60.
#
# PARAMETERS:
#   param : name of the component parameter to set value of
#   value : value to give to the component parameter
#
# RETURNS: N/A
#
# ERRORS: N/A
# 

proc cmpParamSet {param value} {
    global cmpProjHandle
    set wcc [wccCurrentGet $cmpProjHandle]

    # if there is no Wind Component Configuration, we may be in an application
    # project, just return.

    if {$wcc == ""} {
	return
    }

    wccParamSet $cmpProjHandle $wcc $param $value

    return
}

################################################################################
#
# cmpParamGet - get a parameter
#
# SYNOPSIS
# \cs
#   cmpParamGet <param>
# \ce
#
# DESCRIPTION
# This routine gets the value of a component parameter.
#
# For example, the component INCLUDE_IOS has parameter MAX_FDS (max # file
# descriptors that can be open at once). One can use
#
# \cs
# cmpParamGet MAX_FDS
# \ce
#
# to get the current value of that parameter.
#
# PARAMETERS:
#   param : name of the parameter to get value of.
#
# RETURNS: the parameter value, or an empty string if no such parameter exists
#
# ERRORS: N/A
#

proc cmpParamGet {param} {
    global cmpProjHandle
    set wcc [wccCurrentGet $cmpProjHandle]

    # if there is no Wind Component Configuration, we may be in an application
    # project, just return.

    if {$wcc == ""} {
	return ""
    }

    return [wccParamGet $cmpProjHandle $wcc $param]
}

################################################################################
#
# cmpFileListGet - get a list of files in the project
#
# SYNOPSIS
# \cs
#   cmpFileListGet
# \ce
#
# DESCRIPTION
# This routine returns the list of source files in the project.
#
# PARAMETERS: N/A
#
# RETURNS: the list of files in the project
#
# ERRORS: N/A
# 

proc cmpFileListGet {} {
    global cmpProjHandle
    return [prjFileListGet $cmpProjHandle]
}

################################################################################
#
# cmpFileAdd - add a source file to the project
#
# SYNOPSIS
# \cs
#   cmpFileAdd <file>
# \ce
#
# DESCRIPTION
# This routine adds  the given <file> source file to the project.
#
# PARAMETERS
#	file : full path to the file being added
#
# RETURNS: N/A
#
# ERRORS: N/A
#

proc cmpFileAdd {file} {
    global cmpProjHandle
    prjFileAdd $cmpProjHandle $file
}

################################################################################
#
# cmpFileRemove - remove a file from the project
#
# SYNOPSIS
# \cs
#   cmpFileRemove <file>
# \ce
#
# DESCRIPTION
# This routine removes the <file> source file from the project.
#
# PARAMETERS
#	file : full path to the file being removed
#
# RETURNS: N/A
#
# ERRORS: N/A
#

proc cmpFileRemove {file} {
    global cmpProjHandle
    prjFileRemove $cmpProjHandle $file
}

################################################################################
#
# cmpLoad - dynamically load components
#
# SYNOPSIS
# \cs
#   cmpLoad <cmps> [mods]
# \ce
#
# DESCRIPTION
# This routine dynamically loads one or more components to a running target.
#
# To use this, you must first be connected to a tgtsvr using wtxToolAttach.
#
# This routine will extract all needed object modules from the vxWorks archive,
# and link them together. It will also build the initialization code needed to
# initialize the component(s) and all dependent components.
#
# It then loads the modules and the initialization code, and calls the
# initialization code.
#
# Example: To load the vxWorks target shell, do
#
# \cs
# cmpLoad FOLDER_SHELL
# \ce
#
# PARAMETERS
#    cmps : components to load
#    mods : (optional) set of additional object modules to load
#
# RETURNS: N/A
#
# ERRORS: N/A
#
# SEE ALSO: wtxToolAttach
#

proc cmpLoad {cmps {mods ""}} {

    set hProj [cmpProjHandleGet]

    set cxrDoc [cxrDocCreate $hProj]
    set cxrSet [$cxrDoc setCreate]
    $cxrSet = $cmps
    while {[set children [$cxrSet get DEFAULTS]] != ""} {
	$cxrSet = [$cxrSet instances Component]
	$cxrSet = [$cxrSet + $children]
    }
    set cmps [$cxrSet instances Component]
    $cxrSet delete
    wccComponentLoad $hProj $cmps $mods
}

################################################################################
#
# autoscale - automatically scale vxWorks
#
# SYNOPSIS
# \cs
#   autoscale [arg]
# \ce
#
# DESCRIPTION
# Configure vxWorks components based on what is required by the project source
# files.
#
# There are three possible values for the <arg> parameter :
# \is
# \i 'verbose' is the default mode, and tells what scaling can/should be done,
#      and why
# \i 'terse' prints only terse information
# \i 'shrink' causes autoscaling to actually be done
# \ie
#
# PARAMETERS
#    arg : (optional) default is 'verbose'. can also be 'shrink' or 'terse'.
#
# RETURNS: Two lists. First list is the list of components which should be
# added, and the second is the list of components which may be removed.
#

proc autoscale {{verbose verbose}} {

    global cmpInteractive

    set hProj		[cmpProjHandleGet]
    set cxrDoc		[cxrDocCreate $hProj]
    set cxrSet		[$cxrDoc setCreate]
    set cxrSetOrig	[$cxrDoc setCreate]

    $cxrSetOrig = [cmpListGet]

    if {$verbose == "shrink"} {
	set cmpInteractive ""
	cmpRemove [cmpListGet]
	set cmpInteractive true
    }

    set autoscaleInfo [prj_vxWorks_hidden::autoscale $hProj verbose]

    $cxrSet = [lindex $autoscaleInfo 0]

    set added [format "%s" [$cxrSet - [format "%s" [$cxrSetOrig instances]]]]
    set remov [format "%s" [$cxrSetOrig - [format "%s" [$cxrSet instances]]]]

    puts [cmpStdout] "need to add: $added"
    puts [cmpStdout] "can remove: $remov"

    if {$verbose == "verbose"} {

	foreach info [lrange $autoscaleInfo 1 end] {

	    puts [cmpStdout] "module [lindex $info 0]"
	    puts [cmpStdout] "imports symbols [lindex $info 1]"
	    puts [cmpStdout] "pulls in extra modules [lindex $info 2]"

	    if {[lindex $info 3] == ""} {
		continue
	    }
	    puts [cmpStdout] "which corresponds to extra components [lindex\
			      $info 3]"
	}
    }

    if {$verbose == "shrink"} {
	set cmpInteractive ""
	set extra INCLUDE_USER_APPL
	if {"[$cxrSetOrig & INCLUDE_SIO]" == "INCLUDE_SIO"} {
	    lappend extra INCLUDE_SIO
	}
	cmpAdd "$extra [lindex $autoscaleInfo 0]"
	set cmpInteractive true
	cmpBuild
    }

    $cxrSet     delete
    $cxrSetOrig delete

    return [list $added $remov]
}

################################################################################
#
# cmpUnavailListGet - list unavailable components
#
# SYNOPSIS
# \cs
#   cmpUnavailListGet
# \ce
#
# DESCRIPTION
# This routine returns a list of components that are known but not available.
#
# A component is not available if it is not installed in the tree (e.g., an
# optional product), or it requires an unavailable component.
#
# PARAMETERS: N/A
#
# RETURNS: a list of components
#
# ERRORS: N/A
#
# SEE ALSO: cxrUnavailListGet
#

proc cmpUnavailListGet {{opt ""}} {

    set hProj [cmpProjHandleGet]
    set cxrDoc [cxrDocCreate $hProj]
    set mxrDoc [mxrDocCreate $hProj]
    return [cxrUnavailListGet $cxrDoc $mxrDoc $opt]

}

proc cmpUnavailListDetailGet {{opt ""}} {
 
    set hProj [cmpProjHandleGet]
    set component ""
    return [cxrDocValidate1 $hProj $component "unavailDetail"]
    
}

################################################################################
#
# cmpValidate - validate the component configuration
#
# SYNOPSIS
# \cs
#   cmpValidate
# \ce
#
# DESCRIPTION
# Check if the current component configuration is valid.
#
# It returns a list of the form {{cmp1 err1} {cmp2 err2} ...} where <cmpX> is
# the name of a misconfigured component, and <errX> is an error string
# describing what is wrong with the component.
#
# PARAMETERS: N/A
#
# RETURNS: an error list, or an empty string if the configuration is valid.
# 

proc cmpValidate {} {

    global cmpProjHandle
    global cmpInteractive

    set wcc [wccCurrentGet $cmpProjHandle]

    if {$wcc == ""} {
	return ""
    } else {
	set info [wccValidate $cmpProjHandle $wcc]

	if {$cmpInteractive != ""} {
	    foreach elem $info {
		puts [cmpStdout] $elem
	    }
	    return
	}
	return $info
    }
}

################################################################################
#
# cmpTest - test the component descriptor files for errors
#
# SYNOPSIS
# \cs
#   cmpTest [components [buildTest]]
# \ce
#
# DESCRIPTION
# This routine tests the component descriptor files for semantic errors.
#
# For example, it makes sure all components are contained in exactly one folder.
# It prints all diagnostic information to <Standard Output>.
#
# This routine is a diagnostic tool for component developers to test that their
# new components were written correctly.
#
# It has two optional parameters. <components> allows one to restrict the set of
# components being tested.
#
# <buildTest> allows one to verify that each component can be built
# individually. The most common reason a component would fail the build test is
# if the component descriptor does not have a HDR_FILE containing a prototype
# for the components initialization routine.
#
# PARAMETERS
#    components : (optional) list of components to test (default is all)
#    buildTest : (optional) pass any value to see if components can build
#
# RETURNS: N/A
# 

proc cmpTest  {{components ""} {buildTest ""}} {

    global cmpProjHandle

    # for a single component check do something special
    if { $components == "" } { 
         cxrDocValidate $cmpProjHandle $components $buildTest
    } else {
         if {[llength $components] == 1} {
             cxrDocValidate1 $cmpProjHandle $components $buildTest
         }
    }
    
}

################################################################################
#
# cmpInfo - get info about a set of components
#
# SYNOPSIS
# \cs
#   cmpInfo <components>
# \ce
#
# DESCRIPTION
# Print basic information about a component, including the
# \is
# \i 'subtree' (other components required by this component)
# \i 'supertree' (other components that require this component)
# \ie
#
# The 'subtree' and 'supertree' are computed using the engine's dependency
# analyizer.
#
# This routine is primarily used by component authors who want to check on the
# scalability of their components.
#
# PARAMETERS
#    components : list of components to analyze
#
# RETURNS: N/A
#
# ERRORS: N/A
#

proc cmpInfo {components} {

    global cmpProjHandle

    set mxrDoc [mxrDocCreate $cmpProjHandle]
    set cxrDoc [cxrDocCreate $cmpProjHandle]
    set cmpSet [$cxrDoc setCreate]

    set res	""

    $cmpSet = $components

    foreach line [cxrCmpInfoGet $cmpSet $mxrDoc] {

	puts [cmpStdout] $line

	if {$res == ""} {
	    set res $line
	} else {
	    set res [format "%s\n%s" $res $line]
	}
    }

    $cmpSet delete
    return $res
}

################################################################################
#
# cmpSave - save the current project info
#
# SYNOPSIS
# \cs
#   cmpSave
# \ce
#
# DESCRIPTION
# Update the project file with the latest configuration.  No update occurs
# if the project configuration has not changed.
#
# PARAMETERS: N/A
#
# SEE ALSO: cmpBuild
#
# RETURNS: N/A
# 

proc cmpSave {} {
    global cmpProjHandle

    # update file, if changes have been made
    prjSave $cmpProjHandle
}

################################################################################
#
# cmpProjFileGet - get the project file path
#
# SYNOPSIS
# \cs
#   cmpProjFileGet
# \ce
#
# DESCRIPTION
# Get the project file path of the current project.
#
# If no project has been open using 'cmpProjOpen', 'cmpProjCreate' or
# 'cmpProjCopy', this procedure returns an empty String.
#
# PARAMETERS: N/A
#
# RETURNS: The current Project's file path, or an empty String if there is no
#          current project.
#
# ERRORS: N/A
#
# SEE ALSO: cmpProjOpen, cmpProjCreate, cmpProjCopy
#

proc cmpProjFileGet {} {

    global cmpProjFile
    return $cmpProjFile
}

################################################################################
#
# cmpTreeGet - get the component tree
#
# SYNOPSIS
# \cs
#   cmpTreeGet
# \ce
#
# DESCRIPTION
# This procedure returns a Tcl list which represents the components tree.
#
# The returned list is a flat list with <Folder> elements first, then
# <Selection> element, and finally the <Component> elements.
#
# Each element is itself a list of data corresponding to the element. First
# member of this element is the type which is either :
# \is
# \i Folder
# \i Selection
# \i Component
# \i Bundle
# \ie
#
# FOLDERS
# The Folders are described this way :
# \cs
# {Folder <name> <desc> <help> <defaults> <synopsis> <children> <parent> \
#  <isDefault>}
# \ce
#
# SELECTIONS
# Selection elements are described this way :
# \cs
# {Selection <name> <desc> <help> <defaults> <synopsis> <children> <parent> \
#  <isDefault> <archive> <bsp_stubs> <cfg_params> <configlettes> <count> \
#  <excludes> <hdr_files> <hide_unless> <init_after> <init_before> <init_rtn> \
#  <link_datasyms> <link_syms> <macro_nest> <modules> <prototype> <requires> \
#  <shutdown_rtn> <uses _excludes> <_hide_unless> <_init_after> <_init_before> \
#  <_init_order> <_link_datasyms> <_link_syms> <_requires> <_uses>}
# \ce
#
# COMPONENTS
# Components are described this way :
# \cs
# {Component <name> <desc> <help> <synopsis> <parent> <archive> <bsp_stubs> \
#  <cfg_params> <configlettes> <entry_points> <excludes> <exclude_when> \
#  <hdr_files> <hide_unless> <include_when> <init_after> <init_before> \
#  <init_rtn> <link_datasym> <link_syms> <macro_nest> <modules> <pref_domain> \
#  <project> <prototype> <requires> <shutdown_rtn> <term_rtn> \
#  <uses _components> <_defaults> <_excludes> <_exclude_when> <_hide_unless> \
#  <_include_when> <_init_after> <_init_before> <_init_order> <_link_datasyms> \
#  <_link_syms> <_requires> <_uses>}
# \ce
#
# BUNDLES
# The Bundles are described this way :
# \cs
# {Bundle <name> <desc> <help> <synopsis> <components> <parent>}
# \ce
#
#
# RETURNS: A Tcl list representing the Cdf components tree
#
# ERRORS: N/A
#
# INTERNAL
#

proc cmpTreeGet {} {

    global cmpProjHandle

    set cxrDoc		[cxrDocCreate $cmpProjHandle]
    set hProj		$cmpProjHandle

    # get the folders, selections and components from cdf files

    set folders		[format "%s" [$cxrDoc instances Folder]]
    set selections	[format "%s" [$cxrDoc instances Selection]]
    set apis		[format "%s" [$cxrDoc instances Api]]
    set components	[format "%s" [$cxrDoc instances Component]]
    set bundles		[format "%s" [$cxrDoc instances Bundle]]
    set parameters	[format "%s" [$cxrDoc instances Parameter]]

    # creat a unique list of elements

    set folderSet	[$cxrDoc setCreate]
    set selectionSet	[$cxrDoc setCreate]
    set compSet		[$cxrDoc setCreate]
    set bundleSet	[$cxrDoc setCreate]
    set paramSet	[$cxrDoc setCreate]

    set returnedList	{}

    # get the folders first, and append them along with their value to the
    # returned list

    foreach folder $folders {

	$folderSet = $folder

	lappend returnedList [list \
	    Folder \
	    $folder \
	    [format "%s" [$folderSet get NAME]] \
	    [format "%s" [$folderSet get HELP]] \
	    [format "%s" [$folderSet get DEFAULTS]] \
	    [format "%s" [$folderSet get SYNOPSIS]] \
	    [format "%s" [$folderSet get CHILDREN]] \
	    [format "%s" [$folderSet get _CHILDREN]] \
	    [format "%s" [$folderSet get _DEFAULT]] \
	]
    }

    # take care of the selections

    foreach selection $selections {

	$selectionSet = $selection

	lappend returnedList [list \
	    Selection \
	    $selection \
	    [format "%s" [$selectionSet get NAME]] \
	    [format "%s" [$selectionSet get HELP]] \
	    [format "%s" [$selectionSet get DEFAULTS]] \
	    [format "%s" [$selectionSet get SYNOPSIS]] \
	    [format "%s" [$selectionSet get CHILDREN]] \
	    [format "%s" [$selectionSet get _CHILDREN]] \
	    [format "%s" [$selectionSet get _DEFAULT]] \
	    [format "%s" [$selectionSet get ARCHIVE]] \
	    [format "%s" [$selectionSet get BSP_STUBS]] \
	    [format "%s" [$selectionSet get CFG_PARAMS]] \
	    [format "%s" [ \
		prjConfigletteExpand $hProj "[$selectionSet get CONFIGLETTES]" \
	    ]] \
	    [format "%s" [$selectionSet get COUNT]] \
	    [format "%s" [$selectionSet get EXCLUDES]] \
	    [format "%s" [$selectionSet get HDR_FILES]] \
	    [format "%s" [$selectionSet get HIDE_UNLESS]] \
	    [format "%s" [$selectionSet get INIT_AFTER]] \
	    [format "%s" [$selectionSet get INIT_BEFORE]] \
	    [format "%s" [$selectionSet get INIT_RTN]] \
	    [format "%s" [$selectionSet get LINK_DATASYMS]] \
	    [format "%s" [$selectionSet get LINK_SYMS]] \
	    [format "%s" [$selectionSet get MACRO_NEST]] \
	    [format "%s" [$selectionSet get MODULES]] \
	    [format "%s" [$selectionSet get PROTOTYPE]] \
	    [format "%s" [$selectionSet get REQUIRES]] \
	    [format "%s" [$selectionSet get SHUTDOWN_RTN]] \
	    [format "%s" [$selectionSet get USES]] \
	    [format "%s" [$selectionSet get _EXCLUDES]] \
	    [format "%s" [$selectionSet get _HIDE_UNLESS]] \
	    [format "%s" [$selectionSet get _INIT_AFTER]] \
	    [format "%s" [$selectionSet get _INIT_BEFORE]] \
	    [format "%s" [$selectionSet get _INIT_ORDER]] \
	    [format "%s" [$selectionSet get _LINK_DATASYMS]] \
	    [format "%s" [$selectionSet get _LINK_SYMS]] \
	    [format "%s" [$selectionSet get _REQUIRES]] \
	    [format "%s" [$selectionSet get _USES]] \
	]
    }
    foreach selection $apis {

	$selectionSet = $selection

	lappend returnedList [list \
	    Selection \
	    $selection \
	    [format "%s" [$selectionSet get NAME]] \
	    [format "%s" [$selectionSet get HELP]] \
	    [format "%s" [$selectionSet get DEFAULTS]] \
	    [format "%s" [$selectionSet get SYNOPSIS]] \
	    [format "%s" [$selectionSet get _PROVIDES]] \
	    [format "%s" [$selectionSet get _CHILDREN]] \
	    [format "%s" [$selectionSet get _DEFAULT]] \
	    [format "%s" [$selectionSet get ARCHIVE]] \
	    [format "%s" [$selectionSet get BSP_STUBS]] \
	    [format "%s" [$selectionSet get CFG_PARAMS]] \
	    [format "%s" [ \
		prjConfigletteExpand $hProj "[$selectionSet get CONFIGLETTES]" \
	    ]] \
	    [format "%s" [$selectionSet get COUNT]] \
	    [format "%s" [$selectionSet get EXCLUDES]] \
	    [format "%s" [$selectionSet get HDR_FILES]] \
	    [format "%s" [$selectionSet get HIDE_UNLESS]] \
	    [format "%s" [$selectionSet get INIT_AFTER]] \
	    [format "%s" [$selectionSet get INIT_BEFORE]] \
	    [format "%s" [$selectionSet get INIT_RTN]] \
	    [format "%s" [$selectionSet get LINK_DATASYMS]] \
	    [format "%s" [$selectionSet get LINK_SYMS]] \
	    [format "%s" [$selectionSet get MACRO_NEST]] \
	    [format "%s" [$selectionSet get MODULES]] \
	    [format "%s" [$selectionSet get PROTOTYPE]] \
	    [format "%s" [$selectionSet get REQUIRES]] \
	    [format "%s" [$selectionSet get SHUTDOWN_RTN]] \
	    [format "%s" [$selectionSet get USES]] \
	    [format "%s" [$selectionSet get _EXCLUDES]] \
	    [format "%s" [$selectionSet get _HIDE_UNLESS]] \
	    [format "%s" [$selectionSet get _INIT_AFTER]] \
	    [format "%s" [$selectionSet get _INIT_BEFORE]] \
	    [format "%s" [$selectionSet get _INIT_ORDER]] \
	    [format "%s" [$selectionSet get _LINK_DATASYMS]] \
	    [format "%s" [$selectionSet get _LINK_SYMS]] \
	    [format "%s" [$selectionSet get _REQUIRES]] \
	    [format "%s" [$selectionSet get _USES]] \
	]
    }

    # now look for the components

    foreach component $components {

	$compSet = $component

	lappend returnedList [list \
	    Component \
	    $component \
	    [format "%s" [$compSet get NAME]] \
	    [format "%s" [$compSet get HELP]] \
	    [format "%s" [$compSet get SYNOPSIS]] \
	    [format "%s" [concat [$compSet get _CHILDREN] [$compSet get PROVIDES]]] \
	    [format "%s" [$compSet get ARCHIVE]] \
	    [format "%s" [$compSet get BSP_STUBS]] \
	    [format "%s" [concat [$compSet get CFG_PARAMS] [$compSet get LINK_PARAMS]]] \
	    [format "%s" [ \
		prjConfigletteExpand $hProj "[$compSet get CONFIGLETTES]" \
	    ]] \
	    [format "%s" [$compSet get ENTRY_POINTS]] \
	    [format "%s" [$compSet get EXCLUDES]] \
	    [format "%s" [$compSet get EXCLUDE_WHEN]] \
	    [format "%s" [$compSet get HDR_FILES]] \
	    [format "%s" [$compSet get HIDE_UNLESS]] \
	    [format "%s" [$compSet get INCLUDE_WHEN]] \
	    [format "%s" [$compSet get INIT_AFTER]] \
	    [format "%s" [$compSet get INIT_BEFORE]] \
	    [format "%s" [$compSet get INIT_RTN]] \
	    [format "%s" [$compSet get LINK_DATASYM]] \
	    [format "%s" [$compSet get LINK_SYMS]] \
	    [format "%s" [$compSet get MACRO_NEST]] \
	    [format "%s" [$compSet get MODULES]] \
	    [format "%s" [$compSet get PREF_DOMAIN]] \
	    [format "%s" [$compSet get PROJECT]] \
	    [format "%s" [$compSet get PROTOTYPE]] \
	    [format "%s" [$compSet get REQUIRES]] \
	    [format "%s" [$compSet get SHUTDOWN_RTN]] \
	    [format "%s" [$compSet get TERM_RTN]] \
	    [format "%s" [$compSet get USES]] \
	    [format "%s" [$compSet get _COMPONENTS]] \
	    [format "%s" [$compSet get _DEFAULTS]] \
	    [format "%s" [$compSet get _EXCLUDES]] \
	    [format "%s" [$compSet get _EXCLUDE_WHEN]] \
	    [format "%s" [$compSet get _HIDE_UNLESS]] \
	    [format "%s" [$compSet get _INCLUDE_WHEN]] \
	    [format "%s" [$compSet get _INIT_AFTER]] \
	    [format "%s" [$compSet get _INIT_BEFORE]] \
	    [format "%s" [$compSet get _INIT_ORDER]] \
	    [format "%s" [$compSet get _LINK_DATASYMS]] \
	    [format "%s" [$compSet get _LINK_SYMS]] \
	    [format "%s" [$compSet get _REQUIRES]] \
	    [format "%s" [$compSet get _USES]] \
	]
    }

    # look for the bundles

    foreach bundle $bundles {

	$bundleSet = $bundle

	lappend returnedList [list \
	    Bundle \
	    $bundle \
	    [format "%s" [$bundleSet get NAME]] \
	    [format "%s" [$bundleSet get HELP]] \
	    [format "%s" [$bundleSet get SYNOPSIS]] \
	    [format "%s" [$bundleSet get COMPONENTS]] \
	    [format "%s" [$bundleSet get _CHILDREN]] \
	]
    }

    # look for the parameters

    foreach param $parameters {

	$paramSet = $param

	lappend returnedList [list \
	    Parameter \
	    $param \
	    [format "%s" [$paramSet get NAME]] \
	    [format "%s" [$paramSet get HELP]] \
	    [format "%s" [$paramSet get SYNOPSIS]] \
	    [format "%s" [$paramSet get DEFAULT]] \
	    [format "%s" [$paramSet get STORAGE]] \
	    [format "%s" [$paramSet get TYPE]] \
	    [format "%s" [$paramSet get VALUE]] \
	    [format "%s" [concat [$paramSet get _CFG_PARAMS] [$paramSet get _LINK_PARAMS]]] \
	]
    }

    # delete datadoc sets and document

    $folderSet delete
    $selectionSet delete
    $compSet delete
    $bundleSet delete
    $paramSet delete

#    cxrDocDelete $cmpProjHandle

    return $returnedList
}

################################################################################
#
# cmpDependsGet - get component dependencies for a list of components
#
# SYNOPSIS
# \cs
#   cmpDependsGet <components>
# \ce
#
# DESCRIPTION
# This procedure gets the list of comonents on which <components> depend.
#
# PARAMETERS:
#   components : list of component names to get dependencies for
#
# RETURNS: A list of components <components> depend on
#
# SEE ALSO: cmpModuleDependsGet, wccDependsGet, wccCurrentGet
#

proc cmpDependsGet {args} {

    global cmpProjHandle

    set cmpList	{}

    # create a components list from input arguments

    foreach arg $args {
	lappend cmpList $arg
    }

    # check for arguments

    if {[llength $cmpList] == 0} {
	error "cmpDependsGet : input component list is empty"
    }

    set wcc	[wccCurrentGet $cmpProjHandle]

    # if there is no Wind Component Configuration, we may be in an application
    # project, just return.

    if {$wcc == ""} {
	return {}
    }

    set cmpDeps	[wccDependsGet $cmpProjHandle $wcc $cmpList]

    return $cmpDeps
}

################################################################################
#
# cmpDependenciesGet - get component dependencies for a list of components
#
# SYNOPSIS
# \cs
#   cmpDependenciesGet <components>
# \ce
#
# DESCRIPTION
# This procedure gets the list of components on which <components> depend.
# It returns a set of two lists. The first list is the list of components on
# which <components> depend, the second is the list of components depending
# on <components>.
#
# PARAMETERS:
#   components : list of component names to get dependencies for
#
# RETURNS: Two lists of components.
#
# ERRORS:
# \is
# \i cmpDependenciesGet : input component list is empty
# The list of <components> should at least contain one component to get
# dependencies for
# \ie
#
# SEE ALSO: cmpModuleDependsGet, wccDependsGet, wccCurrentGet
#

proc cmpDependenciesGet {args {opt ""}} {

    global cmpProjHandle

    set cmpList	{}

    # create a components list from input arguments

    foreach arg $args {
	lappend cmpList $arg
    }

    # check for arguments

    if {[llength $cmpList] == 0} {
	error "cmpDependenciesGet : input component list is empty"
    }

    set wcc	[wccCurrentGet $cmpProjHandle]

    # if there is no Wind Component Configuration, we may be in an application
    # project, just return.

    if {$wcc == ""} {
	return {}
    }

    if { $opt != ""} {
        set cmpDeps	[wccDependsGet $cmpProjHandle $wcc $cmpList "extended"]
    } else {
        set cmpDeps	[wccDependsGet $cmpProjHandle $wcc $cmpList]
    }

    return $cmpDeps
}

################################################################################
#
# cmpModuleDependsGet - get the modules dependencies for <modules>
#
# SYNOPSIS
# \cs
#   cmpModuleDependsGet <modules>
# \ce
#
# PARAMETERS:
#   modules : a list of modules to get module dependencies for
#
# RETURNS: A list of modules with their associated dependencies
#
# ERRORS:
# \is
# \i cmpModuleDependsGet : input module list is empty
# \ie
#
# SEE ALSO: cmpDependsGet, cmpProjTcGet, wccCurrentGet, mxrDataDocCreate,
# modSubtree
#

proc cmpModuleDependsGet {modules} {

    global cmpProjHandle

    set mxrDoc	[mxrDocCreate $cmpProjHandle]
    set modSet	[$mxrDoc setCreate]

    $modSet = $modules

    set subDeps		[format "%s" [mxrSubtree $modSet extended]]
    set superDeps	[format "%s" [mxrSupertree $modSet extended]]

    $modSet delete

    return [list $subDeps $superDeps]
}

################################################################################
#
# cmpBundleListGet - get the list of bundles for the current project
#
# SYNOPSIS
# \cs
#   cmpBundleListGet [<pattern>]
# \ce
#
# DESCRIPTION
# Get the list of bundles for this project.
#
# If the <pattern> parameter is provided, then only the bundles which names
# match <pattern> are returned ('regexp' matching)
#
# PARAMETERS:
#   pattern : bundle name pattern to match
#
# RETURNS: A list of bundles which names match <pattern>
#
# SEE ALSO: cmpBundleAdd, cmpBundleRemove
#

proc cmpBundleListGet {{pattern ""}} {

    global cmpProjHandle

    set tmpBundles	[prjTagDataGet $cmpProjHandle BUNDLE__LIST]

    set bundles		{}

    if {$pattern == ""} {

	# keep all the bundles

	set bundles $tmpBundles

    } else {

	# just keep the bundles matching <pattern>

	foreach bundle $tmpBundles {
	    if {[regexp $pattern $bundle]} {
		lappend bundles $bundle
	    }
	}
    }

    return $bundles
}

################################################################################
#
# cmpBundleExcludedListGet - get the list of component bundles not included
#
# SYNOPSIS
# \cs
#   cmpBundleExcludedListGet [<pattern>]
# \ce
#
# DESCRIPTION
# This procedure returns the list of bundles which are not included by this
# kernel configuration.
#
# It gets the list of all possible bundles, and removes the bundles which are
# currently used.
#
# Only the bundles which name match <pattern> are returned. If <pattern> is not
# provided, all the bundles are returned.
#
# PARAMETERS:
#   pattern : bundle name pattern to match (default is ".*")
#
# RETURNS: The list of component bundles not included by this kernel
# configuration project
#
# ERRORS: N/A
#
# SEE ALSO: cmpBundleListGet, cmpBundleAdd, cmpBundleRemove
#

proc cmpBundleExcludedListGet {{pattern ""}} {

    global cmpProjHandle

    set cxrDoc	[cxrDocCreate $cmpProjHandle]

    # get all the available bundles from the CDF files

    set tmpBundles	[format "%s" [$cxrDoc instances Bundle]]
    set bundles		{}

    if {$pattern == ""} {

	# keep all the bundles

	set bundles $tmpBundles

    } else {

	# just keep the bundles matching <pattern>

	foreach bundle $tmpBundles {
	    if {[regexp $pattern $bundle]} {
		lappend bundles $bundle
	    }
	}
    }

    set included	[cmpBundleListGet]

    # remove the included bundle from the list

    foreach bundle $included {
	if {[set bundleIx [lsearch -exact $bundles $bundle]] != -1} {
	    set bundles	[lreplace $bundles $bundleIx $bundleIx]
	}
    }

    return $bundles
}

################################################################################
#
# cmpBundleGet - get the description of <bundle> bundle
#
# SYNOPSIS
# \cs
#   cmpBundleGet <bundle>
# \ce
#
# DESCRIPTION
# This procedure gets all the properties defined for a Bundle, and creates
# a list where all those properties are listed along with their values.
#
# The returned list is made of <n> elements where <n> is the number of
# properties for a Bundle. Each of those element is made of:
# \is
# \i The property name
# \i The property value
# \ie
#
# PARAMETERS:
#   bundle : name of the bundle to get description of
#
# RETURNS: A list of properties and their associated value for <bundle>
#
# ERRORS:
# \is
# \i Unknown bundle <bundle>
# \ie
#
# SEE ALSO: cmpBundleListGet
#
# NOMANUAL
#

proc cmpBundleGet {bundle} {

    global cmpProjHandle

    set cxrDoc	[cxrDocCreate $cmpProjHandle]
    set cxrSet	[$cxrDoc setCreate]

    # get all the available bundles from the CDF files

    set bundles	[format "%s" [$cxrDoc instances Bundle]]

    if {[lsearch -exact $bundles $bundle] == -1} {
	error "Unknown bundle $bundle"
    }

    $cxrSet = $bundle
    set bundleDesc	{}

    set properties	[$cxrSet properties]

    foreach property $properties {

	lappend bundleDesc	[list $property \
				      [format "%s" [$cxrSet get $property]]]
    }

    return $bundleDesc
}

################################################################################
#
# cmpBundleAdd - add all components of a bundle to the current project
#
# SYNOPSIS
# \cs
#   cmpBundleAdd <bundle>
# \ce
#
# DESCRIPTION
# This procedure gets all the components the given <bundle> handle. It then
# tries to add each of them to the current project.
#
# A list of three lists is returned :
# \is
# \i 'First list' : the list of components which were really added from
#                     the current project.
# \i 'Second list' : The list of components which were ignored because they
#                      were already included by the current project
# \i 'Third list' : The list of erroneous components which were not added
#                     because they are not currently available
# \ie
#
# PARAMETERS:
#   bundle : name of the bundle to add from current project
#
# RETURNS: A list made of three lists, one for the added component, one for the
# ignored components, and one for the erroneous components.
#
# ERRORS:
# \is
# \i Unknown bundle <bundle>
# \ie
#
# SEE ALSO: cmpUnavailListGet, cmpListGet, cmpAdd, cmpBundleListGet,
# cmpBundleRemove
#

proc cmpBundleAdd {bundle} {

    global cmpProjHandle

    set wcc [wccCurrentGet $cmpProjHandle]

    set cxrDoc	[cxrDocCreate $cmpProjHandle]

    # get all the available bundles from the CDF files

    set bundles	[format "%s" [$cxrDoc instances Bundle]]

    if {[lsearch -exact $bundles $bundle] == -1} {
	error "Unknown bundle $bundle"
    }

    set bundleList	[prjTagDataGet $cmpProjHandle BUNDLE__LIST]

    # create lists of the already included components, and unavailable
    # components.

    set cxrSet	[$cxrDoc setCreate]
    $cxrSet = $bundle
    set unavailComps	[cmpUnavailListGet]
    set includedComps	[cmpListGet]
    set bundleComps	[format "%s" [$cxrSet get COMPONENTS]]
    set l_excl		[cmpListGet softexclude]

    # keep track of the components which will be added, ignored or erroneous

    $cxrSet = $bundleComps
    set ignoredComps	[$cxrSet & $includedComps]
    set errorComps	[$cxrSet & $unavailComps]
    set allowedComps	[$cxrSet - $errorComps]
    $cxrSet = $allowedComps
    set addedComps	[$cxrSet - $ignoredComps]

    # update soft exclude if required
    cxrRequires $cmpProjHandle $cxrSet 0
    set noexcl [$cxrSet & $l_excl]
    if {$noexcl != {}} {
	$cxrSet = $l_excl
	wccComponentSet $cmpProjHandle $wcc same same [$cxrSet - $noexcl]
    }


    $cxrSet delete

    # add this to the bundle list of the project file
    # check if bundle aready included
    if {[lsearch -exact $bundleList $bundle] != -1} {
	puts [cmpStdout] "bundle $bundle already included"
    } else {
	prjTagDataSet	$cmpProjHandle BUNDLE__LIST [concat $bundleList $bundle]
    }

    # check for container conflicts
    set explicit_l [wccComponentListGet $cmpProjHandle $wcc explicit]
    set rm_ex [cxrCheckContainerConflicts $cmpProjHandle $explicit_l $allowedComps]

    if {$rm_ex != {} } {
	cmpRemove $rm_ex noForce
        set addedComps $allowedComps
    } else {
	# recalculate configuration. The function will update the implicit component list.
	set lists [cmpRecalculate $cmpProjHandle]
        set addedComps [lindex $lists 0]
    }


    return [list $addedComps $ignoredComps $errorComps]
}

################################################################################
#
# cmpBundleRemove - remove all the components of given <bundle>
#
# SYNOPSIS
# \cs
#   cmpBundleRemove <bundle>
# \ce
#
# DESCRIPTION
# This procedure gets the list of components handled by <bundle>.
#
# If <bundle> does not exist, 'cmpBundleRemove' exits on error (see the ERROR
# section).
#
# A list of three lists is returned :
# \is
# \i 'First list' : The list of components which were really removed from
#                     the current project
# \i 'Second list' : The list of components which were ignored because they
#                      were not included by the current project
# \i 'Third list' : The list of erroneous components which were not removed
#                     because they are not currently available (thus they were
#                     also not included)
# \ie
#
# PARAMETERS:
#   bundle : the bundle which components should be removed from current project
#
# RETURNS: A list made of removed, ignored and erroneous components.
#
# ERRORS:
# \is
# \i Unknown bundle <bundle>
# \ie
#
# SEE ALSO: cmpUnavailListGet, cmpListGet, cmpRemove, cmpBundleAdd,
# cmpBundleListGet
#

proc cmpBundleRemove {bundle} {

    global cmpProjHandle

    set cxrDoc	[cxrDocCreate $cmpProjHandle]
    set cxrSet	[$cxrDoc setCreate]

    # get all the available bundles from the CDF files

    set bundles	[format "%s" [$cxrDoc instances Bundle]]

    if {[lsearch -exact $bundles $bundle] == -1} {
	error "Unknown bundle $bundle"
    }

    set bundleList	[prjTagDataGet $cmpProjHandle BUNDLE__LIST]

    # check if bundle aready included
    if {[lsearch -exact $bundleList $bundle] == -1} {
	puts [cmpStdout] "bundle $bundle not included"
	return {{} {} {}}
    }

    $cxrSet = $bundle

    set unavailComps	[cmpUnavailListGet]
    set includedComps	[cmpListGet]
    set bundleComps	[format "%s" [$cxrSet get COMPONENTS]]

    # we will have to keep in mind the components which will be removed,
    # ignored or erroneous

    set ignoredComps	{}
    set errorComps	{}
    set removedComps	{}

    foreach comp $bundleComps {

	if {[lsearch -exact $unavailComps $comp] != -1} {

	    # this component is not available, it should not be removed ;)

	    lappend errorComps	$comp

	} elseif {[lsearch -exact $includedComps $comp] == -1} {

	    # component is not in the included components, it has not to be
	    # removed

	    lappend ignoredComps	$comp
	}
    }

    # remove this from the bundle list of the project file

    if {[set bundleIx [lsearch -exact $bundleList $bundle]] != -1} {
	set bundleList	[lreplace $bundleList $bundleIx $bundleIx]
	prjTagDataSet	$cmpProjHandle BUNDLE__LIST $bundleList
    }

    set lists [cmpRecalculate $cmpProjHandle]

    set removedComps [lindex $lists 1]

    $cxrSet delete

    return [list $removedComps $ignoredComps $errorComps]
}

################################################################################
#
# cmpProfileInitSet - set the profile the current project was created with
#
# SYNOPSIS
# \cs
#   cmpProfileInitSet <profile>
# \ce
#
# DESCRIPTION
# This procedure allows to save the name of the profile this kernel component
# configuration project was initialised with (if any).
#
# The given <profile> must be one of the profiles of the list returned by
# 'cmpProfileListGet'.
#
# PARAMETERS:
#   profile : name of the profile to set as the initialisation profile
#
# RETURNS: N/A
#
# ERRORS:
# \is
# \i No such profile : <profile>
# The profile given as an argument is not a member of the profile list returned
# by 'cmpProfileListGet'
# \ie
#
# SEE ALSO: cmpProfileListGet
#

proc cmpProfileInitSet {profile} {

    global cmpProjHandle

    # check if it is a valid profile

    set profiles	[cmpProfileListGet]

    if {[lsearch -exact $profiles $profile] == -1} {
	error "No such profile : $profile"
    }

    # now we can update the project tag

    prjTagDataSet $cmpProjHandle INITIAL_PROFILE $profile

    return
}

################################################################################
#
# cmpProfileInitGet - get the profile the current project was created with
#
# SYNOPSIS
# \cs
#   cmpProfileInitGet
# \ce
#
# DESCRIPTION
# This procedure gets the name of the profile the current kernel configuratio
# project was created with (if any).
#
# PARAMETERS: N/A
#
# RETURNS: The name of the initial profile used to create the current
# kernel configuration, or an empty String if no profile was used.
#
# ERRORS: N/A
#
# SEE ALSO: cmpProfileListGet
#

proc cmpProfileInitGet {} {

    global cmpProjHandle
    return [prjTagDataGet $cmpProjHandle INITIAL_PROFILE]
}

proc cmpFsFileAdd {targetPath hostPath {buildRule ""}} {

    global cmpProjHandle

    prjFsFileAdd $cmpProjHandle $targetPath $hostPath $buildRule
}

proc cmpFsFileGet {targetPath} {

    global cmpProjHandle

    return [prjFsFileGet $cmpProjHandle $targetPath]
}

proc cmpFsFileRemove {targetPath} {

    global cmpProjHandle

    # first check if file is in the list of Target filesystem files

    if {[lsearch -exact [prjFsFileListGet $cmpProjHandle] $targetPath] != -1} {

	if {! [catch {prjFsFileRemove $cmpProjHandle $targetPath} status]} {

	    return 1
	}
    }

    return 0
}

proc cmpFsFileListGet {{pattern ".*"}} {

    global cmpProjHandle

    return [prjFsFileListGet $cmpProjHandle $pattern]
}

proc cmpFsFileCreate {} {

    global cmpProjHandle

    # XXX : fle : for now, only romfs is supported, check if the component is
    # included if this procedure is called.

    set components	[cmpListGet]

    if {([lsearch -exact $components INCLUDE_ROMFS] == -1)} {

	cmpAdd INCLUDE_ROMFS
    }

    return [prjFsFileCreate $cmpProjHandle]
}

################################################################################
#
# cmpBspListGet - get the list of all available BSP
#
# SYNOPSIS
# \cs
#   cmpBspListGet [<bspDir>]
# \ce
#
# DESCRIPTION
# Thie procedure returns a list of all the BSPs the current install contains.
#
# If <bspDir> is not specified, the default BSP directory is set to
# 'Wind::tgtDir'/config
#
# All directories containing a 'Makefile' in the BSP directory are considered
# valid BSPs, and added to the BSP list
#
# PARAMETERS:
#   bspDir : the directory to look for BSP list. Default is 'Wind::tgDir'/config
#
# RETURNS: A list of BSPs
#
# ERRORS: N/A
#
# SEE ALSO: Wind::tgtDir, Wind::bspList
#

proc cmpBspListGet {{bspDir ""}} {

    return [Wind::bspList $bspDir]
}

################################################################################
#
# cmpToolListGet - get a the toolchain list for a given BSP
#
# SYNOPSIS
# \cs
#   cmpToolListGet <bsp>
# \ce
#
# DESCRIPTION
# This procedure gets the list of available toolchains for a given BSP.
#
# It gets the CPU of that BSP by parsing the BSP Makefile, looking for the
# 'CPU = XXX' string (ignoring those in comments).
#
# Once the CPU is retrieved, it looks for the make.<cpu><tool> files in the
# TGT_DIR/h/tool/<toolfamily> directories.
#
# PARAMETERS:
#   bsp : the bsp to look toolchain list for. It may be a full path directory.
#
# RETURNS: The list of tools available for the given <bsp>
#
# ERRORS:
# \is
# \i Invalid bsp <bsp> : could not find a Makefile
# No Makefile was found in the <bsp> directory
# \i Could not read <makefile> : <status>
# The <makefile> file could not be read, <status> should explain why
# \i Could not get CPU from <makefile>
# The CPU macro value could not be retrieved from <makefile>
# \ie
#
# SEE ALSO: Wind::tgtDir, Wind::bspInfo
#

proc cmpToolListGet {bsp} {

    return [lindex [Wind::bspInfo $bsp] 2]
}

################################################################################
#
# cmpBuildRuleListGet - get the list of build rules for the current build spec
#
# SYNOPSIS
# \cs
#   cmpBuildRuleListGet [<pattern>]
# \ce
#
# DESCRIPTION
# Returns the list of possible build rule names for the current build
# specification
#
# If <pattern> is provided, only the build rules which names match <pattern>
# are returned, ales, all rule names are returned.
#
# PARAMETERS:
#   pattern : build rule name pattern to match (default, all)
#
# RETURNS: The list of build rule names for the current build specification.
#
# ERRORS: N/A
#
# SEE ALSO: cmpBuildRule, cmpBuildRuleDefault, cmpBuildRuleRemove
#

proc cmpBuildRuleListGet {{pattern ".*"}} {

    global cmpProjHandle

    set hSpec	[prjBuildCurrentGet $cmpProjHandle]
    set rules	[lsort [prjBuildRuleListGet $cmpProjHandle $hSpec]]

    set current	[prjBuildRuleCurrentGet $cmpProjHandle $hSpec]
    set res	{}

    # only return the required rules, and always put the default rule first

    foreach rule $rules {

	if {[regexp $pattern $rule]} {

	    if {$rule == $current} {
		set res	[linsert $res 0 $rule]
	    } else {
		lappend res $rule
	    }
	}
    }

    return $res
}

################################################################################
#
# cmpBuildRuleAdd - add a new build rule to the current build specification
#
# SYNOPSIS
# \cs
#   cmpBuildRuleAdd <name> <dependencies> <commands>
# \ce
#
# DESCRIPTION
# Add a new build rule to the current build specification for the current
# kernel configuration project
#
# PARAMETERS:
#   name : name of the rule to add
#   dependencies : dependencies the rule should fullfill before being run
#   commands : list of commands to be run by this build rule
#
# RETURNS: N/A
#
# ERRORS: N/A
#
# SEE ALSO: cmpBuildRuleListGet, cmpBuildRuleRemove
#

proc cmpBuildRuleAdd {name dependencies commands} {

    global cmpProjHandle

    # set the command list to an empty list if needed

    set hSpec	[prjBuildCurrentGet $cmpProjHandle]

    if {$commands == ""} {
	set commands	{}
    }

    # the set command should ceate the new rule

    prjBuildRuleSet	$cmpProjHandle $hSpec $name \
			[buildRuleJoin $name $dependencies $commands]
}

################################################################################
#
# cmpBuildRule - get or set a build rule
#
# SYNOPSIS
# \cs
#   cmpBuildRule <name> [<dependencies>] [<commands>]
# \ce
#
# DESCRIPTION
# This procedure allows to get or set a given build rule. If the build rule
# named <name> does not exist yet, it is added if <dependencies> and <commands>
# are provided (aka either <dependencies> or <commands> are not set to ""). To
# add a new rule with empty values, please see 'cmpBuildRuleAdd'
#
# If <dependencies> is provided, the dependencies for the rule named <name> are
# updated, and returned in the result. If no dependencies are provided, the
# existing ones are just kept for the build rule.
#
# If <commands> is provided, the build rule commands are updated and returned
# in the result. If no command list is provided, the existing ones are just kept
# for the current build rule.
#
# PARAMETERS:
#   name : name of the build rule to get (or set)
#   dependencies : dependencies to set for the build rule
#   commands : a list of commands to set for the build rule
#
# RETURNS: A list made of {name deps {cmd1 cmd2 ...}}
#
# ERRORS: N/A
#
# SEE ALSO: cmpBuildRuleListGet, cmpBuildRuleAdd
#

proc cmpBuildRule {name {dependencies ""} {commands ""}} {

    global cmpProjHandle

    set hSpec	[prjBuildCurrentGet $cmpProjHandle]
    set value	[buildRuleSplit [prjBuildRuleGet $cmpProjHandle $hSpec $name]]
    set modify	0

    if {$value == ""} {
	set value	[list $name "" ""]
    }

    # if some dependencies have been given, we should update the rule later

    if {$dependencies != ""} {
	set modify	1
    } else {
	set dependencies	[lindex $value 1]
    }

    # check for new commands

    if {$commands != ""} {
	set modify	1
    } else {
	set commands		[lindex $value 2]
    }

    # modify the build rule if needed

    if {$modify} {
	prjBuildRuleSet	$cmpProjHandle $hSpec $name \
			[buildRuleJoin $name $dependencies $commands]
	set value	[buildRuleSplit [prjBuildRuleGet $cmpProjHandle $hSpec \
							 $name]]
    }

    return $value
}

################################################################################
#
# cmpBuildRuleRemove - remove a build rule from the current build specification
#
# SYNOPSIS
# \cs
#   cmpBuildRuleRemove <name>
# \ce
#
# DESCRIPTION
# This procedure removes the build rule named <name> from the current build
# specification
#
# PARAMETERS:
#   name : name of the build rule to remove
#
# RETURNS: the removed build rule
#
# ERRORS:
# \is
# \i Could not remove build rule <name> : <status>
# There is no build rule named <name>. The <status> should give more information
# on the problem encountered.
# \ie
#
# SEE ALSO: cmpBuildRule
#

proc cmpBuildRuleRemove {name} {

    global cmpProjHandle

    set hSpec	[prjBuildCurrentGet $cmpProjHandle]

    if {[catch {prjBuildRuleGet $cmpProjHandle $hSpec $name} value]} {

	error "Could not remove build rule $name : $value"
    }

    # remove build rule from current kernel configuration

    prjBuildRuleDelete $cmpProjHandle $hSpec $name

    return $value
}

################################################################################
#
# cmpBuildRuleDefault - get or set the current build rule
#
# SYNOPSIS
# \cs
#   cmpBuildRuleDefault [<name>]
# \ce
#
# DESCRIPTION
# Return the current build rule for the current build specification
#
# If <name> is provided, and <name> belongs to the list of build rules (as
# returned by 'cmpBuildRuleListGet'), the current build rule is then set
# to <name>, and <name> is returned.
#
# If <name> is not part of the possible build rules, a warning is logged
# on the standard error.
#
# PARAMETERS:
#   name : name of the build rule to set as the current build rule for the
#          current build specification
#
# RETURNS: the name of the current build rule for the current build
# specification
#
# ERRORS: N/A
#
# SEE ALSO: cmpBuildRuleListGet
#

proc cmpBuildRuleDefault {{name ""}} {

    global cmpProjHandle

    set hSpec	[prjBuildCurrentGet $cmpProjHandle]
    set res	[prjBuildRuleCurrentGet $cmpProjHandle $hSpec]

    if {$name != ""} {

	# first check that the specified rule name is in the rules list

	set rulesList	[prjBuildRuleListGet $cmpProjHandle $hSpec]

	if {[lsearch -exact $rulesList $name] != -1} {
	    prjBuildRuleCurrentSet $cmpProjHandle $hSpec $name
	    set res $name
	} else {
	    prjWarnLog "Rule $name does not belong to the possible rules :\
			$rulesList"
	}
    }

    return $res
}

################################################################################
#
# cmpStdout - get or set this library standard output
#
# SYNOPSIS
# \cs
#   cmpStdout [<channel>]
# \ce
#
# DESCRIPTION
# Get the standard output used by this library.
#
# If <channel> is set the standard output of this library is set to <channel>,
# else, it is set to the default 'stdout'
#
# PARAMETERS:
#   channel : a writable channel as returned by Tcl 'open' procedure
#
# RETURNS: The current standard output channel for this Tcl library
#
# ERRORS: N/A
#
# SEE ALSO: prjStdout
#
# NOMANUAL
#

proc cmpStdout {{channel ""}} {

    return [prjStdout $channel]
}

################################################################################
#
# cmpStderr - get or set this library standard error
#
# SYNOPSIS
# \cs
#   cmpStderr [<channel>]
# \ce
#
# DESCRIPTION
# Get the standard error used by this library.
#
# If <channel> is set the standard error of this library is set to <channel>,
# else, it is set to the default 'stderr'
#
# PARAMETERS:
#   channel : a writable channel as returned by Tcl 'open' procedure
#
# RETURNS: The current standard error channel for this Tcl library
#
# ERRORS: N/A
#
# SEE ALSO: prjStderr
#
# NOMANUAL
#

proc cmpStderr {{channel ""}} {

    return [prjStderr $channel]
}

################################################################################
#
# cmpParamRemove - reset a configuration parameter
#
# SYNOPSIS
# \cs
#   cmpParamRemove <param>
# \ce
#
# DESCRIPTION
# Remove the definition value of a configuration parameter. The specific value
# of the <param> configuration parameter is removed from the current kernel
# configuration.
#
# This is equivalent to '#undef <param>' in a config.h
#
# PARAMETERS:
#   param : name of the parameter which definition should be removed fromt the
#           current kernel configuration
#
# RETURNS: N/A
#
# ERRORS: N/A
#
# SEE ALSO: cmpParamSet
#

proc cmpParamRemove {param} {

    global cmpProjHandle

    set wcc [wccCurrentGet $cmpProjHandle]

    # if there is no Wind Component Configuration, we may be in an application
    # project, just return.

    if {$wcc == ""} {
	return
    }

    wccParamRemove $cmpProjHandle $wcc $param

    return
}

################################################################################
#
# cmpMakefile - get or set the current project Makefile name
#
# SYNOPSIS
# \cs
#   cmpMakefile [<makefile>]
# \ce
#
# DESCRIPTION
# This procedure allows to set or get the current project's Makefile name.
#
# The default value is 'Makefile', but it can be changed, and the generated
# Makefile for the current project will reflect this
#
# PARAMETERS
#   makefile : name of the Makefile to give to the current project
#
# RETURNS: The current Makefile name of the current kernel configuration project
#
# EXAMPLES:
# Create a dedicated Makefile for debug :
# \cs
# cmpMakefile Makefile.debug
# \ce
#
# ERRORS: N/A
#
# SEE ALSO: N/A
#

proc cmpMakefile {{makefile ""}} {

    global cmpProjHandle

    # if a <makefile> is given, remove old one, so that there are no more
    # traces of this name change

    if {$makefile != ""} {

	set makeDir	[file dirname [prjInfoGet $cmpProjHandle fileName]]

	if {$makeDir == "."} {
	    set makeDir	[pwd]
	}

	set makePath	[file join $makeDir [prjMakefile $cmpProjHandle]]
	catch {file delete -force $makePath} status
    }

    return [prjMakefile $cmpProjHandle $makefile]
}

################################################################################
#
# cmpProfileListGet - get the list of possible profiles
#
# SYNOPSIS
# \cs
#   cmpProfileListGet [<pattern>]
# \ce
#
# DESCRIPTION
# Get the list of profiles defined by the CDF files.
#
# The list of profiles is retrieved from the CDF files. If no project is
# currently opened, the CDF files located in the vxWorks components repository
# are used. If a project is opened, then the CDF files used by this project
# are used.
#
# If <pattern> is provided, only profiles which name matches <pattern> are
# returned.
#
# PARAMETERS:
#   pattern : profile name pattern to match
#
# RETURNS: The list of profiles matching <pattern>
#
# ERRORS: N/A
#
# SEE ALSO: cmpProjOpen
#

proc cmpProfileListGet {{pattern ""}} {

    global cmpProjHandle

    set profileList	{}

    if {$cmpProjHandle == ""} {

	# create a cxrDoc using the CDF files in TGT_DIR/config/comps/vxWorks

	set cdfDir	[file join [Wind::tgtDir] config comps vxWorks]
	set cdfList	[glob -nocomplain [file join $cdfDir *.cdf]]
	set cxrDoc	[docCreate CxrDoc]

	foreach cdfFile $cdfList {
	    $cxrDoc import $cdfFile
	}

    } else {

	# get the CDF path from the wccCdfPathGet procedure

	set cxrDoc		[cxrDocCreate $cmpProjHandle]
    }

    set tmpProfileList	[format "%s" [$cxrDoc instances Profile]]

    # filter on the name pattern if needed

    if {$pattern == ""} {
	set profileList	$tmpProfileList
    } else {
	foreach profile $tmpProfileList {
	    if {[regexp -- $pattern $profile]} {
		lappend profileList	$profile
	    }
	}
    }

    return $profileList
}

################################################################################
#
# cmpProfileGet - get the properties of the profile named <name>
#
# SYNOPSIS
# \cs
#   cmpProfileGet <name>
# \ce
#
# DESCRIPTION
# Get the given profile properties.
#
# If a project is currently opened, the profile is retrived from the CDF
# files visible from the project, else, it is retrived from the default
# kernel component CDF directory.
#
# A list of profile properties is returned. First elements is the name of the
# profile itself (should be equal to <name>), then a list of properties.
#
# Each of the properties elements is made of :
# \is
# \i The property name as it appears in the CDF file
# \i The property value as it appears in the CDF file
# \ie
#
# PARAMETERS:
#   name : name of the profile to get properties of
#
# RETURNS: A profile property list, as described above
#
# ERRORS:
# \is
# \i No such profile
# The profile specified does not exist
# \ie
#
# SEE ALSO: cmpProfileListGet
#

proc cmpProfileGet {name} {

    global cmpProjHandle

    set propList	$name

    # now depending on the project handle, create a CxrDoc from wcc, or go
    # and read the CDF files directly

    if {$cmpProjHandle == ""} {

	# create a cxrDoc using the CDF files in TGT_DIR/config/comps/vxWorks

	set cdfDir	[file join [Wind::tgtDir] config comps vxWorks]
	set cdfList	[glob -nocomplain [file join $cdfDir *.cdf]]
	set cxrDoc	[docCreate CxrDoc]

	foreach cdfFile $cdfList {
	    $cxrDoc import $cdfFile
	}

    } else {

	# get the CDF path from the wccCdfPathGet procedure

	set cxrDoc		[cxrDocCreate $cmpProjHandle]
    }

    # check that the given profile is in the list

    if {[lsearch -exact [format "%s" [$cxrDoc instances Profile]] \
		 $name] == -1} {

	error "No such profile"
    }

    # create a Cxr set to get the profile properties

    set cxrSet	[$cxrDoc setCreate]
    $cxrSet = $name
    foreach property [format "%s" [$cxrSet properties]] {
	lappend propList [list $property [format "%s" [$cxrSet get $property]]]
    }

    $cxrSet delete

    return $propList
}

################################################################################
#
# cmpOsName - get or set the name of the OS the current project was created for
#
# SYNOPSIS
# \cs
#   cmpOsNameGet [<osName>]
# \ce
#
# DESCRIPTION
# Get the name of the Operating System the current kernel configuration project
# was created for
#
# If the <osName> parameter is provided, the os name is set to <osName>, and
# the proedure returns <osName>. Any subsequent call to 'cmpOsName' will
# return <osName> unless a new os name is set.
#
# PARAMETERS:
#   osName : name of the os to set for the current kernel configuration project
#
# RETURNS: The name of the Operating System the current kernel configuration
# project was created for
#
# ERRORS: N/A
#

proc cmpOsName {{osName ""}} {

    global cmpProjHandle

    if {$osName != ""} {
	prjTagDataSet $cmpProjHandle OS_NAME $osName
    }

    return [prjTagDataGet $cmpProjHandle OS_NAME]
}

################################################################################
#
# cmpSizeGet - get the size of a component or a component list
#
# SYNOPSIS
# \cs
#   cmpSizeGet [<component> [<component> ...]]
# \ce
#
# DESCRIPTION
# Get the 'text', 'data', 'bss' sizes of a kernel configuration component or
# list of components.
#
# If no <component> is provided, the whole set of kernel condifguration
# components is used (as returned by 'cmpListGet')
#
# PARAMETERS:
#   component : name of a kernel component to get text/data/bss sizes for
#
# RETURNS: A list made of three elements : text size, data size and bss size
# for the given component(s)
#
# ERRORS: N/A
#
# SEE ALSO: cmpListGet, cxrSizeGet
#

proc cmpSizeGet {args} {

    global cmpProjHandle

    set cxrDoc	[cxrDocCreate $cmpProjHandle]
    set mxrDoc	[mxrDocCreate $cmpProjHandle]
    set cxrSet	[$cxrDoc setCreate]

    # gather the list of components

    if {[llength $args] > 0} {
	set components	$args
    } else {
	set components	[cmpListGet]
    }

    $cxrSet = $components
    set cmpSize	[cxrSizeGet $cxrSet $mxrDoc]

    $cxrSet delete

    return $cmpSize
}

################################################################################
#
# cmpProfileAdd - add a profile to a project configuratin
#
# SYNOPSIS
# \cs
#   cmpProfileAdd <project handle> <profile name>
# \ce
#
# DESCRIPTION
# Add all components defined by the profile to the project cofiguration
#
# PARAMETERS:
#   project handle: 
#   profile name: name of the profile to use
#
# RETURNS: no return value
#
# ERRORS: N/A
#
#
proc cmpProfileAdd { hProj profile } {

    proc nestedProfiles {profiles bag} {
	if {$profiles == {}} { return {} }
	$bag = $profiles
	$bag = [$bag instances Profile]
	$bag = [join [$bag get PROFILES]]
	set new [$bag - $profiles]

	return [concat $profiles [nestedProfiles $new $bag]]
    }

    set cxrDoc [cxrDocCreate $hProj]
    set bag [$cxrDoc setCreate]

    # get components, nested profiles and bundles
    set profiles [nestedProfiles $profile $bag]

    $bag = $profiles
    $bag = [$bag get COMPONENTS]
    set components [$bag instances Component]
    $bag = $profiles
    $bag = [join [$bag get BUNDLES]]
    set bundles [$bag instances Bundle]

    $bag = $profiles
    $bag = [join [$bag get PROFILES]]
    if {[$bag & BSP_DEFAULT] != {} } {
	set bspComponents [cmpDefaultProfileGet $hProj]
	if {$bspComponents != {}} {
	    cmpAdd $bspComponents
	}
    }

    if {$components != {}} {
	cmpAdd $components
    }
    foreach bundle $bundles {
	cmpBundleAdd $bundle
    }

    $bag delete
}

################################################################################
#
# cmpDefaultProfileGet - get the list of components defained as the default
# for a given Boars Support Package
#
# SYNOPSIS
# \cs
#   cmpDefaultProfileGet <project hanlde> [ <BSP name> ]
# \ce
#
# DESCRIPTION
# calculate and return the list of components to be included by default.
#
# This function uses the DEFAULTS attribute from the Bsp type.
# If no instance of this type exist with the BSP name, and empty list is returned
# This case is valid and is expected for legacy BSPs for which the tool retrieves
# default configuration by parsing BSP source files.
#
# PARAMETERS:
#   project handle
#   BSP name: the name of the BSP to use. When to value is provided, the function
#   will retrieve the value BSP_DIR from the project and use it as a BSP name
#
# RETURNS: A list of components. The list may be empty.
#
# ERRORS: N/A
#
#
proc cmpDefaultProfileGet { hProj {bsp "" } } {

    if {$bsp == ""} {
	set bsp_dir [prjTagDataGet $hProj BSP_DIR]
	set bsp [file tail $bsp_dir]
    }

    set cxrDoc [cxrDocCreate $hProj]
    set s [$cxrDoc setCreate]

    $s = $bsp

    if {[$s instances] == {}} {
	return ""
    }

    $s = [$s get DEFAULTS]

    set def [$s instances Component]

    return $def
}


source [file join [Wind::base] host resource osconfig tcl prjLib.tcl]
source [file join [Wind::base] host resource osconfig tcl dataDocLib.tcl]
