# 00wnCANBuild.win32.tcl - TCL file to build/clean wnCAN libraries
#
# Copyright (C) 1995-2002 Wind River Systems, Inc.
#
# modification history
# -----------------------
# 01b,19jun03,nhc  Fix the search for VX_CPU_FAMILY on mips and xscale
# 01b,19mar03,nhc  Correct optimization options for diab & handle diable/gnule
# 01a,14oct02,edc  written
#

#################################################################################
# DESCRIPTION																	#
#	This file contains scripts to build/clean wnCAN libraries.					#
#																				#
#################################################################################

menuItemAppend \
	-popup -name wnCANMenu \
	-bypath {&Tools} {&wnCAN}

menuItemAppend \
	wnCANMenu -callback {wnCANBuildDlg "Build wnCAN Library"} \
	"&Build wnCAN Library..." 
menuItemAppend \
	wnCANMenu -callback {wnCANCleanDlg "Clean/Remove wnCAN Library"} \
	"&Clean/Remove wnCAN Library..."

#################################################################################
#
# wnCANToolChainCallback - callback procedure to set the correct levels of
# optimization for diab and gnu
#################################################################################

proc wnCANToolChainCallback {} {
   set toolchainSelected \
      [controlSelectionGet wnCANBuildDlg.toolchainCombo -string]
   set optLevelSelected \
      [controlSelectionGet wnCANBuildDlg.optLevelCombo -string]


   #find out if the tool selected is gnu or not because the optimization
   #options are different for gnu and diab.

   #for diab the options are -O (for level 1) -XO (for level 2)
   #for gnu the options are -O0 (for level 0) -O1 (for level 1) 
   #                        -O2 (for level 2) -O3 (for level 3)
   if {[string last gnu [string tolower $toolchainSelected]] > 0} {
      controlValuesSet wnCANBuildDlg.optLevelCombo {none 1 2 3}
      controlSelectionSet wnCANBuildDlg.optLevelCombo -string $optLevelSelected
   } elseif {[string last diab [string tolower $toolchainSelected]] > 0} {
      #diab compiler has only 3 levels (none, 1, 2)
      controlValuesSet wnCANBuildDlg.optLevelCombo {none 1 2}
      if {$optLevelSelected != "3"} {
         controlSelectionSet wnCANBuildDlg.optLevelCombo -string $optLevelSelected
      } else {
         controlSelectionSet wnCANBuildDlg.optLevelCombo -string 2
      }
   }

}



#################################################################################
#												
# wnCANIncludeDebugInfoCallback - callback procedure to set default optimization
# level for standard (-O2) and debug (-O0)						
#################################################################################

proc wnCANIncludeDebugInfoCallback {} {
	set optLevelSelected \
            [controlSelectionGet wnCANBuildDlg.optLevelCombo -string]

	set incDebugButtonValue [controlChecked wnCANBuildDlg.incDebugButton]

	if {$incDebugButtonValue == 1} {
		if {$optLevelSelected != "none"} {
			controlSelectionSet wnCANBuildDlg.optLevelCombo -string none
		}
	} else {
		if {$optLevelSelected != "2"} {
			controlSelectionSet wnCANBuildDlg.optLevelCombo -string 2
		}
	}
}


#################################################################################
# wnCANBuildDlg - callback procedure to create the Build dialog box.			#
#################################################################################

proc wnCANBuildDlg {title} {
	dialogCreate \
		-name wnCANBuildDlg -title $title \
		-width 220 -height 104 \
		-nocontexthelp \
		-init {::wnCANBuildDlgInit} \
		-controls \
			[list \
				[list label -name toolchainsComboLbl \
					-title "Toolchain:" \
					-xpos 10 -ypos 10 -width 200 -height 10] \
				[list combo -name toolchainCombo \
                              -callback wnCANToolChainCallback \
					-xpos 20 -ypos 25 -width 180 -height 70] \
				[list boolean -auto -name incDebugButton \
			        -title "Include Debug Info" \
					-callback wnCANIncludeDebugInfoCallback \
					-xpos 20 -ypos 46 -width 80 -height 10] \
				[list label -name optLevelComboLbl \
					-title "Optimization Level:" \
					-xpos 100 -ypos 47 -width 65 -height 10] \
				[list combo -name optLevelCombo \
					-xpos 165 -ypos 45 -width 35 -height 70] \
				[list boolean -auto -name rebuildAllButton \
			        -title "Rebuild All" \
					-xpos 20 -ypos 65 -width 80 -height 10] \
				[list button -name buildOkButton \
					-callback wnCANBuildOK \
					-title "Build" \
					-xpos 55 -ypos 84 -width 50 -height 15] \
				[list button -name buildCancelButton \
					-callback "windowClose wnCANBuildDlg" \
					-title "Cancel" \
					-xpos 115 -ypos 84 -width 50 -height 15] \
			]
}


#################################################################################
# wnCANCleanDlg - callback procedure to create the Clean dialog box.			#
#################################################################################

proc wnCANCleanDlg {title} {
	dialogCreate \
		-name wnCANCleanDlg -title $title \
		-width 220 -height 105 \
		-nocontexthelp \
		-init {::wnCANCleanDlgInit} \
		-controls \
			[list \
				[list label -name toolchainsComboLbl \
					-title "Toolchain:" \
					-xpos 10 -ypos 10 -width 200 -height 10] \
				[list combo -name toolchainCombo \
					-xpos 20 -ypos 25 -width 180 -height 70] \
				[list choice -auto -name stdLibChoice \
					-title "Clean/Remove Standard wnCAN Library" \
					-xpos 35 -ypos 50 -width 150 -height 10] \
				[list choice -auto -name dbgLibChoice \
					-title "Clean/Remove Debug wnCAN Library" \
					-xpos 35 -ypos 65 -width 150 -height 10] \
				[list button -name cleanOkButton \
					-callback wnCANCleanOK \
					-title "Clean" \
					-xpos 55 -ypos 85 -width 50 -height 15] \
				[list button -name cleanCancelButton \
					-callback "windowClose wnCANCleanDlg" \
					-title "Cancel" \
					-xpos 115 -ypos 85 -width 50 -height 15] \
			]
}


#################################################################################
# wnCANBuildDlgInit - callback procedure called when the Build dialog is		#
# initialized.																	#
#################################################################################

proc wnCANBuildDlgInit {} {
    
	# Get Tool Chain List

	set toolChains [::prj_vxApp::toolChainListGet]
	foreach toolchain $toolChains {
		if {[${toolchain}::name] != "SIMNTgnu" && [${toolchain}::name] != "SIMNTdiab"} {
			lappend tc_names [${toolchain}::name]
		}
	}

	set tc_names [lsort -decreasing $tc_names]
	set tc_def_name [lindex $tc_names 0]

	controlValuesSet wnCANBuildDlg.toolchainCombo $tc_names
	controlSelectionSet wnCANBuildDlg.toolchainCombo -string $tc_def_name

	controlValuesSet wnCANBuildDlg.optLevelCombo {none 1 2 3}
	controlSelectionSet wnCANBuildDlg.optLevelCombo -string 2

}


#################################################################################
# wnCANCleanDlgInit - callback procedure called when the Clean dialog is		#
# initialized.																	#
#################################################################################

proc wnCANCleanDlgInit {} {
    
	# Get Tool Chain List

	set toolChains [::prj_vxApp::toolChainListGet]
	foreach toolchain $toolChains {
		if {[${toolchain}::name] != "SIMNTgnu" && [${toolchain}::name] != "SIMNTdiab"} {
			lappend tc_names [${toolchain}::name]
		}
	}

	set tc_names [lsort -decreasing $tc_names]
	set tc_def_name [lindex $tc_names 0]

	controlValuesSet wnCANCleanDlg.toolchainCombo $tc_names
	controlSelectionSet wnCANCleanDlg.toolchainCombo -string $tc_def_name

	controlCheckSet wnCANCleanDlg.stdLibChoice 0
	controlCheckSet wnCANCleanDlg.dbgLibChoice 1

}

#################################################################################
# canlookForCPUFamily - Looks for the value of VX_CPU_FAMILY string in the file  
#################################################################################

proc canlookForCPUFamily {filename} {   
   set make_fd [open  $filename r]
   fconfigure $make_fd -translation lf

   set tc_cpu_family ""
   while {[gets $make_fd line] >= 0} {
      set line [string trim $line]
      set line [string toupper $line]
      if {[string range $line 0 12] == "VX_CPU_FAMILY"} {         
         set str_length [string length $line]
         set text [string range $line 13 [expr $str_length - 1]]
         set idx [string first "=" $text]
         set text [string range $text [expr $idx + 1] [expr $str_length - 1]]
         set tc_cpu_family [string trim $text]
         set tc_cpu_family [string tolower $tc_cpu_family]
      }
   }
   close $make_fd   
   return $tc_cpu_family
}


#################################################################################
# wnCANBuildOK - callback procedure called when the user clicks the "Build"		#
# button in the Build dialog.													#
#################################################################################

proc wnCANBuildOK {} {

   set optLevelSelected \
      [controlSelectionGet wnCANBuildDlg.optLevelCombo -string]
   set toolchainSelected \
      [controlSelectionGet wnCANBuildDlg.toolchainCombo -string]

   set incDebugButtonValue [controlChecked wnCANBuildDlg.incDebugButton]
   set build_flags ""

   #find out if the tool selected is gnu or not because the optimization
   #options are different for gnu and diab.

   #for diab the options are -O (for level 1) -XO (for level 2)
   #for gnu the options are -O0 (for level 0) -O1 (for level 1) 
   #                        -O2 (for level 2) -O3 (for level 3)
   if {[string last gnu [string tolower $toolchainSelected]] > 0} {
      if {$optLevelSelected != "none"} {              
         set build_flags "ADDED_CFLAGS=-O${optLevelSelected}"
      } else {
         set build_flags "ADDED_CFLAGS=-O0"
      }
   } elseif {[string last diab [string tolower $toolchainSelected]] > 0} {  
      if {$optLevelSelected == "none"} {
         set build_flags "ADDED_CFLAGS=-Xkill-opt=0"
      } elseif {$optLevelSelected == "1"} {
         set build_flags "ADDED_CFLAGS=-O"
      } elseif {$optLevelSelected == "2"} {
         set build_flags "ADDED_CFLAGS=-XO"
      }
   }

   if {$incDebugButtonValue == 1} {
      #warn user when debug info and optimization is selected 
      if {$optLevelSelected != "none"} {
         set reply [messageBox -yesno \
         "WARNING: Compiling with both debug info and optimization turned on\n\
         will cause source stepping in the debugger to produce unpredictable\n\
         results.  Continue anyway?"]
         if {$reply == "no"} {
            return 0
         }
      }
      set build_flags "$build_flags EXTRA_DEFINE=-g"
   }
   
   wnCANBuildLaunch $build_flags "build"

}


#################################################################################
# wnCANCleanOK - callback procedure called when the user clicks the "Clean"		#
# button in the Clean dialog.													#
#################################################################################

proc wnCANCleanOK {} {

	wnCANBuildLaunch "rclean" "clean"

}


#################################################################################
# wnCANBuildLaunch - callback procedure called by the Build and Clean dialog.	#
# This procedure invokes the make command that builds or cleans the wnCAN		#
# libraries.																	#
#################################################################################

proc wnCANBuildLaunch {build_flags buildType} {

	set toolChains [::prj_vxApp::toolChainListGet]
	if {$buildType == "build"} {
		set toolchainSelected \
            [controlSelectionGet wnCANBuildDlg.toolchainCombo -string]
		set incDebugButtonValue [controlChecked wnCANBuildDlg.incDebugButton]
		set rebuildAllButtonValue [controlChecked wnCANBuildDlg.rebuildAllButton]
		set windowTitle "Building wnCAN library"
	} else {
		set toolchainSelected \
            [controlSelectionGet wnCANCleanDlg.toolchainCombo -string]
		set stdLibChoiceValue [controlChecked wnCANCleanDlg.stdLibChoice]
		set dbgLibChoiceValue [controlChecked wnCANCleanDlg.dbgLibChoice]
		set windowTitle "Cleaning wnCAN library"
	}

	foreach toolchain $toolChains {
		if {[${toolchain}::name] == ${toolchainSelected}} {
			set tc_name [${toolchain}::name]			
			set tc_tool [${toolchain}::tool]
			set tc_family [${toolchain}::family]
			set tc_cpu [${toolchain}::cpu]


			set makeFileName make.${tc_name}
			set makeDir [wtxPath target h tool ${tc_family}]

			# for arm and mips VX_CPU_FAMILY is defined in make.XXX file for PENTIUM SH PPC
			set tc_cpu_family [canlookForCPUFamily ${makeDir}${makeFileName}]   
         
			# for arm and mips VX_CPU_FAMILY is defined in target\h\tool\common\def.arm and 
			# target\h\tool\common\def.mips             
			if {$tc_cpu_family == ""} {
				foreach file [glob [wtxPath target h tool common]defs.*] {               
					if {[string compare $file [wtxPath target h tool common]defs.common] == 0} continue
					set tc_cpu_family [canlookForCPUFamily $file]
					if {$tc_cpu_family != ""} break
				}            
			}               
		}
	}

	set bslash \\
	set workingDir [wtxPath target src drv CAN]

	if {$buildType == "build"} {

		if {$incDebugButtonValue == 1} {
			set libName libwnCAN_debug.a
			set libDirName ${tc_tool}_debug
			set objDirName objwnCAN_debug
		} else {
			set libName libwnCAN.a
			set libDirName ${tc_tool}
			set objDirName objwnCAN
		}

		set libDir [wtxPath target lib ${tc_cpu_family} ${tc_cpu} ${libDirName}]
		set objDir [wtxPath target lib ${tc_cpu_family} ${tc_cpu} ${libDirName} ${objDirName}]

		set libName_flag "LIBNAME=${tc_cpu_family}${bslash}${tc_cpu}${bslash}${libDirName}${bslash}${libName}"
		set objDirName_flag "LIBDIRNAME=${tc_cpu_family}${bslash}${tc_cpu}${bslash}${libDirName}${bslash}${objDirName}"

		set buildTarget "CPU=$tc_cpu TOOL=$tc_tool $build_flags $libName_flag $objDirName_flag"
		set makeCommandLine "make ${buildTarget}"

### Regenerate dependency file

		if {[file exists "${workingDir}depend.${tc_name}"]} {
			file delete -force "${workingDir}depend.${tc_name}"
		}

### Regenerate wnCAN library

		if {[file exists $libDir$libName]} {
			file delete -force $libDir$libName
		}

### Regenerate all object files and library - rebuild all option

		if {$rebuildAllButtonValue == 1} {
			if {[file exists $objDir]} {
				file delete -force $objDir
			}
		}

		set failed 0
		if {[catch {buildLaunch $windowTitle $makeCommandLine $workingDir} result]} {
			messageBox -ok "Build Launch Failed: $result"
			set failed 1
		}

		if {$incDebugButtonValue == 1 && $failed == 0} {

			windowHide wnCANBuildDlg 1

			dialogCreate \
				-name wnCANDbgLibLocDlg -title "wnCAN Debug Library Location" \
				-width 320 -height 90 \
				-nocontexthelp \
				-init {::wnCANDbgLibLocDlgInit "${libDir}${libName}"} \
				-controls \
					[list \
						[list label -name dbgLibLocLbl \
							-title "After the build process (with no build errors), the wnCAN debug\
							library will be located in:" \
							-xpos 10 -ypos 10 -width 300 -height 10] \
						[list text -name dbgLibLocText -readonly \
							-xpos 20 -ypos 25 -width 280 -height 15] \
						[list label -name dbgLibLocComment \
							-title "To use the wnCAN debug library with your project, prepend the\
							library above to your project's LIBS macro." \
							-xpos 10 -ypos 45 -width 300 -height 20] \
						[list button -name dbgLibLocOKButton \
							-callback "windowClose wnCANDbgLibLocDlg" \
							-title "OK" \
							-xpos 135 -ypos 70 -width 50 -height 15] \
					]
		}

		windowClose wnCANBuildDlg

	} else {

		if {$dbgLibChoiceValue == 1} {
			set libName libwnCAN_debug.a
			set libDirName ${tc_tool}_debug
			set objDirName objwnCAN_debug
		} else {
			set libName libwnCAN.a
			set libDirName ${tc_tool}
			set objDirName objwnCAN
		}

		set libDir [wtxPath target lib ${tc_cpu_family} ${tc_cpu} ${libDirName}]
		set objDir [wtxPath target lib ${tc_cpu_family} ${tc_cpu} ${libDirName} ${objDirName}]

		set libName_flag "LIBNAME=${tc_cpu_family}${bslash}${tc_cpu}${bslash}${libDirName}${bslash}${libName}"
		set objDirName_flag "LIBDIRNAME=${tc_cpu_family}${bslash}${tc_cpu}${bslash}${libDirName}${bslash}${objDirName}"

		set buildTarget "CPU=$tc_cpu TOOL=$tc_tool $build_flags $libName_flag $objDirName_flag"
		set makeCommandLine "make ${buildTarget}"

		if {[file exists "${workingDir}depend.${tc_name}"]} {
			file delete -force "${workingDir}depend.${tc_name}"
		}

		if {[catch {buildLaunch $windowTitle $makeCommandLine $workingDir} result]} {
			messageBox -ok "Build Launch Failed: $result"
		}

		if {[file exists $libDir$libName]} {
			file delete -force $libDir$libName
		}
		if {[file exists $objDir]} {
			file delete -force $objDir
		}

		if {$dbgLibChoiceValue == 1} {
			if {[file exists $libDir]} {
				file delete $libDir
			}
		}

		windowClose wnCANCleanDlg

	}

}


#################################################################################
# wnCANDbgLibLocDlgInit - callback procedure called when the debug library		#
# location dialog is initialized.												#
#################################################################################

proc wnCANDbgLibLocDlgInit {dbgLibLoc} {
    
	controlTextSet wnCANDbgLibLocDlg.dbgLibLocText $dbgLibLoc

}
