# cmpScriptHttpLib.tcl - updates a Wind Web Server project with a specified configuration
#
# 
# DESCRIPTION
# 	Given a specified configuration and a project file name, this script
#	updates the project with that configuration
#
# modification history
# --------------------
# 01c,04feb00,csr  fixing SPR #30126 (applications need to write to registry
#                  to save state)
# 01b,07dec99,csr  fixing call usage
# 01a,19nov99,csr	written.
#


source [wtxPath host resource tcl app-config Project]cmpScriptLib.tcl
source [wtxPath host resource tcl app-config Project]Utils.tcl

namespace eval httpScript {
    set OK 1
    set ERROR -1

    set projectFileName ""
    set configurationScript ""


    ###############################################################
    #
    # usage - print usage information
    #

    proc usage {} {
       puts "cmpScriptHttpLib.tcl -p <projectName> -c <configuration script>"
       puts "\t\tprojectName\t\tThe name of the project who's configuration will change"
       puts "\t\tconfigurationScript\tThe configuration to apply to projectName"
    }

    ###############################################################
    #
    # parseArgs - parse the arguments
    #

    proc parseArgs {} {
	global argv
	global httpScript::projectFileName
	global httpScript::configurationScript

	global httpScript::OK
	global httpScript::ERROR

	for {set i 0} {$i < [llength $argv]} {incr i} {

	    set tempArg [lindex $argv $i]

	    if {[string match $tempArg "-p"] || \
		[string match $tempArg "-P"]} {
		if {[llength $argv] > [expr $i + 1]} {
		    set tempStr [lindex $argv [expr $i + 1]]
		    if {[string first "-" $tempStr] != 0} {
			set httpScript::projectFileName [lindex $argv [expr $i + 1]]
		    } else {
			return ${httpScript::ERROR}
		    }
		}
	    } 

	    if {[string match $tempArg "-c"] || \
		[string match $tempArg "-C"]} {
		if {[llength $argv] > [expr $i + 1]} {
		    set tempStr [lindex $argv [expr $i + 1]]
		    if {[string first "-" $tempStr] != 0} {
			set httpScript::configurationScript [lindex $argv [expr $i + 1]]
		    } else {
			return ${httpScript::ERROR}
		    }
		}
	    }
	}

	if {[string match ${httpScript::projectFileName} ""] || \
            [string match ${httpScript::configurationScript} ""]} {
	     return ${httpScript::ERROR}
	}

	return ${httpScript::OK}
    }

}

set parseOk [httpScript::parseArgs]
if {$parseOk == $httpScript::ERROR} {
   httpScript::usage
   exit -1
} 

cmpProjOpen ${httpScript::projectFileName}

source ${httpScript::configurationScript}

set validate [cmpValidate]
if {$validate != ""} {
   puts stderr "Project Errors exist: $validate"
}

set pathToHere [pwd]
append pathToHere "/"

# remove the previous configuration source file, if it exists

set fileList [cmpFileListGet]
for {set i 0} {$i < [llength $fileList]} {incr i} {
    set tempFileName [lindex $fileList $i]
    if {[string match [file tail $tempFileName] httpconf.c]} {
        cmpFileRemove "$tempFileName"
    }
}

cmpFileAdd ${pathToHere}httpconf.c

if {[info exists stepFiles]} {
    for {set i 0} {$i < [llength $stepFiles]} {incr i} {
        set tempFileName $pathToHere
	append tempFileName [lindex $stepFiles $i]

	for {set j 0} {$j < [llength $fileList]} {incr j} {
	    set tempFileNameInProj [lindex $fileList $j]
	    set tempFileNameInProjTail [file tail $tempFileNameInProj]
	    if {[string match [file tail $tempFileName] $tempFileNameInProjTail]} {
		cmpFileRemove $tempFileNameInProj
	    }
	}

	# add the new file

	cmpFileAdd "$tempFileName"
    }
}

cmpProjClose
 
# notify the GUI that the script has completed

set wws2FlagFileToWrite "[file dirname ${httpScript::projectFileName}]/wws2DemoConfigDone"
set tmpFd [open $wws2FlagFileToWrite w]
close $tmpFd
