/***********************************************************************************/
/*                        M-Systems Confidential                                   */
/*           Copyright (C) M-Systems Flash Disk Pioneers Ltd. 1995-2005            */
/*                         All Rights Reserved                                     */
/***********************************************************************************/
/*                            NOTICE OF M-SYSTEMS OEM                              */
/*                           SOFTWARE LICENSE AGREEMENT                            */
/*                                                                                 */
/*      THE USE OF THIS SOFTWARE IS GOVERNED BY A SEPARATE LICENSE                 */
/*      AGREEMENT BETWEEN THE OEM AND M-SYSTEMS. REFER TO THAT AGREEMENT           */
/*      FOR THE SPECIFIC TERMS AND CONDITIONS OF USE,                              */
/*      OR CONTACT M-SYSTEMS FOR LICENSE ASSISTANCE:                               */
/*      E-MAIL = info@m-sys.com                                                    */
/***********************************************************************************/

/***********************************************************************************/
/*																				   */
/*	Header file containing definitions used by external API						   */ 
/*																				   */
/***********************************************************************************/

#ifndef _COMMON_H
#define _COMMON_H

/**************************************************************************/
/* General types definitions                                              */
/*   TrueFFS interface of all packages uses the following types as the    */
/*   variable definision.                                                 */
/**************************************************************************/

typedef int            FLBoolean;
typedef unsigned char  FLByte;           /* 8  bit unsigned variable */
typedef unsigned short FLWord;           /* 16 bit unsigned variable */
typedef unsigned long  FLDword;          /* 32 bit unsigned variable */
typedef unsigned int   FLNative;         /* Native variable at least 16 bits */
typedef signed char    FLSByte;          /* 8  bit signed variable */
typedef signed short   FLSWord;          /* 16 bit signed variable */
typedef signed long    FLSDword;         /* 32 bit signed variable */
typedef int            FLSNative;        /* Signed native variable at least 16 bits */
typedef FLSWord        FLWchar;          /* Unicode type variable */
typedef FLDword        CardAddress ;	 /* Physical offset on card */

/**************************************************************************/
/*                            I O r e q                                 */
/*                                                                      */
/* IOreq is a common structure passed to all TrueFFS functions.         */
/* Refer to the description of individual functions for specific usage  */
/* of fields. Some fields have different names when used by different   */
/* functions, hence the use of unions.                                  */
/*                                                                      */
/**************************************************************************/

typedef unsigned FLHandle;     /* Handle of an open file or drive.     */
                               /* Actually an index to file table or   */
                               /* drive table.                         */


typedef struct {
  FLHandle   irHandle;         /* Handle of file or drive for operation*/
  FLDword    irFlags;          /* function-specific flags              */
  void     * irPath;	       /* path of file for operation 		   */
  void	   * irData;           /* Pointer to user-buffer for operation */
  FLSDword   irLength;         /* No. of bytes, size or position for   */
                               /* operation                            */
  FLSDword   irCount;          /* Count or offset for operaion         */
} IOreq;

/**************************************************************************/
/* Special ioreq field names:                                             */
/* IOReq structure is the basic IO type used for all TrueFFS calls.       */
/* Some of the fields has additional name for more trivial use of the     */
/* function. The duplicate names are defined below.                       */
/**************************************************************************/
#define irSectorCount   irCount
#define irSectorNo      irLength
#define irByteCount     irCount
#define irAddress       irLength

#define FL_GET_SOCKET_FROM_HANDLE(ioreq)		(FLByte)((ioreq)->irHandle & 0x0f)
#define FL_GET_FLASH_PARTITION_FROM_HANDLE(ioreq)	(FLByte)(((ioreq)->irHandle & 0xf0) >> 4)

#endif /*_COMMON_H*/

