/***********************************************************************************/
/*                        M-Systems Confidential                                   */
/*           Copyright (C) M-Systems Flash Disk Pioneers Ltd. 1995-2006            */
/*                         All Rights Reserved                                     */
/***********************************************************************************/
/*                            NOTICE OF M-SYSTEMS OEM                              */
/*                           SOFTWARE LICENSE AGREEMENT                            */
/*                                                                                 */
/*      THE USE OF THIS SOFTWARE IS GOVERNED BY A SEPARATE LICENSE                 */
/*      AGREEMENT BETWEEN THE OEM AND M-SYSTEMS. REFER TO THAT AGREEMENT           */
/*      FOR THE SPECIFIC TERMS AND CONDITIONS OF USE,                              */
/*      OR CONTACT M-SYSTEMS FOR LICENSE ASSISTANCE:                               */
/*      E-MAIL = info@m-sys.com                                                    */
/***********************************************************************************/

/*
 * $Log:   V:/PVCSDB/DiskOnChip/archives/Testing/TrueFFS 6.3/Drop 2.5/3/bddoc/src/_docsys.h-arc  $
 * 
 *    Rev 1.2   Feb 14 2006 10:56:02   DoronC
 * Update copyright note.
 * 
 *    Rev 1.1   Apr 21 2005 02:16:18   oris
 * Added compile time defaults for access layer customization
 * 
 *    Rev 1.0   Mar 27 2005 09:27:22   oris
 * Initial revision.
 * 
 *    Rev 1.0   Dec 09 2004 18:24:34   oris
 * Initial revision.
 * 
 *    Rev 1.1   Jun 13 2004 17:08:58   OriS
 * Removed compilation error message when FL_NO_USE_FUNC is defined
 * 
 *    Rev 1.0   Apr 09 2003 12:16:02   OriS
 * Initial revision.
 * 
 */

/************************************************************************                                                                      * 
 *                          I M P O R T E N T                           *  
 *                                                                      * 
 * The file contains DiskOnChip memory access routines and macros       * 
 * defintions.                                                          * 
 *                                                                      * 
 * In order to use the complete set of TrueFFS memory access routine    * 
 * that allows runtime configuration of each socket access type make    * 
 * sure the FL_NO_USE_FUNC is not defined in FLCUSTOM.H:                * 
 *                                                                      * 
 * If you know the exact configuration of your application you can      * 
 * uncomment the FL_NO_USE_FUNC definition and set the proper access    * 
 * type using the macroe defintion bellow.                              * 
 *                                                                      * 
 ************************************************************************/

#ifndef _DOCSYS_H
#define _DOCSYS_H

#include "nanddefs.h"

#ifdef FL_NO_USE_FUNC 

/*--------------------------------------------------------------------------*/

/* Default access layer for 16 bit configuration */

#ifndef flDirectWrite8BitReg
#ifdef FL_BIG_ENDIAN
#define flDirectWrite8BitReg(flash,offset,val) \
        FLWRITE_IO_WORD((((FLWord)val)<<8),((volatile FLWord FAR0 *)((flash)->win)+((offset)>>1)))
#else
#define flDirectWrite8BitReg(flash,offset,val) \
        FLWRITE_IO_WORD(((FLWord)val),((volatile FLWord FAR0 *)((flash)->win)+((offset)>>1)))
#endif /* FL_BIG_ENDIAN */
#endif /* flDirectWrite8BitReg */

#ifndef flDirectWrite16BitReg
#define flDirectWrite16BitReg(flash,offset,val) \
        FLWRITE_IO_WORD(val,((volatile FLByte FAR0 *)((flash)->win)+(offset)))
#endif /* flDirectWrite16BitReg */

#ifndef flDirectRead8BitReg
#ifdef FL_BIG_ENDIAN
#define flDirectRead8BitReg(flash,offset)                 \
        (Reg8bitType)(((offset & 0x1) == 0) ?             \
		  ((FLREAD_IO_WORD((volatile FLWord FAR0 *)((flash)->win)+((offset)>>1))) >> 8) : \
          ((FLREAD_IO_WORD((volatile FLWord FAR0 *)((flash)->win)+((offset)>>1)))     )   )
#else
#define flDirectRead8BitReg(flash,offset)                 \
        (Reg8bitType)(((offset & 0x1) != 0) ?             \
		  ((FLREAD_IO_WORD((volatile FLWord FAR0 *)((flash)->win)+((offset)>>1))) >> 8) : \
          ((FLREAD_IO_WORD((volatile FLWord FAR0 *)((flash)->win)+((offset)>>1)))     )   )
#endif /* FL_BIG_ENDIAN */
#endif /* flDirectRead8BitReg */

#ifndef flDirectRead16BitReg 
#define flDirectRead16BitReg(flash,offset) \
	    FLREAD_IO_WORD((flash)->win+(offset))
#endif

#ifndef flReadEvenNumberOfBytes
#define flReadEvenNumberOfBytes(flash,offset,dest,count) \
	    TFFSCPY_FROM_IO_16_BITS( dest,((flash)->win + (offset)), count) 
#endif

#ifndef flWriteEvenNumberOfBytes
#define flWriteEvenNumberOfBytes(flash,offset,src,count) \
	    TFFSCPY_TO_IO_16_BITS(((flash)->win + (offset)),src,count)
#endif

#ifndef flRead512Bytes
#define flRead512Bytes(flash,offset,dest)                flReadEvenNumberOfBytes(flash,offset,dest,512)
#endif

#ifndef flWrite512Bytes
#define flWrite512Bytes(flash,offset,src)                flWriteEvenNumberOfBytes(flash,offset,src,512)
#endif

#ifndef flDocWindow
#define flDocWindow(flash)                               (0x2000)
#endif

/*--------------------------------------------------------------------------*/

#else /* FL_NO_USE_FUNC */

/*--------------------------------------------------------------------------*/

/* Redirect to runtime access layer */

#ifndef flDirectWrite8BitReg
#define flDirectWrite8BitReg(flash,offset,val)           (flash)->memWrite8bit((flash)->win,offset,val)
#endif

#ifndef flDirectWrite16BitReg
#define flDirectWrite16BitReg(flash,offset,val)          (flash)->memWrite16bit((flash)->win,offset,val)
#endif

#ifndef flDirectRead8BitReg
#define flDirectRead8BitReg(flash,offset)                (flash)->memRead8bit((flash)->win,offset)
#endif

#ifndef flDirectRead16BitReg 
#define flDirectRead16BitReg(flash,offset)               (flash)->memRead16bit((flash)->win,offset)
#endif

#ifndef flReadEvenNumberOfBytes
#define flReadEvenNumberOfBytes(flash,offset,dest,count) (flash)->memRead((flash)->win,offset,dest,count)
#endif

#ifndef flWriteEvenNumberOfBytes
#define flWriteEvenNumberOfBytes(flash,offset,src,count) (flash)->memWrite((flash)->win,offset,src,count)
#endif

#ifndef flRead512Bytes
#define flRead512Bytes(flash,offset,dest)                flReadEvenNumberOfBytes(flash,offset,dest,512)
#endif

#ifndef flWrite512Bytes
#define flWrite512Bytes(flash,offset,src)                flWriteEvenNumberOfBytes(flash,offset,src,512)
#endif

#ifndef flDocWindow
#define flDocWindow(flash)                               (flash->memWindowSize())
#endif

/*--------------------------------------------------------------------------*/

/* (private) types of DiskOnChip access configurations */

#define FL_8BIT_DOC_ACCESS         0x00000000L /* Has 8 data bits            */
#define FL_16BIT_DOC_ACCESS        0x00000100L /* Has 16 data bits           */
#define FL_XX_DATA_BITS_MASK       0x00000300L /* Mask of the above          */
#define FL_8BIT_FLASH_ACCESS       0x00000400L /* 8 bits of flash per cycle  */
#define FL_16BIT_FLASH_ACCESS      0x00000800L /* 16 bits of flash per cycle */
#define FL_XX_FLASH_ACCESS_MASK    0x00000C00L /* Mask of the above          */

#define FL_ACCESS_USER_DEFINED     0x00001000L /* User defined routines */

/* DiskOnChip routines prototypes */

extern FLStatus FAR1 setBusTypeOfFlash(FLFlash * flash,FLDword access);

#endif /* FL_NO_USE_FUNC */
#endif /* DOCSYS_H */
