/***********************************************************************************/
/*                        M-Systems Confidential                                   */
/*           Copyright (C) M-Systems Flash Disk Pioneers Ltd. 1995-2006            */
/*                         All Rights Reserved                                     */
/***********************************************************************************/
/*                            NOTICE OF M-SYSTEMS OEM                              */
/*                           SOFTWARE LICENSE AGREEMENT                            */
/*                                                                                 */
/*      THE USE OF THIS SOFTWARE IS GOVERNED BY A SEPARATE LICENSE                 */
/*      AGREEMENT BETWEEN THE OEM AND M-SYSTEMS. REFER TO THAT AGREEMENT           */
/*      FOR THE SPECIFIC TERMS AND CONDITIONS OF USE,                              */
/*      OR CONTACT M-SYSTEMS FOR LICENSE ASSISTANCE:                               */
/*      E-MAIL = info@m-sys.com                                                    */
/***********************************************************************************/

/*
 * $Log:   V:/PVCSDB/DiskOnChip/archives/Testing/TrueFFS 6.3/Drop 2.5/3/bddoc/src/_flsoc.h-arc  $
 * 
 *    Rev 1.3   Feb 14 2006 10:56:04   DoronC
 * Update copyright note.
 * 
 *    Rev 1.2   May 10 2005 13:06:04   oris
 * - Bug fix: Code optimization introduced in last version did not work properly - could also cause compilation errors.
 * 
 *    Rev 1.1   Apr 19 2005 02:45:16   oris
 * - Removed old unused compilation flags and minor optimization
 *    when single socket is used.
 * 
 *    Rev 1.0   Mar 27 2005 09:27:22   oris
 * Initial revision.
 * 
 *    Rev 1.0   Dec 09 2004 18:24:34   oris
 * Initial revision.
 * 
 *    Rev 1.0   Apr 09 2003 12:16:02   OriS
 * Initial revision.
 * 
 */

#ifndef _FLSOC_H
#define _FLSOC_H

struct tSocket {
  unsigned        volNo;         /* Volume no. of socket */
  unsigned        serialNo;      /* Serial no. of socket on controller */

  FLBoolean       cardChanged;   /* need media change notification */

  int             VccUsers;      /* No. of current VCC users */
  int             VppUsers;      /* No. of current VPP users */

  PowerState      VccState;      /* Actual VCC state */
  PowerState      VppState;      /* Actual VPP state */
  PowerState      DPDState;      /* Actual DPD state */
  FLByte            curPartition;  /* Current partition to use the socket (set busy) */
#ifdef FL_VERIFY_WRITE
  FLByte            verifyWrite;   /* VerifyWrite mode                               */
#endif /* FL_VERIFY_WRITE */
  FLBoolean       remapped;      /* set to TRUE whenever the socket window is moved */

  void            (*powerOnCallback)(void *flash); /* Notification routine for Vcc on */
  void *          flash;         /* Flash object for callback */

  struct {                       /* Window state                     */
    unsigned int  baseAddress;   /* Physical base as a 4K page       */
    unsigned int  currentPage;   /* Our current window page mapping  */
    void FAR0 *   base;          /* Pointer to window base           */
    FLSDword      size;          /* Window size (must by power of 2) */
    unsigned      speed;         /* in nsec.                         */
    unsigned      busWidth;      /* 8 or 16 bits                     */
  } window;

/*----------------------------------------------------------------------*/
/*                      c a r d D e t e c t e d                         */
/*                                                                      */
/* Detect if a card is present (inserted)                               */
/*                                                                      */
/* Parameters:                                                          */
/*      vol            : Pointer identifying drive                      */
/*                                                                      */
/* Returns:                                                             */
/*      0 = card not present, other = card present                      */
/*----------------------------------------------------------------------*/
  FLBoolean (*cardDetected)(FLSocket * pVol);

/*----------------------------------------------------------------------*/
/*                           V c c O n                                        */
/*                                                                      */
/* Turns on Vcc (3.3/5 Volts). Vcc must be known to be good on exit.      */
/*                                                                      */
/* Parameters:                                                          */
/*      vol            : Pointer identifying drive                                  */
/*                                                                      */
/*----------------------------------------------------------------------*/
  void (*VccOn)(FLSocket * pVol);

/*----------------------------------------------------------------------*/
/*                         V c c O f f                                        */
/*                                                                      */
/* Turns off Vcc.                                                          */
/*                                                                          */
/* Parameters:                                                          */
/*      vol            : Pointer identifying drive                      */
/*                                                                      */
/*----------------------------------------------------------------------*/
  void (*VccOff)(FLSocket * pVol);

#ifdef SOCKET_12_VOLTS

/*----------------------------------------------------------------------*/
/*                           V p p O n                                        */
/*                                                                      */
/* Turns on Vpp (12 Volts. Vpp must be known to be good on exit.      */
/*                                                                      */
/* Parameters:                                                          */
/*      vol            : Pointer identifying drive                                  */
/*                                                                      */
/* Returns:                                                             */
/*      FLStatus      : 0 on success, failed otherwise                            */
/*----------------------------------------------------------------------*/
  FLStatus (*VppOn)(FLSocket * pVol);


/*----------------------------------------------------------------------*/
/*                         V p p O f f                                  */
/*                                                                      */
/* Turns off Vpp.                                                       */
/*                                                                      */
/* Parameters:                                                          */
/*      vol            : Pointer identifying drive                      */
/*                                                                      */
/*----------------------------------------------------------------------*/
  void (*VppOff)(FLSocket * pVol);

#endif      /* SOCKET_12_VOLTS */

/*----------------------------------------------------------------------*/
/*                      i n i t S o c k e t                             */
/*                                                                      */
/* Perform all necessary initializations of the socket or controller    */
/*                                                                      */
/* Parameters:                                                          */
/*      vol            : Pointer identifying drive                      */
/*                                                                      */
/* Returns:                                                             */
/*      FLStatus      : 0 on success, failed otherwise                  */
/*----------------------------------------------------------------------*/
  FLStatus (*initSocket)(FLSocket * pVol);

/*----------------------------------------------------------------------*/
/*                        s e t W i n d o w                             */
/*                                                                      */
/* Sets in hardware all current window parameters: Base address, size,  */
/* speed and bus width.                                                 */
/* The requested settings are given in the 'vol.window' structure.      */
/*                                                                      */
/* If it is not possible to set the window size requested in            */
/* 'vol.window.size', the window size should be set to a larger value,  */
/* if possible. In any case, 'vol.window.size' should contain the       */
/* actual window size (in 4 KB units) on exit.                          */
/*                                                                      */
/* Parameters:                                                          */
/*      vol            : Pointer identifying drive                      */
/*                                                                      */
/*----------------------------------------------------------------------*/
  void (*setWindow)(FLSocket * pVol);


/*----------------------------------------------------------------------*/
/*               s e t M a p p i n g C o n t e x t                      */
/*                                                                      */
/* Sets the window mapping register to a card address.                  */
/*                                                                      */
/* The window should be set to the value of 'vol.window.currentPage',   */
/* which is the card address divided by 4 KB. An address over 128KB,    */
/* (page over 32K) specifies an attribute-space address.                */
/*                                                                      */
/* The page to map is guaranteed to be on a full window-size boundary.  */
/*                                                                      */
/* Parameters:                                                          */
/*      vol            : Pointer identifying drive                      */
/*                                                                      */
/*----------------------------------------------------------------------*/
  void (*setMappingContext)(FLSocket * pVol, unsigned page);

/*----------------------------------------------------------------------*/
/*       g e t A n d C l e a r C a r d C h a n g e I n d i c a t o r    */
/*                                                                      */
/* Returns the hardware card-change indicator and clears it if set.     */
/*                                                                      */
/* Parameters:                                                          */
/*      vol            : Pointer identifying drive                      */
/*                                                                      */
/* Returns:                                                             */
/*      0 = Card not changed, other = card changed                      */
/*----------------------------------------------------------------------*/
  FLBoolean (*getAndClearCardChangeIndicator)(FLSocket * pVol);

/*----------------------------------------------------------------------*/
/*                      w r i t e P r o t e c t e d                     */
/*                                                                      */
/* Returns the write-protect state of the media                         */
/*                                                                      */
/* Parameters:                                                          */
/*      vol            : Pointer identifying drive                      */
/*                                                                      */
/* Returns:                                                             */
/*      0 = not write-protected, other = write-protected                */
/*----------------------------------------------------------------------*/
  FLBoolean (*writeProtected)(FLSocket * pVol);

/*----------------------------------------------------------------------*/
/*            u p d a t e S o c k e t P a r a m s                       */
/*                                                                      */
/* Pass socket parameters to the socket interface layer.                */
/* The structure passed in irData is specific for each socket interface.*/
/*                                                                      */
/* Parameters:                                                          */
/*      vol            : pointer identifying drive                      */
/*      params             : pointer to structure that holds socket     */
/*                    parameters.                                       */
/*                                                                      */
/*----------------------------------------------------------------------*/
  void (*updateSocketParams)(FLSocket * pVol, void FAR1 *params);


/*----------------------------------------------------------------------*/
/*                      f r e e S o c k e t                             */
/*                                                                      */
/* Free resources that were allocated for this socket.                  */
/* This function is called when FLite exits.                            */
/*                                                                      */
/* Parameters:                                                          */
/*      vol            : Pointer identifying drive                      */
/*                                                                      */
/*----------------------------------------------------------------------*/
#ifdef FL_EXIT
  void (*freeSocket)(FLSocket * pVol);
#endif
};


/* See interface documentation of functions in socket.c */

extern FLStatus updateSocketParameters(FLSocket *, void FAR1 *);

extern FLStatus      flInitSockets(void);

#ifdef FL_EXIT
extern void      flExitSocket(FLSocket *);
#endif

#if (FL_SOCKETS != 1)
extern FLByte    flSocketNoOf(const FLSocket *);
#else /* (FL_SOCKETS == 1) */
#define          flSocketNoOf(A) (0)
#endif /* (FL_SOCKETS == 1) */

extern FLSocket* flSocketOf(unsigned volNo);
extern FLBuffer* flBufferOf(unsigned volNo);
#if (defined(FL_VERIFY_WRITE) || defined(VERIFY_VOLUME))
extern FLByte    * flReadBackBufferOf(unsigned volNo);
#endif /* FL_VERIFY_WRITE || VERIFY_VOLUME */
extern void      flNeedVcc(FLSocket *);
extern void      flDontNeedVcc(FLSocket *);
#ifdef SOCKET_12_VOLTS
extern FLStatus  flNeedVpp(FLSocket *);
extern void      flDontNeedVpp(FLSocket *);
#endif
extern void      flSocketSetBusy(FLSocket *, FLBoolean);      /* entry/exit operations */
extern FLBoolean flWriteProtected(FLSocket *); /* write-protection status */
#ifndef FL_FIXED_MEDIA
extern FLStatus  flMediaCheck(FLSocket *);      /* check for media status change */
extern void      flResetCardChanged(FLSocket *);
#endif /* FL_FIXED_MEDIA */
extern unsigned  flGetMappingContext(FLSocket *);  /* Currently mapped 4KB page */
extern void FAR0*flMap(FLSocket *, CardAddress);      /* map and point at card address */
extern void      flSetWindowBusWidth(FLSocket *, unsigned); /* set window data-path */
extern void      flSetWindowSpeed(FLSocket *, unsigned);      /* set window speed (nsec.) */
extern void      flSetWindowSize(FLSocket *, unsigned);      /* in 4KB units */

extern void      flSetPowerOnCallback(FLSocket *, void (*)(void *), void *);
                  /* Set MTD notification for socket power on */
extern void      flIntervalRoutine(FLSocket *);      /* socket interval routine */


#endif /* _FLSOC_H */

