/***********************************************************************************/
/*                        M-Systems Confidential                                   */
/*           Copyright (C) M-Systems Flash Disk Pioneers Ltd. 1995-2003            */
/*                         All Rights Reserved                                     */
/***********************************************************************************/
/*                            NOTICE OF M-SYSTEMS OEM                              */
/*                           SOFTWARE LICENSE AGREEMENT                            */
/*                                                                                 */
/*      THE USE OF THIS SOFTWARE IS GOVERNED BY A SEPARATE LICENSE                 */
/*      AGREEMENT BETWEEN THE OEM AND M-SYSTEMS. REFER TO THAT AGREEMENT           */
/*      FOR THE SPECIFIC TERMS AND CONDITIONS OF USE,                              */
/*      OR CONTACT M-SYSTEMS FOR LICENSE ASSISTANCE:                               */
/*      E-MAIL = info@m-sys.com                                                    */
/***********************************************************************************/

/*
 * $Log:   V:/PVCSDB/DiskOnChip/archives/Testing/TrueFFS 6.3/Drop 2.5/3/surefs/src/_fsdev.h-arc  $
 * 
 *    Rev 1.3   Sep 15 2005 14:26:28   roneng
 * Bug fix in flOpenDirectory API 
 * 
 *    Rev 1.2   Apr 25 2005 02:00:50   oris
 * Cosmetics
 * 
 *    Rev 1.1   Apr 19 2005 03:00:22   oris
 * - Cosmetics
 * 
 *    Rev 1.0   Mar 27 2005 09:26:54   oris
 * Initial revision.
 * 
 *    Rev 1.2   Feb 19 2005 12:23:44   eugen
 * added flFlushAllFiles
 * 
 *    Rev 1.1   Jan 05 2005 21:09:32   eugen
 * Commentaries, headers
 * 
 *    Rev 1.0   Dec 09 2004 18:22:20   oris
 * Initial revision.
 * 
 *    Rev 1.10   Jan 14 2004 22:28:24   oris
 * Added flCloseAllFiles API for closing all files and lowering the global attention flag.
 * 
 *    Rev 1.9   Nov 20 2003 18:47:20   omerk
 * Added TFFS_DLL_API prefix to flBuildGeometry routine.
 * 
 *    Rev 1.8   Nov 19 2003 17:28:36   omerk
 * Insert flBuildGeometry routine prototype
 * 
 *    Rev 1.7   Oct 20 2003 08:59:12   oris
 * Changed FL_LONG_FILENAMES_SUPPORT to a negative FL_NO_LONG_FILENAMES_SUPPORT
 * 
 *    Rev 1.6   Sep 30 2003 17:55:08   oris
 * Changed FILES to FL_FILES
 * Moved definitions for SureFS API to dosformt.h
 * Added FL prefix to most of SureFS definition.
 * Added flFlushFile API
 * Added flAsciiStringToUnicode,flUnicodeStringToAscii and flUnicodeStringLength API
 * Removed flBuildGeometry routine prototype.
 * 
 *    Rev 1.5   Jun 16 2003 01:17:48   oris
 * Added TFFS_DLL_API prefix.
 * 
 *    Rev 1.4   May 16 2003 00:56:22   OriS
 * Removed flPhsycalRead, flPhsycalWrite , flPhsycalErase routines.
 * 
 *    Rev 1.3   May 11 2003 08:26:52   OriS
 * Added support for the FL_NO_QUICK_MOUNT_FEATURE compilation flag.
 * Added support for flWriteQuickMountInfo(ioreq) routine.
 * Changed interface definition from FL_QUICK_MOUNT_INFO_EXIST to FL_ON (quick mount routine)
 * 
 *    Rev 1.2   May 08 2003 23:17:58   OriS
 * Added getQuickMount prototype
 * 
 *    Rev 1.1   Apr 13 2003 20:29:12   OriS
 * Changed FL_EXTENEDED_DISK_INFO to FL_EXTENDED_DISK_INFO
 * 
 *    Rev 1.0   Apr 09 2003 12:16:00   OriS
 * Initial revision.
 * 
 */

#ifndef _FSDEV_H
#define _FSDEV_H

#ifdef FS_FILES

#ifndef FS_READ_ONLY

/*----------------------------------------------------------------------*/
/*                     f l F l u s h B u f f e r                        */
/*                                                                      */
/* If there is relevant data in the RAM buffer then writes it on        */
/*   the flash memory.                                                  */
/*                                                                      */
/* Parameters:                                                          */
/*        irHandle  : Drive number (0, 1, ...)                          */
/*                        bits 11-8 - Logical partition in Flash partition (zero based)*/
/*                        bits 7-4 - Partition # (zero based)           */
/*                        bits 3-0 - Socket # (zero based)              */ 
/*                                                                      */
/* Returns:                                                             */
/*        FLStatus        : 0 on success, otherwise failed              */
/*----------------------------------------------------------------------*/

#define flFlushBuffer(ioreq)        fsCall(FL_FLUSH_BUFFER,ioreq)

#endif	/* FS_READ_ONLY */
/*----------------------------------------------------------------------*/
/*                      f l O p e n F i l e                             */
/*                                                                      */
/* Opens an existing file or creates a new file. Creates a file handle  */
/* for further file processing.                                         */
/*                                                                      */
/* Parameters:                                                          */
/*    irHandle        : Drive number (0, 1, ...)                        */
/*                       bits 11-8 - Logical partition in Flash partition (zero based)*/
/*                       bits 7-4 - Partition # (zero based)            */
/*                       bits 3-0 - Socket # (zero based)               */ 
/*    irFlags         : Access and action options, defined below        */
/*    irPath          : path of file to open                            */
/*    Simple path API : Points to FLSimplePath (8.3 file name,          */
/*                      defined in ioreq.h) array. First character of   */
/*                      the last array member must be null.	            */
/*    ASCII API       : Points to ASCII null terminated character string */
/*    Unicode API     : Points to Unicode null terminated string        */
/*                                                                      */
/* Returns:                                                             */
/*        FLStatus        : 0 on success, otherwise failed              */
/*        irHandle        : New file handle for open file               */
/*                                                                      */
/*----------------------------------------------------------------------*/

#define flOpenFile(ioreq)        fsCall(FL_OPEN_FILE,ioreq)

/*----------------------------------------------------------------------*/
/*                      like above; also :                              */
/*                                                                      */
/*        irCount      : defragmentCloseVal                             */
/*        irLength     : expectedLength                                 */
/*        irFlags      : FSO flags                                      */
/*----------------------------------------------------------------------*/

#define flOpenMMFile(ioreq)      fsCall(FL_OPEN_MM_FILE,ioreq)


/*----------------------------------------------------------------------*/
/*                      f l C l o s e F i l e                           */
/*                                                                      */
/* Closes an open file, records file size and dates in directory and    */
/* releases file handle.                                                */
/*                                                                      */
/* Parameters:                                                          */
/*        irHandle        : Handle of file to close.                    */
/*                                                                      */
/* Returns:                                                             */
/*        FLStatus        : 0 on success, otherwise failed              */
/*----------------------------------------------------------------------*/

#define flCloseFile(ioreq)      fsCall(FL_CLOSE_FILE,ioreq)

#ifndef FL_NO_SURE_FS_SUPPORT

/*----------------------------------------------------------------------*/
/*                      f l F l u s h F i l e                           */
/*                                                                      */
/* Make the same like flCloseFile() except closing file. Records file   */
/* size and dates in directory but does not release file handle.        */
/*                                                                      */
/* Parameters:                                                          */
/*        irHandle        : Handle of file to close.                    */
/*                                                                      */
/* Returns:                                                             */
/*        FLStatus        : 0 on success, otherwise failed              */
/*----------------------------------------------------------------------*/

#define flFlushFile(ioreq)      fsCall(FL_FLUSH_FILE,ioreq)


/*----------------------------------------------------------------------*/
/*                      f l F l u s h A l l F i l e s                   */
/*                                                                      */
/* Flushes cache of all files on that volume, updates directory info    */
/*                                                                      */
/* Parameters:                                                          */
/*    irHandle        : Drive number (0, 1, ...)                        */
/*                       bits 11-8 - Logical partition in Flash partition (zero based)*/
/*                       bits 7-4 - Partition # (zero based)            */
/*                       bits 3-0 - Socket # (zero based)               */ 
/*                                                                      */
/* Returns:                                                             */
/*        FLStatus        : 0 on success, otherwise failed              */
/*----------------------------------------------------------------------*/

#define flFlushAllFiles(ioreq)      fsCall(FL_FLUSH_ALL_FILES,ioreq)

#endif /* FL_NO_SURE_FS_SUPPORT */

/*----------------------------------------------------------------------*/
/*                      f l R e a d F i l e                             */
/*                                                                      */
/* Reads from the current position in the file to the user-buffer.      */
/* Parameters:                                                          */
/*        irHandle     : Handle of file to read.                        */
/*        irData       : Address of user buffer                         */
/*        irLength     : Number of bytes to read. If the read extends   */
/*                       beyond the end-of-file, the read is truncated  */
/*                       at the end-of-file.                            */
/*                                                                      */
/* Returns:                                                             */
/*        FLStatus     : 0 on success, otherwise failed                 */
/*        irLength     : Actual number of bytes read                    */
/*----------------------------------------------------------------------*/

#ifdef FL_LIMIT_UNINTERRUPTED_FILE_ACCESS
#ifdef __cplusplus
extern "C"
{
#endif /* __cplusplus */

TFFS_DLL_API FLStatus NAMING_CONVENTION flReadFile(IOreq FAR2 *ioreq);

#ifdef __cplusplus
}
#endif /* __cplusplus */

#define flDirectReadFile(ioreq)        fsCall(FL_READ_FILE,ioreq)

#else	/* !FL_LIMIT_UNINTERRUPTED_FILE_ACCESS */
#define flReadFile(ioreq)        fsCall(FL_READ_FILE,ioreq)
#endif	/* FL_LIMIT_UNINTERRUPTED_FILE_ACCESS  */


#ifndef FS_READ_ONLY
/*----------------------------------------------------------------------*/
/*                      f l W r i t e F i l e                           */
/*                                                                      */
/* Writes from the current position in the file from the user-buffer.   */
/*                                                                      */
/* Parameters:                                                          */
/*        irHandle        : Handle of file to write.                    */
/*        irData          : Address of user buffer                      */
/*        irLength        : Number of bytes to write.                   */
/*                                                                      */
/* Returns:                                                             */
/*        FLStatus        : 0 on success, otherwise failed              */
/*        irLength        : Actual number of bytes written              */
/*----------------------------------------------------------------------*/

#ifdef FL_LIMIT_UNINTERRUPTED_FILE_ACCESS
#ifdef __cplusplus
extern "C"
{
#endif /* __cplusplus */

TFFS_DLL_API FLStatus NAMING_CONVENTION flWriteFile(IOreq FAR2 *ioreq);

#ifdef __cplusplus
}
#endif /* __cplusplus */

#define flDirectWriteFile(ioreq)        fsCall(FL_WRITE_FILE,ioreq)

#else	/* !FL_LIMIT_UNINTERRUPTED_FILE_ACCESS */
#define flWriteFile(ioreq)        fsCall(FL_WRITE_FILE,ioreq)
#endif	/* FL_LIMIT_UNINTERRUPTED_FILE_ACCESS  */

#endif  /* FS_READ_ONLY */
/*----------------------------------------------------------------------*/
/*                      f l S e e k F i l e                             */
/*                                                                      */
/* Sets the current position in the file, relative to file start, end or*/
/* current position.                                                    */
/* Note: This function will not move the file pointer beyond the        */
/* beginning or end of file, so the actual file position may be         */
/* different from the required. The actual position is indicated on     */
/* return.                                                              */
/*                                                                      */
/* Parameters:                                                          */
/*        irHandle  : File handle to close.                             */
/*        irLength  : Offset to set position.                           */
/*        irFlags   : Method code                                       */
/*                    FL_SEEK_START: absolute offset from start of file */
/*                    FL_SEEK_CURR:  signed offset from current position*/
/*                    FL_SEEK_END:   signed offset from end of file     */
/*                                                                      */
/* Returns:                                                             */
/*        FLStatus        : 0 on success, otherwise failed              */
/*        irLength        : Actual absolute offset from start of file   */
/*----------------------------------------------------------------------*/

#define flSeekFile(ioreq)        fsCall(FL_SEEK_FILE,ioreq)

/*----------------------------------------------------------------------*/
/*                          f l F i n d F i l e                         */
/*                                                                      */
/* Finds a file entry in a directory, optionally modifying the file     */
/* time/date and/or attributes.                                         */
/* Files may be found by handle no. provided they are open, or by name. */
/* Only the Hidden, System or Read-only attributes may be modified.     */
/* Entries may be found for any existing file or directory other than   */
/* the root. A DirectoryEntry structure describing the file is copied   */
/* to a user buffer.                                                    */
/*                                                                      */
/* The DirectoryEntry structure is defined in dosformt.h                */
/*                                                                      */
/* Parameters:                                                          */
/*     irHandle        : If by name: Drive number (socket+partitions)   */
/*                       else      : Handle of open file                */
/*     irPath          : If by name: Specifies a file or directory path */
/*        Simple path API : Points to FLSimplePath (8.3 file name,        */
/*                          defined in ioreq.h) array. First character of */
/*                          the last array member must be null.	          */
/*        ASCII API       : Points to ASCII null terminated character string */
/*        Unicode API     : Points to Unicode null terminated string      */
/*        irFlags      : Options flags                                  */
/*                        FL_FIND_BY_HANDLE: Find open file by handle.     */
/*                                        Default is access by path.    */
/*                        FL_SET_DATETIME:   Update time/date from buffer  */
/*                        FL_SET_ATTRIBUTES: Update attributes from buffer */
/*        irData       : Address of user buffer to receive a            */
/*                       DirectoryEntry structure                       */
/*                                                                      */
/* Returns:                                                             */
/*        irData       : buffer is modified if "get" option             */
/*        FLStatus        : 0 on success, otherwise failed              */
/*----------------------------------------------------------------------*/

#define flFindFile(ioreq)		fsCall(FL_FIND_FILE,ioreq)


/*----------------------------------------------------------------------*/
/*                                                                      */
/* Parameters:                                                          */
/*     irHandle        : If by name: Drive number (socket+partitions)   */
/*                       else      : Handle of open file                */
/*     irPath          : If by name: Specifies a file or directory path */
/*        Simple path API : Points to FLSimplePath (8.3 file name,      */
/*                          defined in ioreq.h) array. First character  */
/*                          of the last array member must be null.	*/
/*        ASCII API       : Points to ASCIIZ string                     */
/*        Unicode API     : Points to Unicode null terminated string    */
/*        irFlags         : Options flags                               */
/*                        FL_FIND_BY_HANDLE: Find open file by handle.  */
/*                                        Default is access by path.    */
/*	  irCount     	  : new attributes (RO,SYS,HID,ARCH)		*/
/*	  irCount         : LEushort  updateTime / createTimeHourMin    */
/*	  irLength   	  : LEushort  updateDate / createDate        	*/
/*                                                                      */
/* Returns:                                                             */
/*        irLength, irCount : modified if "get" function                */
/*        FLStatus        : 0 on success, otherwise failed              */
/*----------------------------------------------------------------------*/
#define	flSetFileAttributes(ioreq)	flGetSetFileInfo(ioreq, FL_SET_ATTRIBUTES)
#define	flGetFileAttributes(ioreq)	flGetSetFileInfo(ioreq, FL_GET_ATTRIBUTES)
#define	flSetFileDateTime(ioreq)	flGetSetFileInfo(ioreq, FL_SET_DATETIME)
#define	flGetFileDateTime(ioreq)	flGetSetFileInfo(ioreq, FL_GET_DATETIME)
#define	flSetFileCreationDateTime(ioreq)	flGetSetFileInfo(ioreq, FL_SET_CREATE_DATETIME)
#define	flGetFileCreationDateTime(ioreq)	flGetSetFileInfo(ioreq, FL_GET_CREATE_DATETIME)

/*----------------------------------------------------------------------*/
/*                 f l F i n d F i r s t F i l e                        */
/*                                                                      */
/* Finds the first file entry in a directory.                           */
/* This function is used in combination with the flFindNextFile call,   */
/* which returns the remaining file entries in a directory sequentially.*/
/* Entries are returned according to the unsorted directory order.      */
/* flFindFirstFile creates a file handle, which is returned by it. Calls*/
/* to flFindNextFile will provide this file handle. When flFindNextFile */
/* returns 'noMoreEntries', the file handle is automatically closed.    */
/* Alternatively the file handle can be closed by a 'closeFile' call    */
/* before actually reaching the end of directory.                       */
/* A DirectoryEntry structure is copied to the user buffer describing   */
/* each file found. This structure is defined in dosformt.h.            */
/*                                                                      */
/* Parameters:                                                          */
/*    irHandle        : Drive number (0, 1, ...)                        */
/*                       bits 11-8 - Logical partition in Flash partition (zero based)*/
/*                       bits 7-4 - Partition # (zero based)            */
/*                       bits 3-0 - Socket # (zero based)               */ 
/*    irPath          : Specifies a directory path                      */
/*    Simple path API : Points to FLSimplePath (8.3 file name,          */
/*                      defined in ioreq.h) array. First character of   */
/*                      the last array member must be null.	            */
/*    ASCII API       : Points to ASCII null terminated character string */
/*    Unicode API     : Points to Unicode null terminated string        */
/*    irData          : Address of user buffer to receive a             */
/*                      DirectoryEntry structure                        */
/*                                                                      */
/* Returns:                                                             */
/*        irHandle       : File handle to use for subsequent operations */
/*        FLStatus       : 0 on success, otherwise failed               */
/*----------------------------------------------------------------------*/

#define        flFindFirstFile(ioreq)        fsCall(FL_FIND_FIRST_FILE,ioreq)

/*----------------------------------------------------------------------*/
/*                 f l O p e n D i r e c t o r y                        */
/*                                                                      */
/*          like findFirst, but does not retrieve the first file        */
/*          will set irData in ioreq to NULL                             */
/*                                                                      */
/*----------------------------------------------------------------------*/

#define        flOpenDirectory(ioreq)        ((ioreq)->irData = NULL,fsCall(FL_FIND_FIRST_FILE,ioreq))

/*----------------------------------------------------------------------*/
/*                 f l F i n d N e x t F i l e                          */
/*                                                                      */
/* See the description of 'flFindFirstFile'.                            */
/*                                                                      */
/* Parameters:                                                          */
/*        irHandle        : File handle returned by flFindFirstFile.    */
/*        irData          : Address of user buffer to receive a         */
/*                          DirectoryEntry structure                    */
/*                                                                      */
/* Returns:                                                             */
/*        FLStatus        : 0 on success, otherwise failed              */
/*----------------------------------------------------------------------*/

#define        flFindNextFile(ioreq)        fsCall(FL_FIND_NEXT_FILE,ioreq)

/*----------------------------------------------------------------------*/
/*                      f l G e t D i s k I n f o                       */
/*                                                                      */
/* Returns general allocation information.                              */
/*                                                                      */
/* The bytes/sector, sector/cluster, total cluster and free cluster     */
/* information are returned into a DiskInfo structure.                  */
/*                                                                      */
/* Parameters:                                                          */
/*        irHandle        : Drive number (0, 1, ...)                    */
/*                        bits 11-8 - Logical partition in Flash partition (zero based)*/
/*                        bits 7-4 - Partition # (zero based)           */
/*                        bits 3-0 - Socket # (zero based)              */ 
/*        irData          : Address of DiskInfo structure               */
/*                                                                      */
/* Returns:                                                             */
/*        FLStatus        : 0 on success, otherwise failed              */
/*----------------------------------------------------------------------*/

#define flGetDiskInfo(ioreq)        fsCall(FL_GET_DISK_INFO,ioreq)

#ifndef FS_READ_ONLY
/*----------------------------------------------------------------------*/
/*                      f l D e l e t e F i l e                         */
/*                                                                      */
/* Deletes a file.                                                      */
/*                                                                      */
/* Parameters:                                                          */
/*        irHandle        : Drive number (0, 1, ...)                    */
/*                        bits 11-8 - Logical partition in Flash partition (zero based)*/
/*                        bits 7-4 - Partition # (zero based)           */
/*                        bits 3-0 - Socket # (zero based)              */ 
/*        irPath          : path of file to delete                      */
/*        Simple path API : Points to FLSimplePath (8.3 file name,        */
/*                          defined in ioreq.h) array. First character of */
/*                          the last array member must be null.	          */
/*        ASCII API       : Points to ASCII null terminated character string */
/*        Unicode API     : Points to Unicode null terminated string      */
/*                                                                      */
/* Returns:                                                             */
/*        FLStatus        : 0 on success, otherwise failed              */
/*----------------------------------------------------------------------*/

#define flDeleteFile(ioreq)        fsCall(FL_DELETE_FILE,ioreq)

/*----------------------------------------------------------------------*/
/*                      f l T r u n c a t e F i l e                     */
/*                                                                      */
/* Truncates a file.                                                      */
/*                                                                      */
/* Parameters:                                                          */
/*        irHandle  : File handle to close.                             */
/*        irLength  : Length to truncate too. Must be less than current length  */
/*                                                                      */
/* Returns:                                                             */
/*        FLStatus        : 0 on success, otherwise failed              */
/*----------------------------------------------------------------------*/
#define flTruncateFile(ioreq)        fsCall(FL_TRUNCATE_FILE,ioreq)

#ifdef FS_RENAME_FILE

/*----------------------------------------------------------------------*/
/*                      f l R e n a m e F i l e                         */
/*                                                                      */
/* Renames a file to another name.                                      */
/*                                                                      */
/* Parameters:                                                          */
/*        irHandle        : Drive number (0, 1, ...)                    */
/*                        bits 11-8 - Logical partition in Flash partition (zero based)*/
/*                        bits 7-4 - Partition # (zero based)           */
/*                        bits 3-0 - Socket # (zero based)              */ 
/*        irPath          : path of existing file                       */
/*        irData          : path of new name.                           */
/*        Simple path API : Points to FLSimplePath (8.3 file name,        */
/*                          defined in ioreq.h) array. First character of */
/*                          the last array member must be null.	          */
/*        ASCII API       : Points to ASCII null terminated character string */
/*        Unicode API     : Points to Unicode null terminated string      */
/*                                                                      */
/* Returns:                                                             */
/*        FLStatus        : 0 on success, otherwise failed              */
/*----------------------------------------------------------------------*/

#define flRenameFile(ioreq)        fsCall(FL_RENAME_FILE,ioreq)

#endif /* FS_RENAME_FILE */

#ifdef FS_SUB_DIRECTORY

/*----------------------------------------------------------------------*/
/*                      f l M a k e D i r                               */
/*                                                                      */
/* Creates a new directory.                                             */
/*                                                                      */
/* Parameters:                                                          */
/*        irHandle        : Drive number (0, 1, ...)                    */
/*                        bits 11-8 - Logical partition in Flash partition (zero based)*/
/*                        bits 7-4 - Partition # (zero based)           */
/*                        bits 3-0 - Socket # (zero based)              */ 
/*        irPath          : path of new directory.                      */
/*        Simple path API : Points to FLSimplePath (8.3 file name,        */
/*                          defined in ioreq.h) array. First character of */
/*                          the last array member must be null.	          */
/*        ASCII API       : Points to ASCII null terminated character string */
/*        Unicode API     : Points to Unicode null terminated string      */
/*                                                                      */
/* Returns:                                                             */
/*        FLStatus        : 0 on success, otherwise failed              */
/*----------------------------------------------------------------------*/

#define flMakeDir(ioreq)        fsCall(FL_MAKE_DIR,ioreq)

/*----------------------------------------------------------------------*/
/*                      f l R e m o v e D i r                           */
/*                                                                      */
/* Removes an empty directory.                                          */
/*                                                                      */
/* Parameters:                                                          */
/*        irHandle        : Drive number (0, 1, ...)                    */
/*                        bits 11-8 - Logical partition in Flash partition (zero based)*/
/*                        bits 7-4 - Partition # (zero based)           */
/*                        bits 3-0 - Socket # (zero based)              */ 
/*        irPath          : path of directory to remove.                */
/*        Simple path API : Points to FLSimplePath (8.3 file name,        */
/*                          defined in ioreq.h) array. First character of */
/*                          the last array member must be null.	          */
/*        ASCII API       : Points to ASCII null terminated character string */
/*        Unicode API     : Points to Unicode null terminated string      */
/*                                                                      */
/* Returns:                                                             */
/*        FLStatus        : 0 on success, otherwise failed              */
/*----------------------------------------------------------------------*/

#define flRemoveDir(ioreq)        fsCall(FL_REMOVE_DIR,ioreq)

#endif /* FS_SUB_DIRECTORY */
#endif /* FS_READ_ONLY */

#endif /* FS_FILES > 0 */


/*----------------------------------------------------------------------*/
/*                      f l M o u n t V o l u m e                       */
/*                                                                      */
/* Mounts, verifies or dismounts the Flash medium.                      */
/*                                                                      */
/* In case the inserted volume has changed, or on the first access to   */
/* the file system, it should be mounted before file operations can be  */
/* done on it.                                                          */
/*                                                                      */
/* The volume automatically becomes unmounted if it is removed or       */
/* changed.                                                             */
/*                                                                      */
/* Parameters:                                                          */
/*        irHandle        : Socket number (0, 1, ...)                   */
/*                        bits 11-8 - Logical partition in Flash partition (zero based)*/
/*                        bits 7-4 - Partition # (zero based)           */
/*                        bits 3-0 - Socket # (zero based)              */
/*        irSectorNo      : First absolute sector of logical partition  */
/*                          (if bit 13 is set)                          */
/*        irSectorCount   : Number of sectors in logical partition      */
/*                          (if bit 13 is set)                          */
/*        irFlags        : 				                */
/*			  bit 14 - FL_MOUNT_FOR_BLOCK_IO 		*/
/*                        bit 13 - FL_MEDIA_WITHOUT_MBR                 */
/*                        bit 12 - FL_MOUNT_ON_GIVEN_RANGE              */
/*                                                                      */
/*                                                                      */
/* Returns:                                                             */
/*        FLStatus        : 0 on success, otherwise failed              */
/*----------------------------------------------------------------------*/

#define flLogMountVolume(ioreq)        fsCall(FL_MOUNT_VOLUME,ioreq)

/*----------------------------------------------------------------------*/
/*                   f l D i s m o u n t V o l u m e                    */
/*                                                                      */
/* Dismounts the volume.                                                */
/* This call is not normally necessary, unless it is known the volume   */
/* will soon be removed.                                                */
/*                                                                      */
/* Parameters:                                                          */
/*        irHandle        : Drive number (0, 1, ...)                    */
/*                        bits 11-8 - Logical partition in Flash partition (zero based)*/
/*                        bits 7-4 - Partition # (zero based)           */
/*                        bits 3-0 - Socket # (zero based)              */
/*                                                                      */
/* Returns:                                                             */
/*        FLStatus                : 0 on success, otherwise failed      */
/*----------------------------------------------------------------------*/

#define flLogDismountVolume(ioreq)        fsCall(FL_DISMOUNT_VOLUME,ioreq)

/*----------------------------------------------------------------------*/
/*                   f l C l o s e A l l F i l e s                      */
/*                                                                      */
/* closes all opened files, and reset the Global Attention flag.        */
/*                                                                      */
/* Parameters:                                                          */
/*        irHandle        : Drive number (0, 1, ...)                    */
/*                        bits 11-8 - Logical partition in Flash partition (zero based)*/
/*                        bits 7-4 - Partition # (zero based)           */
/*                        bits 3-0 - Socket # (zero based)              */
/*                                                                      */
/* Returns:                                                             */
/*        FLStatus                : 0 on success, otherwise failed      */
/*----------------------------------------------------------------------*/

#define flCloseAllFiles(ioreq)        fsCall(FL_CLOSE_ALL_FILES,ioreq)


/*----------------------------------------------------------------------*/
/*                 f l S e c t o r s I n V o l u m e                    */
/*                                                                      */
/* Returns number of virtual sectors in volume.                         */
/*                                                                      */
/* In case the inserted volume is not mounted, returns current status.  */
/*                                                                      */
/* Parameters:                                                          */
/*        irHandle  : Drive number (0, 1, ...)                          */
/*                        bits 11-8 - Logical partition in Flash partition (zero based)*/
/*                        bits 7-4 - Partition # (zero based)           */
/*                        bits 3-0 - Socket # (zero based)              */ 
/*                                                                      */
/* Returns:                                                             */
/*        FLStatus        : 0 on success, otherwise failed              */
/*      irLength        : number of virtual sectors in volume           */
/*----------------------------------------------------------------------*/

#define flLogSectorsInVolume(ioreq)        fsCall(FL_SECTORS_IN_VOLUME,ioreq)


#ifndef FS_READ_ONLY

#ifdef FS_FORMAT_VOLUME
/*----------------------------------------------------------------------*/
/*                      f l F o r m a t F S                             */
/*                                                                      */
/*                                                                      */
/* Parameters:                                                          */
/*        irHandle        : Socket number (0, 1, ...)                   */
/*                        bits 11-8 - Logical partition in Flash partition (zero based)*/
/*                        bits 7-4 - Partition # (zero based)           */
/*                        bits 3-0 - Socket # (zero based)              */
/*        irSectorNo      : First absolute sector of logical partition  */
/*                          (if bit 13 is set)                          */
/*        irSectorCount   : Number of sectors in logical partition   */
/*                          (if bit 13 is set)                          */
/*        irData          : Address of the FATFormatParams  structure   */
/*        irFlags        : 				                */
/*			  bit 14 - FL_DO_NOT_UPDATE_MBR 		*/
/*                        bit 13 - FL_MEDIA_WITHOUT_MBR                 */
/*                        bit 12 - FL_MOUNT_ON_GIVEN_RANGE              */
/*                                                                      */
/* Returns:                                                             */
/*        FLStatus        : 0 on success, otherwise failed              */
/*----------------------------------------------------------------------*/

#define flFormatFS(ioreq)        fsCall(FL_FORMAT_FS,ioreq)

#endif /* FS_FORMAT_VOLUME */

#ifdef FS_CREATE_LOGICAL_PARTITIONS
/*----------------------------------------------------------------------*/
/*                                                                      */
/*                                                                      */
/* Parameters:                                                          */
/*        irHandle        : Socket number (0, 1, ...)                   */
/*                        bits 7-4 - Partition # (zero based)           */
/*                        bits 3-0 - Socket # (zero based)              */
/*        irCount         : Number of LogicalPartitionParams structures */
/*        irData          : Address of array of LogicalPartitionParams structures */
/*        irFlags         : 0 - reserved                                */
/*                                                                      */
/* Returns:                                                             */
/*        FLStatus        : 0 on success, otherwise failed              */
/*----------------------------------------------------------------------*/

#define flCreateLogicalPartitions(ioreq)        fsCall(FL_CREATE_LOGICAL_PARTITIONS,ioreq)
#endif /* FS_FIND_LOGICAL_PARTITIONS */

#endif /* FS_READ_ONLY */


#ifdef FS_FIND_LOGICAL_PARTITION
/*----------------------------------------------------------------------*/
/*                                                                      */
/*                                                                      */
/* Parameters:                                                          */
/*        irHandle        : Socket number (0, 1, ...)                   */
/*                        bits 11-8 - Logical partition in Flash partition (zero based)*/
/*                        bits 7-4 - Partition # (zero based)           */
/*                        bits 3-0 - Socket # (zero based)              */
/*        irFlags         : 0 - reserved                                */
/*        irData          : Address of the LogPartitionInfo structure   */
/*                                                                      */
/* Returns:                                                             */
/*        FLStatus        : 0 on success, otherwise failed              */
/*----------------------------------------------------------------------*/

#define flFindLogicalPartition(ioreq)        fsCall(FL_FIND_LOGICAL_PARTITION,ioreq)
#endif	/* FS_FIND_LOGICAL_PARTITION */

#ifdef FS_ABS_READ_WRITE

/*----------------------------------------------------------------------*/
/*                           f l A b s R e a d                          */
/*                                                                      */
/* Reads absolute sectors by sector no.                                 */
/*                                                                      */
/* Parameters:                                                          */
/*        irHandle        : Drive number (0, 1, ...)                    */
/*                        bits 11-8 - Logical partition in Flash partition (zero based)*/
/*                        bits 7-4 - Partition # (zero based)           */
/*                        bits 3-0 - Socket # (zero based)              */ 
/*        irData          : Address of user buffer to read into         */
/*        irSectorNo      : First sector no. to read (sector 0 is the   */
/*                          DOS boot sector).                           */
/*        irSectorCount   : Number of consectutive sectors to read      */
/*                                                                      */
/* Returns:                                                             */
/*        FLStatus        : 0 on success, otherwise failed              */
/*        irSectorCount        : Number of sectors actually read        */
/*----------------------------------------------------------------------*/

#define flLogRead(ioreq)        fsCall(FL_ABS_READ,ioreq)

/*----------------------------------------------------------------------*/
/*                           f l G e t B P B                            */
/*                                                                      */
/* Reads the BIOS Parameter Block from the boot sector                  */
/*                                                                      */
/* Parameters:                                                          */
/*        irHandle        : Drive number (0, 1, ...)                    */
/*                        bits 11-8 - Logical partition in Flash partition (zero based)*/
/*                        bits 7-4 - Partition # (zero based)           */
/*                        bits 3-0 - Socket # (zero based)              */ 
/*        irData          : Address of user buffer to read BPB into     */
/*                                                                      */
/* Returns:                                                             */
/*        FLStatus        : 0 on success, otherwise failed              */
/*----------------------------------------------------------------------*/

#define flGetBPB(ioreq)                fsCall(FL_GET_BPB,ioreq)


#ifndef FS_READ_ONLY
/*----------------------------------------------------------------------*/
/*                         f l A b s W r i t e                          */
/*                                                                      */
/* Writes absolute sectors by sector no.                                */
/*                                                                      */
/* Parameters:                                                          */
/*        irHandle        : Drive number (0, 1, ...)                    */
/*                        bits 11-8 - Logical partition in Flash partition (zero based)*/
/*                        bits 7-4 - Partition # (zero based)           */
/*                        bits 3-0 - Socket # (zero based)              */ 
/*        irData          : Address of user buffer to write from        */
/*        irSectorNo      : First sector no. to write (sector 0 is the  */
/*                          DOS boot sector).                           */
/*        irSectorCount   : Number of consecutive sectors to write      */
/*                                                                      */
/* Returns:                                                             */
/*        FLStatus        : 0 on success, otherwise failed              */
/*        irSectorCount   : Number of sectors actually written          */
/*----------------------------------------------------------------------*/

#define flLogWrite(ioreq)        fsCall(FL_ABS_WRITE,ioreq)

/*----------------------------------------------------------------------*/
/*                         f l A b s D e l e t e                        */
/*                                                                      */
/* Marks absolute sectors by sector no. as deleted.                     */
/*                                                                      */
/* Parameters:                                                          */
/*        irHandle        : Drive number (0, 1, ...)                    */
/*                        bits 11-8 - Logical partition in Flash partition (zero based)*/
/*                        bits 7-4 - Partition # (zero based)           */
/*                        bits 3-0 - Socket # (zero based)              */ 
/*        irSectorNo      : First sector no. to delete (sector 0 is the */
/*                          DOS boot sector).                           */
/*        irSectorCount   : Number of consectutive sectors to delete    */
/*                                                                      */
/* Returns:                                                             */
/*        FLStatus        : 0 on success, otherwise failed              */
/*        irSectorCount        : Number of sectors actually deleted     */
/*----------------------------------------------------------------------*/

#define flLogDelete(ioreq)        fsCall(FL_ABS_DELETE,ioreq)

#endif /* FS_READ_ONLY */
#endif /* FS_ABS_READ_WRITE */


/*----------------------------------------------------------------------*/
/*                         f l A b s D e l e t e                        */
/*                                                                      */
/* Marks absolute sectors by sector no. as deleted.                     */
/*                                                                      */
/* Parameters:                                                          */
/*        irHandle        : 0 (reserved)                                */
/*        irLength        : 0 (reserved)                                */
/*        irFlags         : 0 (reserved)                                */
/*        irCount         : 0 (reserved)                                */
/*                                                                      */
/* Returns:                                                             */
/*        irLength        : current FS version number (0x6300)          */
/*        FLStatus        : 0 on success, otherwise failed              */
/*----------------------------------------------------------------------*/

#define flGetFSVersionNumber(ioreq)	fsCall(FL_GET_FS_VERSION,ioreq)
#define flInitFS(ioreq)			flGetFSVersionNumber(ioreq)

/*----------------------------------------------------------------------*/
/*                         f s C a l l                                  */
/*                                                                      */
/* Common entry-point to all file-system functions. Macros are          */
/* to call individual function, which are separately described below.   */
/*                                                                      */
/* Parameters:                                                          */
/*      function        : file-system function code (listed below)      */
/*      ioreq           : IOreq structure                               */
/*                                                                      */
/* Returns:                                                             */
/*      FLStatus        : 0 on success, otherwise failed                */
/*----------------------------------------------------------------------*/

#ifdef __cplusplus
extern "C"
{
#endif /* __cplusplus */

TFFS_DLL_API FLStatus NAMING_CONVENTION fsCall(FLFunctionNo functionNo, IOreq FAR2 *ioreq);

TFFS_DLL_API FLStatus NAMING_CONVENTION flGetSetFileInfo(IOreq FAR2 *ioreq, int fileInfoCmd);

TFFS_DLL_API FLBoolean NAMING_CONVENTION flFileOpenForReadSharedForWrite(IOreq FAR2 *ioreq);
TFFS_DLL_API FLBoolean NAMING_CONVENTION flFileOpenForWriteSharedForRead(IOreq FAR2 *ioreq);

#ifdef FS_EXIT

/*----------------------------------------------------------------------*/
/*                            f s E x i t                               */
/*                                                                      */
/* If the application ever exits, fsExit should be called before exit.  */
/* fsExit unmounts all FS volumes, then calls flExit			*/
/*                                                                      */
/* Parameters:                                                          */
/*        None                                                          */
/*                                                                      */
/* Returns:                                                             */
/*        Nothing                                                       */
/*----------------------------------------------------------------------*/
TFFS_DLL_API void NAMING_CONVENTION fsExit(void);

#endif /* FS_EXIT */


#ifndef FL_NO_LONG_FILENAMES_SUPPORT

/*-------------------------------------------------------------------------------------------
*				asciiToUnicode
* Translates ANSI character null terminated string to Unicode string.
* Input:	inBuf - points to ASCII null terminated string to be translated to Unicode
*		outBufSize - buffer size in bytes
* Output:	outBuf - buffer to receive translated Unicode string (UTF-16 or UCS-2)
*			since the range of possible values are 0..0xFF, there is no distinction between the above
* Return:	flOK or other for fault
*------------------------------------------------------------------------------------------*/

TFFS_DLL_API FLStatus NAMING_CONVENTION asciiToUnicode(FLWord *outBuf, FLByte *inBuf, FLWord outBufSize);

/*-------------------------------------------------------------------------------------------
*				f l A s c i i S t r i n g T o U n i c o d e
* Translates ANSI character null terminated string to Unicode string.
* Input:	ioreq->irData - points to ASCII null terminated string to be translated to Unicode
*					ioreq->irLength - buffer size in bytes
* Output:	ioreq->irPath - buffer to receive translated Unicode string
* Return:	flOK or other for fault
*------------------------------------------------------------------------------------------*/

#define flAsciiStringToUnicode(ioreq) asciiToUnicode((FLWord*)(ioreq)->irPath, (FLByte*)(ioreq)->irData, (FLWord)(ioreq)->irLength)


/*-------------------------------------------------------------------------------------------
*				UTF8ToUnicode
* Translates UTF8-encoded character null terminated string to Unicode string.
* Input:	inBuf - points to UTF8-encoded null terminated string to be translated
*		outBufSize - buffer size in bytes
* Output:	outBuf - buffer to receive translated Unicode string
*			if the unicode is UCS-2 or BMP-only, or your string may have
*			unpaired surrogate characters, you can use simpler CESU_8 translation
*			if CESU_8 is not defined, full UTF-16 range will be supported,
*			and translation will report error on unpaired surrogates.
*			In either case overlong detection is performed; normalization is NOT done
* Return:	flOK or other for fault
*------------------------------------------------------------------------------------------*/

TFFS_DLL_API FLStatus NAMING_CONVENTION UTF8ToUnicode(FLWord *outBuf, FLByte *inBuf, FLWord outBufSize);

/*-------------------------------------------------------------------------------------------
*				unicodeToAscii
* Translates Unicode string to ANSI character null terminated string (if possible).
* Input:	inBuf - points to Unicode null terminated string to be translated to ASCII
*			Since the range of acceptable values are 0..0xFF, there is no distinction
*			between UCS-2 and UTF-16
*		outBufSize - buffer size in bytes
* Output:	outBuf - buffer to receive translated ASCII string
* Return:	flOK or other for fault
*------------------------------------------------------------------------------------------*/


TFFS_DLL_API FLStatus NAMING_CONVENTION unicodeToAscii(FLByte *outBuf, FLWord *inBuf, FLWord outBufSize);

/*-------------------------------------------------------------------------------------------
*				f l U n i c o d e S t r i n g T o A s c i i
* Translates Unicode string to ANSI character null terminated string.
* Input:	ioreq->irData - points to Unicode null terminated string to be translated to ASCII
*					ioreq->irLength - buffer size in bytes
* Output:	ioreq->irPath - buffer to receive translated ASCII string
* Return:	flOK or other for fault
*------------------------------------------------------------------------------------------*/

#define flUnicodeStringToAscii(ioreq) unicodeToAscii((FLByte*)(ioreq)->irPath, (FLWord*)(ioreq)->irData, (FLWord)(ioreq)->irLength)


/*-------------------------------------------------------------------------------------------
*				unicodeToUTF8
* Translates Unicode string to UTF8-encoded character null terminated string (if possible).
* Input:	inBuf - points to 16-bit Unicode null terminated string to be translated.
*			if the unicode is UCS-2 or BMP-only, or your string may have
*			unpaired surrogate characters, you can use simpler CESU_8 translation
*			if CESU_8 is not defined, full UTF-16 range will be supported,
*			and translation will report error on unpaired surrogates.
*		outBufSize - buffer size in bytes
* Output:	outBuf - buffer to receive translated UTF-8 string
* Return:	flOK or other for fault
*------------------------------------------------------------------------------------------*/

TFFS_DLL_API FLStatus NAMING_CONVENTION unicodeToUTF8 (FLByte *outBuf, FLWord *inBuf, FLWord outBufSize);

/*-------------------------------------------------------------------------------------------
*	f l U n i c o d e S t r i n g L e n g t h
* Calculates Unicode string length in number of characters. The result length does not 
* contains terminating null character.
* Input:  ioreq->irPath - Unicode null terminating string
* Output: ioreq->irData - points to 32 bit value that contains number of characters not 
*							including terminating null character.
* Return: always flOK
*------------------------------------------------------------------------------------------*/

TFFS_DLL_API FLStatus NAMING_CONVENTION flUnicodeStringLength(IOreq FAR2 *ioreq);

#endif /* FL_NO_LONG_FILENAMES_SUPPORT */

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* _FSDEV_H */

