/* archColdfire.h - Motorola ColdFire specific header */

/* 
Copyright (c) 2000,2001,2005,2006 Wind River Systems, Inc. 

The right to copy, distribute, modify or otherwise make use 
of this software may be licensed only pursuant to the terms 
of an applicable Wind River license agreement. 
*/

/*
modification history
--------------------
01p,10may06,mcm  Fix for WIND00044193 - the alignment for off_t64 should not
                 be modified since it is a type that is used for more than
                 just syscalls.
01o,25apr06,rec  change sysMpuInit to sysAcrInit
01n,29mar06,rec  add sysMpuInit prototype - SPR-119290
                 redefine off_t64; - SPR-118807
01m,14feb06,scm  undef MAC & HFP, explicitly define "PORTABLE" options...
01l,19jan06,rec  fix merge error
01k,19jan06,rec  add wrsStackAdjust
01j,16dec05,rec  fix ESF_VEC_MASK value
01i,01nov05,rec	 add macros for unsupported V4e instructions
01h,18aug05,rec	 move syscall trap definition to archColdfireCommon.h
01g,01jul05,rec	 review comments
01f,23jun05,rec	 moved trcLib_PORTABLE to archColdfireCommon.h
01e,21jun05,mcm	 Use '!' instead of '|' for 'bit or' in assembly code.
01d,07apr05,rec	 base6 update
		 moved arch configuration definitions to here from other files
		 add SR bit definitions
01c,13mar01,sn	 SPR 73723 - define supported toolchains
01b,21jun00,dh	 Removed all non-coldfire stuff.
01a,17mar00,dra	 Written based on T1.0.1 Coldfire and T2 m68k
*/

#ifndef __INCarchColdfireh
#define __INCarchColdfireh

#ifdef __cplusplus
extern "C" {
#endif

#include <arch/coldfire/archColdfireCommon.h>

#if defined(_ASMLANGUAGE)
#define _BIT_OR		!
#else
#define _BIT_OR		|
#endif

/* 
 * include MAC (5206e and 53xx) CPUs
 * and V4 enhancements
 * 
 */
#define MCF52PLUS		/* additions to instruction set for MCF5200 */

#undef INCLUDE_MAC		/* multiply and accumulate instructions */
#undef INCLUDE_HW_FP            /* hardware floating point support */

#if (CPU==MCF5400)
#define INCLUDE_V4		/* Coldfire V4 instruction set enhancements */
#endif

/* This architecture uses the PORTABLE code set */

#define PORTABLE
#define _WRS_BASE6_SCHEDULER

/* defines explicitly */

#define schedLib_PORTABLE
#define workQLib_PORTABLE
#define bLib_PORTABLE
#define dllLib_PORTABLE
#define qPriHeapLib_PORTABLE
#define sllLib_PORTABLE
#define ffsLib_PORTABLE
#define semBLib_PORTABLE
#define semCLib_PORTABLE
#define semLib_PORTABLE
#define semMLib_PORTABLE
	
/* status register bit definitions */

#define SR_M_BIT    12		/* 0 = Interrupt state */
#define SR_S_BIT    13		/* 1 = Supervisor mode */
#define SR_T_BIT    15		/* 1 = Trace enabled */
#define SR_VEC_BIT  18		/* bit position of vector field */

#define SR_I_MASK   0x0700	/* mask for interrupt level field */
#define SR_M_MASK   0x1000	/* 0 = Interrupt state */
#define SR_S_MASK   0x2000	/* 1 = Supervisor mode */
#define SR_INT_DIS  (SR_I_MASK _BIT_OR SR_M_MASK _BIT_OR SR_S_MASK)
#define SR_INT_EN   (SR_M_MASK _BIT_OR SR_S_MASK)

#define ESF_VEC_MASK   0x03fc0000 /* mask for vector number field */

#define _WRS_ENABLE_IDLE_INTS	{intUnlock(SR_INT_EN);}

#define SR_FMT_MSK  0x03fc0000	/* Format mask (works with 32bit SR */
#define SR_INIT_VAL 0x40003000	/* initialized value for fs/sr */
#define FMT_INIT_VAL	0x4000	/* initialized value for format only */

#define D0	0	/* data registers 0 to 7 */
#define D1	1
#define D2	2
#define D3	3
#define D4	4
#define D5	5
#define D6	6
#define D7	7

#define A0	8	/* address registers 0 to 6 */
#define A1	9
#define A2	10
#define A3	11
#define A4	12
#define A5	13
#define A6	14
#define A7	15	/* a7 is the stack pointer */

#define SR	16	/* status register */

#define INTENT_TRAP_NUM	    1			/* interrupt enter */
#define DBG_TRAP_NUM	    2			/* debug trap number */

#define TRAP0_INST	0x4e40			/* opcode for trap 0 */
#define TRACE_BIT	0x8000			/* trace bit in 68K SR */
#define DBG_BREAK_INST	(TRAP0_INST + DBG_TRAP_NUM)	
						/* breakpoint instruction */

#define SYSCALL_SIZE	(sizeof (INSTR) / sizeof (INSTR))
#define SYSCALL_INST	(TRAP0_INST + SYSCALL_TRAP_NUM)	  

#define PUT_USP(arg)	.word	(0x4e60+arg)  /* mov.l Ay, USP */
#define GET_USP(arg)	.word	(0x4e68+arg)  /* mov.l USP, Ay */

/* missing V4e instructions from diab assembler */

#define PUT_ASID(arg)   .word   0x4e7b,(0x0003+(arg<<12)) /* movec rx, asid */
#define PUT_MMUBAR(arg) .word   0x4e7b,(0x0008+(arg<<12)) /* movec rx, mmubar */

#undef _ARCH_SUPPORTS_PROTECT_INTERRUPT_STACK /* hardware limitation */

#define _WRS_OSM_INIT

#define _ARCH_SUPPORTS_DCC

#define _ARCH_MULTIPLE_CACHELIB TRUE

/* 
 * coldfire relocates the supervisor stack to an aliased address.
 * This makes them TLB miss-proof 
 */

#define _WRS_SUP_STACK_RELOC(p) (p + wrsStackAdjust)
#define _WRS_SUP_STACK_UNRELOC(p) (p - wrsStackAdjust)

#define MMU_GLOBAL_CONTEXT  0       /* kernel uses ASID = 0 */

#ifndef _ASMLANGUAGE
extern  void    sysAcrInit (void);
extern unsigned int wrsStackAdjust;
extern unsigned int wrsDmaMallocAdjust;
#endif 

#ifdef __cplusplus
}
#endif

#endif /* __INCarchColdfireh */
