/* asmColdfire.h - Coldfire assembler definitions header file */

/* 
Copyright (c) 2000,2001,2005 Wind River Systems, Inc. 

The right to copy, distribute, modify or otherwise make use 
of this software may be licensed only pursuant to the terms 
of an applicable Wind River license agreement. 
*/

/*
modification history
--------------------
01l,28sep05,rec  add asid
01k,14jun05,rec	 detab file
01j,25may05,rec	 review comments
01i,29apr05,rec	 base6 - add floating point support,
		 sp offsets moved from dbgColdfireLib.h
01h,28nov01,dee	 add macros for adding underscores - based on compiler
01g,19oct01,dee	 Merge from T2.1.0
01f,25jun01,dee	 Add pc->%pc mapping for gnu syntax
01e,20nov00,dh	Add gnu regs for bc/dc/ic for CPUSHL instruction.
01d,20nov00,dh	Put CPUSHL back to old mc68k style.
01c,26oct00,dh	 added new 5407 cache functions
01b,14jun00,dh	 GNU support merged from earlier T2/coldfire work
01a,17mar00,dra	 Written based on T1.0.1 Coldfire and T2 m68k
*/

#ifndef __INCasmColdfireh
#define __INCasmColdfireh

#ifdef __cplusplus
extern "C" {
#endif

/* a6 offsets to arguments */

#define ARG1	8
#define ARG1W	10
#define ARG2	12
#define ARG2W	14
#define ARG3	16
#define ARG3W	18
#define ARG4	20
#define ARG5	24
#define ARG6	28
#define ARG7	32
#define ARG8	36
#define ARG9	40
#define ARG10	44
#define ARG11	48
#define ARG12	52

#define DARG1	8		/* double arguments */
#define DARG1L	12
#define DARG2	16
#define DARG2L	20
#define DARG3	24
#define DARG3L	28
#define DARG4	32
#define DARG4L	36

/* sp offsets to arguments */

#define SP_ARG0		0
#define SP_ARG1		4
#define SP_ARG1W	6
#define SP_ARG2		8
#define SP_ARG2W	10
#define SP_ARG3		12
#define SP_ARG3W	14
#define SP_ARG4		16
#define SP_ARG5		20
#define SP_ARG6		24
#define SP_ARG7		28
#define SP_ARG8		32
#define SP_ARG9		36
#define SP_ARG10	40
#define SP_ARG11	44
#define SP_ARG12	48

#ifdef _ASMLANGUAGE

/* Define a macro for the cpushl instruction */

#define CPUSHL(cache,reg)	cpushl	cache,reg

/*
 * The LEADING_UNDERSCORE macro should be defined to TRUE for toolchains
 * that do NOT prefix a leading underscore character, i.e. "_", to
 * symbols.  Define the macro to FALSE when using a toolchain that
 * does add a leading underscore character to symbols.
 */

#define DIAB_LEADING_UNDERSCORE TRUE

#if (DIAB_LEADING_UNDERSCORE == TRUE)

#define FUNC(func)	    _##func
#define FUNC_LABEL(func)    _##func:
#define VAR(name)	    _##name

/* DiabData compiler/assembler generates external symbols with leading
 * underscore and understands registers without "%".
 * So no need for strange definitions here ... ;-)
*/


#else /* DIAB_LEADING_UNDERSCORE is FALSE, using gnu */

/*
 * Added some defines to add prefixes to registers to satisfy the 2.9x gcc.
*/

#define FUNC(func)	    func
#define FUNC_LABEL(func)    func:
#define VAR(name)	    name
#define d0	%d0
#define d1	%d1
#define d2	%d2
#define d3	%d3
#define d4	%d4
#define d5	%d5
#define d6	%d6
#define d7	%d7
#define d8	%d8
#define d9	%d9
#define d10	%d10
#define d11	%d11
#define d12	%d12
#define d13	%d13
#define d14	%d14
#define d15	%d15

#define a0	%a0
#define a1	%a1
#define a2	%a2
#define a3	%a3
#define a4	%a4
#define a5	%a5
#define a6	%a6
#define a7	%a7
#define a8	%a8
#define a9	%a9
#define a10	%a10
#define a11	%a11
#define a12	%a12
#define a13	%a13
#define a14	%a14
#define a15	%a15

#define sr	%sr
#define pc	%pc

/* Define the Coldfire control registers */

#define cacr	%cacr
#define vbr	%vbr
#define sp	%sp
#define acr0	%acr0
#define acr1	%acr1
#if (CPU==MCF5400)
#define asid    %asid
#define acr2	%acr2
#define acr3	%acr3
#define mmubar	%mmubar
#endif
#define acc	%acc
#define macsr	%macsr
#define mask	%mask
#define mbar	%mbar

/* the rambar registers are in uppercase to avoid a double-expansion */

#define rambar0 %RAMBAR0
#define rambar1 %RAMBAR1
#define rambar	%RAMBAR0

/* cache sets. Not really registers, but ... */

#define bc	%bc
#define dc	%dc
#define ic	%ic

/* Here are the routines defined in assembler files: */

#define _bcopy			bcopy
#define _bcopyBytes		bcopyBytes
#define _bcopyWords		bcopyWords
#define _bcopyLongs		bcopyLongs
#define _bfill			bfill
#define _bfillBytes		bfillBytes
#define _cacheCPUSH		cacheCPUSH
#define _cacheCINV		cacheCINV
#define _cacheWriteBufferFlush	cacheWriteBufferFlush
#define _cacheDTTR0ModeSet	cacheDTTR0ModeSet
#define _cacheDTTR0Set		cacheDTTR0Set
#define _cacheDTTR0Get		cacheDTTR0Get
#define _cacheCACRSet		cacheCACRSet
#define _cacheACRSet		cacheACRSet
#define _cacheColdUnifiedDataDisable	cacheColdUnifiedDataDisable
#define _cacheColdUnifiedPushAll	cacheColdUnifiedPushAll
#define _cacheColdUnifiedPushLine	cacheColdUnifiedPushLine
#define _cacheMCF5204PushLine		cacheMCF5204PushLine
#define _cacheMCF5407DataDisable	cacheMCF5407DataDisable
#define _cacheMCF5407DataPushAll	cacheMCF5407DataPushAll
#define _cacheMCF5407DataPushLine	cacheMCF5407DataPushLine
#define _cacheMCF5407InstDisable	cacheMCF5407InstDisable
#define _cacheMCF5407InstPushAll	cacheMCF5407InstPushAll
#define _cacheMCF5407InstPushLine	cacheMCF5407InstPushLine
#define _dllInsert		dllInsert
#define _dllAdd			dllAdd
#define _dllRemove		dllRemove
#define _dllGet			dllGet
#define _excBsrTbl		excBsrTbl
#define _excStub		excStub
#define _excIntStub		excIntStub
#define _ffsMsb			ffsMsb
#define _ffs			ffs
#define _fppSave		fppSave
#define _fppRestore		fppRestore
#define _fppProbeSup		fppProbeSup
#define _fppProbeTrap		fppProbeTrap
#define _fppDtoDx		fppDtoDx
#define _fppDxtoD		fppDxtoD
#define _fpcrGet		fpcrGet
#define _fpcrSet		fpcrSet
#define _intLevelSet		intLevelSet
#define _intLock		intLock
#define _intUnlock		intUnlock
#define _intVBRSet		intVBRSet
#define _qPriBMapPut		qPriBMapPut
#define _qPriBMapGet		qPriBMapGet
#define _qPriBMapRemove		qPriBMapRemove
#define _qPriHeapUp		qPriHeapUp
#define _qPriHeapDown		qPriHeapDown
#define _semGive		semGive
#define _semTake		semTake
#define _semBGive		semBGive
#define _semBTake		semBTake
#define _semQGet		semQGet
#define _semQPut		semQPut
#define _semOTake		semOTake
#define _semClear		semClear
#define _semCGive		semCGive
#define _semCTake		semCTake
#define _semMGive		semMGive
#define _semMTake		semMTake
#define _semMGive		semMGive
#define _semMTake		semMTake
#define __sigCtxLoad		_sigCtxLoad
#define __sigCtxSave		_sigCtxSave
#define _sigsetjmp		sigsetjmp
#define _setjmp			setjmp
#define _sllPutAtHead		sllPutAtHead
#define _sllPutAtTail		sllPutAtTail
#define _sllGet			sllGet
#define _cksum			cksum
#define __insque		_insque
#define __remque		_remque
#define _a5RegGet		a5RegGet
#define _a5RegSet		a5RegSet
#define _vxMemProbeSup		vxMemProbeSup
#define _vxMemProbeTrap		vxMemProbeTrap
#define _vxTas			vxTas
#define _exit			exit
#define _vxSSEnable		vxSSEnable
#define _vxSSDisable		vxSSDisable
#define _wdbDbgBpStub		wdbDbgBpStub
#define _wdbDbgTraceStub	wdbDbgTraceStub
#define _wdbDbgBreakpoint	wdbDbgBreakpoint
#define _wdbDbgTrace		wdbDbgTrace
#define _windExit		windExit
#define _vxTaskEntry		vxTaskEntry
#define _intEnt			intEnt
#define _intExit		intExit
#define _windIntStackSet	windIntStackSet
#define _intEntTrap		intEntTrap
#define _windLoadContext	windLoadContext
#define _reschedule		reschedule
#define _workQAdd0		workQAdd0
#define _workQAdd1		workQAdd1
#define _workQAdd2		workQAdd2
#define _workQDoWork		workQDoWork
#define __func_evtLogM1		_func_evtLogM1
#define __func_evtLogT0		_func_evtLogT0
#define __func_evtLogTSched	_func_evtLogTSched
#define __func_sigTimeoutRecalc _func_sigTimeoutRecalc
#define __func_trgCheck		_func_trgCheck
#define __setjmpSetup		_setjmpSetup
#define _errno			errno
#define _evtAction		evtAction
#define _excExcHandle		excExcHandle
#define _excIntHandle		excIntHandle
#define _ffsMsbTbl		ffsMsbTbl
#define _intCnt			intCnt
#define _intExit		intExit
#define _intLockIntSR		intLockIntSR
#define _intLockMask		intLockMask
#define _intLockTaskSR		intLockTaskSR
#define _kernelIsIdle		kernelIsIdle
#define _kernelState		kernelState
#define _pJobPool		pJobPool
#define _readyQHead		readyQHead
#define _semClass		semClass
#define _semGiveDefer		semGiveDefer
#define _semGiveTbl		semGiveTbl
#define _semInstClass		semInstClass
#define _semIntRestrict		semIntRestrict
#define _semInvalid		semInvalid
#define _semMGiveKern		semMGiveKern
#define _semMGiveKernWork	semMGiveKernWork
#define _semMPendQPut		semMPendQPut
#define _semTakeTbl		semTakeTbl
#define _smObjPoolMinusOne	smObjPoolMinusOne
#define _sysBusTas		sysBusTas
#define _taskIdCurrent		taskIdCurrent
#define _taskSwapTable		taskSwapTable
#define _taskSwitchTable	taskSwitchTable
#define _trgEvtClass		trgEvtClass
#define _vxIntStackBase		vxIntStackBase
#define _windPendQGet		windPendQGet
#define _windPendQPut		windPendQPut
#define _workQIsEmpty		workQIsEmpty
#define _workQPanic		workQPanic
#define _workQReadIx		workQReadIx
#define _workQWriteIx		workQWriteIx
#define _wvEvtClass		wvEvtClass

/* New globals */

#define __coprocCfg	_coprocCfg

#endif	/* _DIAB_TOOL */
#endif	/* _ASMLANGUAGE */

#ifdef __cplusplus
}
#endif

#endif /* __INCasmColdfireh */
