/* cacheColdfireLib.h - Coldfire cache library header file */

/* 
Copyright (c) 2000,2001,2005 Wind River Systems, Inc. 

The right to copy, distribute, modify or otherwise make use 
of this software may be licensed only pursuant to the terms 
of an applicable Wind River license agreement. 
*/

/*
modification history
--------------------
01m,20mar06,rec  fix base6 cache - SPR-118709
01l,03nov05,rec  add cacheAimLibInit prototypes
01k,02nov05,scm  correct typo...
01j,01nov05,scm  modify for cache Aim support...
01i,21oct05,rec	 548x control bits
01h,19sep05,rec	 mmu support
01g,14jun05,rec	 detab file
01f,02mar01,dh	Remove composite C_CACR_CINVA and C_CACR_ENABLE macros for
		 54xx
01e,13feb01,dh	Oops. Removed a #define too soon! (C_CACR_ENABLE) ... Put it
		 back!
01d,12feb01,dh	Remove C_CACR_ENABLE macro (54xx only)
01c,18jan01,dh	Add ACR defs for other CPUs (was only for MCF5400)
01b,21jun00,dh	 Removed all non-Coldfire stuff
01a,17mar00,dra	 Written based on T1.0.1 Coldfire and T2 m68k
*/

#ifndef __INCcacheColdfireLibh
#define __INCcacheColdfireLibh

#ifdef __cplusplus
extern "C" {
#endif

#include <vxWorks.h>

#ifndef _ASMLANGUAGE

IMPORT STATUS cacheMCF5407LibInit (CACHE_MODE iMode, CACHE_MODE dMode);
IMPORT STATUS cacheMCF548xLibInit (CACHE_MODE iMode, CACHE_MODE dMode);
IMPORT STATUS cacheAim5407LibInit (CACHE_MODE iMode, CACHE_MODE dMode);
IMPORT STATUS cacheAim548xLibInit (CACHE_MODE iMode, CACHE_MODE dMode);

IMPORT   void cacheAimMCF5400ACRSet (int acrNum, int newValue);
IMPORT   void cacheAimMCF5400CACRSet (int newValue);
IMPORT   void cacheAimMCF5400InstInvalidate (void);
IMPORT   void cacheAimMCF5400DataInvalidate (void);


IMPORT ULONG    normalCACR;         /* copy of CACR register for later use */
IMPORT ULONG    currentCACR;        /* copy of CACR register for current use */

IMPORT ULONG    currentACR[4];      /* copies of ACRs for current use */
IMPORT ULONG    normalACR[4];       /* copies of ACR registers for later use */

/* definitions to allow for non-cacheAim direct calls */
                                                                                
#define cacheACRSet cacheAimMCF5400ACRSet
#define cacheCACRSet cacheAimMCF5400CACRSet
                                                                                
#endif /* _ASMLANGUAGE */

#define CACHE_INH_PRECISE	0x100	/* cache inhibited, precise mode */
#define CACHE_INH_IMPRECISE	0x200	/* cache inhibited, imprecise mode */

#define CACHE_LINE		0x1	/* CPUSH granularity */
#define CACHE_PAGE		0x2
#define CACHE_ALL		0x3

#if (CPU==MCF5400)

#define C_CACR_DEC	    0x80000000	/* enable data cache */
#define C_CACR_DW	    0x40000000	/* data write protect */
#define C_CACR_DESB	    0x20000000	/* enable data store buffer */
#define C_CACR_DDPI	    0x10000000	/* disable CPUSHL invalidation (data) */
#define C_CACR_DHLCK	    0x08000000	/* data cache half-lock */
#define C_CACR_DDCM	    0x06000000	/* data cache mode ... */
#define C_CACR_DDCM_WRTHRU  0x00000000	/* ... write through, imprecise */
#define C_CACR_DDCM_CPBACK  0x02000000	/* ... copyback */
#define C_CACR_DDCM_INHP    0x04000000	/* ... inhibited, precise */
#define C_CACR_DDCM_INHI    0x06000000	/* ... inhibited, imprecise */
#define C_CACR_DCINVA	    0x01000000	/* data cache invalidate */
#define C_CACR_BEC	    0x00080000	/* enable branch cache */
#define C_CACR_BCINVA	    0x00040000	/* branch cache invalidate */
#define C_CACR_IEC	    0x00008000	/* enable instruction cache */
#define C_CACR_DNFB	    0x00002000	/* cache-inh. fill buffer */
#define C_CACR_IDPI	    0x00001000	/* disable CPUSHL invalidation (instr)*/
#define C_CACR_IHLCK	    0x00000800	/* instruction cache half-lock */
#define C_CACR_IDCM	    0x00000400	/* instruction cache mode */
#define C_CACR_ICINVA	    0x00000100	/* instruction cache invalidate */

#define CACHE_SET_COUNT		4
#define CACHE_LINE_SHIFT	4
#define CACHE_LINE_SIZE		(1 << CACHE_LINE_SHIFT)

/* new V4e control bits */

#define C_CACR_DDSP	    0x00800000	/* default data supv prot */
#define C_CACR_IDSP	    0x00000080	/* default instruction supv prot */
#define C_CACR_EUSP	    0x00000020	/* Enable user stack protect */
#define C_CACR_DF	    0x00000080	/* Disable hardware floating point */
#define C_CACR_ZERO	    0x0073424f	/* reserved, must be 0 */

/* V4 and V4e have different cache size */

#define M548X_DATA_CACHE_SIZE		0x8000
#define M548X_INSTRUCTION_CACHE_SIZE	0x8000

#define M5407_DATA_CACHE_SIZE		0x2000
#define M5407_INSTRUCTION_CACHE_SIZE	0x4000

#endif

/* Access control registers. Same for all known Coldfires (so far ;-) */

/* for page sizes 16 meg or greater (C_ACR_AMM = 0) */
#define C_ACR_BASE	0xff000000	/* Base address (top 8 bits) */
#define C_ACR_ADMASK	0x00ff0000	/* Address mask (1 = don't care) */
#define C_ACR_ADMASK_SHIFT	 8	/* number of bits to shift size >> */

/* for page sizes 1..16 meg (C_ACR_AMM = 1) */
#define C_ACR_BASE_1M	0xfff00000	/* Base address (1-meg page sizes) */
#define C_ACR_ADMASK_1M 0x000f0000	/* Address mask (1-meg page sizes) */
#define C_ACR_ADMASK_SHIFT_1M	 4	/* number of bits to shift size >> */

#define C_ACR_RANGE	(C_ACR_BASE|C_ACR_MASK)
#define C_ACR_E		0x00008000	/* ACR enable */
#define C_ACR_S		0x00006000	/* Supervisor ... */
#define C_ACR_S_USER	0x00000000	/* ... only user access */
#define C_ACR_S_SUPER	0x00002000	/* ... only supervisor access */
#define C_ACR_S_ANY	0x00004000	/* ... any access */
#define C_ACR_AMM	0x00000400	/* Address Mask Mode */
#define C_ACR_CM	0x00000060	/* Cache mode ... */
#define C_ACR_CM_WRTHRU 0x00000000	/* ... writethrough */
#define C_ACR_CM_CPBACK 0x00000020	/* ... copyback */
#define C_ACR_CM_INHP	0x00000040	/* ... inhibited, precise */
#define C_ACR_CM_INHI	0x00000060	/* ... inhibited, imprecise */
#define C_ACR_CM_MASK	0xffffff9f      /* to mask off cache bits */
#define C_ACR_SP	0x00000008	/* Supervisor Protect */
#define C_ACR_W		0x00000004	/* write-protect (acr0 & acr1 only) */


#ifdef __cplusplus
}
#endif

#endif /* __INCcacheColdfireLibh */
