/* dsmColdfireLib.h - Coldfire disassembler header */
/* 
Copyright (c) 2000,2005 Wind River Systems, Inc. 

The right to copy, distribute, modify or otherwise make use 
of this software may be licensed only pursuant to the terms 
of an applicable Wind River license agreement. 
*/

/*
modification history
--------------------
01f,14jun05,rec  detab file
01e,25may05,rec  review comments
01d,04may05,rec  base6 mods
01c,29sep00,bittner   MAC instructions and v4 (5407) instruction set enhancements
01b,21jun00,dh   Removed all non-coldfire stuff
01a,17mar00,dra  Written based on T1.0.1 Coldfire and T2 m68k
*/

#ifndef __INCdsmColdfireLibh
#define __INCdsmColdfireLibh

#ifdef __cplusplus
extern "C" {
#endif

#include <vwModNum.h>


/* dsmLib status codes */

#define S_dsmLib_UNKNOWN_INSTRUCTION    (M_dsmLib | 1)


/* instruction types */

#define itBra           0
#define itChk           1
#define itComplete      2
#define itDynBit        3
#define itExg           4
#define itImm           5
#define itMoveB         6
#define itMovep         7
#define itMovem         8
#define itMoveq         9
#define itNbcd          10
#define itNegx          11
#define itOr            12
#define itQuick         13
#define itBcd           14
#define itRegShift      15
#define itStop          16
#define itSwap          17
#define itTrap          18
#define itX             19
#define itMoveW         20
#define itMoveL         21
#define itMoveFSR       22
#define itMoveTSR       23
#define itMoveCCR       24
#define itMoveUSP       25
#define itStatBit       26
#define itLea           27
#define itLink          28
#define itUnlk          29
#define itDb            30
#define itScc           31
#define itAdda          32
#define itCmpm          33
#define itExt           34
#define itRTD           35
#define itMoves         36
#define itMoveFCCR      37
#define itMovec         38
#define itChk2          39
#define itBfchg         40
#define itBfext         41
#define itBfins         42
#define itCallm         43
#define itLinkL         44
#define itBkpt          45
#define itTrapcc        46
#define itPack          47
/* #define itTwoWord    48   XXX unused */
#define itImmCCR        49
#define itImmTSR        50
#define itCas           51
#define itCas2          52
#define itRtm           53
#define itDivW          54
#define itDivL          55
#define itMoveA         56
#define itCpBcc         57
#define itCpDbcc        58
#define itCpTrapcc      59
#define itCpGen         60
#define itCpScc         61
#define itCpSave        62
#define itMemShift      63
#define itCinv          64
#define itCinva         65
#define itCpush         66
#define itCpusha        67
#define itMove16        68
#define itMove16L       69

/* fpp instructions */
#define itFpp           99
#define itFb            100
#define itFrestore      101
#define itFsave         102
#define itFabs          103
#define itFacos         104
#define itFadd          105
#define itFasin         106
#define itFatan         107
#define itFatanh        108
#define itFcmp          109
#define itFcos          110
#define itFcosh         111
#define itFdb           112
#define itFdiv          113
#define itFetox         114
#define itFetoxm1       115
#define itFgetexp       116
#define itFgetman       117
#define itFint          118
#define itFintrz        119
#define itFlog10        120
#define itFlog2         121
#define itFlogn         122
#define itFlognp1       123
#define itFmod          124
#define itFmove         125
#define itFmovek        126
#define itFmovel        127
#define itFmovecr       128
#define itFmovem        129
#define itFmovemc       130
#define itFmul          131
#define itFneg          132
#define itFnop          133
#define itFrem          134
#define itFscale        135
#define itFs            136
#define itFsgldiv       137
#define itFsglmul       138
#define itFsin          139
#define itFsincos       140
#define itFsinh         141
#define itFsqrt         142
#define itFsub          143
#define itFtan          144
#define itFtanh         145
#define itFtentox       146
#define itFtrap         147
#define itFtst          148
#define itFtwotox       149

#define itWDebug        154

/* Multiply and Accumulate (MAC) instructions for Coldfire 5206 & 53xx */
#define itMoveFACC      155
#define itMoveFMACSR    156
#define itMoveFMASK     157
#define itMoveTACC      158
#define itMoveTMACSR    159
#define itMoveTMASK     160
#define itMoveTCCR      161
#define itMac           162
#define itMsac          163
#define itMacl          164
#define itMsacl         165

/* Coldfire version 4 instruction set enhancements */
#define itMove3q        166
#define itMvs           167
#define itMvz           168
#define itSats          169
#define itIntouch       170

typedef struct
    {
    /* The variables are in decreasing order of size for storage optimization */
    char *name;
    int type;
    unsigned short op1;
    unsigned short op2;
    unsigned short mask1;
    unsigned short mask2;
    UINT8 modemask;
    UINT8 regmask;
    } INST;

/* Control registers */

#define SFC     0x000   /* Source Function Code register */
#define DFC     0x001   /* Destination Function Code register */
#define USP     0x800   /* User Stack Pointer */
#define VBR     0x801   /* Vector Base Register */

#define CACR    0x002   /* Cache Control Register */
#define TC      0x003   /* Translation Control Register */
#define ACR0    0x004   /* Access Control Register 0 */
#define ACR1    0x005   /* Access Control Register 1 */
#define ACR2    0x006   /* Access Control Register 2 */
#define ACR3    0x007   /* Access Control Register 3 */
#define ROMBAR  0xc00   /* ROM Base Address Register */
#define ROMBAR0 0xc04   /* ROM Base Address Register 0 */
#define ROMBAR1 0xc05   /* ROM Base Address Register 1 */
#define MBAR    0xc0f   /* Module Base Address Register */

/* function declarations */

#if defined(__STDC__) || defined(__cplusplus)

/* b6mod - add oFn, oArg args to dsmData, dsmInst prototypes */

IMPORT    int          dsmData (USHORT *binInst, int address, FUNCPTR oFn, 
                            int oArg);
IMPORT    int          dsmInst (USHORT *binInst, int address, 
                            VOIDFUNCPTR prtAddress, 
                            FUNCPTR oFn, int oArg);
IMPORT    int          dsmNbytes (USHORT *binInst);

#else

IMPORT    int          dsmData ();
IMPORT    int          dsmInst ();
IMPORT    int          dsmNbytes ();

#endif  /* __STDC__ */

#ifdef __cplusplus
}
#endif

#endif /* __INCdsmColdfireLibh */
