/* esfColdfire.h - Coldfire exception stack frames */

/*
Copyright (c) 2000,2002,2005-2006 Wind River Systems, Inc.

The right to copy, distribute, modify or otherwise make use
of this software may be licensed only pursuant to the terms
of an applicable Wind River license agreement.
*/

/*
modification history
--------------------
01g,20apr06,j_b  add ESF mask macros
01f,13apr06,mdo  Fix bug in extracting the format status bits
01e,01jul05,rec  review comments
01d,14jun05,rec  detab file
01c,14feb02,rec  SPR 70272, fix ESFCOLD_FS_UNPACK
01b,19jun00,ur   Removed all non-Coldfire stuff.
01a,17mar00,dra  Written based on T1.0.1 Coldfire and T2 m68k
*/

#ifndef __INCesfColdfireh
#define __INCesfColdfireh

#ifdef __cplusplus
extern "C" {
#endif

#ifdef  _ASMLANGUAGE

#define SIZEOF_ESFCOLD  8
#define OFF_ESF_EXCSTAT 0
#define OFF_ESF_SR      2
#define OFF_ESF_PC      4

#else

/*
 * ESFCOLD: ColdFire format stack frame.
 * Used by the ColdFire processors for all exceptions.
 */

typedef struct
    {
    unsigned int        format:4;               /* format */
    unsigned int        fshi:2;                 /* fault status (high bits) */
    unsigned int        vector:8;               /* fault vector */
    unsigned int        fslo:2;                 /* fault status (low bits) */
    unsigned int        sr:16;                  /* status register */
    INSTR *             pc;                     /* program counter */
    } ESFCOLD;

#endif  /* _ASMLANGUAGE */

#define ESFCOLD_FMT_MASK                0xF0000000
#define ESFCOLD_FS_MASK                 0x0C030000
#define ESFCOLD_VECT_MASK               0x03FC0000
#define ESFCOLD_SR_MASK                 0x0000FFFF

#define ESFCOLD_FMT_LONG_ALIGN          0x40000000

#define ESFCOLD_FS_INSTR_FETCH          4
#define ESFCOLD_FS_OP_WRITE             8
#define ESFCOLD_FS_WRITE_PROT           9
#define ESFCOLD_FS_OP_READ              12


#define ESFCOLD_FS_UNPACK(fs)   ((((fs >> 8)&0xc) | (fs & 3)) & 0xf)
#define ESFCOLD_FORMAT_UNPACK(fmt)      ((fmt) >> 12)

#ifdef __cplusplus
}
#endif

#endif /* __INCesfColdfireh */
