/* excColdfireLib.h - exception library header */

/* 
Copyright (c) 2000,2005 Wind River Systems, Inc. 

The right to copy, distribute, modify or otherwise make use 
of this software may be licensed only pursuant to the terms 
of an applicable Wind River license agreement. 
*/

/*
modification history
--------------------
01e,19oct05,rec  remove __STDC__ conditional
01d,25may05,rec  review comments
01c,02may05,rec  add user and supervisor mode macros
01b,21jun00,dh   Removed all non-coldfire stuff
01a,17mar00,dra  Written based on T1.0.1 Coldfire and T2 m68k
*/

#ifndef __INCexcColdfireLibh
#define __INCexcColdfireLibh

#ifdef __cplusplus
extern "C" {
#endif

typedef struct
    {
    UINT16  valid;      /* indicators that following fields are valid */
    UINT16  vecNum;     /* vector number */
    INSTR * pc;         /* program counter */
    ULONG   accessAddr; /* access address */
    UINT16  statusReg;  /* status register */
    UINT16  funcCode;   /* func code (or ESF type if EXC_INVALID_TYPE) */
    UINT16  instrReg;   /* instruction register */
    UINT16  pad;        /* pad to four byte boundary */
    } EXC_INFO;

#define _WRS_IS_SUPV_EXC() (((excInfo.statusReg & SR_S_MASK) == 0) ? \
                        FALSE : TRUE)
#define _WRS_IS_USER_EXC() (((excInfo.statusReg & SR_S_MASK) == 0) ? \
                        TRUE : FALSE)

/* exception info valid bits */

#define EXC_VEC_NUM             0x01    /* vector number valid */
#define EXC_PC                  0x02    /* pc valid */
#define EXC_STATUS_REG          0x04    /* status register valid */
#define EXC_ACCESS_ADDR         0x08    /* access address valid */
#define EXC_FUNC_CODE           0x10    /* function code valid */
#define EXC_INSTR_REG           0x20    /* instruction register valid */
#define EXC_FSLW                0x40    /* fault status long word valid */
#define EXC_INVALID_TYPE        0x80    /* special indicator: ESF type was bad;
                                         * type is in funcCode field */


/* vector numbers of specific exceptions */

#define LOW_VEC         2       /* lowest vector initialized */
#define BUS_ERROR_VEC   2       /* bus error vector */
#define ADRS_ERROR_VEC  3       /* address error vector */
#define FORMAT_ERROR    14      /* last program error vector */
#define TRAP_0_VEC      32      /* start of trap vectors */
#define USER_VEC_START  64      /* start of user interrupt vectors */
#define HIGH_VEC        255     /* highest vector initialized */


/* variable declarations */

extern FUNCPTR  excExcepHook;   /* add'l rtn to call when exceptions occur */

/* function declarations */

extern void     excStub (void);
extern void     excIntStub (void);

#ifdef __cplusplus
}
#endif

#endif /* __INCexcColdfireLibh */
