/* fppColdfireLib.h - Coldfire floating-point coprocessor support library header */

/*
Copyright (c) 2000,2005-2006 Wind River Systems, Inc.

The right to copy, distribute, modify or otherwise make use
of this software may be licensed only pursuant to the terms
of an applicable Wind River license agreement.
*/

/*
modification history
--------------------
01e,30mar06,mdo  Update to add FP state frame contents
01d,14jun05,rec  detab file
01c,23may05,rec  colfire port
01b,21jun00,dh   Removed all non-coldfire stuff
01a,17mar00,dra  Written based on T1.0.1 Coldfire and T2 m68k
*/

#ifndef __INCfppColdfireLibh
#define __INCfppColdfireLibh

#ifdef __cplusplus
extern "C" {
#endif

/* FP_CONTEXT structure offsets */

#define FPCR            0x02    /* OFFSET(FP_CONTEXT, fpcr)             */
#define FPSR            0x0C    /* OFFSET(FP_CONTEXT, fpsr)             */
#define FPIAR           0x50    /* OFFSET(FP_CONTEXT, fpiar)            */
#define FP              0x10    /* OFFSET(FP_CONTEXT, fp[0])            */


#ifndef _ASMLANGUAGE

/* number of fp registers on coprocessor */

#define FP_NUM_DREGS    8

typedef struct fpStateFrame
	{
	UINT16 formatWord;
	UINT16 fpcr;
	UINT32 excOperand_hi;
	UINT32 excOperand_lo;
	UINT32 fpsr;
	} FP_STATE_FRAME;

typedef struct fpContext        /* FP_CONTEXT */
    {
    FP_STATE_FRAME fpFrame;       /* floating-point state frame */
    double      fp[FP_NUM_DREGS];       /* 8  doubles           :  64 */
    int         fpiar;                  /*                      :   4 */
    } FP_CONTEXT;

#define FPREG_SET FP_CONTEXT

#define FP_OFFSET(n)    (FP + (n)*sizeof(double))
	
#define FP_CR_INITVAL   0   /* configuration value for FPCR */

/* variable declarations */

extern REG_INDEX fpRegName[];           /* f-point data register table */
extern REG_INDEX fpCtlRegName[];        /* f-point control register table */


/* function declarations */

extern void         fppArchInit (void);
extern FP_CONTEXT * fppCtxCreate (int tid);
extern STATUS       fppCtxDelete (FP_CONTEXT * pCtx);
extern STATUS       fppProbe (void);
extern void         fppSave (FP_CONTEXT *  pFpContext);
extern void         fppRestore (FP_CONTEXT *  pFpContext);
extern STATUS       fppProbeSup (void);
extern void         fppProbeTrap (void);
extern int          fpcrGet (void);
extern void         fpcrSet (int value);
extern void         fppEnable (void);
extern void         fppDisable (void);

#endif  /* _ASMLANGUAGE */

#ifdef __cplusplus
}
#endif

#endif /* __INCfppColdfireLibh */



