/* macColdfireLib.h - Coldfire MAC coprocessor support library header */

/* 
Copyright (c) 2005 Wind River Systems, Inc. 

The right to copy, distribute, modify or otherwise make use 
of this software may be licensed only pursuant to the terms 
of an applicable Wind River license agreement. 
*/

/*
modification history
--------------------
01b,14jun05,rec  detab file
01a,23may05,rec  written
*/

#ifndef __INCmacColdfireLibh
#define __INCmacColdfireLibh

#ifdef __cplusplus
extern "C" {
#endif

#include <regs.h>

/* MAC_CONTEXT structure offsets */

#define MAC             0x00    /* OFFSET(MAC_CONTEXT, mac)             */
#define MACSR           0x04    /* OFFSET(MAC_CONTEXT, macsr)           */
#define MACMASK         0x08    /* OFFSET(MAC_CONTEXT, maciar)          */


#ifndef _ASMLANGUAGE

typedef struct macregSet                        /* MACREG_SET */
    {
    int         mac;                    /*                      :   4 */
    int         macsr;                  /*                      :   4 */
    int         mask;                   /*                      :   4 */
    } MACREG_SET;

typedef struct macContext       /* MAC_CONTEXT */
    {
    MACREG_SET   macRegSet;     /* floating-point register set */
    } MAC_CONTEXT;                              /*                TOTAL : 76 */

/* variable declarations */

extern REG_INDEX macRegName[];          /* f-point data register table */
extern REG_INDEX macCtlRegName[];       /* f-point control register table */


/* function declarations */

extern void         macArchInit (void);
extern MAC_CONTEXT * macCtxCreate (int tid);
extern STATUS       macCtxDelete (MAC_CONTEXT * pCtx);
extern STATUS       macProbe (void);
extern void         macSave (MAC_CONTEXT *  pFpContext);
extern void         macRestore (MAC_CONTEXT *  pFpContext);
extern STATUS       macProbeSup (void);
extern void         macProbeTrap (void);

#endif  /* _ASMLANGUAGE */

#ifdef __cplusplus
}
#endif

#endif /* __INCmacColdfireLibh */



