/* mcf5475.h - Freescale MCF5475 CPU control registers */

/* Copyright 2005-2006 Wind River Systems, Inc. */

/*
modification history
--------------------
01f,24mar06,j_b  add arbiter register bit definitions
01e,08feb06,mdo  Add EPORT defines
01d,21dec05,mdo  Add additional PCI controller defines
01c,02dec05,j_b  fix comment formatting for apigen
01b,03nov05,j_b  add parentheses around 'x' arguments
01a,06jun05,lsg  Created from m5282.h
*/

/*
 * This file contains I/O addresses and related constants for the MCF5475.
 * Each section is referenced to the appropriate chapter in "MCF5475 Reference
 * Manual".
 */

#ifndef __INCmcf5475h
#define __INCmcf5475h

#ifdef __cplusplus
extern "C" {
#endif

/*
 * Need to use a distinct cast macro in order to not conflict with other
 * include files.
 */

#ifdef  _ASMLANGUAGE
#define CAST_M5475(x)
#else   /* _ASMLANGUAGE */
#define CAST_M5475(x)       (x)
#endif  /* _ASMLANGUAGE */

/*
 *
 * Chapter 9 System Integration Unit
 *
 */

/* Reset Status Register */
#define M5475_RSR(base)    (CAST_M5475(volatile UINT32 *)((base) + 0x00000044))

/* Bit definitions and macros for Reset Status Register */
#define RSR_RST     0x00000001
#define RSR_RSTWD   0x00000002
#define RSR_RSTJTG  0x00000008

/* JTAG ID Register */
#define M5475_JTAGID(base)  (CAST_M5475(volatile UINT32 *)((base) + 0x00000050))

/* Bit definitions and macros for JTAG ID Register */
#define JTAGID_REV          (0xF0000000)
#define JTAGID_PROCESSOR    (0x0FFFFFFF)
#define JTAGID_MCF5485      (0x0800C01D)
#define JTAGID_MCF5484      (0x0800D01D)
#define JTAGID_MCF5483      (0x0800E01D)
#define JTAGID_MCF5482      (0x0800F01D)
#define JTAGID_MCF5481      (0x0801001D)
#define JTAGID_MCF5480      (0x0801101D)
#define JTAGID_MCF5475      (0x0801201D)
#define JTAGID_MCF5474      (0x0801301D)
#define JTAGID_MCF5473      (0x0801401D)
#define JTAGID_MCF5472      (0x0801501D)
#define JTAGID_MCF5471      (0x0801601D)
#define JTAGID_MCF5470      (0x0801701D)

/*
 *
 * Internal Clocks and bus architecture - Chapter 10
 *
 */
/* Extended Local (XL) bus Arbiter Register */
#define M5475_XARB_CFG(base)    \
                          (CAST_M5475(volatile UINT32 *)((base) + 0x00000240))
#define M5475_XARB_VER(base)    \
                          (CAST_M5475(volatile UINT32 *)((base) + 0x00000244))
#define M5475_XARB_SR(base)     \
                          (CAST_M5475(volatile UINT32 *)((base) + 0x00000248))
#define M5475_XARB_IMR(base)    \
                          (CAST_M5475(volatile UINT32 *)((base) + 0x0000024C))
#define M5475_XARB_ADRCAP(base) \
                          (CAST_M5475(volatile UINT32 *)((base) + 0x00000250))
#define M5475_XARB_SIGCAP(base) \
                          (CAST_M5475(volatile UINT32 *)((base) + 0x00000254))
#define M5475_XARB_ADRTO(base)  \
                          (CAST_M5475(volatile UINT32 *)((base) + 0x00000258))
#define M5475_XARB_DATTO(base)  \
                          (CAST_M5475(volatile UINT32 *)((base) + 0x0000025c))
#define M5475_XARB_BUSTO(base)  \
                          (CAST_M5475(volatile UINT32 *)((base) + 0x00000260))
#define M5475_XARB_PRIEN(base)  \
                          (CAST_M5475(volatile UINT32 *)((base) + 0x00000264))
#define M5475_XARB_PRI(base)    \
                          (CAST_M5475(volatile UINT32 *)((base) + 0x00000268))


/* Bit definitions and macros for M5475_XLB */
#define M5475_XARB_CFG_PLDIS    (0x80000000)
#define M5475_XARB_CFG_SP(x)    (((x) & 0x7) << 8)
#define M5475_XARB_CFG_PM(x)    (((x) & 0x3) << 5)
#define M5475_XARB_CFG_BA       (0x00000008)
#define M5475_XARB_CFG_DT       (0x00000004)
#define M5475_XARB_CFG_AT       (0x00000002)

#define M5475_XARB_SR_SEA       (0x00000100)
#define M5475_XARB_SR_MM        (0x00000080)
#define M5475_XARB_SR_TTA       (0x00000040)
#define M5475_XARB_SR_TTR       (0x00000020)
#define M5475_XARB_SR_ECW       (0x00000010)
#define M5475_XARB_SR_TTM       (0x00000008)
#define M5475_XARB_SR_BA        (0x00000004)
#define M5475_XARB_SR_DT        (0x00000002)
#define M5475_XARB_SR_AT        (0x00000001)

#define M5475_XARB_IMR_SEAE     (0x00000100)
#define M5475_XARB_IMR_MME      (0x00000080)
#define M5475_XARB_IMR_TTAE     (0x00000040)
#define M5475_XARB_IMR_TTRE     (0x00000020)
#define M5475_XARB_IMR_ECWE     (0x00000010)
#define M5475_XARB_IMR_TTME     (0x00000008)
#define M5475_XARB_IMR_BAE      (0x00000004)
#define M5475_XARB_IMR_DTE      (0x00000002)
#define M5475_XARB_IMR_ATE      (0x00000001)


/*
 *
 * General purpose timers - Chapter 11
 *
 */

/*
 *
 * Interrupt Controller Modules - Chapter 13
 *
 */

/* Interrupt Controller Units */

#define M5475_INTC0     0

/* Register read/write macros */
#define M5475_INTC_IPRH(base)    (CAST_M5475(volatile UINT32 *)((base) + 0x700))
#define M5475_INTC_IPRL(base)    (CAST_M5475(volatile UINT32 *)((base) + 0x704))
#define M5475_INTC_IMRH(base)    (CAST_M5475(volatile UINT32 *)((base) + 0x708))
#define M5475_INTC_IMRL(base)    (CAST_M5475(volatile UINT32 *)((base) + 0x70C))
#define M5475_INTC_INTFRCH(base) (CAST_M5475(volatile UINT32 *)((base) + 0x710))
#define M5475_INTC_INTFRCL(base) (CAST_M5475(volatile UINT32 *)((base) + 0x714))
#define M5475_INTC_IRLR(base)    (CAST_M5475(volatile UINT8  *)((base) + 0x718))
#define M5475_INTC_IACKLPR(base) (CAST_M5475(volatile UINT8  *)((base) + 0x719))
#define M5475_INTC_ICR(base,n)  \
                   (CAST_M5475(volatile UINT8  *)((base) + 0x740 + (n) * 0x001))
#define M5474_INTC_SWIACK(base)  (CAST_M5475(volatile UINT8  *)((base) + 0x7e0))
#define M5475_INTC_LnIACK(base,n)   \
                   (CAST_M5475(volatile UINT8  *)((base) + 0x7E4 + (n) * 0x004))

/* Bit definitions and macros for M5475_INTC_IPRH */

#define M5475_INTC_IPRH_INT32          (0x00000001)
#define M5475_INTC_IPRH_INT33          (0x00000002)
#define M5475_INTC_IPRH_INT34          (0x00000004)
#define M5475_INTC_IPRH_INT35          (0x00000008)
#define M5475_INTC_IPRH_INT36          (0x00000010)
#define M5475_INTC_IPRH_INT37          (0x00000020)
#define M5475_INTC_IPRH_INT38          (0x00000040)
#define M5475_INTC_IPRH_INT39          (0x00000080)
#define M5475_INTC_IPRH_INT40          (0x00000100)
#define M5475_INTC_IPRH_INT41          (0x00000200)
#define M5475_INTC_IPRH_INT42          (0x00000400)
#define M5475_INTC_IPRH_INT43          (0x00000800)
#define M5475_INTC_IPRH_INT44          (0x00001000)
#define M5475_INTC_IPRH_INT45          (0x00002000)
#define M5475_INTC_IPRH_INT46          (0x00004000)
#define M5475_INTC_IPRH_INT47          (0x00008000)
#define M5475_INTC_IPRH_INT48          (0x00010000)
#define M5475_INTC_IPRH_INT49          (0x00020000)
#define M5475_INTC_IPRH_INT50          (0x00040000)
#define M5475_INTC_IPRH_INT51          (0x00080000)
#define M5475_INTC_IPRH_INT52          (0x00100000)
#define M5475_INTC_IPRH_INT53          (0x00200000)
#define M5475_INTC_IPRH_INT54          (0x00400000)
#define M5475_INTC_IPRH_INT55          (0x00800000)
#define M5475_INTC_IPRH_INT56          (0x01000000)
#define M5475_INTC_IPRH_INT57          (0x02000000)
#define M5475_INTC_IPRH_INT58          (0x04000000)
#define M5475_INTC_IPRH_INT59          (0x08000000)
#define M5475_INTC_IPRH_INT60          (0x10000000)
#define M5475_INTC_IPRH_INT61          (0x20000000)
#define M5475_INTC_IPRH_INT62          (0x40000000)
#define M5475_INTC_IPRH_INT63          (0x80000000)

/* Bit definitions and macros for M5475_INTC_IPRL */

#define M5475_INTC_IPRL_INT1           (0x00000002)
#define M5475_INTC_IPRL_INT2           (0x00000004)
#define M5475_INTC_IPRL_INT3           (0x00000008)
#define M5475_INTC_IPRL_INT4           (0x00000010)
#define M5475_INTC_IPRL_INT5           (0x00000020)
#define M5475_INTC_IPRL_INT6           (0x00000040)
#define M5475_INTC_IPRL_INT7           (0x00000080)
#define M5475_INTC_IPRL_INT8           (0x00000100)
#define M5475_INTC_IPRL_INT9           (0x00000200)
#define M5475_INTC_IPRL_INT10          (0x00000400)
#define M5475_INTC_IPRL_INT11          (0x00000800)
#define M5475_INTC_IPRL_INT12          (0x00001000)
#define M5475_INTC_IPRL_INT13          (0x00002000)
#define M5475_INTC_IPRL_INT14          (0x00004000)
#define M5475_INTC_IPRL_INT15          (0x00008000)
#define M5475_INTC_IPRL_INT16          (0x00010000)
#define M5475_INTC_IPRL_INT17          (0x00020000)
#define M5475_INTC_IPRL_INT18          (0x00040000)
#define M5475_INTC_IPRL_INT19          (0x00080000)
#define M5475_INTC_IPRL_INT20          (0x00100000)
#define M5475_INTC_IPRL_INT21          (0x00200000)
#define M5475_INTC_IPRL_INT22          (0x00400000)
#define M5475_INTC_IPRL_INT23          (0x00800000)
#define M5475_INTC_IPRL_INT24          (0x01000000)
#define M5475_INTC_IPRL_INT25          (0x02000000)
#define M5475_INTC_IPRL_INT26          (0x04000000)
#define M5475_INTC_IPRL_INT27          (0x08000000)
#define M5475_INTC_IPRL_INT28          (0x10000000)
#define M5475_INTC_IPRL_INT29          (0x20000000)
#define M5475_INTC_IPRL_INT30          (0x40000000)
#define M5475_INTC_IPRL_INT31          (0x80000000)

/* Bit definitions and macros for M5475_INTC_IMRH */

#define M5475_INTC_IMRH_INT_MASK32     (0x00000001)
#define M5475_INTC_IMRH_INT_MASK33     (0x00000002)
#define M5475_INTC_IMRH_INT_MASK34     (0x00000004)
#define M5475_INTC_IMRH_INT_MASK35     (0x00000008)
#define M5475_INTC_IMRH_INT_MASK36     (0x00000010)
#define M5475_INTC_IMRH_INT_MASK37     (0x00000020)
#define M5475_INTC_IMRH_INT_MASK38     (0x00000040)
#define M5475_INTC_IMRH_INT_MASK39     (0x00000080)
#define M5475_INTC_IMRH_INT_MASK40     (0x00000100)
#define M5475_INTC_IMRH_INT_MASK41     (0x00000200)
#define M5475_INTC_IMRH_INT_MASK42     (0x00000400)
#define M5475_INTC_IMRH_INT_MASK43     (0x00000800)
#define M5475_INTC_IMRH_INT_MASK44     (0x00001000)
#define M5475_INTC_IMRH_INT_MASK45     (0x00002000)
#define M5475_INTC_IMRH_INT_MASK46     (0x00004000)
#define M5475_INTC_IMRH_INT_MASK47     (0x00008000)
#define M5475_INTC_IMRH_INT_MASK48     (0x00010000)
#define M5475_INTC_IMRH_INT_MASK49     (0x00020000)
#define M5475_INTC_IMRH_INT_MASK50     (0x00040000)
#define M5475_INTC_IMRH_INT_MASK51     (0x00080000)
#define M5475_INTC_IMRH_INT_MASK52     (0x00100000)
#define M5475_INTC_IMRH_INT_MASK53     (0x00200000)
#define M5475_INTC_IMRH_INT_MASK54     (0x00400000)
#define M5475_INTC_IMRH_INT_MASK55     (0x00800000)
#define M5475_INTC_IMRH_INT_MASK56     (0x01000000)
#define M5475_INTC_IMRH_INT_MASK57     (0x02000000)
#define M5475_INTC_IMRH_INT_MASK58     (0x04000000)
#define M5475_INTC_IMRH_INT_MASK59     (0x08000000)
#define M5475_INTC_IMRH_INT_MASK60     (0x10000000)
#define M5475_INTC_IMRH_INT_MASK61     (0x20000000)
#define M5475_INTC_IMRH_INT_MASK62     (0x40000000)
#define M5475_INTC_IMRH_INT_MASK63     (0x80000000)

/* Bit definitions and macros for M5475_INTC_IMRL */

#define M5475_INTC_IMRL_MASKALL        (0x00000001)
#define M5475_INTC_IMRL_INT_MASK1      (0x00000002)
#define M5475_INTC_IMRL_INT_MASK2      (0x00000004)
#define M5475_INTC_IMRL_INT_MASK3      (0x00000008)
#define M5475_INTC_IMRL_INT_MASK4      (0x00000010)
#define M5475_INTC_IMRL_INT_MASK5      (0x00000020)
#define M5475_INTC_IMRL_INT_MASK6      (0x00000040)
#define M5475_INTC_IMRL_INT_MASK7      (0x00000080)
#define M5475_INTC_IMRL_INT_MASK8      (0x00000100)
#define M5475_INTC_IMRL_INT_MASK9      (0x00000200)
#define M5475_INTC_IMRL_INT_MASK10     (0x00000400)
#define M5475_INTC_IMRL_INT_MASK11     (0x00000800)
#define M5475_INTC_IMRL_INT_MASK12     (0x00001000)
#define M5475_INTC_IMRL_INT_MASK13     (0x00002000)
#define M5475_INTC_IMRL_INT_MASK14     (0x00004000)
#define M5475_INTC_IMRL_INT_MASK15     (0x00008000)
#define M5475_INTC_IMRL_INT_MASK16     (0x00010000)
#define M5475_INTC_IMRL_INT_MASK17     (0x00020000)
#define M5475_INTC_IMRL_INT_MASK18     (0x00040000)
#define M5475_INTC_IMRL_INT_MASK19     (0x00080000)
#define M5475_INTC_IMRL_INT_MASK20     (0x00100000)
#define M5475_INTC_IMRL_INT_MASK21     (0x00200000)
#define M5475_INTC_IMRL_INT_MASK22     (0x00400000)
#define M5475_INTC_IMRL_INT_MASK23     (0x00800000)
#define M5475_INTC_IMRL_INT_MASK24     (0x01000000)
#define M5475_INTC_IMRL_INT_MASK25     (0x02000000)
#define M5475_INTC_IMRL_INT_MASK26     (0x04000000)
#define M5475_INTC_IMRL_INT_MASK27     (0x08000000)
#define M5475_INTC_IMRL_INT_MASK28     (0x10000000)
#define M5475_INTC_IMRL_INT_MASK29     (0x20000000)
#define M5475_INTC_IMRL_INT_MASK30     (0x40000000)
#define M5475_INTC_IMRL_INT_MASK31     (0x80000000)

/* Bit definitions and macros for M5475_INTC_INTFRCH */

#define M5475_INTC_INTFRCH_INTFRC32    (0x00000001)
#define M5475_INTC_INTFRCH_INTFRC33    (0x00000002)
#define M5475_INTC_INTFRCH_INTFRC34    (0x00000004)
#define M5475_INTC_INTFRCH_INTFRC35    (0x00000008)
#define M5475_INTC_INTFRCH_INTFRC36    (0x00000010)
#define M5475_INTC_INTFRCH_INTFRC37    (0x00000020)
#define M5475_INTC_INTFRCH_INTFRC38    (0x00000040)
#define M5475_INTC_INTFRCH_INTFRC39    (0x00000080)
#define M5475_INTC_INTFRCH_INTFRC40    (0x00000100)
#define M5475_INTC_INTFRCH_INTFRC41    (0x00000200)
#define M5475_INTC_INTFRCH_INTFRC42    (0x00000400)
#define M5475_INTC_INTFRCH_INTFRC43    (0x00000800)
#define M5475_INTC_INTFRCH_INTFRC44    (0x00001000)
#define M5475_INTC_INTFRCH_INTFRC45    (0x00002000)
#define M5475_INTC_INTFRCH_INTFRC46    (0x00004000)
#define M5475_INTC_INTFRCH_INTFRC47    (0x00008000)
#define M5475_INTC_INTFRCH_INTFRC48    (0x00010000)
#define M5475_INTC_INTFRCH_INTFRC49    (0x00020000)
#define M5475_INTC_INTFRCH_INTFRC50    (0x00040000)
#define M5475_INTC_INTFRCH_INTFRC51    (0x00080000)
#define M5475_INTC_INTFRCH_INTFRC52    (0x00100000)
#define M5475_INTC_INTFRCH_INTFRC53    (0x00200000)
#define M5475_INTC_INTFRCH_INTFRC54    (0x00400000)
#define M5475_INTC_INTFRCH_INTFRC55    (0x00800000)
#define M5475_INTC_INTFRCH_INTFRC56    (0x01000000)
#define M5475_INTC_INTFRCH_INTFRC57    (0x02000000)
#define M5475_INTC_INTFRCH_INTFRC58    (0x04000000)
#define M5475_INTC_INTFRCH_INTFRC59    (0x08000000)
#define M5475_INTC_INTFRCH_INTFRC60    (0x10000000)
#define M5475_INTC_INTFRCH_INTFRC61    (0x20000000)
#define M5475_INTC_INTFRCH_INTFRC62    (0x40000000)
#define M5475_INTC_INTFRCH_INTFRC63    (0x80000000)

/* Bit definitions and macros for M5475_INTC_INTFRCL */

#define M5475_INTC_INTFRCL_INTFRC1     (0x00000002)
#define M5475_INTC_INTFRCL_INTFRC2     (0x00000004)
#define M5475_INTC_INTFRCL_INTFRC3     (0x00000008)
#define M5475_INTC_INTFRCL_INTFRC4     (0x00000010)
#define M5475_INTC_INTFRCL_INTFRC5     (0x00000020)
#define M5475_INTC_INTFRCL_INT6        (0x00000040)
#define M5475_INTC_INTFRCL_INT7        (0x00000080)
#define M5475_INTC_INTFRCL_INT8        (0x00000100)
#define M5475_INTC_INTFRCL_INT9        (0x00000200)
#define M5475_INTC_INTFRCL_INT10       (0x00000400)
#define M5475_INTC_INTFRCL_INTFRC11    (0x00000800)
#define M5475_INTC_INTFRCL_INTFRC12    (0x00001000)
#define M5475_INTC_INTFRCL_INTFRC13    (0x00002000)
#define M5475_INTC_INTFRCL_INTFRC14    (0x00004000)
#define M5475_INTC_INTFRCL_INT15       (0x00008000)
#define M5475_INTC_INTFRCL_INTFRC16    (0x00010000)
#define M5475_INTC_INTFRCL_INTFRC17    (0x00020000)
#define M5475_INTC_INTFRCL_INTFRC18    (0x00040000)
#define M5475_INTC_INTFRCL_INTFRC19    (0x00080000)
#define M5475_INTC_INTFRCL_INTFRC20    (0x00100000)
#define M5475_INTC_INTFRCL_INTFRC21    (0x00200000)
#define M5475_INTC_INTFRCL_INTFRC22    (0x00400000)
#define M5475_INTC_INTFRCL_INTFRC23    (0x00800000)
#define M5475_INTC_INTFRCL_INTFRC24    (0x01000000)
#define M5475_INTC_INTFRCL_INTFRC25    (0x02000000)
#define M5475_INTC_INTFRCL_INTFRC26    (0x04000000)
#define M5475_INTC_INTFRCL_INTFRC27    (0x08000000)
#define M5475_INTC_INTFRCL_INTFRC28    (0x10000000)
#define M5475_INTC_INTFRCL_INTFRC29    (0x20000000)
#define M5475_INTC_INTFRCL_INTFRC30    (0x40000000)
#define M5475_INTC_INTFRCL_INTFRC31    (0x80000000)

/* Bit definitions and macros for M5475_INTC_IRLR */

#define M5475_INTC_IRLR_IRQ(x)         (((x) & 0x7F) << 1)

/* Bit definitions and macros for M5475_INTC_IACKLPR */

#define M5475_INTC_IACKLPR_PRI(x)      (((x) & 0x0F) << 0)
#define M5475_INTC_IACKLPR_LEVEL(x)    (((x) & 0x07) << 4)

/* Bit definitions and macros for M5475_INTC_ICRn */

#define M5475_INTC_ICRn_IP(x)          (((x) & 0x07) << 0)
#define M5475_INTC_ICRn_IL(x)          (((x) & 0x07) << 3)

/*
 * EPORT
 */
#define M5475_EPORT1    1
#define M5475_EPORT2    2
#define M5475_EPORT3    3
#define M5475_EPORT4    4
#define M5475_EPORT5    5
#define M5475_EPORT6    6
#define M5475_EPORT7    7

#define M5475_EPORT_EPPAR(base) (CAST_M5475(volatile UINT16 *)((base) + 0xF00))
#define M5475_EPORT_EPPAR_LEVEL 0x00
#define M5475_EPORT_EPPAR_RISE  0x01
#define M5475_EPORT_EPPAR_FALL  0x02
#define M5475_EPORT_EPPAR_RANDF 0x03
#define M5475_EPORT_EPPAR_SET(x,y) (x) << 2*(y);

#define M5475_EPORT_EPIER(base) (CAST_M5475(volatile UINT8 *)((base) + 0xF05))
#define M5475_EPIER_EPORT1  1 << M5475_EPORT1
#define M5475_EPIER_EPORT2  1 << M5475_EPORT2
#define M5475_EPIER_EPORT3  1 << M5475_EPORT3
#define M5475_EPIER_EPORT4  1 << M5475_EPORT4
#define M5475_EPIER_EPORT5  1 << M5475_EPORT5
#define M5475_EPIER_EPORT6  1 << M5475_EPORT6
#define M5475_EPIER_EPORT7  1 << M5475_EPORT7

#define M5475_EPORT_EPFR(base) (CAST_M5475(volatile UINT8 *)((base) + 0xF0C))

/*
 *
 * GPIO Module - Chapter 15
 *
 */
/* Register read/write macros */

/* GPIO Modules */
#define PODR    0x0000
#define PDDR    0x0010
#define PPDSDR  0x0020
#define PCLRR   0x0030

/* GPIO Registers */
#define FBCTL   0x0000
#define FBCS    0x0001
#define DMA     0x0002
#define FEC0H   0x0004
#define FEC0L   0x0005
#define FEC1H   0x0006
#define FEC1L   0x0007
#define FECI2C  0x0008
#define PCIBG   0x0009
#define PCIBR   0x000a
#define PSC3PSC2 0x000c
#define PSC1PSC0 0x000d
#define DSPI    0x000e

#define M5475_GPIO_REG(base, module, reg)   \
                ((base) + 0x00000A00 + (module) + (reg))

#define M5475_GPIO(base,module,reg) \
                (CAST_M5475(volatile UINT8 *) M5475_GPIO_REG(base,module,reg))

#define M5475_GPIO_PAR_FBCTL(base)  \
                (CAST_M5475(volatile UINT16 *)((base) + 0x000A40))
#define M5475_GPIO_PAR_FBCS(base)   \
                (CAST_M5475(volatile UINT8 *)((base) + 0x000A42))
#define M5475_GPIO_PAR_DMA(base)    \
                (CAST_M5475(volatile UINT8 *)((base) + 0x000A43))
#define M5475_GPIO_PAR_FECI2CIRQ(base)  \
                (CAST_M5475(volatile UINT16 *)((base) + 0x000A44))
#define M5475_GPIO_PAR_PCIBG(base)  \
                (CAST_M5475(volatile UINT16 *)((base) + 0x000A48))
#define M5475_GPIO_PAR_PCIBG_ALL        0x000003ff
#define M5475_GPIO_PAR_PCIBR(base)  \
                (CAST_M5475(volatile UINT16 *)((base) + 0x000A4A))
#define M5475_GPIO_PAR_PCIBR_ALL        0x000003ff
#define M5475_GPIO_PAR_PSC3(base)   \
                (CAST_M5475(volatile UINT8 *)((base) + 0x000A4C))
#define M5475_GPIO_PAR_PSC2(base)   \
                (CAST_M5475(volatile UINT8 *)((base) + 0x000a4D))
#define M5475_GPIO_PAR_PSC1(base)   \
                (CAST_M5475(volatile UINT8 *)((base) + 0x000a4E))
#define M5475_GPIO_PAR_PSC0(base)   \
                (CAST_M5475(volatile UINT8 *)((base) + 0x000a4F))
#define M5475_GPIO_PAR_DSPI(base)   \
                (CAST_M5475(volatile UINT16 *)((base) + 0x000a50))
#define M5475_GPIO_PAR_TIMER(base)  \
                (CAST_M5475(volatile UINT8 *)((base) + 0x000a52))

#define M5475_GPIO_PAR_FBCS_CS(x)     ((0x1 << x)&0x3e)

#define M5475_GPIO_PAR_FBCTL_BWE3         0x4000
#define M5475_GPIO_PAR_FBCTL_BEW2         0x1000
#define M5475_GPIO_PAR_FBCTL_BEW1         0x0400
#define M5475_GPIO_PAR_FBCTL_BEW0         0x0100
#define M5475_GPIO_PAR_FBCTL_OE           0x0040
#define M5475_GPIO_PAR_FBCTL_RWB_RW       0x0030
#define M5475_GPIO_PAR_FBCTL_RWB_TBST     0x0020
#define M5475_GPIO_PAR_FBCTL_TA           0x0004
#define M5475_GPIO_PAR_FBCTL_ALE_TBST     0x0002
#define M5475_GPIO_PAR_FBCTL_ALE          0x0003

#define M5475_GPIO_PAR_FECI2CIRQ_E07      0x8000
#define M5475_GPIO_PAR_FECI2CIRQ_E0MII    0x4000
#define M5475_GPIO_PAR_FECI2CIRQ_E0MDIO   0x2000
#define M5475_GPIO_PAR_FECI2CIRQ_E0MDC    0x1000
#define M5475_GPIO_PAR_FECI2CIRQ_E17      0x0800
#define M5475_GPIO_PAR_FECI2CIRQ_E1MII    0x0400
#define M5475_GPIO_PAR_FECI2CIRQ_E1MDIO   0x0300
#define M5475_GPIO_PAR_FECI2CIRQ_E1MDC    0x00c0
#define M5475_GPIO_PAR_FECI2CIRQ_SDA      0x0008
#define M5475_GPIO_PAR_FECI2CIRQ_SCL      0x0004
#define M5475_GPIO_PAR_FECI2CIRQ_IRQ6     0x0002
#define M5475_GPIO_PAR_FECI2CIRQ_IRQ5     0x0001

#define M5475_GPIO_PAR_PSC_CTS0_CTS       0xc0
#define M5475_GPIO_PAR_PSC_CTS0_BCLK      0x80
#define M5475_GPIO_PAR_PSC_RTS0_FSYNC     0x20
#define M5475_GPIO_PAR_PSC_RTS0_RTS       0x30
#define M5475_GPIO_PAR_PSC_RXD0_RXD       0x08
#define M5475_GPIO_PAR_PSC_TXD0_TXD       0x04


#define M5475_DEF_PAR_FECI2CIRQ_VAL       0x03C3

/*
 *
 * SDRAM (SDRAM) - Chapter 18
 *
 */
/* SDRAM Controller Memory Map */
#define M5475_SDRAMC_DS(base)     (CAST_M5475(volatile UINT32 *)((base) + 0x04))
#define M5475_SDRAMC_CS0CFG(base) (CAST_M5475(volatile UINT32 *)((base) + 0x20))
#define M5475_SDRAMC_CS1CFG(base) (CAST_M5475(volatile UINT32 *)((base) + 0x24))
#define M5475_SDRAMC_CS2CFG(base) (CAST_M5475(volatile UINT32 *)((base) + 0x28))
#define M5475_SDRAMC_CS3CFG(base) (CAST_M5475(volatile UINT32 *)((base) + 0x2c))

#define M5475_SDRAMC_SDMR(base)  (CAST_M5475(volatile UINT32 *)((base) + 0x100))
#define M5475_SDRAMC_SDCR(base)  (CAST_M5475(volatile UINT32 *)((base) + 0x104))
#define M5475_SDRAMC_DCFG1(base) (CAST_M5475(volatile UINT32 *)((base) + 0x108))
#define M5475_SDRAMC_DCFG2(base) (CAST_M5475(volatile UINT32 *)((base) + 0x10c))

#define M5475_SDRAMC_SDRAMDS_SB_E(x) (((x) & 0x3) << 8)
#define M5475_SDRAMC_SDRAMDS_SB_C(x) (((x) & 0x3) << 6)
#define M5475_SDRAMC_SDRAMDS_SB_A(x) (((x) & 0x3) << 4)
#define M5475_SDRAMC_SDRAMDS_SB_S(x) (((x) & 0x3) << 2)
#define M5475_SDRAMC_SDRAMDS_SB_D(x) ((x) & 0x3)

#define M5475_SDRAMC_SDRAMDS_DRIVE_8MA      0x2
#define M5475_SDRAMC_SDRAMDS_DRIVE_16MA     0x1
#define M5475_SDRAMC_SDRAMDS_DRIVE_24MA     0x0
#define M5475_SDRAMC_SDRAMDS_DRIVE_7_6MA    0x2
#define M5475_SDRAMC_SDRAMDS_DRIVE_13MA     0x1
#define M5475_SDRAMC_SDRAMDS_DRIVE_15MA     0x0
#define M5475_SDRAMC_SDRAMDS_DRIVE_HI_Z     0x3

#define M5475_SDRAMC_CSnCFG_BA(x)           (((x) & 0x0fff) << 20)
#define M5475_SDRAMC_CSnCFG_CSSZ(x)         ((x) & 0x1f)
#define M5475_SDRAMC_CSnCFG_CSSZ_DISABLED   0x0
#define M5475_SDRAMC_CSnCFG_CSSZ_1MB        0x13
#define M5475_SDRAMC_CSnCFG_CSSZ_2MB        0x14
#define M5475_SDRAMC_CSnCFG_CSSZ_4MB        0x15
#define M5475_SDRAMC_CSnCFG_CSSZ_8MB        0x16
#define M5475_SDRAMC_CSnCFG_CSSZ_16MB       0x17
#define M5475_SDRAMC_CSnCFG_CSSZ_32MB       0x18
#define M5475_SDRAMC_CSnCFG_CSSZ_64MB       0x19
#define M5475_SDRAMC_CSnCFG_CSSZ_128MB      0x1a
#define M5475_SDRAMC_CSnCFG_CSSZ_256MB      0x1b
#define M5475_SDRAMC_CSnCFG_CSSZ_512MB      0x1c
#define M5475_SDRAMC_CSnCFG_CSSZ_1GB        0x1d
#define M5475_SDRAMC_CSnCFG_CSSZ_2GB        0x1e
#define M5475_SDRAMC_CSnCFG_CSSZ_4GB        0x1f

#define M5475_SDRAMC_SDRAMCFG1_SRD2RW(x)    (((x) & 0xf) << 28)
#define M5475_SDRAMC_SDRAMCFG1_SWT2RD(x)    (((x) & 0x7) << 24)
#define M5475_SDRAMC_SDRAMCFG1_RDLAT(x)     (((x) & 0xf) << 20)
#define M5475_SDRAMC_SDRAMCFG1_ACT2RW(x)    (((x) & 0x7) << 16)
#define M5475_SDRAMC_SDRAMCFG1_PRE2ACT(x)   (((x) & 0x7) << 12)
#define M5475_SDRAMC_SDRAMCFG1_REF2ACT(x)   (((x) & 0xf) << 8)
#define M5475_SDRAMC_SDRAMCFG1_WTLAT(x)     (((x) & 0x7) << 4)

#define M5475_SDRAMC_SDRAMCFG2_BRD2PRE(x)   (((x) & 0xf) << 28)
#define M5475_SDRAMC_SDRAMCFG2_BWT2RW(x)    (((x) & 0xf) << 24)
#define M5475_SDRAMC_SDRAMCFG2_BRD2WT(x)    (((x) & 0xf) << 20)
#define M5475_SDRAMC_SDRAMCFG2_BL(x)        (((x) & 0xf) << 16)

/*
 * SDRAM Timing Info
 * Note: Values from DBug m54x5evb.h
 */
#define M5475_SDRAM_TWR       2       /* in clocks */
#define M5475_SDRAM_CASL      2.5     /* in clocks */
#define M5475_SDRAM_TRCD      20      /* in ns */
#define M5475_SDRAM_TRP       20      /* in ns */
#define M5475_SDRAM_TRFC      75      /* in ns */
#define M5475_SDRAM_TREFI     7800    /* in ns */

/*
 * SDRAMC Control Register
 */
#define M5475_SDRAMC_SDCR_MODE_EN       0x80000000
#define M5475_SDRAMC_SDCR_CKE           0x40000000
#define M5475_SDRAMC_SDCR_DDR           0x20000000
#define M5475_SDRAMC_SDCR_REF           0x10000000
#define M5475_SDRAMC_SDCR_MUX(x)        (((x) & 0x03) << 24)
#define M5475_SDRAMC_SDCR_AP            0x00800000
#define M5475_SDRAMC_SDCR_DRIVE         0x00400000
#define M5475_SDRAMC_SDCR_RCNT(x)       (((x) & 0x3f) << 16)
#define M5475_SDRAMC_SDCR_DQS_OE0       0x00000100
#define M5475_SDRAMC_SDCR_DQS_OE1       0x00000200
#define M5475_SDRAMC_SDCR_DQS_OE2       0x00000400
#define M5475_SDRAMC_SDCR_DQS_OE3       0x00000800
#define M5475_SDRAMC_SDCR_DQS_ALL       0x00000f00
#define M5475_SDRAMC_SDCR_BUFF          0x00000010
#define M5475_SDRAMC_SDCR_IREF          0x00000004
#define M5475_SDRAMC_SDCR_IPALL         0x00000002

/*
 * SDRAMC Mode Register
 */
#define M5475_SDRAMC_SDMR_BNKAD_LMR     0
#define M5475_SDRAMC_SDMR_BNKAD_LEMR    0x40000000
#define M5475_SDRAMC_SDMR_AD(x)         (((x) & 0xfff) << 18)
#define M5475_SDRAMC_SDMR_CMD           0x00010000

/*
 * SDRAM mode register
 */
#define M5475_SDRAM_MR_BLEN_SDR_ONE         0
#define M5475_SDRAM_MR_BLEN_TWO             0x1
#define M5475_SDRAM_MR_BLEN_FOUR            0x2
#define M5475_SDRAM_MR_BLEN_EIGHT           0x3
#define M5475_SDRAM_MR_BLEN_FULL_PAGE       0x7

#define M5475_SDRAM_MR_BT_SEQUENTIAL        0
#define M5475_SDRAM_MR_BT_INTERLEAVED       0x8

#define M5475_SDRAM_MR_CASL(x)              (((x) & 0x7) << 4)

#define M5475_SDRAM_MR_OPMODE_NORMAL        0
#define M5475_SDRAM_MR_OPMODE_RESET_DLL     0x100


/*
 *
 * Programmable Serial Controller (PSC) - Chapter 25
 *
 */

/* Register read/write macros */
#define M5475_PSC_REG(base, offset, chan)   \
            ((base) + (offset) + ((chan) * 0x100))

#define M5475_PSC_MR(base,x)    \
            (CAST_M5475(volatile UINT8 *) M5475_PSC_REG(base, 0x008600, x))
/*
 * For the M5475, SR is actually a 16 bit register. The coldfireSio driver,
 * however, uses only the top 8 bits.
 */
#define M5475_PSC_SR(base,x)   \
            (CAST_M5475(volatile UINT8 *) M5475_PSC_REG(base, 0x008604, x))
#define M5475_PSC_CSR(base,x)  \
            (CAST_M5475(volatile UINT8 *) M5475_PSC_REG(base, 0x008604, x))
#define M5475_PSC_CR(base,x)   \
            (CAST_M5475(volatile UINT8 *) M5475_PSC_REG(base, 0x008608, x))
/*
 * For the M5475, Receive buffer is a 32 bit register. The coldfireSio driver
 * accesses RB and TB as 8 bit registers.
 */
#define M5475_PSC_RB(base,x)   \
            (CAST_M5475(volatile UINT8 *) M5475_PSC_REG(base, 0x00860c, x))
#define M5475_PSC_TB(base,x)   \
            (CAST_M5475(volatile UINT8 *) M5475_PSC_REG(base, 0x00860c, x))
#define M5475_PSC_IPCR(base,x) \
            (CAST_M5475(volatile UINT8 *) M5475_PSC_REG(base, 0x008610, x))
#define M5475_PSC_ACR(base,x)  \
            (CAST_M5475(volatile UINT8 *) M5475_PSC_REG(base, 0x008610, x))
/*
 * Interrupt status register is 16 bits.  The coldfireSio driver uses only the
 * top 8 bits
 */
#define M5475_PSC_ISR(base,x)  \
            (CAST_M5475(volatile UINT8 *) M5475_PSC_REG(base, 0x008614, x))
#define M5475_PSC_IMR(base,x)  \
            (CAST_M5475(volatile UINT8 *) M5475_PSC_REG(base, 0x008614, x))
#define M5475_PSC_CTUR(base,x) \
            (CAST_M5475(volatile UINT8 *) M5475_PSC_REG(base, 0x008618, x))
#define M5475_PSC_CTLR(base,x) \
            (CAST_M5475(volatile UINT8 *) M5475_PSC_REG(base, 0x00861c, x))
#define M5475_PSC_IP(base,x)   \
            (CAST_M5475(volatile UINT8 *) M5475_PSC_REG(base, 0x008634, x))
#define M5475_PSC_OPSET(base,x) \
            (CAST_M5475(volatile UINT8 *) M5475_PSC_REG(base, 0x008638, x))
#define M5475_PSC_SICR(base,x) \
            (CAST_M5475(volatile UINT8 *) M5475_PSC_REG(base, 0x008640, x))
#define M5475_PSC_OPRESET(base,x) \
            (CAST_M5475(volatile UINT8 *) M5475_PSC_REG(base, 0x00863c, x))

/*
 *
 * Chip Select Module - Chapter 17
 *
 */

/* Register read/write macros */

#define M5475_CSAR0(base)     (CAST_M5475(volatile UINT32 *)((base) + 0x000500))
#define M5475_CSMR0(base)     (CAST_M5475(volatile UINT32 *)((base) + 0x000504))
#define M5475_CSCR0(base)     (CAST_M5475(volatile UINT32 *)((base) + 0x000508))

#define M5475_CSAR1(base)     (CAST_M5475(volatile UINT32 *)((base) + 0x00050C))
#define M5475_CSMR1(base)     (CAST_M5475(volatile UINT32 *)((base) + 0x000510))
#define M5475_CSCR1(base)     (CAST_M5475(volatile UINT32 *)((base) + 0x000514))

#define M5475_CSAR2(base)     (CAST_M5475(volatile UINT32 *)((base) + 0x000518))
#define M5475_CSMR2(base)     (CAST_M5475(volatile UINT32 *)((base) + 0x00051C))
#define M5475_CSCR2(base)     (CAST_M5475(volatile UINT32 *)((base) + 0x000520))

#define M5475_CSAR3(base)     (CAST_M5475(volatile UINT32 *)((base) + 0x000524))
#define M5475_CSMR3(base)     (CAST_M5475(volatile UINT32 *)((base) + 0x000528))
#define M5475_CSCR3(base)     (CAST_M5475(volatile UINT32 *)((base) + 0x00052c))

#define M5475_CSAR4(base)     (CAST_M5475(volatile UINT32 *)((base) + 0x000530))
#define M5475_CSMR4(base)     (CAST_M5475(volatile UINT32 *)((base) + 0x000534))
#define M5475_CSCR4(base)     (CAST_M5475(volatile UINT32 *)((base) + 0x000538))

#define M5475_CSAR5(base)     (CAST_M5475(volatile UINT32 *)((base) + 0x00053C))
#define M5475_CSMR5(base)     (CAST_M5475(volatile UINT32 *)((base) + 0x000540))
#define M5475_CSCR5(base)     (CAST_M5475(volatile UINT32 *)((base) + 0x000544))

#define M5475_CSAR(base,x)     \
            (CAST_M5475(volatile UINT32 *)((base) + 0x000500 + ((x) * 0x00C)))
#define M5475_CSMR(base,x)     \
            (CAST_M5475(volatile UINT32 *)((base) + 0x000504 + ((x) * 0x00C)))
#define M5475_CSCR(base,x)     \
            (CAST_M5475(volatile UINT32 *)((base) + 0x000508 + ((x) * 0x00C)))

/* Bit definitions and macros for M5475_CSAR */
#define M5475_CSAR_BA(x)        ((x) & 0xFFFF0000)

/* Bit definitions and macros for M5475_CSMR */
#define M5475_CSMR_V            (0x00000001)
#define M5475_CSMR_WP           (0x00000100)
#define M5475_CSMR_BAM(x)       (((x) & 0x0000FFFF) << 16)

#define M5475_CSMR_BAM_4G       (0xFFFF0000)
#define M5475_CSMR_BAM_2G       (0x7FFF0000)
#define M5475_CSMR_BAM_1G       (0x3FFF0000)
#define M5475_CSMR_BAM_1024M    (0x3FFF0000)
#define M5475_CSMR_BAM_512M     (0x1FFF0000)
#define M5475_CSMR_BAM_256M     (0x0FFF0000)
#define M5475_CSMR_BAM_128M     (0x07FF0000)
#define M5475_CSMR_BAM_64M      (0x03FF0000)
#define M5475_CSMR_BAM_32M      (0x01FF0000)
#define M5475_CSMR_BAM_16M      (0x00FF0000)
#define M5475_CSMR_BAM_8M       (0x007F0000)
#define M5475_CSMR_BAM_4M       (0x003F0000)
#define M5475_CSMR_BAM_2M       (0x001F0000)
#define M5475_CSMR_BAM_1M       (0x000F0000)
#define M5475_CSMR_BAM_1024K    (0x000F0000)
#define M5475_CSMR_BAM_512K     (0x00070000)
#define M5475_CSMR_BAM_256K     (0x00030000)
#define M5475_CSMR_BAM_128K     (0x00010000)
#define M5475_CSMR_BAM_64K      (0x00000000)

/* Bit definitions and macros for M5475_CSCR */

#define M5475_CSCR_BSTW         (0x0008)
#define M5475_CSCR_BSTR         (0x0010)
#define M5475_CSCR_BEM          (0x0020)
#define M5475_CSCR_PS(x)        (((x) & 0x0003) << 6)
#define M5475_CSCR_AA           (0x0100)
#define M5475_CSCR_WS(x)        (((x) & 0x003F) << 10)
#define M5475_CSCR_WS_SHIFT     10

#define M5475_CSCR_WRAH(x)      (((x) & 0x0003) << 16)
#define M5475_CSCR_RDAH(x)      (((x) & 0x0003) << 18)
#define M5475_CSCR_ASET(x)      (((x) & 0x0003) << 20)
#define M5475_CSCR_SWSEN        0x00800000
#define M5475_CSCR_SWS(x)       (((x) & 0x3f) << 26)


#define M5475_CSCR_PS_8         (0x0040)
#define M5475_CSCR_PS_16        (0x0080)
#define M5475_CSCR_PS_32        (0x0000)

/*
 *
 * Slice Timers - Chapter 12
 *
 */
/* Register read/write macros */
#define MCF5475_NUM_SLICE_TIMERS    2

#define M5475_SLT_STCNT0(base) \
            (CAST_M5475(volatile UINT32 *)((base) + 0x000900))
#define M5475_SLT_SCR0(base)   \
            (CAST_M5475(volatile UINT32 *)((base) + 0x000904))
#define M5475_SLT_SCNT0(base)  \
            (CAST_M5475(volatile UINT32 *)((base) + 0x000908))
#define M5475_SLT_SSR0(base)   \
            (CAST_M5475(volatile UINT32 *)((base) + 0x00090C))

#define M5475_SLT_STCNT1(base) \
            (CAST_M5475(volatile UINT32 *)((base) + 0x000910))
#define M5475_SLT_SCR1(base)   \
            (CAST_M5475(volatile UINT32 *)((base) + 0x000914))
#define M5475_SLT_SCNT1(base)  \
            (CAST_M5475(volatile UINT32 *)((base) + 0x000918))
#define M5475_SLT_SSR1(base)   \
            (CAST_M5475(volatile UINT32 *)((base) + 0x00091C))


#define M5475_SLT_SCRn_RUN      0x04000000
#define M5475_SLT_SCRn_WAIT     0x00000000
#define M5475_SLT_SCRn_IEN      0x02000000
#define M5475_SLT_SCRn_NORMAL   0x01000000
#define M5475_SLT_SCRn_IDLE     0x00000000

#define M5475_SLT_SSRn_BE       0x02000000
#define M5475_SLT_SSRn_ST       0x01000000

/*
 *
 * PCI
 *
 */

/* PCI Host Bus Controller Type 0 Configuration Registers */

#define M5475_PCI_PCIIDR(base) \
            (CAST_M5475(volatile UINT32 *)((base) + 0x000b00))

/* PCI Status/Command Register */
#define M5475_PCI_PCISCR(base) \
            (CAST_M5475(volatile UINT32 *)((base) + 0x000b04))
#define M5475_PCI_PCISCR_IO 0x00000001  /* IO Access Control */
#define M5475_PCI_PCISCR_M  0x00000002  /* Memory Access Control */
#define M5475_PCI_PCISCR_B  0x00000004  /* bus master enable */
#define M5475_PCI_PCISCR_MW 0x00000010  /* Memory Write and Invalidate */

#define M5475_PCI_PCICCRIR(base) \
            (CAST_M5475(volatile UINT32 *)((base) + 0x000b08))

/* Configuration 1 Register */
#define M5475_PCI_PCICR1(base) \
            (CAST_M5475(volatile UINT32 *)((base) + 0x000b0c))
#define M5475_PCI_PCICR1_LAT_MASK         0x0000ff00
#define M5475_PCI_PCICR1_LAT_SHIFT        8

#define M5475_PCI_PCIBAR0(base) \
            (CAST_M5475(volatile UINT32 *)((base) + 0x000b10))
#define M5475_PCI_PCIBAR0_MASK          0x3FFFF

#define M5475_PCI_PCIBAR1(base) \
            (CAST_M5475(volatile UINT32 *)((base) + 0x000b14))
#define M5475_PCI_PCIBAR1_MASK          0x3FFFFFFF
#define M5475_PCI_PCIBAR1_PREF          0x00000008

#define M5475_PCI_PCICCPR(base) \
            (CAST_M5475(volatile UINT32 *)((base) + 0x000b28))
#define M5475_PCI_PCISID(base) \
            (CAST_M5475(volatile UINT32 *)((base) + 0x000b2c))
#define M5475_PCI_PCIERBAR(base) \
            (CAST_M5475(volatile UINT32 *)((base) + 0x000b30))
#define M5475_PCI_PCICPR(base) \
            (CAST_M5475(volatile UINT32 *)((base) + 0x000b34))

/* Configuration 2 Register */
#define M5475_PCI_PCICR2(base) \
            (CAST_M5475(volatile UINT32 *)((base) + 0x000b3c))
#define M5475_PCI_PCICR2_MAX_LAT          0xFF
#define M5475_PCI_PCICR2_MIN_GNT          0xFF

/* Global Status/Control Register */
#define M5475_PCI_PCIGSCR(base) \
            (CAST_M5475(volatile UINT32 *)((base) + 0x000b60))
#define M5475_PCI_PCIGSCR_SEE             0x00001000
#define M5475_PCI_PCIGSCR_PEE             0x00002000
#define M5475_PCI_PCIGSCR_SE              0x10000000
#define M5475_PCI_PCIGSCR_PE              0x20000000

#define M5475_PCI_PCITBATR0(base) \
            (CAST_M5475(volatile UINT32 *)((base) + 0x000b64))
#define M5475_PCI_PCITBATR0_MASK            0x0003FFFF
#define M5475_PCI_PCITBATR0_ENABLE          0x00000001

#define M5475_PCI_PCITBATR1(base) \
            (CAST_M5475(volatile UINT32 *)((base) + 0x000b68))
#define M5475_PCI_PCITBATR1_MASK            0x3FFFFFFF
#define M5475_PCI_PCITBATR1_ENABLE          0x00000001


#define M5475_PCI_PCITCR(base) \
            (CAST_M5475(volatile UINT32 *)((base) + 0x000b6c))
#define M5475_PCI_PCIIW0BTAR(base) \
            (CAST_M5475(volatile UINT32 *)((base) + 0x000b70))
#define M5475_PCI_PCIIW1BTAR(base) \
            (CAST_M5475(volatile UINT32 *)((base) + 0x000b74))
#define M5475_PCI_PCIIW2BTAR(base) \
            (CAST_M5475(volatile UINT32 *)((base) + 0x000b78))

/* Initiator Window Configuration Register bits */
#define M5475_PCI_PCIIWCR(base) \
            (CAST_M5475(volatile UINT32 *)((base) + 0x000b80))
#define M5475_PCIIWCR_WIN0_SET(x)   (x) << 24
#define M5475_PCIIWCR_WIN1_SET(x)   (x) << 16
#define M5475_PCIIWCR_WIN2_SET(x)   (x) << 8
#define M5475_PCI_PCIIWCR_ENABLE          0x1
#define M5475_PCI_PCIIWCR_READ_SINGLE     0x0
#define M5475_PCI_PCIIWCR_READ_LINE       0x2
#define M5475_PCI_PCIIWCR_READ_MULTI      0x4
#define M5475_PCI_PCIIWCR_MEM             0x0
#define M5475_PCI_PCIIWCR_IO              0x8


/* Initiator Control Register */
#define M5475_PCI_PCIICR(base) \
            (CAST_M5475(volatile UINT32 *)((base) + 0x000b84))
#define M5475_PCI_PCIICR_REE              0x04000000
#define M5475_PCI_PCIICR_IAE              0x02000000
#define M5475_PCI_PCIICR_TAE              0x01000000

/* Initiator Status Register */
#define M5475_PCI_PCIISR(base) \
            (CAST_M5475(volatile UINT32 *)((base) + 0x000b88))
#define M5475_PCI_PCIISR_TA               0x01000000
#define M5475_PCI_PCIISR_IA               0x02000000
#define M5475_PCI_PCIISR_RE               0x04000000

/* Configuration Status Register */
#define M5475_PCI_PCICAR(base) \
            (CAST_M5475(volatile UINT32 *)((base) + 0x000bf8))
#define M5475_PCICAR_DWORD(x)             (((x)&0x0000003F)<<2)
#define M5475_PCICAR_FUNCNUM(x)           (((x)&0x00000007)<<8)
#define M5475_PCICAR_DEVNUM(x)            (((x)&0x0000001F)<<11)
#define M5475_PCICAR_BUSNUM(x)            (((x)&0x000000FF)<<16)
#define M5475_PCI_PCICAR_E               0x80000000

/*
 *
 * PCI Arbiter
 *
 */
/* Control Register */
#define M5475_PCI_PACR(base) \
            (CAST_M5475(volatile UINT32 *)((base) + 0x000C00))
#define M5475_PCI_PACR_INTPRI             0x00000001
#define M5475_PCI_PACR_EXTMPRI            0x0000003E
#define M5475_PCI_PACR_INTMINTEN          0x00010000
#define M5475_PCI_PACR_EXTMINTEN          0x003E0000

/*
 *
 * Chapter 11: General Purpose Timers
 *
 */
#define M5475_GPT_GMS0(base)  (CAST_M5475(volatile UINT32 *)((base) + 0x000800))
#define M5475_GPT_GMS1(base)  (CAST_M5475(volatile UINT32 *)((base) + 0x000810))
#define M5475_GPT_GMS2(base)  (CAST_M5475(volatile UINT32 *)((base) + 0x000820))
#define M5475_GPT_GMS3(base)  (CAST_M5475(volatile UINT32 *)((base) + 0x000830))

#define M5475_GPT_GCIR0(base) (CAST_M5475(volatile UINT32 *)((base) + 0x000804))
#define M5475_GPT_GCIR1(base) (CAST_M5475(volatile UINT32 *)((base) + 0x000814))
#define M5475_GPT_GCIR2(base) (CAST_M5475(volatile UINT32 *)((base) + 0x000824))
#define M5475_GPT_GCIR3(base) (CAST_M5475(volatile UINT32 *)((base) + 0x000834))

#define M5475_GPT_GPWM0(base) (CAST_M5475(volatile UINT32 *)((base) + 0x000808))
#define M5475_GPT_GPWM1(base) (CAST_M5475(volatile UINT32 *)((base) + 0x000818))
#define M5475_GPT_GPWM2(base) (CAST_M5475(volatile UINT32 *)((base) + 0x000828))
#define M5475_GPT_GPWM3(base) (CAST_M5475(volatile UINT32 *)((base) + 0x000838))

#define M5475_GPT_GSR0(base)  (CAST_M5475(volatile UINT32 *)((base) + 0x00080C))
#define M5475_GPT_GSR1(base)  (CAST_M5475(volatile UINT32 *)((base) + 0x00081C))
#define M5475_GPT_GSR2(base)  (CAST_M5475(volatile UINT32 *)((base) + 0x00082C))
#define M5475_GPT_GSR3(base)  (CAST_M5475(volatile UINT32 *)((base) + 0x00083C))


#define M5475_GPT_GMS_OCPW(x)               ((UINT32)(x) << 24)
#define M5475_GPT_GMS_OCT_FORCE_LOW         0x00000000
#define M5475_GPT_GMS_OCT_PULSE_HIGH        0x00100000
#define M5475_GPT_GMS_OCT_PULSE_LOW         0x00200000
#define M5475_GPT_GMS_OCT_PULSE_TOGGLE      0x00300000

#define M5475_GPT_GMS_ICT_ANY               0
#define M5475_GPT_GMS_ICT_RISING_EDGE       0x00010000
#define M5475_GPT_GMS_ICT_FALLING_EDGE      0x00020000
#define M5475_GPT_GMS_ICT_ANY_IP_PULSE      0x00030000

#define M5475_GPT_GMS_WDEN                  0x00008000

#define M5475_GPT_GMS_CE                    0x00001000

#define M5475_GPT_GMS_SC_STOP               0
#define M5475_GPT_GMS_SC_CONT               0x00000400

#define M5475_GPT_GMS_OD                    0x00000200

#define M5475_GPT_GMS_IEN                   0x00000100

#define M5475_GPT_GMS_GPIO_IP               0x00000010
#define M5475_GPT_GMS_GPIO_OP_LOW           0x00000020
#define M5475_GPT_GMS_GPIO_OP_HIGH          0x00000030

#define M5475_GPT_GMS_GPIO_TMS_IPCAP        0x00000001
#define M5475_GPT_GMS_GPIO_TMS_OPCAP        0x00000002
#define M5475_GPT_GMS_GPIO_TMS_PWM          0x00000003
#define M5475_GPT_GMS_GPIO_TMS_GPIO         0x00000004


#define M5475_GPT_GCIR_PRE(x)               ((UINT32)(x) << 16)
#define M5475_GPT_GCIR_CNT(x)               ((UINT32)(x) & 0x0000ffff)

#define M5475_GPT_GPWM_WIDTH(x)             ((UINT32)(x) << 16)
#define M5475_GPT_GPWM_PWMOP_HIGH           0x00000100
#define M5475_GPT_GPWM_LOAD                 0x00000001

#define M5475_GPT_GSR_CAPTURE(x)            ((UINT32)(x) << 16)
#define M5475_GPT_GSR_OVF(x)                (((UINT32)(x) & 0x7) << 12)
#define M5475_GPT_GSR_PIN                   0x00000100
#define M5475_GPT_GSR_TXEP                  0x00000008
#define M5475_GPT_GSR_PWMP                  0x00000004
#define M5475_GPT_GSR_COMP                  0x00000002
#define M5475_GPT_GSR_CAPT                  0x00000001

/*
 *
 * Chapter 29: Fast Ethernet Controller (FEC)
 *
 */
#define M5475_FEC_EIR(base,f)   \
        (CAST_M5475(volatile UINT32 *)((base) + (0x009000 + f* 0x800) + 0x4))
#define M5475_FEC_EIMR(base,f)  \
        (CAST_M5475(volatile UINT32 *)((base) + (0x009000 + f* 0x800) + 0x8))
#define M5475_FEC_ECR(base,f)   \
        (CAST_M5475(volatile UINT32 *)((base) + (0x009000 + f* 0x800) + 0x24))
#define M5475_FEC_MDATA(base,f) \
        (CAST_M5475(volatile UINT32 *)((base) + (0x009000 + f* 0x800) + 0x40))
#define M5475_FEC_MSCR(base,f)  \
        (CAST_M5475(volatile UINT32 *)((base) + (0x009000 + f* 0x800) + 0x44))
#define M5475_FEC_MIBC(base,f)  \
        (CAST_M5475(volatile UINT32 *)((base) + (0x009000 + f* 0x800) + 0x64))
#define M5475_FEC_RCR(base,f)   \
        (CAST_M5475(volatile UINT32 *)((base) + (0x009000 + f* 0x800) + 0x84))
#define M5475_FEC_RHR(base,f)   \
        (CAST_M5475(volatile UINT32 *)((base) + (0x009000 + f* 0x800) + 0x88))
#define M5475_FEC_TCR(base,f)   \
        (CAST_M5475(volatile UINT32 *)((base) + (0x009000 + f* 0x800) + 0xC4))

#define M5475_FEC_PALR(base,f)  \
        (CAST_M5475(volatile UINT32 *)((base) + (0x009000 + f* 0x800) + 0xE4))
#define M5475_FEC_PAHR(base,f)  \
        (CAST_M5475(volatile UINT32 *)((base) + (0x009000 + f* 0x800) + 0xE8))
#define M5475_FEC_OPD(base,f)   \
        (CAST_M5475(volatile UINT32 *)((base) + (0x009000 + f* 0x800) + 0xEC))

#define M5475_FEC_IAUR(base,f)  \
        (CAST_M5475(volatile UINT32 *)((base) + (0x009000 + f* 0x800) + 0x118))
#define M5475_FEC_IALR(base,f)  \
        (CAST_M5475(volatile UINT32 *)((base) + (0x009000 + f* 0x800) + 0x11C))
#define M5475_FEC_GAUR(base,f)  \
        (CAST_M5475(volatile UINT32 *)((base) + (0x009000 + f* 0x800) + 0x120))
#define M5475_FEC_GALR(base,f)  \
        (CAST_M5475(volatile UINT32 *)((base) + (0x009000 + f* 0x800) + 0x124))

#define M5475_FEC_TFWR(base,f)  \
        (CAST_M5475(volatile UINT32 *)((base) + (0x009000 + f* 0x800) + 0x144))
#define M5475_FEC_RFDR(base,f)  \
        (CAST_M5475(volatile UINT32 *)((base) + (0x009000 + f* 0x800) + 0x184))
#define M5475_FEC_RFSR(base,f)  \
        (CAST_M5475(volatile UINT32 *)((base) + (0x009000 + f* 0x800) + 0x188))
#define M5475_FEC_RFCR(base,f)  \
        (CAST_M5475(volatile UINT32 *)((base) + (0x009000 + f* 0x800) + 0x18C))
#define M5475_FEC_RLRFP(base,f) \
        (CAST_M5475(volatile UINT32 *)((base) + (0x009000 + f* 0x800) + 0x190))
#define M5475_FEC_RLWFP(base,f) \
        (CAST_M5475(volatile UINT32 *)((base) + (0x009000 + f* 0x800) + 0x194))
#define M5475_FEC_RFAR(base,f)  \
        (CAST_M5475(volatile UINT32 *)((base) + (0x009000 + f* 0x800) + 0x198))
#define M5475_FEC_RFRP(base,f)  \
        (CAST_M5475(volatile UINT32 *)((base) + (0x009000 + f* 0x800) + 0x19C))
#define M5475_FEC_RFWP(base,f)  \
        (CAST_M5475(volatile UINT32 *)((base) + (0x009000 + f* 0x800) + 0x1A0))
#define M5475_FEC_TFDR(base,f)  \
        (CAST_M5475(volatile UINT32 *)((base) + (0x009000 + f* 0x800) + 0x1A4))
#define M5475_FEC_TFSR(base,f)  \
        (CAST_M5475(volatile UINT32 *)((base) + (0x009000 + f* 0x800) + 0x1A8))
#define M5475_FEC_TFCR(base,f)  \
        (CAST_M5475(volatile UINT32 *)((base) + (0x009000 + f* 0x800) + 0x1AC))
#define M5475_FEC_TLRFP(base,f) \
        (CAST_M5475(volatile UINT32 *)((base) + (0x009000 + f* 0x800) + 0x1B0))
#define M5475_FEC_TLWFP(base,f) \
        (CAST_M5475(volatile UINT32 *)((base) + (0x009000 + f* 0x800) + 0x1B4))
#define M5475_FEC_TFAR(base,f)  \
        (CAST_M5475(volatile UINT32 *)((base) + (0x009000 + f* 0x800) + 0x1B8))
#define M5475_FEC_TFRP(base,f)  \
        (CAST_M5475(volatile UINT32 *)((base) + (0x009000 + f* 0x800) + 0x1Bc))
#define M5475_FEC_TFWP(base,f)  \
        (CAST_M5475(volatile UINT32 *)((base) + (0x009000 + f* 0x800) + 0x1C0))
#define M5475_FEC_FRST(base,f)  \
        (CAST_M5475(volatile UINT32 *)((base) + (0x009000 + f* 0x800) + 0x1C4))
#define M5475_FEC_CTCWR(base,f) \
        (CAST_M5475(volatile UINT32 *)((base) + (0x009000 + f* 0x800) + 0x1C8))

/*
 *
 * Multi-Channel DMA (DMA)
 *
 */

/* Register read/write macros */
#define M5475_DMA_TASKBAR(base) (CAST_M5475(volatile UINT32 *)((base) + 0x8000))
#define M5475_DMA_CP(base)      (CAST_M5475(volatile UINT32 *)((base) + 0x8004))
#define M5475_DMA_EP(base)      (CAST_M5475(volatile UINT32 *)((base) + 0x8008))
#define M5475_DMA_VP(base)      (CAST_M5475(volatile UINT32 *)((base) + 0x800c))
#define M5475_DMA_PTC(base)     (CAST_M5475(volatile UINT32 *)((base) + 0x8010))

#define M5475_DMA_TSKSZ0(base)  (CAST_M5475(volatile UINT32 *)((base) + 0x8060))
#define M5475_DMA_TSKSZ1(base)  (CAST_M5475(volatile UINT32 *)((base) + 0x8064))
#define M5475_DMA_DBGCOMPn(base)(CAST_M5475(volatile UINT32 *)((base) + 0x8070))
#define M5475_DMA_DBGCTL(base)  (CAST_M5475(volatile UINT32 *)((base) + 0x8078))
#define M5475_DMA_DBGSTAT(base) (CAST_M5475(volatile UINT32 *)((base) + 0x807c))
#define M5475_DMA_PTDDBG(base)  (CAST_M5475(volatile UINT32 *)((base) + 0x8080))

#define M5475_DMA_DIPR(base)    (CAST_M5475(volatile UINT32 *)((base) + 0x8014))
#define M5475_DMA_DIMR(base)    (CAST_M5475(volatile UINT32 *)((base) + 0x8018))
#define M5475_DMA_IMCR(base)    (CAST_M5475(volatile UINT32 *)((base) + 0x805c))

#define M5475_DMA_TASK(base, n) \
                (CAST_M5475(volatile UINT16 *)((base) + 0x801c + n*2))


/* Bit definitions and macros for M5475_DMA_DIPR */
#define M5475_DMA_DIPR_TASK0           (0x00000001)
#define M5475_DMA_DIPR_TASK1           (0x00000002)
#define M5475_DMA_DIPR_TASK2           (0x00000004)
#define M5475_DMA_DIPR_TASK3           (0x00000008)
#define M5475_DMA_DIPR_TASK4           (0x00000010)
#define M5475_DMA_DIPR_TASK5           (0x00000020)
#define M5475_DMA_DIPR_TASK6           (0x00000040)
#define M5475_DMA_DIPR_TASK7           (0x00000080)
#define M5475_DMA_DIPR_TASK8           (0x00000100)
#define M5475_DMA_DIPR_TASK9           (0x00000200)
#define M5475_DMA_DIPR_TASK10          (0x00000400)
#define M5475_DMA_DIPR_TASK11          (0x00000800)
#define M5475_DMA_DIPR_TASK12          (0x00001000)
#define M5475_DMA_DIPR_TASK13          (0x00002000)
#define M5475_DMA_DIPR_TASK14          (0x00004000)
#define M5475_DMA_DIPR_TASK15          (0x00008000)

/* Bit definitions and macros for M5475_DMA_DIMR */
#define M5475_DMA_DIMR_TASK0           (0x00000001)
#define M5475_DMA_DIMR_TASK1           (0x00000002)
#define M5475_DMA_DIMR_TASK2           (0x00000004)
#define M5475_DMA_DIMR_TASK3           (0x00000008)
#define M5475_DMA_DIMR_TASK4           (0x00000010)
#define M5475_DMA_DIMR_TASK5           (0x00000020)
#define M5475_DMA_DIMR_TASK6           (0x00000040)
#define M5475_DMA_DIMR_TASK7           (0x00000080)
#define M5475_DMA_DIMR_TASK8           (0x00000100)
#define M5475_DMA_DIMR_TASK9           (0x00000200)
#define M5475_DMA_DIMR_TASK10          (0x00000400)
#define M5475_DMA_DIMR_TASK11          (0x00000800)
#define M5475_DMA_DIMR_TASK12          (0x00001000)
#define M5475_DMA_DIMR_TASK13          (0x00002000)
#define M5475_DMA_DIMR_TASK14          (0x00004000)
#define M5475_DMA_DIMR_TASK15          (0x00008000)

/* Bit definitions and macros for M5475_DMA_IMCR */
#define M5475_DMA_IMCR_SRC16(x)        (((x)&0x00000003) << 0)
#define M5475_DMA_IMCR_SRC17(x)        (((x)&0x00000003) << 2)
#define M5475_DMA_IMCR_SRC18(x)        (((x)&0x00000003) << 4)
#define M5475_DMA_IMCR_SRC19(x)        (((x)&0x00000003) << 6)
#define M5475_DMA_IMCR_SRC20(x)        (((x)&0x00000003) << 8)
#define M5475_DMA_IMCR_SRC21(x)        (((x)&0x00000003) << 10)
#define M5475_DMA_IMCR_SRC22(x)        (((x)&0x00000003) << 12)
#define M5475_DMA_IMCR_SRC23(x)        (((x)&0x00000003) << 14)
#define M5475_DMA_IMCR_SRC24(x)        (((x)&0x00000003) << 16)
#define M5475_DMA_IMCR_SRC25(x)        (((x)&0x00000003) << 18)
#define M5475_DMA_IMCR_SRC26(x)        (((x)&0x00000003) << 20)
#define M5475_DMA_IMCR_SRC27(x)        (((x)&0x00000003) << 22)
#define M5475_DMA_IMCR_SRC28(x)        (((x)&0x00000003) << 24)
#define M5475_DMA_IMCR_SRC29(x)        (((x)&0x00000003) << 26)
#define M5475_DMA_IMCR_SRC30(x)        (((x)&0x00000003) << 28)
#define M5475_DMA_IMCR_SRC31(x)        (((x)&0x00000003) << 30)
#define M5475_DMA_IMCR_SRC16_FEC0RX    (0x00000000)
#define M5475_DMA_IMCR_SRC17_FEC0TX    (0x00000000)
#define M5475_DMA_IMCR_SRC18_FEC0RX    (0x00000020)
#define M5475_DMA_IMCR_SRC19_FEC0TX    (0x00000080)
#define M5475_DMA_IMCR_SRC20_FEC1RX    (0x00000100)
#define M5475_DMA_IMCR_SRC21_DREQ1     (0x00000000)
#define M5475_DMA_IMCR_SRC21_FEC1TX    (0x00000400)
#define M5475_DMA_IMCR_SRC22_FEC0RX    (0x00001000)
#define M5475_DMA_IMCR_SRC23_FEC0TX    (0x00004000)
#define M5475_DMA_IMCR_SRC24_CTM0      (0x00010000)
#define M5475_DMA_IMCR_SRC24_FEC1RX    (0x00020000)
#define M5475_DMA_IMCR_SRC25_CTM1      (0x00040000)
#define M5475_DMA_IMCR_SRC25_FEC1TX    (0x00080000)
#define M5475_DMA_IMCR_SRC26_USBEP4    (0x00000000)
#define M5475_DMA_IMCR_SRC26_CTM2      (0x00200000)
#define M5475_DMA_IMCR_SRC27_USBEP5    (0x00000000)
#define M5475_DMA_IMCR_SRC27_CTM3      (0x00800000)
#define M5475_DMA_IMCR_SRC28_USBEP6    (0x00000000)
#define M5475_DMA_IMCR_SRC28_CTM4      (0x01000000)
#define M5475_DMA_IMCR_SRC28_DREQ1     (0x02000000)
#define M5475_DMA_IMCR_SRC28_PSC2RX    (0x03000000)
#define M5475_DMA_IMCR_SRC29_DREQ1     (0x04000000)
#define M5475_DMA_IMCR_SRC29_CTM5      (0x08000000)
#define M5475_DMA_IMCR_SRC29_PSC2TX    (0x0C000000)
#define M5475_DMA_IMCR_SRC30_FEC1RX    (0x00000000)
#define M5475_DMA_IMCR_SRC30_CTM6      (0x10000000)
#define M5475_DMA_IMCR_SRC30_PSC3RX    (0x30000000)
#define M5475_DMA_IMCR_SRC31_FEC1TX    (0x00000000)
#define M5475_DMA_IMCR_SRC31_CTM7      (0x80000000)
#define M5475_DMA_IMCR_SRC31_PSC3TX    (0xC0000000)



/*
 *
 * Interrupt Numbers
 *
 */

/*
 * Interrupt numbers on the 5475 are at fixed positions above
 * the base value (INT_NUM_BASE) defined in the BSP (m5475evb.h.)
 */

/* INTC0 Interrupt Source Assignments */

#define INT_NUM_EPORT1        (INT_NUM_BASE+1)     /* edge port 1 */
#define INT_NUM_EPORT2        (INT_NUM_BASE+2)     /* edge port 2 */
#define INT_NUM_EPORT3        (INT_NUM_BASE+3)     /* edge port 3 */
#define INT_NUM_EPORT4        (INT_NUM_BASE+4)     /* edge port 4 */
#define INT_NUM_EPORT5        (INT_NUM_BASE+5)     /* edge port 5 */
#define INT_NUM_EPORT6        (INT_NUM_BASE+6)     /* edge port 6 */
#define INT_NUM_EPORT7        (INT_NUM_BASE+7)     /* edge port 7 */

#define INT_NUM_USB_EP0       (INT_NUM_BASE+15)    /* USB 2.0 end point 0 */
#define INT_NUM_USB_EP1       (INT_NUM_BASE+16)    /* USB 2.0 end point 1 */
#define INT_NUM_USB_EP2       (INT_NUM_BASE+17)    /* USB 2.0 end point 2 */
#define INT_NUM_USB_EP3       (INT_NUM_BASE+18)    /* USB 2.0 end point 3 */
#define INT_NUM_USB_EP4       (INT_NUM_BASE+19)    /* USB 2.0 end point 4 */
#define INT_NUM_USB_EP5       (INT_NUM_BASE+20)    /* USB 2.0 end point 5 */
#define INT_NUM_USB_EP6       (INT_NUM_BASE+21)    /* USB 2.0 end point 6 */
#define INT_NUM_USB_GEN       (INT_NUM_BASE+22)    /* USB 2.0 general */
#define INT_NUM_USB_CORE      (INT_NUM_BASE+23)    /* USB 2.0 core */
#define INT_NUM_USB_OR_ALL    (INT_NUM_BASE+24)    /* USB 2.0 OR of all INTs */

#define INT_NUM_DSPI_OUFL     (INT_NUM_BASE+25)    /* DSPI over/underflow */
#define INT_NUM_DSPI_RFOF     (INT_NUM_BASE+26)    /* Receive FIFO overflow */
#define INT_NUM_DSPI_RFDF     (INT_NUM_BASE+27)    /* Receive FIFO drain */
#define INT_NUM_DSPI_TFUF     (INT_NUM_BASE+28)    /* Receive FIFO underflow */
#define INT_NUM_DSPI_TCF      (INT_NUM_BASE+29)    /* Transfer complete */
#define INT_NUM_DSPI_TFFF     (INT_NUM_BASE+30)    /* Transmit FIFO fill */
#define INT_NUM_DSPI_EOQF     (INT_NUM_BASE+31)    /* End of queue */

#define INT_NUM_PSC3          (INT_NUM_BASE+32)     /* PSC3 */
#define INT_NUM_PSC2          (INT_NUM_BASE+33)     /* PSC2 */
#define INT_NUM_PSC1          (INT_NUM_BASE+34)     /* PSC1 */
#define INT_NUM_PSC0          (INT_NUM_BASE+35)     /* PSC0 */

#define INT_NUM_TC            (INT_NUM_BASE+36)     /* COMM timer interrupts */

#define INT_NUM_SEC           (INT_NUM_BASE+37)     /* SEC */

#define INT_NUM_FEC1          (INT_NUM_BASE+38)     /* FEC 1 */
#define INT_NUM_FEC0          (INT_NUM_BASE+39)     /* FEC 0 */
#define INT_NUM_I2C           (INT_NUM_BASE+40)     /* I2C */
#define INT_NUM_PCIARB        (INT_NUM_BASE+41)     /* PCI Arbiter */
#define INT_NUM_CBPCI         (INT_NUM_BASE+42)     /* Comm bus PCI interrupt */
#define INT_NUM_XLBPCI        (INT_NUM_BASE+43)     /* XLB PCI */
#define INT_NUM_XLBARB        (INT_NUM_BASE+47)     /* XLB ARB to CPU INTs */
#define INT_NUM_DMA           (INT_NUM_BASE+48)     /* Multichannel DMA */
#define INT_NUM_SLT1          (INT_NUM_BASE+53)     /* Slice timer 1 */
#define INT_NUM_SLT0          (INT_NUM_BASE+54)     /* Slice timer 0 */

#define INT_NUM_GPT3          (INT_NUM_BASE+59)     /* Gen. purpose timer 3 */
#define INT_NUM_GPT2          (INT_NUM_BASE+60)     /* Gen. purpose timer 2 */
#define INT_NUM_GPT1          (INT_NUM_BASE+61)     /* Gen. purpose timer 1 */
#define INT_NUM_GPT0          (INT_NUM_BASE+62)     /* Gen. purpose timer 0 */

#ifdef __cplusplus
}
#endif

#endif /* __INCmcf5475h */
