/* regsColdfire.h - Coldfire registers */

/* 
Copyright (c) 2000,2001,2005,2006 Wind River Systems, Inc. 

The right to copy, distribute, modify or otherwise make use 
of this software may be licensed only pursuant to the terms 
of an applicable Wind River license agreement. 
*/

/*
modification history
--------------------
01i,14feb06,scm  add offsets for mmuCtx & pRgnTbl for signals support...
01h,19jan06,rec  fix merge errors
01g,19jan06,rec  add mmuCtx to register set
01f,16dec05,rec  fix SIZEOF_REG_SET to account for pRgnTbl
01e,29sep05,rec  add OFF_TCB_RGN_TBL and pRgnTbl
01d,28apr05,rec  change HASMAC to FORMAT
                 remvove MAC registers.  Not needed with coproc abstraction
01c,27nov01,dee  remove references to MCF5200
01b,07jul00,dh   added MAC support for 5206e and 53xx
01a,17mar00,dra  Written based on T1.0.1 Coldfire and T2 m68k
*/

#ifndef __INCregsColdfireh
#define __INCregsColdfireh

#ifdef __cplusplus
extern "C" {
#endif

#define DREG_NUM        8       /* has 8 32-bit data registers */
#define AREG_NUM        8       /* has 8 32-bit address registers */
#define ADREG_NUM       (AREG_NUM + DREG_NUM)
#define FPREG_NUM       8       /* has 8 64-bit fp data registers */

#ifdef  _ASMLANGUAGE

#define OFF_REG_D0      0
#define OFF_REG_D1      4
#define OFF_REG_D2      8
#define OFF_REG_D3      12
#define OFF_REG_D4      16
#define OFF_REG_D5      20
#define OFF_REG_D6      24
#define OFF_REG_D7      28
#define OFF_REG_A0      32
#define OFF_REG_A1      36
#define OFF_REG_A2      40
#define OFF_REG_A3      44
#define OFF_REG_A4      48
#define OFF_REG_A5      52
#define OFF_REG_A6      56
#define OFF_REG_A7      60
/* b6mod - MAC registers removed from TCB. Not needed with coproc abstraction */
/* b6mod -add user stack pointer */
#define OFF_REG_USP     64
#define OFF_REG_FORMAT  68
#define OFF_REG_SR      70
#define OFF_REG_PC      72      
#define OFF_REG_RGN_TBL 76      
#define OFF_MMU_CTX     80

#define SIZEOF_REG_SET	(OFF_MMU_CTX + 4)

/* Common pseudonyms */

#define OFF_REG_SP      OFF_REG_A7
#define OFF_REG_FP      OFF_REG_A6

/* used to quickly find the current RTP region table */
#define OFF_TCB_RGN_TBL (WIND_TCB_REGS + OFF_REG_RGN_TBL)

#define WIND_TCB_PRGNTBL        (WIND_TCB_REGS+OFF_REG_RGN_TBL)
#define WIND_TCB_MMUCTX         (WIND_TCB_REGS+OFF_MMU_CTX)

#else

extern int _coprocCfg;

typedef struct reg_set          /* REG_SET - Coldfire register set */
    {
    ULONG dataReg[DREG_NUM];    /* data registers */
    ULONG addrReg[AREG_NUM];    /* address registers */
    ULONG usp;                  /* user stack pointer */    
    USHORT format;              /* format register */
    USHORT sr;                  /* status register (must be second to last) */
    INSTR *pc;                  /* program counter (must be last phys reg) */
    int *pRgnTbl;               /* ptr to RTP mmu region tbl (not real) */
    int mmuCtx;			/* mmu context */    
    } REG_SET;

/* some common names for registers */

#define fpReg           addrReg[6]      /* frame pointer */
#define spReg           addrReg[7]      /* stack pointer */
#define reg_pc          pc              /* program counter */
#define reg_sp          addrReg[7]      /* stack pointer */
#define reg_fp          addrReg[6]      /* frame pointer */

#define  D_REG_BASE     0x00            /* data reg's base offset to REG_SET */
#define  D_REG_OFFSET(n)        (D_REG_BASE + (n)*sizeof(ULONG))
#define  A_REG_OFFSET(n)        D_REG_OFFSET(DREG_NUM + n)
#define  USP_OFFSET             A_REG_OFFSET(AREG_NUM)
#define  FORMAT_OFFSET          (USP_OFFSET + sizeof(ULONG))
#define  SR_OFFSET              (FORMAT_OFFSET+ sizeof(USHORT))
#define  PC_OFFSET              (SR_OFFSET + sizeof(USHORT))

#define  PRGNTBL_OFFSET         (PC_OFFSET + sizeof(INSTR *))
#define  MMUCTX_OFFSET          (PRGNTBL_OFFSET + sizeof(int *))
                                                                                
#endif  /* _ASMLANGUAGE */

#ifdef __cplusplus
}
#endif

#endif /* __INCregsColdfireh */
