/* syscallColdfire.h - Coldfire specific System Call Infrastructure header */

/* 
Copyright (c) 2005-2006 Wind River Systems, Inc. 

The right to copy, distribute, modify or otherwise make use 
of this software may be licensed only pursuant to the terms 
of an applicable Wind River license agreement. 
*/

/*
modification history
--------------------
01i,29mar06,scm  correct types...
01h,23feb06,scm  add syscall "PC"...
01g,17feb06,scm  correct offsets
01f,14feb06,scm  redo for signals support...
01e,26jan06,scm  fix _SYSCALL_USER_SP_CHECK...
01d,23nov05,rec	 fix offsets for user stack pointer
01c,18aug05,rec	 test fixes
01b,01jul05,rec	 review comments
01a,27may03,rec	 written.
*/

/*
DESCRIPTION
This header contains Coldfire specific definitions and constants used by
the System Call Infrastructure library.

*/

#ifndef __INCsyscallColdfireh
#define __INCsyscallColdfireh

#ifndef _ASMLANGUAGE
#include <taskLib.h>
#endif

#ifdef __cplusplus
extern "C" {
#endif

    /* defines */

#define syscallDispatch_PORTABLE	/* Use the portable dispatcher */

/* these offsets define the service frame format */

#define SYSCALL_ENTRY_STATE_OFFSET	 4 /* signal offset past return address */

#define SYSCALL_ENTRY_PARGS_OFFSET	 0 
#define SYSCALL_ENTRY_SCN_OFFSET	(SYSCALL_ENTRY_PARGS_OFFSET + 32)
#define SYSCALL_ENTRY_PUSTACK_OFFSET	(SYSCALL_ENTRY_SCN_OFFSET + 4)
#define SYSCALL_ENTRY_FORMAT_OFFSET	(SYSCALL_ENTRY_PUSTACK_OFFSET + 4)
#define SYSCALL_ENTRY_STATREG_OFFSET	(SYSCALL_ENTRY_FORMAT_OFFSET + 2)
#define SYSCALL_ENTRY_RTN_OFFSET	(SYSCALL_ENTRY_STATREG_OFFSET + 2)
#define SYSCALL_ENTRY_SCPC_OFFSET	(SYSCALL_ENTRY_RTN_OFFSET + 4)

/*
 * Coming into syscall, we require the use of exception stack...
 * these offsets define spare registers needed to keep syscall frame 
 * at the top of the exception stack...
 */

#define SC_ENTRY_PRV_STK_PTR_OFFSET   (SYSCALL_ENTRY_SCPC_OFFSET + 4)
#define SC_ENTRY_D0_REG_OFFSET        (SC_ENTRY_PRV_STK_PTR_OFFSET + 4)
#define SC_ENTRY_D1_REG_OFFSET        (SC_ENTRY_D0_REG_OFFSET + 4)
#define SC_ENTRY_A0_REG_OFFSET        (SC_ENTRY_D1_REG_OFFSET + 4)
#define SC_ENTRY_A1_REG_OFFSET        (SC_ENTRY_A0_REG_OFFSET + 4)
#define SC_ENTRY_ESF_FRMT_SR_OFFSET   (SC_ENTRY_A1_REG_OFFSET + 4)
#define SC_ENTRY_ESF_RTN_ADDR_OFFSET  (SC_ENTRY_ESF_FRMT_SR_OFFSET + 4)


/* these offsets define the user trap frame */

/* supervisor mode RTPs must have the frame pointer adjusted */

#define SYSCALL_USR_ESF_OFFSET		(7 * 4)	 /* room for regs d0-d1/a0-a1 */

#define SYSCALL_USR_SCN_OFFSET		(0 )
#define SYSCALL_USR_USRRTN_OFFSET	(SYSCALL_USR_SCN_OFFSET + 4)
#define SYSCALL_USR_ARG_OFFSET		(SYSCALL_USR_USRRTN_OFFSET + 4)

/* actual syscall frame size : arguments, regs, and spare regs */

#define SYSCALL_ENTRY_FRAME_SIZE        ((8*4) + (5*4) + (7*4))

/* syscall frame size adjustment : arguments, regs */

#define SC_ENTRY_FRAME_ADJUSTMENT       ((8*4) + (5*4))

#define _SYSCALL_USER_SP_CHECK(sp)	

    /* typedefs */

#ifndef _ASMLANGUAGE

    /*
     * The SYSCALL_ENTRY_STATE structure defines the saved machine state
     * when the system call trap is taken. This information is architecture
     * specific, and is used by the system call dispatcher to restart system
     * calls that have been interrupted by the occurance of signals.
     * System call restart is achieved by restoring the saved state at the
     * time of the system call trap, and re-issuing the system call.
     *
     * All arguments to the system call are saved on the kernel stack, and 
     * the address of that array is passed as an argument to the dispatcher.
     * The layout of this structure must exactly match the ordering of members
     * of the system call entry frame in src/arch/coldfire/syscallALib.s.
     *
     */

typedef struct syscall_entry_state
    {
    ULONG    args[8];	     /* argument list */
    ULONG    scn;	     /* System Call Number (SCN) */
    ULONG   *pUStack;	     /* user-mode stack pointer */
    ULONG    statusReg;      /* saved Status Register (SR) */
    ULONG    pc;	     /* return to caller pc */
    ULONG    scPc;	     /* syscall pc - return address back into syscallTrapHandler */

    /*
     * these offsets define spare registers needed to keep syscall frame
     * at the top of the exception stack...
     */

    ULONG    prv_stk_ptr;    /* saved previous stack pointer */
    ULONG    d0_reg;         /* saved data reg D0 */
    ULONG    d1_reg;         /* saved data reg D1 */
    ULONG    a0_reg;         /* saved address reg A0 */
    ULONG    a1_reg;         /* saved address reg A1 */
    ULONG    esf_frmt_sr;    /* ESF - format/sr regs */
    ULONG    esf_rtn_addr;   /* ESF - return address */
    } SYSCALL_ENTRY_STATE;
    
#endif /* _ASMLANGUAGE */

#ifdef __cplusplus
}
#endif

#endif /* __INCsyscallColdfireh */
