/* sh7785.h - SH7785 header */

/*
 * Copyright (c) 2005-2006 Wind River Systems, Inc.
 *
 * The right to copy, distribute, modify or otherwise make use
 * of this software may be licensed only pursuant to the terms
 * of an applicable Wind River license agreement.
 */

/*
modification history
--------------------
01g,01mar06,h_k  fixed apigen error.
01f,24sep04,h_k  cleaned up.
01e,20dec05,jpb  cleanup and merge 
01d,08dec05,jpb  Review INTC2, add INTC2_SETPRIO macro 
01c,05dec05,gce  Added PMB & EXPMASK related defines.
01b,15nov05,jpb  review, cleanup
01a,05oct05,gce  written based on [sh7780.h 01e]
*/

/*
DESCRIPTION
This file contains I/O addresses and related constants for the SH7785 chip.
*/

#ifndef __INCsh7785h
#define __INCsh7785h

#ifdef __cplusplus
extern "C" {
#endif

/* INTC (INTerrupt Controller) register addresses */

#define INTC_ICR0       ((volatile UINT32 *)0xffd00000) /* control register 0 */
#define INTC_ICR1       ((volatile UINT32 *)0xffd0001c) /* control register 1 */
#define INTC_INTPRI     ((volatile UINT32 *)0xffd00010) /* priority register */
#define INTC_INTREQ     ((volatile UINT32 *)0xffd00024) /* source register */
#define INTC_INTMSK0    ((volatile UINT32 *)0xffd00044) /* mask register 0 */
#define INTC_INTMSK1    ((volatile UINT32 *)0xffd00048) /* mask register 1 */
#define INTC_INTMSK2    ((volatile UINT32 *)0xffd40080) /* mask register 2 */
#define INTC_INTMSKCLR0 ((volatile UINT32 *)0xffd00064) /* mask clear reg 0 */
#define INTC_INTMSKCLR1 ((volatile UINT32 *)0xffd00068) /* mask clear reg 1 */
#define INTC_INTMSKCLR2 ((volatile UINT32 *)0xffd40084) /* mask clear reg 2 */
#define INTC_NMIFCR     ((volatile UINT32 *)0xffd000c0) /* NMI flag control */
#define INTC_USERIMASK  ((volatile UINT32 *)0xffd30000) /* user int mask */

/* INTC ICR (Interrupt Control Register) bit definitions */

#define ICR_NMI_STATUS          ((UINT32)0x80000000)    /* input level status */
#define ICR_NMI_BLOCK_MODE      ((UINT32)0x02000000)    /* NMI block mode */
#define ICR_NMI_LEAD_EDGE       ((UINT32)0x01000000)    /* leading edge */
#define ICR_NMI_TRAIL_EDGE      ((UINT32)0x0)           /* trailing edge */
#define ICR_NMI_IRL0_PIN_MODE   ((UINT32)0x00800000)    /* IRLM0 pin mode */
#define ICR_NMI_IRL1_PIN_MODE   ((UINT32)0x00400000)    /* IRLM1 pin mode */

/* ICR0 (INTerrupt Controller Register 0) definitions */

#define INTC_ICR0_NMIL          (UINT32)0x80000000
#define INTC_ICR0_MAI           (UINT32)0x40000000
#define INTC_ICR0_NMIB          (UINT32)0x02000000
#define INTC_ICR0_NMIE          (UINT32)0x01000000
#define INTC_ICR0_IRLM0         (UINT32)0x00800000
#define INTC_ICR0_IRLM1         (UINT32)0x00400000

/* ICR1 (INTerrupt Controller Register 1) definitions */

#define INTC_ICR1_FAL_EDGE      0
#define INTC_ICR1_RIS_EDGE      1
#define INTC_ICR1_LOW_LVL       2
#define INTC_ICR1_HIGH_LVL      3

#define INTC_ICR1_IRQ0_SHIFT    30
#define INTC_ICR1_IRQ1_SHIFT    28
#define INTC_ICR1_IRQ2_SHIFT    26
#define INTC_ICR1_IRQ3_SHIFT    24
#define INTC_ICR1_IRQ4_SHIFT    22
#define INTC_ICR1_IRQ5_SHIFT    20
#define INTC_ICR1_IRQ6_SHIFT    18
#define INTC_ICR1_IRQ7_SHIFT    16

/* INTREQ (INTerrupt Request Register) definitions */

#define INTC_REQ_IR0            (UINT32)0x80000000
#define INTC_REQ_IR1            (UINT32)0x40000000
#define INTC_REQ_IR2            (UINT32)0x20000000
#define INTC_REQ_IR3            (UINT32)0x10000000
#define INTC_REQ_IR4            (UINT32)0x08000000
#define INTC_REQ_IR5            (UINT32)0x04000000
#define INTC_REQ_IR6            (UINT32)0x02000000
#define INTC_REQ_IR7            (UINT32)0x01000000

/* INTMSK and INTMSKCLR Registers definitions */

#define INTC_MSK0_00            (UINT32)0x80000000
#define INTC_MSK0_01            (UINT32)0x40000000
#define INTC_MSK0_02            (UINT32)0x20000000
#define INTC_MSK0_03            (UINT32)0x10000000
#define INTC_MSK0_04            (UINT32)0x08000000
#define INTC_MSK0_05            (UINT32)0x04000000
#define INTC_MSK0_06            (UINT32)0x02000000
#define INTC_MSK0_07            (UINT32)0x01000000
#define INTC_MSK0_ALL           (UINT32)0xff000000

#define INTC_MSK1_10            (UINT32)0x80000000
#define INTC_MSK1_11            (UINT32)0x40000000
#define INTC_MSK1_ALL           (UINT32)0xc0000000

#define INTC_MSK2_ALL           (UINT32)0xfffffffe

/* NMIFCR (NMI Fragment Controll Register) definitions */

#define INTC_NMIFCR_NMIL	(UINT32)0x80000000

/* USERIMASK (User Mode Interrupt Mask Register) definitions */

#define INTC_USERIMASK_WKEY	(UINT32)0xa5000000
#define INTC_USERIMASK_SHIFT	4

/* INTC2 (INTerrupt Controller 2 (INT2 in doc)) register addresses */

#define INTC2_PRI0  ((volatile UINT32 *)0xffd40000) /* priority 0 setup */
#define INTC2_PRI1  ((volatile UINT32 *)0xffd40004) /* priority 1 setup */
#define INTC2_PRI2  ((volatile UINT32 *)0xffd40008) /* priority 2 setup */
#define INTC2_PRI3  ((volatile UINT32 *)0xffd4000c) /* priority 3 setup */
#define INTC2_PRI4  ((volatile UINT32 *)0xffd40010) /* priority 4 setup */
#define INTC2_PRI5  ((volatile UINT32 *)0xffd40014) /* priority 5 setup */
#define INTC2_PRI6  ((volatile UINT32 *)0xffd40018) /* priority 6 setup */
#define INTC2_PRI7  ((volatile UINT32 *)0xffd4001c) /* priority 7 setup */
#define INTC2_PRI8  ((volatile UINT32 *)0xffd40020) /* priority 8 setup */
#define INTC2_PRI9  ((volatile UINT32 *)0xffd40024) /* priority 9 setup */

#define INTC2_A0    ((volatile UINT32 *)0xffd40030) /* src (mask not affected)*/
#define INTC2_A1    ((volatile UINT32 *)0xffd40034) /* source (mask affected) */
#define INTC2_MSKRG ((volatile UINT32 *)0xffd40038) /* Interrupt mask         */
#define INTC2_MSKCR ((volatile UINT32 *)0xffd4003c) /* Interrupt mask clear   */

#define INTC2_B0    ((volatile UINT32 *)0xffd40040) /* Interrupt request mod */ 
#define INTC2_B1    ((volatile UINT32 *)0xffd40044) /* Interrupt request mod */ 
#define INTC2_B2    ((volatile UINT32 *)0xffd40048) /* Interrupt request mod */ 
#define INTC2_B3    ((volatile UINT32 *)0xffd4004c) /* Interrupt request mod */ 
#define INTC2_B4    ((volatile UINT32 *)0xffd40050) /* Interrupt request mod */ 
#define INTC2_B5    ((volatile UINT32 *)0xffd40054) /* Interrupt request mod */ 
#define INTC2_B6    ((volatile UINT32 *)0xffd40058) /* Interrupt request mod */ 
#define INTC2_B7    ((volatile UINT32 *)0xffd4005c) /* Interrupt request mod */ 

#define INTC2_GPIC  ((volatile UINT32 *)0xffd40090) /* gpio Interrupt        */

/* Setting SH7785 Interupts priority */

/* Refer to SH7785 documentation:

     Register   |   28-24    |   20-16    |   12-8     |    4-0 
    -------------------------------------------------------------------------
    INTC2_PRI0  |  TMU-ch0   |  TMU-ch1   |  TMU-ch2   |  TMU-ch2 Input Capture 
    INTC2_PRI1  |  TMU-ch3   |  TMU-ch4   |  TMU-ch5   |  Reserve(#)
    INTC2_PRI2  |  SCIF-ch0  |  SCIF-ch1  |  SCIF-ch2  |  SCIF-ch3 
    INTC2_PRI3  |  SCIF-ch4  |  SCIF-ch5  |  WDT       |  Reserve(#)
    INTC2_PRI4  |  H-UDI     |  DMAC(0)   |  DMAC(1)   |  Reserve(#)
    INTC2_PRI5  |  HAC-ch0   |  HAC-ch1   |  PCIC(0)   |  PCIC(1) 
    INTC2_PRI6  |  PCIC(2)   |  PCIC(3)   |  PCIC(4)   |  PCIC(5) 
    INTC2_PRI7  |  SIOF      |  HSPI      |  MMCIF     |  Reserve(#)
    INTC2_PRI8  |  FLCTL     |  GPIO      |  SSI-ch0   |  SSI-ch1 
    INTC2_PRI9  |  DU        |  GDTA      |  Reserve(#)|  Reserve(#)

(#). Please always set the value to write in to 0. 
*/
/*
    Don't forget, priorities are divided per two when present.
    So we multiply per two before storage.
*/


#define INTC2_SETREGPRIO(reg_,lwb_,pri_)  \
    *(reg_) = (*(reg_) & ~(0x1f << (lwb_))) | (((pri_) & 0x0f) << ((lwb_)+1))

#define _INTC2_TMU_CH0_REG              INTC2_PRI0
#define _INTC2_TMU_CH0_LWB              24
#define _INTC2_TMU_CH1_REG              INTC2_PRI0
#define _INTC2_TMU_CH1_LWB              16
#define _INTC2_TMU_CH2_REG              INTC2_PRI0
#define _INTC2_TMU_CH2_LWB              8
#define _INTC2_TMU_CH3_REG              INTC2_PRI1
#define _INTC2_TMU_CH3_LWB              24
#define _INTC2_TMU_CH4_REG              INTC2_PRI1
#define _INTC2_TMU_CH4_LWB              16
#define _INTC2_TMU_CH5_REG              INTC2_PRI1
#define _INTC2_TMU_CH5_LWB              8

#define _INTC2_SCIF_CH0_REG             INTC2_PRI2
#define _INTC2_SCIF_CH0_LWB             24
#define _INTC2_SCIF_CH1_REG             INTC2_PRI2
#define _INTC2_SCIF_CH1_LWB             16
#define _INTC2_SCIF_CH2_REG             INTC2_PRI2
#define _INTC2_SCIF_CH2_LWB             8
#define _INTC2_SCIF_CH3_REG             INTC2_PRI2
#define _INTC2_SCIF_CH3_LWB             0
#define _INTC2_SCIF_CH4_REG             INTC2_PRI3
#define _INTC2_SCIF_CH4_LWB             24
#define _INTC2_SCIF_CH5_REG             INTC2_PRI3
#define _INTC2_SCIF_CH5_LWB             16

#define _INTC2_DMAC_0_REG               INTC2_PRI4
#define _INTC2_DMAC_0_LWB               16
#define _INTC2_DMAC_1_REG               INTC2_PRI4
#define _INTC2_DMAC_1_LWB               8

#define _INTC2_PCIC_0_REG               INTC2_PRI5
#define _INTC2_PCIC_0_LWB               8
#define _INTC2_PCIC_1_REG               INTC2_PRI5
#define _INTC2_PCIC_1_LWB               0
#define _INTC2_PCIC_2_REG               INTC2_PRI6
#define _INTC2_PCIC_2_LWB               24
#define _INTC2_PCIC_3_REG               INTC2_PRI6
#define _INTC2_PCIC_3_LWB               16
#define _INTC2_PCIC_4_REG               INTC2_PRI6
#define _INTC2_PCIC_4_LWB               8
#define _INTC2_PCIC_5_REG               INTC2_PRI6
#define _INTC2_PCIC_5_LWB               0

/* Timer Unit chan. 2 Input Capture */

#define _INTC2_TMU_CH2_INPUT_CAP_REG    INTC2_PRI0
#define _INTC2_TMU_CH2_INPUT_CAP_LWB    0

/* WatchDog Timer */

#define _INTC2_WDT_INTERVAL_TIMER_REG   INTC2_PRI3
#define _INTC2_WDT_INTERVAL_TIMER_LWB   8

/* User Debug Interface */

#define _INTC2_H_UDI_REG                INTC2_PRI4
#define _INTC2_H_UDI_LWB                24

/* Audio Codec Interface */

#define _INTC2_HAC_CH0_REG              INTC2_PRIO5 
#define _INTC2_HAC_CH0_LWB              24          
#define _INTC2_HAC_CH1_REG              INTC2_PRIO5
#define _INTC2_HAC_CH1_LWB              16  

/* Serial I/O with FIFO */

#define _INTC2_SIOF_REG                 INTC2_PRI7
#define _INTC2_SIOF_LWB                 24         

/* Serial Peripheral Interface */    

#define _INTC2_HSPI_REG                 INTC2_PRI7
#define _INTC2_HSPI_LWB                 16

/* Multimedia Card Interface */

#define _INTC2_MMCIF_REG                INTC2_PRI7
#define _INTC2_MMCIF_LWB                8           

/* NAND FLash Memory Controller */

#define  _INT2C_FLCTL_REG               INTC2_PRI8  
#define  _INT2C_FLCTL_LWB               24

/* General purpose I/O */

#define  _INT2C_GPIO_REG                INTC2_PRI8  
#define  _INT2C_GPIO_LWB                16

/* Sound Serial Interface */ 

#define _INTC2_SSI_CH0_REG              INTC2_PRIO8           
#define _INTC2_SSI_CH0_LWB              8 
#define _INT2C_SSI_CH1_REG              INTC2_PRIO8
#define _INT2C_SSI_CH1_LWB              0 

/* Display Unit */

#define _INTC2_DU_REG                   INTC2_PRI9
#define _INTC2_DU_LWB                   24

/* GDTA Interrupt */

#define _INTC2_GDTA_REG                 INTC2_PRI9
#define _INTC2_GDTA_LWB                 16


#define INTC2_SETPRIO(org_, pri_)  \
    INTC2_SETREGPRIO(_INTC2_##org_##_REG,_INTC2_##org_##_LWB,pri_)

/*
 * Example usage:
 *  #define SYS_TMU0_INT_ENABLE(pri)  INTC2_SETPRIO(TMU_CH0, pri)
 * or 
 *  intConnect(INUM_TO_IVEC(INUM_PCI_SERR), pciIntSERR, 0);
 *  INTC2_SETPRIO(PCIC_0, INT_LVL_PCISERR);
 */

/* FRQCR (FReQuency Control Register) */

#define FRQCR       ((volatile UINT32 *)0xffc80000)     /* */    

/* FRQCR (FReQuency Control Register) bit definitions */

#define FRQCR_DEFAULT           ((UINT32)0x10444046)

/* STBCR (Stanby Control Register) */

#define STBCR       ((volatile UINT8  *)0xffc00004)     /* */    
#define STBCR2      ((volatile UINT8  *)0xffc00010)     /* */    


/*******************************************/
/* WDT (WatchDog Timer) register addresses */
/*******************************************/

#define WDT_WDTST_WR    ((volatile UINT32 *)0xffcc0000) /* stop time reg */
#define WDT_WTCSR_WR    ((volatile UINT32 *)0xffcc0004) /* ctrl/status reg */
#define WDT_WDTBST_WR   ((volatile UINT32 *)0xffcc0008) /* stop time reg */
#define WDT_WTCNT_RD    ((volatile UINT32 *)0xffcc0010) /* counter */
#define WDT_WDTBCNT_RD  ((volatile UINT32 *)0xffcc0018) /* base counter */

/* WDT WDTST (Watchdog timer stop time register) register bit def */

#define WDTST_PASSWD        0x5a000000          /* WDTST write password */
#define WDTST_VALMASK       0x00000FFF          /* Count value on bit0-11 */

/* WDT WTCSR (Watchdog Timer Control/Status Register) bit definitions */

#define WTCSR_PASSWD        0xa5000000	/* WTCSR write password */
#define WTCSR_TIMER_DISABLE 0x00000000	/* stop count */

#define WTCSR_TIMER_ENABLE  0x00000080	/* TME - TIMER ENABLE */
#define WTCSR_WATCHDOG_MODE 0x00000040	/* 1:WDT mode - 0:Interval Timer    */
#define WTCSR_RSTS_MANUAL   0x00000020	/* 1:Manual reset - 0:Power-On reset */
#define WTCSR_WOVF_STATUS   0x00000010	/* overflow status in watchdog mode */
#define WTCSR_IOVF_STATUS   0x00000008	/* overflow status in interval mode */

/* WDT WDTBST (Watchdog timer base stop time register) bit definitions */

#define WDTBST_PASSWD       0x55000000          /* WDTBST write password */
#define WDTBST_VALMASK      0x000FFFFF          /* Count value on bit0-19 */

/*
 * TMU (32bit x 6 ch TiMer Unit) register addresses 
 *
 * The 7751 is a 32 bit x 5ch Timer Unit. The 7780 has 1 additional
 * timer (timer5). Sh7780 uses the same registers and addresses for  
 * the first 3 timers definitions (timer0 to timer2) but different
 * addresses for timers 3 & 4 and new addresses for timer5.
 */

#define TMU_TSTR0   ((volatile UINT8  *)0xffd80004)
					/* Timer start register 0    R/W */
					/* Common channels 0, 1 & 2      */
#define TMU_TCOR0   ((volatile UINT32 *)0xffd80008)
					/* Timer constant register 0 R/W */
#define TMU_TCNT0   ((volatile UINT32 *)0xffd8000c)
					/* Timer counter 0           R/W */
#define TMU_TCR0    ((volatile UINT16 *)0xffd80010)
					/* Timer control register 0  R/W */
#define TMU_TCOR1   ((volatile UINT32 *)0xffd80014)
					/* Timer constant register 1 R/W */
#define TMU_TCNT1   ((volatile UINT32 *)0xffd80018)
					/* Timer counter 1           R/W */
#define TMU_TCR1    ((volatile UINT16 *)0xffd8001c)
					/* Timer control register 1  R/W */
#define TMU_TCOR2   ((volatile UINT32 *)0xffd80020)
					/* Timer constant register 2 R/W */
#define TMU_TCNT2   ((volatile UINT32 *)0xffd80024)
					/* Timer counter 2           R/W */
#define TMU_TCR2    ((volatile UINT16 *)0xffd80028)
					/* Timer control register 2  R/W */
#define TMU_TCPR2   ((volatile UINT32 *)0xffd8002c)
					/* Input capture register 2   R  */
#define TMU_TSTR1   ((volatile UINT8  *)0xffdc0004)
					/* Timer start register 1    R/W */
					/* Common channels 3, 4 & 5      */
#define TMU_TCOR3   ((volatile UINT32 *)0xffdc0008)
					/* Timer constant register 3 R/W */
#define TMU_TCNT3   ((volatile UINT32 *)0xffdc000c)
					/* Timer counter 3           R/W */
#define TMU_TCR3    ((volatile UINT16 *)0xffdc0010)
					/* Timer control register 3  R/W */
#define TMU_TCOR4   ((volatile UINT32 *)0xffdc0014)
					/* Timer constant register 4 R/W */
#define TMU_TCNT4   ((volatile UINT32 *)0xffdc0018)
					/* Timer counter 4           R/W */
#define TMU_TCR4    ((volatile UINT16 *)0xffdc001c)
					/* Timer control register 4  R/W */
#define TMU_TCOR5   ((volatile UINT32 *)0xffdc0020)
					/* Timer constant register 5 R/W */
#define TMU_TCNT5   ((volatile UINT32 *)0xffdc0024)
					/* Timer counter 5           R/W */
#define TMU_TCR5    ((volatile UINT16 *)0xffdc0028)
					/* Timer control register 5  R/W */

#define TMU_TIMER_NBR     6   /* SH7785 has 6 Timers */

#define TOCR_DEFAULT    ((UINT8)0x00)
#define TOCR_TCOE       ((UINT8)0x01)   /* RTC output */
#define TOCR_TCLK_IN    TOCR_DEFAULT
#define TOCR_TCLK_OUT   TOCR_TCOE

/* TMU_TSTR  - TMU_TSTR_START_CNTL (Timer STart Register) bit definitions */

#define TMU_TSTR        TMU_TSTR0     /* For backward compatibility */

#define TSTR_STOP_ALL   ((UINT8)0x00)       /* stop counter 0/1/2   */
#define TSTR_STR2       ((UINT8)0x04)       /* start counter 2      */
#define TSTR_STR1       ((UINT8)0x02)       /* start counter 1      */
#define TSTR_STR0       ((UINT8)0x01)       /* start counter 0      */

/* TMU_TSTR2 - TMU_TSTR2_START_CNTL (Timer STart Register2) bit definitions */

#define TMU_TSTR2        TMU_TSTR1     /* For backward compatibility */

#define TSTR2_STOP_ALL  ((UINT8)0x00)       /* stop counter 3/4/5   */
#define TSTR2_STR5      ((UINT8)0x04)       /* start counter 5      */
#define TSTR2_STR4      ((UINT8)0x02)       /* start counter 4      */
#define TSTR2_STR3      ((UINT8)0x01)       /* start counter 3      */

/* TMU_TCR_CNTL (Timer Control Register) bit definitions */

#define TCR_DEFAULT     ((UINT16)0x0000)
#define TCR_ICPF        ((UINT16)0x0200)
				/* input-capture status (ch.2 only)     */
#define TCR_UNF         ((UINT16)0x0100)
				/* underflow status                     */
#define TCR_ICPE1       ((UINT16)0x0080)
				/* enable input-capture (ch.2 only)     */
#define TCR_ICPE0       ((UINT16)0x0040)
				/* enable input-capture interrupt       */
#define TCR_UNIE        ((UINT16)0x0020)
				/* enable underflow interrupt           */
#define TCR_CKEG_LEAD   ((UINT16)0x0000)
				/* count on leading edge                */
#define TCR_CKEG_TRAIL  ((UINT16)0x0008)
				/* count on trailing edge               */
#define TCR_CKEG_BOTH   ((UINT16)0x0010)
				/* use both edge (up & down)            */
#define TCR_TPSC_P4     ((UINT16)0x0000)
				/* count by P/4                         */
#define TCR_TPSC_P16    ((UINT16)0x0001)
				/* count by P/16                        */
#define TCR_TPSC_P64    ((UINT16)0x0002)
				/* count by P/64                        */
#define TCR_TPSC_P256   ((UINT16)0x0003)
				/* count by P/256                       */
#define TCR_TPSC_P1024  ((UINT16)0x0004)
				/* count by P/1024                      */
#define TCR_TPSC_RTCCLK ((UINT16)0x0006)
				/* count by RTCCLK                      */
#define TCR_TPSC_TCLK   ((UINT16)0x0007)
				/* count by TCLK - do not set on chan 3,4,5 */

/* GPIO register addresseS */

#define GPIO_PACR   ((volatile UINT16 *)0xffe70000)
				/* Port A Control Register */
#define GPIO_PBCR   ((volatile UINT16 *)0xffe70002)
				/* Port B Control Register */
#define GPIO_PCCR   ((volatile UINT16 *)0xffe70004)
				/* Port C Control Register */
#define GPIO_PDCR   ((volatile UINT16 *)0xffe70006)
				/* Port D Control Register */
#define GPIO_PECR   ((volatile UINT16 *)0xffe70008)
				/* Port E Control Register */
#define GPIO_PFCR   ((volatile UINT16 *)0xffe7000a)
				/* Port F Control Register */
#define GPIO_PGCR   ((volatile UINT16 *)0xffe7000c)
				/* Port G Control Register */
#define GPIO_PHCR   ((volatile UINT16 *)0xffe7000e)
				/* Port H Control Register */
#define GPIO_PJCR   ((volatile UINT16 *)0xffe70010)
				/* Port J Control Register */
#define GPIO_PKCR   ((volatile UINT16 *)0xffe70012)
				/* Port K Control Register */
#define GPIO_PLCR   ((volatile UINT16 *)0xffe70014)
				/* Port L Control Register */
#define GPIO_PMCR   ((volatile UINT16 *)0xffe70016)
				/* Port M Control Register */
#define GPIO_PNCR   ((volatile UINT16 *)0xffe70018)
				/* Port N Control Register */
#define GPIO_PPCR   ((volatile UINT16 *)0xffe7001a)
				/* Port P Control Register */
#define GPIO_PQCR   ((volatile UINT16 *)0xffe7001c)
				/* Port Q Control Register */
#define GPIO_PRCR   ((volatile UINT16 *)0xffe7001e)
				/* Port R Control Register */

#define GPIO_PADR   ((volatile UINT8 *)0xffe70020)  /* Port A data register */
#define GPIO_PBDR   ((volatile UINT8 *)0xffe70022)  /* Port B data register */
#define GPIO_PCDR   ((volatile UINT8 *)0xffe70024)  /* Port C data register */
#define GPIO_PDDR   ((volatile UINT8 *)0xffe70026)  /* Port D data register */
#define GPIO_PEDR   ((volatile UINT8 *)0xffe70028)  /* Port E data register */
#define GPIO_PFDR   ((volatile UINT8 *)0xffe7002A)  /* Port F data register */
#define GPIO_PGDR   ((volatile UINT8 *)0xffe7002C)  /* Port G data register */
#define GPIO_PHDR   ((volatile UINT8 *)0xffe7002E)  /* Port H data register */
#define GPIO_PJDR   ((volatile UINT8 *)0xffe70030)  /* Port J data register */
#define GPIO_PKDR   ((volatile UINT8 *)0xffe70032)  /* Port K data register */
#define GPIO_PLDR   ((volatile UINT8 *)0xffe70034)  /* Port L data register */
#define GPIO_PMDR   ((volatile UINT8 *)0xffe70036)  /* Port M data register */
#define GPIO_PNDR   ((volatile UINT8 *)0xffe70038)  /* Port N data register */
#define GPIO_PPDR   ((volatile UINT8 *)0xffe7003a)  /* Port P data register */
#define GPIO_PQDR   ((volatile UINT8 *)0xffe7003c)  /* Port Q data register */
#define GPIO_PRDR   ((volatile UINT8 *)0xffe7003e)  /* Port R data register */

#define GPIO_PEPUPR ((volatile UINT8 *)0xffe70048)
				/* Port E pull-up control register */
#define GPIO_PHPUPR ((volatile UINT8 *)0xffe7004E)
				/* Port H pull-up control register */
#define GPIO_PJPUPR ((volatile UINT8 *)0xffe70050)
				/* Port J pull-up control register */
#define GPIO_PKPUPR ((volatile UINT8 *)0xffe70052)
				/* Port K pull-up control register */
#define GPIO_PLPUPR ((volatile UINT8 *)0xffe70054)
				/* Port L pull-up control register */
#define GPIO_PMPUPR ((volatile UINT8 *)0xffe70056)
				/* Port M pull-up control register */
#define GPIO_PNPUPR ((volatile UINT8 *)0xffe70058)
				/* Port N pull-up control register */
#define GPIO_PPUPR1 ((volatile UINT16 *)0xffe70060)
				/* Input pin pull-up control register 1 */
#define GPIO_PPUPR2 ((volatile UINT16 *)0xffe70062)
				/* Input pin pull-up control register 2 */
#define GPIO_P1MSELR ((volatile UINT16 *)0xffe70080)
				/* Peripheral module select register */
#define GPIO_P2MSELR ((volatile UINT16 *)0xffe70082)
				/* Peripheral module select register */

/* UBC (User Break Controller) register addresses */

#define UBC_CBR0    ((volatile UINT32 *)0xff200000) /* match cond. set 0    */
#define UBC_CRR0    ((volatile UINT32 *)0xff200004) /* match oper. set 0    */
#define UBC_CAR0    ((volatile UINT32 *)0xff200008) /* match addr. set 0    */
#define UBC_CAMR0   ((volatile UINT32 *)0xff20000c) /* match addr. mask 0   */
#define UBC_CBR1    ((volatile UINT32 *)0xff200020) /* match cond. set 1    */
#define UBC_CRR1    ((volatile UINT32 *)0xff200024) /* match oper. set 1    */
#define UBC_CAR1    ((volatile UINT32 *)0xff200028) /* match addr. set 1    */
#define UBC_CAMR1   ((volatile UINT32 *)0xff20002c) /* match addr. mask 1   */
#define UBC_CDR1    ((volatile UINT32 *)0xff200030) /* match data set 1     */
#define UBC_CDMR1   ((volatile UINT32 *)0xff200034) /* match data mask 1    */
#define UBC_CETR1   ((volatile UINT32 *)0xff200038) /* exe. cnt. enable 1   */
#define UBC_CCMFR   ((volatile UINT32 *)0xff200600) /* channel match flag   */
#define UBC_CBCR    ((volatile UINT32 *)0xff200620) /* break control        */

/* CPG (Clock Pulse Generator ) registers addresses */

#define CPG_FRQCR0  ((volatile UINT32 *)0xffc80000)
				/* Frequency control register 0 */
#define CPG_FRQCR1  ((volatile UINT32 *)0xffc80004)
				/* Frequency control register 1 */
#define CPG_FRQMCR1 ((volatile UINT32 *)0xffc80014)
				/* Frequency spectrum reg 1     */
#define CPG_SLPCR   ((volatile UINT32 *)0xffc80020)
				/* Sleep control register       */
#define CPG_PLLCR   ((volatile UINT32 *)0xffc80024)
				/* PLL control register         */
#define CPG_MSTPCR0 ((volatile UINT32 *)0xffc80030)
				/* Standby control register 0   */
#define CPG_MSTPCR1 ((volatile UINT32 *)0xffc80034)
				/* Standby control register 1   */
#define CPG_MSTPMR  ((volatile UINT32 *)0xffc80044)
				/* Standby display register     */

/* SCIF (Serial Communication Interface with FIFO) register addresses */
 
/* 
 * Use UINT32 sh7780ScifGetChanBaseAddr(int channel)
 *
 */
#define SCIF_CHAN_NBR        6   /* SH7785 has 6 SCIF Channels */

#define SCIF0_BASE      0xFFEA0000
#define SCIF1_BASE      0xFFEB0000
#define SCIF2_BASE      0xFFEC0000
#define SCIF3_BASE      0xFFED0000
#define SCIF4_BASE      0xFFEE0000
#define SCIF5_BASE      0xFFEF0000

#define SCIF_BASE_ADDR(ch_)  ((UINT32)(0xFFEA0000 + ((ch_)<<16)))


#define SCIF_OFFSET_SCSMR     (0x00)  /* serial mode              */
#define SCIF_OFFSET_SCBRR     (0x04)  /* bit rate                 */
#define SCIF_OFFSET_SCSCR     (0x08)  /* serial control           */
#define SCIF_OFFSET_SCFTDR    (0x0c)  /* transmit FIFO data       */
#define SCIF_OFFSET_SCFSR     (0x10)  /* serial status            */
#define SCIF_OFFSET_SCFRDR    (0x14)  /* receive FIFO data        */
#define SCIF_OFFSET_SCFCR     (0x18)  /* FIFO control             */
#define SCIF_OFFSET_SCTFDR    (0x1c)  /* Transmit FIFO data count */
#define SCIF_OFFSET_SCRFDR    (0x20)  /* Receive FIFO data count  */
#define SCIF_OFFSET_SCSPTR    (0x24)  /* serial port reg          */
#define SCIF_OFFSET_SCLSR     (0x28)  /* line status              */
#define SCIF_OFFSET_SCRER     (0x2c)  /* serial error             */

#define SCIF_SCSMR(ch_)    \
	((volatile UINT16 *)(SCIF_BASE_ADDR(ch_)+SCIF_OFFSET_SCSMR ))
						/* serial mode              */
#define SCIF_SCBRR(ch_)    \
	((volatile UINT8  *)(SCIF_BASE_ADDR(ch_)+SCIF_OFFSET_SCBRR ))
						/* bit rate                 */
#define SCIF_SCSCR(ch_)    \
	((volatile UINT16 *)(SCIF_BASE_ADDR(ch_)+SCIF_OFFSET_SCSCR ))
						/* serial control           */
#define SCIF_SCFTDR(ch_)   \
	((volatile UINT8  *)(SCIF_BASE_ADDR(ch_)+SCIF_OFFSET_SCFTDR))
						/* transmit FIFO data       */
#define SCIF_SCFSR(ch_)    \
	((volatile UINT16 *)(SCIF_BASE_ADDR(ch_)+SCIF_OFFSET_SCFSR ))
						/* serial status            */
#define SCIF_SCFRDR(ch_)   \
	((volatile UINT8  *)(SCIF_BASE_ADDR(ch_)+SCIF_OFFSET_SCFRDR))
						/* receive FIFO data        */
#define SCIF_SCFCR(ch_)    \
	((volatile UINT16 *)(SCIF_BASE_ADDR(ch_)+SCIF_OFFSET_SCFCR ))
						/* FIFO control             */
#define SCIF_SCTFDR(ch_)   \
	((volatile UINT16 *)(SCIF_BASE_ADDR(ch_)+SCIF_OFFSET_SCTFDR))
						/* Transmit FIFO data count */
#define SCIF_SCRFDR(ch_)   \
	((volatile UINT16 *)(SCIF_BASE_ADDR(ch_)+SCIF_OFFSET_SCRFDR))
						/* Receive FIFO data count  */
#define SCIF_SCSPTR(ch_)   \
	((volatile UINT16 *)(SCIF_BASE_ADDR(ch_)+SCIF_OFFSET_SCSPTR))
						/* serial port reg          */
#define SCIF_SCLSR(ch_)    \
	((volatile UINT16 *)(SCIF_BASE_ADDR(ch_)+SCIF_OFFSET_SCLSR ))
						/* line status              */
#define SCIF_SCRER(ch_)    \
	((volatile UINT16 *)(SCIF_BASE_ADDR(ch_)+SCIF_OFFSET_SCRER ))
						/* serial error             */

/* For backward compatibility */
#define SCIF_SCSMR0     SCIF_SCSMR(0)     
#define SCIF_SCBRR0     SCIF_SCBRR(0) 
#define SCIF_SCSCR0     SCIF_SCSCR(0) 
#define SCIF_SCFTDR0    SCIF_SCFTDR(0)
#define SCIF_SCFSR0     SCIF_SCFSR(0) 
#define SCIF_SCFRDR0    SCIF_SCFRDR(0)
#define SCIF_SCFCR0     SCIF_SCFCR(0) 
#define SCIF_SCTFDR0    SCIF_SCTFDR(0)
#define SCIF_SCRFDR0    SCIF_SCRFDR(0)
#define SCIF_SCSPTR0    SCIF_SCSPTR(0)
#define SCIF_SCLSR0     SCIF_SCLSR(0) 
#define SCIF_SCRER0     SCIF_SCRER(0) 

#define SCIF_SCSMR1     SCIF_SCSMR(1)     
#define SCIF_SCBRR1     SCIF_SCBRR(1) 
#define SCIF_SCSCR1     SCIF_SCSCR(1) 
#define SCIF_SCFTDR1    SCIF_SCFTDR(1)
#define SCIF_SCFSR1     SCIF_SCFSR(1) 
#define SCIF_SCFRDR1    SCIF_SCFRDR(1)
#define SCIF_SCFCR1     SCIF_SCFCR(1) 
#define SCIF_SCTFDR1    SCIF_SCTFDR(1)
#define SCIF_SCRFDR1    SCIF_SCRFDR(1)
#define SCIF_SCSPTR1    SCIF_SCSPTR(1)
#define SCIF_SCLSR1     SCIF_SCLSR(1) 
#define SCIF_SCRER1     SCIF_SCRER(1) 


/* Local Bus State Controller (LBSC) register definitions */

#define LBSC_MMSELR ((volatile UINT32 *)0xff400020) /* Memory Adrs Map Select*/ 
#define LBSC_BCR    ((volatile UINT32 *)0xff801000) /* Bus Control Register */
#define LBSC_CS0BCR ((volatile UINT32 *)0xff802000) /* CS0 Bus Control Reg */
#define LBSC_CS1BCR ((volatile UINT32 *)0xff802010) /* CS1 Bus Control Reg */
#define LBSC_CS2BCR ((volatile UINT32 *)0xff802020) /* CS2 Bus Control Reg */
#define LBSC_CS4BCR ((volatile UINT32 *)0xff802040) /* CS4 Bus Control Reg */
#define LBSC_CS5BCR ((volatile UINT32 *)0xff802050) /* CS5 Bus Control Reg */
#define LBSC_CS6BCR ((volatile UINT32 *)0xff802060) /* CS6 Bus Control Reg */
#define LBSC_CS0WCR ((volatile UINT32 *)0xff802008) /* CS0 Wait Control Reg */
#define LBSC_CS1WCR ((volatile UINT32 *)0xff802018) /* CS1 Wait Control Reg */
#define LBSC_CS2WCR ((volatile UINT32 *)0xff802028) /* CS2 Wait Control Reg */
#define LBSC_CS4WCR ((volatile UINT32 *)0xff802048) /* CS4 Wait Control Reg */
#define LBSC_CS5WCR ((volatile UINT32 *)0xff802058) /* CS5 Wait Control Reg */
#define LBSC_CS6WCR ((volatile UINT32 *)0xff802068) /* CS6 Wait Control Reg */
#define LBSC_CS5PCR ((volatile UINT32 *)0xff802070) /* CS5 PCMCIA Control Reg */
#define LBSC_CS6PCR ((volatile UINT32 *)0xff802080) /* CS6 PCMCIA Control Reg */

/* Specific PCI Control Registers */

#define PCIIECR         ((volatile UINT32 *)0xfe000008)
					/* PCIC enable control register */
#define PCIIECR_ENBL    0x00000001
					/* PCI Enable Bit */

#ifndef PCIC_PCI_CONFIG_BASE
#define PCIC_PCI_CONFIG_BASE    0xfe040000
					/* configuration register 0 */
#endif  /* PCIC_PCI_CONFIG_BASE */

#define PCIC_CONFIG_0   ((volatile UINT32 *)(PCIC_PCI_CONFIG_BASE+0x00))
					/* PCI config reg 0  */
#define PCIC_CONFIG_1   ((volatile UINT32 *)(PCIC_PCI_CONFIG_BASE+0x04))
					/* PCI config reg 1  */
#define PCIC_CONFIG_2   ((volatile UINT32 *)(PCIC_PCI_CONFIG_BASE+0x08))
					/* PCI config reg 2  */
#define PCIC_CONFIG_3   ((volatile UINT32 *)(PCIC_PCI_CONFIG_BASE+0x0C))
					/* PCI config reg 3  */
#define PCIC_CONFIG_4   ((volatile UINT32 *)(PCIC_PCI_CONFIG_BASE+0x10))
					/* PCI config reg 4  */
#define PCIC_CONFIG_5   ((volatile UINT32 *)(PCIC_PCI_CONFIG_BASE+0x14))
					/* PCI config reg 5  */
#define PCIC_CONFIG_6   ((volatile UINT32 *)(PCIC_PCI_CONFIG_BASE+0x18))
					/* PCI config reg 6  */
#define PCIC_CONFIG_7   ((volatile UINT32 *)(PCIC_PCI_CONFIG_BASE+0x1C))
					/* PCI config reg 7  */
#define PCIC_CONFIG_8   ((volatile UINT32 *)(PCIC_PCI_CONFIG_BASE+0x20))
					/* PCI config reg 8  */
#define PCIC_CONFIG_9   ((volatile UINT32 *)(PCIC_PCI_CONFIG_BASE+0x24))
					/* PCI config reg 9  */
#define PCIC_CONFIG_10  ((volatile UINT32 *)(PCIC_PCI_CONFIG_BASE+0x28))
					/* PCI config reg 10 */
#define PCIC_CONFIG_11  ((volatile UINT32 *)(PCIC_PCI_CONFIG_BASE+0x2C))
					/* PCI config reg 11 */
#define PCIC_CONFIG_12  ((volatile UINT32 *)(PCIC_PCI_CONFIG_BASE+0x30))
					/* PCI config reg 12 */
#define PCIC_CONFIG_13  ((volatile UINT32 *)(PCIC_PCI_CONFIG_BASE+0x34))
					/* PCI config reg 13 */
#define PCIC_CONFIG_14  ((volatile UINT32 *)(PCIC_PCI_CONFIG_BASE+0x38))
					/* PCI config reg 14 */
#define PCIC_CONFIG_15  ((volatile UINT32 *)(PCIC_PCI_CONFIG_BASE+0x3C))
					/* PCI config reg 15 */
#define PCIC_CONFIG_16  ((volatile UINT32 *)(PCIC_PCI_CONFIG_BASE+0x40))
					/* PCI config reg 16 */
#define PCIC_CONFIG_17  ((volatile UINT32 *)(PCIC_PCI_CONFIG_BASE+0x44))
					/* PCI config reg 17 */

/* PCI local registers */

#define PCICR           ((volatile UINT32 *)(PCIC_PCI_CONFIG_BASE+0x100))
					/* PCI Control Register */

/* PCI Control Register Bit field Mask */
#define PCICR_CFINIT    0x0001		/* set to initialise pcic */  
#define PCICR_INTA      0x0004		/* Software control of INTA */
#define PCICR_SERR      0x0008		/* Software control of SERR */
#define PCICR_BMABT     0x0040		/* ctrls arbitration mode bus master */
#define PCICR_BYTESWAP  0x0100		/* data byte endian swap little/big  */
#define PCICR_KEY       0xa5000000	/* Key to enable reg. write */

#define PCILSR0         ((volatile UINT32 *)(PCIC_PCI_CONFIG_BASE+0x104))
				/* Local space register 0 for PCI       */
#define PCILSR1         ((volatile UINT32 *)(PCIC_PCI_CONFIG_BASE+0x108))
				/* Local space register 1 for PCI       */
#define PCILAR0         ((volatile UINT32 *)(PCIC_PCI_CONFIG_BASE+0x10C))
				/* Local address register 0 for PCI     */
#define PCILAR1         ((volatile UINT32 *)(PCIC_PCI_CONFIG_BASE+0x110))
				/* Local address register 1 for PCI     */
#define PCIINT          ((volatile UINT32 *)(PCIC_PCI_CONFIG_BASE+0x114))
				/* PCI interrupt register               */
#define PCIINTM         ((volatile UINT32 *)(PCIC_PCI_CONFIG_BASE+0x118))
				/* PCI interrupt mask register          */
#define PCIAIR          ((volatile UINT32 *)(PCIC_PCI_CONFIG_BASE+0x11C))
				/* Error Address data register for PCI  */
#define PCICIR          ((volatile UINT32 *)(PCIC_PCI_CONFIG_BASE+0x120))
				/* Error Command data register for PCI  */

/* Reserved Register            0xfe040124                    */
/* Reserved Register            0xfe040128                    */
/* Reserved Register            0xfe04012C                    */

#define PCIAINT         ((volatile UINT32 *)(PCIC_PCI_CONFIG_BASE+0x130))
				/* PCI arbiter interrupt register       */
#define PCIAINTM        ((volatile UINT32 *)(PCIC_PCI_CONFIG_BASE+0x134))
				/* PCI arbiter interrupt mask register  */
#define PCIBMP          ((volatile UINT32 *)(PCIC_PCI_CONFIG_BASE+0x138))
				/* PCI arbiter bus master error information */

/* Reserved Register            0xfe04013C                  */

#define PCIPAR          ((volatile UINT32 *)(PCIC_PCI_CONFIG_BASE+0x1C0))
				/* PCI PIO Address register             */

/* Reserved Register            0xfe0401c4                    */
/* Reserved Register            0xfe0401c8                    */

#define PCIPINT         ((volatile UINT32 *)(PCIC_PCI_CONFIG_BASE+0x1CC))
				/* PCI power managment IT register      */
#define PCIPINTM        ((volatile UINT32 *)(PCIC_PCI_CONFIG_BASE+0x1D0))
				/* PCI power managment IT mask reg.     */

/* Reserved Register            0xfe0401d4                    */

#define PCIMBR0         ((volatile UINT32 *)(PCIC_PCI_CONFIG_BASE+0x1E0))
				/* PCI Memory bank register 0          */
#define PCIMBMR0        ((volatile UINT32 *)(PCIC_PCI_CONFIG_BASE+0x1E4))
				/* PCI Memory bank mask register 0     */
#define PCIMBR1         ((volatile UINT32 *)(PCIC_PCI_CONFIG_BASE+0x1E8))
				/* PCI Memory bank register 1          */
#define PCIMBMR1        ((volatile UINT32 *)(PCIC_PCI_CONFIG_BASE+0x1EC))
				/* PCI Memory bank mask register 1     */
#define PCIMBR2         ((volatile UINT32 *)(PCIC_PCI_CONFIG_BASE+0x1F0))
				/* PCI Memory bank register 2          */
#define PCIMBMR2        ((volatile UINT32 *)(PCIC_PCI_CONFIG_BASE+0x1F4))
				/* PCI Memory bank mask register 2     */
#define PCIIOBR         ((volatile UINT32 *)(PCIC_PCI_CONFIG_BASE+0x1F8))
				/* PCI I/O bank register               */
#define PCIIOBMR        ((volatile UINT32 *)(PCIC_PCI_CONFIG_BASE+0x1FC))
				/* PCI I/O bank mask register          */

#define PCICSCR0        ((volatile UINT32 *)(PCIC_PCI_CONFIG_BASE+0x210))
				/* PCI cache snoop control register 0  */
#define PCICSCR1        ((volatile UINT32 *)(PCIC_PCI_CONFIG_BASE+0x214))
				/* PCI cache snoop control register 1  */
#define PCICSAR0        ((volatile UINT32 *)(PCIC_PCI_CONFIG_BASE+0x218))
				/* PCI cache snoop address register 0  */
#define PCICSAR1        ((volatile UINT32 *)(PCIC_PCI_CONFIG_BASE+0x21C))
				/* PCI cache snoop address register 1  */

#define PCIPDR          ((volatile UINT32 *)(PCIC_PCI_CONFIG_BASE+0x220))
				/* PCI PIO data register               */

#ifndef PCIIO
#define PCIIO           (0xfe200000)
				/* PCI IO space base address,accesable 8, */
#endif  /* PCIIO */


/* DMAC */
/* From SH7785 Documentation 14_DMAC_0.4.00

Channel Name                                Abbrev.  R/W    P4 address  Size
0       DMA source address register 0       SAR0    R/W     0xFC808020  32 
0       DMA destination address register 0  DAR0    R/W     0xFC808024  32 
0       DMA transfer count register 0       TCR0    R/W     0xFC808028  32 
0       DMA Channel control register 0      CHCR0   R/W*1   0xFC80802C  32 
1       DMA source address register 1       SAR1    R/W     0xFC808030  32 
1       DMA destination address register 1  DAR1    R/W     0xFC808034  32 
1       DMA transfer count register 1       TCR1    R/W     0xFC808038  32 
1       DMA Channel control register 1      CHCR1   R/W*1   0xFC80803C  32 
2       DMA source address register 2       SAR2    R/W     0xFC808040  32 
2       DMA destination address register 2  DAR2    R/W     0xFC808044  32 
2       DMA transfer count register 2       TCR2    R/W     0xFC808048  32 
2       DMA Channel control register 2      CHCR2   R/W*1   0xFC80804C  32 
3       DMA source address register 3       SAR3    R/W     0xFC808050  32 
3       DMA destination address register 3  DAR3    R/W     0xFC808054  32 
3       DMA transfer count register 3       TCR3    R/W     0xFC808058  32 
3       DMA Channel control register 3      CHCR3   R/W*1   0xFC80805C  32 
0 to 5  DMA operation register 0            DMAOR0  R/W*2   0xFC808060  16 
4       DMA source address register 4       SAR4    R/W     0xFC808070  32 
4       DMA destination address register 4  DAR4    R/W     0xFC808074  32 
4       DMA transfer count register 4       TCR4    R/W     0xFC808078  32 
4       DMA Channel control register 4      CHCR4   R/W*1   0xFC80807C  32 
5       DMA source address register 5       SAR5    R/W     0xFC808080  32 
5       DMA destination address register 5  DAR5    R/W     0xFC808084  32 
5       DMA transfer count register 5       TCR5    R/W     0xFC808088  32 
5       DMA Channel control register 5      CHCR5   R/W*1   0xFC80808C  32 
0       DMA source address register B0      SARB0   R/W     0xFC808120  32 
0       DMA destination address register B0 DARB0   R/W     0xFC808124  32 
0       DMA transfer count register B0      TCRB0   R/W     0xFC808128  32 
1       DMA source address register B1      SARB1   R/W     0xFC808130  32 
1       DMA destination address register B1 DARB1   R/W     0xFC808134  32 
1       DMA transfer count register B1      TCRB1   R/W     0xFC808138  32 
2       DMA source address register B2      SARB2   R/W     0xFC808140  32 
2       DMA destination address register B2 DARB2   R/W     0xFC808144  32 
2       DMA transfer count register B2      TCRB2   R/W     0xFC808148  32 
3       DMA source address register B3      SARB3   R/W     0xFC808150  32 
3       DMA destination address register B3 DARB3   R/W     0xFC808154  32 
3       DMA transfer count register B3      TCRB3   R/W     0xFC808158  32 
0/1     DMA extension resource selector 0   DMARS0  R/W     0xFC809000  16 
2/3     DMA extension resource selector 1   DMARS1  R/W     0xFC809004  16 
4/5     DMA extension resource selector 2   DMARS2  R/W     0xFC809008  16 
                                                            *=*=*=*=*=
6       DMA source address register 6       SAR6    R/W     0xFCC08020  32 
6       DMA destination address register 6  DAR6    R/W     0xFCC08024  32 
6       DMA transfer count register 6       TCR6    R/W     0xFCC08028  32 
6       DMA Channel control register 6      CHCR6   R/W*1   0xFCC0802C  32 
7       DMA source address register 7       SAR7    R/W     0xFCC08030  32 
7       DMA destination address register 7  DAR7    R/W     0xFCC08034  32 
7       DMA transfer count register 7       TCR7    R/W     0xFCC08038  32 
7       DMA Channel control register 7      CHCR7   R/W*1   0xFCC0803C  32 
8       DMA source address register 8       SAR8    R/W     0xFCC08040  32 
8       DMA destination address register 8  DAR8    R/W     0xFCC08044  32 
8       DMA transfer count register 8       TCR8    R/W     0xFCC08048  32 
8       DMA Channel control register 8      CHCR8   R/W*1   0xFCC0804C  32 
9       DMA source address register 9       SAR9    R/W     0xFCC08050  32 
9       DMA destination address register 9  DAR9    R/W     0xFCC08054  32 
9       DMA transfer count register 9       TCR9    R/W     0xFCC08058  32 
9       DMA Channel control register 9      CHCR9   R/W*1   0xFCC0805C  32 
6 to 11 DMA operation register 1            DMAOR1  R/W*2   0xFCC08060  16 
10      DMA source address register 10      SAR10   R/W     0xFCC08070  32 
10      DMA destination address register 10 DAR10   R/W     0xFCC08074  32 
10      DMA transfer count register 10      TCR10   R/W     0xFCC08078  32 
10      DMA Channel control register 10     CHCR10  R/W*1   0xFCC0807C  32 
11      DMA source address register 11      SAR11   R/W     0xFCC08080  32 
11      DMA destination address register 11 DAR11   R/W     0xFCC08084  32 
11      DMA transfer count register 11      TCR11   R/W     0xFCC08088  32 
11      DMA Channel control register 11     CHCR11  R/W*1   0xFCC0808C  32 
6       DMA source address register B6      SARB6   R/W     0xFCC08120  32 
6       DMA destination address register B6 DARB6   R/W     0xFCC08124  32 
6       DMA transfer count register B6      TCRB6   R/W     0xFCC08128  32 
7       DMA source address register B7      SARB7   R/W     0xFCC08130  32 
7       DMA destination address register B7 DARB7   R/W     0xFCC08134  32 
7       DMA transfer count register B7      TCRB7   R/W     0xFCC08138  32 
8       DMA source address register B8      SARB8   R/W     0xFCC08140  32 
8       DMA destination address register B8 DARB8   R/W     0xFCC08144  32 
8       DMA transfer count register B8      TCRB8   R/W     0xFCC08148  32 
9       DMA source address register B9      SARB9   R/W     0xFCC08150  32 
9       DMA destination address register B9 DARB9   R/W     0xFCC08154  32 
9       DMA transfer count register B9      TCRB9   R/W     0xFCC08158  32 
6/7     DMA extension resource selector 3   DMARS3  R/W     0xFCC09000  16 
8/9     DMA extension resource selector 4   DMARS4  R/W     0xFCC09004  16 
10/11   DMA extension resource selector 5   DMARS5  R/W     0xFCC09008  16 

[Note]    
1. In order to clear a flag, only zero write after 1 read is possible for HE
   of CHCR, and TE bit. 
2. In order to clear a flag, only zero write after 1 read is possible for AE of
   DMAOR, and a NMIF bit. 
3. Please do not perform access of those other than specification access size. 
4. HE of CHCR and the synchronous clock of TE bit are Bck(s), and the
   synchronous clock of HE of CHCR and bits other than TE bit is Pck. 
*/

#define DMAC_BASE_ADDR1     0xFC808020  /* SH7780 - Channels 0 to 5  */
#define DMAC_BASE_ADDR2     0xFCC08020  /* SH7780 - Channels 6 to 11 */
#define DMAC_DMAOR_TYPE     UINT16

/* Use BSP function UINT32 sysGetDmacChanBaseAddr(int chan) + DMAC_OFFSET_xxx 
 * to access standart DMA registers */

#define DMAC_OFFSET_CHAN0   0x00        /* Channels 0  &  6 */
#define DMAC_OFFSET_CHAN1   0x10        /* Channels 1  &  7 */
#define DMAC_OFFSET_CHAN2   0x20        /* Channels 2  &  8 */
#define DMAC_OFFSET_CHAN3   0x30        /* Channels 3  &  9 */
#define DMAC_OFFSET_DMAOR   0x40        /* Channels 0 to 5 & 6 to 11 */
#define DMAC_OFFSET_CHAN4   0x50        /* Channels 4  & 10 */
#define DMAC_OFFSET_CHAN5   0x60        /* Channels 5  & 11 */

#define DMAC_OFFSET_SAR     0x00
#define DMAC_OFFSET_DAR     0x04
#define DMAC_OFFSET_TCR     0x08
#define DMAC_OFFSET_CHCR    0x0C

#define DMAC_ADDR_SAR(ch_)      (sysGetDmacChanBaseAddr(ch_) + DMAC_OFFSET_SAR)
#define DMAC_ADDR_DAR(ch_)      (sysGetDmacChanBaseAddr(ch_) + DMAC_OFFSET_DAR)
#define DMAC_ADDR_TCR(ch_)      (sysGetDmacChanBaseAddr(ch_) + DMAC_OFFSET_TCR)
#define DMAC_ADDR_CHCR(ch_)     (sysGetDmacChanBaseAddr(ch_) + DMAC_OFFSET_CHCR)

#define DMAC_SAR(ch_)           ((volatile UINT32 *)(DMAC_ADDR_SAR(ch_)))
#define DMAC_DAR(ch_)           ((volatile UINT32 *)(DMAC_ADDR_DAR(ch_))) 
#define DMAC_TCR(ch_)           ((volatile UINT32 *)(DMAC_ADDR_TCR(ch_))) 
#define DMAC_CHCR(ch_)          ((volatile UINT32 *)(DMAC_ADDR_CHCR(ch_)))

#define DMAC_DMARS0             ((volatile UINT16 *)0xfc809000)
					/* DMA extension resource selector 0 */
#define DMAC_DMARS1             ((volatile UINT16 *)0xfc809004)
					/* DMA extension resource selector 1 */
#define DMAC_DMARS2             ((volatile UINT16 *)0xfc809008)
					/* DMA extension resource selector 2 */
#define DMAC_DMARS3             ((volatile UINT16 *)0xfcc09000)
					/* DMA extension resource selector 3 */
#define DMAC_DMARS4             ((volatile UINT16 *)0xfcc09004)
					/* DMA extension resource selector 4 */
#define DMAC_DMARS5             ((volatile UINT16 *)0xfcc09008)
					/* DMA extension resource selector 5 */

#define DMAC_DMAOR0 \
	((volatile DMAC_DMAOR_TYPE *)(DMAC_BASE_ADDR1 + DMAC_OFFSET_DMAOR))
#define DMAC_DMAOR1 \
	((volatile DMAC_DMAOR_TYPE *)(DMAC_BASE_ADDR2 + DMAC_OFFSET_DMAOR))

#define DMAC_CHAN_NBR       12  /* SH7785 contains 12 DMA channels */

/* Bit definitions */

/* CHCRx */
#define DMAC_CHRCR_DST_MODE_ADRS_FIX    0x00000000
				/* 15-14: Destination address fixed         */
#define DMAC_CHRCR_DST_MODE_ADRS_INCR   0x00004000
				/* 15-14: Destination address increment     */
#define DMAC_CHRCR_DST_MODE_ADRS_DECR   0x00008000
				/* 15-14: Destination address decrement     */
#define DMAC_CHRCR_SRC_MODE_ADRS_FIX    0x00000000
				/* 13-12: Source address fixed              */
#define DMAC_CHRCR_SRC_MODE_ADRS_INCR   0x00001000
				/* 13-12: Source address increment          */
#define DMAC_CHRCR_SRC_MODE_ADRS_DECR   0x00002000
				/* 13-12: Source address decrement          */
#define DMAC_CHRCR_RES_AUTO_EXT_EXT     0x00000400
				/* 11-8 : Resource select auto request      */
#define DMAC_CHRCR_TM_BURST             0x00000080
				/* 7    : transmit mode burst               */
#define DMAC_CHRCR_TS_QUAD              0x00000000
				/* 5-4  : Transmit size -> long long word   */
#define DMAC_CHRCR_TS_BYTE              0x00000010
				/* 5-4  : Transmit size -> byte             */
#define DMAC_CHRCR_TS_SHORT             0x00000020
				/* 5-4  : Transmit size -> word             */
#define DMAC_CHRCR_TS_LONG              0x00000030
				/* 5-4  : Transmit size -> long word        */
#define DMAC_CHRCR_TS_32B               0x00000040
				/* 5-4  : Transmit size -> 32 Byte          */
#define DMAC_CHRCR_IE_ENABLE            0x00000004
				/* 2    : Interrupt enable                  */
#define DMAC_CHRCR_TE                   0x00000002
				/* 1    : Transfer End                      */
#define DMAC_CHRCR_ENABLE               0x00000001
				/* 0    : DMAC Enable                       */

/* DMAOR */
#define DMAC_DMAOR_AE			((DMAC_DMAOR_TYPE) 0x0004)
				/* 2: address error flag    */
#define DMAC_DMAOR_NMI			((DMAC_DMAOR_TYPE) 0x0002)
				/* 1: NMI flag              */
#define DMAC_DMAOR_MASTER_ENABLE	((DMAC_DMAOR_TYPE) 0x0001)
				/* 0: DMAC master enable    */

/* NAND Flash Memory Controller (FLCTL) registers addresses */

#define FLCTL_FLCMNCR       ((volatile UINT32 *)0xffe90000)
				/* Common control register              */
#define FLCTL_FLCMDCR       ((volatile UINT32 *)0xffe90004)
				/* Command control register             */
#define FLCTL_FLCMCDR       ((volatile UINT32 *)0xffe90008)
				/* Command code register                */
#define FLCTL_FLADR         ((volatile UINT32 *)0xffe9000c)
				/* Address register                     */
#define FLCTL_FLDATAR       ((volatile UINT32 *)0xffe90010)
				/* Data register                        */
#define FLCTL_FLDTCNTR      ((volatile UINT32 *)0xffe90014)
				/* Data counter register                */
#define FLCTL_FLINTDMACR    ((volatile UINT32 *)0xffe90018)
				/* Interrupt DMA control register       */
#define FLCTL_FLBSYTMR      ((volatile UINT32 *)0xffe9001c)
				/* Ready busy timeout setting register  */
#define FLCTL_FLBSYCNT      ((volatile UINT32 *)0xffe90020)
				/* Ready busy timeout counter (R ONLY!) */
#define FLCTL_FLDTFIFO      ((volatile UINT32 *)0xffe90024)
				/* Data FIFO register                   */
#define FLCTL_FLECFIFO      ((volatile UINT32 *)0xffe90028)
				/* Control code FIFO register           */
#define FLCTL_FLTRCR        ((volatile UINT8  *)0xffe9002c)
				/* Transfer control register            */
#define FLCTL_FLADR2        ((volatile UINT32 *)0xffe9003c)
				/* Address register 2                   */

/* Exception Mask : Enables/Disable future unsupported SH features. */

#define SH4A_EXPMASK        ((volatile UINT32 *)0xFF2F0004)
				/* For SH4A only - From PVR REV30       */

#define EXPMASK_RTEDS       0x01
				/* Allows instruction in RTE slot */
#define EXPMASK_BRDSSLP     0x02
				/* Allows SLEEP instruction in delay slot */
#define EXPMASK_MMCAW       0x10
				/* Allows mem mapped associative cache write */

/*
 * Specific vector numbers for the sh7780.
 *
 * Most of vector numbers is defined in ivSh.h.
 * Some vectors need to be undefined and redefined
 * for the 7780.
 */

/*---------------------------------------------------------------------------*/
/*      exception/interrupt name     (code >> 5)   No. vector  entry point   */
/*---------------------------------------------------------------------------*/

/* TMU */
#undef  INUM_TMU0_UNDERFLOW
#undef  INUM_TMU1_UNDERFLOW
#undef  INUM_TMU2_UNDERFLOW
#undef  INUM_TMU2_INPUT_CAPTURE
#undef  INUM_TMU3_UNDERFLOW
#undef  INUM_TMU4_UNDERFLOW
#undef  INUM_TMU5_UNDERFLOW
#undef  INUM_DMAC_DMTE0
#undef  INUM_DMAC_DMTE1
#undef  INUM_DMAC_DMTE2
#undef  INUM_DMAC_DMTE3
#undef  INUM_DMAC_DMTER

#define INUM_TMU0_UNDERFLOW         (0x580 >> 5)
#define INUM_TMU1_UNDERFLOW         (0x5a0 >> 5)
#define INUM_TMU2_UNDERFLOW         (0x5c0 >> 5)
#define INUM_TMU2_INPUT_CAPTURE     (0x5e0 >> 5)

#define INUM_TMU3_UNDERFLOW         (0xE00 >> 5)
#define INUM_TMU4_UNDERFLOW         (0xE20 >> 5)
#define INUM_TMU5_UNDERFLOW         (0xE40 >> 5)

/* DMAC(0) */ 
#define INUM_DMAC_DMTE0             (0x620 >> 5)
#define INUM_DMAC_DMTE1             (0x640 >> 5)
#define INUM_DMAC_DMTE2             (0x660 >> 5)
#define INUM_DMAC_DMTE3             (0x680 >> 5)
#define INUM_DMAC_DMTE4             (0x6a0 >> 5)
#define INUM_DMAC_DMTE5             (0x6c0 >> 5)

/* DMAC(1) */
#define INUM_DMAC_DMTE6             (0x880 >> 5)
#define INUM_DMAC_DMTE7             (0x8a0 >> 5)
#define INUM_DMAC_DMTE8             (0x8c0 >> 5)
#define INUM_DMAC_DMTE9             (0x8e0 >> 5)
#define INUM_DMAC_DMTE10            (0x900 >> 5)
#define INUM_DMAC_DMTE11            (0x920 >> 5)

/* DMAError */
#define INUM_DMAC_DMA0              (0x6e0 >> 5)
#define INUM_DMAC_DMA1              (0x940 >> 5)

/* SCIF */
#undef  INUM_SCIF2_ERROR
#undef  INUM_SCIF2_RX
#undef  INUM_SCIF2_BREAK
#undef  INUM_SCIF2_TX

#define INUM_SCIF0_ERROR            (0x700 >> 5) /*  56  0x0e0  */
#define INUM_SCIF0_RX               (0x720 >> 5) /*  57  0x0e4  */
#define INUM_SCIF0_BREAK            (0x740 >> 5) /*  58  0x0e8  */
#define INUM_SCIF0_TX               (0x760 >> 5) /*  59  0x0ec  */
#define INUM_SCIF1_ERROR            (0x780 >> 5) 
#define INUM_SCIF1_RX               (0x7a0 >> 5) 
#define INUM_SCIF1_BREAK            (0x7c0 >> 5) 
#define INUM_SCIF1_TX               (0x7e0 >> 5) 
#define INUM_SCIF2_ALL              (0x980 >> 5) 
#define INUM_SCIF3_ALL              (0x9a0 >> 5) 
#define INUM_SCIF4_ALL              (0x9c0 >> 5) 
#define INUM_SCIF5_ALL              (0x9e0 >> 5) 

/* PCI */
#undef  INUM_PCI_SERR               
#undef  INUM_PCI_DMA3               
#undef  INUM_PCI_DMA2               
#undef  INUM_PCI_DMA1               
#undef  INUM_PCI_DMA0               
#undef  INUM_PCI_PWONREQ            
#undef  INUM_PCI_PWDWREQ            
#undef  INUM_PCI_ERR                

#define INUM_PCI_SERR               (0xa00 >> 5) /* 80  0x140  */
#define INUM_PCI_INTA               (0xa20 >> 5) /* 81  0x144  */
#define INUM_PCI_INTB               (0xa40 >> 5) /* 82  0x148  */
#define INUM_PCI_INTC               (0xa60 >> 5) /* 83  0x14c  */
#define INUM_PCI_INTD               (0xa80 >> 5) /* 84  0x150  */
#define INUM_PCI_ERR                (0xaa0 >> 5) /* 85  0x154  */
#define INUM_PCI_PWD123             (0xac0 >> 5) /* 86  0x158  */
#define INUM_PCI_PWD0               (0xae0 >> 5) /* 87  0x15C  */


#define IV_SCIF0_ERI                INUM_TO_IVEC (INUM_SCIF0_ERROR)
#define IV_SCIF0_RXI                INUM_TO_IVEC (INUM_SCIF0_RX)
#define IV_SCIF0_TXI                INUM_TO_IVEC (INUM_SCIF0_TX)
#define IV_SCIF0_BRK                INUM_TO_IVEC (INUM_SCIF0_BREAK)

#define IV_SCIF1_ERI                INUM_TO_IVEC (INUM_SCIF1_ERROR)
#define IV_SCIF1_RXI                INUM_TO_IVEC (INUM_SCIF1_RX)
#define IV_SCIF1_TXI                INUM_TO_IVEC (INUM_SCIF1_TX)
#define IV_SCIF1_BRK                INUM_TO_IVEC (INUM_SCIF1_BREAK)

/* WatchDog Timer */
#undef INUM_WDT_INTERVAL_TIMER  
#define INUM_WDT_INTERVAL_TIMER     (0x560 >> 5) /* 43  0x0ac  (vbr + 0x600) */

/* User Debug Interface */
#define INUM_H_UDI                  (0x600 >> 5)

/* Serial I/O with FIFO */
#define INUM_SIOFI                  (0xce0 >> 5)

/* Multimedia Card Interface */
#define INUM_MMCIF_FSTAT            (0xd00 >> 5)
#define INUM_MMCIF_TRAN             (0xd20 >> 5)
#define INUM_MMCIF_ERR              (0xd40 >> 5)
#define INUM_MMCIF_FRDY             (0xd60 >> 5)

/* Display Unit */
#define INUM_DU                     (0xd80 >> 5)

/* Serial Peripheral Interface */
#define INUM_HSPII                  (0x960 >> 5)

/* Graphic Display (GDTA) Interrupt */
#define INUM_GDTA_GACLI             (0xda0 >> 5)
#define INUM_GDTA_GAMCI             (0xdc0 >> 5)
#define INUM_GDTA_GAERI             (0xde0 >> 5)

/* Sound Serial Interface */  
#define INUM_SSII_CH0               (0xe80 >> 5)
#define INUM_SSII_CH1               (0xea0 >> 5)

/* Audio Codec Interface */
#define INUM_HACI_CH0               (0xec0 >> 5)
#define INUM_HACI_CH1               (0xee0 >> 5)

/* NAND FLash Memory Controller (FLCTL) */
#define INUM_FLCTL_FLSTE            (0xf00 >> 5)
#define INUM_FLCTL_FLTEND           (0xf20 >> 5)
#define INUM_FLCTL_FLTRQ0           (0xf40 >> 5)
#define INUM_FLCTL_FLTRQ1           (0xf60 >> 5)

/* General Purpose I/O */
#define INUM_GPIO_0                 (0xf80 >> 5)
#define INUM_GPIO_1                 (0xfa0 >> 5)
#define INUM_GPIO_2                 (0xfc0 >> 5)
#define INUM_GPIO_3                 (0xfe0 >> 5)

#ifdef __cplusplus
}
#endif

#endif /* __INCsh7785h */
