/***********************************************************************************/
/*                        M-Systems Confidential                                   */
/*           Copyright (C) M-Systems Flash Disk Pioneers Ltd. 1995-2006            */
/*                         All Rights Reserved                                     */
/***********************************************************************************/
/*                            NOTICE OF M-SYSTEMS OEM                              */
/*                           SOFTWARE LICENSE AGREEMENT                            */
/*                                                                                 */
/*      THE USE OF THIS SOFTWARE IS GOVERNED BY A SEPARATE LICENSE                 */
/*      AGREEMENT BETWEEN THE OEM AND M-SYSTEMS. REFER TO THAT AGREEMENT           */
/*      FOR THE SPECIFIC TERMS AND CONDITIONS OF USE,                              */
/*      OR CONTACT M-SYSTEMS FOR LICENSE ASSISTANCE:                               */
/*      E-MAIL = info@m-sys.com                                                    */
/***********************************************************************************/

/*
 * $Log:   V:/PVCSDB/DiskOnChip/archives/Testing/TrueFFS 6.3/Drop 2.5/3/bddoc/src/bddefs.h-arc  $
 * 
 *    Rev 1.5   Feb 14 2006 10:56:04   DoronC
 * Update copyright note.
 * 
 *    Rev 1.4   Dec 27 2005 16:22:46   DoronC
 * Add SetTl and lockIO in order to be used by formatCall in BD package.
 * 
 *    Rev 1.3   Dec 05 2005 11:24:02   DoronC
 * Fix compilation errors & warnings under GNU compiler.
 * 
 *    Rev 1.2   Nov 13 2005 10:02:02   DoronC
 * Change header files to have external headers for binary release.
 * 
 *    Rev 1.1   Apr 20 2005 21:14:38   oris
 * Changed include directive (stdcomp.h -> flstdcmp.h)
 * 
 *    Rev 1.0   Mar 27 2005 09:27:16   oris
 * Initial revision.
 * 
 *    Rev 1.1   Jan 05 2005 21:03:06   eugen
 * Commentaries, headers
 * 
 *    Rev 1.0   Dec 09 2004 18:24:24   oris
 * Initial revision.
 * 
 *    Rev 1.4   Oct 17 2004 15:00:14   OriS
 * - Removed warnings
 * - New code for single sector read cache
 * 
 *    Rev 1.3.2.2   Aug 31 2004 11:20:36   eugen
 * Commented out "typedef struct Volume Volume;"
 * since it is problematic on some compilers
 * 
 *    Rev 1.3   Jan 14 2004 22:28:52   oris
 * Added new file flag - FL_FILE_MUST_CREATE - 2 - Create file even if found
 * 
 *    Rev 1.2   Sep 30 2003 17:55:56   oris
 * Added globalAttFlag field to Volume structure.
 * Added FL prefix to file flag definitions
 * 
 *    Rev 1.1   May 18 2003 13:28:32   OriS
 * Removed prototype of dismountVolume(Volume vol), initDone and setBusy. They are now used only by blockdev.c
 * 
 *    Rev 1.0   Apr 09 2003 12:15:58   OriS
 * Initial revision.
 * 
 */

#ifndef BDDEFS_H
#define BDDEFS_H

#include "flflash.h"
#include "fltl.h"
#include "flsocket.h"
#include "flbuffer.h"
#include "docsys.h"


typedef struct {
  FLSByte       flags;                  /* See description in flreq.h */

  FLDword	sectorsInVolume;
  unsigned	mountCount;		/* can be opened/closed several times */

  FLMutex*      volExecInProgress;
  FLFlash FAR2* flash;                  /* flash structure for low level operations */
  TL            tl;                     /* Translation layer methods */
  FLSocket      *socket;                /* Pointer to socket */

  FLBoolean     globalAttFlag;        /* indication of global attention flag */
} Volume;

#define VOLUME_NOT_OCCUPIED	0
#define VOLUME_LOW_LVL_MOUNTED	1  /* Volume is mounted for low level operations */
#define VOLUME_MOUNTED		2  /* Volume is mounted                          */
#define VOLUME_12BIT_FAT	4  /* Volume uses 12-bit FAT                     */
#define VOLUME_ABS_MOUNTED	8  /* Volume is mounted for abs calls            */
#define VOLUME_OCCUPIED		32 /* Volume record in use                       */
#define VOLUME_MOUNTED_NO_MBR	64
#define VOLUME_MOUNTED_ON_RANGE	128


#define execInProgress (pVol->volExecInProgress)

extern Volume   vols[FL_VOLUMES];

FLStatus lockForIO(FLByte socket, FLByte partition, FLBoolean onOff);
void setTL(FLByte socket, FLByte partition, TL * fsVol);

#endif
