/***********************************************************************************/
/*                        M-Systems Confidential                                   */
/*           Copyright (C) M-Systems Flash Disk Pioneers Ltd. 1995-2006            */
/*                         All Rights Reserved                                     */
/***********************************************************************************/
/*                            NOTICE OF M-SYSTEMS OEM                              */
/*                           SOFTWARE LICENSE AGREEMENT                            */
/*                                                                                 */
/*      THE USE OF THIS SOFTWARE IS GOVERNED BY A SEPARATE LICENSE                 */
/*      AGREEMENT BETWEEN THE OEM AND M-SYSTEMS. REFER TO THAT AGREEMENT           */
/*      FOR THE SPECIFIC TERMS AND CONDITIONS OF USE,                              */
/*      OR CONTACT M-SYSTEMS FOR LICENSE ASSISTANCE:                               */
/*      E-MAIL = info@m-sys.com                                                    */
/***********************************************************************************/

/*
 * $Log:   V:/PVCSDB/DiskOnChip/archives/Testing/TrueFFS 6.3/Drop 2.5/3/bddoc/src/docbdk.h-arc  $
 * 
 *    Rev 1.5   Mar 02 2006 08:18:30   DoronC
 * Change default page size to support H1 devices.
 * 
 *    Rev 1.4   Feb 14 2006 10:59:14   DoronC
 * Update cpyright note.
 * 
 *    Rev 1.3   Nov 13 2005 10:02:02   DoronC
 * Change header files to have external headers for binary release.
 * 
 *    Rev 1.2   Apr 19 2005 12:15:48   oris
 * Compilation error
 * 
 *    Rev 1.1   Apr 19 2005 02:47:02   oris
 * - Replaced BDK_PAGE_SIZE with BDK_SECTORS_PER_PAGE,
 *   and therefore moved it from docbdk.h
 * - Added hidden customization of the buffer size used for recovery
 *   from flash write failures (BDK_SECTORS_PER_WRITE).
 * 
 *    Rev 1.0   Mar 27 2005 09:27:16   oris
 * Initial revision.
 * 
 *    Rev 1.2   Mar 27 2005 09:16:30   oris
 * - Dynamic bad blocks support.
 * 
 *    Rev 1.1   Jan 05 2005 21:03:08   eugen
 * Commentaries, headers
 * 
 *    Rev 1.0   Dec 09 2004 18:24:26   oris
 * Initial revision.
 * 
 *    Rev 1.2   Oct 17 2004 15:01:54   OriS
 * - Moved customization of the amount of binary partitions to 
 *   flcustom.h (MAX_BINARY_PARTITIONS_PER_DRIVE)
 * 
 *    Rev 1.1   Apr 13 2003 20:29:22   OriS
 * Fixed compilation error for BDK
 * 
 *    Rev 1.0   Apr 09 2003 12:15:58   OriS
 * Initial revision.
 * 
 */

/************************************************************************/
/* Caution: The BDK_ACCESS compilation flag is for M-SYSTEMS internal   */
/*          use ONLY. This flag is used by M-SYSTEMS drivers and        */
/*          therfore it is SHOULD NOT be used by this package           */
/************************************************************************/

/*****************************************************************************
* File Header                                                                *
* -----------                                                                *
* Name : docbdk.h                                                            *
*                                                                            *
* Description : This file contains the binary partition defintions , data    *
*               structures and function prototypes.                          *
*                                                                            *
* Note : The file exports 2 interfaces each under its own compilation flag:  *
*                                                                            *
*        BDK package - Standalone package that exports routines for binary   *
*                      partitions handling(MTD_STANDALONE compilation flag). *
*        OSAK module - Separated module of the OSAK package that exports a   *
*                      common entry point to the same routines. (BDK_ACCESS  *
*                      compilation flag).                                    *
*                                                                            *
* Warning : Do not use this file with the BDK_ACCESS compilation flag unless *
*           you own the full OSAK package.                                   *
*****************************************************************************/

#ifndef DOCBDK_H
#define DOCBDK_H
/*---------------------------------------------------------------------*/
/* Include the proper header files.                                    */
/*---------------------------------------------------------------------*/

#include "flcommon.h"

#ifndef MTD_STANDALONE
#ifdef BDK_ACCESS
#include "flstruct.h"
#endif /* BDK_ACCESS */
#endif /* MTD_STANDALONE */

/* Use the define bellow to set the size of the page buffer used by the BDK */
#ifndef BDK_SECTORS_PER_PAGE
#define BDK_SECTORS_PER_PAGE		  (8L /* 2KB PAGE */)
#endif /* BDK_SECTORS_PER_PAGE */

#ifndef BDK_SECTORS_PER_WRITE
#define BDK_SECTORS_PER_WRITE        (4L/*Planes*/*BDK_SECTORS_PER_PAGE/*Sectors per page*/*2/*MLC*/)
#endif /* BDK_SECTORS_PER_WRITE */


/* BDK specific flag area */
#define ERASE_BEFORE_WRITE              8
#define BDK_SPARE_BLOCKS_SIGN           "S4BB" /* Must be the same as BINARY_SPARE_BLOCKS_SIGN (FLFORMAT.H) */


/*----------------------------------------------------------------------*/
/*                b d k F i n d D i s k O n C h i p                     */
/*                                                                      */
/* Find and identify DiskOnChip device if installed in the system. The	*/
/*search address are set by the flregisterDOCXXXSOC call in flcustom.c  */
/*file.												                    */
/*                                                                      */
/*                                                                      */
/* Parameters:                                                          */
/*        docAddress        : Pointer to return value of DiskOnChip		*/
/*							 address (if found).		                */
/*        docSize           : Pointer to return value of DiskOnChip		*/
/*							 address window size (if found).            */
/*                                                                      */
/* Returns:                                                             */
/*        FLStatus        : 0 on success (DiskOnChip found)				*/
/*							otherwise for failure			            */
/*----------------------------------------------------------------------*/
FLStatus bdkFindDiskOnChip       (CardAddress FAR2 *docAddress,
								  FLDword FAR2 *docSize );
/*----------------------------------------------------------------------*/
/*						b d k I n i t				                    */
/*                                                                      */
/* Initialize the BDK SW package. This function is normaly called		*/
/*automatically by the first call to the BDK package functions.			*/
/*                                                                      */
/* Parameters:                                                          */
/*        None.                                                         */
/*                                                                      */
/* Returns:                                                             */
/*        FLStatus        : 0 on success otherwise for failure          */
/*----------------------------------------------------------------------*/
void     bdkInit( void );

#ifdef FL_EXIT 
/*----------------------------------------------------------------------*/
/*						b d k E X i t				                    */
/*                                                                      */
/* Reset the BDK SW package. This function is normaly called as the last*/
/*reference to the BDK package, providing clean exit and clearing all	*/
/*allocated memory.														*/
/*                                                                      */
/* Parameters:                                                          */
/*        None.                                                         */
/*                                                                      */
/* Returns:                                                             */
/*        FLStatus        : 0 on success otherwise for failure          */
/*----------------------------------------------------------------------*/
void     bdkExit                 (void);
#endif /* FL_EXIT */

/*----------------------------------------------------------------------*/
/*					b d k S e t B o o t P a r t i t i o n N o	        */
/*                                                                      */
/* Set the current BDK partition in use. The default partition is		*/
/*partition 0. For operation on other partitions, this routine should be*/ 
/*called prior to actual command. The partition set is valid until the  */
/*next time this function is called.									*/
/*                                                                      */
/* Parameters:                                                          */
/*        partitionNo	    : partition number to be access next.       */
/*                                                                      */
/* Returns:                                                             */
/*        FLStatus        : 0 on success otherwise for failure          */
/*----------------------------------------------------------------------*/
FLStatus bdkSetBootPartitionNo   (FLByte partitionNo);

/*----------------------------------------------------------------------*/
/*				b d k G e t B o o t P a r t i t i o n I n f o	        */
/*                                                                      */
/* Get information regarding the actual size and used size of a binary  */
/*(sub) partition.														*/
/*                                                                      */
/* Parameters:                                                          */
/*        startUnit			: First block containing data (usually 0)   */
/*        partitionSize		: Pointer to output value of full			*/
/*							 sub partition size.				        */
/*        realPartitionSize	: Pointer to return value of used size.     */
/*        unitSize			: Pointer to return value of unit size.     */
/*        signature			: Signature of retrieved sub partition.     */
/*                                                                      */
/* Returns:                                                             */
/*        FLStatus        : 0 on success otherwise for failure          */
/*----------------------------------------------------------------------*/
FLStatus bdkGetBootPartitionInfo (FLWord startUnit,
                 FLDword FAR2 *partitionSize,
                 FLDword FAR2 *realPartitionSize,
                 FLDword FAR2 *unitSize,
                 FLByte FAR2 *signature);

/*----------------------------------------------------------------------*/
/*					b d k C o p y B o o t A r e a				        */
/*                                                                      */
/* Copy data from binary partition to RAM								*/
/*                                                                      */
/* Parameters:                                                          */
/*        startAddressPtr	: Pointer to start RAM address			    */
/*        wStartUnit		: No' of first block in partition(usualy 0).*/
/*        dwAreaLen			: Length of read area.  			        */
/*        bCheckSumPtr		: Pointer to output chksum calculation.     */
/*        signPtr			: Signature of retrieved sub partition.     */
/*                                                                      */
/* Returns:                                                             */
/*        FLStatus        : 0 on success otherwise for failure          */
/*----------------------------------------------------------------------*/
FLStatus bdkCopyBootArea         (FLByte FAR1 *startAddressPtr,
								  FLWord wStartUnit,
								  FLDword dwAreaLen,
								  FLByte FAR2 *bCheckSumPtr,
								  FLByte FAR2 *signPtr);

/*----------------------------------------------------------------------*/
/*					b d k C o p y B o o t A r e a I n i t		        */
/*                                                                      */
/* Initialize read process from BDK area. This routine must be called	*/
/*prior to any series of bdkCopyBootAreaBlock calls. The areaLen set by */
/*this routine must be total len of all calls to following				*/
/*bdkCopyBootAreaBlock calls.											*/
/*                                                                      */
/* Parameters:                                                          */
/*        startUnit		    : first unit to read (must contain data).   */
/*        areaLen			: length of bdk are to be read.		        */
/*        signature			: Signature of retrieved sub partition.     */
/*                                                                      */
/* Returns:                                                             */
/*        FLStatus        : 0 on success otherwise for failure          */
/*----------------------------------------------------------------------*/
FLStatus bdkCopyBootAreaInit     (FLWord startUnit,
                 FLDword areaLen,
                 FLByte FAR2 *signature);

/*----------------------------------------------------------------------*/
/*					b d k C o p y B o o t A r e a B l o c k		        */
/*                                                                      */
/* read the next chunk of data from BDK area. This routine call must	*/
/*follow a call to bdkCopyBootAreaInit and cannot be used without		*/
/*proper init call.														*/
/*                                                                      */
/* Parameters:                                                          */
/*        buf			    : Buffer where to read the data.	        */
/*        bufferLen			: Length of current read buffer.	        */
/*        checkSum		    : Pointer for output ChkSum calculation of	*/
/*							 read data.							        */
/*                                                                      */
/* Returns:                                                             */
/*        FLStatus        : 0 on success otherwise for failure          */
/*----------------------------------------------------------------------*/
FLStatus bdkCopyBootAreaBlock    (FLByte FAR1 *buf ,
                 FLDword bufferLen,
                 FLByte FAR2 *checkSum);

/*----------------------------------------------------------------------*/
/*					b d k U p d a t e B o o t A r e a I n i t	        */
/*                                                                      */
/* XXXX	*/
/* XXXX	*/
/* XXXX	*/
/*                                                                      */
/* Parameters:                                                          */
/*        xx			    : xxxx								        */
/*                                                                      */
/* Returns:                                                             */
/*        FLStatus        : 0 on success otherwise for failure          */
/*----------------------------------------------------------------------*/
FLStatus bdkUpdateBootAreaInit   (FLWord  startUnit,
                 FLDword  areaLen,
                 FLByte updateFlag,
                 FLByte FAR2 *signature );

/*----------------------------------------------------------------------*/
/*					x	        */
/*                                                                      */
/* XXXX	*/
/* XXXX	*/
/* XXXX	*/
/*                                                                      */
/* Parameters:                                                          */
/*        xx			    : xxxx								        */
/*                                                                      */
/* Returns:                                                             */
/*        FLStatus        : 0 on success otherwise for failure          */
/*----------------------------------------------------------------------*/
FLStatus bdkUpdateBootAreaBlock  (FLByte FAR1 *buf ,
                 FLDword bufferLen );

/*----------------------------------------------------------------------*/
/*					x	        */
/*                                                                      */
/* XXXX	*/
/* XXXX	*/
/* XXXX	*/
/*                                                                      */
/* Parameters:                                                          */
/*        xx			    : xxxx								        */
/*                                                                      */
/* Returns:                                                             */
/*        FLStatus        : 0 on success otherwise for failure          */
/*----------------------------------------------------------------------*/
FLStatus bdkGetProtectionType    (FLWord * protectionType);

/*----------------------------------------------------------------------*/
/*					x	        */
/*                                                                      */
/* XXXX	*/
/* XXXX	*/
/* XXXX	*/
/*                                                                      */
/* Parameters:                                                          */
/*        xx			    : xxxx								        */
/*                                                                      */
/* Returns:                                                             */
/*        FLStatus        : 0 on success otherwise for failure          */
/*----------------------------------------------------------------------*/
FLStatus bdkInsertKey            (FLByte FAR1* key);

/*----------------------------------------------------------------------*/
/*					x	        */
/*                                                                      */
/* XXXX	*/
/* XXXX	*/
/* XXXX	*/
/*                                                                      */
/* Parameters:                                                          */
/*        xx			    : xxxx								        */
/*                                                                      */
/* Returns:                                                             */
/*        FLStatus        : 0 on success otherwise for failure          */
/*----------------------------------------------------------------------*/
FLStatus bdkRemoveKey            (void);

/*----------------------------------------------------------------------*/
/*					x	        */
/*                                                                      */
/* XXXX	*/
/* XXXX	*/
/* XXXX	*/
/*                                                                      */
/* Parameters:                                                          */
/*        xx			    : xxxx								        */
/*                                                                      */
/* Returns:                                                             */
/*        FLStatus        : 0 on success otherwise for failure          */
/*----------------------------------------------------------------------*/
FLStatus bdkStickyLock           (void);

/*----------------------------------------------------------------------*/
/*					x	        */
/*                                                                      */
/* XXXX	*/
/* XXXX	*/
/* XXXX	*/
/*                                                                      */
/* Parameters:                                                          */
/*        xx			    : xxxx								        */
/*                                                                      */
/* Returns:                                                             */
/*        FLStatus        : 0 on success otherwise for failure          */
/*----------------------------------------------------------------------*/
FLStatus bdkLockEnable           (FLByte enable);

/*----------------------------------------------------------------------*/
/*					x	        */
/*                                                                      */
/* XXXX	*/
/* XXXX	*/
/* XXXX	*/
/*                                                                      */
/* Parameters:                                                          */
/*        xx			    : xxxx								        */
/*                                                                      */
/* Returns:                                                             */
/*        FLStatus        : 0 on success otherwise for failure          */
/*----------------------------------------------------------------------*/
FLStatus bdkChangeKey            (FLByte FAR1* key);
/*----------------------------------------------------------------------*/
/*					x	        */
/*                                                                      */
/* XXXX	*/
/* XXXX	*/
/* XXXX	*/
/*                                                                      */
/* Parameters:                                                          */
/*        xx			    : xxxx								        */
/*                                                                      */
/* Returns:                                                             */
/*        FLStatus        : 0 on success otherwise for failure          */
/*----------------------------------------------------------------------*/
FLStatus bdkSetProtectionType    (FLWord newType);

/*----------------------------------------------------------------------*/
/*					x	        */
/*                                                                      */
/* XXXX	*/
/* XXXX	*/
/* XXXX	*/
/*                                                                      */
/* Parameters:                                                          */
/*        xx			    : xxxx								        */
/*                                                                      */
/* Returns:                                                             */
/*        FLStatus        : 0 on success otherwise for failure          */
/*----------------------------------------------------------------------*/
FLStatus bdkCreateBootArea       (FLWord noOfBlocks,
                 FLByte FAR2 * oldSign,
                 FLByte FAR2 * newSign);

/*----------------------------------------------------------------------*/
/*					x	        */
/*                                                                      */
/* XXXX	*/
/* XXXX	*/
/* XXXX	*/
/*                                                                      */
/* Parameters:                                                          */
/*        xx			    : xxxx								        */
/*                                                                      */
/* Returns:                                                             */
/*        FLStatus        : 0 on success otherwise for failure          */
/*----------------------------------------------------------------------*/
FLStatus bdkEraseBootArea        (FLWord startUnit,
                 FLWord noOfBlocks,
                 FLByte FAR2 * signature);

/*----------------------------------------------------------------------*/
/*					x	        */
/*                                                                      */
/* XXXX	*/
/* XXXX	*/
/* XXXX	*/
/*                                                                      */
/* Parameters:                                                          */
/*        xx			    : xxxx								        */
/*                                                                      */
/* Returns:                                                             */
/*        FLStatus        : 0 on success otherwise for failure          */
/*----------------------------------------------------------------------*/
FLStatus bdkGetUniqueID(FLByte FAR1* buf);

/*----------------------------------------------------------------------*/
/*					x	        */
/*                                                                      */
/* XXXX	*/
/* XXXX	*/
/* XXXX	*/
/*                                                                      */
/* Parameters:                                                          */
/*        xx			    : xxxx								        */
/*                                                                      */
/* Returns:                                                             */
/*        FLStatus        : 0 on success otherwise for failure          */
/*----------------------------------------------------------------------*/
FLStatus bdkReadOtp(FLWord offset,FLByte FAR1 * buffer,FLWord length);

/*----------------------------------------------------------------------*/
/*					x	        */
/*                                                                      */
/* XXXX	*/
/* XXXX	*/
/* XXXX	*/
/*                                                                      */
/* Parameters:                                                          */
/*        xx			    : xxxx								        */
/*                                                                      */
/* Returns:                                                             */
/*        FLStatus        : 0 on success otherwise for failure          */
/*----------------------------------------------------------------------*/
FLStatus bdkWriteAndLockOtp(const FLByte FAR1 * buffer,FLWord length);

/*----------------------------------------------------------------------*/
/*					x	        */
/*                                                                      */
/* XXXX	*/
/* XXXX	*/
/* XXXX	*/
/*                                                                      */
/* Parameters:                                                          */
/*        xx			    : xxxx								        */
/*                                                                      */
/* Returns:                                                             */
/*        FLStatus        : 0 on success otherwise for failure          */
/*----------------------------------------------------------------------*/
FLStatus bdkGetOtpSize(FLDword FAR2* sectionSize, FLDword FAR2* usedSize,
               FLWord FAR2* locked);

/*----------------------------------------------------------------------*/
/*					x	        */
/*                                                                      */
/* XXXX	*/
/* XXXX	*/
/* XXXX	*/
/*                                                                      */
/* Parameters:                                                          */
/*        xx			    : xxxx								        */
/*                                                                      */
/* Returns:                                                             */
/*        FLStatus        : 0 on success otherwise for failure          */
/*----------------------------------------------------------------------*/
FLStatus bdkWriteIPL             (FLByte FAR1 * buf , FLDword bufLen , FLDword offset,FLDword flags);

/*----------------------------------------------------------------------*/
/*						        */
/*                                                                      */
/* XXXX	*/
/* XXXX	*/
/* XXXX	*/
/*                                                                      */
/* Parameters:                                                          */
/*        xx			    : xxxx								        */
/*                                                                      */
/* Returns:                                                             */
/*        FLStatus        : 0 on success otherwise for failure          */
/*----------------------------------------------------------------------*/
FLStatus bdkCopyBootAreaFile     ( FLSByte FAR2 *fname,
                   FLWord startUnit,
                   FLDword areaLen,
                   FLByte FAR2 *checkSum,
                   FLByte FAR2 *signature );

/*----------------------------------------------------------------------*/
/*						        */
/*                                                                      */
/* XXXX	*/
/* XXXX	*/
/* XXXX	*/
/*                                                                      */
/* Parameters:                                                          */
/*        xx			    : xxxx								        */
/*                                                                      */
/* Returns:                                                             */
/*        FLStatus        : 0 on success otherwise for failure          */
/*----------------------------------------------------------------------*/
FLStatus bdkUpdateBootAreaFile(FLSByte FAR2 *fname, FLWord startUnit,
                   FLDword areaLen, FLByte FAR2 *signature);

#endif /* DOCBDK_H */




