/***********************************************************************************/
/*                        M-Systems Confidential                                   */
/*           Copyright (C) M-Systems Flash Disk Pioneers Ltd. 1995-2005            */
/*                         All Rights Reserved                                     */
/***********************************************************************************/
/*                            NOTICE OF M-SYSTEMS OEM                              */
/*                           SOFTWARE LICENSE AGREEMENT                            */
/*                                                                                 */
/*      THE USE OF THIS SOFTWARE IS GOVERNED BY A SEPARATE LICENSE                 */
/*      AGREEMENT BETWEEN THE OEM AND M-SYSTEMS. REFER TO THAT AGREEMENT           */
/*      FOR THE SPECIFIC TERMS AND CONDITIONS OF USE,                              */
/*      OR CONTACT M-SYSTEMS FOR LICENSE ASSISTANCE:                               */
/*      E-MAIL = info@m-sys.com                                                    */
/***********************************************************************************/

/***********************************************************************************/
/*																				   */
/*	Header file for doch_api module												   */ 
/*																				   */
/***********************************************************************************/

/*
 * $Log:   V:/PVCSDB/DiskOnChip/archives/Test for 7.x/src/H3/doch_api.h-arc  $
 * 
 *    Rev 1.28   Jun 14 2006 17:40:30   Polina.Marimont
 * 1. runtime solution for IRQ enabling
 * 2. reduce ATA timeout, when using SPI HAL
 * 
 *    Rev 1.27   May 18 2006 14:41:18   polina.marimont
 * v1.40Fin1, PVCS log inside
 */

#ifndef _DOCH_API
#define _DOCH_API

#include "flcommon.h"
#include "flchkdef.h"
#include "_dochapi.h"
#include "blockdev.h"


/************************/
/*	Exported routines	*/
/************************/

#ifdef __cplusplus
extern "C" {
#endif

extern FLBoolean gDochIrqEnabled[];

DOCH_Error bdCallDOCH(int functionNo, IOreq *ioreq);
extern DOCH_Error doch_init_socket (int socketNo, DOCH_InitSocket);
extern DOCH_Error  doch_release_socket( int socketNo );

extern FLDword gDochAccessNanosec;


/*----------------------------------------------------------------------*/
/*                   f l B u i l d G e o m e t r y                      */
/*                                                                      */
/* Get C/H/S information of the disk according to number of sectors.    */
/*                                                                      */
/* Parameters:                                                          */
/*  capacity    : Number of Sectors in Volume                           */
/*  cylinders   : Pointer to Number of Cylinders                        */
/*  heads       : Pointer to Number of Heads                            */
/*  sectors     : Pointer to Number of Sectors per Track                */
/*  oldFormat   : True for one sector per culoster                      */
/*  wIrHandle   : Handle of the disk partition to check                 */
/*                                                                      */
/*----------------------------------------------------------------------*/
TFFS_DLL_API void NAMING_CONVENTION flBuildGeometry(FLDword capacity, FLDword FAR2 *cylinders,
         FLDword FAR2 *heads, FLDword FAR2 *sectors, FLBoolean oldFormat, FLWord wIrHandle);

/* Management routines */
DOCH_Error flDOCHAtaReset(FLByte socketNum);
DOCH_Error DochSDKInit(void);
void DochSDKExit(void);
DOCH_Error DochRegisterSocket(FLDword dwAddress);

/* User define component registration routine */
DOCH_Error flRegisterDochComponents(void);
/* Access layer registration routines */
DOCH_Error hal_init_nor (FLSNative  socketNo);
DOCH_Error hal_init_spi (FLSNative  socketNo);
DOCH_Error hal_init_sim (FLSNative  socketNo);
DOCH_Error hal_init_nor_noFunc (FLSNative  socketNo);
DOCH_Error hal_init_spi_noFunc (FLSNative  socketNo);
DOCH_Error hal_init_sim_noFunc (FLSNative  socketNo);
DOCH_Error hal_init_user_routines(FLSNative  socketNo, DOCH_BusAccess_routines accessPointerStruct);

void DOCH_SetBits(FLDword* var, FLDword mask, FLByte offset, FLDword val);

#ifdef __cplusplus
}
#endif

#endif /*_DOCH_API*/

