/***********************************************************************************/
/*                        M-Systems Confidential                                   */
/*           Copyright (C) M-Systems Flash Disk Pioneers Ltd. 1995-2006            */
/*                         All Rights Reserved                                     */
/***********************************************************************************/
/*                            NOTICE OF M-SYSTEMS OEM                              */
/*                           SOFTWARE LICENSE AGREEMENT                            */
/*                                                                                 */
/*      THE USE OF THIS SOFTWARE IS GOVERNED BY A SEPARATE LICENSE                 */
/*      AGREEMENT BETWEEN THE OEM AND M-SYSTEMS. REFER TO THAT AGREEMENT           */
/*      FOR THE SPECIFIC TERMS AND CONDITIONS OF USE,                              */
/*      OR CONTACT M-SYSTEMS FOR LICENSE ASSISTANCE:                               */
/*      E-MAIL = info@m-sys.com                                                    */
/***********************************************************************************/

/*
 * $Log:   V:/PVCSDB/DiskOnChip/archives/Testing/TrueFFS 6.3/Drop 2.5/3/common/dosformt.h-arc  $
 * 
 *    Rev 1.8   Mar 15 2006 11:34:16   DoronC
 * Change include path in order to narow needed h files.
 * 
 *    Rev 1.7   Feb 14 2006 10:59:16   DoronC
 * Update cpyright note.
 * 
 *    Rev 1.6   Feb 02 2006 17:34:02   DoronC
 * Remove FL_PACKED_STRUCTS_SUPPORT and use only
 *  FL_NO_PACKED_STRUCTS_SUPPORT.
 * 
 *    Rev 1.5   Dec 26 2005 08:15:06   DoronC
 * Added formatCall routine to support new interface for logical 
 * format in order to support format in all packages.
 * 
 *    Rev 1.4   Dec 08 2005 07:39:54   DoronC
 * Fix compilation errors when FAT32 is enabled.
 * 
 *    Rev 1.3   Dec 05 2005 11:24:08   DoronC
 * Fix compilation errors & warnings under GNU compiler.
 * 
 *    Rev 1.2   Dec 04 2005 11:38:26   DoronC
 * Finalizing package for version 7.0.0
 * 
 *    Rev 1.1   Nov 13 2005 10:16:28   DoronC
 * Change header files to have external headers for binary release.
 * 
 *    Rev 1.0   Mar 27 2005 09:27:26   oris
 * Initial revision.
 * 
 *    Rev 1.2   Feb 19 2005 12:31:22   eugen
 * now uses FL_PACKED_STRUCTURES_BEFORE and _AFTER
 * 
 *    Rev 1.0   Dec 09 2004 18:23:42   oris
 * Initial revision.
 * 
 *    Rev 1.8   Oct 17 2004 15:04:14   OriS
 * - Renamed reserved field in DirectoryEntry to lowerCaseFlags
 *        ASCII interface 	- FL_LONG_FILENAMES_ASCII_SUPPORT
 *        UTF interface 	- FL_LONG_FILENAMES_UTF8_SUPPORT
 *        User specific 	- FL_TRANSLATE_FROM_UNICODE and 
 *                                   	  FL_TRANSLATE_TO_UNICODE user specific
 *        None for Unicode
 * 
 *    Rev 1.7   Jun 15 2004 08:45:52   eugen
 * renamed reserved field in DirectoryEntry to lowerCaseFlags
 * 
 *    Rev 1.6   Dec 01 2003 16:08:00   oris
 * Added support for windows packed structure pragma using the FL_WINDOWS_PACKING compilation flag.
 * 
 *    Rev 1.5   Nov 16 2003 18:55:12   omerk
 * Remove the Marcos: FL_PACKED_STRUCTURES_BEGIN,  FL_PACKED_STRUCTURES_END .
 * 
 * 
 *    Rev 1.4   Nov 09 2003 17:06:08   OmerK
 * Added default FL_PACKED_STRUCTURES_BEGIN and FL_PACKED_STRUCTURES_END for forced packed structures.
 * 
 *    Rev 1.3   Oct 20 2003 09:01:22   oris
 * Changed word variable to FLWord.
 * 
 *    Rev 1.2   Sep 30 2003 17:57:10   oris
 * Moved none API definitions to _dosformt.h and added FL prefix to most of SureFS definition.
 * Moved SureFS API definitions from _blkdev.h
 * Added FL prefix to SECTOR_SIZE and SECTOR_SIZE_BITS
 * 
 *    Rev 1.1   Sep 01 2003 19:09:06   oris
 * - Added LfnDirectoryEntry structure.
 * - Added a customizable packed structure prefix macro , to force packing of the FAT structures.
 * - Added SureFS flag definitions. 
 * 
 *    Rev 1.0   Apr 09 2003 12:15:58   OriS
 * Initial revision.
 * 
 */

#ifndef DOSFORMT_H
#define DOSFORMT_H

#include "flbase.h"
#include "fltl.h"


#define DRIVE_NUMBER            0x80
#define EXTENDED_BOOT_SIGNATURE 0x29
#define BACKUP_OF_BOOT_SECTOR   6
#define FSI_LEADING_SIGNATURE   0x41615252
#define FSI_STRUCTURE_SIGNATURE 0x61417272

#ifndef FL_ROOT_DIR_ENTRIES
#define FL_ROOT_DIR_ENTRIES 512
#endif /* FL_ROOT_DIR_ENTRIES */


/* The BIOS parameter block (a part of the boot sector)		    */
/* Note that this is NOT the customary definition of the BPB    */
/* (customary is to start it on 'bytesPerSector'). To avoid the */
/* nuisance of a structure that starts at an odd offset, we add */
/* the first 11 bytes of the boot sector here.			        */

#ifndef FL_NO_PACKED_STRUCTS_SUPPORTED

#ifdef FL_WINDOWS_PACKING
#pragma pack(push, 1)
#endif /* FL_WINDOWS_PACKING */


#else /* FL_NO_PACKED_STRUCTS_SUPPORTED */

#endif /* FL_NO_PACKED_STRUCTS_SUPPORTED */

typedef struct 
{
    FLDword   size;
    FLByte    clusterSize;
}DiskParamsTable;

#ifndef FL_NO_PACKED_STRUCTS_SUPPORTED

typedef FL_PACKED_STRUCTURES_BEFORE struct FL_PACKED_STRUCTURES_AFTER {
  BPB	bpb;
  FLByte	physicalDriveNo;
  FLSByte	reserved0;
  FLSByte	extendedBootSignature;
  FLSByte	volumeId[4];
  FLSByte	volumeLabel[11];
  FLSByte	systemId[8];
  FLSByte	bootstrap[448];
  LEushort	signature;
} DOSBootSector;
typedef FL_PACKED_STRUCTURES_BEFORE struct FL_PACKED_STRUCTURES_AFTER {
  BPB	    bpb;
  LEulong	FAT32Size;
  LEushort  extFlags;
  LEushort  FSVer;
  LEulong   RootClus;
  LEushort  FSInfo;
  LEushort  bckUpBootSect;
  FLByte    reserved[12];
  FLByte    physicalDriveNo;
  FLSByte	reserved0;
  FLSByte	extendedBootSignature;
  FLSByte	volumeId[4];
  FLSByte	volumeLabel[11];
  FLSByte	systemId[8];
  FLSByte	bootstrap[420];
  LEushort	signature;
} FAT32BootSector;

typedef FL_PACKED_STRUCTURES_BEFORE struct FL_PACKED_STRUCTURES_AFTER {
  LEulong FSI_LeadSig;
  FLByte  FSI_Reserved1[480];
  LEulong FSI_StructSig;
  LEulong FSI_Free_Count;
  LEulong FSI_Nxt_Free;
  FLByte  FSI_Reserved2[14];
  LEushort	signature;
}FAT32FSInfo;
#else /* FL_NO_PACKED_STRUCTS_SUPPORTED */

#define FL_DOSBOOTSECTOR_BPB_OFFSET			0	/* size 36 */
#define FL_DOSBOOTSECTOR_PHYSICAL_DRIVE_NO_OFFSET	36	/* size 1 */
#define FL_DOSBOOTSECTOR_RESERVED0_OFFSET		37	/* size 1 */
#define FL_DOSBOOTSECTOR_EXTENDED_BOOT_SIGNATURE_OFFSET	38	/* size 1 */
#define FL_DOSBOOTSECTOR_VOLUME_ID_OFFSET		39	/* size 4, array */
#define FL_DOSBOOTSECTOR_VOLUME_LABEL_OFFSET		43	/* size 11, array */
#define FL_DOSBOOTSECTOR_SYSTEM_ID_OFFSET		54	/* size 8, array */
#define FL_DOSBOOTSECTOR_BOOTSTRAP_OFFSET		62	/* size 448, array */
#define FL_DOSBOOTSECTOR_SIGNATURE_OFFSET		510	/* size 2 */
#define FL_DOSBOOTSECTOR_SIZE				512

#define FL_FAT32BOOTSECTOR_BPB_OFFSET			0	/* size 36 */
#define FL_FAT32BOOTSECTOR_FAT32SIZE_OFFSET	    36	/* size 4 */
#define FL_FAT32BOOTSECTOR_EXTFLAGS_OFFSET		40	/* size 2 */
#define FL_FAT32BOOTSECTOR_FSVER_OFFSET	        42	/* size 2 */ 
#define FL_FAT32BOOTSECTOR_ROOTCLUSTER_OFFSET	44	/* size 4 */
#define FL_FAT32BOOTSECTOR_FSINFO_OFFSET		48	/* size 2 */
#define FL_FAT32BOOTSECTOR_BCKUPBOOTSECT_OFFSET	50	/* size 2*/
#define FL_FAT32BOOTSECTOR_RESERVED_OFFSET	    52	/* size 12, array */
#define FL_FAT32BOOTSECTOR_PHYDRIVENO_OFFSET	64	/* size 1 */
#define FL_FAT32BOOTSECTOR_RESERVED0_OFFSET	    65	/* size 1*/
#define FL_FAT32BOOTSECTOR_EXTSIGNATURE_OFFSET	66	/* size 1 */
#define FL_FAT32BOOTSECTOR_VOLUMEID_OFFSET	    67	/* size 4, array */
#define FL_FAT32BOOTSECTOR_VOLUMELABEL_OFFSET	71	/* size 11, array */
#define FL_FAT32BOOTSECTOR_SYSTEMID_OFFSET	    82	/* size 8 */
#define FL_FAT32BOOTSECTOR_BOOTSTRAP_OFFSET	    90	/* size 420, array */
#define FL_FAT32BOOTSECTOR_SIGNATURE_OFFSET		510	/* size 2 */
#define FL_FAT32BOOTSECTOR_SIZE				    512



#define FL_FAT32_FSINFO_LEADSIGNATURE_OFFSET	0   /* size 4*/
#define FL_FAT32_FSINFO_RESERVED1_OFFSET		4   /* size 480, array*/
#define FL_FAT32_FSINFO_STRUCTSIGNATURE_OFFSET	484 /* size 4*/
#define FL_FAT32_FSINFO_FREECOUNT_OFFSET		488 /* size 4*/
#define FL_FAT32_FSINFO_NEXTFREE_OFFSET			492 /* size 4*/
#define FL_FAT32_FSINFO_RESERVED2_OFFSET		496 /* size 4*/
#define FL_FAT32_FSINFO_SIGNATURE_OFFSET		510 /* size 14, array*/
#define FL_FAT32_FSINFO_SIZE				    512

typedef FLByte DOSBootSector[FL_DOSBOOTSECTOR_SIZE];
typedef FLByte FAT32BootSector[FL_FAT32BOOTSECTOR_SIZE];
typedef FLByte FAT32FSInfo[FL_FAT32_FSINFO_SIZE];
#endif /* FL_NO_PACKED_STRUCTS_SUPPORTED */



#ifndef FL_NO_PACKED_STRUCTS_SUPPORTED

typedef FL_PACKED_STRUCTURES_BEFORE struct FL_PACKED_STRUCTURES_AFTER {
	FLByte	   orderNum;			/* 6 significant bits of LFN order number */
	Unaligned  name1[5];			/* part 1 of scattered name */
	FLByte	   attributes;			/* file attributes, must be RO, System, Hidden, Volume */
	FLByte	   reserved;			/* must be 0 */
	FLByte	   checksum;			/* checksum created from short form name */
	LEushort   name2[6];			/* part 2 of scattered name */
	LEushort   startingCluster;		/* must be 0 */
	LEushort   name3[2];			/* part 3 of scattered name */
} LfnDirectoryEntry;
#else /* FL_NO_PACKED_STRUCTS_SUPPORTED */

#define FL_LFN_DIRECTORY_ENTRY_ORDER_NUM_OFFSET			0	/* size 1 */
#define FL_LFN_DIRECTORY_ENTRY_NAME1_OFFSET			1	/* size 10, unaligned array of 2 */
#define FL_LFN_DIRECTORY_ENTRY_ATTRIBUTES_OFFSET		11	/* size 1 */
#define FL_LFN_DIRECTORY_ENTRY_RESERVED_OFFSET			12	/* size 1 */
#define FL_LFN_DIRECTORY_ENTRY_CHECKSUM_OFFSET			13	/* size 1 */
#define FL_LFN_DIRECTORY_ENTRY_NAME2_OFFSET			14	/* size 12, array of 2 */
#define FL_LFN_DIRECTORY_ENTRY_STARTING_CLUSTER_OFFSET		26	/* size 2 */
#define FL_LFN_DIRECTORY_ENTRY_NAME3_OFFSET			28	/* size 4, array of 2 */
#define FL_LFN_DIRECTORY_ENTRY_SIZE				32

typedef FLByte LfnDirectoryEntry[FL_LFN_DIRECTORY_ENTRY_SIZE];
#endif /* FL_NO_PACKED_STRUCTS_SUPPORTED */

/* Directory entry attribute bits */



#ifndef FL_NO_PACKED_STRUCTS_SUPPORTED

typedef FL_PACKED_STRUCTURES_BEFORE struct FL_PACKED_STRUCTURES_AFTER {
  /* First partition entry starts here. We do not map it as a	*/
  /* separate structure because it is not longword aligned	*/
  FLByte	activeFlag;	/* 80h = bootable */
  FLByte        startingHead;
  LEushort	startingCylinderSector;
  FLSByte       type;
  FLByte	endingHead;
  LEushort	endingCylinderSector;
  Unaligned4	startingSectorOfPartition;
  Unaligned4	sectorsInPartition;
  /* Partition entries 2,3 and 4 are structured as the 1st partition */
} Partition;
#else /* FL_NO_PACKED_STRUCTS_SUPPORTED */

#define FL_PARTITION_ACTIVE_FLAG_OFFSET				0	/* size 1 */
#define FL_PARTITION_STARTING_HEAD_OFFSET			1	/* size 1 */
#define FL_PARTITION_STARTING_CYLINDER_SECTOR_OFFSET		2	/* size 2 */
#define FL_PARTITION_TYPE_OFFSET				4	/* size 1 */
#define FL_PARTITION_ENDING_HEAD_OFFSET				5	/* size 1 */
#define FL_PARTITION_ENDING_CYLINDER_SECTOR_OFFSET		6	/* size 2 */
#define FL_PARTITION_STARTING_SECTOR_OF_PARTITION_OFFSET	8	/* size 4 */
#define FL_PARTITION_SECTORS_IN_PARTITION_OFFSET		12	/* size 4 */
#define FL_PARTITION_SIZE					16

typedef FLByte Partition[FL_PARTITION_SIZE];
#endif /* FL_NO_PACKED_STRUCTS_SUPPORTED */


#ifndef FL_NO_PACKED_STRUCTS_SUPPORTED

typedef FL_PACKED_STRUCTURES_BEFORE struct FL_PACKED_STRUCTURES_AFTER {
  FLSByte       reserved1[0x1A6];
  Unaligned4    passwordInfo[3];  /* M-Systems proprietary */
  FLSByte       reserved2[0xC];   /* NT4 or NT5 signature place */

  /* First partition entry starts here. We do not map it as a	*/
  /* separate structure because it is not longword aligned	*/
  Partition     ptEntry[4];
  LEushort      signature;	/* = PARTITION_SIGNATURE */
} PartitionTable;
#else /* FL_NO_PACKED_STRUCTS_SUPPORTED */

#define FL_PARTITION_TABLE_RESERVED1_OFFSET			0	/* size 0x1A6 */
#define FL_PARTITION_TABLE_PASSWORD_INFO_OFFSET			422	/* size 12, unaligned array of 4 */
#define FL_PARTITION_TABLE_RESERVED2_OFFSET			434	/* size 12, array */
#define FL_PARTITION_TABLE_PT_ENTRY_OFFSET			446	/* size 4x16, array of partition entries */
#define FL_PARTITION_TABLE_SIGNATURE_OFFSET			510	/* size 2 */
#define FL_PARTITION_TABLE_SIZE					512

typedef FLByte PartitionTable[FL_PARTITION_TABLE_SIZE];
#endif /* FL_NO_PACKED_STRUCTS_SUPPORTED */

#ifndef FL_NO_PACKED_STRUCTS_SUPPORTED

#ifdef FL_WINDOWS_PACKING
#pragma pack(pop)
#endif /* FL_WINDOWS_PACKING */

#endif /* FL_NO_PACKED_STRUCTS_SUPPORTED */

#define PARTITION_ENTRIES	4


/** Values of irFlags for flOpenFile: */

#define ACCESS_MODE_MASK   3 /* Mask for access mode bits */

/* Individual flags */

#define FL_ACCESS_CREATE      2 /* Create new file      */

/* commands to flGetSetFileInfo (in addition to above) */
#define FL_GET_DATETIME        -1 /* Get date/time                        */
#define FL_GET_ATTRIBUTES      -2 /* Get attributes                       */
#define FL_GET_CREATE_DATETIME -8 /* Get creation date/time               */


#ifndef LFN_NAME_MAX_CHARACTERS
#define LFN_NAME_MAX_CHARACTERS			255
#endif /* LFN_NAME_MAX_CHARACTERS */

#ifndef PATH_SEGMENTS_MAX_NUM
#define PATH_SEGMENTS_MAX_NUM			10
#endif /* PATH_SEGMENTS_MAX_NUM */

#ifndef PATH_CHARACTERS_MAX_NUM
#define PATH_CHARACTERS_MAX_NUM			LFN_NAME_MAX_CHARACTERS
#endif  /* PATH_CHARACTERS_MAX_NUM */

#ifndef FL_MAX_TREE_LEVELS
#define FL_MAX_TREE_LEVELS	  ( PATH_CHARACTERS_MAX_NUM / 2 + 1 )
#endif  /* FL_MAX_TREE_LEVELS */

#ifndef FL_ROOT_DIR_ENTRIES
#define FL_ROOT_DIR_ENTRIES(socket,partition) 512
#endif /* FL_ROOT_DIR_ENTRIES */

#define UNICODE_CHAR_PER_LFN_ENTRY		13

#define SHORT_NAME_MAX_LENGTH	        8
#define EXTENSION_MAX_LENGTH	        3
#define NAME_8_3_MAX_LENGTH	            12		/* "tttttttt.xxx" */

/* Directory entry attribute bits for internal use */
#define ATTR_SUREFS_GEN_ATTENTION_FLAG	0x40
#define ATTR_SUREFS_REN_ATTENTION_FLAG	0x80
#define ATTR_SUREFS_ATTENTION_FLAG_MASK	(ATTR_SUREFS_GEN_ATTENTION_FLAG |	\
										ATTR_SUREFS_REN_ATTENTION_FLAG)

#define ATTR_MASK	((unsigned char)(FL_ATTR_READ_ONLY | FL_ATTR_HIDDEN | FL_ATTR_SYSTEM | \
                                     FL_ATTR_VOL_LABEL | FL_ATTR_DIRECTORY | FL_ATTR_ARCHIVE))
#define LFN_ATTRIBUTES	((unsigned char)(FL_ATTR_READ_ONLY | FL_ATTR_HIDDEN | \
                                         FL_ATTR_SYSTEM | FL_ATTR_VOL_LABEL))

#define DIRECTORY_ENTRY_SIZE	         32

#define DIRECTORY_ENTRIES_PER_SECTOR	 (FL_SECTOR_SIZE / DIRECTORY_ENTRY_SIZE)
#define MAX_ROOT_DIR_SECTORS		((0x10000/DIRECTORY_ENTRIES_PER_SECTOR) - 1)

#define SET_GLOBAL_ATTENTION_FLAG        0
#define RESET_GLOBAL_ATTENTION_FLAG	     0xffff
#define GLOBAL_ATT_FLAG_FAT_ENTRY        1

#define	NEVER_USED_DIR_ENTRY             0

#ifndef FL_NO_PACKED_STRUCTS_SUPPORTED
#define	DELETED_DIR_ENTRY                ((FLSByte) 0xe5)
#else /*FL_NO_PACKED_STRUCTS_SUPPORTED*/
#define	DELETED_DIR_ENTRY                ((FLByte) 0xe5)
#endif /*FL_NO_PACKED_STRUCTS_SUPPORTED*/

/* FAT definitions */

#define	FAT_FREE                         0
#define	FAT_BAD_CLUSTER	                 0xfff7
#define	FAT_LAST_CLUSTER                 0xffff  /* actually any of 0xfff8-0xffff */

/* Partition table definitions */

#define PARTITION_SIGNATURE	             0xaa55

#define CYLINDER_SECTOR(cylinder,sector) ((sector) + ((cylinder & 0xff) << 8) + ((cylinder & 0x300) >> 2))


#ifndef FL_MAX_PARTITION_DEPTH
#define FL_MAX_PARTITION_DEPTH           0x08
#endif /* FL_MAX_PARTITION_DEPTH */

#define LFN_SHORT_NAME			0		/* for 8.3 short file name	*/
#define LFN_LONG_NAME			1		/* for long file name		*/
#define LFN_INVALID_SET			2		/* found invalid file name characters				*/
#define SFN_LOWERCASE_NAME8		0x08		/* these 2 bit-flags are to support */
#define SFN_LOWERCASE_EXT3		0x10		/* NT VFAT lowercase bits */

/*-------------------------------------------------------------------------------------------
*				F L P a t h S e g m e n t
* The structure is used when analyzing path string.
* Parameters:	name - address of first character of the path segment (unicode null terminated
*						string. For example, for first segment of /first/second/third.ext it 
*						will point on "first".
*				length - number of characters of the path segment
*				buffSize - in case of retrieving file name this field contains size of supplied buffer
*										in characters.
*				get - if TRUE, this structure is used to retrieve found file name 
*						(getFirstFileName()). If FALSE, this structure contains file name to 
*						be opened/created/found	(findFileName()).
*------------------------------------------------------------------------------------------*/
typedef struct {
	FLWchar FAR1	*name;
	short           length;
	short           buffSize;
	FLBoolean       get;
	FLBoolean       reserved;
} FLPathSegment;

/*--------------------------------------------------------------------------
*				F L L f n S e a r c h
* The FLLfnSearch structer is used when searching directory by specified	
* file name.																
* Parameters:	
*		neverUsed - indicates that found at least one directory entry in sector
*					that was never used.
*		verifiedChars - Number of characters that successfully compared	with	
*						file name or found in free entries in previous sector	
*		checksum - LFN checksum found in previous sector							
*		lastNum - LFN order number of last LFN entry found in previous sector		
*		idx - directory entry index of found entry								
*		status - LFN_SHORT_NAME, LFN_LONG_NAME, LFN_UPPERCASE_FOR_SHORT,
*					LFN_INVALID_SET
*---------------------------------------------------------------------------*/
typedef struct {
	unsigned	verifiedChars;		
	unsigned	lastNum;
	unsigned	idx;
	int		status;
	unsigned char	checksum;
	FLBoolean	neverUsed;
} FLLfnSearch;



#endif /* DOSFORMT_H */

