/* msi.h - Message Signaled Interrupts header file */

/* Copyright (c) 2005 Wind River Systems, Inc */
            
/*
modification history
--------------------
01a,24jun05,mdo  Written.
*/
 
#ifndef msi_H
#define msi_H
 
#ifdef __cplusplus
extern "C" {
#endif

/* Message Signaled Interrupts registers */
                                             
/* Message Control Register */


#define PCI_MSI_CTL                 0x02    /* Offset of MSI control register */
    #define  PCI_MSI_CTL_ENABLE     0x01    /* MSI enable */
    #define  PCI_MSI_CTL_MSG_ALLOC  0x70    /* Number of Messages allocated */
    #define  PCI_MSI_CTL_MSG_MAX    0x0E    /* Maximum Messages Allowed */
    #define  PCI_MSI_CTL_64BIT      0x80    /* 64-bit addresses allowed */

/* Message Address Register */


#define PCI_MSI_ADDR_LO             0x04    /* Offset of MSI Address Lower 32 bits */
                                            /* Used for 32-bit MSI */  
#define PCI_MSI_ADDR_HI             0x08    /* Offset of MSI Address Upper 32 bits */
                                            /* Used with 64-bit MSI */

/* Message Data Register */


#define PCI_MSI_DATA_32             0x08    /* Offset of MSI Data Register */
                                            /* Used for 32-bit MSI */
#define PCI_MSI_DATA_64             0x0C    /* Offset of MSI Data Register */
                                            /* Used for 64-bit MSI */

#define MSI_ENABLED (ctlReg)      (ctlReg & PCI_MSI_CTL_ENABLE)
#define ADDR_64BIT (ctlReg)       (ctlReg & PCI_MSI_CTL_64BIT)

#define MSI_ENABLE_MSG (ctlReg, msgNum) \
    ctlReg |= (((msgNum >> 1) << 4) & PCI_MSI_CTL_MESS_ALLOC);

#define MSI_ENABLE (ctlReg, msgNum) MSI_ENABLE_MSG (ctlReg, msgNum); \
    ctlReg |= PCI_MSI_FLAGS_ENABLE

#define MSI_DISABLE (ctlReg)        ctlReg &= ~PCI_MSI_CTL_ENABLE
    
#ifdef __cplusplus
}
#endif
                                 /* Used for 64-bit MSI */
#endif /* msi_H */
 
