/***********************************************************************************/
/*                        M-Systems Confidential                                   */
/*           Copyright (C) M-Systems Flash Disk Pioneers Ltd. 1995-2006            */
/*                         All Rights Reserved                                     */
/***********************************************************************************/
/*                            NOTICE OF M-SYSTEMS OEM                              */
/*                           SOFTWARE LICENSE AGREEMENT                            */
/*                                                                                 */
/*      THE USE OF THIS SOFTWARE IS GOVERNED BY A SEPARATE LICENSE                 */
/*      AGREEMENT BETWEEN THE OEM AND M-SYSTEMS. REFER TO THAT AGREEMENT           */
/*      FOR THE SPECIFIC TERMS AND CONDITIONS OF USE,                              */
/*      OR CONTACT M-SYSTEMS FOR LICENSE ASSISTANCE:                               */
/*      E-MAIL = info@m-sys.com                                                    */
/***********************************************************************************/

/*
 * $Log:   V:/PVCSDB/DiskOnChip/archives/Testing/TrueFFS 6.3/Drop 2.5/3/bddoc/src/fatfilt.h-arc  $
 * 
 *    Rev 1.2   Feb 14 2006 10:59:16   DoronC
 * Update cpyright note.
 * 
 *    Rev 1.1   Dec 05 2005 11:24:08   DoronC
 * Fix compilation errors & warnings under GNU compiler.
 * 
 *    Rev 1.0   Mar 27 2005 09:27:18   oris
 * Initial revision.
 * 
 *    Rev 1.1   Dec 23 2004 19:35:44   oris
 * Drop 2.1
 * 
 *    Rev 1.0   Dec 09 2004 18:24:28   oris
 * Initial revision.
 * 
 *    Rev 1.0   Apr 09 2003 12:15:58   OriS
 * Initial revision.
 * 
 */

/*******************************************************************************
 *                                                                             * 
 *                         Module: FATFILT                                     * 
 *                                                                             * 
 *  This module implements installable FAT12/16 filters. It supports up to     *
 *  SOCKETS sockets, with up to FL_MAX_DISKS_PER_SOCKET disks per socket.      * 
 *  Each disk can contain up to FL_MAX_PARTS_PER_DISK partitions on it, with   * 
 *  maximum depth of partition nesting in extended partitions equal to         * 
 *  MAX_PARTITION_DEPTH.                                                       *
 *                                                                             * 
 *  In order for this module to work, disks must be abs.mounted rather then    * 
 *  mounted. In latter case, this module won't detect any of disk's            * 
 *  partitions, and won't install FAT filters.                                 * 
 *                                                                             * 
 *  This module uses more then 512 bytes of stack space in case if MALLOC is   * 
 *  not enabled.                                                               * 
 *                                                                             * 
 *******************************************************************************/


#ifndef FLFF_H
#define FLFF_H



#include "dosformt.h"



/* number of entries in disk's partition table */

#define  FL_PART_TBL_ENTRIES  4

/* max number of partitions (filesystem volumes) per disk */
 
#define  FL_MAX_PARTS_PER_DISK  (FL_PART_TBL_ENTRIES + FL_MAX_PARTITION_DEPTH)




/* 
 * Generic 'initialization status' type 
 */

typedef enum { 

    flStateNotInitialized = 0, 
    flStateInitInProgress = 1, 
    flStateInitialized    = 2 

    } FLState; 


/* 
 * Disk partition (filesystem volume). Multiple partitions are allowed
 * on the disk.
 */

typedef struct {

    int        handle;             /* disk's TFFS handle                  */
    int        type;               /* FAT16_PARTIT                        */
    int        flags;              /* VOLUME_12BIT_FAT etc.               */
    FLBoolean  ffEnabled;          /* FAT filter is enabled on that part. */
    SectorNo   startSecNo;         /* sectorNo where partition starts     */
    SectorNo   sectors;            /* (info) total sectors in partition   */
    SectorNo   firstFATsecNo;      /* sectorNo of 1st sector of 1st FAT   */
    SectorNo   lastFATsecNo;       /* sectorNo of last sector of 1st FAT  */
    SectorNo   firstDataSecNo;
    unsigned   clusterSize;        /* Cluster size in sectors             */

} FLffVol;


/*
 * Disk with multiple partitions. Multiple disks are allowed on socket. 
 */

typedef struct {

    int        handle;             /* disk's TFFS handle              */
    FLState    ffstate;            /* FAT filter init. state          */
    int        parts;              /* total FAT12/16 partitions found */
    FLffVol  * part[FL_MAX_PARTS_PER_DISK];
    SectorNo   secToWatch;         /* used to track disk partitioning */
    FLSByte  * buf;                /* scratch buffer                  */

} FLffDisk;


/* 
 * Master Boot Record/Extended Boot Record of the disk 
 */

typedef struct {

    FLSByte            reserved[0x1be];

    struct {
        FLByte         activeFlag;    /* 80h = bootable */
        FLByte         startingHead;
        LEushort       startingCylinderSector;
        FLSByte        type;
        FLByte         endingHead;
        LEushort       endingCylinderSector;
        Unaligned4     startingSectorOfPartition;
        Unaligned4     sectorsInPartition;
    } parts [FL_PART_TBL_ENTRIES];

    LEushort           signature;    /* = PARTITION_SIGNATURE */

} flMBR;




/*
 * FAT Filter API 
 */

#if defined(FL_ABS_READ_WRITE) && !defined(FL_READ_ONLY)

  extern FLStatus     ffCheckBeforeWrite (IOreq FAR2 *ioreq);
  extern FLStatus     flffControl (int devNo, int partNo, FLState state);
  extern FLffDisk*    flffInfo    (int devNo);

#endif

#endif /* FLFF_H */

