/***********************************************************************************/
/*                        M-Systems Confidential                                   */
/*           Copyright (C) M-Systems Flash Disk Pioneers Ltd. 1995-2006            */
/*                         All Rights Reserved                                     */
/***********************************************************************************/
/*                            NOTICE OF M-SYSTEMS OEM                              */
/*                           SOFTWARE LICENSE AGREEMENT                            */
/*                                                                                 */
/*      THE USE OF THIS SOFTWARE IS GOVERNED BY A SEPARATE LICENSE                 */
/*      AGREEMENT BETWEEN THE OEM AND M-SYSTEMS. REFER TO THAT AGREEMENT           */
/*      FOR THE SPECIFIC TERMS AND CONDITIONS OF USE,                              */
/*      OR CONTACT M-SYSTEMS FOR LICENSE ASSISTANCE:                               */
/*      E-MAIL = info@m-sys.com                                                    */
/***********************************************************************************/

/*
 * $Log:   V:/PVCSDB/DiskOnChip/archives/Testing/TrueFFS 6.3/Drop 2.5/3/bddoc/src/flflash.h-arc  $
 * 
 *    Rev 1.8   Feb 14 2006 11:05:14   DoronC
 * Update copyright note.
 * 
 *    Rev 1.7   Dec 04 2005 11:41:12   DoronC
 * Finalizing package for TrueFFS 7.0.0
 * 
 *    Rev 1.6   Nov 13 2005 10:24:46   DoronC
 * Change header files to have external headers for binary release.
 * 
 *    Rev 1.5   Oct 10 2005 17:49:46   roneng
 * - Add plant BBT debuging definitions
 * 
 *    Rev 1.4   Sep 12 2005 13:09:48   roneng
 * Remove G4_SUPPORT definition
 * 
 *    Rev 1.3   Aug 14 2005 14:58:14   roneng
 * Add G4 definitions
 * 
 *    Rev 1.2   Apr 25 2005 02:03:34   oris
 * - Moved special API flags from _flflash.h
 * - Removed MAX_SECTORS_PER_BLOCK definition
 * 
 *    Rev 1.1   Apr 21 2005 15:46:24   oris
 * Split some of the content to flsystyp.h
 * 
 *    Rev 1.0   Mar 27 2005 09:27:24   oris
 * Initial revision.
 * 
 *    Rev 1.3   Mar 27 2005 09:16:58   oris
 * - Added modes to plant BBT API
 * 
 *    Rev 1.2   Mar 07 2005 22:04:42   oris
 * -	Added missing fields to DMA record
 * -	Changed DBB marking to have a better bit distance.
 * 
 *    Rev 1.1   Feb 01 2005 09:19:36   eugen
 * new flash contstants
 * 
 *    Rev 1.0   Dec 09 2004 18:24:28   oris
 * Initial revision.
 * 
 *    Rev 1.7   Oct 17 2004 15:46:12   oris
 * Added some new interfaces / definitions:
 *      IPL modes :
 * - FL_USING_BUFFERS - Allow accessing the IPL using                         
 *                                               buffers smaller then IPL size.
 * - FL_IPL_PAGED_RAM_MODE  - Written in Paged RAM mode         
 * - FL_IPL_VIRTUAL_RAM_MODE - Written in Virtual RAM mode         
 * - FL_IPL_ALERNATE_MAP - Written with X-Scale mode enabled
 *     BBT marks :
 * - FL_BLOCK_IS_MARKED_AS_UNERASABLE 	0
 * - FL_BLOCK_IS_MARKED_AS_UNWRITABLE 	0x11
 * - FL_BLOCK_IS_MARKED_AS_UNREADABLE 	0x55
 * - FL_BLOCK_IS_MARKED_AS_GOOD                 	0xFF
 * 
 *    Rev 1.6   Sep 22 2004 13:03:14   roneng
 * Add TFFS6.3 definitions for M512-G3 MTD basic functionality
 * 
 *    Rev 1.5   Jan 27 2004 08:14:02   oris
 * Changed comments
 * 
 *    Rev 1.4   Sep 01 2003 19:07:54   oris
 * - Added new supported technologies definitions:
 *    FL_SUPPORT_WRITE_CACHE     0x10
 *    FL_2K_ADDRESS_SPACE        0x20
 *    FL_SUPPORT_SEQUENTIAL_READ 0x40
 * 
 *    Rev 1.3   Aug 10 2003 13:48:08   roneng
 * 
 * The following definition were added to flashTechnology Types
 * - FL_SUPPORT_WRITE_CACHE
 * - FL_2K_ADDRESS_SPACE
 * - FL_SUPPORT_SEQUENTIAL_READ
 * 
 * 
 *    Rev 1.2   Jun 16 2003 01:18:18   oris
 * Added TFFS_DLL_API prefix.
 * 
 *    Rev 1.1   Jun 05 2003 03:11:58   oris
 * Bug fix - wrong FAR level qualifier in access layer routines.
 * 
 *    Rev 1.0   Apr 09 2003 12:15:58   OriS
 * Initial revision.
 * 
 */

#ifndef FLFLASH_H
#define FLFLASH_H

#include "flbase.h"

#ifndef MTD_STANDALONE
#include "flsocket.h"
#else
#include "flbuffer.h"
#endif /* MTD_STANDALONE */

/* Some useful types for mapped Flash locations */

typedef volatile FLByte FAR0 * FlashPTR;
typedef volatile unsigned short int FAR0 * FlashWPTR;
typedef volatile FLDword FAR0 * FlashDPTR;
typedef FLWord FlashType;        /* JEDEC id */
typedef volatile FLByte FAR0* NDOC2window;


#define NOT_FLASH          0

/* Media types */
#define NOT_DOC_TYPE              0
#define DOC_TYPE                  1
#define MDOC_TYPE                 2
#define DOC2000TSOP_TYPE          3
#define MDOCP_TYPE                4
#define MDOCP_16_TYPE             5
#define DOC_H1_TYPE               6
#define M512_G3_TYPE              7  
#define M256_P3_TYPE              8 
#define M256_G3_TYPE              9  
#define M128_P3_TYPE              10 
#define M1G_G3_TYPE               11
#define M512_P3_TYPE              12 
#define M1G_G4_TYPE               13
#define M512_P4_TYPE              14 
#define H3_TYPE                   15
#define AMD_NOR_TYPE              100 
#define SEIJA_TYPE                200

/* page characteristics flags */
#define  BIG_PAGE    0x0100             /* page size > 100H*/
#define  FULL_PAGE   0x0200                  /* no partial page programming*/
#define  BIG_ADDR    0x0400             /* 4 byte address cycle */

/* MTD write routine mode flags */
#define FL_DATA          0      /* Read/Write data area                */
#define OVERWRITE        1      /* Overwriting non-erased area         */
#define EDC              2      /* Activate ECC/EDC                    */
#define EXTRA            4      /* Read/write spare area               */
#define DELETE_FLAG      0x20   /* Write sector with delete mark on sector flags*/
#define NO_SECOND_TRY    0x8000 /* do not read again on EDC error      */
#define TRANSACTION_BITS 0x0F00 /* bits that are saved for transaction no' */


/* protection specific defintions */
#define PROTECTION_KEY_LENGTH    8 /* Size of protection key in bytes    */  
#define MAX_PROTECTED_PARTITIONS 2 /* Max Number of protected partitiosn */
#define DEFAULT_KEY              "00000000"

/* IPL modes */
#define FL_IPL_MODE_SA          2  /* Not used any more, done automatically by */
#define FL_IPL_MODE_XSCALE      4  /* Not used any more, done automatically by */

#define FL_IPL_READ_FROM_START   0
#define FL_IPL_DOWNLOAD_AND_READ 1


/* Bit 5 Quad Redundency of IPL mode                                               */
#define FL_IPL_PAGE_QUAD_RED	32

                                        /*       smaller then IPL size                */
/* OTP specific defintions */
#define CUSTOMER_ID_LEN          4
#define UNIQUE_ID_LEN            16

/* BBT block types */
#define BBT_GOOD_UNIT            0xff
#define BBT_UNMARKED_BAD_UNIT    0xf0
#define BBT_UNAVAIL_UNIT         0x1
#define BBT_BAD_UNIT             0x0

/* Unmarked bad blocks types */
#define FL_BLOCK_IS_MARKED_AS_UNERASABLE 0x55
#define FL_BLOCK_IS_MARKED_AS_UNWRITABLE 0x22
#define FL_BLOCK_IS_MARKED_AS_UNREADABLE 0x88
#define FL_BLOCK_IS_MARKED_AS_GOOD       0xff
#define FL_BLOCK_IS_MARKED_AS_NEED_ERASE 0xfe
#define FL_MAX_BIT_DISTANCE_DBB          0x20 

/* Flag bit values */
#define SUSPEND_FOR_WRITE        1        /* MTD provides suspend for write */
#define NFTL_ENABLED             2        /* Flash can run NFTL             */
#define INFTL_ENABLED            4        /* Flash can run INFTL            */
#define EXTERNAL_EPROM           8        /* Can support external eprom     */
#define SAFTL_ENABLED            0x10     /* Flash can run SAFTL            */

/* Flash Technology types */
#define FL_SUPPORT_MLC             0x01  
#define FL_SUPPORT_RELIABLE        0x02
#define FL_SUPPORT_FAST            0x04
#define FL_SUPPORT_NO_MATCHING     0x08
#define FL_SUPPORT_WRITE_CACHE     0x10
#define FL_2K_ADDRESS_SPACE        0x20
#define FL_SUPPORT_SEQUENTIAL_READ 0x40

/* Bus types */
#define FL_SRAM_MODE        0   /* Data bus and Address bus are diffrent buses            */
#define FL_MUX_MODE         1   /* Data and Address buses are multiplexed on the same bus */
#define FL_LPC_MODE         2   /* Data and Address buses are both using the LPC bus      */
#define FL_ICP_MODE         3   


/* Debugging flags */
#define DEBUG_ERASE_OPERATION_ENTER    0  /* Before start of an erase operation */
#define DEBUG_ERASE_OPERATION_EXIT     3  /* After end of erase operation       */
#define DEBUG_WRITE_OPERATION_ENTER    1  /* Before start of an write operation */
#define DEBUG_WRITE_OPERATION_EXIT     4  /* After end of write operation       */
#define DEBUG_READ_OPERATION           2  /* Before read operation              */
#define DEBUG_INIT_ENDED               5  /* Variables are initialized          */
#define DEBUG_EXIT_CALLED              6  /* Variables are not initialized      */
#define DEBUG_FORMAT_OPERATION_ENTER1  7  /* Before start format - 2 calls are  */
#define DEBUG_FORMAT_OPERATION_ENTER2  8  /* required 2 pass 2 parameters.      */
#define DEBUG_FORMAT_OPERATION_EXIT    9  /* After end format operation         */
#define DEBUG_UNFORMAT_OPERATION_ENTER 10 /* Before start unformat operation    */
#define DEBUG_UNFORMAT_OPERATION_EXIT  11 /* After end unformat operation       */
#define DEBUG_ERASEBD_OPERATION_ENTER  12 /* Before start ERASE BD operation    */
#define DEBUG_ERASEBD_OPERATION_EXIT   13 /* After end ERASE BD operation       */
#define DEBUG_IS_GOOD_BLOCK            255 /* Called by MTD to check block     */

/* prepareForPowerChanges routine flags */
#define FL_PREPARE_FOR_SUSPEND   2

/*************************************/
/* DMA type flags - FL_DMA_CONFIG    */
/*************************************/

/* Note: DMA_Params_S is defined in flsystyp.h */
#define DMA_OPEN_CHANNEL               0
#define DMA_CONFIG_TRANSACTION         1
#define DMA_FREE_CHANNEL               2    
#define DMA_WAIT_FOR_TRANSACTION_END   3



/****************************/
/*   Special services API   */
/****************************/


/*** MTD Soecial services values   ***/ 
#define FL_DMA_WITHOUT_ECC              0x20  
#define FL_DMA_WITH_ECC                 0  

/* MTD registration information */
extern int noOfMTDs;        /* No. of MTDs actually registered */

/* Flash array identification structure */
typedef struct tFlash FLFlash;                /* Forward definition */

#ifndef FL_NO_USE_FUNC


#endif /* FL_NO_USE_FUNC */

#include "_flflash.h"

#endif /* FLFLASH_H */

