/***********************************************************************************/
/*                        M-Systems Confidential                                   */
/*           Copyright (C) M-Systems Flash Disk Pioneers Ltd. 1995-2003            */
/*                         All Rights Reserved                                     */
/***********************************************************************************/
/*                            NOTICE OF M-SYSTEMS OEM                              */
/*                           SOFTWARE LICENSE AGREEMENT                            */
/*                                                                                 */
/*      THE USE OF THIS SOFTWARE IS GOVERNED BY A SEPARATE LICENSE                 */
/*      AGREEMENT BETWEEN THE OEM AND M-SYSTEMS. REFER TO THAT AGREEMENT           */
/*      FOR THE SPECIFIC TERMS AND CONDITIONS OF USE,                              */
/*      OR CONTACT M-SYSTEMS FOR LICENSE ASSISTANCE:                               */
/*      E-MAIL = info@m-sys.com                                                    */
/***********************************************************************************/

/*
 * $Log:   V:/PVCSDB/DiskOnChip/archives/Testing/TrueFFS 6.3/Drop 2.5/3/common/flformat.h-arc  $
 * 
 *    Rev 1.6   Jul 06 2005 23:34:16   OriS
 * 
 * Cleaned up the standard format params
 * 
 *    Rev 1.4   Jun 07 2005 06:19:00   OriS
 * - Documentation - changed flFormatPhysicalDriver() to flFlashFormat().
 * 
 *    Rev 1.3   May 19 2005 17:47:06   oris
 * There used to be duplication of definition:
 *    - TL_BDTL_STD_SPACE_FACTOR
 *    - FL_NORMAL_SPACING
 * They are now defined as the same defintion.
 * 
 *    Rev 1.2   Apr 19 2005 02:55:40   oris
 * Removed unused flags
 * 
 *    Rev 1.1   Mar 29 2005 10:55:58   Polina.Marimont
 * macros instead of numbes
 * 
 *    Rev 1.0.1.0   Mar 27 2005 14:04:44   Polina.Marimont
 * macros instead of numbers
 * 
 *    Rev 1.0   Mar 27 2005 09:27:28   oris
 * Initial revision.
 * 
 *    Rev 1.3   Mar 27 2005 09:21:22   oris
 * - Removed unused definitions
 * - Added the only valid definition for fast area factor FL_NORMAL_SPACING
 * - Changed flags field in BDTL format parameters record
 * - Added spareBlocks and reserved fields to binary partition record
 * - Added reserved fields for general format record
 * - Renamed the block device format records with the number 3
 * - Added 3 predefined configurations: TL_NO_BOOST / TL_BASIC_BOOST / TL_ENHANCED_PERFORMANCE
 * 
 *    Rev 1.2   Mar 07 2005 22:11:10   oris
 * -	Changed comments
 * 
 *    Rev 1.1   Dec 23 2004 19:35:46   oris
 * Drop 2.1
 * 
 *    Rev 1.0   Dec 09 2004 18:23:42   oris
 * Initial revision.
 * 
 *    Rev 1.9   Oct 17 2004 15:10:06   OriS
 * Added TL_FAIL_ON_UNMARKED_BB flag to irFlags field. 
 * Not currently supported by 6.2.1 alpha1 version.
 * 
 *    Rev 1.8   Jan 27 2004 08:14:02   oris
 * Changed comments
 * 
 *    Rev 1.7   Oct 26 2003 19:06:24   oris
 * Added TL_DO_NOT_PERFORM_DOWNLOAD formatting flag
 * 
 *    Rev 1.6   Sep 30 2003 17:58:22   oris
 * Added TL_SINGLE_FLOOR definition for the format routine
 * 
 *    Rev 1.5   Sep 01 2003 19:07:46   oris
 * - Changed explanation of the various fields.
 * 
 *    Rev 1.4   Jun 01 2003 17:05:04   oris
 * Added paranthesis for TL_SPL_FORMAT and TL_FAST_PARTITION definition to simplefy the code.
 *  
 *    Rev 1.3   May 11 2003 08:26:32   OriS
 * Moved irFlags bits definition from blockdev.h.
 * Changed signOffset field to a more appropriate name (flags).
 * 
 *    Rev 1.2   May 08 2003 23:16:18   OriS
 * Changed TL_SIMPLE_MATCHING and TL_RUGGESIZED flags.
 * 
 *    Rev 1.1   May 06 2003 11:54:50   OriS
 * Changed Sdword variable type to FLSDword
 * 
 *    Rev 1.0   Apr 09 2003 12:16:00   OriS
 * Initial revision.
 * 
 */

/*****************************************************************************
* File Header                                                                *
* -----------                                                                *
* Name : flformat.h                                                          *
*                                                                            *
* Description : This file contains data strucutres passed to the format      *
*               routines.                                                    *
*                                                                            *
* Note : If dos format is also needed include dosformt.h instead             *
*                                                                            *
*****************************************************************************/

#ifndef FORMAT_H
#define FORMAT_H

#include "flbase.h"

/* IOreq->irFlags values for flFlashFormat routine:
 *
 * TL_NORMAL_FORMAT / TL_LEAVE_BINARY_AREA / TL_LEAVE_SOME_BINARY_AREA 
 *
 * TL_SINGLE_FLOOR / TL_DO_NOT_PERFORM_DOWNLOAD
 */

#define TL_LEAVE_BINARY_AREA       0x8  /* Is assumed to be a single bit */
#define TL_LEAVE_SOME_BINARY_AREA  0x9  /* Must include TL_LEAVE_BINARY_AREA bits */
#define TL_SINGLE_FLOOR            0x10 /* Format only a selection of the floors  */
#define TL_DO_NOT_PERFORM_DOWNLOAD 0x20 /* Do not issue a download operation at   */
                                        /* end of the format routine.             */
#define TL_FAIL_ON_UNMARKED_BB     0x80 /* Fail in case an unmarked bad block is  */
                                        /* Found.                                 */

/* Internal Partition flags - N O T   F O R   U S E */

#define TL_SIMPLE_MATCHING         0x10 /* Use simple matching for partition */
#define TL_FORCE_REL               0x20 /* Use RELIABLE mode                 */
#define TL_FAST_MODE               0x40 /* Use FAST mode                     */
#define TL_NO_PLANES               0x80 /* Do not match blocks for planes    */


/* External Disk Partition Flags:
 *
 * TL_NORMAL_FORMAT
 */

#define TL_NORMAL_FORMAT       0


/* External Binary Partition Flags 
 *
 * TL_NORMAL_FORMAT / TL_SPL_FORMAT 
 */

#define TL_SPL_FORMAT          (TL_SIMPLE_MATCHING | TL_NO_PLANES | TL_FORCE_REL) /* SPL partition */


/* External spacing factors - only 1 value
 *
 */

#define FL_NORMAL_SPACING      1

/*********************************************/
/* Formatting parameter structure definition */
/*********************************************/

/*----------------------------------------------*/
/* User BDTL Partition Format Parameters record */
/*----------------------------------------------*/

typedef struct {
FLDword   length;
        /* The size in bytes of the usable storage space. The size 
           rounded upwards to a multiplication of a block size.
           The size of the last partition will calculated automatically,
           but if the requested size is greater then the remaining space
           an error code will be returned,. Requesting zero size for any
      partition but the last will generate an flBadParameters status. */
FLDword   fastAreaLength;      
FLWord    lengthType;
FLWord    fastAreaLengthType;
FLWord    fastAreaVirtualFactor; /* Should be set to FL_NORMAL_SPACING */
FLWord    noOfSpareUnits;
        /* BDTL needs at least one spare erase unit in order to function
           as a read/write media. It is possible to specify more than one
           spare unit, which takes more media space. The advantage of
           specifying more than one spare unit is that if one of the flash
           erase units becomes bad and un-erasable in the future, then one
           of the spare units can replace it. In that case, a second spare
           unit enables TrueFFS to continue its read/write functionality,
           whereas if no second spare unit is available the media goes into
           read-only mode. The standard value used is 3 */

FLDword   BDTLFP_0_RFU_0; /* Reserved 0  */
FLDword   BDTLFP_1_RFU_0; /* Reserved 0  */
FLDword   BDTLFP_2_RFU_0; /* Reserved 0  */
FLDword   BDTLFP_3_RFU_0; /* Reserved 0  */

FLByte    protectionKey[8];   /* The key for the protection*/
FLByte    protectionType;
       /* PROTECTABLE          - Can receive protection           */
       /* READ_PROTECTED       - Protect against read operations  */
       /* WRITE_PROTECTED      - Protect against write operations */
       /* LOCK_ENABLED         - Enables the hardware lock signal */
       /* PROTECTABLE          - This partition can be protected  */
       /* CHANGEABLE_PROTECTION - protection type can be changed */
       /* The ratio between the real media size and the virtual size
          reported to the file system when compression is active. */
} BDTLPartitionFormatParams3;

/*------------------------------------------------*/
/* User binary Partition Format Parameters record */
/*------------------------------------------------*/

typedef struct {
FLDword length;      /* Required number of usable bytes (rounded to units) */
                     /* in the  partition.                                 */

FLWord  lengthType;  /* Must be FL_LENGTH_IN_BYTES                         */

FLByte  sign[4];     /* signature of the binary  partition to format. 
                        The signature 0xFFFF FFFF is not a valid signature */

FLDword flags;       /* TL_NORMAL_FORMAT / TL_SPL_FORMAT                   */

FLWord spareBlocks;  /* Spare blocks for DBB support                       */

FLDword   BDKFP_0_RFU_0; /* Reserved 0  */
FLDword   BDKFP_1_RFU_0; /* Reserved 0  */
FLDword   BDKFP_2_RFU_0; /* Reserved 0  */
FLDword   BDKFP_3_RFU_0; /* Reserved 0  */

/* TL_NORMAL_FORMAT / TL_SPL_FORMAT */

FLByte   protectionKey[8];   /* The key for the protection*/
FLByte   protectionType;
       /* PROTECTABLE          - Can receive protection           */
       /* READ_PROTECTED       - Protect against read operations  */
       /* WRITE_PROTECTED      - Protect against write operations */
       /* LOCK_ENABLED         - Enables the hardware lock signal */
       /* PROTECTABLE          - This partition can be protected  */
       /* CHANGEABLE_PROTECTION - protection type can be changed */
} BinaryPartitionFormatParams3;


/*-----------------------------------------------------------------*/
/* User Format Parameters record for flFlashFormat routine         */
/*-----------------------------------------------------------------*/

typedef FLStatus (*FLProgressCallback)(FLWord totalUnitsToFormat, FLWord totalUnitsFormattedSoFar);

typedef struct {

/*****************************/
/* Device formatting section */
/*****************************/

FLByte  percentUse;
       /* NAND flash inherently contains some bad blocks. TrueFFS handles 
	    * those blocks by managing a pool of spare blocks also called transfer
		* units. This format parameter specifies the percentage of the media 
		* to use.
        *
        * Starting from TrueFFS 6.3.0 alpha 4 this field is no longer a
        * configurable formatting argument. You must set this field to 
        * TL_DEFAULT_PERCENTAGE, letting TrueFFS to choose this value for you.
		*/

FLByte  noOfBDTLPartitions;
      /* Indicates the number of Disk partition (1 to 4). */

FLByte  noOfBinaryPartitions;
       /* Indicates the number of binary partitions (up to 3). 0 will
        * cause formatting with no binary partition. This value is ignored
        * if TL_LEAVE_BINARY_AREA flag is set in the irFlags field the ioreq 
		*/

BDTLPartitionFormatParams3   FAR2* BDTLPartitionInfo;
       /* BDTL partition information array  */

BinaryPartitionFormatParams3 FAR2* binaryPartitionInfo;
       /* Binary partition information array*/

/***********************************/
/* Special format features section */
/***********************************/

FLByte  cascadedDeviceNo;    /* Not used */

FLByte  noOfCascadedDevices; /* Not used */

FLProgressCallback progressCallback;
   /* Progress callback routine, will be called if not NULL.
      The callback routine is called after erasing each unit,
      and its parameters are the total number of erase units
      to format and the number erased so far.
      The callback routine returns a Status value. A value of
      OK (0) allows formatting to continue. Any other value
      will abort the formatting with the returned status code. */
   /* In case the FL_REPORT_MOUNT_PROGRESS compilation flag is set the
      routine will also report the mount progress of the mount routine
     for each of the Disk partitions. A 0,0 return values will report 
     the end of a format status and of a mount operation */

FLDword  FP_0_RFU_0;       /* Reserved 0 */

FLDword  FP_1_RFU_0;       /* Reserved 0 */

/* Note the following section is not used by for DiskOnChips */
/*************************************************************/

FLSDword bootImageLen;      /* Reserved 0 */

FLDword  vmAddressingLimit; /* Reserved 0 */

FLWord   embeddedCISlength; /* Reserved 0 */

FLByte   FAR1 * embeddedCIS; /* Reserved NULL */
} FormatParams3;

/*----------------------------------------------------------*/
/* Format Parameters record passed to the translation layer */
/*----------------------------------------------------------*/

typedef struct {

  /* Global device info */

  FLByte             percentUse;

  FLByte             noOfBDTLPartitions;

  FLByte             noOfBinaryPartitions;

  BDTLPartitionFormatParams3   FAR2* BDTLPartitionInfo;

  BinaryPartitionFormatParams3 FAR2* binaryPartitionInfo;

  FLDword            flags;

  /* First volume info */

  FLByte             protectionKey[8];

  FLByte             protectionType;

  FLByte             noOfSpareUnits;

  /* Special features */

  FLByte             cascadedDeviceNo;

  FLByte             noOfCascadedDevices;

  FLProgressCallback progressCallback;

  FLDword            osakVersion;

  /* None DiskOnChip parameters */

  FLSDword           bootImageLen;

  FLSDword           dataPartitionLen;

  FLDword            vmAddressingLimit;

  FLWord             embeddedCISlength;

  FLByte FAR1 *      embeddedCIS;

} TLFormatParams;

#define BINARY_SIGNATURE_NAME    4
#define BINARY_SIGNATURE_LEN     8
#define BINARY_SPARE_BLOCKS_SIGN "S4BB" /* Must be the same as BINARY_SPARE_BLOCKS_SIGN (FLFORMAT.H) */
/* #define SIGN_SPL       "" */

/* Standard initializer for BDTLPartitionFormatParams3 structure */

#define TL_DEFAULT_PERCENTAGE     0 /* Use default percentage of the media      */
#define TL_NO_BOOST               0 /* Do not use any performance boost         */ 
#define TL_BASIC_BOOST            1 /* Use basic 1% performance boost           */
#define TL_ENHANCED_PERFORMANCE 100 /* Define as enhanced performance partition */

#define TL_BDTL_STD_SPACE_FACTOR	FL_NORMAL_SPACING
#define TL_BDTL_STD_SPARE_UNITS_NO	3
#define STD_BDTL_PARAMS3 {0,                       /* Total  Length       */ \
                          TL_BASIC_BOOST,          /* Fast   Length       */ \
                          FL_LENGTH_IN_BYTES,      /* Total  Length type  */ \
                          FL_LENGTH_IN_PERCENTS,   /* Fast   Length type  */ \
                          TL_BDTL_STD_SPACE_FACTOR,/* Spaced Factor       */ \
                          TL_BDTL_STD_SPARE_UNITS_NO,/* No of spare units   */ \
                          0,0,0,0,                 /* Reserved 0          */ \
                          {0,0,0,0,0,0,0,0},0}     /* No H/W protection   */

/* Standard initializer for BinaryPartitionFormatParams3 structure */

#define TL_BDK_STD_SPARE_BLOCKS_NO	1
#define STD_BINARY_PARAMS3 {0,                  /* Partition length       */ \
                            FL_LENGTH_IN_BYTES, /* Partition length type  */ \
                            {'B','I','P','O'},  /* Signature              */ \
				            TL_SPL_FORMAT,      /* Partition types - SPL  */ \
							TL_BDK_STD_SPARE_BLOCKS_NO,/* Number of spare blocks */ \
                            0,0,0,0,            /* Reserved 0             */ \
							{0,0,0,0,0,0,0,0},  /* Protection key         */ \
							0}                  /* Protection type        */ \

/* Standard initializer for FormatParams3 structure */

#define STD_FORMAT_PARAMS3 {TL_DEFAULT_PERCENTAGE, /* Percent used               */ \
                            1,                     /* No of Disk Partitions      */ \
							0,                     /* No of Binary Partitions    */ \
							NULL,                  /* Disk partition structure   */ \
							NULL,                  /* Binary partition structure */ \
							0,                     /* This cascaded device       */ \
							0,                     /* Total cascaded devices     */ \
							NULL,                  /* Progress call back         */ \
							0, 0,                  /* Reserved 0                 */ \
							0, 0, 0, NULL}         /* Reserved 0                 */

/*------------------------------------------------------------------------------*/
/*                                                                              */
/*                           File System Format routines                        */
/*                                                                              */
/*------------------------------------------------------------------------------*/

/* describes 1 logical partition, array of such are given to flCreateLogicalPartitions routine */

typedef struct {
FLDword length;
FLWord lengthType;
FLByte pType;
FLByte flags;
} LogicalPartitionParams;

/* flag for LogicalPartitionParams */
#define FL_MARK_PARTITION_BOOTABLE	0x80

/* standard parameters to partition whole media (TL partition) as 1 logical partition */
#define STD_LOG_PARTITION_PARAMS	{0,                         /* Partition length       */ \
                                     FL_LENGTH_IN_BYTES,        /* Bytes, Sectors ...     */ \
									 DOS4_PARTIT,               /* Partition type         */ \
									 FL_MARK_PARTITION_BOOTABLE}/* Either bootable or not */ \

/*------------------------------------------------------------------------------*/

/* FAT Format Parameters record - given to flFormatFS */

typedef struct {
FLWord		noOfRootEntries;
FLWord		noOfSectorsPerCluster;

FLByte		noOfFATs;
FLByte		pType;
FLByte		reserved1;
FLByte		reserved2;

FLByte         volumeId[4];  /* FAT partition identification number */

FLByte FAR1 *  volumeLabel;  /* FAT partition label string. If NULL, no label */

} FATFormatParams;


#define FS_USE_DEFAULT_CLUSTER_SIZE	0

#define STD_FAT_FORMAT_PARAMS	{512,                         /* No of entries in the rood directory */ \
                                 FS_USE_DEFAULT_CLUSTER_SIZE, /* No of sectors per cluster           */ \
								 1,                           /* No of FAT copies                    */ \
								 0,                           /* Partition type                      */ \
								 0, 0,                        /* Reseved 0                           */ \
								 {0, 0, 0, 0},                /* FAT Volume ID                       */ \
								 NULL}                        /* FAT Lable                           */ \

/*------------------------------------------------------------------------------*/
/* used by flFindLogicalPartition */

typedef struct {
SectorNo bootSectorNo;
SectorNo sectorsInVolume;
SectorNo MBR;
int MBRslot;
} LogPartitionInfo;
/*------------------------------------------------------------------------------*/

#endif /* FORMAT_H */
