
/******************************************************************************* 
 *                                                                             * 
 *                        M-Systems Confidential                               * 
 *           Copyright (C) M-Systems Flash Disk Pioneers Ltd. 1995-2003        * 
 *                         All Rights Reserved                                 * 
 *                                                                             * 
 ******************************************************************************* 
 *                                                                             * 
 *                            NOTICE OF M-SYSTEMS OEM                          * 
 *                           SOFTWARE LICENSE AGREEMENT                        * 
 *                                                                             * 
 *      THE USE OF THIS SOFTWARE IS GOVERNED BY A SEPARATE LICENSE             * 
 *      AGREEMENT BETWEEN THE OEM AND M-SYSTEMS. REFER TO THAT AGREEMENT       * 
 *      FOR THE SPECIFIC TERMS AND CONDITIONS OF USE,                          * 
 *      OR CONTACT M-SYSTEMS FOR LICENSE ASSISTANCE:                           * 
 *      E-MAIL = info@m-sys.com                                                * 
 *                                                                             * 
 ******************************************************************************* 
 *                                                                             * 
 *                         Module: FLSYSTEM                                    * 
 *                                                                             * 
 *  This module implements VxWorks-to-TFFS bindings.                           *
 *                                                                             * 
 *******************************************************************************/




/*
 * $Log:   V:/PVCSDB/DiskOnChip/archives/version 6.x/TrueFFS/examples/drivers/VxWorks/system/flsystem.h-arc  $
 * 
 *    Rev 1.1   Aug 03 2004 20:24:22   andrayk
 * tffsRAMxxx macros removed
 * 
 *    Rev 1.0   12 Aug 2003 01:58:12   andrayk
 * Initial revision.
 */




#ifndef FLSYSTEM_H
#define FLSYSTEM_H


#ifdef __cplusplus
extern "C" {
#endif


/* VxWorks #includes */
#include <vxWorks.h>
#include <tickLib.h>
#include <sysLib.h>
#include <semLib.h>
#include <ioLib.h>
#include <errnoLib.h>
#include <taskLib.h>
#include <memLib.h>

/* ANSI C #includes */
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <time.h>
#include <ctype.h>


/*          CPU target
 *
 * Use compiler switches or insert a #pragma here to select the CPU type
 * you are targeting.
 *
 * If the target is an Intel 80386 or above, also uncomment the CPU_i386
 * definition.
 */

/* defined in Makefile */


/*          NULL constant
 *
 * Some compilers require a different definition for the NULL pointer
 */

/* using VxWorks default */


/*          Little-endian/big-endian
 *
 * If your machine uses the big-endian (Motorola) format, uncomment the
 * following line.
 *
 * This compilation flag makes sure that FAT and translation layer structures
 * will be written in little-endian (Intel) format for integers. It also 
 * affects the way DiskOnChip 16 bits registers are analized.
 */

/* using VxWorks #define _BYTE_ORDER here */

#ifndef _BYTE_ORDER
#error "byte order is not defined"
#else  /* _BYTE_ORDER */
#if (_BYTE_ORDER == _BIG_ENDIAN)
#define FL_BIG_ENDIAN
#else  /* (_BYTE_ORDER == _BIG_ENDIAN) */
#undef FL_BIG_ENDIAN
#endif /* (_BYTE_ORDER == _BIG_ENDIAN) */
#endif /* _BYTE_ORDER */


/*          Far pointers
 *
 * Specify here which pointers may be far, if any.
 * Far pointers are usually relevant only to 80x86 architectures.
 *
 * Specify FAR_LEVEL:
 *   0 -    if using a flat memory model or having no far pointers.
 *   1 -        if only the socket window may be far
 *   2 -    if only the socket window and caller's read/write buffers
 *      may be far.
 *   3 -    if socket window, caller's read/write buffers and the
 *      caller's I/O request packet may be far
 */

#define FAR_LEVEL  0


/* 
 *          Routines for 32-bit transfers to/from DiskOnChip I/O window
 */

#define tffscpy flcpy32
#define tffscmp flcmp32
#define tffsset flset32

extern void* flcpy32 (void *dest, const void *src, size_t count);       /* flsysvxw.c */
extern void* flset32 (void *dest, int val, size_t count);               /* flsysvxw.c */
extern int   flcmp32 (const void *dest, const void *src, size_t count); /* flsysvxw.c */


/*          Pointer arithmetic
 *
 * The following macros define machine- and compiler-dependent macros for
 * handling pointers to physical window addresses. The definitions below are
 * for PC real-mode Borland-C.
 *
 * 'physicalToPointer' translates a physical flat address to a (far) pointer.
 * Note that if when your processor uses virtual memory, the code should
 * map the physical address to virtual memory, and return a pointer to that
 * memory (the size parameter tells how much memory should be mapped).
 *
 * 'addToFarPointer' adds an increment to a pointer and returns a new
 * pointer. The increment may be as large as your window size. The code
 * below assumes that the increment may be larger than 64 KB and so performs
 * huge pointer arithmetic.
 */

#define physicalToPointer(physical,size,drive)  ((void *) (physical))
#define pointerToPhysical(ptr)                  ((unsigned long)(ptr))
#define addToFarPointer(base,increment)         ((void *) ((unsigned char *) (base) + (increment)))
#define freePointer(ptr,size)                   { /* not used */ }
#define subFromFarPointer(base,increment)		((void *) &(((unsigned char *)(base))[-increment]))
#define distanceBetweenFarPointers(ptr1, ptr2)	(((unsigned long)(ptr1)) - ((unsigned long)(ptr2)))


/*
 *          signed/unsigned char
 *
 * It is assumed that 'char' is signed. If this is not your compiler
 * default, use compiler switches, or insert a #pragma here to define this.
 *
 */

/* char is signed by default in GNU C */


/*          Mutex type
 *
 * If you intend to access the FLite API in a multi-tasking environment,
 * you may need to implement some resource management and mutual-exclusion
 * of FLite with mutex & semaphore services that are available to you. In
 * this case, define here the Mutex type you will use, and provide your own
 * implementation of the Mutex functions incustom.c
 *
 * By default, a Mutex is defined as a simple counter, and the Mutex
 * functions in custom.c implement locking and unlocking by incrementing
 * and decrementing the counter. This will work well on all single-tasking
 * environment, as well as on many multi-tasking environments.
 */

typedef SEM_ID FLMutex;


/*          Memory allocation
 *
 * The translation layers (e.g. FTL, NFTL, INFTL, SAFTL) need to allocate
 * memory to handle the Flash media. The required size depends on the media
 * being handled.
 *
 * You may choose to use the standard 'malloc' and 'free' to handle such
 * memory allocations, provide your own equivalent routines, or you may
 * choose not to define any memory allocation routine. In this case, the
 * memory will be allocated statically at compile-time on the assumption of
 * the largest media configuration you need to support. This is the simplest
 * choice, but may cause your RAM requirements to be larger than you
 * actually need.
 *
 * If you define routines other than malloc & free, they should have the
 * same parameters and return types as malloc & free. You should either code
 * these routines in flcustom.c or include them when you link your application.
 */

extern void* flmalloc (size_t nBytes);
extern void  flfree (void *p);

#define FL_MALLOC(bytes)  flmalloc((size_t)(bytes))
#define FL_FREE(p)        flfree((p))


/*          Debug mode
 *
 * Customize the various TrueFFS debug print macros, to suite your application needs. 
 */

typedef int  (*FLMsgHandler) (const char *str, ...);

extern FLMsgHandler  tffsMsgHandler;             /* pointer to TrueFFS message handler */

extern int  tffsMsgSave (const char *msg, ...);  /* default TrueFFS message handler */

extern int  tffsMsgErrZone;     /* active zones for TrueFFS error messages */ 
extern int  tffsMsgWarnZone;    /* active zones for TrueFFS warnings */ 
extern int  tffsMsgFlowZone;    /* active zones for TrueFFS control flow messages */ 
extern int  tffsMsgBufSize;     /* size of diagnostic buffer in KBytes */

#define DBG_PRINT_ERR(zone,str)       if(tffsMsgErrZone  & zone) tffsMsgHandler(str)
#define DBG_PRINT_ERR_PRM(zone,str)   if(tffsMsgErrZone  & zone) tffsMsgHandler str
#define DBG_PRINT_WRN(zone,str)       if(tffsMsgWarnZone & zone) tffsMsgHandler(str)
#define DBG_PRINT_WRN_PRM(zone,str)   if(tffsMsgWarnZone & zone) tffsMsgHandler str
#define DBG_PRINT_FLOW(zone,str)      if(tffsMsgFlowZone & zone) tffsMsgHandler(str)
#define DBG_PRINT_FLOW_PRM(zone,str)  if(tffsMsgFlowZone & zone) tffsMsgHandler str


/*
 *          First IOCTL function number
 *
 * When using TrueFFS' IOCTL functions you have to define the code of the first 
 * TrueFFS IOCTL function (after that the functions get consecutive increasing numbers).
 * This number should be out of the range of the standard IOCTL codes used by your 
 * operating system.
 *
 */

#define FL_IOCTL_START  0x9000


/*          Default calling convention
 *
 * C compilers usually use the C calling convention to routines (cdecl), but
 * often can also use the pascal calling convention, which is somewhat more
 * economical in code size. Some compilers also have specialized calling
 * conventions which may be suitable. Use compiler switches or insert a
 * #pragma here to select your favorite calling convention.
 */

/* using compiler's default calling convention */


/*          Naming convention
 *
 * In case the calling application uses a different convention then the one 
 * used to compile TrueFFS you can use the NAMING_CONVENTION definition. 
 * The NAMING_CONVENTION definition is added as a qualifier to all TrueFFS 
 * exported API. A good example is a c++ application that uses TrueFFS, 
 * which was compiled using standard C.If this is not the case simply leave 
 * the NAMING_CONVENTION defined as empty macro.
 */

#define NAMING_CONVENTION 


/*          System clean up routine
 *
 * This routine will be called every time TrueFFS exist. 
 * The implementation is found in flsysvxw.c
 */

extern void flSysfunExit(void);

#define FL_SYS_FUNC_RELEASE flSysfunExit()


/*          Control over the priority of the TrueFFS thread 
 *
 * Some of TrueFFS time is spend on polling the flash ready\busy signal.
 * TrueFFS supplies a verity of ways to utilize this time, one of which is 
 * to inform the OS that TrueFFS priority can be lowered for the time being.
 */

#define FL_LowerPriorityThread(flashPtr)  { /* not used */ }
#define FL_RaisePriorityThread(flashPtr)  { /* not used */ }


/*                                                                   
 * Uncomment the FL_NO_INIT_MMU_PAGES definition for:
 *                                                                        
 * In order to skip initialization of first and last byte of the given buffer.
 * When the user buffer resides on separated memory pages the read        
 * operation may cause a page fault. Some CPU's return from a page        
 * fault (after loading the new page) and reread the bytes that caused    
 * the page fault from the new loaded page. In order to prevent such a    
 * case the first and last bytes of the buffer are written.
 *                                                                        
 */

#define FL_NO_INIT_MMU_PAGES


/*          Improve BCH performance
 * 
 * You can improove the way TrueFFS utilizes 32bit (or larger) native variables
 * by uncommenting the defintion bellow.
 */

#define FL_ASSUME_NATIVE_IS_32BITS


/*          Remove older basic types.
 *
 * TrueFFS 5.x used the byte, word, dword, Sbyte, Sword and Sdword basic types.
 * Since these names are used by several major OSs, such as Windows and 
 * VxWorks, it was decided to rename these types with less general names 
 * (See the FLxxx types below). As many TrueFFS-based applications already
 * use these types, TrueFFS still automatically defines them. If these 
 * definitions cause your compiler to report redefinition errors, simple 
 * define FL_DISABLE_OLD_TRUEFFS_TYPES and TrueFFS will automatically remove
 * the definition. 
 */

#define FL_DISABLE_OLD_TRUEFFS_TYPES


/*          Alignment of structure members
 *
 *
 *  Pack all the structures for now.
 */

/* should be defined in Makefile */

extern void flDelayMsecs(unsigned  millisec);
#define FL_DELAY_MSEC(interval) flDelayMsecs(interval)

extern void flsleep(unsigned long  microsec);
#define FL_TFFS_SLEEP(handle,microseconds) flsleep(microseconds)

#ifdef __cplusplus
}
#endif

/************************/
/* DOCH Specific		*/
/************************/

/*PCI EVB specific*/
#define HIB_PCI_INTERRUPT_COUNTER   0x7FFC  /*Constant, No offset*/
#define HIB_PCI_CONTROL_REGISTER    0x7FFE  /*Constant, No offset*/

/*	PCI Memory windows	*/
typedef enum {
	PCI_MEM_WIN_0		=	0x0,	/*0..32KB*/
	PCI_MEM_WIN_1		=	0x1,	/*32.64KB*/
	PCI_MEM_WIN_2		=	0x2,	/*64..96KB*/
	PCI_MEM_WIN_3		=	0x3		/*96.1282KB*/
} DOCH_PCIMemoryWindow;

/*	PCI Window Size */
typedef enum {
	PCI_WIN_SIZE_8K		=	0x00,	/*Generate CS only for first 8KB*/
	PCI_WIN_SIZE_32K	=	0x40
} DOCH_PCIWinSize;

#define PCI_HW_RST			0x10	

/*	PCI Master/Slave */
typedef enum {
	PCI_MASTER			=	0x00,
	PCI_SLAVE			=	0x20
} DOCH_PCIMasterSlave;

/*	PCI Bus Access	*/
typedef enum {
	BUS_ACESS_8_BIT		=	0x80,
	BUS_ACESS_16_BIT	=	0x00
} DOCH_PCIBusAccess;

#define PCI_LOCK_SIGNAL		0x1000
#define PCI_IRQ_SIGNAL		0x2000
#define PCI_DMA_SIGNAL		0x4000
#define PCI_BUSY_SIGNAL		0x8000

void setWindowOffset(int using8KB, volatile unsigned char * baseAddr);
#define DOCH_SET_WINDOW_OFFSET(using8KB, baseAddress)	(setWindowOffset(using8KB, baseAddress))
#endif /* FLSYSTEM_H */



