/***********************************************************************************/
/*                        M-Systems Confidential                                   */
/*           Copyright (C) M-Systems Flash Disk Pioneers Ltd. 1995-2003            */
/*                         All Rights Reserved                                     */
/***********************************************************************************/
/*                            NOTICE OF M-SYSTEMS OEM                              */
/*                           SOFTWARE LICENSE AGREEMENT                            */
/*                                                                                 */
/*      THE USE OF THIS SOFTWARE IS GOVERNED BY A SEPARATE LICENSE                 */
/*      AGREEMENT BETWEEN THE OEM AND M-SYSTEMS. REFER TO THAT AGREEMENT           */
/*      FOR THE SPECIFIC TERMS AND CONDITIONS OF USE,                              */
/*      OR CONTACT M-SYSTEMS FOR LICENSE ASSISTANCE:                               */
/*      E-MAIL = info@m-sys.com                                                    */
/***********************************************************************************/

/*
 * $Log:   V:/PVCSDB/DiskOnChip/archives/Testing/TrueFFS 6.3/Drop 2.5/3/surefs/src/fsdefs.h-arc  $
 * 
 *    Rev 1.1   Jun 07 2005 06:23:40   OriS
 * -	Bug fix - Number of logical partition array was the size of FL_VOLUMES and not FS_VOLUMES.
 * 
 *    Rev 1.0   Mar 27 2005 09:26:52   oris
 * Initial revision.
 * 
 *    Rev 1.5   Mar 07 2005 22:15:52   oris
 * Comments
 * 
 *    Rev 1.4   Feb 19 2005 12:24:58   eugen
 * new field (cacheStartCluster) for faster cache flush
 * 
 *    Rev 1.3   Feb 01 2005 10:13:38   eugen
 * file cache support - new File fields
 * 
 *    Rev 1.2   Jan 05 2005 21:09:32   eugen
 * Commentaries, headers
 * 
 *    Rev 1.1   Dec 23 2004 19:35:54   oris
 * Drop 2.1
 * 
 *    Rev 1.0   Dec 09 2004 18:22:18   oris
 * Initial revision.
 * 
 *    Rev 1.3   Jan 14 2004 22:28:52   oris
 * Added new file flag - FL_FILE_MUST_CREATE - 2 - Create file even if found
 * 
 *    Rev 1.2   Sep 30 2003 17:55:56   oris
 * Added globalAttFlag field to Volume structure.
 * Added FL prefix to file flag definitions
 * 
 *    Rev 1.1   May 18 2003 13:28:32   OriS
 * Removed prototype of dismountVolume(Volume vol), initDone and setBusy. They are now used only by blockdev.c
 * 
 *    Rev 1.0   Apr 09 2003 12:15:58   OriS
 * Initial revision.
 * 
 */

#ifndef FSDEFS_H
#define FSDEFS_H

#include "fltl.h"
#include "sscache.h"

/* ----------------------------------------------------------------------------------- */
/*typedef*/ struct FSVolume {
  FLByte        flags;                  /* See description in flreq.h */
  FLByte	logPartition;

  SectorNo      bootSectorNo;           /* Sector no. of DOS boot sector */
  SectorNo	sectorsInVolume;

/* these are needed to implement replaceFATsector... do it in FS-through interface */
  SectorNo	firstFATSectorNo;       /* Sector no. of 1st FAT */
  SectorNo	secondFATSectorNo;      /* Sector no. of 2nd FAT */
  SectorNo	rootDirectorySectorNo;  /* Sector no. of root directory */
  SectorNo	firstDataSectorNo;      /* 1st cluster sector no. */
  unsigned      sectorsPerCluster;      /* Cluster size in sectors */
  unsigned      maxCluster[2];          /* highest cluster no. in each space */
  unsigned      bytesPerCluster;        /* Bytes per cluster */
  unsigned      numberOfFATS;           /* number of FAT copies */
  unsigned      sectorsPerFAT;          /* Sectors per FAT copy */
  unsigned      sectorsInRootDirectory; /* No. of sectors in root directory */
  unsigned      allocationRover[2];     /* rover pointers for allocation (for FAST and NORMAL spaces) */
  					/* if they are 0, it means that the corresponding space is full */
  unsigned	mountCount;

#ifdef FS_FILES
#ifndef FL_EXTENDED_SECTOR_CACHE
  FLBuffer      volBuffer;              /* Define a sector buffer */
#else
  FLSectorCache	volBuffer;              /* Define array of sector buffers */
#endif
#endif /* FS_FILES */

  TL            tl;                     /* Translation layer methods */

  FLBoolean     globalAttFlag;        /* indication of global attention flag */

};

/* ----------------------------------------------------------------------------------- */

#define FL_FIRST_FAST_VALID_CLUSTER		2
#define FL_FIRST_NORMAL_VALID_CLUSTER		(pVol->maxCluster[FL_FAST_SPACE]+1)

#define FL_LAST_FAST_VALID_CLUSTER		pVol->maxCluster[FL_FAST_SPACE]
#define FL_LAST_NORMAL_VALID_CLUSTER		pVol->maxCluster[FL_NORMAL_SPACE]
/* these are really valid ones, not + 1; it is important because + 1 may not fit in 16-bit
and last valid is numberOfClusters+1  */

#define FL_FIRST_VALID_CLUSTER			FL_FIRST_FAST_VALID_CLUSTER
#define FL_LAST_VALID_CLUSTER			FL_LAST_NORMAL_VALID_CLUSTER
/* these are across both spaces */
/* ----------------------------------------------------------------------------------- */

#ifdef FS_FILES
typedef struct {
  FLSDword      currentPosition;        /* current byte offset in file */
#define         ownerDirCluster currentPosition /* 1st cluster of owner directory */
  FLSDword      fileSize;               /* file size in bytes */

#ifdef FS_MM_PERFORMANCE_TUNEUPS
  FLDword	defragmentCloseVal;
#endif  /* FS_MM_PERFORMANCE_TUNEUPS */
  
  unsigned      currentCluster;         /* cluster of current position */
  unsigned      flags;                  /* See description below */

  SectorNo      directorySector;        /* sector of directory containing file */

#ifdef FS_FILE_SHARING_SUPPORT
  FLWord        writeHandle;		/* valid if less than FS_FILES; 0xFFFF means write-opened file is referenced */
#endif	/* FS_FILE_SHARING_SUPPORT */
  
  FLByte        directoryIndex;         /* entry no. in directory sector */
  FLByte	FSOnumber;		/* specific file group characteristics */
/* note that fsdb uses FLHandle, but here it is actually limited with Byte... OK? */

  FSVolume *    fileVol;                /* Drive of file */

#ifdef FS_SUPPORT_FILE_CACHE
  FLDword	cacheBufferStart;		/* from the beginning of the file */
  FLByte *	cacheBufferData;		/* NULL if there no file-specific cache */
  FLDword	cacheBufferLength;		/* 0 if no data is buffered */
  unsigned	cacheStartCluster;
#endif
} File;


/* File flag definitions */
#define FL_FILE_MUST_CREATE        2    /* Create file even if found */
#define FL_FILE_MODIFIED           4    /* File was modified */
#define FL_FILE_IS_OPEN            8    /* File entry is used */
#define FL_FILE_IS_DIRECTORY    0x10    /* File is a directory */
#define FL_FILE_IS_ROOT_DIR     0x20    /* File is root directory */
#define FL_FILE_READ_ONLY       0x40    /* Writes not allowed */
#define FL_FILE_MUST_OPEN       0x80    /* Create file if not found */

/* these flags can also be ORed with FSO flags !!.. they are unsigned, because irFlags are unsigned */

#ifdef FS_FILE_SHARING_SUPPORT


#define FL_WRITE_HANDLE_REFERENCED	0xFFFF

#define isSharedAndOpenForRead(_file_)			((_file_)->writeHandle < FS_FILES)
#define isSharedAndOpenForWrite(_file_)			((_file_)->writeHandle == FL_WRITE_HANDLE_REFERENCED)
#define setFileSharedAndOpenForRead(_file_, _handle_)	(_file_)->writeHandle = (_handle_)
#define setFileNotShared(_file_)			(_file_)->writeHandle = FS_FILES
#define setFileSharedAndOpenForWrite(_file_)		(_file_)->writeHandle = FL_WRITE_HANDLE_REFERENCED

#endif	/* FS_FILE_SHARING_SUPPORT */


#endif	/* FS_FILES > 0 */

/* ----------------------------------------------------------------------------------- */
extern FLMutex fsGlobalMutex;

extern FSVolume fsVols[FS_VOLUMES];
FLStatus dismountFS(FSVolume * pVol,FLStatus status);

#ifdef FS_FILES
void initFS(void);
#endif

/* bdCall links either with BD-stub implementation, or with BD directly */
#ifdef __cplusplus
extern "C"
{
#endif /* __cplusplus */
TFFS_DLL_API FLStatus NAMING_CONVENTION bdCall(FLFunctionNo functionNo, IOreq FAR2 *ioreq);
#ifdef __cplusplus
}
#endif /* __cplusplus */

/* in BD stub */
void setTL(FLByte socket, FLByte partition, FSVolume * pVol);
FLStatus lockForIO(FLByte socket, FLByte partition, FLBoolean onOff);

/* needed by BD stub in case of !FIXED_MEDIA */
FLStatus dismountAllFSVolumes(FLByte socket, FLByte partition, FLBoolean lockAndAbsDismount);

#endif
