/***********************************************************************************/
/*                        M-Systems Confidential                                   */
/*           Copyright (C) M-Systems Flash Disk Pioneers Ltd. 1995-2006            */
/*                         All Rights Reserved                                     */
/***********************************************************************************/
/*                            NOTICE OF M-SYSTEMS OEM                              */
/*                           SOFTWARE LICENSE AGREEMENT                            */
/*                                                                                 */
/*      THE USE OF THIS SOFTWARE IS GOVERNED BY A SEPARATE LICENSE                 */
/*      AGREEMENT BETWEEN THE OEM AND M-SYSTEMS. REFER TO THAT AGREEMENT           */
/*      FOR THE SPECIFIC TERMS AND CONDITIONS OF USE,                              */
/*      OR CONTACT M-SYSTEMS FOR LICENSE ASSISTANCE:                               */
/*      E-MAIL = info@m-sys.com                                                    */
/***********************************************************************************/

/*
 * $Log:   V:/PVCSDB/DiskOnChip/archives/Version 6.3.2/TrueFFS FS/src/g4_mtd.h-arc  $
 * 
 *    Rev 1.7   Feb 14 2006 11:09:42   DoronC
 * Update copyright note.
 * 
 *    Rev 1.6   Dec 25 2005 17:08:30   rafib
 * Add missing reliable SLC sequence.
 * 
 *    Rev 1.5   Dec 06 2005 07:55:10   DoronC
 * Add OTP feature for G4.
 * 
 *    Rev 1.4   Dec 04 2005 13:15:08   DoronC
 * Finalizing packages for trueFFS 7.0.0.
 * 
 *    Rev 1.3.1.0   Nov 14 2005 10:08:10   RonenG
 * Add FAST FRE# definition
 * 
 *    Rev 1.3   Oct 09 2005 19:09:52   roneng
 * Reduce timeout for P.C project
 * 
 *    Rev 1.2   Sep 20 2005 12:35:16   roneng
 * Remove definition supporting G4 Virtual IPL download
 * 
 *    Rev 1.1   Sep 19 2005 10:17:50   Tal.Heller
 * initial
 *
 */

/************************************************************************
 * Name    : G4_mtd.h													*
 *																		*
 * Written : Ronen Golan												*
 *																		*
 * Updated : Tal Heller                                                 *
 *                                                                      *
 * Abstract: This file contains the prototypes of the M1G MTD			*
 ************************************************************************/

#ifndef MTD_G4_H
#define MTD_G4_H


/***************** Intenral unused flags - DO NOT USE **********************/

/* No HW DMA Type #1 support in code */
#define NO_G4_HW_DMA_TYPE_1_SUPPORT

/***************************  Definitions    ****************************/
#define MTD_VARS         ((MTDVarsStruct *) flashPtr->mtdVars)
#define MTD_VARS_BUFFER  (MTD_VARS->buffer->flData)        

/****************************  Typedefs      ****************************/
#ifndef FL_IRQ_RB_INIT    
#define FL_IRQ_RB_INIT(wSockeNo)            FL_INT_RB_DISABLED
#endif

#ifndef FL_DPD_INIT
#define FL_DPD_INIT(wSockeNo)               FL_DPD_DISABLED
#endif

#ifndef FL_PAGE_MODE_INIT
#define FL_PAGE_MODE_INIT(wSockeNo)         FL_DISABLED
#endif

#ifndef FL_TURBO_MODE_INIT
#define FL_TURBO_MODE_INIT(wSockeNo)        FL_NONTURBO_MODE
#endif

#ifndef FL_DPS_2_COPIES_INIT
#define FL_DPS_2_COPIES_INIT(wSockeNo)      FL_DISABLED 
#endif

#ifndef FL_DMA_INIT
#define FL_DMA_INIT(wSockeNo)               FL_DMA_HW_DISABLED
#endif


/************************* Run Time Definitions *************************/
#define MAX_BANKS              4
#define PAGE_INFO_LENGTH       7
#define BCH_ECC_LENGTH         7 
#define HM_ECC_LENGTH          1 
#define FOUNDRY_DATA_SIZE      0x200  
#define FOUNDRY_RESERVED_SIZE  8



#ifndef G4_Write8BitReg
#define G4_Write8BitReg(flash,offset,val)                  flDirectWrite8BitReg(flash,offset,val)
#endif

#ifndef G4_DirectWrite16BitReg
#define G4_DirectWrite16BitReg(flash,offset,val)           flDirectWrite16BitReg(flash,offset,val)
#endif

#ifndef G4_DirectRead8BitReg
#define G4_DirectRead8BitReg(flash,offset)                 flDirectRead8BitReg(flash,offset)
#endif

#ifndef G4_DirectRead16BitReg
#define G4_DirectRead16BitReg(flash,offset)                flDirectRead16BitReg(flash,offset)
#endif

#ifndef G4_ReadEvenNumberOfBytes
#define G4_ReadEvenNumberOfBytes(flash,offset,dest,count)  flReadEvenNumberOfBytes(flash,offset,dest,count)
#endif

#ifndef G4_WriteEvenNumberOfBytes
#define G4_WriteEvenNumberOfBytes(flash,offset,src,count)  flWriteEvenNumberOfBytes(flash,offset,src,count)
#endif

#ifndef G4_Read512Bytes
#define G4_Read512Bytes(flash,offset,dest)                flRead512Bytes(flash,offset,dest)
#endif
#ifndef G4_Write512Bytes
#define G4_Write512Bytes(flash,offset,src)                flWrite512Bytes(flash,offset,src)
#endif


#ifndef G4_DocWindow
#define G4_DocWindow(flash)                               flDocWindow(flash)
#endif

/*************************    Alignment      *********************/

#ifndef G4_DOC_DMA_BUS_MASK
#define G4_DOC_DMA_BUS_MASK        3 /* Allign to 32 bits */
#endif



/*************************    Include files      *********************/

/***************************     MEMORY MAP       **********************/
#define G4_IPL_AREA_0              0x0000
#define G4_IPL_AREA_1              0x1800
#define G4_IPL_AREA_0_PAGE_1       0x0400

#ifndef FL_XSCALE_BOOT_MODE
#define G4_FLASH_DATA_REG_ALIASED  0x0800 
#else /* Do not read from 4 bytes aligned address */
#define G4_FLASH_DATA_REG_ALIASED  0x0802 /* Prevent access to 32bit alligned address */ 
#endif /* FL_XSCALE_BOOT_MODE */

#define G4_MEMORY_MAP_SIZE         0x2000    /* Size of memory window is - 8K */

/************************** Register Address Map ************************/
/**************************   DOC CONTROL REGS   ************************/
#define G4_PAGE_COMMAND_REG           0x0030  /* WRITE      */
#define G4_PAGE_SELECT_REG            0x0070  /* WRITE      */
#define G4_CHIP_ID_REG                0x1000  /* READ       */ 
#define G4_CHIP_ID_CONFIRM_REG        0x1074  /* READ+WRITE */
#define G4_NOP_REG                    0x103E  /* READ+WRITE */ 
#define G4_ALIAS_RESOLUTION_REG       0x1004  /* READ+WRITE */ 
#define G4_BUS_LOCK_REGISTER          0x1006  /* READ+WRITE */ 
#define G4_BUS_SWAP_REG               0x1008  /* READ+WRITE */ 
#define G4_DEVICE_ID_SELECT_REG       0x100A  /* READ+WRITE */ 
#define G4_DOC_CONTROL_REG            0x100C  /* READ+WRITE */ 
#define G4_DOC_CONTROL_CONFIRM_REG    0x1072  /* READ+WRITE */ 
#define G4_CONFIGURATION_REG          0x100E  /* READ+WRITE */ 
#define G4_INTERRUPT_CONTROL_REG      0x1010  /* READ+WRITE */ 
#define G4_FOUNDRY_TEST_REG           0x1076  /* READ+WRITE */ 
#define G4_OUTPUT_CONTROL_REG         0x1014  /* READ+WRITE */ 
#define G4_DPD_CONTROL_REG            0x107C  /* READ+WRITE */  
#define G4_BURST_CONTROL_REG          0x101C  /* READ+WRITE */
#define G4_INTERRUPT_STATUS_REG       0x1020  /* READ+WRITE */
#define G4_FIRST_GOOD_UNIT_REG        0x1022  /* READ+WRITE */
#define G4_DMA_CONTROL_REG_0          0x1078  /* READ+WRITE */
#define G4_DMA_CONTROL_REG_1          0x107A  /* READ+WRITE */    
#define G4_LAST_DOWNLOAD_RAM_PAGE     0x1028  /* READ       */   
#define G4_VIRTUAL_RAM_STATUS_PAGE    0x1024  /* READ       */   
#define G4_INTERFACE_PAGE_REG         0x1012  /* READ+WRITE */ 

#define G4_OUTPUT_PIPELINE_REG		   0x1050
#define G4_FLASH_DATA_BYTE_REG		   0x1052

/**************************   FLASH REGISTERS  **************************/ 
#define G4_FLASH_CONTROL_REG          0x1038  /* READ+WRITE */ 
#define G4_FLASH_SEQUENCE_REG         0x1032  /* READ+WRITE */ 
#define G4_FLASH_COMMAND_REG          0x1034  /* READ+WRITE */ 
#define G4_FLASH_ADDRESS_REG          0x1036  /* READ+WRITE */ 
#define G4_FLASH_DATA_REG             0x103C  /* READ+WRITE */ 
#define G4_FLASH_END_OF_DATA_REG      0x101E  /* READ+WRITE */
/**************************    ECC REGISTERS   **************************/
#define G4_FLASH_BCH_PARITY_REG       0x1048  /* READ+WRITE */ 
#define G4_HAMMING_PARITY_REG         0x1046  /* READ       */ 
#define G4_ECC_PRESET_REG             0x1044  /* READ+WRITE */ 
#define G4_ECC_CONTROL_REG_0          0x1040  /* READ+WRITE */ 
#define G4_ECC_CONTROL_REG_1          0x1042  /* READ+WRITE */ 
/************************** PROTECTION REGISTERS **************************/
#define G4_PROTECTION_STATUS_REG      0x1056  /* READ       */ 
#define G4_DPS0_ADDRESS_LOW_REG       0x1060  /* READ       */ 
#define G4_DPS0_ADDRESS_HIGH_REG      0x1062  /* READ       */ 
#define G4_DPS1_ADDRESS_LOW_REG       0x1064  /* READ       */ 
#define G4_DPS1_ADDRESS_HIGH_REG      0x1066  /* READ       */ 
#define G4_DPS0_STATUS_REG            0x106c  /* READ       */ 
#define G4_DPS1_STATUS_REG            0x106e  /* READ       */  
#define G4_DPS_0_KEY_REG              0x105c  /* WRITE 	 */ 
#define G4_DPS_1_KEY_REG              0x105e  /* WRITE      */
/**************************  DOWNLOAD REGISTERS  **************************/
#define G4_DOWNLOAD_STATUS_REG        0x1070  /* READ+WRITE */ 
/**************************  M_COTP   REGISTERS  **************************/
#define G4_COTP_STATUS_REG            0x1058  /* READ       */


#define G4_COTP_STS_DOWNLOAD_REG	   0x0078  /* READ       */
#define G4_COTP_SELECT_REG		       0x007C  /* READ       */
#define G4_UNIQE_ID_DOWNLOAD_REG      0x0080  /* READ       */


/**************************************************************************/

/**********************************************************************************
 *****************************   Register Description   ***************************
 **********************************************************************************/

/*****************************   G4_BUS_SWAP_REG       **************************/
#define G4_SWAP_L                      0x0001
#define G4_SWAP_H                      0x0100 
#define G4_SWAP_DATA_BYTES             (G4_SWAP_H | G4_SWAP_L)

/*****************************   G4_DOC_CONTROL_REG    **************************/
#define G4_MODE_BITS                   0x03
#define G4_MODE_WRITE_ENABLE           0x04
#define G4_BDETCT_RESET                0x08  
#define G4_RSTIN_RESET                 0x10
#define G4_RAM_WE                      0x20

/*****************************  G4_CONFIGURATION_REG   **************************/
#define G4_VDET_OUTPUT                 0x01
#define G4_BOOT_DETECTOR_INHIBIT       0x08
#define G4_MAX_ID_BITS                 0x30
#define G4_IF_CFG_STATUS               0x80

/***************************** G4_INTERRUPT_CONTROL_REG *************************/
#define G4_IRQ_ENABLED                 0x8000
#define G4_IRQ_EDGE_SENSITIVE          0x4000
#define G4_IRQ_ENABLE_DMA_INT          0x2000
#define G4_IRQ_ENABLE_BCH_ECC_INT      0x0800
#define G4_IRQ_ENABLE_DATA_OVF_INT     0x0400
#define G4_IRQ_ENABLE_PROTECTION_INT   0x0200
#define G4_IRQ_ENABLE_FREADY_INT       0x0100
#define G4_IRQ_MASK_DMA_INT            0x0020
#define G4_IRQ_MASK_BCH_ECC_INT        0x0008
#define G4_IRQ_MASK_DATA_OVF_INT       0x0004
#define G4_IRQ_MASK_PROTECTION_INT     0x0002
#define G4_IRQ_MASK_FREADY_INT         0x0001

/*****************************  G4_INTERRUPT_STATUS_REG  *************************/
#define G4_IRQ_DMA_PENDING_INT          0x20
#define G4_IRQ_BCH_PENDING_ECC_INT      0x08
#define G4_IRQ_DATA_OVF_PENDING_INT     0x04
#define G4_IRQ_PROTECTION_PENDING_INT   0x02
#define G4_IRQ_FREADY_PENDING_INT       0x01
  
/*****************************    G4_FOUNDRY_TEST_REG    *************************/
#define G4_SOFT_RESET                  0x80
#define G4_WR_ALL_PIN                  0x08   
#define G4_TEST_A_MODE                 0x04   /* In this mode the boot detector will be incremented by 11H instead of 01H */
#define G4_DOWNLOAD_TEST_MODE          0x02   /* Download test mode - data is downloaded from D[7:0] instead of the flash */
#define G4_FLASH_TEST_MODE             0x01   /* When set provides all the pin signals on the I/O pins */

/*****************************    G4_OUTPUT_CONTROL_REG  *************************/
#define G4_ASSERT_BUSY_DURING_DNLD     0x01
#define G4_DISABLE_PULL_UP_BITS        0x02
#define G4_TURBO_MODE                  0x04
#define G4_PAGE_READ_MODE              0x10 

/*******************************  G4_DPD_CONTROL_REG     *************************/
#define G4_POWER_DOWN_OK               0x80
#define G4_POWER_DOWN_MODE_BITS        0x0F

/*******************************    G4_DMA_CONTROL_REG_0     *********************/
#define G4_DMA_ENABLE                  0x8000l
#define G4_DMA_PAUSE                   0x4000l
#define G4_DMA_EDGE_MODE               0x2000l
#define G4_DMA_DMARQ_POLARITY          0x1000l
#define G4_DMA_NO_ECC                  0x0800l
#define G4_SECTOR_COUNT_BITS           0x01FFul
  
/*******************************    G4_DMA_CONTROL_REG_1     *********************/
#define G4_NEGATE_COUNT_BITS           0x0FFF

/******************************* G4_BURST_MODE_CONTROL_REG   *********************/
#define G4_BURST_ENABLE                0x0001
#define G4_BURST_CLOCK_INVERT          0x0002
#define G4_BURST_HOLD                  0x0004
#define G4_FIFO_MODE                   0x0008
#define G4_EBRA                        0x0010
#define G4_LATP1                       0x0020
#define G4_BURST_LATENCY_BITS          0x0F00
#define G4_BURST_LENGTH_BITS           0xF000


/*******************************   G4_FLASH_CONTROL_REG  **************************/
#define G4_FWP_SIGNAL_ACTIVE           0x20   /* After protocol violation detected we must clear this bit */
#define G4_FCE_SIGNAL_ACTIVE           0x10   /* Controls the assertion of the CE# to the BANK defined by the BANK bits */ 
#define G4_SEQ_STATE_MACHINE_READY     0x08
#define G4_PROTECTION_ERROR_DETECTED   0x04   /* Protection Error & will be cleared after FLASH reset command */
#define G4_SEQUENCE_ERROR_DETECTED     0x02   /* Sequence Error detected & will be cleared only by FLASH reset command */
#define G4_FLASH_READY                 0x01   /* FLASH READY/BUSY# output signal */
#define G4_FLASH_FAST_FRE              0x40   /* Reduced FRE# oulse generator  */

/*******************************   G4_ECC_CONTROL_REG_0  ************************/
#define G4_FLASH_READ_MODE             0x8000 
#define G4_AUTO_ECC_READ_WRITE_EN      0x4000     
#define G4_HAMMING_EN                  0x2000
#define G4_BCH_EN                      0x1000     
#define G4_DATA_COUNTER_BITS           0x0FFF    
#define G4_ECC_0_READ_DATA_EXTRA       0xB20F
#define G4_ECC_0_READ_DATA_EXTRA_NOECC 0x820F
#define G4_ECC_0_READ_EXTRA_ECC        0xA008
#define G4_ECC_0_READ_HW_DMA		    0x9210

#define G4_ECC_0_WRITE_DATA_EXTRA       0x320F
#define G4_ECC_0_WRITE_DATA_EXTRA_NOECC 0x020F

/*******************************   G4_ECC_CONTROL_REG_1  ************************/
#define G4_HM_DATA_NUM_BITS            0x0F
#define G4_PAGE_READ_IS_USED           0x20
#define G4_IGNORE_DATA                 0x40
#define G4_BCH_SYNDROM_ERROR           0x80                 

/****************************** G4_PROTECTION_STATUS_REG *************************/
#define G4_FOUNDRY_OTP_LOCKED          0x01
#define G4_CUSTOMER_OTP_LOCKED         0x02
#define G4_LOCK_INPUT_NEGATED          0x04
#define G4_STICKY_LOCK_ACTIVATED       0x08
#define G4_PROTECTION_ENABLED          0x10
#define G4_DONT_DOWNLOAD_IPL           0x20
#define G4_PROTECTION_ERROR            0x80

/*******************************         DPS0_STATUS_REG     *************************/
#define G4_OTP_PROTECTED               0x01
#define G4_READ_PROTECTED              0x02 
#define G4_WRITE_PROTECTED             0x04
#define G4_HW_LOCK_ENABLED             0x08
#define G4_KEY_OK                      0x80

/*******************************  G4_FIRST_GOOD_UNIT_REG   *************************/
#define G4_GOOD_UNIT_WRITE_EN          0x80
#define G4_GOOD_UNIT_NUM_BITS          0x7F

/*******************************   FLASH_TYPE      *****************************/
#define G4_SLC_TYPE                   0x1
#define G4_MLC_TYPE                   0x2
#define G4_ERASE_MARK                 0xAA

/*******************************  G4_DOWNLOAD_STATUS_REG   *************************/
#define G4_DPS0_DOWNLOAD_ERROR_BITS    0x03
#define G4_DPS0_DOWNLOAD_REDUNDANT     0x01
#define G4_DPS1_DOWNLOAD_ERROR_BITS    0x0C
#define G4_DPS1_DOWNLOAD_REDUNDANT     0x04
#define G4_IPL_DOWNLOAD_ERROR_BITS     0x30
#define G4_IPL_DOWNLOAD_REDUNDANT      0x10
#define G4_OTP_DOWNLOAD_ERROR          0x40 
#define G4_DOWNLOAD_ERROR_BITS         0x4A
#define G4_DOWNLOAD_IN_PROGRESS        0x80

/****************************** G4_VIRTUAL_RAM_STATUS_PAGE  ************************/
#define G4_VIRTUAL_RAM_SUPPORTED       0x80
#define G4_VIRTUAL_RAM_ENABLED         0x01 
#define G4_ALTERNATE_MAP               0x02 
#define G4_QUAD_RED_BIT                0x04
#define G4_VIRTUAL_RAM_DISABLE         0x08 
#define G4_PAGED_RAM_FIXED_1K_LOW      0x00
#define G4_PAGED_RAM_FIXED_1K_HIGH     0x01
#define G4_VIRTUAL_RAM_LOADED_BOTTOM   0x02
#define G4_VIRTUAL_RAM_LOADED_TOP      0x03
#define G4_IPL_MODE_BITS               0x07
   
/****************************** G4_PAGE_SELECT_REG           ***********************/
#define G4_PAGE_SELECT_SEQ             0x0080
#define G4_PAGE_SELECT_DIRECT          0x0000
#define G4_PAGE_SELECT_BITS            0x017F
#define G4_IPL_PAGE_SEL_BIT6_1         0x007F
#define G4_IPL_PAGE_SEL_BIT7           0x0080

/****************************** G4_LAST_DOWNLOAD_RAM_PAGE    ***********************/
#define G4_NEXT_UNIT_MASK_BITS         0xFF80
#define G4_NEXT_PAGE_MASK_BITS         0x007F

/****************************** G4_PAGE_COMMAND_REG          ***********************/
#define G4_PAGE_CMD_READ_NEXT          0x71

/****************************** G4_COTP_STATUS_REG           ***********************/
#define G4_COTP_STATUS_REG_BITS        0x1F  

#define G4_DPS_REVISION_NUMBER         0x10 

/*******************************        ENUMERATIONS         **************************/

/******************************* Flash Parameters  *****************************/
#define G4_SECTOR_BCH_PROTECTED  SECTOR_DATA_SIZE+SECTOR_PAGEINFO_SIZE+SECTOR_HM_SIZE
#define G4_SECTOR_HM_PROTECTED   SECTOR_PAGEINFO_SIZE    

/******************************* Flash Bank interface  *****************************/
#define G4_ONE_FLASH_8_BITS      0x00	 /* Only one 8 bit flash is connected  */
#define G4_ONE_FLASH_16_BITS     0x01   /* only one 16 bit flash is connected */
#define G4_TWO_FLASHES_CASCADED  0x02   /* Two 8 bit flashes cascaded         */
#define G4_TWO_FLASHES_SEPERATED 0x03   /* Two 8 bit flashes connected to the */
                                         /* same bank but the distinction is   */
                                         /* by the BL bit                      */

/*******************************    Flash Status Reg   *****************************/
#define G4_OPERATION_OK          0x00
#define G4_OPERATION_FAILED      0x01   

/*******************************     IRQ FLAGS        *****************************/
#define G4_NO_IRQ_TRIGGER             0x0
#define G4_IRQ_ENABLE                 0x1
#define G4_IRQ_FIFO_ERROR             0x2
#define G4_IRQ_PROTECTION_ERROR       0x4
#define G4_IRQ_FREADY_TRIGGER         0x8
#define G4_IRQ_CLEAR_IRQ_FREADY       0x10
#define G4_IRQ_CLEAR_IRQ_PROTECTION   0x20
#define G4_IRQ_CLEAR_IRQ_FIFO         0x40
     
/*******************************     ECC FLAGS         *****************************/
#define G4_NO_ECC                     0x0
#define G4_BCH_ENABLE                 0x2
#define G4_HM_ENABLE                  0x4
#define G4_AUTO_PARITY                0x8

/*******************************     INTLV_FLAG       *****************************/
#define G4_INTLV_1_8_BIT              0x01
#define G4_INTLV_1_16_BIT             0x02    /* Currently not supported */
#define G4_INTLV_2_CASCADED           0x04    /* Two 8 bits flash are cascaded to create one virtual 16 bit flash */     
#define G4_INTLV_2_SEPERATED          0x08    /* Two seperate 8 bit flash exists in each bank */

/*******************************     DOWNLOAD FLAGS   *****************************/
#define G4_COPY_0                     0x0
#define G4_COPY_1                     0x1
#define G4_COPY_2                     0x2
#define G4_COPY_3                     0x3
#define G4_FIRST_GOOD_COPY            0x10 
#define G4_ALL_COPIES                 0x20
#define G4_QUAD_COPY                  0x80

/*******************************    LOCK FLAG         *****************************/
#define G4_LOCKED	                    1
#define G4_NOT_LOCKED                 0

/*******************************  PROTECTION FLAG     *****************************/
#define G4_PROTECTED                  1
#define G4_NOT_PROTECTED              0

/*******************************  CORRUPTION          *****************************/ 
#define G4_COPY_GOOD                  0x0
#define G4_COPY_CORRUPTED             0x1

/*******************************  PROTECTION FLAGS    *****************************/ 
#define G4_NO_PROTECTION              0x0 
#define G4_WP                         0x1
#define G4_RP                         0x2
#define G4_HW_LOCK                    0x4
/*******************************    DOWNLOAD FLAGS    *****************************/
#define G4_DOWNLOAD_OK                0x0
#define G4_DOWNLOAD_ERROR             0x1  
#define G4_DOWNLOAD_MAIN_UNIT_ERROR   0x2

/*******************************   ERASE_FLAGS        *****************************/
#define G4_WAIT_FOR_READY             0x1

/**************************** wFlags definition ***********************************/
#define FL_DPS_2_COPIES_BIT              0x1
#define FL_PAGE_MODE_FLAG_BIT            0x2
#define FL_ALT_MAP                       0x4
#define HW_DMA_ENABLED                   0x8

/**************************** G4 - DMA definition ***********************************/
#define G4_NO_DMA			             0x0
#define G4_DMA_TYPE_1		             0x1
#define G4_DMA_TYPE_2		             0x2

/*******************************   Micellencious      *****************************/
#define G4_CHIP_ID                    0x0400   
#define G4_CHIP_ID_COMPLEMENT         0xFBFF
#define M256_CHIP_ID                   0x0100   
#define M256_CHIP_ID_COMPLEMENT        0xFEFF
#define G4_CHIP_ID                    0x0400   
#define G4_CHIP_ID_COMPLEMENT         0xFBFF
#define G4_KEY_LENGTH                 8

#ifndef FL_SIMULATE_FAILURES
#define G4_BUSY_DELAY                 400000L
#else
#define G4_BUSY_DELAY                 40000L
#endif

#define G4_PD_DELAY                   30000L
#define G4_DOWNLOAD_DELAY             400000L  
#define G4_FIFO_DELAY                 30000L
#define G4_ICMU_DELAY                 30000L
#define PAGED_RAM_TIMEOUT              300000L
#define G4_ALIAS_RESOLUTION           17
#define G4_UNIQUE_ID_LENGTH           16      
#define G4_FOUNDRY_OTP_SIZE           16               
#define G4_DEFAULT_START_ADR          0xC8000L
#define G4_DEFAULT_STOP_ADR           0xE0000L
#define G4_OTP_LOCK_MARK              0
#define G4_OTP_START_PAGE             8
#define G4_OTP_LOCK_PAGE              0x3E
#define G4_CUSTOMER_OTP_SIZE          15L*1024L
#define G4_BITS_IN_BYTE               8
#define G4_CHANGEABLE_PROTECTED_AREA  2
#define G4_TOTAL_PROTECTED_AREA       2
#define G4_BBT_BLOCKS_PERPAGE_SHIFT   12
#define G4_BBT_BLOCKS_PERPAGE_MASK    0xfff
#define G4_MLC_BB_PERCENTAGE          5
#define G4_NUM_OF_PLANES              1 
#define G4_BBT_PAGE                   4
#define G4_DPS_0_MAIN_UNIT            1
#define G4_DPS_0_REDUNDANT_UNIT       2
#define G4_DPS_1_MAIN_UNIT            3
#define G4_DPS_1_REDUNDANT_UNIT       4
#define G4_FIRST_IPL_PAGE_IN_UNIT     4
#define G4_RAM_READS_TO_EXIT_DPD      14 /*** Fits also turbo mode ***/
#define OP_COMPLETED			       0x0 
#define G4_MLC_BAD_BLOCK_PERCENTAGE   5
#define G4_FOTP_SIZE                  0x20
#define G4_PRODUCT_TYPE_LOCATION      2 
#define G4_MAX_NUMBER_OF_PLANES       1
#define G4_IPL_1G_SIZE                (246L*1024L)
#define G4_IPL_HIGHER_FLOORS          2048
#define G4_IPL_RAM_SIZE	           2048
#define FL_AREA_A					   0
#define FL_AREA_C					   0x50
#define G4_IPL_MAX_NUM_OF_PAGES_IN_UNIT 62


#define FL_MLC_MODE                    0x1
#define FL_SLC_MODE_READ_ERASE         0x2
#define FL_SLC_REL_MODE_PROGRAM        0x4
#define FL_SLC_FAST_MODE_PROGRAM       0x8

#define FL_MLC_FAST_MODE_READ          0x10
#define FL_MLC_FAST_MODE_PROGRAM       0x20


#define G4_IRQ_LEVEL_TYPE             0x0 
#define G4_IRQ_EDGE_TYPE              0x4000 
#define G4_IRQ_GLOBAL_INT_ENABLE      0x8000

#define G4_IRQ_RB_EN                  0x0001
#define G4_IRQ_PROTECTION_ERR_EN      0x0002
#define G4_IRQ_DCNT_OVF_ERR_EN        0x0004
#define G4_IRQ_BCH_ERR_EN             0x0008
/*#define G4_IRQ_RTC_EN               0x0010*/
#define G4_IRQ_DMA_COMPLETE_EN        0x0020

#define G4_IRQ_RB_UNMASK              0x0100
#define G4_IRQ_PROTECTION_ERR_UNMASK  0x0200
#define G4_IRQ_DCNT_OVF_ERR_UNMASK    0x0400
#define G4_IRQ_BCH_ERR_UNMASK         0x0800
#define G4_IRQ_RTC_UNMASK             0x0100
#define G4_IRQ_DMA_COMPLETE_UNMASK    0x0200

/*** MDOC1G-G4 read access time          ***/
/* Should be updated */
#define G4_READ_ACCESS_TIME_SRAM      104
#define G4_READ_ACCESS_TIME_NORMAL    56
#define G4_READ_ACCESS_TIME_PAGED     77 

#define G4_SUBTYPE_G4                 0x00
/*#define G4_SUBTYPE_G4_LP            0x01*/

#define G4_VIRTUAL_RAM_MAX_SIZE       8192   /* 8KB  */
#define G4_PAGED_RAM_MAX_SIZE         251904  /* 246KB for Dual red. */

#define G4_READ_PAGED_RAM_DELAY       20
#define G4_VIRTUAL_RAM_NOPS           1000

#define G4_ALL_FLOORS                 5  


#define G4_PAGES_PER_BLOCK_BITS       7

#define G4_SECTORS_IN_PAGE_BITS       2
#define G4_SECTORS_IN_PAGE            (1<<G4_SECTORS_IN_PAGE_BITS)

#define G4_SECTORS_IN_UNIT_BITS       9
#define G4_SECTORS_IN_UNIT            (1<<G4_SECTORS_IN_UNIT_BITS)

#define G4_UNITS_IN_FLOOR_BITS        9 /*** (2^9=512 Blocks) ***/
#define G4_UNITS_IN_FLOOR             (1<<G4_UNIT_BITS)

#define G4_SECTORS_IN_FLOOR_BITS      (G4_SECTORS_IN_UNIT_BITS+G4_UNITS_IN_FLOOR_BITS)
#define G4_SECTORS_IN_FLOOR           (1<<G4_SECTORS_IN_FLOOR_BITS)

#define G4_MCOTP_PAGES                31
#define G4_MCOTP_PAGE_SIZE            511



#define G4_MAX_DOWNLOAD_TIME_MSEC     20      

#define G4_NOPS_EXIT_BURST			   19	

#define G4_DPS_SIZE                   16  

/**********************************************************************************/

/***************************       Enumarations     ********************************/    
typedef enum FL_DPS_IndexEnum
{ 
  DPS_0 = 0,
  DPS_1 = 1
} FL_DPS_IndexE ;

typedef enum FL_AsicModeEnum 
{
  RESET_MODE      = 0,
  NORMAL_MODE     = 1,
  POWER_DOWN_MODE = 2 
} FL_AsicModeE;

typedef enum FL_EccModeEnum
{
  ECC_WRITE_MODE = 0,
  ECC_READ_MODE  = 1	  
}FL_EccModeE;

typedef enum FL_IrqTriggerEnum
{
  IRQ_PROTECTION ,
  IRQ_FREADY     ,
  IRQ_FIFO_ERROR ,
  IRQ_NOT_PENDING 
} FL_IrqTriggerE;

typedef enum FL_IrqTypeEnum
{
  LEVEL_TYPE_INT = 0,
  EDGE_TYPE_INT  = 1
} FL_IrqTypeE;

typedef enum FL_BusConfigEnum
{            
  SRAM_MODE = 0 ,
  MUX_MODE  = 1 ,
  LPC_MODE  = 2 ,
  ICP_MODE  = 3 
} FL_BusConfigE;

typedef enum FL_G4_FlashCommandsEnum
{
  /***   Read commands            ***/
  READ_AREA_1_CMD              = 0x00,
  READ_AREA_2_CMD              = 0x30, 
  READ_CACHE_NOT_LAST_PAGE_CMD = 0x31,
  READ_CACHE_LAST_PAGE_CMD     = 0x3F, 
  READ_COLUMN_CHANGE_1_CMD     = 0x05,      
  READ_COLUMN_CHANGE_2_CMD     = 0xE0,      
  /***   Output commands          ***/
  READ_STATUS_CMD              = 0x70,   
  /***   Program commands         ***/
  PAGE_PROG_CYCLE_1_CMD        = 0x80, 
  PAGE_PROGRAM_CONFIRM_CMD     = 0x10, 
  CACHE_PROGRAM_CONFIRM_CMD    = 0x15, 
  PAGE_PROG_CHANGE_COLUMN_CMD  = 0x85,
  /***  Erase commands            ***/
  BLOCK_ADDRESS_INPUT          = 0x60,
  PAGE_ERASE_CYCLE_2_CMD       = 0xD0,
  /*** Other commands             ***/
  RESET_COMMAND                = 0xFF,
  /*** Mode commands              ***/
  SET_SLC_READ_ERASE_CMD       = 0xA3,
  SET_SLC_PGM_CMD              = 0xA4,
  SET_MLC_FAST_READ_CMD        = 0xB3,
  SET_MLC_FAST_PGM_CMD         = 0xB4,
  SET_SLC_REL_PGM_CMD          = 0x22
}FL_G4_FlashCommandsE ;

typedef enum FL_ResetTriggerEnum
{
  BOOT_DETECTOR_TRIGGER,
  RSTIN_TRIGGER, 
  NOT_KNOWN_TRIGGER
} FL_ResetTriggerE ; 

typedef enum FL_G4_SequenceID_Enum
{
  ERASE_SEQ                      = 36, 
  PAGE_PROGRAM_SEQ               = 22, 
  PAGE_PROGRAM_CONFIRM_SEQ       = 30, 
  CACHE_PROGRAM_CONFIRM_SEQ      = 33, 
  PAGE_PROGRAM_CHANGE_COLUMN_SEQ = 26, 
  READ_CACHE_NOT_LAST_PAGE_SEQ   = 14,
  READ_CACHE_LAST_PAGE_SEQ       = 18, 
  RESET_SEQ                      = 0, 
  READ_SEQ                       = 3,
  STATUS_READ_SEQ                = 41,
  SET_SLC_READ_ERASE_SEQ         = 58,
  SET_SLC_REL_PGM_SEQ            = 60,
  SET_SLC_FAST_PGM_SEQ           = 63,
  SET_MLC_FAST_MLC_READ_SEQ      = 65,
  SET_MLC_FAST_MLC_PGM_SEQ       = 67,
  READ_CHANGE_COLUMN_SEQ         = 9,
  SET_SLC_REL_SEQ                = 69 
} FL_G4_SequenceID_E ; 


typedef enum FL_Reg8_Enum
{ 
  G4_OUTPUT_CONTROL_REG_E			= 0,
  G4_DPD_CONTROL_REG_E				= 1,
  G4_ECC_CONTROL_REG_1_E			= 2,
  G4_CONFIGURATION_REG_E			= 3,
  G4_VIRTUAL_RAM_STATUS_PAGE_REG_E = 4
} FL_Reg8_E ;

typedef enum FL_Reg16_Enum
{ 
  G4_BUS_SWAP_REG_E				= 0,
  G4_INTERRUPT_CONTROL_REG_E		= 1,
  G4_DMA_CONTROL_REG_0_E			= 2,
  G4_DMA_CONTROL_REG_1_E			= 3,
  G4_BURST_CONTROL_REG_E			= 4

} FL_Reg16_E ;

typedef enum FL_DPS_OFFSET_E
{ 
  DPS_LOW_UNIT_LSB       = 0x00 ,
  DPS_LOW_UNIT_MSB       = 0x01 ,
  DPS_HIGH_UNIT_LSB      = 0x02 ,
  DPS_HIGH_UNIT_MSB      = 0x03 ,
  DPS_PROTECTION_KEY     = 0x04 ,
  DPS_PROTECTION_TYPE    = 0x0C ,
  DPS_VRAM_MODE          = 0x0D ,
  DPS_FILL_ZERO          = 0x0E 	  
} FL_DPS_E ;


/*******************************      STRUCTS         ******************************/

typedef struct  
{
  /*************************    Input parameters    ********************************/
  PhyUnitType    phyUnit ;        /*** The actual physical units					    ***/
  FLByte         phyPage ;         /*** The page that the units reside in			    ***/
  FLWord         sectorOffset;     /*** Offset when working in RAW mode					***/    
  FLDword        opFlags ;         /*** Operation flags								    ***/  
  FLWord         noOfPhySectors ;  /*** Number of physical units to access in  parallel ***/  
  FLByte         floor ;           /*** The floor                                       ***/       
  FLByte FAR1   *mainBuffer ;      /*** Pointer to the main buffer						***/
  FLByte FAR1   *extraBuffer;      /*** Pointer to the extra buffer					    ***/
  FLByte         verifyWrite ;     /*** Verify that write succeeded						***/
  FLBoolean      fReadDataCache ;    
  /*************************    RawMode parameters   *******************************/ 
  FLWord         offset;           /*** Offset when working in RAW mode					***/    
  FLWord         length;           /*** Length when working in raw mode					***/ 
  /**************************    Output parameters   ********************************/
  FLSByte        maxBitErrors;     /*** In verify write option it returns the Max errors in sector read ***/ /* cahr is used because of the EDC code */
  FLDword        noOfSectorsPassed;/*** Number of sectors that were successfully passed ***/  							        
  FLDword        freeSectorsFound; /*** Number of  free sectors found				    ***/   
  /**********************************************************************************/
} G4_OperationArgsStruct ,*G4_OperationArgsStructPtr ;

typedef struct 
{   
  LEushort lowUnit               ; /* 0x00-0x01 */
  LEushort highUnit              ; /* 0x02-0x03 */
  FLByte   protectionKey[8]      ; /* 0x04-0x0B */
  FLByte   protectionType        ; /* 0x0C      */
  FLByte   virtualRamMode        ; /* 0x0D      */  	 
  FLByte   fillWithZero[2]       ; /* Reserved  */
} DPS_Struct ,*DPS_StructPtr ;

typedef struct 
{
  FLDword  LockMark ;
  LEulong  COTP_Size ;
} customerLockStruct,*customerLockStructPtr ;

typedef struct 
{
  FLWord wG4_BUS_SWAP_REG_Val ;
  FLByte bG4_CONFIGURATION_REG_Val ; 
  FLWord wG4_INTERRUPT_CONTROL_REG_Val ;
  FLByte bG4_OUTPUT_CONTROL_REG_Val ;
  FLByte bG4_ECC_CONTROL_REG_1_Val ;
  FLByte bG4_DPD_CONTROL_REG ; 
  FLWord wG4_DMA_CONTROL_REG_0 ; 
  FLWord wG4_DMA_CONTROL_REG_1 ;
  FLWord wG4_BURST_CONTROL_REG ;
  FLByte bG4_VIRTUAL_RAM_STATUS_PAGE ;
} G4_RegSavedVal_S,*G4_RegSavedVal_P ;

#endif /* MTD_G4_H */


