/***********************************************************************************/
/*                        M-Systems Confidential                                   */
/*           Copyright (C) M-Systems Flash Disk Pioneers Ltd. 1995-2006            */
/*                         All Rights Reserved                                     */
/***********************************************************************************/
/*                            NOTICE OF M-SYSTEMS OEM                              */
/*                           SOFTWARE LICENSE AGREEMENT                            */
/*                                                                                 */
/*      THE USE OF THIS SOFTWARE IS GOVERNED BY A SEPARATE LICENSE                 */
/*      AGREEMENT BETWEEN THE OEM AND M-SYSTEMS. REFER TO THAT AGREEMENT           */
/*      FOR THE SPECIFIC TERMS AND CONDITIONS OF USE,                              */
/*      OR CONTACT M-SYSTEMS FOR LICENSE ASSISTANCE:                               */
/*      E-MAIL = info@m-sys.com                                                    */
/***********************************************************************************/

/*
 * $Log:   V:/PVCSDB/DiskOnChip/archives/Testing/TrueFFS 6.3/Drop 2.5/3/bddoc/src/h1_exp.h-arc  $
 * 
 *    Rev 1.2   Feb 14 2006 11:09:42   DoronC
 * Update copyright note.
 * 
 *    Rev 1.1   Jun 07 2005 06:24:10   OriS
 * - Documentation - Spelling mistake.
 * 
 *    Rev 1.0   Mar 27 2005 09:27:20   oris
 * Initial revision.
 * 
 *    Rev 1.3   Sep 30 2003 17:54:52   oris
 * changed comments
 * 
 *    Rev 1.2   Jun 05 2003 03:15:50   oris
 * H1_phyGetEraseMark routines was missing a FAR qualifier.
 * 
 *    Rev 1.1   May 16 2003 00:58:08   OriS
 * Added H1_protectionKeyInsert prototype.
 * 
 *    Rev 1.0   Apr 09 2003 12:15:56   OriS
 * Initial revision.
 * 
 */

/************************************************************************
 * Name    : H1_exp.h                                                 *
 *                                                                      *
 * Written : Ronen Golan                                                *
 *                                                                      *
 * Abstract: This file contains the exported from the physical MTD      *
 ************************************************************************/

#ifndef _H1_EXP_H
#define _H1_EXP_H

/***********************************************************************
 * Name:  H1_AsicSetFloor                                                  
 *                                                                      
 * Write to DeviceSelector register.                                    
 *                                                                      
 * Parameters:                                                          
 *      flashPtr     : Pointer identifying drive                             
 *      bFloor     : The selected Floor (0,1,2,3)
 ***********************************************************************/
#define H1_AsicSetFloor(flashPtr,bFloor) \
        H1_Write8BitReg(flashPtr,H1_DEVICE_ID_SELECT_REG,(bFloor))

/***********************************************************************
 * Name : H1_DPS0_LowProtectedUnit                                                  
 *   returns DPS0 low Protected unit  
 *
 ***********************************************************************/
#define H1_DPS0_LowProtectedUnit(volume) \
        H1_Read16BitReg(volume,H1_DPS0_ADDRESS_LOW_REG)  

/***********************************************************************
 * Name : H1_DPS0_HighProtectedUnit                                                  
 *   returns DPS0 high protected unit  
 *
 ***********************************************************************/
#define H1_DPS0_HighProtectedUnit(volume) \
        H1_Read16BitReg(volume,H1_DPS0_ADDRESS_HIGH_REG)

/***********************************************************************
 * Name : H1_DPS1_LowProtectedUnit                                                  
 *   returns DPS1 low Protected unit  
 *
 ***********************************************************************/
#define H1_DPS1_LowProtectedUnit(volume) \
        H1_Read16BitReg(volume,H1_DPS1_ADDRESS_LOW_REG)

/***********************************************************************
 * Name : H1_DPS1_HighProtectedUnit                                                  
 *   returns DPS1 high Protected unit  
 *
 ***********************************************************************/
#define H1_DPS1_HighProtectedUnit(volume) \
        H1_Read16BitReg(volume,H1_DPS1_ADDRESS_HIGH_REG)

/***********************************************************************
 * Name : H1_DPS0_LowProtectedBank                                                  
 *   returns DPS0 low Protected unit  
 *
 ***********************************************************************/
#define H1_DPS0_LowProtectedBank(volume) \
        ((H1_Read8BitReg (volume,H1_DPS0_STATUS_REG) & H1_LOW_BANK_BITS)>>3)  

/***********************************************************************
 * Name : H1_DPS0_HighProtectedBank                                                  
 *   returns DPS0 high Protected unit  
 *
 ***********************************************************************/
#define H1_DPS0_HighProtectedBank(volume) \
        ((H1_Read8BitReg (volume,H1_DPS0_STATUS_REG) & H1_HIGH_BANK_BITS)>>5)  

/***********************************************************************
 * Name : H1_DPS1_LowProtectedBank                                                  
 *   returns DPS0 low Protected unit  
 *
 ***********************************************************************/
#define H1_DPS1_LowProtectedBank(volume) \
        ((H1_Read8BitReg (volume,H1_DPS1_STATUS_REG) & H1_LOW_BANK_BITS)>>3)  

/***********************************************************************
 * Name : H1_DPS1_HighProtectedBank                                                  
 *   returns DPS1 high Protected unit  
 *
 ***********************************************************************/
#define H1_DPS1_HighProtectedBank(volume) \
        ((H1_Read8BitReg (volume,H1_DPS1_STATUS_REG) & H1_HIGH_BANK_BITS)>>5)  
  
/***********************************************************************
 * Name : H1_DPS0_UnitLocation                                                  
 *   returns DPS0 unit location in the Flash 
 *
 ***********************************************************************/
#define H1_DPS0_UnitLocation(volume) \
        (H1_Read8BitReg(volume,H1_DPS0_POINTER_REG))

/***********************************************************************
 * Name : H1_DPS1_UnitLocation                                                  
 *   returns DPS1 unit location in the Flash 
 *
 ***********************************************************************/
#define H1_DPS1_UnitLocation(volume) \
        (H1_Read8BitReg(volume,H1_DPS1_POINTER_REG))
                                     
/***********************************************************************
 * Name : H1_IsDPS0_WP                                                 
 *   Checks if dps is write protected 
 *
 * Returns:        
 *   None 0  - DPS_0 is write protected          
 *   0       - DPS_0 is NOT write protected 
 ***********************************************************************/
#define H1_IsDPS0_WP(volume) \
        (H1_Read8BitReg(volume,H1_DPS0_STATUS_REG) & H1_MTD_WRITE_PROTECTED)
           
/***********************************************************************
 * Name : H1_IsDPS1_WP                                                 
 *   Checks if dps is write protected 
 *
 * Returns:        
 *   None 0  - DPS_1 is write protected          
 *   0       - DPS_1 is NOT write protected 
 ***********************************************************************/
#define H1_IsDPS1_WP(volume) \
        (H1_Read8BitReg(volume,H1_DPS1_STATUS_REG) & H1_MTD_WRITE_PROTECTED)

/***********************************************************************
 * Name : H1_IsDPS0_RP                                                 
 *   Checks if DPS is read protected 
 *
 * Returns:        
 *   None 0  - DPS_0 is read protected
 *   0       - DPS_0 is NOT read protected
 ***********************************************************************/
#define H1_IsDPS0_RP(volume) \
        (H1_Read8BitReg(volume,H1_DPS0_STATUS_REG) & H1_MTD_READ_PROTECTED)

/***********************************************************************
 * Name : H1_IsDPS1_RP                                                 
 *   Checks if DPS is read protected 
 *
 * Returns:        
 *   None 0  - DPS_1 is read protected
 *   0       - DPS_1 is NOT read protected
 ***********************************************************************/
#define H1_IsDPS1_RP(volume) \
        (H1_Read8BitReg(volume,H1_DPS1_STATUS_REG) & H1_MTD_READ_PROTECTED)

/***********************************************************************
 * Name : H1_IsDPS0_LOCK_Enabled                                                  
 *   Checks if DPS is HW_LOCK protected 
 *
 * Returns:        
 *   None 0  - DPS_0 HW locked          
 *   0       - DPS_0 is not HW locked
 ***********************************************************************/
#define H1_IsDPS0_LOCK_Enabled(volume) \
        (H1_Read8BitReg(volume,H1_DPS0_STATUS_REG) & H1_HW_LOCK_ENABLED)

/***********************************************************************
 * Name : H1_IsDPS1_LOCK_Enabled                                                
 *   Checks if DPS is HW_LOCK protected 
 *
 * Returns:        
 *   None 0   - DPS_1 HW locked          
 *   0        - DPS_1 is not HW locked
 ***********************************************************************/
#define H1_IsDPS1_LOCK_Enabled(volume) \
        (H1_Read8BitReg(volume,H1_DPS1_STATUS_REG) & H1_HW_LOCK_ENABLED)
 
/***********************************************************************
 * Name : H1_Is_LOCK_Asserted                                                 
 *   Checks if the LOCK# H/W signal is assterted 
 *
 * Returns:        
 *   None 0  - H/W LOCK# signal is assterted         
 *   0       - H/W LOCK# signal is asserted
 ***********************************************************************/
#define H1_Is_LOCK_Asserted(volume) \
        (H1_Read8BitReg(volume,H1_PROTECTION_STATUS_REG) & H1_LOCK_INPUT_ASSERTED)

/***********************************************************************
 * Name : H1_Is_StickyLOCK_Asserted                                                 
 *   Checks if the sticky lock signal is activated (asserted)
 *
 * Returns:        
 *   None 0  - Sticky lock activated (asserted)         
 *   0       - Sticky lock not activated (not asserted)   
 ***********************************************************************/
#define H1_Is_StickyLOCK_Asserted(volume) \
        (H1_Read8BitReg(volume,H1_OUTPUT_CONTROL_REG) & H1_SOFTWRAE_LOCK_ENABLED)

/***********************************************************************
 * Name : H1_IsDPS0_KeyInserted                                                 
 *   Checks if the key is correct for DPS_0
 *
 * Returns:        
 *   None 0  - Key is correct The protected area is not locked          
 *   0       - Key is not the right one - DPS is protected 
 ***********************************************************************/
#define H1_IsDPS0_KeyInserted(volume) \
        (H1_Read8BitReg(volume,H1_DPS0_STATUS_REG) & H1_KEY_OK)

/***********************************************************************
 * Name : H1_IsDPS1_KeyInserted
 *   Checks if the key is correct for DPS_1
 *
 * Returns:        
 *   NOT_LOCKED  - Key is correct The protected area is not locked          
 *   LOCKED      - Key is not the right one - DPS is protected 
 ***********************************************************************/
#define H1_IsDPS1_KeyInserted(volume) \
        (H1_Read8BitReg(volume,H1_DPS1_STATUS_REG) & H1_KEY_OK)


/************************************************************************
 **********************  Exported functions   ***************************
 ************************************************************************/

/***********************************************************************
 *  Name: H1_Write16BitReg                                                                                   
 *    Write to 16 bit register                                 
 *                                                                      
 * Parameters:
 *      flashPtr             : Pointer identifying drive                      
 *      wOffset              : The Offset of register from base address
 *      wValue               : The 16 bit value that should be written 
 *                             to this register.
 * Returns: Value of the 16 bit register
 * Note:    In big endian the word in RAM memory as:
 *          Offset 0 - Msb byte 
 *          Offset 1 - Lsb byte  
 *          But the OREN 16 bit registers work in Little endian 
 *          which means that the data is stored as follows 
 *          Offset 0 - Lsb byte 
 *          Offset 1 - Msb byte  
 *          In order to convert wValue from big to little endian format
 *          we convert it to LEushort format which store the word in little
 *          endian format and write it to ASIC
 ***********************************************************************/
#ifndef FL_BIG_ENDIAN
#define H1_Write16BitReg(flashPtr,wOffset,wValue) \
  H1_DirectWrite16BitReg (flashPtr,wOffset,wValue) 
#else
#define H1_Write16BitReg(flashPtr,wOffset,wValue) \
  { \
    LEushort LEus_Temp ; \
    toLE2 (LEus_Temp,wValue) ; \
    H1_DirectWrite16BitReg (flashPtr,wOffset,*(FLWord *)LEus_Temp) ; \
  } 
#endif

/***********************************************************************
 *  Name: H1_Read16BitReg
 *    Read 16 bit register                                 
 *                                                                      
 * Parameters:
 *      flashPtr             : Pointer identifying drive                      
 *      wOffset              : The Offset of register from base address
 * Returns: Value of the 16 bit register
 ***********************************************************************/
FLWord  H1_Read16BitReg(FLFlash *flashPtr,FLWord wOffset) ; 

/***********************************************************************
 * Name : H1_isGoodBlock
 *   Determines if a block is a good block or a bad block 
 *
 * Parameters:                                                            
 *   volume             : Pointer identifying drive    
 *   wUnit              : The Block that we wants to check
 *   bOpFlags           : USE_BBT_CACHE - Checks if the page is already in the 
 *                        MTD_VARS->buffer->flData (because it was read on the last call)
 *                        and if it does => Read the buffer without accessing the flash
 *                        ALWAYS_READ_FROM_FLASH - When you are not sure that the 
 *                        MTD_VARS->buffer->flData holds the bbt information           
 *                        use this option which forces the routine to read the bbt page   
 *  Note : When u want to use the isGoodBlock in a routine 
 *         It is the safest to call the first time with ALWAYS_READ_FROM_FLASH 
 *         and in the next calls to use USE_BBT_CACHE  
 *  Returns: 
 *    fIsGoodBlock :
 *           TRUE  => Block is a good block. 
 *           FALSE => Block is a bad block.
 *   flOK on success, any other value on failure.
 ************************************************************************/
FLStatus  H1_isGoodBlock          (FLFlash *volume,FLDword dwUnit,FLByte bOpFlags,FLBoolean *fIsGoodBlock) ;

/************************************************************************
 * Name:  H1_asicSetMode                                              *
 *                                                                      *
 * Set the controller (ASIC) operation mode.                            *
 *                                                                      *
 * Parameters:                                                          *
 *      volume     : Pointer identifying drive                          *
 *      mode    : One of the modes below:                               *
 *                RESET_MODE      = 0                                   *
 *                NORMAL_MODE     = 1                                   *    
 *                POWER_DOWN_MODE = 2                                   *
 *                                                                      *
 * Note: The mode is common to all cascaded floors.                     *
 ************************************************************************/
void  H1_asicSetMode          (FLFlash *volume, FL_AsicModeE AsicMode) ; 

/***********************************************************************
 * Name:  H1_asicSetBank  
 *
 * Select with which bank you want to work (with no write protection)                                     
 *                                                                      
 * Parameters:                                                          
 *      volume     : Pointer identifying drive                             
 *
 ***********************************************************************/
void  H1_asicSetBank          (FLFlash *volume,FLByte Bank) ;

/***********************************************************************
 * Name : H1_erase_Seq 
 *   Executes the Erase sequence that can erase 1..N units in parallel
 * 
 * Parameters:                                                          
 *       flashPtr           : Pointer identifying drive           
 *       wUnitNoPtr         : An array that holds the units that should 
 *                            be erased,each unit should be in a different plane 
 *                            (Number of planes is 1..4 so this array is mamximum 4)
 *       bNoOfPhyUnits      : Number of units that should be erased (1..4)
 *       dwOpFlags          : MTD_DO_NOT_WAIT  - Do not poll waiting for R/B# to be Ready      
 *                            Ori - not supported yet.
 *       bFloor             : The specified floor 
 *       bBank              : The specified bank 
 *  Returns: 
 *  ErrorStatus:
 *      flOK on success, any other value on failure
 ***********************************************************************/
FLStatus H1_erase_Seq (FLFlash *flashPtr,PhyUnitType *wUnitNoPtr,
                         FLByte bNoOfPhyUnits,FLDword dwOpFlags,
						 FLByte bFloor,FLByte bBank) ;

/***********************************************************************
 *  Name : H1_read_Seq  
 *   Executes the FLASH Read sequence command for reading one sector  
 *   
 * Parameters:     
 *       volume                : Pointer identifying drive           
 * readArgs inputs :
 *       PhyUnit            : An pointer to the physical unit that the sector needed to be read is in 
 *       phyPage            : The page that would be read  
 *       MainBuffer         : The Pointer to user buffer containing the data
 *                            that should be written to the sector main area.
 *       ExtraBuffer        : Pointer to user buffer containing the data
 *                              that should be written to the sector info area. 
 *       dwOpFlags          : MTD_DATA      - Read from  the main  area of the physical sector 
 *                              MTD_EXTRA     - Read from the extra area of the physical sector
 *                            MTD_RAW_MODE  - Read with fixed offset and with no EDC/ECC correction
 * readArgs_OUTPUTS:
 *       MaxBitErrors       : The Maximum of bit errors found (0..5)
 *       FreeSectorsFound   : Number of free sectors found (0 or 1) 
 *       NoOfSectorsPassed  : Number of Sectors read successfully (0 or 1)  
 *  ErrorStatus:
 *  NOTE : It assumkes that page size is 512 => PAGE_SIZE=FL_SECTOR_SIZE   
 ***********************************************************************/
FLStatus   H1_read_Seq             (FLFlash *volume,H1_OperationArgsStructPtr READ_ARGS);

/***********************************************************************
 *  Name : H1_RawRead_Seq  
 *   Executes the FLASH raw read sequence command for reading one sector  
 *   
 * Parameters:                                                          
 *    flashPtr              : Pointer identifying drive
 * readArgs inputs :
 *       PhyUnit            : An pointer to the physical unit that the sector needed to be read is in 
 *       phyPage            : The page that would be read  
 *       mainBuffer         : The Pointer to user buffer containing the data
 *                            that should be written to the sector main area.
 *       dwOpFlags          : MTD_RAW_MODE    - Read with fixed offset and with no EDC/ECC correction
 *       length             : Number of bytes in MTD_RAW_MODE
 *       offset             : Page offset in MTD_RAW_MODE
 *
 * readArgs_OUTPUTS: None
 * ErrorStatus:
 ***********************************************************************/
FLStatus H1_RawRead_Seq  (FLFlash *flashPtr,H1_OperationArgsStructPtr readArgs) ;

/***********************************************************************
 * Name : H1_pageProgram_Seq 
 *   Executes the Page program sequence command                                                                     
 *   This routine can write in parallel to 1..N pages (N = Number of planes)
 * 
 * Parameters:                                                          
 *       volume             : Pointer identifying drive           
 *       PhyUnits           : An array that holds the number of the 1..N 
 *                            physical units that needs to be written in parallel
 *       phyPage            : The page that would be programmed in all units in parallel
 *       noOfPhySectors     : Number of physical sectors that needs to be programmeed                              
 *                              (This is the size of the PhySectors array)
 *       MainBuffer         : The Pointer to user buffer containing the data
 *                            that should be written to the sector main area.
 *       ExtraBuffer        : Pointer to user buffer containing the data
 *                              that should be written to the sector info area. 
 *       VerifyWriteMode    : An indicator of the read after write mode.
 *                              FL_ON will force a read after write operation
 *                            while FL_OFF will disable it !!!. 
 *       dwOpFlags            : MTD_DATA        - Write to the main  area of the physical sector 
 *                              MTD_EXTRA       - Write to the extra area of the physical sector 
 *                            MTD_DO_NOT_WAIT - Return without waiting for the R/B# to be ready
 *                                              in the last parallel PageProgram command   
 *                                        
 *          
 *  Returns: 
 *  ErrorStatus:
 *     
 ***********************************************************************/
FLStatus    H1_pageProgram_Seq      (FLFlash *volume,H1_OperationArgsStructPtr WRITE_ARGS);

/***********************************************************************
 * Name : H1_phyWriteEraseMark 
 *   Writes the erase mark to the physical unit  
 * 
 * Parameters:    
 *   Inputs:
 *     flashPtr   : The flash struct
 *     phyUnit    : The physical unit to check erase mark 
 *    dwEraseMark : The erase Mark that would be written 
 *    bFloor      : The specified floor 
 *   Outputs:
 *     EraseCount : The Erase count of the physical erase unit 
 *     
 ***********************************************************************/
FLStatus H1_phyWriteEraseMark (FLFlash *flashPtr,PhyUnitType phyUnit,FLByte bEraseMark,
								 FLByte bFloor,FLByte bBank) ;


/***********************************************************************
 * Name : H1_phyGetEraseCount 
 *   Gets the Number of times a logical erase unit has been erased 
 * 
 * Parameters:    
 *   Inputs:
 *     volume    : The flash struct
 *     StartUnit : The physical unit to check erase count 
 *   Outputs:
 *     EraseCount: The Erase count of the physical erase unit 
 *     
 ***********************************************************************/
FLStatus   H1_phyGetEraseCount     (FLFlash *volume,PhyUnitType phyUnit,FLDword *dwEraseCount) ;

/***********************************************************************
 * Name : H1_phyGetEraseMark 
 *   Reads the erase mark from the physical unit  
 * 
 * Parameters:    
 *   Inputs:
 *     flashPtr   : The flash struct
 *     phyUnit    : The physical unit to check erase mark 
 *     bEraseMark : The erase Mark that was read 
 *     bPage      : The page offset in the unit
 *     bFloor     : The floor 
 *     bBank      : The bank    
 *   Outputs:
 *     EraseCount : The Erase count of the physical erase unit 
 ***********************************************************************/
FLStatus H1_phyGetEraseMark (FLFlash *flashPtr,PhyUnitType phyUnit,
			       FLByte FAR1 *bEraseMark,FLByte bPage,FLByte bFloor,FLByte bBank) ;

/***********************************************************************
 * Name: H1_DPS_Read                                                        
 *   Read DPS data structure from DPS unit of the current floor         
 *                                                                        
 * Parameters:                                                          
 *    volume     : Pointer identifying drive                                 
 *  bDPS_No  : The dps number                                            
 *  bFlags   : H1_COPY_0-Read the 1st copy                                   
 *             H1_COPY_1-Read the 2nd copy                                     
 *             H1_COPY_2-Read the 3rd copy                                    
 *             H1_COPY_3-Read the 4th copy                                    
 *             H1_FIRST_GOOD_COPY- Read the first good copy                 
 *             find the first one that is good.                            
 * Returns:                                                              
 *    FLStatus    : 0 on success, otherwise failed.                        
 ***********************************************************************/
FLStatus   H1_DPS_Read             (FLFlash *volume,FLByte *DPS_PTR,
                                    FL_DPS_IndexE DPS_No,FLByte Flags,FLByte Floor);

/***********************************************************************
 * Name: H1_DPS_Write                                                         
 *   Writes a new DPS to the flash                                                    
 *   It erases the main and redundency units , writes                    
 *    a new DPS struct                                                    
 * Parameters:                                                          
 *    volume         : Pointer identifying drive                            
 *  DPS_No   : The dps number                                            
 *  DPS_PTR  : The DPS struct                                            
 *    bFloor   : The Asic FLOOR                                            
 * Returns:                                                              
 *    FLStatus    : 0 on success, otherwise failed.                        
 * Assumption   : If SYS_INTLV=2 than the interleave would be written   
 *                as 0                                                    
 *                INTLV_2_CASCADED always                                 
 ***********************************************************************/
FLStatus   H1_DPS_Write            (FLFlash *volume,FLByte *DPS,
                                      FL_DPS_IndexE DPS_No,FLByte Floor);

/************************************************************************
 * Name: H1_tryKey
 *   Try to write the KEY to the DPS_0/DPS_ 1 KEY Register
 *
 * Parameters:
 *  volume        : Pointer identifying drive
 *  bDPS_No       : The  DPS number    0/1
 *  bKey          : The 8 bytes key
 *  bFloor        : The floor in which the DPS is in
 ************************************************************************/
FLStatus    H1_tryKey               (FLFlash *volume,FLByte bDPS_No,FLByte FAR1* bKey,
                                       FLByte bFloor);

/***********************************************************************
 * Name: H1_protectionKeyInsert 
 *   Sends protection key only to protected areas.
 *
 * Parameters:
 *  flashPtr   - FLFlash struct
 *  bDPS_No : indicated which protection area to work on. 0 or 1.
 *  Key     : An 8 FLByte long array containing the protection password.          
 *  Notes
 *   1. The key should be sent to all of the device's floors even if 
 *      one of the floors indicated that the key did not fit.
 *      The key should not be sent to a floor that indicates that 
 *      it's key was already inserted. 
 *      This way allows us to open partition that might have different
 *      keys for different floors as a result of a bug or a power failure.
 *   2. Before sending the given key the MTD will try and send "00000000" 
 *      (Ascii) key.
 ***********************************************************************/
FLStatus H1_protectionKeyInsert(FLFlash *flashPtr,FLByte bDPS_No,FLByte FAR1* Key);

/********************************************************************************
 *  Name : H1_phyDoc2Identify 
 *    This routine does the following 
 *    1. Initialization of volume->mtdVars pointer
 *    2. Find the interleave & if_cfg
 *    2. Hooking the correct access routines according to the flash interleave & IF_CFG
 *    3. DOC init - Insert DOC into NORMAL mode.
 *    4  Find board & bank properties                            
 *           - Find the system interleave 1/2                    
 *           - Find how many floors exist                    
 *           - Find how many banks are in each floor            
 *           - Find how many banks are in the last floor        
 *           - Find floor size                                
 *    5. Find Logical Block properties                            
 *           - Find LogicalUnitsInBank                              
 *           - Find LogicalUnitsInFloor                            
 *    6. Find Logical Sector Properties                          
 *          - Find SectorsInPage                                
 *          - Find SectorsInUnit                            
 *          - Find SectorsInLogicalUnit                        
 *            - Find SectorsInChip                            
 *          - Find SectorsInBank                            
 *            - Find SectorsInFloor                                
 *    7. Update FLFLASH struct                                   
 *    8. Update pointers to MTD exported functions 
 *  
 *      volume: The flash struct.
 * Status:
 * Assumptions : It assumes that all flashes are of the same kind (MAKER & CODE ID)       
 *******************************************************************************/
FLStatus   H1_phyDoc2Identify      (FLFlash *volume) ; 
 
/************************************************************************ 
 * Name:  H1_docSearchForWindow                                       *
 *                                                                      * 
 * The routine searches for DOC and if found it updates relevant        *  
 * socket struct fields												    *
 *                                                                      * 
 * Parameters:                                                          * 
 *      dwLowAddress : Low range of search                              * 
 *      dwHighAddress: High range of search                             *        
 *                                                                      * 
 * Returns:                                                             *  
 *    FLStatus    : 0 on success, otherwise failure                     *
 * Note:This routine is used in the BDK only							*  
 ************************************************************************/
FLStatus H1_docSearchForWindow (FLSocket *socket, FLDword dwLowAddress,
			    			      FLDword dwHighAddress);

/*----------------------------------------------------------------------*/
/* Name:  H1_docPlusFreeWindow                                        */
/*                                                                      */
/* Free any resources used for the DiskOnChip window                    */
/*                                                                      */
/* Parameters:                                                          */
/*      socket      :   Record used to store the sockets parameters     */
/*                                                                      */
/* Returns: None                                                        */
/*                                                                      */
/* NOTE: This routine is used only by virtual memory systems in order   */
/*       to unmap the DiskOnChip window.                                */
/*                                                                      */
/*----------------------------------------------------------------------*/ 
void H1_docPlusFreeWindow(FLSocket * socket);

/************************************************************************
 * Name:  H1_asicSetInNormalMode                                      *
 *                                                                      *
 * Set the controller (ASIC) In Normal mode                             *
 *                                                                      *
 * Parameters:                                                          *
 *      flashPtr : Pointer identifying drive                            *
 *      mode     : One of the modes below:                              *
 *                  RESET_MODE      = 0                                 *
 *                  NORMAL_MODE     = 1                                 *    
 *                  POWER_DOWN_MODE = 2                                 *
 *                                                                      *
 * Note: The mode is common to all cascaded floors.                     * 
 ************************************************************************/
FLStatus H1_asicSetInNormalMode (FLFlash *flashPtr) ;

/***********************************************************************
 * Name : H1_download 
 *   Forces a software download of the controller in all floors,
 *   the routine waits until download finishes & put the chip in normal 
 *   mode.
 *  
 * Parameters:    
 *     flashPtr              : The flash struct
 *     flashPtr->noOfFloors  : No of floors in the device
 * The routine return the following error codes:                        
 *     flOK          : Success
 *     flBadDownload : Download failed or failed to identify the controller
 *     
 ***********************************************************************/
FLStatus H1_download (FLFlash *flashPtr) ; 

/***********************************************************************
 * Name : H1_completeOperation
 *   Completes the Erase flash operation 
 *   1. Operates the EraseCallBackPtr routine 
 ***********************************************************************/
FLStatus H1_completeOperation (FLFlash *flashPtr) ;

/***********************************************************************
 * Name: H1_protection_VirtualKeyInsert 
 *   In case that matching units are not adjacent,there might be a 
 *   case that we wouldd need more than one DPS to protect one partition
 *
 * Parameters:
 *  flashPtr   - FLFlash struct
 *  bDPS_No : indicated which protection area to work on. 
 *  Key     : An 8 FLByte long array containing the protection password.          
 *  Notes
 *   1. The key should be sent to all of the device's floors even if 
 *      one of the floors indicated that the key did not fit.
 *      The key should not be sent to a floor that indicates that 
 *      it's key was already inserted. 
 *      This way allows us to open partition that might have different
 *      keys for different floors as a result of a bug or a power failure.
 *   2. Before sending the given key the MTD will try and send "00000000" 
 *      (Ascii) key.
 ***********************************************************************/
FLStatus H1_protection_VirtualKeyInsert(FLFlash *flashPtr,FLByte bDPS_No,FLByte FAR1* Key) ;

/********************************************************************************
 *  Name : M512_CheckandRecoverResetMode 
 *  - Check for RESET_MODE in entry & exit points. 
 *  - If In entry point RESET_MODE detected 
 *    - Report an ERROR 
 *    - If DO_NOT_RECOVER compilation flag is set return flSuspendModeDetected else continue operation.
 *  - If In exit point RESET_MODE detected 
 *    - Report an error
 *    - return flSuspendModeDetected. 
 * Parameters:
 *  flashPtr   - FLFlash struct
 *  bMode      - M512_ENTRY_POINT : Check for RESET_MODE at entry point 
 *  bMode      - M512_EXIT_POINT  : Check for RESET_MODE at entry point 
 * Status: 
 * Assumptions :   
 *******************************************************************************/
FLStatus H1_CheckandRecoverResetMode(FLFlash *flashPtr,FLByte bMode) ;


#endif /* _H1_EXP_H */

