/* vxbTimerLib.h - vxbus timer interfaces header file */

/* Copyright (c) 2006 Wind River Systems, Inc. */

/*
modification history
--------------------
01b,21jun06,pdg  new design incorporated
01a,06Mar06,pdg  written
*/

/*
DESCRIPTION

This library provides the timer specific interfaces

*/

#ifndef _INC_vxbTimerLibH
#define _INC_vxbTimerLibH

#ifdef __cplusplus
extern "C" {
#endif

/* method id for the timer driver */
#define VXB_METHOD_TIMER_FUNC_GET	0x02

/* features provided by the timer */

#define VXB_TIMER_CAN_INTERRUPT		0x00000001
#define VXB_TIMER_INTERMEDIATE_COUNT	0x00000002 /* timestamp & sysXDelay */
#define VXB_TIMER_SIZE_32		0x00000004 
#define VXB_TIMER_SIZE_64		0x00000008 


/* flags used while creating the timer */

#define VXB_TIMER_RESET_AT_ISR		0x00000010

struct vxbTimerFunctionality
    {
    UINT32	clkFrequency;	/* clock frequency */
    UINT32	minPeriod;	/* min period timer can provide */
    UINT32	maxPeriod;	/* max period timer can provide */
    UINT32	features;	/* features of timer */
    UINT32	ticksPerSecond;	/* ticks per second */

    /* allocate dedicated timer */
    STATUS (*timerAllocate)
        (
        VXB_DEVICE_ID	pInst,	 /* IN */
        UINT32		flags,	 /* IN */
        void **		pCookie	 /* OUT */
        );
 
    /* release dedicated timer */
    STATUS (*timerRelease)
        (
        VXB_DEVICE_ID	pInst,	 /* IN */
        void *		pCookie	 /* IN */
        );
    
    /* get count between rollover/ISR events */
    STATUS (*timerRolloverGet)
        (
        void *	pCookie,	/* IN */
        UINT32 *count		/* OUT */
        );
 
    /* 
     * get current timestamp timer count value.
     * The driver should ensure returning the count as that of 
     * an up counter. 
     */
    STATUS (*timerCountGet)
        (
        void *		pCookie,	/* IN */
        UINT32 *	count		/* OUT */
        );

    /* disable the timer */
 
    STATUS (*timerDisable)
        (
        void *	pCookie		/* IN */
        ); 

    /* enable the timer */

    STATUS (*timerEnable)
        (
        void *	pCookie,	/* IN */
        UINT32	maxTimerCount	/* IN */
        ); 

    /* set the function to be called on every interrupt */

    STATUS (*timerISRSet)
        (
        void *	pCookie,	/* IN */
        FUNCPTR	pFunc,		/* IN */
        int	arg		/* IN */
        );
 
    /* enable the timer */

    STATUS (*timerEnable64)
        (
        void *	pCookie,	/* IN */
        UINT64	maxTimerCount	/* OUT */
        ); 

    /* get count between rollover/ISR events */

    STATUS (*timerRolloverGet64)
        (
        void *	pCookie,	/* IN */
        UINT64 *count		/* IN */
        );

    /* 
     * get current timestamp timer count value.
     * The driver should ensure returning the count as that of 
     * an up counter. 
     */
    STATUS (*timerCountGet64)
        (
        void *		pCookie,	/* IN */
        UINT64 *	count		/* OUT */
        );
    };

extern void sysDelay (void);

extern void sysMsDelay
    (
    int	delayTime
    );

extern void sysUsDelay
    (
    int	delayTime
    );

#ifdef __cplusplus
}
#endif

#endif /* _INC_vxbTimerLibH */

