/***********************************************************************************/
/*                        M-Systems Confidential                                   */
/*           Copyright (C) M-Systems Flash Disk Pioneers Ltd. 1995-2006            */
/*                         All Rights Reserved                                     */
/***********************************************************************************/
/*                            NOTICE OF M-SYSTEMS OEM                              */
/*                           SOFTWARE LICENSE AGREEMENT                            */
/*                                                                                 */
/*      THE USE OF THIS SOFTWARE IS GOVERNED BY A SEPARATE LICENSE                 */
/*      AGREEMENT BETWEEN THE OEM AND M-SYSTEMS. REFER TO THAT AGREEMENT           */
/*      FOR THE SPECIFIC TERMS AND CONDITIONS OF USE,                              */
/*      OR CONTACT M-SYSTEMS FOR LICENSE ASSISTANCE:                               */
/*      E-MAIL = info@m-sys.com                                                    */
/***********************************************************************************/

/*
 * $Log:   V:/PVCSDB/DiskOnChip/archives/Testing/TrueFFS 6.3/Drop 2.5/3/bddoc/src/m512_exp.h-arc  $
 * 
 *    Rev 1.3   Feb 14 2006 11:09:46   DoronC
 * Update copyright note.
 * 
 *    Rev 1.2   Jul 17 2005 19:33:02   oris
 * - Bug fix - In case higher floors finished their download operation 
 *   after lower floors, the download operation would have failed with
 *   flTimeOut error.
 * 
 *    Rev 1.1   Jun 07 2005 06:25:44   OriS
 * - Esthetics - Removed the exporting of isgoodblock routine.
 * 
 *    Rev 1.0   Mar 27 2005 09:27:20   oris
 * Initial revision.
 * 
 *    Rev 1.2   Mar 07 2005 22:05:12   oris
 * -	Removed warning
 * -	Internal DPS read and write routine interface was changed due to the packed structures alignment support.
 * 
 *    Rev 1.1   Feb 01 2005 09:20:14   eugen
 * burst mode macros, AsicSetFloor fix
 * 
 *    Rev 1.0   Dec 09 2004 18:24:30   oris
 * Initial revision.
 * 
 *    Rev 1.6   Sep 22 2004 12:58:20   roneng
 * MTD for TFFS6.3 - Basic functionality features
 * 
 *    Rev 1.5   Jan 27 2004 08:14:02   oris
 * Changed comments
 * 
 *    Rev 1.4   Sep 30 2003 17:54:14   oris
 * changed comments
 * 
 *    Rev 1.3   Jun 02 2003 08:53:30   roneng
 * Change printings
 * 
 *    Rev 1.2   May 16 2003 00:57:40   OriS
 * Added M512_protectionKeyInsert prototype.
 * 
 *    Rev 1.1   Apr 13 2003 20:29:38   OriS
 * Added fast read routine.
 * 
 *    Rev 1.0   Apr 09 2003 12:15:54   OriS
 * Initial revision.
 * 
 */

/************************************************************************
 * Name    : M512_exp.h                                                 *
 *                                                                      *
 * Written : Ronen Golan                                                *
 *                                                                      *
 * Abstract: This file contains the exported from the physical MTD      *
 ************************************************************************/

#ifndef _M512_EXP_H
#define _M512_EXP_H


/****************************** Macros *********************************/
/***********************************************************************
 * Name : M512_DPS0_LowProtectedUnit                                                  
 *   returns DPS0 low Protected unit  
 *
 ***********************************************************************/
#define M512_DPS0_LowProtectedUnit(volume) \
        (M512_Read16BitReg(volume,M512_DPS0_ADDRESS_LOW_REG))  

/***********************************************************************
 * Name : M512_DPS0_HighProtectedUnit                                                  
 *   returns DPS0 low Protected unit  
 *
 ***********************************************************************/
#define M512_DPS0_HighProtectedUnit(volume) \
        (M512_Read16BitReg(volume,M512_DPS0_ADDRESS_HIGH_REG))

/***********************************************************************
 * Name : M512_DPS1_LowProtectedUnit                                                  
 *   returns DPS0 low Protected unit  
 *
 ***********************************************************************/
#define M512_DPS1_LowProtectedUnit(volume) \
        (M512_Read16BitReg(volume,M512_DPS1_ADDRESS_LOW_REG))

/***********************************************************************
 * Name : M512_DPS1_HighProtectedUnit                                                  
 *   returns DPS0 low Protected unit  
 *
 ***********************************************************************/
#define M512_DPS1_HighProtectedUnit(volume) \
        (M512_Read16BitReg(volume,M512_DPS1_ADDRESS_HIGH_REG))
  
/***********************************************************************
 * Name : M512_DPS0_UnitLocation                                                  
 *   returns DPS0 unit location in the Flash 
 *
 ***********************************************************************/
#define M512_DPS0_UnitLocation(volume)               1

/***********************************************************************
 * Name : M512_DPS1_UnitLocation                                                  
 *   returns DPS1 unit location in the Flash 
 *
 ***********************************************************************/
#define M512_DPS1_UnitLocation(volume)               3
                                     
/***********************************************************************
 * Name : M512_IsDPS0_WP                                                 
 *   Checks if dps is write protected 
 *
 * Returns:        
 *   None 0  - DPS_0 is write protected          
 *   0       - DPS_0 is NOT write protected 
 ***********************************************************************/
#define M512_IsDPS0_WP(volume) \
        (M512_Read8BitReg(volume,M512_DPS0_STATUS_REG) & M512_WRITE_PROTECTED)
           
/***********************************************************************
 * Name : M512_IsDPS1_WP                                                 
 *   Checks if dps is write protected 
 *
 * Returns:        
 *   None 0  - DPS_1 is write protected          
 *   0       - DPS_1 is NOT write protected 
 ***********************************************************************/
#define M512_IsDPS1_WP(volume) \
        (M512_Read8BitReg(volume,M512_DPS1_STATUS_REG) & M512_WRITE_PROTECTED)

/***********************************************************************
 * Name : M512_IsDPS0_RP                                                 
 *   Checks if DPS is read protected 
 *
 * Returns:        
 *   None 0  - DPS_0 is read protected
 *   0       - DPS_0 is NOT read protected
 ***********************************************************************/
#define M512_IsDPS0_RP(volume) \
        (M512_Read8BitReg(volume,M512_DPS0_STATUS_REG) & M512_READ_PROTECTED)

/***********************************************************************
 * Name : M512_IsDPS1_RP                                                 
 *   Checks if DPS is read protected 
 *
 * Returns:        
 *   None 0  - DPS_1 is read protected
 *   0       - DPS_1 is NOT read protected
 ***********************************************************************/
#define M512_IsDPS1_RP(volume) \
        (M512_Read8BitReg(volume,M512_DPS1_STATUS_REG) & M512_READ_PROTECTED)

/***********************************************************************
 * Name : M512_IsDPS0_LOCK_Enabled                                                  
 *   Checks if DPS is HW_LOCK protected 
 *
 * Returns:        
 *   None 0  - DPS_0 HW locked          
 *   0       - DPS_0 is not HW locked
 ***********************************************************************/
#define M512_IsDPS0_LOCK_Enabled(volume) \
        (M512_Read8BitReg(volume,M512_DPS0_STATUS_REG) & M512_HW_LOCK_ENABLED)

/***********************************************************************
 * Name : M512_IsDPS1_LOCK_Enabled                                                
 *   Checks if DPS is HW_LOCK protected 
 *
 * Returns:        
 *   None 0   - DPS_1 HW locked          
 *   0        - DPS_1 is not HW locked
 ***********************************************************************/
#define M512_IsDPS1_LOCK_Enabled(volume) \
        (M512_Read8BitReg(volume,M512_DPS1_STATUS_REG) & M512_HW_LOCK_ENABLED)
 
/***********************************************************************
 * Name : M512_Is_LOCK_Negated                                                 
 *   Checks if the LOCK# H/W signal is assterted 
 *
 * Returns:        
 *   None 0  - H/W LOCK# signal is assterted         
 *   0       - H/W LOCK# signal is asserted
 ***********************************************************************/
#define M512_Is_LOCK_Negated(volume) \
        (M512_Read8BitReg(volume,M512_PROTECTION_STATUS_REG) & M512_LOCK_INPUT_NEGATED)

/***********************************************************************
 * Name : M512_Is_StickyLOCK_Asserted                                                 
 *   Checks if the sticky lock signal is activated (asserted)
 *
 * Returns:        
 *   None 0  - Sticky lock activated (asserted)         
 *   0       - Sticky lock not activated (not asserted)   
 ***********************************************************************/
#define M512_Is_StickyLOCK_Asserted(volume) \
        (M512_Read8BitReg(volume,M512_PROTECTION_STATUS_REG) & M512_STICKY_LOCK_ACTIVATED)

/***********************************************************************
 * Name : M512_IsDPS0_KeyInserted                                                 
 *   Checks if the key is correct for DPS_0
 *
 * Returns:        
 *   None 0  - Key is correct The protected area is not locked          
 *   0       - Key is not the right one - DPS is protected 
 ***********************************************************************/
#define M512_IsDPS0_KeyInserted(volume) \
        (M512_Read8BitReg(volume,M512_DPS0_STATUS_REG) & M512_KEY_OK)

/***********************************************************************
 * Name : M512_IsDPS1_KeyInserted                                                 
 *   Checks if the key is correct for DPS_1
 *
 * Returns:        
 *   None 0  - Key is correct The protected area is not locked          
 *   0       - Key is not the right one - DPS is protected 
 ***********************************************************************/
#define M512_IsDPS1_KeyInserted(volume) \
        (M512_Read8BitReg(volume,M512_DPS1_STATUS_REG) & M512_KEY_OK)

/***********************************************************************
 * Name : M512_IsDPS0_OTP_Protected
 *   Checks if DPS_0 is OTP protected 
 *
 * Returns:        
 *   0           - DPS is not OTP protected
 *   None 0      - DPS is OTP protected 
 ***********************************************************************/
#define M512_IsDPS0_OTP_Protected(volume) \
        (M512_Read8BitReg(volume,M512_DPS0_STATUS_REG) & M512_OTP_PROTECTED)

/***********************************************************************
 * Name : M512_IsDPS1_OTP_Protected
 *   Checks if DPS_0 is OTP protected 
 *
 * Returns:        
 *   0           - DPS is not OTP protected
 *   None 0      - DPS is OTP protected 
 ***********************************************************************/
#define M512_IsDPS1_OTP_Protected(volume) \
        (M512_Read8BitReg(volume,M512_DPS1_STATUS_REG) & M512_OTP_PROTECTED)

/***********************************************************************
 *  Name: M512_Write16BitReg                                                                                   
 *    Write to 16 bit register                                 
 *                                                                      
 * Parameters:
 *      flashPtr             : Pointer identifying drive                      
 *      wOffset              : The Offset of register from base address
 *      wValue               : The 16 bit value that should be written 
 *                             to this register.
 * Returns: Value of the 16 bit register
 * Note:    In big endian the word in RAM memory as:
 *          Offset 0 - Msb byte 
 *          Offset 1 - Lsb byte  
 *          But the M512 16 bit registers work in Little endian 
 *          which means that the data is stored as follows 
 *          Offset 0 - Lsb byte 
 *          Offset 1 - Msb byte  
 *          In order to convert wValue from big to little endian format
 *          we convert it to LEushort format which store the word in little
 *          endian format and write it to ASIC
 ***********************************************************************/
#ifndef FL_BIG_ENDIAN
#define M512_Write16BitReg(flashPtr,wOffset,wValue) \
  M512_DirectWrite16BitReg (flashPtr,wOffset,wValue) 
#else
#define M512_Write16BitReg(flashPtr,wOffset,wValue) \
  M512_DirectWrite16BitReg(flashPtr,wOffset,(FLWord)((FLByte)(wValue>>8)|((FLByte)(wValue)<<8)))
#endif

/***********************************************************************
 * Name:  M512_AsicSetFloor                                                  
 *                                                                      
 * Write to DeviceSelector register to set the desired floor .                                    
 *                                                                      
 * Parameters:                                                          
 *      flashPtr     : Pointer identifying drive                             
 *      bFloor     : The selected Floor (0,1,2,3)
 ***********************************************************************/
#define M512_AsicSetFloor(flashPtr,bFloor) \
  M512_Write8BitReg(flashPtr,M512_DEVICE_ID_SELECT_REG,bFloor) ; \
  MTD_VARS->bCurrentFloor = bFloor ; \
  M512_Write8BitReg (flashPtr,M512_FLASH_CONTROL_REG,M512_FCE_SIGNAL_ACTIVE)


/************************************************************************
 **********************  Exported functions   ***************************
 ************************************************************************/

/********************************************************************************
 *  Name : M512_CheckandRecoverResetMode 
 *  - Check for RESET_MODE in entry & exit points. 
 *  - If In entry point RESET_MODE detected 
 *    - Report an ERROR 
 *    - If DO_NOT_RECOVER compilation flag is set return flSuspendModeDetected else continue operation.
 *  - If In exit point RESET_MODE detected 
 *    - Report an error
 *    - return flSuspendModeDetected. 
 * Parameters:
 *  flashPtr   - FLFlash struct
 *  bMode      - M512_ENTRY_POINT : Check for RESET_MODE at entry point 
 *  bMode      - M512_EXIT_POINT  : Check for RESET_MODE at entry point 
 * Status: 
 * Assumptions :   
 *******************************************************************************/
FLStatus M512_CheckandRecoverResetMode(FLFlash *flashPtr,FLByte bMode) ;


/************************************************************************
 * Name:  M512_asicSetMode                                              *
 *                                                                      *
 * Set the controller (ASIC) operation mode.                            *
 *                                                                      *
 * Parameters:                                                          *
 *      volume     : Pointer identifying drive                          *
 *      mode    : One of the modes below:                               *
 *                RESET_MODE      = 0                                   *
 *                NORMAL_MODE     = 1                                   *    
 *                POWER_DOWN_MODE = 2                                   *
 *                                                                      *
 * Note: The mode is common to all cascaded floors.                     *
 ************************************************************************/
void  M512_asicSetMode          (FLFlash *volume, FL_AsicModeE AsicMode) ; 

/***********************************************************************
 * Name : M512_erase_Seq 
 *   Executes the Erase sequence that can erase 1..N units in parallel
 * 
 * Parameters:                                                          
 *       volume             : Pointer identifying drive           
 *       wPhyUnitNo         : An array that holds the units that should 
 *                            be erased,each unit should be in a different plane 
 *                            (Number of planes is 1..4 so this array is mamximum 4)
 *       bNoOfPhyUnits      : Number of units that should be erased (1..4)
 *       Flags              : MTD_DO_NOT_WAIT  - Do not poll waiting for R/B# to be Ready      
 *       bFloor             : The desired floor                                          
 *  Returns: 
 *  ErrorStatus:
 *     
 ***********************************************************************/
FLStatus  M512_erase_Seq            (FLFlash *volume,PhyUnitType *UnitNo,FLByte NoOfPhyUnits,
                                     FLDword Flags,FLByte bFloor); 

/***********************************************************************
 *  Name : M512_read_Seq  
 *   Executes the FLASH Read sequence command for reading one sector  
 *   
 * Parameters:     
 *       volume                : Pointer identifying drive           
 * readArgs inputs :
 *       PhyUnit            : An pointer to the physical unit that the sector needed to be read is in 
 *       phyPage            : The page that would be read  
 *       MainBuffer         : The Pointer to user buffer containing the data
 *                            that should be written to the sector main area.
 *       ExtraBuffer        : Pointer to user buffer containing the data
 *                              that should be written to the sector info area. 
 *       dwOpFlags          : MTD_DATA      - Read from  the main  area of the physical sector 
 *                              MTD_EXTRA     - Read from the extra area of the physical sector
 *                            MTD_RAW_MODE  - Read with fixed offset and with no EDC/ECC correction
 * readArgs_OUTPUTS:
 *       MaxBitErrors       : The Maximum of bit errors found (0..5)
 *       FreeSectorsFound   : Number of free sectors found (0 or 1) 
 *       NoOfSectorsPassed  : Number of Sectors read successfully (0 or 1)  
 *  ErrorStatus:
 *  NOTE : It assumkes that page size is 512 => PAGE_SIZE=FL_SECTOR_SIZE   
 ***********************************************************************/
FLStatus     M512_read_Seq             (FLFlash *volume,M512_OperationArgsStructPtr READ_ARGS);

/***********************************************************************
 *  Name : M512_readData_Seq  
 *   Executes the FLASH Read sequence command for reading sectors      
 *
 * Parameters:                                                          
 *    flashPtr              : Pointer identifying drive
 * readArgs inputs :
 *       PhyUnit            : An pointer to the physical unit that the sector needed to be read is in 
 *       phyPage            : The page that would be read  
 *       mainBuffer         : The Pointer to user buffer containing the data
 *                            that should be written to the sector main area.
 *       extraBuffer        : Pointer to user buffer containing the data
 *                              that should be written to the sector info area. 
 *       dwOpFlags          : MTD_DATA        - Read from  the main  area of the physical sector 
 *                            MTD_EXTRA       - Read from the extra area of the physical sector                                                  
 *                            MTD_REL_MODE    - Force the reliable mode. 
 *                            MTD_FAST_MODE   - Forces the fast mode. 
 *       sectorOffset       : The sector offset in the page 0 / 1
 *             
 * readArgs_OUTPUTS:
 *       MaxBitErrors       : The Maximum of bit errors found (0..5)
 *       FreeSectorsFound   : Number of free sectors found (0 or 1) 
 *       NoOfSectorsPassed  : Number of Sectors read successfully (0 or 1)  
 * +--------------------------------------------------------------------------------+ *
 * |   Supported OpFlags |   MainBuffer   |  ExtraBuffer  |    Operation            | *
 * |   ----------------- |   ----------   |  -----------  |    ---------            | *
 * | MTD_DATA            |     Valid      |    Valid      | Read DATA + ECC         | *
 * | MTD_DATA|MTD_EXTRA  |     Valid      |    Valid      | Read DATA + Extra + ECC | *
 * +--------------------------------------------------------------------------------+ *
 **************************************************************************************/
FLStatus M512_readData_Seq   (FLFlash *flashPtr,M512_OperationArgsStructPtr readArgs) ;

/***********************************************************************
 * Name : M512_pageProgram_Seq 
 *   Executes the Page program sequence command                                                                     
 *   This routine can write in parallel to 1..N pages (N = Number of planes)
 * 
 * Parameters:                                                          
 *       volume             : Pointer identifying drive           
 *       PhyUnits           : An array that holds the number of the 1..N 
 *                            physical units that needs to be written in parallel
 *       phyPage            : The page that would be programmed in all units in parallel
 *       noOfPhySectors     : Number of physical sectors that needs to be programmeed                              
 *                              (This is the size of the PhySectors array)
 *       MainBuffer         : The Pointer to user buffer containing the data
 *                            that should be written to the sector main area.
 *       ExtraBuffer        : Pointer to user buffer containing the data
 *                              that should be written to the sector info area. 
 *       VerifyWriteMode    : An indicator of the read after write mode.
 *                              FL_ON will force a read after write operation
 *                            while FL_OFF will disable it !!!. 
 *       dwOpFlags            : MTD_DATA        - Write to the main  area of the physical sector 
 *                              MTD_EXTRA       - Write to the extra area of the physical sector 
 *                            MTD_DO_NOT_WAIT - Return without waiting for the R/B# to be ready
 *                                              in the last parallel PageProgram command   
 *                                        
 *          
 *  Returns: 
 *  ErrorStatus:
 *     
 ***********************************************************************/
FLStatus     M512_pageProgram_Seq      (FLFlash *volume,M512_OperationArgsStructPtr WRITE_ARGS);

/***********************************************************************
 * Name : M512_phyWriteEraseMark 
 *   Writes the erase mark to the physical unit  
 * 
 * Parameters:    
 *   Inputs:
 *     volume     : The flash struct
 *     phyUnit    : The physical unit to check erase mark 
 *    dwEraseMark : The erase Mark that would be written 
 *   Outputs:
 *     EraseCount : The Erase count of the physical erase unit 
 *     
 ***********************************************************************/
FLStatus    M512_phyWriteEraseMark    (FLFlash *volume,PhyUnitType phyUnit,FLByte bEraseMark)  ; 


/***********************************************************************
 * Name : M512_phyGetEraseCount 
 *   Gets the Number of times a logical erase unit has been erased 
 * 
 * Parameters:    
 *   Inputs:
 *     volume    : The flash struct
 *     StartUnit : The physical unit to check erase count 
 *     bFloor    : The floor  
 *   Outputs:
 *     EraseCount: The Erase count of the physical erase unit 
 *     
 ***********************************************************************/
FLStatus M512_phyGetEraseCount (FLFlash *flashPtr,PhyUnitType *PhyUnitArray,FLDword *dwEraseCount,FLByte bFloor) ;


/***********************************************************************
 * Name : M512_phyGetEraseMark 
 *   Reads the erase mark from the physical unit  
 * 
 * Parameters:    
 *   Inputs:
 *     volume     : The flash struct
 *     phyUnit    : A pointer to the physical unit array  
 *     bEraseMark : The erase Mark that was read 
 *     bFloor     : The floor
 *   Outputs:
 *     EraseCount : The Erase count of the physical erase unit 
 *     
 ***********************************************************************/
FLStatus    M512_phyGetEraseMark      (FLFlash *volume,PhyUnitType *phyUnit,FLByte *bEraseMark,FLByte bFloor) ;

/************************************************************************
 * Name: M512_DPS_Write                                                         
 *   Writes a new DPS to the flash                                                    
 *   It erases the main and redundency units , writes                    
 *   of a new DPS struct                                                
 * Parameters:                                                          
 *    flashPtr         : Pointer identifying drive                            
 *  DPS_No   : The dps number                                            
 *  DPS_PTR  : The DPS struct                                            
 *    bFloor   : The Asic FLOOR                                            
 * Returns:                                                              
 *    FLStatus    : 0 on success, otherwise failed.                        
 * Assumption   : If SYS_INTLV=2 than the interleave would be written   
 *                as 0                                                    
 *                M512_INTLV_2_CASCADED always                                 
 ************************************************************************/
FLStatus M512_DPS_Write (FLFlash *flashPtr,FLByte *DPS,FL_DPS_IndexE DPS_No,FLByte bFloor) ;

/***********************************************************************
 * Name: M512_DPS_Read                                                  
 *   Read DPS data structure from DPS unit of the current floor         
 *                                                                      
 * Parameters:                                                          
 *    flashPtr     : Pointer identifying drive                          
 *  bDPS_No  : The dps number                                           
 *  bFlags   : M512_COPY_0-Read the 1st copy                            
 *             M512_COPY_1-Read the 2nd copy                             
 *             M512_COPY_2-Read the 3rd copy                            
 *             M512_COPY_3-Read the 4th copy                            
 *             M512_FIRST_GOOD_COPY- Read the first good copy            
 *             find the first one that is good.                         
 * Returns:                                                             
 *    FLStatus    : 0 on success, otherwise failed.                     
 ***********************************************************************/
FLStatus M512_DPS_Read(FLFlash *flashPtr,FLByte *DPS_Ptr,FL_DPS_IndexE DPS_No,FLByte bFlags,
					   FLByte bFloor) ;

/************************************************************************
 * Name: M512_tryKey
 *   Try to write the KEY to the DPS_0/DPS_ 1 KEY Register
 *
 * Parameters:
 *  volume        : Pointer identifying drive
 *  bDPS_No       : The  DPS number    0/1
 *  bKey          : The 8 bytes key
 *  bFloor        : The floor in which the DPS is in
 ************************************************************************/
FLStatus      M512_tryKey               (FLFlash *volume,FLByte bDPS_No,FLByte FAR1* bKey,
                                         FLByte bFloor);

/***********************************************************************
 * Name: M512_protectionKeyInsert 
 *   Sends protection key only to protected areas.
 *
 * Parameters:
 *  flashPtr   - FLFlash struct
 *  bDPS_No : indicated which protection area to work on. 0 or 1.
 *  Key     : An 8 FLByte long array containing the protection password.          
 *  Notes
 *   1.    The key should be sent to all of the device's floors even if 
 *      one of the floors indicated that the key did not fit.
 *      The key should not be sent to a floor that indicates that 
 *      it's key was already inserted. 
 *      This way allows us to open partition that might have different
 *      keys for different floors as a result of a bug or a power failure.
 *   2.    Before sending the given key the MTD will try and send "00000000" 
 *      (Ascii) key.
 ***********************************************************************/
FLStatus M512_protectionKeyInsert(FLFlash *flashPtr,FLByte bDPS_No,FLByte FAR1* Key);

/********************************************************************************
 *  Name : M512_phyDoc2Identify 
 *    This routine does the following 
 *    1. Initialization of volume->mtdVars pointer
 *    2. Find the interleave & if_cfg
 *    2. Hooking the correct access routines according to the flash interleave & IF_CFG
 *    3. DOC init - Insert DOC into NORMAL mode.
 *    4  Find board & bank properties                            
 *           - Find the system interleave 1/2                    
 *           - Find how many floors exist                    
 *           - Find how many banks are in each floor            
 *           - Find how many banks are in the last floor        
 *           - Find floor size                                
 *    5. Find Logical Block properties                            
 *           - Find LogicalUnitsInBank                              
 *           - Find LogicalUnitsInFloor                            
 *    6. Find Logical Sector Properties                          
 *          - Find SectorsInPage                                
 *          - Find SectorsInUnit                            
 *          - Find SectorsInLogicalUnit                        
 *            - Find SectorsInChip                            
 *          - Find SectorsInBank                            
 *            - Find SectorsInFloor                                
 *    7. Update FLFLASH struct                                   
 *    8. Update pointers to MTD exported functions 
 *  
 *      volume: The flash struct.
 * Status:
 * Assumptions : It assumes that all flashes are of the same kind (MAKER & CODE ID)       
 *******************************************************************************/
FLStatus      M512_phyDoc2Identify      (FLFlash *volume) ; 

/***********************************************************************
 * Name : M512_Read8BitReg
 *  Read 8 bit register in two stages
 *  1. Write the address to the M512_ADDRESS_REG
 *  2. Read from the specified address (only important for debugging)
 *
 * Parameters:
 *    flashPtr:   The flash struct.
 *    wOffset:    The offset of the FLASH_DATA_REGISTER
 *
 * Returns: The value of this register 
 ***********************************************************************/
FLByte M512_Read8BitReg(FLFlash *flashPtr,FLWord wOffset) ;

/***********************************************************************
 * Name : M512_Read16BitReg
 *  Read 16 bit register in two stages
 *  1. Write the address to the M512_ADDRESS_REG
 *  2. Read from the specified address (only important for debugging)
 *
 * Parameters:
 *    flashPtr:   The flash struct.
 *    wOffset:    The offset of the FLASH_DATA_REGISTER
 *
 * Returns: The value of this register 
 ***********************************************************************/
FLWord M512_Read16BitReg(FLFlash *flashPtr,FLWord wOffset) ;

#endif /* _M512_EXP_H */

