/***********************************************************************************/
/*                        M-Systems Confidential                                   */
/*           Copyright (C) M-Systems Flash Disk Pioneers Ltd. 1995-2006            */
/*                         All Rights Reserved                                     */
/***********************************************************************************/
/*                            NOTICE OF M-SYSTEMS OEM                              */
/*                           SOFTWARE LICENSE AGREEMENT                            */
/*                                                                                 */
/*      THE USE OF THIS SOFTWARE IS GOVERNED BY A SEPARATE LICENSE                 */
/*      AGREEMENT BETWEEN THE OEM AND M-SYSTEMS. REFER TO THAT AGREEMENT           */
/*      FOR THE SPECIFIC TERMS AND CONDITIONS OF USE,                              */
/*      OR CONTACT M-SYSTEMS FOR LICENSE ASSISTANCE:                               */
/*      E-MAIL = info@m-sys.com                                                    */
/***********************************************************************************/

/*
 * $Log:   V:/PVCSDB/DiskOnChip/archives/Testing/TrueFFS 6.3/Drop 2.5/3/bddoc/src/nanddefs.h-arc  $
 * 
 *    Rev 1.9   Mar 02 2006 08:25:08   DoronC
 * Add support for new H1 1GB device from Hynix.
 * 
 *    Rev 1.8   Feb 14 2006 11:09:48   DoronC
 * Update copyright note.
 * 
 *    Rev 1.7   Sep 12 2005 19:06:00   roneng
 * Remove unused definitions bu hidden compilation flag
 * 
 *    Rev 1.6   Jul 19 2005 19:01:44   roneng
 * Add H1H MTD flash type
 * 
 *    Rev 1.2   Apr 19 2005 02:51:28   oris
 * - Removed unused compilation flags - 
 *   (VERIFY_ERASE, MTD_RECONSTRUCT_BBT) 
 * 
 *    Rev 1.1   Apr 04 2005 20:17:12   oris
 * - Removed RAM_MTD (unused) compilation flag.
 * 
 *    Rev 1.0   Mar 27 2005 09:27:22   oris
 * Initial revision.
 * 
 *    Rev 1.0   Dec 09 2004 18:24:32   oris
 * Initial revision.
 * 
 *    Rev 1.4   Jun 09 2004 15:11:22   oris
 * Added comment
 * 
 *    Rev 1.3   Jun 08 2004 18:32:30   roneng
 * Add 90nm MLC 2KP support
 * 
 *    Rev 1.2   Dec 03 2003 17:59:08   roneng
 * Add definition for Toshiba 2K Page SLC 1GBit & 2GBit
 * 
 *    Rev 1.1   Aug 10 2003 13:57:18   roneng
 * - Add definition for 1GBit,2Gbit & 4GBit samsung flashes 
 * 
 *    Rev 1.0   Apr 09 2003 12:15:56   OriS
 * Initial revision.
 * 
 */

#ifndef NANDDEFS_H
#define NANDDEFS_H

#include "flflash.h"
#ifdef MTD_STANDALONE
#include "mtdsa.h"
#endif /* MTD_STANDALONE */

typedef FLByte Reg8bitType;
typedef FLWord Reg16bitType;

typedef struct {
  FLWord        pageSize;
  FLWord        pageAndTailSize;
  FLWord        vendorID;
  FLWord        chipID;
  FLDword       pageMask;        /* ...these............... */
  FLWord        pageAreaSize;    /* .......variables....... */
  FLWord        tailSize;        /* .............interleave */
  FLWord        noOfBlocks;      /* total erasable blocks in flash device*/
  FLWord        pagesPerBlock;
  FLByte        currentFloor;    /*    0 .. totalFloors-1   */
  FLDword       floorSize;       /*    in bytes             */
  FLDword       floorSizeMask;
  FLByte        floorSizeBits;
  FLByte        if_cfg;          /* host access type        */
  FLWord        flags;           /* bitwise: BIG_PAGE, SLOW_IO etc. */
  FLBuffer*             buffer;          /* buffer for map through buffer */
#if (defined(FL_VERIFY_WRITE) || defined(VERIFY_VOLUME))
  FLByte*               readBackBuffer;  /* buffer for map through buffer */
#endif /* FL_VERIFY_WRITE || VERIFY_VOLUME */
  FLWord                win_io;          /* pointer to DOC CDSN_IO          */
  NDOC2window           win;             /* pointer to DOC memory window    */
} NFDC21Vars;

#define NFDC21thisVars   ((NFDC21Vars *) flash->mtdVars)
#define NFDC21thisWin    (NFDC21thisVars->win)
#define NFDC21thisIO     (NFDC21thisVars->win_io)
#define NFDC21thisBuffer (NFDC21thisVars->buffer->flData)

#ifndef FL_REMOVE_UNUSED_DEFINITIONS

/* Flash commands - Commented out as it causes name clashes */

#define SERIAL_DATA_INPUT   0x80
#define READ_MODE           0x00
#define READ_MODE_2         0x50
#define RESET_FLASH         0xff
#define SETUP_WRITE         0x10
#define SETUP_ERASE         0x60
#define CONFIRM_ERASE       0xd0
#define READ_STATUS         0x70
#define READ_ID             0x90
#define SUSPEND_ERASE       0xb0
#define REGISTER_READ       0xe0

/* commands for moving flash pointer to areeas A,B or C of page */
typedef enum { AREA_A = READ_MODE, AREA_B = 0x1, AREA_C = READ_MODE_2 } PointerOp;

#define FAIL             0x01    /* error in block erase   */
#define EXTRA_LEN        8       /* In memory of 4MB chips */ 
#define SECTOR_EXTRA_LEN 16   

#endif /* FL_REMOVE_UNUSED_DEFINITIONS */

     /* Flash IDs*/

#define KM29N16000_FLASH    0xec64
#define KM29N32000_FLASH    0xece5
#define KM29V64000_FLASH    0xece6
#define KM29V128000_FLASH   0xec73
#define KM29V256000_FLASH   0xec75
#define KM29V512000_FLASH   0xec76
#define KM29V1024000_FLASH  0xec79
#define K9F1G08U0M_FLASH    0xecc1
#define K9F2G08U0M_FLASH    0xecda
#define K9F4G08U0M_FLASH    0xecdc

#define NM29N16_FLASH       0x8f64
#define NM29N32_FLASH       0x8fe5
#define NM29N64_FLASH       0x8fe6
#define TC5816_FLASH        0x9864
#define TC5832_FLASH        0x98e5
#define TC5864_FLASH        0x98e6
#define TC58128_FLASH       0x9873
#define TC58256_FLASH       0x9875
#define TC58512_FLASH       0x9876
#define TC581024_FLASH      0x9877

#define TC58100FT           0x9879   
#define TC58010FT           0x9879
#define KF2816U0C           0xEC53 
#define KF2832U0C           0xEC55
#define TC58020FT           0x98DA   

#define TC58NVG0S3AFT05     0x98F1  
#define TC58NVG1S3AFT05     0x98DA 
#define TC58NVG2D4BFT00     0x98DC   
#define HY27UF162G2M        0xADCA
#define HY27UT084G2M        0xADDC
#endif /* NANDDEFS_H */
