/*
    NI CONFIDENTIAL
    (c) Copyright 2002 National Instruments Corp.
    All rights reserved.

    Owners:  darin.gillis

	niTracetool.h - header needed to control installation of custom NI tracetool hooks
*/
#include <vxWorksCommon.h>
#include <base/b_struct_timespec.h>
#include <wvLib.h>
#include <private\eventP.h>
#include <private\evtLogLibP.h>

/* make sure these user events don't collide with those user events
   defined in labview's source (pheventlog.h) */
#define EVENTLOGGING_SLEEP					0x506
#define EVENTLOGGING_SLEEPDONE				0x508
#define EVENTLOGGING_MEMMGR_OSMEMMGRSTART	0x206
#define EVENTLOGGING_MEMMGR_OSMEMMGRDONE	0x208

/* flags to indicate which memory operation is being instrumented
   similar to those for c-runtime func _CrtSetAllocHook() */
#define NI_ALLOC_ROUTINE         0
#define NI_REALLOC_ROUTINE       1
#define NI_FREE_ROUTINE          2
#define NI_MEMADDTOPOOL_ROUTINE  3

#ifdef __cplusplus
extern "C" {
#endif
    
/* Called by LV to turn on the wait/sleep logging (called "detailed" logging in the trace tool) */
void SetDetailedLogging(BOOL detailed);
    
/* Configures newRtn to be called before and after a memory or sleep call */
FUNCPTR setPreMemOpRoutine(FUNCPTR newRtn);
FUNCPTR setPostMemOpRoutine(FUNCPTR newRtn);
FUNCPTR setPreSleepOpRoutine(FUNCPTR newRtn);
FUNCPTR setPostSleepOpRoutine(FUNCPTR newRtn);
    
/* Logs that the enterance and exit to memory or sleep spans */
void logEnterMemorySpan(const int allocType, const int numBytes);
void logExitMemorySpan(const int allocType, const int oldMemPtr, const int numBytes);
void logEnterSleepSpan(const struct timespec*time);
void logExitSleepSpan(void);

#define CALL_PRE_MEMORY_OPERATION_ROUTINE(x, y) \
    if (gPreMemOperationRtn != NULL)  \
        (* gPreMemOperationRtn)(x, y);  

#define CALL_POST_MEMORY_OPERATION_ROUTINE(x, y, z) \
    if (gPostMemOperationRtn != NULL)  \
        (* gPostMemOperationRtn)(x, y, z);  

#define CALL_PRE_SLEEP_OPERATION_ROUTINE(x) \
    if (gPreSleepOperationRtn != NULL)  \
        (* gPreSleepOperationRtn)(x);  

#define CALL_POST_SLEEP_OPERATION_ROUTINE() \
    if (gPostSleepOperationRtn != NULL)  \
        (* gPostSleepOperationRtn)();  

/* original uninstrumented VxWorks OS calls can be called with these signatures */
int real_nanosleep(const struct timespec *rqtp, struct timespec *rmtp);
/* LOCAL void * real_memPartAlignedAllocInternal (FAST PART_ID partId, unsigned nBytes, unsigned alignment); */
/* LOCAL STATUS real_memPartFreeInternal (PART_ID partId, char * pBlock); */
STATUS real_memPartAddToPool(FAST PART_ID partId, FAST char *pPool, FAST unsigned poolSize);

#ifdef __cplusplus
}
#endif
