/* adrSpaceLibP.h - Address Space Library header */

/* 
 * Copyright (c) 2004-2005 Wind River Systems, Inc.
 *
 * The right to copy, distribute, modify or otherwise make use 
 * of this software may be licensed only pursuant to the terms 
 * of an applicable Wind River license agreement. 
 */ 

/*
modification history
--------------------
01d,09sep05,zl   removed ADR_SPACE_RGN_CTL structure.
01c,27jun05,zl   added new alloc, free, map, unmap APIs and options.
01b,22sep04,zl   post-inspection fixes.
01a,31aug04,zl   created.
*/

#ifndef __INCadrSpaceLibPh
#define __INCadrSpaceLibPh

#ifdef __cplusplus
extern "C" {
#endif

/* includes */

#include <vxWorks.h>
#include <vmLib.h>

/* address space types */

#define ADR_SPACE_TYPE_KERNEL		0x00000001
#define ADR_SPACE_TYPE_RTP		0x00000000
#define ADR_SPACE_TYPE_MASK		0x00000001

/* 
 * the alloc and map options are defined so that they don't 
 * overlap with the MMU attributes, since they can be OR-ed together.
 */

#define ADR_SPACE_OPT_CONTIG		0x80000000
#define ADR_SPACE_OPT_NONCONTIG		0x00000000
#define ADR_SPACE_OPT_CONTIG_MASK	0x80000000

#define ADR_SPACE_OPT_MAPPED		0x40000000
#define ADR_SPACE_OPT_UNMAPPED		0x00000000
#define ADR_SPACE_OPT_MAPPED_MASK	0x40000000

#define ADR_SPACE_OPT_NOALIAS		0x20000000
#define ADR_SPACE_OPT_ALIAS		0x00000000
#define ADR_SPACE_OPT_ALIAS_MASK	0x20000000


#define ADR_SPACE_OPT_ALLOC_MASK	(ADR_SPACE_OPT_MAPPED_MASK | \
					 ADR_SPACE_OPT_CONTIG_MASK | \
					 ADR_SPACE_OPT_ALIAS_MASK)

#define ADR_SPACE_OPT_MAP_MASK		(ADR_SPACE_OPT_CONTIG_MASK | \
					 ADR_SPACE_OPT_ALIAS_MASK)

/* unmap and free attributes */

#define ADR_SPACE_OPT_PHYS_NOFREE	0x00000001
#define ADR_SPACE_OPT_PHYS_FREE		0x00000000
#define ADR_SPACE_OPT_PHYS_FREE_MASK	0x00000001



#ifndef _ASMLANGUAGE

/* typedefs */

typedef struct adr_space_private_info
    {
    PAGE_POOL_ID	userRgnPoolId;		/* user region free pool */
    PAGE_POOL_ID	kernelRgnPoolId;		/* kernel region free pool */
    PAGE_POOL_ID	kernelVirtPgPoolId;	/* kernel virtual page pool */
    VIRT_ADDR		kernelMemTop;		/* top of kernel memory */
    VIRT_ADDR		localMemLocalAdrs;	/* LOCAL_MEM_LOCAL_ADRS */
    PHYS_MEM_DESC	*pPhysMemDesc;		/* mem.desc table */
    int			physMemDescNumEnt;	/* number of entries */
    
    } ADR_SPACE_PRIVATE_INFO;


typedef struct adrSpaceInitParams
    {
    PHYS_MEM_DESC *	pPhysMemDesc;		/* physMemDesc pointer */
    int 		physMemDescNumEnt;	/* number of elements */
    VIRT_ADDR   	localMemLocalAdrs;	/* where memory begins */
    VIRT_ADDR   	kernelMemTop;		/* top of kernel mem (heap) */
    VIRT_ADDR   	overlapBaseAdr;		/* overlapped rgn base adrs */
    UINT        	overlapMaxSize;		/* overlapped rgn max size */
    } ADR_SPACE_INIT_PARAMS;

/* globals */

extern PAGE_POOL_ID globalRAMPgPoolId;

/* function prototypes */

extern STATUS 	adrSpaceLibInit (VIRT_ADDR localMemLocalAdrs,
				 VIRT_ADDR kernelMemTop,
				 PHYS_MEM_DESC *pPhysMemDesc,
				 int physMemDescNumEnt,
				 VIRT_ADDR overlapBaseAdr,
				 UINT overlapMaxSize);
extern PAGE_POOL_ID adrSpaceCreate (UINT options);
extern PAGE_POOL_ID adrSpaceCreateAt (VIRT_ADDR virtAdr, UINT minBytes,
				      UINT options);
extern STATUS	adrSpaceDelete (PAGE_POOL_ID pgPoolId);
extern VIRT_ADDR adrSpacePageAlloc (VM_CONTEXT_ID vmContext, VIRT_ADDR virtAddr,
				    PHYS_ADDR physAddr, UINT numPages,
				    UINT options);
extern STATUS	adrSpacePageFree (VM_CONTEXT_ID vmContext, VIRT_ADDR virtAddr,
				  UINT numPages, UINT options);
extern STATUS	adrSpacePageMap (VM_CONTEXT_ID vmContext, VIRT_ADDR virtAdr, 
				 PHYS_ADDR physAdr, UINT numPages, 
				 UINT options);
extern STATUS	adrSpacePageUnmap (VM_CONTEXT_ID vmContext, VIRT_ADDR virtAdr, 
				   UINT numPages, UINT options);
extern void	adrSpacePrivateInfoGet (ADR_SPACE_PRIVATE_INFO * pInfo);
extern STATUS 	adrSpaceShowInit ();

#endif /* _ASMLANGUAGE */

#ifdef __cplusplus
}
#endif

#endif /* __INCadrSpaceLibPh */
