/* pgMgrLibP.h - page manager library private header */

/* 
 * Copyright (c) 1999-2005 Wind River Systems, Inc. 
 *
 *
 * The right to copy, distribute, modify or otherwise make use 
 * of this software may be licensed only pursuant to the terms 
 * of an applicable Wind River license agreement. 
 */ 

/*
modification history
--------------------
01i,07sep05,zl   added numPagesMapped to PAGE_MGR_OBJ.
01h,27jun05,zl   cleanup based on new adrSpaceLib APIs.
01g,03jun05,yvp  Added #ifndef	_ASMLANGUAGE. 
                 Updated copyright. #include now with angle-brackets.
01f,24jan05,aeg  added pgMgrPrivateUnmap() function prototype (SPR #106381).
01e,13sep04,zl   cleaned up MM private API
01d,10sep04,gls  cleanup
01c,08jul04,tam  changed PAGE_MGR_ATTR_ALLOC_CONTIG_DEFAULT to be 
		 PAGE_MGR_ATTR_ALLOC_CONTIG
01b,19may04,tam  moved definition from pgMgrLib.h
01a,22oct03,gls  proted from AE version 01b
*/

#ifndef __INCpgMgrLibPh
#define __INCpgMgrLibPh

#ifdef __cplusplus
extern "C" {
#endif

/* includes */

#include <vxWorks.h>
#include <dllLib.h>
#include <vmLib.h>
#include <private/adrSpaceLibP.h>

/* the page manager can set protection, cache and SPL attributes */

#define PAGE_MGR_ATTR_MMU_MSK		(MMU_ATTR_PROT_MSK | \
					 MMU_ATTR_CACHE_MSK | \
					 MMU_ATTR_SPL_MSK)

#define PAGE_MGR_ATTR_MMU_GET(attr)	((attr) & PAGE_MGR_ATTR_MMU_MSK)

#ifndef	_ASMLANGUAGE

/* typedefs */

typedef unsigned int PAGE_MGR_ATTR;
typedef unsigned int PAGE_MGR_OPTS;

typedef struct pgMgrObj
    {
    PAGE_POOL_ID	virtPgPoolId;	/* virtual page pool */
    PAGE_POOL_ID 	physPgPoolId; 	/* physical page pool */
    VM_CONTEXT_ID	vmContextId;	/* VM context ID */
    DL_LIST		mmapList;	/* list of mmap blocks */ 
    PAGE_MGR_OPTS	options;        /* page manager options */
    UINT		numPagesMapped;	/* number of mapped pages */
    } PAGE_MGR_OBJ;

typedef struct pgMgrInfo
    {
    UINT		numPages;	/* number of pages */
    UINT		numPagesMapped;	/* number of pages */
    } PAGE_MGR_INFO;

/* function prototypes */

STATUS	    pgMgrLibInit (void);
PAGE_MGR_ID pgMgrCreate (PAGE_MGR_OPTS options);
STATUS	    pgMgrDelete (PAGE_MGR_ID pgMgrId);
VIRT_ADDR   pgMgrPageAlloc (PAGE_MGR_ID pgMgrId, UINT numPages,
                          PAGE_MGR_ATTR attr);
VIRT_ADDR   pgMgrPageAllocAt (PAGE_MGR_ID pgMgrId, VIRT_ADDR virtAdr,
                            PHYS_ADDR physAdr, UINT numPages,
                            PAGE_MGR_ATTR attr);
STATUS 	    pgMgrPageFree (PAGE_MGR_ID pgMgrId, VIRT_ADDR virtAdr,
                              UINT numPages);
STATUS	    pgMgrPageMap (PAGE_MGR_ID pgMgrId, VIRT_ADDR virtAdr,
                          PHYS_ADDR physAdr, UINT numPages, PAGE_MGR_ATTR attr);
STATUS	    pgMgrPageUnmap (PAGE_MGR_ID, VIRT_ADDR, UINT);
STATUS	    pgMgrPageAttrSet (PAGE_MGR_ID pgMgrId, VIRT_ADDR virtAdr,
                              UINT numPages, MMU_ATTR mmuAttr);
STATUS	    pgMgrPageAttrGet (PAGE_MGR_ID pgMgrId, VIRT_ADDR virtAdr,
                              MMU_ATTR * pMmuAttr);
STATUS	    pgMgrVirtToPhys (PAGE_MGR_ID pgMgrId, VIRT_ADDR virtAdr,
                             PHYS_ADDR * pPhysAdr);
STATUS	    pgMgrPhysToVirt (PAGE_MGR_ID pgMgrId, PHYS_ADDR physAdr,
                             VIRT_ADDR * pVirtAdr);
PAGE_MGR_ID pgMgrInit (PAGE_MGR_OBJ * pPgMgr, PAGE_MGR_OPTS opts);
void        pgMgrShowInit (void);
STATUS	    pgMgrPrivateUnmap (PAGE_MGR_ID pgMgrId);
STATUS	    pgMgrInfoGet (PAGE_MGR_ID pgMgrId, PAGE_MGR_INFO * pgMgrInfo);

#endif	/* _ASMLANGUAGE */

#ifdef __cplusplus
}
#endif

#endif /* __INCpgMgrLibPh */
