/*---------------------------------------------------------------------------
               ----> DO NOT REMOVE THE FOLLOWING NOTICE <----

      Copyright (c) 2003 - 2005 Datalight, Inc.  All Rights Reserved.

  Datalight, Incorporated is a Washington State corporation located at:

        21520 30th Dr SE, Suite 110,      Tel:  425-951-8086
        Bothell, WA  98021                Fax:  425-951-8094
        USA                               Http://www.datalight.com

  This software, including without limitation, all source code and documen-
  tation, is the confidential, trade secret property of Datalight, Inc., and
  is protected under the copyright laws of the United States and other juris-
  dictions.

  ---> Portions of the software are patent pending <---

  In addition to civil penalties for infringement of copyright under applic-
  able U.S. law, 17 U.S.C. 1204 provides criminal penalties for violation of
  (a) the restrictions on circumvention of copyright protection systems found
  in 17 U.S.C. 1201 and (b) the protections for the integrity of copyright
  management information found in 17 U.S.C. 1202.

  U.S. Government Restricted Rights:  Use, duplication, reproduction, or
  transfer of this commercial product and accompanying documentation is
  restricted in accordance with FAR 12.212 and DFARS 227.7202 and by a
  License Agreement.

  IN ADDITION TO COPYRIGHT AND PATENT LAW, THIS SOFTWARE IS PROTECTED UNDER A
  SOURCE CODE AGREEMENT, NON-DISCLOSURE AGREEMENT (NDA), OR SIMILAR BINDING
  CONTRACT BETWEEN DATALIGHT, INC. AND THE LICENSEE ("BINDING AGREEMENT").
  YOUR RIGHT, IF ANY, TO COPY, PUBLISH, MODIFY OR OTHERWISE USE THE SOFTWARE,
  IS SUBJECT TO THE TERMS AND CONDITIONS OF THE BINDING AGREEMENT.  BY USING
  THE SOFTWARE IN ANY MANNER, IN WHOLE OR IN PART, YOU AGREE TO BE BOUND BY
  THE TERMS OF THE BINDING AGREEMENT.  CONTACT DATALIGHT, INC. AT THE ADDRESS
  SET FORTH ABOVE IF YOU OBTAINED THIS SOFTWARE IN ERROR.
---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------
                              Description

    This header file defines the Nucleus specific API function interface
    to Reliance.

    This header file, along with any sub-headers it may include, defines
    the entire set of structures, macros, and function prototypes necessary
    to write a program that interfaces with Reliance.
---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------
                              Revision History
    $Log: reliance_nucleus.h $
    Revision 1.3  2006/03/22 03:12:47  Pauli
    Changed FILE compatibility to 2.5 by default.
    Revision 1.2  2005/12/19 18:55:22Z  pauli
    Added support for Nucleus FILE 3.1.
    Revision 1.1  2005/12/07 01:14:00Z  Pauli
    Initial revision
    Revision 1.4  2005/12/07 01:13:59Z  Pauli
    Updated to only include dcl_nucleus.h if necessary.
    Revision 1.3  2005/12/02 18:05:44Z  Pauli
    Merge with 2.0 product line, build 458.
    Revision 1.4  2005/10/27 22:31:24Z  Pauli
    Added configuration option to support the Nucleus POSIX component.
    Revision 1.3  2005/09/15 19:44:36Z  Pauli
    Added support for FILE MAXSECTORS setting.
    Revision 1.2  2005/05/10 21:29:02Z  Pauli
    Changed rloilcmn.h to be included using a relative path.
    Revision 1.1  2005/05/05 15:59:12Z  PaulI
    Initial revision

---------------------------------------------------------------------------*/

#ifndef RELIANCE_NUCLEUS_H_INCLUDED
#define RELIANCE_NUCLEUS_H_INCLUDED


#ifndef DCLFUNC
  #define DCLFUNC(fn)                 (Rel##fn)
#endif


#ifndef DCL_H_INCLUDED
  #ifndef DCL_NUCLEUS_H_INCLUDED
    /*  If neither dcl.h nor dcl_nucleus.h have already been included
        by higher level headers, we must be here because we're being
        included by externally built software which will not have
        the DCL include path.  Therefore use a path relative to the
        current directory.
    */                
    #include "..\dcl\include\dcl_nucleus.h"
  #endif
#endif


/*  Nucleus defines
*/
#include <plus\nucleus.h>

/*  Reliance defines for transaction mode flags.
*/
#include "rloilcmn.h"


/****************************************************************
    BEGIN Set FILE Compatibility Mode
****************************************************************/


/*  Determine Nucleus FILE compatibility mode.
*/
#ifdef DCL_NUCLEUS_FILE_VERSION

/*  Set the Nucleus FILE compatibility mode based on the Reliance
    build process settings.
*/
#if DCL_NUCLEUS_FILE_VERSION >= DCL_NUCLEUS_FILE_3_X
#define REL_FILE_3_X_SUPPORT    1
#else
#define REL_FILE_3_X_SUPPORT    0
#endif

#else

/*  Manually set the Nucleus FILE compatibility mode.
*/
#define REL_FILE_3_X_SUPPORT    0

#endif /* #ifdef DCL_NUCLEUS_FILE_VERSION */


/****************************************************************
    END Set FILE Compatibility Mode
****************************************************************/


/****************************************************************
    BEGIN FILE Definitions
****************************************************************/

#if REL_FILE_3_X_SUPPORT

/*  Nucleus FILE 3.1 Compatibility Definitions
*/


#include <file\pcdisk.h>

/*  The definitions needed in this file should be moved to pcdisk.h.

    NUSERFILES
    MAXSECTORS
    ANORMAL
    ARDONLY
    AHIDDEN
    ASYSTEM
    AVOLUME
    ADIRENT
    ARCHIVE
*/
#include <file\fs\fat\inc\fat_defs.h>


/*  format parameters
*/
typedef struct _REL_FMT_PARAMS
{
    UINT32      ulBlockSize;
}REL_FMT_PARAMS;


STATUS          REL_Init_FS(CHAR *pszFsName);


/*  TODO:
    FILE 3.1 will soon have these defined and we will not need these.
*/

/*  IOCTL commands for talking to the block device
*/
#define REL_IOCTL_BASE              0x5100
#define REL_IOCTL_DISCARD_SUPPORT   (REL_IOCTL_BASE + 1)
#define REL_IOCTL_DISCARD           (REL_IOCTL_BASE + 2)
#define REL_IOCTL_FLUSH             (REL_IOCTL_BASE + 3)
#define REL_IOCTL_PARAMS            (REL_IOCTL_BASE + 4)


/*  structure for passing discard parameters through ioctl interface
*/
typedef struct
{
    UINT32          ulStartSector;
    UINT32          ulNumSectors;
} DISCARD_PARAMS;



#else


/*  Nucleus FILE 2.5 Compatibility Definitions
*/
#include "..\os\nucleus\include\relFs_FILE_25.h"


#endif /* #if REL_FILE_3_X_SUPPORT */


/****************************************************************
    END FILE Definitions
****************************************************************/



/****************************************************************
    BEGIN Reliance Definitions
****************************************************************/


/*  Maximum file size supported by the signed 32-bit API
*/
#define REL_NU_MAX_FILESIZE (0x7FFFFFFFL)


/*  Maximum Number of open Files/Directories per drive
*/
#define REL_MAX_OPEN_FILES  (NUSERFILES)


/* Maximum number of sectors that will be transferred to/from any block device.
*/
#define REL_MAX_SECTORS   (MAXSECTORS)


/*  Seek Flags

    For compatibility, these are the same values used by Nucleus FILE.
*/
#define REL_SEEK_SET    PSEEK_SET   /* seek from beginning of file */
#define REL_SEEK_CUR    PSEEK_CUR   /* seek from current offset */
#define REL_SEEK_END    PSEEK_END   /* seek from end of file */


/*  Nucleus port specific error codes.

    For compatibility, these are the same values used by Nucleus FILE.
*/
#define REL_SUCCESS             NU_SUCCESS
#define REL_ERR_ACCES           NUF_ACCES
#define REL_ERR_BADUSER         NUF_BAD_USER
#define REL_ERR_BADDRIVE        NUF_BADDRIVE
#define REL_ERR_BADFILE         NUF_BADFILE
#define REL_ERR_BADPARM         NUF_BADPARM
#define REL_ERR_BLOCKSIZE       NUF_INVALID_CSIZE
#define REL_ERR_EXIST           NUF_EXIST
#define REL_ERR_FORMAT          NUF_FORMAT
#define REL_ERR_INVNAME         NUF_INVNAME
#define REL_ERR_INVPARCMB       NUF_INVPARCMB
#define REL_ERR_INTERNAL        NUF_INTERNAL
#if DCL_NUCLEUS_FILE_VERSION >= DCL_NUCLEUS_FILE_3_X
#define REL_ERR_IN_USE          NUF_IN_USE
#else
#define REL_ERR_IN_USE          NUF_ACCES
#endif
#define REL_ERR_IO_ERROR        NUF_IO_ERROR
#define REL_ERR_LONGPATH        NUF_LONGPATH
#define REL_ERR_MAXFILE_SIZE    NUF_MAXFILE_SIZE
#define REL_ERR_NO_MEMORY       NUF_NO_MEMORY
#define REL_ERR_NOEMPTY         NUF_NOEMPTY
#define REL_ERR_NOFILE          NUF_NOFILE
#define REL_ERR_NOSPC           NUF_NOSPC
#define REL_ERR_NOT_OPENED      NUF_NOT_OPENED
#define REL_ERR_PEMFILE         NUF_PEMFILE
#define REL_ERR_SHARE           NUF_SHARE


/*  Nucleus port specific file and directory attributes.

    For compatibility, these are the same values used by Nucleus FILE.
*/
#define NU_ATTR_NORMAL      ANORMAL
#define NU_ATTR_READ_ONLY   ARDONLY
#define NU_ATTR_HIDDEN      AHIDDEN
#define NU_ATTR_SYSTEM      ASYSTEM
#define NU_ATTR_VOL_LABEL   AVOLUME
#define NU_ATTR_DIRECTORY   ADIRENT
#define NU_ATTR_ARCHIVE     ARCHIVE



/*-------------------------------------------------------------------
    Create masks for all the supported and unsupported
    transaction types.
-------------------------------------------------------------------*/
#define REL_TRANSACT_SUPPORTED_MASK         \
(                                           \
    REL_TRANSACT_MANUAL        |            \
    REL_TRANSACT_DIR_CREATE    |            \
    REL_TRANSACT_DIR_DELETE    |            \
    REL_TRANSACT_DIR_RENAME    |            \
    REL_TRANSACT_FILE_CREATE   |            \
    REL_TRANSACT_FILE_WRITE    |            \
    REL_TRANSACT_FILE_FLUSH    |            \
    REL_TRANSACT_FILE_TRUNCATE |            \
    REL_TRANSACT_FILE_CLOSE    |            \
    REL_TRANSACT_FILE_RENAME   |            \
    REL_TRANSACT_FILE_DELETE   |            \
    REL_TRANSACT_ATTRIBUTES    |            \
    REL_TRANSACT_VOL_FULL                   \
)

#define REL_TRANSACT_UNSUPPORTED_MASK (~(REL_TRANSACT_SUPPORTED_MASK))

#define REL_TRANSACT_DEFAULT                \
(                                           \
    REL_TRANSACT_DIR_CREATE    |            \
    REL_TRANSACT_DIR_DELETE    |            \
    REL_TRANSACT_DIR_RENAME    |            \
    REL_TRANSACT_FILE_CLOSE    |            \
    REL_TRANSACT_FILE_FLUSH    |            \
    REL_TRANSACT_FILE_RENAME   |            \
    REL_TRANSACT_FILE_DELETE   |            \
    REL_TRANSACT_ATTRIBUTES    |            \
    REL_TRANSACT_VOL_FULL                   \
)


/*  relFs_Check() bit flag options
*/
#define REL_CHK_OPT_VERBOSE     0x01
#define REL_CHK_OPT_NOPROMPT    0x02
#define REL_CHK_OPT_DUMP_INDEX  0x04
#define REL_CHK_OPT_QUIET       0x08
#define REL_CHK_OPT_SHOW_ERRORS 0x10

#define REL_CHK_OPT_DEFAULT     (  \
      REL_CHK_OPT_NOPROMPT      |  \
      REL_CHK_OPT_SHOW_ERRORS   )


/*  Reliance specific API.
*/
STATUS          relFs_Show(CHAR *pszDrive);
STATUS          relFs_Check(CHAR *pszDrive, UINT32 ulOptions);
STATUS          relFs_Test(CHAR *pszCmdLine);
STATUS          relFs_TransactionPoint(CHAR *pszDrive);
STATUS          relFs_SetTransactionMode(CHAR *pszDrive, UINT32 ulMode);
STATUS          relFs_GetTransactionMode(CHAR *pszDrive, UINT32 *pulMode);
STATUS          relFs_SetTimedThreshold(CHAR *pszDrive, UINT32 ulThreshold);
STATUS          relFs_GetTimedThreshold(CHAR *pszDrive, UINT32 *pulThreshold);
#if REL_FILE_3_X_SUPPORT
STATUS          relFs_FormatDevice(CHAR *pszDevice, CHAR *pszFileSystem, UINT32 ulBlockSize);
STATUS          relFs_FormatMountedDrive(CHAR *pszDrive, UINT32 ulBlockSize);
#endif
STATUS          relFs_FormatOpenDrive(CHAR *pszDrive, UINT32 ulBlockSize);
STATUS          relFs_IsRelianceDrive(CHAR *pszDrive);


/*  This is the Reliance Move API. Nucleus FILE does not have a Move API.
    NU_Rename() does not support move operations.  
*/
STATUS          relFs_Move(CHAR *pszName, CHAR * szNewName);


/****************************************************************
    END Reliance Definitions
****************************************************************/


#endif
