/*---------------------------------------------------------------------------
               ----> DO NOT REMOVE THE FOLLOWING NOTICE <----

                  Copyright (c) 2003 - 2005 Datalight, Inc.
                    All Rights Reserved.  Patent Pending.

  Datalight, Incorporated is a Washington State corporation located at:

        21520 30th Dr SE, Suite 110,      Tel:  425-951-8086
        Bothell, WA  98021                Fax:  425-951-8094
        USA                               Http://www.datalight.com

  This software, including without limitation, all source code and documen-
  tation, is the confidential, trade secret property of Datalight, Inc., and
  is protected under the copyright laws of the United States and other juris-
  dictions.

  In addition to civil penalties for infringement of copyright under applic-
  able U.S. law, 17 U.S.C. 1204 provides criminal penalties for violation of
  (a) the restrictions on circumvention of copyright protection systems found
  in 17 U.S.C. 1201 and (b) the protections for the integrity of copyright
  management information found in 17 U.S.C. 1202.

  U.S. Government Restricted Rights:  Use, duplication, reproduction, or
  transfer of this commercial product and accompanying documentation is
  restricted in accordance with FAR 12.212 and DFARS 227.7202 and by a
  License Agreement.

  IN ADDITION TO COPYRIGHT AND PATENT LAW, THIS SOFTWARE IS PROTECTED UNDER A
  SOURCE CODE AGREEMENT, NON-DISCLOSURE AGREEMENT (NDA), OR SIMILAR BINDING
  CONTRACT BETWEEN DATALIGHT, INC. AND THE LICENSEE ("BINDING AGREEMENT").
  YOUR RIGHT, IF ANY, TO COPY, PUBLISH, MODIFY OR OTHERWISE USE THE SOFTWARE,
  IS SUBJECT TO THE TERMS AND CONDITIONS OF THE BINDING AGREEMENT.  BY USING
  THE SOFTWARE IN ANY MANNER, IN WHOLE OR IN PART, YOU AGREE TO BE BOUND BY
  THE TERMS OF THE BINDING AGREEMENT.  CONTACT DATALIGHT, INC. AT THE ADDRESS
  SET FORTH ABOVE IF YOU OBTAINED THIS SOFTWARE IN ERROR.
---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------
                              Description
---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------
                              Revision History
    $Log: rlcorapi.h $
    Revision 1.3  2006/03/29 22:04:58  brandont
    Added conditioned feature to look up a path based on an INode index.
    Revision 1.2  2006/02/27 23:14:02Z  brandont
    Added interface to retrieve an INode link count.  Merged from 2.0 code
    base.
    Revision 1.13  2006/01/03 18:15:48Z  johnb
    added TFS_INODE_ATTR_LINKCOUNT and Linux Port
    Revision 1.1  2005/09/02 23:51:50Z  Pauli
    Initial revision
    Revision 1.2  2005/09/03 00:51:49Z  Pauli
    Merge with 2.0 product line, build 458.
    Revision 1.12  2005/09/03 00:51:49Z  brandont
    Added dot-dot emulation helper variables to the sTfsOpenDir structure.
    Revision 1.11  2005/08/26 23:47:30Z  brandont
    Revision 1.10  2005/08/18 23:54:11Z  brandont
    Changed some name space variables in structures to use CDF limits
    instead of port specific limits.  Updated the prototype for
    TfsCoreInitializeVolume.
    Revision 1.9  2005/08/11 21:58:25Z  brandont
    Added an extra argument to the sTfsOpenDir structure for returning the
    index number for the found entry.
    Revision 1.8  2005/08/04 18:01:46Z  brandont
    Added a boolean directory flag in the opendir structure.  Moved the
    sTfsDiskInfo structure from rlcore.h.
    Revision 1.7  2005/07/28 21:09:35Z  brandont
    Moved more of the API specific defines and structure definitions from
    rlcore.h.  Updated the core API function prototypes.
    Revision 1.6  2005/07/19 00:51:57Z  jeremys
    Updated a macro with a new function member name.
    Revision 1.5  2005/06/30 21:58:13Z  brandont
    Moved the TFS_INODE_ATTR_??? set of defines from rlcore.h.  Moved
    the TFS_CDF_ATTR_??? set of defines from rlcore.h. Removed an
    obsolete TfsCoreMatchName function prototype.
    Revision 1.4  2005/06/14 18:33:59Z  PaulI
    Added APIs for getting and setting the volume label.
    Revision 1.3  2005/05/18 23:25:59Z  garyp
    Cleaned up the prototypes for readability.
    Revision 1.2  2005/03/16 17:29:38Z  PaulI
    Added prototype for accessing transaction timestamp.
    Revision 1.1  2005/02/23 01:22:18Z  brandont
    Initial revision
---------------------------------------------------------------------------*/

#ifndef RLCORAPI_H
#define RLCORAPI_H      1


/*  metaroot transaction point modes
*/
#define TFS_TRANMODE_MANUAL     1
#define TFS_TRANMODE_AUTO       2
#define TFS_TRANMODE_TIMED      3
#define TFS_TRANMODE_DEFAULT    TFS_TRANMODE_AUTO


/*  default masks for findfirst / find next
*/
#define TFS_DEFAULT_MUST_MASK    0
#define TFS_DEFAULT_CAN_MASK     TFS_UINT32_MAX


/*  attribute fields accessible by OS Integration Layer (OIL)
*/
#define TFS_INODE_ATTR_FILESIZE     0   /* file size in bytes */
#define TFS_INODE_ATTR_FLAGS        1   /* CDF attributes */
#define TFS_INODE_ATTR_CREATEDATE   2   /* create date/time */
#define TFS_INODE_ATTR_LASTDATE     3   /* last modified date/time */
#define TFS_INODE_ATTR_ACCESSDATE   4   /* last modified date/time */
#define TFS_INODE_ATTR_LINKCOUNT    5   /* link count */


/*  Reliance file/directory attributes
*/
#define TFS_CDF_ATTR_ARCHIVE    0x0100  /* Archive */
#define TFS_CDF_ATTR_SYSTEM     0x0200  /* System */
#define TFS_CDF_ATTR_HIDDEN     0x0400  /* Hidden */
#define TFS_CDF_ATTR_READONLY   0x0800  /* Read-only */
#define TFS_CDF_ATTR_DIRECTORY  0x1000  /* Directory */
#define TFS_CDF_ATTR_LABEL      0x2000  /* Volume label */
#define TFS_CDF_ATTR_MASK       0x3f00  /* Used mask of CDF attributes */


/*  signature for an OpenDir structure
*/
#define TFS_OPENDIR_SIG             0x5249444FUL /* ODIR */


/*  sTfsOpenDir

    Used by TfsCoreOpenDirectory, TfsCoreRewindDirectory,
    TfsCoreReadFromDirectory, and TfsCoreCloseDirectory.
*/
typedef struct {
    tfsUint32 ulSignature;
    sTfsVolume * pVolume;
    tfsIndex ulINodeIndex;
#if TFS_DOTDOT_EMULATION
    tfsIndex ulParentINodeIndex;
    tfsUint8 uDotDotCount;
#endif
    tfsOffset ulEntryOffset;
    tfsChar szName[REL_CDF_NAME_LENGTH + 1];
    tfsBool fIsDir;
    tfsIndex ulIndexForFoundEntry;
} sTfsOpenDir;


/*  Signature for an FindDir structure
*/
#define TFS_FINDDIR_SIG             0x4649444FUL /* FDIR */


/*  sTfsFindDir

    Used by TfsCoreFindFirst, TfsCoreFindNext, and TfsCoreFindClose
*/
typedef struct {
    tfsUint32 ulSignature;
    sTfsOpenDir sOpenDir;
    tfsUint16 uMustMaskAttr;        /* bits that are set MUST be present */
    tfsUint16 uCanMaskAttr;         /* bits that are set CAN be present */
    tfsChar szNameMask[REL_CDF_NAME_LENGTH + 1];
} sTfsFindDir;


/*  sTfsDiskInfo

    Used by TfsCoreGetVolumeInfo
*/
struct TfsTagDiskInfo
{
    tfsUint32       iBlockSize; /* size of a block */
    tfsBlock        iNumBlocks; /* # of blocks on volume */
    tfsBlock        iFreeBlocks;        /* # of free blocks on volume */
    tfsBlock        iUsedBlocks;        /* # of used blocks on volume */
    tfsTime         ullCreationDate;    /* format timestamp */
};
typedef struct TfsTagDiskInfo sTfsDiskInfo;

/*
    The OS Integration Layer uses these Reliance APIs
*/


/*  TFS Core entry points
*/
TFSERR TfsCoreInitialize( void );
TFSERR TfsCoreUninitialize( void );
TFSERR TfsCoreInitializeVolume( sTfsVolume ** ppVolume,
    sOilBlkDevInfo * pBlkDevInfo,
    tfsUint32 ulPortPathLengthLimit,
    tfsUint32 ulPortNameLengthLimit,
    tfsChar * szPortIllegalCharacters );
TFSERR TfsCoreUninitializeVolume( sTfsVolume * pVolume);
TFSERR TfsCoreMount( sTfsVolume * pVolume);
TFSERR TfsCoreUnMount( sTfsVolume * pVolume);
TFSERR TfsCoreTransactionPoint( sTfsVolume * pVolume);
TFSERR TfsCoreIoctl( sTfsVolume * pVolume, int iIoctlCmd, void * pBuffer);
TFSERR TfsCoreGetVolumeInfo( sTfsVolume * pVolume, sTfsDiskInfo * pDiskInfo);
TFSERR TfsCoreMakeDirectory( sTfsVolume * pVolume, const tfsChar * szPath, tfsUint32 ulAttributes);
TFSERR TfsCoreRemoveDirectory( sTfsVolume * pVolume, const tfsChar * szPath);
TFSERR TfsCoreOpenDirectory( sTfsVolume * pVolume, const tfsChar * szPath, sTfsOpenDir * pOpenDir);
TFSERR TfsCoreCreateFile( sTfsVolume * pVolume, const tfsChar * szPath, tfsUint32 ulAttributes, tfsHandle * piHandle);
TFSERR TfsCoreOpenFile( sTfsVolume * pVolume, const tfsChar * szPath, tfsHandle * piHandle);
TFSERR TfsCoreReadFromFile( sTfsVolume * pVolume, tfsHandle iHandle, tfsOffset iOffset, void * pBuffer, tfsOffset * piLength);
TFSERR TfsCoreWriteToFile( sTfsVolume * pVolume, tfsHandle iHandle, tfsOffset iOffset, void * pBuffer, tfsOffset * piLength);
TFSERR TfsCoreTruncateFile( sTfsVolume * pVolume, tfsHandle iHandle, tfsOffset iOffset);
TFSERR TfsCoreCloseFile( sTfsVolume * pVolume, tfsHandle iHandle);
TFSERR TfsCoreDeleteFile( sTfsVolume * pVolume, const tfsChar * szPath);
TFSERR TfsCoreRename( sTfsVolume * pVolume, const tfsChar * szSrcPath, const tfsChar * szDstPath);
TFSERR TfsCoreAddHardLink( sTfsVolume * pVolume, const tfsChar * szExistingPath, const tfsChar * szHardLinkPath);
TFSERR TfsCoreSetAttributes( sTfsVolume * pVolume, tfsHandle iHandle, int iField, void * piValue);
TFSERR TfsCoreGetAttributes( sTfsVolume * pVolume, tfsHandle iHandle, int iField, void * piValue);
TFSERR TfsCoreExist( sTfsVolume * pVolume, const tfsChar * szPath, tfsUint32 * pulAttributes, tfsHandle * piHandle);
TFSERR TfsCoreGetTransactionTimeStamp( sTfsVolume * pVolume, tfsTime * pullTime);
TFSERR TfsCoreFindFirst( sTfsVolume * pVolume, const tfsChar * szPath, tfsUint16 uMustMask, tfsUint16 uCanMask, sTfsFindDir * pFindDir);
TFSERR TfsCoreFindNext( sTfsFindDir * pFindDir);
TFSERR TfsCoreFindClose( sTfsFindDir * pFindDir);
TFSERR TfsCoreReadFromDirectory( sTfsOpenDir * pOpenDir);
TFSERR TfsCoreRewindDirectory( sTfsOpenDir * pOpenDir);
TFSERR TfsCoreCloseDirectory( sTfsOpenDir * pOpenDir);
TFSERR TfsCoreSetVolumeLabel( sTfsVolume * pVolume, const tfsChar * szLabel);
TFSERR TfsCoreGetVolumeLabel( sTfsVolume * pVolume, tfsChar * szLabel );
TFSERR TfsCoreIsValidGeometry( const sTfsGeometry * pGeometry, tfsBool * pfIsValid );
#if REL_FEATURE_REVERSE_LOOKUP
TFSERR TfsCoreHandleToPath( sTfsVolume * pVolume, tfsHandle ulHandle, tfsChar * szPath );
#endif

#endif /* #ifndef RLCORAPI_H */
