/*---------------------------------------------------------------------------
               ----> DO NOT REMOVE THE FOLLOWING NOTICE <----

                  Copyright (c) 2003 - 2005 Datalight, Inc.
                    All Rights Reserved.  Patent Pending.

  Datalight, Incorporated is a Washington State corporation located at:

        21520 30th Dr SE, Suite 110,      Tel:  425-951-8086
        Bothell, WA  98021                Fax:  425-951-8094
        USA                               Http://www.datalight.com

  This software, including without limitation, all source code and documen-
  tation, is the confidential, trade secret property of Datalight, Inc., and
  is protected under the copyright laws of the United States and other juris-
  dictions.

  In addition to civil penalties for infringement of copyright under applic-
  able U.S. law, 17 U.S.C. 1204 provides criminal penalties for violation of
  (a) the restrictions on circumvention of copyright protection systems found
  in 17 U.S.C. 1201 and (b) the protections for the integrity of copyright
  management information found in 17 U.S.C. 1202.

  U.S. Government Restricted Rights:  Use, duplication, reproduction, or
  transfer of this commercial product and accompanying documentation is
  restricted in accordance with FAR 12.212 and DFARS 227.7202 and by a
  License Agreement.

  IN ADDITION TO COPYRIGHT AND PATENT LAW, THIS SOFTWARE IS PROTECTED UNDER A
  SOURCE CODE AGREEMENT, NON-DISCLOSURE AGREEMENT (NDA), OR SIMILAR BINDING
  CONTRACT BETWEEN DATALIGHT, INC. AND THE LICENSEE ("BINDING AGREEMENT").
  YOUR RIGHT, IF ANY, TO COPY, PUBLISH, MODIFY OR OTHERWISE USE THE SOFTWARE,
  IS SUBJECT TO THE TERMS AND CONDITIONS OF THE BINDING AGREEMENT.  BY USING
  THE SOFTWARE IN ANY MANNER, IN WHOLE OR IN PART, YOU AGREE TO BE BOUND BY
  THE TERMS OF THE BINDING AGREEMENT.  CONTACT DATALIGHT, INC. AT THE ADDRESS
  SET FORTH ABOVE IF YOU OBTAINED THIS SOFTWARE IN ERROR.
---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------
                              Description
    TFS is first and foremost a reliable file system with high performance.
    TFS is fully rentrant, allowing multiple threads/tasks to use it
    simulataneously, even while transactioning to disk.

    The disk contains four structures to track the disk data: the MetaRoot,
    iMap, iNodes and the Index. Each structure takes up one or more blocks.

    The MetaRoot is the root of everything valid. Two copies of the MetaRoot
    are created on the disk.

    The iMap lists each block available to the file system. Each file and/or
    directory is kept track of through use of an iNode. The iNode contains
    everything

    The iNodes keep track of "files", which includes both files and
    directories. Directory entries point to iNodes, which allows for
    hardlinks. iNodes point to indirect blocks for large files.

    The Index keeps track of certain meta data: iNodes and indirect blocks.
---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------
                              Revision History
    $Log: rlcore.h $
    Revision 1.5  2006/03/29 22:47:18  brandont
    Added conditioned feature to look up a path based on an INode index.
    Revision 1.4  2006/03/16 20:37:55Z  pauli
    Added check to see if TFS_FILENUM is defined before using it to
    prevent build warnings.
    Revision 1.3  2006/02/07 01:56:40Z  brandont
    Conditioned TFS_MAX_BLOCK_SIZE so that it would be possible to
    override the CDF settings even in obfuscated code by defining
    TFS_MAX_BLOCK_SIZE in the relconf.h file.
    Revision 1.2  2006/01/26 04:24:43Z  brandont
    Merged with Reliance 2.0 code base.
    Revision 1.35  2005/12/07 23:57:40Z  brandont
    Corrected comments for sTfsDirEntry.
    Revision 1.1  2005/12/02 23:52:44Z  Pauli
    Initial revision
    Revision 1.2  2005/12/02 23:52:44Z  Pauli
    Merge with 2.0 product line, build 458.
    Revision 1.34  2005/09/28 01:03:34Z  brandont
    Added padding bytes to the sTfsVolume and sTfsINode structures to
    provide 64-bit alignment of the tfsTime members.
    Revision 1.33  2005/08/23 23:41:47Z  brandont
    Implemented fixed sized packets used to store directory entry and the
    entry name.
    Revision 1.32  2005/08/19 00:01:50Z  brandont
    Removed the unused structure sTfsThreadInfo.  Added fields to
    sTfsVolume structure for port specific name limits, path limits, and
    illegal characters.
    Revision 1.31  2005/08/15 23:17:20Z  brandont
    Added a define for the minimum number of logical blocks required to
    format a Reliance volume.
    Revision 1.30  2005/08/12 22:50:53Z  brandont
    Added a define for a default volume label.
    Revision 1.29  2005/08/11 00:45:46Z  brandont
    Removed two unused temp path variables in the sTfsVolume structure.
    Revision 1.28  2005/08/05 04:25:11Z  brandont
    Set the size of sTfsVolume ulIMapeEntries to 32-bits.
    Revision 1.27  2005/08/04 18:01:46Z  brandont
    Moved the sTfsDiskInfo structure to rlcorapi.h.
    Revision 1.26  2005/07/28 19:30:40Z  brandont
    Reserved an index entry for the volume label. Created an entry specific
    directory attribute for volume label.  Changed the size of the entry length
    field to 32-bits in the directory entry structure.  Removed obsolete
    structures and defines.  Changed the ALL flags attribute define to be a
    16-bit value.  Moved the buffer specific defines to rlbuffer.h.
    Revision 1.25  2005/07/08 01:53:47Z  brandont
    Renamed ulNodeIndex in sTfsDirEntry structure to ulINodeIndex.
    Revision 1.24  2005/07/08 00:51:12Z  brandont
    Updated the TfsDiscardCapable and TfsRemoveCapable macros
    with the new sTfsVolume structure member names.
    Revision 1.23  2005/07/06 02:20:02Z  brandont
    Moved the TFS_INODE_ATTR_??? set of defines to rlcorapi.h.  Moved
    the TFS_CDF_ATTR_??? set of defines to rlcorapi.h. Renamed
    sTfsVolume structure members.  Renamed sTfsMaster structure
    members.  Renamed sTfsMetaRoot structure members. Renamed
    sTfsINode structure members.  Renamed sTfsIndirect structure
    members.  Renamed sTfsDirEntry structure members.  Updated all
    structures and defines for use with the new on disk format.  Removed
    obsolete defines.  General reformatting.  Removed obsolete structures.
    Added defines for hard coded index entries (bad file et al.).
    Revision 1.22  2005/06/24 00:39:26Z  brandont
    Added the reliance specific options that are no longer port specific with
    the introduction of CDF.  Incremented the on-disk version number thus
    sacraficing backwards compatibility.
    Revision 1.21  2005/06/14 18:35:41Z  PaulI
    Added define for volume label file name.  Added the volume creation
    timestamp to the volume and disk info structures.
    Revision 1.20  2005/05/13 20:56:22Z  garyp
    Minor formatting changes -- no functional changes.
    Revision 1.19  2005/04/29 20:19:49Z  brandont
    Removed externs for the cache globals.
    Revision 1.18  2005/04/28 00:59:39Z  PaulI
    Added discard enable condition around discard extent table code.
    Revision 1.17  2005/04/27 00:18:47Z  brandont
    Made TFS_INODE_FB_CHANGED obsolete.
    Revision 1.16  2005/04/20 02:15:13Z  brandont
    Renamed the iMapNextAlloc field of the meta root structure to
    iMapNextFree.  Removed the iMapNextFree field from the TfsVolume structure.
    Revision 1.15  2005/04/19 17:49:53Z  PaulI
    Reset layout version to be compatible with previous release.
    Revision 1.14  2005/04/15 23:23:19Z  Pauli
    Set layout to exclusive XP/Nucleus number.
    Revision 1.13  2005/03/25 23:30:36Z  Pauli
    Removed 64-bit file size define.
    Revision 1.12  2005/03/22 22:37:08Z  brandont
    Increased the demo transaction limit to 50,000.
    Revision 1.11  2005/03/22 19:16:11Z  brandont
    Removed define for TFS_SIZEOF_TfsTime.  Added the reserved fields
    to the sizeof define TFS_SIZEOF_sTfsMaster.  Reduced the size of
    TFS_MASTER_RESERVED for backwards compatibility.  Moved the
    typedef for sTfsVolume to rltypes.h.  Moved an IOCTL function number
    and IOCTL function structure to rlioctl.h.
    Revision 1.10  2005/03/16 16:19:52Z  PaulI
    Added last transaction timestamp to volume structure.
    Revision 1.9  2005/03/11 02:12:48Z  brandont
    Removed an obsolete global extern reference.
    Revision 1.8  2005/03/11 00:42:10Z  PaulI
    Added logic to save the last allocated or freed index offset to reduce
    the time it takes to find a free entry.
    Revision 1.7  2005/02/23 00:16:13Z  brandont
    Added include for rlextent.h since the extent structures are used by
    several of the core APIs.  Removed the generic types used throughout
    Reliance into rltypes.h.  Expanded the macros that dereferenced
    global volume structures.  Moved the extent structure types to the
    rlextent.h module.  Assigned structure names for IO and OIL data so
    that void pointers could be avoided.  Removed the macros that
    dereference global structure members.  Made gpTfsVolumeList private
    to the driver.  Changed the IO capabilities macros to take a
    sTfsVolume * instead of using the global volume structure.  Added the
    IMap structure definition.  Added a progress function pointer type.
    Removed the I/O bucket concept.
    Revision 1.6  2004/12/31 01:17:52Z  GaryP
    Updated to use indents of 4, hard tabs of 8 (if used), and a standard
    comment style.
    Revision 1.5  2004/12/08 01:36:56Z  brandont
    Added defines for the index file name and the bad block file name.  Removed
    old tfs version strings and numbers.
    Revision 1.4  2004/12/01 21:02:27Z  brandont
    Removed the memory debug macros and defines from this module.
    Revision 1.3  2004/11/29 21:07:21Z  brandont
    Removed the recently added pLogHandle field from the pVolume structure.
    Revision 1.2  2004/11/22 23:25:28Z  brandont
    Added a typedef for log handle.  Moved the TFS error codes to rlerr.h.
    Revision 1.1  2004/09/23 00:50:08Z  brandont
    Initial revision
---------------------------------------------------------------------------*/

#ifndef TFS_CORE_H
#define TFS_CORE_H   1


/*  Enable the ability to look up paths based on a handle (INode index)

    This feature is incompatible with Reliance exchangable format.
*/
#ifndef REL_FEATURE_REVERSE_LOOKUP
#define REL_FEATURE_REVERSE_LOOKUP       FALSE
#endif


/*  Default volume label
*/
#define TFS_LABEL_DEFAULT       "NOLABEL"


/*  Reliance maximum block size

    Valid values: 512, 1024, 2048, 4096, 8192, 16384, 32768, and 65536
*/
#ifndef TFS_MAX_BLOCK_SIZE
#define TFS_MAX_BLOCK_SIZE      65536
#endif


/*  Reliance minimum block size.
*/
#define TFS_MIN_BLOCK_SIZE      128


/*  There should be some number of usable blocks when we finished
    formatting.  This number is arbutrary.
*/
#define REL_FORMAT_MIN_BLOCKS       100


/*  Included for sTfsVolume structure which uses the entent type
*/
#include <rlextent.h>


/*  names to be be used for the index and bad block list
*/
#define TFS_SZINDEX_NAME   ".tfsindex"  /* note: no TfsStr() */
#define TFS_SZBADBLOCK_NAME   ".tfsbad" /* note: no TfsStr() */
#define TFS_SZVOLUMELABEL_NAME  ".tfslabel" /* note: no TfsStr() */


/*  # of transaction points on the TFS volume before disk becomes read-only
*/
#define TFS_DEMO_LIMIT        50000


/*  Additional debug code
*/
#define TFS_CHECK_IMAP_ENTRY_COUNTS    0        /* IR ? */


/**************************************

    Global information

***************************************/

/*  layout version is the version of data physically laid out. Only changes
    if TFS on-disk layout changes. Independant of TFS_VERSION.

    0x105 = Original implementation
    0x106 = Implemented common data format for meta roots
    0x107 = Implemented common data format for IMAPs
    0x108 = Implemented common data format for index
    0x108 = Implemented common data format for DIR entries
    0x109 = Implemented common data format for INode entries
    0x10A = Exclusive disk format for Nucleus and XP Driver (1.15)
    0x10B = New CDF structures and using Unicode on disk
    0x10C = CDF with the addition of the reverse lookup feature
*/
#if !REL_FEATURE_REVERSE_LOOKUP
#define TFS_LAYOUT_VERSION 0x10B
#else
#define TFS_LAYOUT_VERSION 0x10C
#endif




#define TFS_INVALID_BLOCK     TFS_UINT32_MAX


/*  the initial blocks on a freshly formatted disk
*/
#define TFS_INIT_MASTER       0


/*
    =========================================================================
    Master Block information
    =========================================================================
*/


/*  Number of metaroots used by Reliance
*/
#define TFS_NUM_METAROOTS       2


/*  Number of reserved fields in the master block
*/
#define TFS_MASTER_RESERVED     2


/*  Number of unused bytes before the master block structure when residing on
    the media
*/
#define TFS_MASTER_OFFSET       0x40   /* size of boot code */


/*  Master block checksum and signature
*/
#define TFS_MASTER_SIG          0x5453414DUL      /* 'MAST' */
#define TFS_MASTER_CHECKSUM     TFS_MASTER_SIG


/*  Common data format master block size.
*/
#define TFS_SIZEOF_sTfsMaster   0x30


/*  The Master Block is stored at the very beginning of the disk. It
    describes were to find the MetaRoots, iMaps and contains the
    initial boot code.

    Some fields are reserved to allow for systems to boot from the
    boot sector. See the DOS OS Integration Layer for an example.
*/
typedef struct {
    tfsUint32       ulSignature;    /* 00h 'MAST' */
    tfsUint16       uFmtBldNumber;  /* 04h Build number that formatted the media */
    tfsUint16       uTfsLayout;     /* 06h Layout version (must match) */
    tfsBlockSize    ulBlockSize;    /* 08h Logical block size */
    tfsBlock        ulNumBlocks;    /* 0Ch Total logical blocks */
    tfsBlock        ulMetaBlock[TFS_NUM_METAROOTS]; /* 10h block locations of MetaRoots */
    tfsTime         ullCreateDate;  /* 18h Date/time microseconds since Jan 1, 1970 */
    tfsBlock        ulNumIMapBlocks;/* 20h Number of iMap blocks */
    tfsUint32       ulReserved[TFS_MASTER_RESERVED];  /* 24h Reserved for future use */
    tfsUint32       ulCheckSum;      /* 2Ch Checksum of master block structure */
    tfsUint8       *pucCDFBuffer;   /* Pointer to the CDF buffer, does not reside on disk */
} sTfsMasterBlock;


/*
    =========================================================================
    MetaRoot information
    =========================================================================
*/


/*  Master block checksum and signature
*/
#define TFS_META_SIG            0x4154454DUL    /* 'META' */
#define TFS_META_CHECKSUM       TFS_META_SIG


/*  Common data format metaroot
*/
#define TFS_SIZEOF_sTfsMetaRoot 0x40


/*  This is the number of IMap blocks that the meta root can manage
*/
#define TFS_META_ENTRIES(_ulBlockSize) \
    (((_ulBlockSize) - TFS_SIZEOF_sTfsMetaRoot) * TFS_META_EPB)


/*  Number of reserved fields in the metaroot
*/
#define TFS_META_RESERVED       5


/*  MetaRoot entry fields
*/
#define TFS_META_MASK           0x03    /* mask for one entry */
#define TFS_META_SHIFT          1       /* shift to get to next entry */
#define TFS_META_EPB            4       /* entries per word */
#define TFS_META_IMAP_CURRENT   0x01    /* entry bitfield: current iMap */
#define TFS_META_IMAP_MODIFIED  0x02    /* entry bitfield: iMap has been modified */


/*  The MetaRoot describes the file system on disk. It is written redundantly
    across multiple blocks so that we can recover from failure to write the
    MetaRoot. It is exactly one block in size.

    *** REGARDING COMMON DATA FORMAT ***

    Any changes to this structure intale reviewing the following
    functions/macros:

    TfsMetaRootCDFRead
    TfsMetaRootWrite
    TfsMetaRootIsValid
    TfsMetaRootMount
    TFS_SIZEOF_sTfsMetaRoot

    Also, using sizeof(sTfsMetaRoot) does not give you the size of the meta
    root on disk (in common data format).  Use TFS_SIZEOF_sTfsMetaRoot

    *** Important ***

    There are pieces of code that make assumptions about the order of the
    structure members.  Do not move rearrange the elements in the structure
    without checking all the uses of that structure.  An example code snippet
    follows where iValue is the new value of an imap entry and iOldValue was
    the previous value of an imap entry.

    pMetaRoot   = gpTfsVolume->pMetaRoot;
    piBlocks = &(pMetaRoot->iFreeBlocks);
    ++piBlocks[iValue];
    --piBlocks[iOldValue];

    ***************
*/
typedef struct {
    tfsUint32       ulSignature;    /* 00h 'META' */
    tfsUint32       ulCounter;      /* 04h Used to determine most recent MetaRoot */
    tfsBlock        ulIndexBlock;   /* 08h Block number of index iNode */
    tfsBlock        ulMapNextAlloc; /* 0Ch Next block to allocate */
    tfsBlock        ulMapStart[TFS_NUM_METAROOTS];  /* 10h Start of each IMap block */
    tfsBlock        ulFreeBlocks;   /* 18h Number of free blocks */
    tfsBlock        ulUsedBlocks;   /* 1Ch Number of used blocks */
    tfsBlock        ulBadBlocks;    /* 20h Number of bad blocks */
    tfsUint32       ulPortModified; /* 24h Bits representing which ports have modified the media */
    tfsUint32       ulReserved[TFS_META_RESERVED];   /* 28h Reserved for future used */
    tfsUint32       ulCheckSum;     /* 3Ch Checksum of metaroot structure */
    tfsUint8        ucEntries[1];   /* 40h Keeps track of active imap blocks */
} sTfsMetaRoot;


/**************************************

    Volume information

***************************************/

#define TFS_VOLUME_SIG     0x554C4F56UL /* VOLU */

struct TfsTagVolume
{
    tfsUint32       ulSignature;        /* 00h volume signature */
    struct TfsTagVolume * pNext;        /* 04h next volume ptr */
    tfsBool         fAccessOk;          /* 08h allow/disallow access to volume */
    tfsBool         fReadOnly;          /* 0ch is volume set to read only? */
    tfsBool         fDemoOver;          /* 10h demo over. Volume also becomes read-only */
    tfsUint32       ulBlockSize;        /* 14h Block size for this mounted volume */
    tfsBlock        ulNumBlocks;        /* 18h Total number of blocks on this volume */
    tfsBlock        ulNumIMapBlocks;    /* 1ch Number of iMap blocks */
    tfsBlock        ulReservedBlocks;   /* 20h Number of blocks reserved for delete operations */
    tfsBlock        ulBranchedBlocks;   /* 24h number of branched blocks */
    tfsBlock        ulNewBlocks;        /* 28h number of new blocks */
    tfsOffset       ulInPlaceSize;      /* 2ch in-place size for a given iNode tree */
    tfsOffset       ulDirectSize;       /* 30h direct size for a given iNode tree */
    tfsOffset       ulIndirectSize;     /* 34h indirect size for a given iNode tree */
    tfsOffset       ulMaxFileSize;      /* 38h max file size for a given iNode tree */
    tfsUint16       uNodeEntries;       /* 40h iNode entries */
    tfsUint16       uIndirectEntries;   /* 42h indirect entires */
    tfsUint32       ulIMapEntries;      /* 44h imap entries */
    tfsUint32       ulCapabilities;     /* 48h capabilities of IO device */
#if TFS_ENABLE_DISCARD
    sTfsExtentTable * pExtentTable;     /* 4ch if discard capable, need a volume extent */
#endif
    tfsBool         fMetaRootIsBranched; /* 50h is meta root branched? */
    tfsUint8        ucWhichMetaRoot;    /* 54h which meta root? */
    tfsUint8        ucPad1[3];          /* 55h Unused alignment padding */
    tfsUint32       ulTranMode;         /* 58h transaction mode (auto transact or not) */
    tfsUint32       ulTransactionBits;  /* 5ch transact on what conditions? */
    sTfsMetaRoot *  pMetaRoot;          /* 60h pointer to volume meta root (allocated) */
    tfsBlock        ulMetaBlock[TFS_NUM_METAROOTS]; /* 64h block locations of MetaRoots */
    sRelOilData *   pOilData;           /* 6ch OIL specific data / volume */
    sRelIoData *    pIoData;            /* 70h IO specific data / volume */
    tfsOffset       ulLastIndexOffset;  /* 74h Where to begin looking for a free index entry */
    tfsTime         ullLastTransaction; /* 78h timestamp of last transaction */
    tfsTime         ullCreationDate;    /* 80h volume timestamp, when it was formatted */
    tfsBool         fOnMediaUnicode;    /* 88h */
    tfsUint32       ulPortPathLengthLimit;  /* 8ch Port specific path length limit */
    tfsUint32       ulPortNameLengthLimit;  /* 90h Port specific name length limit */
    tfsChar *       szPortIllegalCharacters; /* 94h Port specific illegal characters */
};


#define TfsVolumeIsValid(_pVolume) ((_pVolume) && (_pVolume)->ulSignature == TFS_VOLUME_SIG)

#ifdef TFS_FILENUM
#if (TFS_FILENUM != TFS_METALIB_C)
extern sTfsVolume *gpTfsVolume;         /* current volume */
#endif
#endif


/*  # of blocks to remain free for use by delete operations
*/
#define TFS_RESERVED_BLOCKS         20


/*  IO layer capabilities
*/
#define TFS_IO_CAP_REMOVABLE  0x0001    /* device supports removable media */
#define TFS_IO_CAP_DISCARD    0x0002    /* device supports block discard */


#define TfsDiscardCapable(_pVolume)     ((_pVolume)->ulCapabilities & TFS_IO_CAP_DISCARD)
#define TfsRemoveCapable(_pVolume)      ((_pVolume)->ulCapabilities & TFS_IO_CAP_REMOVABLE)


/*
    =========================================================================
    IMap information
    =========================================================================
*/


/*  imap info
*/
#define TFS_IMAP_SIG        0x50414D49UL    /* 'IMAP' */
#define TFS_IMAP_FIELDS     2   /* Number of non-entry fields in the IMap */
#define TFS_IMAP_RESERVED   1   /* Number of reserved entries in the IMap */


/*  Each iMap entry can be one of 4 types. Each entry is 2 bits in size.
*/
#define TFS_IMAP_FREE       0x00 /* block is free for use */
#define TFS_IMAP_BRANCHED   0x01 /* block contains "branched" data, not yet freed */
#define TFS_IMAP_INUSE      0x02 /* block contains existing data. cannot be written to */
#define TFS_IMAP_NEW        0x03 /* block newly allocated. can be written to */


/*  masks & things for dealing with individual iMap entries
*/
#define TFS_IMAP_BRANCH_MASK    0xAA    /* mask off "branched" and "new" bits */
#define TFS_IMAP_EPB            4       /* entries per byte */
#define TFS_IMAP_EPW            (TFS_IMAP_EPB * 4)  /* entries per 32-bit word */
#define TFS_IMAP_SHIFT          1       /* bits per entry as a << shift value */
#define TFS_IMAP_MASK           0x03    /* mask for these bits */


/*  TfsTagIMap

    The iMap is a set of blocks that contain allocation bitmaps for the
    entire disk.

    *** Important ***

    Do not reference these structure members directly.  Use the CDF macros to
    retrieve the information.  Use add the sizeof each structure element to
    compute the offset of ucEntries.

    **************
*/
typedef struct {
    tfsUint32       iSignature; /* signature to verify this is an iMap block */
    tfsUint32       iReserved[TFS_IMAP_RESERVED];       /* reserved for future use */
    tfsUint8        ucEntries[1];       /* blocks bits in this iMap block */
} sTfsIMap;


/*  This is the offset of the "iEntries" field in the IMAP.  It is now
    constant with the CDF changes.  This is only used by the checker to double
    check the calculations using a different method.
*/
#define TFS_IMAP_OFFSET_ENTRIES     8


/*  For those places where code needed to know the sizeof sTfsIMap.  Note that
    the sTfsImap structure is treated as if it contains 3 tfsUint32 fields for
    calculation purposes.
*/
#define TFS_SIZEOF_sTfsIMap         0x0c


/*
    =========================================================================
    INode information
    =========================================================================
*/


/*  iNode info
*/
#define TFS_INODE_SIG           0x444F4E49UL      /* 'INOD' */


/*  Number of reserved fields in the INode
*/
#if !REL_FEATURE_REVERSE_LOOKUP
#define TFS_INODE_RESERVED       5
#else
#define TFS_INODE_RESERVED       4
#endif

/*  maximum possible file/dir size (in bytes)
*/
#define TFS_INODE_MAXSIZE       (TFS_UINT32_MAX - (((TFS_MAX_BLOCK_SIZE) * 2) - 1))


/*  Common data format INode structure.
*/
#define TFS_SIZEOF_sTfsINode    0x40


/*  # of 32-bit "fields" in an iNode
*/
#define TFS_INODE_FIELDS        (TFS_SIZEOF_sTfsINode / sizeof(tfsBlock))


/*  blocks are kept track of in one of 4 methods:
*/
#define TfsINodeGetMode(_pINode)    \
    ((_pINode)->uAttributes & TFS_INODE_FB_MODE)
#define TfsINodeSetMode(_pINode, _iMode)    \
    TfsINodeSetAttributes(&((_pINode)->uAttributes), (_iMode), TFS_INODE_FB_MODE)
#define TfsINodeInPlace(_pINode)    \
    (TfsINodeGetMode(_pINode) == TFS_INODE_FB_INPLACE)
#define TfsINodeDirect(_pINode)     \
    (TfsINodeGetMode(_pINode) == TFS_INODE_FB_DIRECT)
#define TfsINodeIndirect(_pINode)   \
    (TfsINodeGetMode(_pINode) == TFS_INODE_FB_INDIRECT)
#define TfsINodeInDouble(_pINode)   \
    (TfsINodeGetMode(_pINode) == TFS_INODE_FB_INDOUBLE)


/*  iNode iFlags masks
    (note: some functions use mode like an enum, must be lowest bits)
*/
#define TFS_INODE_FB_MODE       0x0003  /* mask for file mode bits */
#define TFS_INODE_FB_INPLACE    0x0000  /* file data is in the iNode */
#define TFS_INODE_FB_DIRECT     0x0001  /* file data is in blocks */
#define TFS_INODE_FB_INDIRECT   0x0002  /* file data is in indirect blocks */
#define TFS_INODE_FB_INDOUBLE   0x0003  /* file data is in double indirect blocks */
#define TFS_INODE_FB_PERMANENT  0x0040  /* Internal attribute for index and root */
#if REL_FEATURE_REVERSE_LOOKUP
#define TFS_INODE_FB_NOLOOKUP   0x0080  /* Internal attribute for not allowing name lookups from an index */
#endif


/*  An iNode describes every file and directory on a TFS disk. It is really
    the root of a tree structure that defines a file so that data can be
    accessed rapidly even when using random I/O.

    The iNode contains everything about a file except its name. Names are
    stored in directories, which entries point to iNodes.

    *** REGARDING COMMON DATA FORMAT ***

    Any changes to this structure intale reviewing the following
    functions/macros:

    TfsINodeCDFRead
    TfsINodeCDFWrite
    TfsINodeCDFIsValid

    Also, using sizeof(sTfsINode) does not give you the size of the INode
    on disk (in common data format).  Use TFS_SIZEOF_sTfsINode.
*/
typedef struct {
    tfsUint32   ulSignature;        /* 00h 'INOD' */
    tfsIndex    ulIndex;            /* 04h points back to index */
#if TFS_USING_64BIT_FILESIZES
    tfsUint64   ullFileSize;        /* 08h File size in bytes */
#else
    tfsUint32   ullFileSize;        /* 08h File size in bytes */
    tfsUint32   ulPad;              /* 0ch Unused */
#endif
    tfsTime     ullCreateDate;      /* 10h when created (in milliseconds) */
    tfsTime     ullModifiedDate;    /* 18h when last modified (in milliseconds) */
    tfsTime     ullAccessDate;      /* 20h when last accessed (in milliseconds) */
    tfsUint16   uAttributes;        /* 28h CDF attributes */
    tfsUint16   uLinkCount;         /* 2Ah Number of entries linked to this INode */
#if !REL_FEATURE_REVERSE_LOOKUP
    tfsUint32   ulReserved[TFS_INODE_RESERVED]; /* 2Ch Reserved for future use */
#else
    tfsIndex    ulParentIndex;      /* 2Ch index of parent directory */
    tfsUint32   ulReserved[TFS_INODE_RESERVED]; /* 30h Reserved for future use */
#endif
    tfsBlock *  pulEntries;         /* 34h Entries (or file data in direct mode) */
    tfsUint8 *  pucCDFBuffer;       /* 38h Pointer to the CDF buffer */
}sTfsINode;


#if !TFS_ENABLE_64BIT_TYPES
#error 64-bit support required by INode CDF implementation
#endif


/*
    =========================================================================
    Indirect INode information
    =========================================================================
*/


/*  Indirect blocks keep track of file blocks indirectly. Each indirect block
    is allocated from the index just like iNodes.
*/
#define TFS_INDIRECT_SIG            0x49444E49UL   /* 'INDI' */
#define TFS_INDOUBLE_SIG            0x494C4244UL   /* 'DBLI' */


/*  Common data format INode indirect structure.
*/
#if 0
#define TFS_INDIRECT_RESERVED       1
#define TFS_SIZEOF_sTfsIndirect     0x18
#else
#define TFS_INDIRECT_RESERVED       \
    ((TFS_SIZEOF_sTfsINode - 0x14)/sizeof(tfsUint32))
#define TFS_SIZEOF_sTfsIndirect     \
    (0x14 + (TFS_INDIRECT_RESERVED * sizeof(tfsUint32)))
#endif


/*  sTfsIndirect

    Indirect or double indirect block
*/
typedef struct {
    tfsUint32   ulSignature;        /* 00h 'INDI' or 'DBLI' */
    tfsIndex    ulIndex;            /* 04h Index for this meta block */
    tfsIndex    ulINodeIndex;       /* 08h Owner's INode index */
    tfsUint32   ulParentIndex;      /* 0Ch Parent meta block index */
    tfsUint32   ulSequenceNumber;   /* 10h # within parent's table */
    tfsUint32   ulReserved[TFS_INDIRECT_RESERVED]; /* 14h Reserved for future use */
    tfsBlock *  pulEntries;         /* 18h */
    tfsUint8 *  pucCDFBuffer;       /* 1Ch Pointer to the CDF buffer */
} sTfsIndirect;


/*  helper structure for coordinates in an iNode tree
*/
typedef struct {
    sTfsINode * pINode;     /* Pointer to INode */
    tfsUint16   uDouble;    /* Double indirect block pointing to indirect block */
    tfsUint16   uIndirect;  /* Indirect block pointing to this block (0 means none) */
    tfsUint16   uBlock;     /* Logical block number */
    tfsUint32   ulOffset;   /* Offset within the block */
} sTfsINodeCoord;


/*
    =========================================================================
    Index information
    =========================================================================
*/


/*  signature for index data
*/
#define TFS_INDEX_SIG           0x58444E49UL /* INDX */


/*  for unused index entries (when iNodes are deleted)
*/
#define TFS_INDEX_UNUSED        0


/*  hard-coded original index #s at format time (0 is reserved)
*/
#define TFS_INDEX_HEADER        0   /* Hard-coded index set to TFS_INDEX_SIG */
#define TFS_INDEX_INDEX         1   /* Hard-coded index for index file */
#define TFS_INDEX_ROOTDIR       2   /* Hard-coded index for root dir */
#define TFS_INDEX_BADFILE       3   /* Hard-coded index for bad file */
#define TFS_INDEX_LABEL         4   /* Hard-coded index for volume label */
#define TFS_INDEX_FIRST_FREE    5   /* First usable index after format */

/*
    =========================================================================
    Directory information
    =========================================================================
*/


/*  ucEntryAttributes bits
*/
#define TFS_DIR_INUSE               0x01
#define TFS_DIR_ISDIR               0x02
#define TFS_DIR_ISLABEL             0x04


/*
*/
#define TFS_DIR_PACKET_ENTRY        0x80
#define TFS_SIZEOF_DIR_PACKET       0x10


/*  Common data format directory entry.
*/
#define TFS_SIZEOF_sTfsDirEntry     0x0f


/*  a single entry on disk, refers to both directories and files

    *** REGARDING COMMON DATA FORMAT ***

    Any changes to this structure intale reviewing the following
    functions/macros:

    TfsDirCDFReadEntry
    TfsDirCDFWriteEntry
    RelFmtAddRootDirEntry

    Also, using sizeof(sTfsDirEntry) does not give you the size of the dir
    entry on disk (in common data format).  Use TFS_SIZEOF_sTfsDirEntry.
*/
typedef struct {
    tfsUint8    ucPacketFlags;
    tfsUint16   uLC;                /* 00h Limitted checksum of known portable characters */
    tfsUint32   ulEntryLength;      /* 02h Length of entry (may be longer than name) */
    tfsUint16   uUnicodeNameLength; /* 06h Length of name in bytes */
    tfsUint8    ucEntryAttributes;  /* 08h Attributes for this entry */
    tfsUint8    ucReserved;         /* 09h Reserved for future use */
    tfsIndex    ulINodeIndex;       /* 0ah Index of the INode for this entry */
                                    /* 0eh */
} sTfsDirEntry;


/*  defaults for must and can mask
*/
#define TFS_MASK_DEF_MUST  0
#define TFS_MASK_DEF_CAN   0xffff


/**************************************

    Other Structures and Defines

***************************************/

/*  flags for TfsValidatePath
*/
#define TFS_NAME_MUST_EXIST      0x0001 /* must exist */
#define TFS_NAME_CANNOT_EXIST 0x0002    /* must not exist */
#define TFS_NAME_DIR_ONLY     0x0004    /* must be a directory */
#define TFS_NAME_FILE_ONLY    0x0008    /* must be a file */
#define TFS_NAME_SYSTEM_OK    0x0010    /* OK to operate on system file? */
#define TFS_NAME_ALL_FLAGS    0x001F    /* all of the above flags */

/*  kill points
*/
#define TFS_KILL_SIG       0x4B494C4CUL /* KILL */
#define TFS_KILL_NONE         0x00000000
#define TFS_KILL_INODE_ALLOC  0x00000001
#define TFS_KILL_INODE_FREE      0x00000002
#define TFS_KILL_IMAP_ALLOC      0x00000004
#define TFS_KILL_IMAP_FREE    0x00000008
#define TFS_KILL_INDEX_ALLOC  0x00000010
#define TFS_KILL_INDEX_FREE      0x00000020
#define TFS_KILL_DATA_ALLOC      0x00000040
#define TFS_KILL_DATA_FREE    0x00000080
#define TFS_KILL_META_WRITE      0x00000100
#define TFS_KILL_INODE_BRANCH 0x00000200
#define TFS_KILL_IMAP_BRANCH  0x00000400
#define TFS_KILL_INDEX_BRANCH 0x00000800
#define TFS_KILL_DATA_BRANCH  0x00001000
#define TFS_KILL_RENAME       0x00002000
#define TFS_KILL_DIR_FREE     0x00004000
#define TFS_KILL_DIR_ALLOC    0x00008000
#define TFS_KILL_IOWRITE      0x00010000

/*  kill actions
*/
#define TFS_KILLACT_RESET     0 /* reset the system (default) */
#define TFS_KILLACT_HALT      1 /* halt the system */
#define TFS_KILLACT_BADWRITE  2 /* simulate bad write to the disk */

#if TFS_TEST_KILL
#define TfsKillPoint(_iBit) TfsKill(_iBit, gszTfsAssertModule, __LINE__)
extern tfsUint32 giTfsKillCount;
extern tfsBool  gfTfsKillFlush;
extern tfsBool  gfTfsKilled;
#else
#define TfsKillPoint(_iBit)
#endif

/*  volume information
*/
struct TfsTagVolumeInfo
{
    tfsUint32       iBlockSize; /* block size on volume */
    tfsBlock        iNumBlocks; /* total # of blocks in volume */
    tfsBlock        iFreeBlocks;        /* free blocks in volume */
    tfsBlock        iFileBlocks;        /* blocks in user files */
    tfsBlock        iDirBlocks; /* blocks in directories */
    tfsBlock        iSystemBlocks;      /* blocks used by file system */
    tfsBlock        iBadBlocks; /* block marked bad on system */
    tfsBlock        iDirCount;  /* # of directories */
    tfsBlock        iFileCount; /* # of files */
};
typedef struct TfsTagVolumeInfo sTfsVolumeInfo;

/*  initialize a geometry to default state
*/
#define TfsInitializeGeometry(_pGeometry) ((_pGeometry)->iBlockSize = (_pGeometry)->iNumBlocks = 0)

/*  helper structure for read/write functions
*/
struct TfsTagRdWr
{
    sTfsINode      *pINode;
    tfsOffset       iOffset;
    tfsOffset       iLength;
    tfsBuffer      *pBuffer;
};
typedef struct TfsTagRdWr sTfsRdWr;

/*  helper structure for determine # of blocks required for file write
*/
struct TfsTagNewBlocks
{
    tfsBlock        iMetaBranched;      /* indirect/double blocks to branch */
    tfsBlock        iMetaNew;   /* indirect/double blocks to add */
    tfsBlock        iBranched;  /* data blocks to branch */
    tfsBlock        iNew;       /* data blocks to add */
    tfsBlock        iTotal;     /* total blocks (add all of above) */
};
typedef struct TfsTagNewBlocks sTfsNewBlocks;

/*  for use with progress bars
*/
typedef void(*fnTfsProgress)(tfsBlock iValue,tfsBlock iLimit);
#define __REL_DEF_fnTfsProgress


/**************************************

    Other types and defines

***************************************/

/*  from tfslog.c
*/
extern tfsBool  gfTfsLogVerbose;

/*  from tfoio.c
*/
extern tfsBlock *gpiTfsIoBadBlocks;

#endif /* TFS_CORE_H */
