/*---------------------------------------------------------------------------
               ----> DO NOT REMOVE THE FOLLOWING NOTICE <----

      Copyright (c) 2003 - 2004 Datalight, Inc.  All Rights Reserved.

  Datalight, Incorporated is a Washington State corporation located at:

        21520 30th Dr SE, Suite 110,      Tel:  425-951-8086
        Bothell, WA  98021                Fax:  425-951-8094
        USA                               Http://www.datalight.com

  This software, including without limitation, all source code and documen-
  tation, is the confidential, trade secret property of Datalight, Inc., and
  is protected under the copyright laws of the United States and other juris-
  dictions.

  ---> Portions of the software are patent pending <---

  In addition to civil penalties for infringement of copyright under applic-
  able U.S. law, 17 U.S.C. 1204 provides criminal penalties for violation of
  (a) the restrictions on circumvention of copyright protection systems found
  in 17 U.S.C. 1201 and (b) the protections for the integrity of copyright
  management information found in 17 U.S.C. 1202.

  U.S. Government Restricted Rights:  Use, duplication, reproduction, or
  transfer of this commercial product and accompanying documentation is
  restricted in accordance with FAR 12.212 and DFARS 227.7202 and by a
  License Agreement.

  IN ADDITION TO COPYRIGHT AND PATENT LAW, THIS SOFTWARE IS PROTECTED UNDER A
  SOURCE CODE AGREEMENT, NON-DISCLOSURE AGREEMENT (NDA), OR SIMILAR BINDING
  CONTRACT BETWEEN DATALIGHT, INC. AND THE LICENSEE ("BINDING AGREEMENT").
  YOUR RIGHT, IF ANY, TO COPY, PUBLISH, MODIFY OR OTHERWISE USE THE SOFTWARE,
  IS SUBJECT TO THE TERMS AND CONDITIONS OF THE BINDING AGREEMENT.  BY USING
  THE SOFTWARE IN ANY MANNER, IN WHOLE OR IN PART, YOU AGREE TO BE BOUND BY
  THE TERMS OF THE BINDING AGREEMENT.  CONTACT DATALIGHT, INC. AT THE ADDRESS
  SET FORTH ABOVE IF YOU OBTAINED THIS SOFTWARE IN ERROR.
---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------
                              Description

    Reliance Crash Test header.

---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------
                              Revision History
    $Log: rlcrash.h $
    Revision 1.1  2004/12/31 03:17:52  Pauli
    Initial revision
    Revision 1.1  2004/12/31 03:17:52Z  Garyp
    Initial revision
    Revision 1.1  2004/12/31 02:17:52Z  pauli
    Initial revision
    Revision 1.2  2004/12/31 01:17:52Z  GaryP
    Updated to use indents of 4, hard tabs of 8 (if used), and a standard
    comment style.
    Revision 1.1  2004/10/01 21:46:56Z  brandont
    Initial revision
    Revision 1.5  2004/08/27 00:29:56  brandont
    - IR ? *.* Changed header on all the source modules to the FlashFx style header
---------------------------------------------------------------------------*/

#ifndef __CRASH_TEST_H
#define __CRASH_TEST_H 1

#include "reliance.h"
#include "rlcore.h"
#include "rllib.h"
#include "rloilapi.h"

#include "rlctopts.h"
#include "rlctoil.h"


#define CREATE_FILE     0x0001
#define CREATE_DIR      0x0002

#define STEP_INTO_DIR   0x0001
#define STEP_STAY       0x0002
#define STEP_UP_ONE     0x0001
#define STEP_UP_TWO     0x0002
#define STEP_TO_ROOT    0x0004


typedef struct
{
    tfsUint32       ulRecordNum;
    tfsUint16       wDo;
    tfsUint16       wStep;
    tfsByte         bData;
    tfsInt16        iFileIndex;
} TEST_STEP;

typedef struct
{
    tfsChar         szTestLogFileName[MAX_PATH];
    tfsChar         szStatusLogFileName[MAX_PATH];
    tfsChar         szTestPath[MAX_PATH];
    tfsInt16        iVerbose;
    tfsInt16        iReboot;
    tfsUint32       ulSeed;
    tfsUint32       ulDelay;
    tfsInt16        iCreateFile;
    tfsInt16        iCreateDir;
    tfsInt16        iStepIntoDir;
    tfsInt16        iStay;
    tfsInt16        iStepUpOne;
    tfsInt16        iStepUpTwo;
    tfsInt16        iStepToRoot;
} TEST_DEF;

typedef struct
{
    tfsUint32       ulSeed;
} LOG_HEADER;

/*  Crash Test entry point

    Input
    szSettingsFile is the full path of the settings file to load

    Output
    Returns SUCCESS if no errors, FAILURE if an error occurred.
    If automatic reboot is turned on in the settings file, the
    machine should always reboot and never return unless there is
    an error.
*/
tfsInt16        RelCrashTest(
    tfsChar * szSettingsFile);

/*  Initialize Log Header 
*/
void            InitializeLogHeader(
    TEST_DEF * spTestDef,
    LOG_HEADER * pLogHeader);

/*  CreateTestLog

    Creates the test log file.  Writes the header to the file and 1 MB of
    0xFF check sum bytes.
*/
tfsInt16        CreateTestLog(
    TEST_DEF * spTestDef,
    LOG_HEADER * spLogHeader);

/*  Run Test 
*/
tfsInt16        BeginTesting(
    TEST_DEF * spTestDef);

/*  Verifies that Log file and disk from previous run match.
    If errors, returns FAILURE otherwise SUCCESS. 
*/
tfsInt16        CheckDisk(
    TEST_DEF * spTestDef);

/*  Searches for all files and dirs on the test disk and deletes them.
    If errors, returns FAILURE, otherwise SUCCESS. 
*/
tfsInt16        FormatDisk(
    TEST_DEF * spTestDef);

/*  Searches for the file or dir with the highest index in the specified path and
    returns that index. The crash test creates files and dirs and names them
    in numerical order.  Therefore the last file or dir in a dir will have the
    highest index value. 
*/
tfsInt16        GetLastIndex(
    tfsChar * szPath,
    TEST_DEF * spTestDef);

/*  Appends a message to a file 
*/
void            LogMessage(
    tfsChar * szFileName,
    tfsChar * szMsg);

/*  Formats the error message and outputs it and logs it. 
*/
void            OutputError(
    TEST_DEF * spTestDef,
    tfsInt16 iTFSErrorNo,
    tfsChar * szMsg);

/*  Formats, and outputs and logs the test start message.  It includes
    the date, time, seed value and delay value. 
*/
void            OutputStart(
    TEST_DEF * spTestDef);

/*  Reads a line from a file, up to iMaxLen characters.  The buffer
    must be atleast iMaxLen+1 is size.

    Returns FAILURE if an error occurred, otherwise SUCCESS.
*/
tfsInt16        CTReadLine(
    TFSFILE * fp,
    tfsChar * szLine,
    tfsInt16 iMaxLen);

/*  Contatenates two strings together.  szDest must be large enough
    to hold both strings 
*/
void            CTStrCat(
    tfsChar * szDest,
    tfsChar * szSrc);

/*  Loads the test settings from the specified file. 
*/
tfsInt16        LoadSettings(
    tfsChar * szFileName,
    TEST_DEF * spTestDef);

#endif
