/*---------------------------------------------------------------------------
               ----> DO NOT REMOVE THE FOLLOWING NOTICE <----

      Copyright (c) 2003 - 2004 Datalight, Inc.  All Rights Reserved.

  Datalight, Incorporated is a Washington State corporation located at:

        21520 30th Dr SE, Suite 110,      Tel:  425-951-8086
        Bothell, WA  98021                Fax:  425-951-8094
        USA                               Http://www.datalight.com

  This software, including without limitation, all source code and documen-
  tation, is the confidential, trade secret property of Datalight, Inc., and
  is protected under the copyright laws of the United States and other juris-
  dictions.

  ---> Portions of the software are patent pending <---

  In addition to civil penalties for infringement of copyright under applic-
  able U.S. law, 17 U.S.C. 1204 provides criminal penalties for violation of
  (a) the restrictions on circumvention of copyright protection systems found
  in 17 U.S.C. 1201 and (b) the protections for the integrity of copyright
  management information found in 17 U.S.C. 1202.

  U.S. Government Restricted Rights:  Use, duplication, reproduction, or
  transfer of this commercial product and accompanying documentation is
  restricted in accordance with FAR 12.212 and DFARS 227.7202 and by a
  License Agreement.

  IN ADDITION TO COPYRIGHT AND PATENT LAW, THIS SOFTWARE IS PROTECTED UNDER A
  SOURCE CODE AGREEMENT, NON-DISCLOSURE AGREEMENT (NDA), OR SIMILAR BINDING
  CONTRACT BETWEEN DATALIGHT, INC. AND THE LICENSEE ("BINDING AGREEMENT").
  YOUR RIGHT, IF ANY, TO COPY, PUBLISH, MODIFY OR OTHERWISE USE THE SOFTWARE,
  IS SUBJECT TO THE TERMS AND CONDITIONS OF THE BINDING AGREEMENT.  BY USING
  THE SOFTWARE IN ANY MANNER, IN WHOLE OR IN PART, YOU AGREE TO BE BOUND BY
  THE TERMS OF THE BINDING AGREEMENT.  CONTACT DATALIGHT, INC. AT THE ADDRESS
  SET FORTH ABOVE IF YOU OBTAINED THIS SOFTWARE IN ERROR.
---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------
                              Description

---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------
                              Revision History
    $Log: rlerr.h $
    Revision 1.4  2006/03/29 22:05:01  brandont
    Added conditioned feature to look up a path based on an INode index.
    Revision 1.3  2006/01/26 21:55:59Z  brandont
    Added TFSERR_DISCARDTABLEFULL which was overwritten during
    a merge.
    Revision 1.2  2006/01/26 04:26:34Z  brandont
    Merged with Reliance 2.0 code base.
    Revision 1.7  2006/01/10 20:51:34Z  thomd
    Added new error code, TFSERR_NOT_SAME_VOL
    Revision 1.6  2005/12/17 02:32:25Z  brandont
    Added error codes for TFSERR_IO_ARG, TFSERR_IO_INIT, and
    TFSERR_IO_IOCTL.
    Revision 1.1  2005/12/02 18:10:48Z  Pauli
    Initial revision
    Revision 1.2  2005/12/02 18:10:48Z  Pauli
    Merge with 2.0 product line, build 458.
    Revision 1.5  2005/08/08 18:02:54Z  brandont
    Corrected stale comment.
    Revision 1.4  2005/08/04 20:07:09Z  brandont
    Added the checker specific error codes.
    Revision 1.3  2005/02/18 21:33:46Z  brandont
    Removed the nondescriptive I/O error define and created new defines
     for specific types of I/O errors.
    Revision 1.2  2004/12/31 01:17:52Z  GaryP
    Updated to use indents of 4, hard tabs of 8 (if used), and a standard
    comment style.
    Revision 1.1  2004/11/20 01:46:36Z  brandont
    Initial revision
---------------------------------------------------------------------------*/

#ifndef __RLERR_H
#define __RLERR_H    1


/*  error codes from routines
*/
#define TFSERR_NONE             0   /* call worked, no error */
#define TFSERR_HELP             1   /* invalid command-line or help displayed */
#define TFSERR_QUIT             2   /* quit on purpose */
#define TFSERR_INTERNAL         3   /* internal error */
#define TFSERR_MEMORY           4   /* problem with memory */
#define TFSERR_CACHE            5   /* problem with cache coherency */
#define TFSERR_UNMOUNTED        6   /* file system is unmounted */
#define TFSERR_SHUTDOWN         7   /* allows mount to indicate improper shut-down */
#define TFSERR_INODE            8   /* problem with the iNode or internal structures */
#define TFSERR_RDONLY           9   /* s is write protected (read only) */
#define TFSERR_DENIED           10  /* access denied */
#define TFSERR_NOTFOUND         11  /* directory entry not found */
#define TFSERR_EOF              12  /* end of file (reading) */
#define TFSERR_FULL             13  /* disk full (couldn't allocate enough space) */
#if 0
/*  Obsolete non-descriptive error
*/
#define TFSERR_IO          14   /* error reading/writing data from/to the disk */
#endif
#define TFSERR_EXISTS           15  /* file exists when it shouldn't */
#define TFSERR_BADHANDLE        16  /* invalid handle */
#define TFSERR_BADPATH          17  /* invalid path */
#define TFSERR_LASTENTRY        18  /* no more entries on readdir/findnext */
#define TFSERR_PATHTOOLONG      19  /* name or path was too long */
#define TFSERR_SAMEDIR          20  /* attempt to remove current directory */
#define TFSERR_BADCMD           21  /* bad cmdline */
#define TFSERR_BADCHECK         22  /* range check failed on the command */
#define TFSERR_TOO_FEW_ARGS     23  /* not enough cmdline arguments */
#define TFSERR_WRITE_TOO_LONG   24  /* write too long (wraps around) */
#define TFSERR_INIT             25  /* failed to initialize */
#define TFSERR_UNFORMATTED      30  /* no MetaRoot, so disk is unformatted */
#define TFSERR_WRONGVERSION     31  /* wrong TFS version layout */
#define TFSERR_COMPARE          32  /* results didn't compare in test */
#define TFSERR_TOOMANY          33  /* too many entries in the directory */
#define TFSERR_BLOCKSIZE        34  /* block size out of compile time range */
#define TFSERR_KILL             35  /* killed (on purpose) */
#define TFSERR_LOGERR           36  /* log error (used with setjmp/longjmp) */
#define TFSERR_NOPARTITIONS     37  /* no TFS partitions found */
#define TFSERR_TOOLARGE         38  /* boot file too large */
#define TFSERR_VERSION          39  /* Incompatible version */
#define TFSERR_NOTINSTALLED     40  /* Reliance not installed */
#define TFSERR_INDEX_FULL       41  /* Index file at max size */
#define TFSERR_IO_MEDIACHANGED  42  /* The media has been changed */
#define TFSERR_IO_DISCARD       43  /* Error discarding blocks */
#define TFSERR_IO_WRITE         44  /* Error writing blocks */
#define TFSERR_IO_READ          45  /* Error reading blocks */
#define TFSERR_IO_VERIFY        46  /* Error verifying blocks */
#define TFSERR_IO_FLUSH         47  /* Error flushing blocks */
#define TFSERR_IO_FORCECHANGE   48  /* Error forcing media change */
#define TFSERR_IO_RESET         49  /* Error reseting the block device */
#define TFSERR_IO_ARG           50  /* Invalid argument passed to IO layer */
#define TFSERR_IO_INIT          51  /* Error initializing block device */
#define TFSERR_IO_IOCTL         52  /* Error TfsIoIoctl() */
#define TFSERR_NOT_SAME_VOL     53  /* volume mismatch or null argument */
#define TFSERR_DISCARDTABLEFULL 54  /* Discard table is full and cannot be expanded */
#if REL_FEATURE_REVERSE_LOOKUP
#define TFSERR_MULTILINK        55  /* No reverse lookup for files with multiple links */
#endif

/*  test suite specific error codes
*/
#define TFSERR_SKIPPED          100 /* test skipped */
#define TFSERR_LASTRECORD       101 /* hit the end of the record list */

/*  Define error constant as catch-all when mapping from system errors to TFS error constants
*/
#define TFSERR_GENERAL          128

/*  ORed with error code to allow errors to be passed through
*/
#define TFSERR_PASSTHROUGH    0xCC00
#define TfsErrPassThrough(iErr)  (((iErr) & TFSERR_PASSTHROUGH) == TFSERR_PASSTHROUGH)


/*  error codes that are currently unique to the Checker
*/
#define TFSERR_CHECKER_BASE             200
#define TFSERR_INVALID_MASTER_BLOCK     TFSERR_CHECKER_BASE + 0  /* 200 */
#define TFSERR_UNREADABLE_METAROOT      TFSERR_CHECKER_BASE + 1
#define TFSERR_INVALID_METAROOT         TFSERR_CHECKER_BASE + 2
#define TFSERR_INVALID_META             TFSERR_CHECKER_BASE + 3
#define TFSERR_INVALID_IMAP_BLOCK       TFSERR_CHECKER_BASE + 4
#define TFSERR_UNREADABLE_IMAP_BLOCK    TFSERR_CHECKER_BASE + 5
#define TFSERR_INVLD_FILESIZE           TFSERR_CHECKER_BASE + 6
#define TFSERR_COMMIT_FAILED            TFSERR_CHECKER_BASE + 7
#define TFSERR_INVALID_META_MAP         TFSERR_CHECKER_BASE + 8
#define TFSERR_INVALID_INODE_BLOCK      TFSERR_CHECKER_BASE + 9
#define TFSERR_DUPLICATE_INDEX_ENTRY    TFSERR_CHECKER_BASE + 10 /* 210 */
#define TFSERR_LOST_BLOCK               TFSERR_CHECKER_BASE + 11
#define TFSERR_MISSING_ALLOC_IN_IMAP    TFSERR_CHECKER_BASE + 12
#define TFSERR_INDEX_UNUSED             TFSERR_CHECKER_BASE + 13
#define TFSERR_INVALID_BLOCK_ADDRESS    TFSERR_CHECKER_BASE + 14
#define TFSERR_SIGNAL_EXIT              TFSERR_CHECKER_BASE + 15
#define TFSERR_INCOMPATIBLE_BLOCKSIZE   TFSERR_CHECKER_BASE + 16


#endif /* #ifndef __RLERR_H */
