/*---------------------------------------------------------------------------
               ----> DO NOT REMOVE THE FOLLOWING NOTICE <----

                  Copyright (c) 2003 - 2005 Datalight, Inc.
                    All Rights Reserved.  Patent Pending.

  Datalight, Incorporated is a Washington State corporation located at:

        21520 30th Dr SE, Suite 110,      Tel:  425-951-8086
        Bothell, WA  98021                Fax:  425-951-8094
        USA                               Http://www.datalight.com

  This software, including without limitation, all source code and documen-
  tation, is the confidential, trade secret property of Datalight, Inc., and
  is protected under the copyright laws of the United States and other juris-
  dictions.

  In addition to civil penalties for infringement of copyright under applic-
  able U.S. law, 17 U.S.C. 1204 provides criminal penalties for violation of
  (a) the restrictions on circumvention of copyright protection systems found
  in 17 U.S.C. 1201 and (b) the protections for the integrity of copyright
  management information found in 17 U.S.C. 1202.

  U.S. Government Restricted Rights:  Use, duplication, reproduction, or
  transfer of this commercial product and accompanying documentation is
  restricted in accordance with FAR 12.212 and DFARS 227.7202 and by a
  License Agreement.

  IN ADDITION TO COPYRIGHT AND PATENT LAW, THIS SOFTWARE IS PROTECTED UNDER A
  SOURCE CODE AGREEMENT, NON-DISCLOSURE AGREEMENT (NDA), OR SIMILAR BINDING
  CONTRACT BETWEEN DATALIGHT, INC. AND THE LICENSEE ("BINDING AGREEMENT").
  YOUR RIGHT, IF ANY, TO COPY, PUBLISH, MODIFY OR OTHERWISE USE THE SOFTWARE,
  IS SUBJECT TO THE TERMS AND CONDITIONS OF THE BINDING AGREEMENT.  BY USING
  THE SOFTWARE IN ANY MANNER, IN WHOLE OR IN PART, YOU AGREE TO BE BOUND BY
  THE TERMS OF THE BINDING AGREEMENT.  CONTACT DATALIGHT, INC. AT THE ADDRESS
  SET FORTH ABOVE IF YOU OBTAINED THIS SOFTWARE IN ERROR.
---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------
                              Description

    TFS OS Integration Layer

    Defines and prototypes for a generic OS Integration Layer.

    +-----------------------------------------+
    |        OS Integration Layer            /|
    +---------|--------------+--------------+ |
    |         v              | OS Functions   |
    |      TFS Core  -------->                |
    |         |              | C Library      |
    +---------v--------------+--------------+ |
    |        OS device I/O functions         \|
    +-----------------------------------------+

---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------
                              Revision History
    $Log: rloilapi.h $
    Revision 1.2  2006/01/26 04:29:47  brandont
    Merged with Reliance 2.0 code base.
    Revision 1.10  2006/01/12 02:49:51Z  brandont
    Added I/O layer function pointer prototypes.
    Revision 1.9  2005/12/17 04:19:32Z  brandont
    Added prototypes for RelToolNameToIo() and TfsIoIoctl().
    Revision 1.1  2005/07/23 00:00:30Z  Pauli
    Initial revision
    Revision 1.2  2005/07/23 01:00:29Z  Pauli
    Merge with 2.0 product line, build 458.
    Revision 1.8  2005/07/23 01:00:29Z  brandont
    Removed prototypes for TfsDirMatchName and RelCaseCharCmp.
    Revision 1.7  2005/05/18 20:57:09Z  garyp
    Cleaned up the prototypes for readability.
    Revision 1.6  2005/02/19 02:12:59Z  brandont
    Updated the OIL function prototypes.  Moved the CLib function
    prototypes to rlclib.h.
    Revision 1.5  2004/12/31 01:17:53Z  GaryP
    Updated to use indents of 4, hard tabs of 8 (if used), and a standard
    comment style.
    Revision 1.4  2004/12/02 03:08:37Z  brandont
    Updated prototypes for TfsCoreMount.
    Revision 1.3  2004/11/23 02:50:55Z  brandont
    Moved the logging function prototypes to rllog.h. Moved the os services
    function prototypes to rlosserv.h.  Removed the prototypes for the
    non-existant TfsTaskXXX functions.
    Revision 1.2  2004/11/05 18:57:24Z  brandont
    Updated prototype for TfsCacheReadBlock.
    Revision 1.1  2004/09/29 20:14:30  brandont
    Initial revision
---------------------------------------------------------------------------*/

#ifndef RLOILAPI_H
#define RLOILAPI_H      1


/*  The OS Integration Layer must provide these functions for the TFS Core to
    do its work.

    Most of these functions have been written already, and can be used as is.
    Some (or worst case all) of them must be modified to work with the
    particular operating system, both for the internal driver, and for the
    external utilities (such as TfsFormat).
*/


/*  I/O function types

    These function types exist for places where the I/O layer must be
    redirected to accomidate OS implementations.
*/
typedef sOilBlkDevInfo * (* tDeviceNameToIo)(tfsChar * szDevName);
typedef TFSERR (* tIoInitialize)(sOilBlkDevInfo * pBlkDevInfo, sRelIoData * * ppIoData);
typedef TFSERR (* tIoGetVolumeGeometry)(sRelIoData * pIoData, sTfsGeometry * pLogicalGeometry);
typedef TFSERR (* tIoSetVolumeGeometry)(sRelIoData * pIoData, sTfsGeometry * pLogicalGeometry);
typedef TFSERR (* tIoDiscardBlocks)(sRelIoData * pIoData, tfsBlock iStartBlock, tfsBlock iNumBlocks);
typedef TFSERR (* tIoVerify)(sRelIoData * pIoData, void * pBuffer, tfsBlock iStartBlock, tfsBlock iNumBlocks);
typedef TFSERR (* tIoWrite)(sRelIoData * pIoData, void * pBuffer, tfsBlock iStartBlock, tfsBlock iNumBlocks);
typedef TFSERR (* tIoReset)(sRelIoData * pIoData);
typedef TFSERR (* tIoFlush)(sRelIoData * pIoData);
typedef TFSERR (* tIoUninitialize)(sRelIoData * pIoData);
typedef TFSERR (* tIoForceMediaChange)(sRelIoData * pIoData);


/*  I/O functions
*/
sOilBlkDevInfo * RelToolNameToIo( tfsChar * szDevName );
TFSERR TfsIoInitialize(sOilBlkDevInfo * pBlkDevInfo, sRelIoData ** ppIoData);
TFSERR TfsIoUninitialize(sRelIoData * pIoData);
TFSERR TfsIoGetCapabilities(sRelIoData * pIoData, tfsUint32 * piCapabilities);
TFSERR TfsIoSetVolumeGeometry(sRelIoData * pIoData, sTfsGeometry * pLogicalGeometry);
TFSERR TfsIoGetVolumeGeometry(sRelIoData * pIoData, sTfsGeometry * pLogicalGeometry);
TFSERR TfsIoDiscardBlocks(sRelIoData * pIoData, tfsBlock iStartBlock, tfsBlock iNumBlocks);
TFSERR TfsIoFlush(sRelIoData * pIoData);
TFSERR TfsIoSetBadBlockList(sRelIoData * pIoData, unsigned iNumExtents, sTfsExtent * pBadBlockList);
TFSERR TfsIoVerify(sRelIoData * pIoData, void * pBuffer, tfsBlock iStartBlock, tfsBlock iNumBlocks);
TFSERR TfsIoRead(sRelIoData * pIoData, void * pBuffer, tfsBlock iStartBlock, tfsBlock iNumBlocks);
TFSERR TfsIoWrite(sRelIoData * pIoData, void * pBuffer, tfsBlock iStartBlock, tfsBlock iNumBlocks);
TFSERR TfsIoGetMediaChangeStatus(sRelIoData * pIoData, tfsBool * pfMediaChangeStatus);
TFSERR TfsIoReset(sRelIoData * pIoData);
TFSERR TfsIoForceMediaChange(sRelIoData * pIoData);


/*  Nonstandard I/O functions
*/
TFSERR TfsIoIoctl(sRelIoData * pIoData, tfsUint32 ulFunction, void * pArg );


/*
*/
tfsBool RelMatchNameMask(tfsChar * szMask, tfsChar * szName);


#endif /* RLOILAPI_H */
