/*---------------------------------------------------------------------------
               ----> DO NOT REMOVE THE FOLLOWING NOTICE <----

      Copyright (c) 2003 - 2004 Datalight, Inc.  All Rights Reserved.

  Datalight, Incorporated is a Washington State corporation located at:

        21520 30th Dr SE, Suite 110,      Tel:  425-951-8086
        Bothell, WA  98021                Fax:  425-951-8094
        USA                               Http://www.datalight.com

  This software, including without limitation, all source code and documen-
  tation, is the confidential, trade secret property of Datalight, Inc., and
  is protected under the copyright laws of the United States and other juris-
  dictions.

  ---> Portions of the software are patent pending <---

  In addition to civil penalties for infringement of copyright under applic-
  able U.S. law, 17 U.S.C. 1204 provides criminal penalties for violation of
  (a) the restrictions on circumvention of copyright protection systems found
  in 17 U.S.C. 1201 and (b) the protections for the integrity of copyright
  management information found in 17 U.S.C. 1202.

  U.S. Government Restricted Rights:  Use, duplication, reproduction, or
  transfer of this commercial product and accompanying documentation is
  restricted in accordance with FAR 12.212 and DFARS 227.7202 and by a
  License Agreement.

  IN ADDITION TO COPYRIGHT AND PATENT LAW, THIS SOFTWARE IS PROTECTED UNDER A
  SOURCE CODE AGREEMENT, NON-DISCLOSURE AGREEMENT (NDA), OR SIMILAR BINDING
  CONTRACT BETWEEN DATALIGHT, INC. AND THE LICENSEE ("BINDING AGREEMENT").
  YOUR RIGHT, IF ANY, TO COPY, PUBLISH, MODIFY OR OTHERWISE USE THE SOFTWARE,
  IS SUBJECT TO THE TERMS AND CONDITIONS OF THE BINDING AGREEMENT.  BY USING
  THE SOFTWARE IN ANY MANNER, IN WHOLE OR IN PART, YOU AGREE TO BE BOUND BY
  THE TERMS OF THE BINDING AGREEMENT.  CONTACT DATALIGHT, INC. AT THE ADDRESS
  SET FORTH ABOVE IF YOU OBTAINED THIS SOFTWARE IN ERROR.
---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------
                              Description

    Structures and defines to support the TFS Test Suite. The TFS Test Suite
    is designed to operate both stand-alone, interacting with the disk
    through the O/S Integration Layer, and as a unit test for the TFS core.

    Each test cleans up after itself

---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------
                              Revision History
    $Log: rltest.h $
    Revision 1.7  2006/03/01 19:14:17  joshuab
    Updated types of gpszTfsCategory and gpszTfsSubtest.
    Revision 1.6  2006/02/01 21:15:12Z  Pauli
    Added prototype for TfsTLibRand to replace RelOsRand.
    Revision 1.5  2006/01/09 17:23:35Z  Pauli
    Replaced TFS endian and ptr math macros with DCL.
    Revision 1.4  2005/12/26 09:40:09Z  Garyp
    Eliminated the REL_MAX_TEST_BLOCK_SIZE hack.
    Revision 1.3  2005/12/22 07:55:23Z  garyp
    Hacked to provide a way to avoid allocating unnecessary memory.
    Revision 1.2  2005/12/08 21:57:47Z  pauli
    Replaced gbTfsTestBlock with gpTfsTestBuffer which is an allocated
    32-bit aligned buffer.
    Revision 1.1  2005/12/02 18:21:16Z  Pauli
    Initial revision
    Revision 1.2  2005/12/02 18:21:15Z  Pauli
    Merge with 2.0 product line, build 458.
    Revision 1.6  2005/07/28 19:39:22Z  brandont
    Removed extern reference for an obsolete global.
    Revision 1.5  2005/02/23 02:22:13Z  brandont
    Added rlcmdln.h and rlclib.h header files.
    Revision 1.4  2004/12/31 01:17:53Z  GaryP
    Updated to use indents of 4, hard tabs of 8 (if used), and a standard
    comment style.
    Revision 1.3  2004/12/04 04:12:07Z  brandont
    Renamed TfsCLibRand to RelOsRand.  Renamed TfsOsAlloc to RelMalloc.
    Revision 1.2  2004/11/23 18:36:59Z  PaulI
    Modified TfsTLibRandInRange to work correctly when min and max are equal.
    Revision 1.1  2004/09/27 18:14:34Z  brandont
    Initial revision
    Revision 1.13  2004/08/27 00:33:42  brandont
    - IR ? *.* Changed header on all the source modules to the FlashFx style header
---------------------------------------------------------------------------*/

#ifndef TFS_TEST_H
#define TFS_TEST_H   1

#include <rlcmdln.h>
#include <rlclib.h>

/*  valid repeat range limits
*/
#define  TFS_MIN_PERF_REPEAT  1
#define  TFS_MAX_PERF_REPEAT  65535UL

/*  maximum # of directories or files that can be open at once for test suite
*/
#define TFS_MAX_OPEN_DIRS           3
#define TFS_MAX_OPEN_FILES          10

struct TfsTagTestCommonInfo
{
    tfsUint32       iBufferSize;
    tfsUint32       iMaxNameLen;
    tfsUint32       iMaxPathLen;
    sTfsGeometry    sGeometry;
};
typedef struct TfsTagTestCommonInfo sTfsTestCommonInfo;

/*  the initial state of the disk for a test
*/
struct TfsTagTestInitState
{
    tfsChar        *szName;     /* name of file/dir */
    tfsUint32       iFlags;     /* type of file/dir to create, with permissions */
    int             iCount;     /* # of files/dirs */
    tfsOffset       iLength;    /* length for file */
    tfsUint32       iFill;      /* fill value for file */
};
typedef struct TfsTagTestInitState sTfsTestInitState;

/*  all tests use file names within restrictive 8.3 limits
*/
#define TFS_TEST_BASIC_FLEN      12

#define TFS_TEST_SIG       0x54534554UL /* 'TEST' */

/*  #define gszTfsTestRecordPattern      TfsStr("File: %" ##TFS_TEST_BASIC_FLEN "s, Record: 0x%08X, Size: 0x%04X, ReWrites: 0x%04X")
*/
#define gszTfsTestRecordPattern     TfsStr("File: %s, Record: 0x%LX, Size: 0x%LX, Updates: 0x%hX")

/*  Define a constant that can be used to reserve stack space for a buffer in which to sprintf the record pattern string
    Doubling the format string is overly generous, but should be safe
*/
#define gcchTfsFormattedRecordPattern  (TfsArraySize (gszTfsTestRecordPattern) * 2)

/*  signature for record file
*/
#define TFS_RECORD_SIG        0x41544144UL      /* 'RECO' */

/*  allow updating database files in multiple directions
*/
typedef enum eTagTfsDirection
{
    tfsTestDirectionForward = 0,
    tfsTestDirectionReverse,
    tfsTestDirectionRandom,     /* simulates random */

    tfsTestDirectionOptions     /* # of options */
} eTfsTestDirection;

/*  a single record in a "database" file. used for testing
*/
struct sTfsTagTestRecord
{
    tfsUint32       iSignature; /* signature */
    tfsOffset       iRecordNumber;      /* zero-based record number */
    tfsOffset       iRecordSize;        /* # bytes in record (header + "data") */
    tfsUint32       iRewrites;  /* # times record has been overwritten - zero on create */
};
typedef struct sTfsTagTestRecord sTfsTestRecord;

/*  used to keep track of each database file
*/
struct sTfsTagTestDataBase
{
    TFSFILE        *fp;         /* file handle */
    tfsChar        *szPath;     /* ptr to path */
    eTfsTestDirection iDirection;       /* direction */
    tfsOffset       iRecordSize;        /* record size */
    tfsOffset       iCurrentRecord;     /* current record # in file */
    tfsOffset       iNumRecords;        /* # records in file */
    tfsOffset       iRecordsWritten;    /* records written this pass (0-iNumRecords) */
    tfsOffset       iFileSize;  /* file size */
    tfsUint32       iRewrites;  /* # of rewrites in database */
    tfsBool         fFull;      /* we hit disk full on this pass */
    tfsBool         fPassComplete;
    tfsOffset       iRandFirst; /* used only for random direction */
    tfsOffset       iRandLast;  /* used only for random direction */
    tfsBool         fPingPong;  /* used only for random direction */
};
typedef struct sTfsTagTestDataBase sTfsTestDataBase;


/*  used to keep track of each volume being tested
*/
struct sTfsTagTestVolumeInfo
{
    sTfsVolume     *pVolume;
    sTfsGeometry    sGeometry;
    tfsChar         szDeviceName[TFS_NAMELEN + 1];
};
typedef struct sTfsTagTestVolumeInfo sTfsTestVolumeInfo;


#define TFS_RECORD_NOTUSED    TFS_UINT32_MAX
#define TfsDataBasePassComplete(pDataBase) ((pDataBase)->fPassComplete)
#define TfsDataBaseGotDiskFull(pDataBase) ((pDataBase)->fFull)
#define TfsDataBaseAtStart(pDataBase) ((pDataBase)->iCurrentRecord == TFS_RECORD_NOTUSED)
#define TfsDataBaseIsOpen(pDataBase) ((pDataBase)->fp)

/*  get a random number in this range
*/
#define TfsTLibRandInRange(iMin,iMax) (((iMax) > (iMin)) ? \
   (TfsTLibRand((iMax) - (iMin)) + iMin) : iMin)

/*  report interval for long tests
*/
extern tfsUint32 giTfsTestReportInterval;


#define TFS_TEST_RANDOM_SEED  0

/*  extern for global random seed
*/
extern tfsUint32 giTfsRandomSeed;
tfsUint32       TfsTLibSRand(tfsUint32 iSeed);
tfsUint32       TfsTLibRand(tfsUint32 iLimit);

#define TFS_TEST_BUFFER_SIZE (TFS_MAX_BLOCK_SIZE * 3)
 
#if TFS_FILENUM != TFU_TEST_C
extern tfsChar *gszRoot;
extern tfsChar  gszPathSeparator[];
extern tfsChar  gszWildCard[];
extern tfsChar  gszWildChar[];
extern sTfsTestCommonInfo gsTfsTestCommonInfo;
extern tfsChar  gszTfsLevel[TFS_NAMELEN + 1];
extern tfsChar  gszTfsPath[TFS_PATHLEN + 1];
extern tfsByte *gpTfsTestBuffer;
#endif

#if TFS_FILENUM != TFT_TEST_C
extern tfsChar  gszTfsBasicHelp[];
extern sTfsCmdLineSubTest gsTfsBasicSubTests[];
extern tfsChar  gszTfsTypeHelp[];
extern sTfsCmdLineSubTest gsTfsTypeSubTests[];
#endif

#if TFS_FILENUM != TFT_CORE_C
extern tfsChar  gszTfsCoreHelp[];
extern sTfsCmdLineSubTest gsTfsCoreSubTests[];
#endif

#if TFS_FILENUM != TFT_PERF_C
extern tfsChar  gszTfsPerfHelp[];
extern sTfsCmdLineRange gsTfsPerfRepeatRange;
extern tfsUint32 giTfsPerfRepeat;
extern sTfsCmdLineSubTest gsTfsPerfSubTests[];
#endif

#if TFS_FILENUM != TFT_FILL_C
extern tfsChar  gszTfsFillHelp[];
extern sTfsCmdLineSubTest gsTfsFillSubTests[];
#endif

#if TFS_FILENUM != TFT_LARGE_C
extern tfsChar  gszTfsLargeHelp[];
extern sTfsCmdLineSubTest gsTfsLargeSubTests[];
#endif

#if TFS_FILENUM != TFT_CHECK_C
extern tfsChar  gszTfsTestCheckHelp[];
extern sTfsCmdLineSubTest gsTfsCheckSubTests[];
#endif

#if TFS_FILENUM != TFT_KILL_C
extern tfsChar  gszTfsKillHelp[];
extern sTfsCmdLineSubTest gsTfsKillSubTests[];
#endif

#if TFS_FILENUM != TFT_RANDOM_C
extern tfsChar  gszTfsRandomHelp[];
extern sTfsCmdLineSubTest gsTfsRandomSubTests[];
#endif

#if TFS_FILENUM != TFT_FRAG_C
extern tfsChar  gszTfsFragHelp[];
extern sTfsCmdLineSubTest gsTfsFragSubTests[];
#endif

/*  logging globals
*/
extern char *gpszTfsCategory;
extern char *gpszTfsSubtest;
extern tfsUint32 giTfsPassedTests;
extern tfsUint32 giTfsSkippedTests;
extern tfsClock giTfsClock;


/*  Macros to set / clear / test individual bits in a bit array
    Bits are addressed from low-order bit to high-order bit w/in bytes
*/
#define TfsBitArrayAlloc(pbf,iBits,fInit)                   \
{                                                  \
   tfsUint32   iBytes = ((iBits) / 8) + (((iBits) & 7) ? 1 : 0);  \
   (pbf) = (tfsBuffer *)DclMemAlloc (iBytes);                 \
   if (NULL != (pbf))                                    \
   {                                               \
      DclMemSet ((pbf), (fInit) ? 0xff : 0, iBytes);        \
   }                                               \
}
#define TfsBitArrayTest(pbf,index)  ((*(tfsBuffer *)DclPtrAdd ((pbf), (index)/8) & (1 << ((tfsByte)(index) & 7))) ? TFS_TRUE : TFS_FALSE)
#define TfsBitArraySet(pbf,index)   (*(tfsBuffer *)DclPtrAdd ((pbf), ((index)/8)) |= (1 << ((tfsByte)(index) & 7)))
#define TfsBitArrayClear(pbf,index) (*(tfsBuffer *)DclPtrAdd ((pbf), ((index)/8)) &= ~(1 << ((tfsByte)(index) & 7)))
#define TfsBitArraySetFlag(pbf,index,fFlag)     ((fFlag) ? TfsBitArraySet(pbf,index) : TfsBitArrayClear(pbf,index))


/*  hard-coded parameters for FILL tests
*/
#define TFS_TEST_BUFFER_BLOCKS      10  /* Number of blocks to use for working buffers in tests */
#define TFS_TEST_DEFAULT_BLOCK_SIZE       1024
#define  TFS_TEST_DIRNAME_LENGTH_LIMIT    8192
#define  TFS_TEST_DIRPATH_LENGTH_LIMIT    ((tfsUint16)(TFS_TEST_DIRNAME_LENGTH_LIMIT * 4))


/*  hard-coded parameters for LARGE tests
*/
#define  TFS_TEST_HUGE_RECORD_SIZE  0x10113L    /* Arbitrary wierd number somewhat over 64K */

#define  TFS_TEST_LARGE_REWRITE_COUNT  5
#define TFS_TEST_LARGE_UPDATES_PER_TRANSACTION  3

#define TFS_TEST_LARGE_MULTI_FILE_DEFAULT_COUNT 3
#define TFS_TEST_LARGE_MULTI_FILE_DEFAULT_SIZE  (1024UL * 1024UL)

/*  Define test-specific errors
*/
#define  TFS_TESTERR    0x4000  /* differentiate test-defined error from TFS core errors */
#define TFS_TESTERR_BADDATA      (TFS_TESTERR   | 100)

#define TfsTLibMaxPathLen()   (int)(gsTfsTestCommonInfo.iMaxPathLen)

/*  multi-volume support
*/
#define TFS_TEST_MAX_VOLUMES  4
extern sTfsTestVolumeInfo gpTfsTestVolumes[TFS_TEST_MAX_VOLUMES];
extern tfsUint32 giTfsTestCurrentVolume;
extern unsigned giTfsTestVolumes;
unsigned        TfsTestNumberOfVolumes(void);
int             TfsTestSetCurrentVolume(unsigned iVolume);

#if TFS_TEST_CHECK
extern tfsBool  gfTfsCheckNow;
void            TfsTestCheckNow(void);
#endif

#if TFS_TEST_KILL
extern tfsBool  gfTfsTestKillBlock;
void            TfsTestKillBlock(void);
#endif

#endif /* TFS_TEST_H */
