/*---------------------------------------------------------------------------
               ----> DO NOT REMOVE THE FOLLOWING NOTICE <----

                  Copyright (c) 2003 - 2006 Datalight, Inc.
                    All Rights Reserved.  Patent Pending.

  Datalight, Incorporated is a Washington State corporation located at:

        21520 30th Dr SE, Suite 110,      Tel:  425-951-8086
        Bothell, WA  98021                Fax:  425-951-8094
        USA                               Http://www.datalight.com

  This software, including without limitation, all source code and documen-
  tation, is the confidential, trade secret property of Datalight, Inc., and
  is protected under the copyright laws of the United States and other juris-
  dictions.

  In addition to civil penalties for infringement of copyright under applic-
  able U.S. law, 17 U.S.C. 1204 provides criminal penalties for violation of
  (a) the restrictions on circumvention of copyright protection systems found
  in 17 U.S.C. 1201 and (b) the protections for the integrity of copyright
  management information found in 17 U.S.C. 1202.

  U.S. Government Restricted Rights:  Use, duplication, reproduction, or
  transfer of this commercial product and accompanying documentation is
  restricted in accordance with FAR 12.212 and DFARS 227.7202 and by a
  License Agreement.

  IN ADDITION TO COPYRIGHT AND PATENT LAW, THIS SOFTWARE IS PROTECTED UNDER A
  SOURCE CODE AGREEMENT, NON-DISCLOSURE AGREEMENT (NDA), OR SIMILAR BINDING
  CONTRACT BETWEEN DATALIGHT, INC. AND THE LICENSEE ("BINDING AGREEMENT").
  YOUR RIGHT, IF ANY, TO COPY, PUBLISH, MODIFY OR OTHERWISE USE THE SOFTWARE,
  IS SUBJECT TO THE TERMS AND CONDITIONS OF THE BINDING AGREEMENT.  BY USING
  THE SOFTWARE IN ANY MANNER, IN WHOLE OR IN PART, YOU AGREE TO BE BOUND BY
  THE TERMS OF THE BINDING AGREEMENT.  CONTACT DATALIGHT, INC. AT THE ADDRESS
  SET FORTH ABOVE IF YOU OBTAINED THIS SOFTWARE IN ERROR.
---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------
                              Description

    This header file contains macros and symbols used to implement the
    trace facility in Reliance.
---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------
                              Revision History
    $Log: rltrace.h $
    Revision 1.3  2006/01/06 01:34:11  Garyp
    Added RELTRACE_FORMAT.
    Revision 1.2  2005/12/16 00:32:32Z  pauli
    Added trace flag for enabling all zones.
    Revision 1.1  2005/10/12 00:05:52Z  Pauli
    Initial revision
    Revision 1.2  2005/10/12 01:05:52Z  pauli
    Added trace class for KTF downloader state machine.
    Revision 1.1  2005/08/21 10:47:20Z  Garyp
    Initial revision
    Revision 1.4  2005/08/21 10:47:20Z  garyp
    Documentation update.
    Revision 1.3  2005/08/18 21:04:12Z  pauli
    Removed OIL trace mask.
    Revision 1.2  2005/08/02 18:22:26Z  pauli
    Added trace option for the index module.
    Revision 1.1  2005/07/13 13:09:04Z  pauli
    Initial revision
    Revision 1.2  2005/06/15 16:53:11Z  PaulI
    Added trace codes for a IO layer and errors.
    Changed the trace macro to be TRACEPRINTF().
    Revision 1.1  2005/04/11 00:14:20Z  garyp
    Initial revision
---------------------------------------------------------------------------*/

/*-------------------------------------------------------------------
    Trace Classes

    The various RELTRACE_class values are used in the RELTRACEPRINTF
    macro.

    Use the TRACEALWAYS class to generate a message regardless what
    the RELTRACEMASK setting is in relconf.h (if the debug level is
    OK).

    A maximum of 32 levels are currently supported since the
    RELTRACEMASK is a bitmapped field, 32-bits wide.
-------------------------------------------------------------------*/
#define RELTRACE_VOL            1
#define RELTRACE_FILE           2
#define RELTRACE_FSDK           3
#define RELTRACE_DRIVER         4
#define RELTRACE_CORE           5
#define RELTRACE_IO             6
#define RELTRACE_ERROR          7
#define RELTRACE_EFILE          8
#define RELTRACE_INDEX          9
#define RELTRACE_KTFDL         10
#define RELTRACE_FORMAT        11


/*-------------------------------------------------------------------
    Trace Mask Bits

    The various RELTRACEBIT_class values are used to create the
    RELTRACEMASK value which is used to customize the trace output.
    This is a bit-mapped mask that is defined in relconf.h.
-------------------------------------------------------------------*/
#define RELTRACEBIT_NONE        0
#define RELTRACEBIT_ALL         0xFFFFFFFF
#define RELTRACEBIT_VOL         ((D_UINT32)1 << (RELTRACE_VOL-1))
#define RELTRACEBIT_FILE        ((D_UINT32)1 << (RELTRACE_FILE-1))
#define RELTRACEBIT_FSDK        ((D_UINT32)1 << (RELTRACE_FSDK-1))
#define RELTRACEBIT_DRIVER      ((D_UINT32)1 << (RELTRACE_DRIVER-1))
#define RELTRACEBIT_CORE        ((D_UINT32)1 << (RELTRACE_CORE-1))
#define RELTRACEBIT_IO          ((D_UINT32)1 << (RELTRACE_IO-1))
#define RELTRACEBIT_ERROR       ((D_UINT32)1 << (RELTRACE_ERROR-1))
#define RELTRACEBIT_EFILE       ((D_UINT32)1 << (RELTRACE_EFILE-1))
#define RELTRACEBIT_INDEX       ((D_UINT32)1 << (RELTRACE_INDEX-1))
#define RELTRACEBIT_KTFDL       ((D_UINT32)1 << (RELTRACE_KTFDL-1))
#define RELTRACEBIT_FORMAT      ((D_UINT32)1 << (RELTRACE_FORMAT-1))


/*-------------------------------------------------------------------
    Trace Macros

    RELPRINTF()
        This macro is used to display debug output if the module
        in question is compiled with a D_DEBUG level >= to "lev".
        The syntax is as follows:

        RELPRINTF(lev, ("printf fmt string\n", arg1, arg2, etc));

    RELTRACEPRINTF()
        This macro is used to display debug output if the module
        in question is compiled with a D_DEBUG level >= to "lev"
        and if the RELTRACEMASK value set in relconf.h specifies
        that the given class is to be handled.

        The "flags" parameter may be TRACEINDENT, TRACEUNDENT, or
        TRACENODENT.  Indents must match undents, however the debug
        level "lev" does NOT need to be the same where indents and
        undents are specified.

        The "RELTRACE_class" value may be "TRACEALWAYS" so that no
        class match is required in the TRACERELMASK value.  This
        is useful for debug output that you want to always display
        (assuming "lev" matches), similar to RELPRINTF(), however
        unlike RELPRINTF() it gives the ability to specify
        indentation.

        The syntax is as follows:

        RELTRACEPRINTF((MAKETRACEFLAGS(RELTRACE_class, lev, flags),
            "printf fmt string\n", arg1, arg2, etc));
-------------------------------------------------------------------*/
#if D_DEBUG > 0

int RelDebugPrintf(const char * pszFmt, ...);
int RelTracePrintf(D_UINT32 ulFlags, const char * pszFmt, ...);

#define RELPRINTF(lev, txt) (((lev)<=D_DEBUG) ? (void)RelDebugPrintf txt : ((void)0))
#define RELTRACEPRINTF(params)  RelTracePrintf params

#else

#define RELPRINTF(lev, txt)
#define RELTRACEPRINTF(params)

#endif

