/* wdbCommIfLib.h - header file for the WDB agents communication interface */

/*
 * Copyright (c) 1994-2005 Wind River Systems, Inc.  
 *
 * The right to copy, distribute, modify or otherwise make use  of this 
 * software may be licensed only pursuant to the terms of an applicable Wind 
 * River license agreement.
 */

/*
modification history
--------------------
01f,06dec05,dbt  Remove references to sockaddr_in.
01e,01dec05,dbt  Removed references to timeval structure.
01d,25apr02,jhw  Added C++ support (SPR 76304).
01c,03jun95,ms	 changed read/write to sendto/rcvfrom
01b,05apr95,ms 	 new data types.
01a,23sep94,ms 	 written.
*/

#ifndef __INCwdbCommIfLibh
#define __INCwdbCommIfLibh

#ifdef __cplusplus
extern "C" {
#endif

/* includes */

#include <wdb/wdb.h>

/* definitions */

#define WDB_COMM_MODE_POLL      1
#define WDB_COMM_MODE_INT       2

/* data types */

/*
 * WDB_COMM_IF - interface used by the agent to talk to the host.
 */

typedef void * WDB_COMM_ID;

typedef struct wdb_comm_if
    {
    WDB_COMM_ID	      commId;
    int (*rcvfrom)    (WDB_COMM_ID commId, char * addr, UINT32 len,
			void * pAddr, UINT32 addrLength, INT32 timeout);
    int (*sendto)     (WDB_COMM_ID commId, char * addr, UINT32 len,
			void * pAddr, UINT32 addrLength);
    int (*modeSet)    (WDB_COMM_ID commId, UINT32 newMode);
    int (*cancel)     (WDB_COMM_ID commId);
    int (*hookAdd)    (WDB_COMM_ID commId, void (*rout)(), UINT32 arg);
    int (*notifyHost) (WDB_COMM_ID commId);
    } WDB_COMM_IF;

/*
 * WDB_DRV_IF - interface between a driver and the agents pseudo-UDP/IP stack
 *
 * The default agent configuration uses the native VxWorks UDP/IP stack which
 * has its own interface to a driver (a net_if structure).
 * The agent can also be configured to use a pseudo-UDP/IP stack that
 * is independant of the OS. The pseudo-UDP/IP stack is used for
 * communication with the system mode agent.
 * The WDB_DRV_IF structure allows new drivers to be installed in the
 * pseudo-UDP/IP stack.
 */

typedef struct wdb_drv_if
    {
    UINT32	mode;			/* available device modes */
    UINT32  	mtu;			/* maximum transmit packet size */ 
    void    	(*stackRcv)();		/* callback when pkt arrives */

    /* driver routines */

    void *   	devId;			/* device ID */
    STATUS	(*pollRtn)();		/* poll device for data */
    STATUS 	(*pktTxRtn)();		/* transmit a packet */
    STATUS 	(*modeSetRtn)();	/* toggle driver mode */
    int		(*ioctl)();		/* misc */
    } WDB_DRV_IF;

#ifdef __cplusplus
}
#endif

#endif  /* __INCwdbCommIfLibh */

