/* wdbLibP.h - private header file for remote debug agent */

/*
 * Copyright (c) 1994-2005 Wind River Systems, Inc.  
 *
 * The right to copy, distribute, modify or otherwise make use  of this 
 * software may be licensed only pursuant to the terms of an applicable Wind 
 * River license agreement.
 */

/*
modification history
--------------------
02d,08dec05,dbt  Added wdbCommMtuGet() prototype.
02c,06dec05,dbt  Added WDB_NTOHx and WDB_HTONx macros
02b,28nov05,dbt  Removed wdbExternCmdRun().
		 Added macros to isolate code of external and task mode agents
02a,07nov05,dbt  Updated wdbConnectHookAdd() prototype.
		 Removed wdbDisconnectHookAdd().
		 Removed wdbExternRegsSet() & wdbExternRegsGet() prototypes.
		 Removed wdbMemRtLibInit() prototype.
01z,01aug05,bpn  Add new register buffer.
01y,28may04,elg  Add new function pointers.
01x,08apr04,elg  Define wdbMemChunk.
		 Remove obsolete type WDB_REG_SET_TYPE.
01w,13jan04,elg  Add prototype of wdbCnctModePost().
01v,26sep03,tbu  Moved definition of wdbMbufInit & wdbSioTest  to wdbOsLib.h
01u,15sep03,elg  Add definition of wdbSioTest().
01t,27aug03,elg  Add declarations of context event routines.
01s,15jul03,pch  Conditional compilation cleanup
01r,26jun03,elg  Add definition of wdbSuspendSystem() as extern routine.
01q,27feb03,elg  Merge file with BSD.
01p,04feb03,elg  Add WDB merged protocol.
01o,20jul00,dbt  Added wdbExternCmdRun() prototype.
01n,03feb00,elg  Fix component dependencies problems.
01m,31jan00,dbt  Added wdbPdUpdateNotify() prototype.
01l,21jan00,dbt  Added WDB_USR_SVC_STRUCT structure.
01k,25oct99,elg  Rename PD creation and deletion hooks.
01j,15oct99,dbt  added wdbConnectHookAdd() and wdbDisconnectHookAdd()
                 prototypes.
01i,19aug99,elg  Add hooks for protection domains.
01h,29jun99,elg  Add WDB_CORE support.
01g,25apr02,jhw  Added C++ support (SPR 76304).
01f,14sep01,jhw  Added wdbExternRegsGet() and wdbExternRegsSet() function
                 prototypes.
		 Removed __STDC__ precompiler conditional.
01e,11jun98,dbt  removed wdbTgtHasFpp() prototype.
01d,03jun95,ms	 added prototype for wdbTargetIsConnected()
01c,05apr95,ms   new data types.
01b,06feb95,ms	 removes wdbRpcLib specific things.
01a,20sep94,ms   written.
*/

#ifndef __INCwdbLibPh
#define __INCwdbLibPh

#ifdef __cplusplus
extern "C" {
#endif

/* includes */

#include <wdb/wdb.h>
#include <wdb/wdbRtIfLib.h>
#include <wdb/wdbCommIfLib.h>

/* defines */

#define WDB_STATE_EXTERN_RUNNING        1

#define	WDB_IS_NOW_EXTERNAL()		(wdbMode & WDB_MODE_SYSTEM)

#define	wdbSuspendSystemHere(callBack,callBackArg)			\
    if (_func_wdbSuspendSystem != NULL)					\
	_func_wdbSuspendSystem (NULL, (callBack), (callBackArg))

#define	wdbSuspendSystem(pRegSet,callBack,callBackArg)			\
    if (_func_wdbSuspendSystem != NULL)					\
	_func_wdbSuspendSystem ((pRegSet), (callBack), (callBackArg))

#define	wdbResumeSystem()						\
    if (_func_wdbResumeSystem != NULL)					\
	_func_wdbResumeSystem ()

/* Byte swapping macros */

#if	_BYTE_ORDER == _BIG_ENDIAN
#define	WDB_NTOHL(x)	(x)
#define	WDB_HTONL(x)	(x)
#define	WDB_NTOHS(x)	(x)
#define	WDB_HTONS(x)	(x)
#else   /* BYTE_ORDER == BIG_ENDIAN */ 
#define WDB_NTOHL(x)	((((x) & 0x000000ff) << 24) | \
			 (((x) & 0x0000ff00) <<  8) | \
			 (((x) & 0x00ff0000) >>  8) | \
			 (((x) & 0xff000000) >> 24))

#define WDB_HTONL(x)	((((x) & 0x000000ff) << 24) | \
			 (((x) & 0x0000ff00) <<  8) | \
			 (((x) & 0x00ff0000) >>  8) | \
			 (((x) & 0xff000000) >> 24))

#define WDB_NTOHS(x)	((((x) & 0x00ff) << 8) | \
			 (((x) & 0xff00) >> 8))

#define WDB_HTONS(x)	((((x) & 0x00ff) << 8) | \
			 (((x) & 0xff00) >> 8))
#endif	/* BYTE_ORDER == BIG_ENDIAN */

/* data types */

typedef struct wdb_usr_svc		/* user service structure */
    {
    FUNCPTR	func;			/* service entry point */
    UINT32 	svcId;			/* service ID */
    } WDB_USR_SVC;

/* agent variables */

extern UINT32  		wdbCommMtu;
extern UINT32		wdbAvailOptions;
extern UINT32		wdbMode;
extern BOOL		wdbOneShot;
extern UINT32		wdbAvailModes;
extern BOOL		wdbIsInitialized;
extern FUNCPTR		wdbTaskRestartHook;
extern char 		wdbRegsBuffer[];
extern UINT32		wdbRegsBufferSize;
extern VOIDFUNCPTR	_func_wdbResumeSystem;
extern VOIDFUNCPTR	_func_wdbExternNotifyHost;
extern VOIDFUNCPTR	_func_wdbExternRcvHook;
extern VOIDFUNCPTR	_func_wdbSuspendSystem;
extern void *		pWdbXport;
extern WDB_COMM_IF *	pWdbCommIf;


/* function prototypes */

extern void	wdbInfoGet	 	(WDB_AGENT_INFO * pInfo);
extern void	wdbNotifyHost		(void);
extern BOOL	wdbTargetIsConnected	(void);

extern STATUS	wdbModeSet	 	(INT32 newMode);
extern BOOL	wdbIsNowExternal	(void);
extern BOOL	wdbRunsExternal		(void);
extern BOOL	wdbIsNowTasking		(void);
extern BOOL	wdbRunsTasking		(void);
extern void	wdbCtxCreateNotify	(void * pNode);
extern void	wdbCtxDeleteNotify	(void * pNode);
extern void	wdbCtxEvtLibInit	(void);
extern STATUS	wdbConnectHookAdd	(BOOL connect, FUNCPTR connectHook);
extern void	wdbCmdLoop		(void);
extern void	wdbRtLibInit		(WDB_RT_IF * pWdbRtIf);
extern void	wdbCnctModePost		(UINT32 mode);
extern STATUS	wdbExternRegLibInit	(void);
extern STATUS	wdbExternEnterHookAdd	(void (*hook) ());
extern STATUS	wdbExternExitHookAdd	(void (*hook) ());
extern STATUS	wdbTaskRegLibInit	(void);
extern UINT32	wdbCommMtuGet		(void);

/* globals */

extern WDB_IU_REGS *    pWdbExternSystemRegs;

#ifdef __cplusplus
}
#endif

#endif /* __INCwdbLibPh */
