/* wdbOsLib.h - WDB vxWorks specific header */

/*
 * Copyright (c) 2003-2005 Wind River Systems, Inc.  
 *
 * The right to copy, distribute, modify or otherwise make use  of this 
 * software may be licensed only pursuant to the terms of an applicable Wind 
 * River license agreement.
 */

/*
modification history
--------------------
01j,20dec05,dbt  Added VxWorks specific APIs from wdbLib.h.
01i,05dec05,dbt  Removed inclusion of RPC header files.
		 Cleanup.
01h,28nov05,dbt  Moved WDB_CTX_XXX definitions to wdbOsLib.h.
01g,14nov05,dbt  Added WDB_IU_REGS definition.
01f,23mar04,elg  Macros to get information in context are now OS specific.
01e,12sep03,tbu  Added inclusion of netinet/in_systm.h
01d,29aug03,elg  Redefine GOPHER_STRTOL to do strtoul() (fix SPR #90223).
		 Include vxWorks.h first (fix SPR #90034).
01c,14jul03,pch  Conditional compilation cleanup
01b,06may03,elg  Fix compilation error when Altivec is supported.
01a,03jan03,elg  Written.
*/

#ifndef	__INCwdbOsLibh
#define	__INCwdbOsLibh

#ifdef	__cplusplus
extern "C" {
#endif

/*
DESCRIPTION

The vxWorks specific header contains definitions that cannot be used by other
OSes.
*/

/* includes */

#include <vxWorks.h>	/* this file must be included first */

#include <types/vxANSI.h>
#include <netBufLib.h>
#include <net/mbuf.h>

#include <ctype.h>
#include <limits.h>
#include <sioLib.h>
#include <stdlib.h>
#include <string.h>
#include <regs.h>
#include <dspLib.h>
#include <taskHookLib.h>
#include <intLib.h>
#include <fppLib.h>

#include <wdb/wdbDbgLib.h>

/* data types */

typedef REG_SET	WDB_IU_REGS;	/* WDB register set */
struct brkpt;			/* Breakpoint structure */

/* OS definitions */

#define	WDB_KERNEL_ID		-1	/* kernel ID in WDB agent */

/* access to registers */

#define	WDB_PC(ptr)		(ptr)->reg_pc
#define	WDB_FP(ptr)		(ptr)->reg_fp
#define	WDB_SP(ptr)		(ptr)->reg_sp

/* debug info */

/* on vxWorks, PC does not need to be changed */

#define	_WRS_ADJUST_PC_AFTER_SW_BP(pRegs)

/* macros to get and set ID from context structure */

#define	WDB_CTX_SYS_INFO_SET(pArgs)					\
	{								\
	(pArgs) [0] = 0;						\
	(pArgs) [1] = 0;						\
	}

#define	WDB_CTX_TASK_INFO_SET(pArgs, tid, pid)				\
	{								\
	(pArgs) [0] = (TGT_INT_T) (tid);				\
	(pArgs) [1] = (TGT_INT_T) (pid);				\
	}

#define	WDB_CTX_RTP_INFO_SET(pArgs, pid)				\
	{								\
	(pArgs) [0] = (TGT_INT_T) (pid);				\
	(pArgs) [1] = 0;						\
	}

/* macros used to save/load external agent context */


#ifndef WDB_CTX_LOAD
#define WDB_CTX_LOAD(pRegs) _sigCtxLoad(pRegs)
#endif  /* WDB_CTX_LOAD */

#ifndef WDB_CTX_SAVE
#define WDB_CTX_SAVE(pRegs) _sigCtxSave(pRegs)
#endif  /* WDB_CTX_SAVE */

#ifndef	WDB_CTX_SETUP
#define	WDB_CTX_SETUP(pRegs, pStack, taskEntry, pArgs)			\
	_sigCtxSetup ((pRegs), (pStack), (taskEntry), (pArgs))
#endif	/* WDB_CTX_SETUP */

#ifndef WDB_CTX_RTN_VALUE_SET
#define WDB_CTX_RTN_VALUE_SET(pRegs,retVal) \
	_sigCtxRtnValSet ((pRegs),(retVal))
#endif	/* WDB_CTX_RTN_VALUE_SET */
	
/* globals */
	
extern void	wdbMbufInit		(struct mbuf *, UINT32,
					 CL_BLK *, UINT32);	
extern void	wdbSioTest		(SIO_CHAN *, int, char);
extern void	wdbMdlSymSyncLibInit	(void);
extern void     wdbSysBpLibInit		(struct brkpt * pBps, UINT32 bpCnt);
extern void     wdbTaskBpLibInit	(void);
extern STATUS	wdbTaskModeInit		(UINT32 spawnStackSize,
					 UINT32 taskPriority,
					 UINT32 taskOptions,
					 UINT32 stackSize,
					 UINT32 maxRestarts,
					 UINT32 restartTime);
extern void	wdbRtpLibInit		(int priority);
extern void	wdbRtpBpLibInit		(void);
extern void	wdbTaskHookInit		(void);
extern void	wdbBpSyncLibInit	(void);
	
#ifdef	__cplusplus
}
#endif	/* __cplusplus */

#endif	/* __INCwdbOsLibh */
