/* altq_subr_var.h - definitions for the ALTQ subroutines library */

/*
 * Copyright (c) 2006 Wind River Systems, Inc.
 *
 * The right to copy, distribute, modify or otherwise make use
 * of this software may be licensed only pursuant to the terms
 * of an applicable Wind River license agreement.
 */

/*
modification history
--------------------
01e,28feb06,kch  Removed machClkUsepcc from altq_config_params (SPR#118453).
01d,10feb06,kch  Changed machClkUsepcc type to BOOL. Also added prototypes 
                 for altq init routines.
01c,09feb06,kch  Renamed altqSubrInit() to altq_subr_init().
01b,08feb06,kch  Added maxQueueAllowed configuration parameter.
01a,17jan06,kch  written.
*/

#ifndef _INCaltq_config_varh
#define _INCaltq_config_varh

#include <cfgDefs.h>

#ifdef __cplusplus
extern "C" {
#endif

#ifdef _WRS_KERNEL

typedef struct altq_config_params {

    CFG_DATA_HDR    cfgh;

    /* queue spec */

    /* 
     * The defaultQlimit specifies maximum number of queues allowed for 
     * each ALTQ queuing discipline. The limit applies globally to every
     * queueing disciplines configured in the system. For correct operation,
     * this value should be less than the maximum number of queues allowed 
     * in the system for ALTQ.
     */
    u_int16_t   defaultQlimit;	/* default queue size limit */
    u_int8_t    defaultPriority;	/* priority */
    int         defaultIfqDrvMaxLen;   

    /*
     * The maxQueueAllowed specifies the number of queues allowed in the 
     * system for all the queueing disciplines
     */
    int         maxQueueAllowed;   /* maximum number of queues allowed */

    /* machine dependent clock related includes */
    u_int32_t   machClkFreq;   /* machine clock frequency */
    void        (*machClkInit) (void);
    u_int64_t   (*readMachClk) (void);

} ALTQ_CONFIG_PARAMS;


/* globals */
IMPORT ALTQ_CONFIG_PARAMS  altqConfigParams;

/* function prototypes */
IMPORT int altq_subr_init(void *altqSubrInitValues);
IMPORT int altq_cbq_init(void);
IMPORT int altq_priq_init(void);
IMPORT int altq_red_init(void);
IMPORT int altq_rio_init(void);

#endif /* _WRS_KERNEL */

#ifdef __cplusplus
}
#endif

#endif /* _INCaltq_config_varh */
