/* nfs3Lib.h - Network File System library header */

/*
 * Copyright (c) 1984-2006 Wind River Systems, Inc.
 *
 * The right to copy, distribute or otherwise make use of this software
 * may be licensed only pursuant to the terms of an applicable Wind River
 * license agreement.
 */

/*
modification history
--------------------
01h,23jan06,pcm  added nfs3StableWriteGet() and nfs3StableWriteSet()
01g,20jan06,md   added pathconf support
01f,19dec05,pcm  re-arranged structure of NFS3_FD
01e,07dec05,pcm  changed structure of NFS3_FD
01d,05nov04,vvv  added nfsCommon.h (SPR #103655)
01c,23aug04,rp   merged from COMP_WN_IPV6_BASE6_ITER5_TO_UNIFIED_PRE_MERGE
01b,28may04,niq  Merging from base6 label POST_ITER5_FRZ16_REBASE (ver
                 /main/vdt/base6_itn5_networking-int/1)
01a,10oct03,snd  written                                      
*/

#ifndef __INCnfs3Libh
#define __INCnfs3Libh

#ifdef __cplusplus
extern "C" {
#endif

#include "vwModNum.h"
#include "dirent.h"
#include "iosLib.h"
#include "ioLib.h"
#include "sys/stat.h"
#include "xdr_mnt3.h"
#include "xdr_nfs3.h"
#include "nfs3Defines.h"
#include "nfsCommon.h"
#include "vnode.h"

#include "private/nfsCacheLibP.h"
    
#define  NAME3_MAX 255
/* nfs file descriptor */

typedef struct nfs3_fd              /* NFS3_FD - NFS file descriptor */
    {
    NFS3_DEV *     nfsDev;          /* Pointer to this file's NFS device */
    SEM_ID         nfsFdSem;        /* Accessing semaphore */
    NFS_CACHE *    pCache;          /* NFS Cache info */
    char           curFilename [NAME_MAX+1]; /* Current file name */    
    voff_t         offset;          /* Current offset in file */
    unsigned int   mode;            /* (O_RDONLY, O_WRONLY, O_RDWR) */

    nfs_fh3        fileHandle;      /* File's file handle */
    nfs_fh3        dirHandle;       /* File's directory file handle */
    fattr3         fileAttr;        /* File's attributes */
    cookie3 *      pdir_cookie;     /* This is the NFS3 directory cookie */
    cookieverf3    dir_cookieverf;  /* The directory entry's cookie verifier */
                                    /* Stored here because there is no space */
                                    /* in the Directory entry DIR structure */
    READDIR3res *  dirCache;        /* Cache of dir list entries */
    } NFS3_FD;

/* function declarations */

extern STATUS   nfs3DirMount (char * hostName, dirpath dirname,
                              nfs_fh3 * pFileHandle);

extern STATUS nfs3DirUnmount  (char * hostName, dirpath dirName);
extern STATUS nfs3ExportRead  (char * hostName, exports * pExports);
extern void   nfs3FileAttrGet (fattr3 * pFattr, struct stat * pStat);

extern STATUS nfs3FsAttrGet   (char * pHostName, nfs_fh3 * pDirHandle,
                               struct statfs * pArg);

extern int    nfs3FileRead    (char * hostName, nfs_fh3 * pHandle,
                               voff_t offset, unsigned int count,
                               char * buf, fattr3 * pNewAttr);
extern STATUS nfs3FileRemove  (char * hostName, nfs_fh3 * pMountHandle,
                               char * fullFileName);
extern int    nfs3FileWrite   (char * hostName, nfs_fh3 * pHandle,
                               voff_t offset, unsigned int count,
                               char * data, fattr3 * pNewAttr);

extern int    nfs3LookUpByName (char * hostName, char * fileName,
                                nfs_fh3 * pMountHandle, LOOKUP3res * pDirOpRes,
                                nfs_fh3 * pDirHandle, BOOL removeFlag);
extern int    nfs3ThingCreate  (char * hostName, char * fullFileName, 
                                nfs_fh3 * pMountHandle, CREATE3res * pDirOpRes,
                                nfs_fh3 * pDirHandle, u_int mode); 
                                 
extern STATUS nfs3DirReadOne  (NFS3_FD *pFd,DIR *  pDir);

extern STATUS nfs3FsInfoGet   (char * pHostName, nfs_fh3 * pMountHandle,
                               nfsinfo * pArg);
extern STATUS nfs3Rename      (char * hostName, nfs_fh3 * pMountHandle, 
                               char * oldName,  nfs_fh3 * pOldDirHandle,
                               char * newName);

extern STATUS nfs3MntUnmountAll (char * hostName); 

extern STATUS nfs3MntDump       (char * hostName);

extern STATUS nfs3ExportRead    (char * hostName, exports * pExports);

extern stable_how nfs3StableWriteGet (void);
extern STATUS     nfs3StableWriteSet (stable_how howStable);

extern STATUS   nfs3PathconfGet (char * pHostName, nfs_fh3 * pDirHandle,
                                nfs_pathconf * pArg);


#ifdef __cplusplus
}
#endif

#endif /* __INCnfs3Libh */
